{% load misago_admin_form %}
{% if field|is_radio_select_field %}
  {% include "misago/admin/form/radio_select.html" %}
{% elif field|is_select_field %}
  {% include "misago/admin/form/select.html" %}
{% elif field|is_multiple_choice_field %}
  {% include "misago/admin/form/multiple_choice.html" %}
{% elif field|is_textarea_field %}
  <textarea {% render_attrs widget class_name="form-control" %} id="{{ field.id_for_label }}" name="{{ widget.name }}" {% render_bool_attrs widget %} {% if field.field.disabled %}disabled{% endif %}>{% if widget.value is not None %}{{ widget.value }}{% endif %}</textarea>
{% else %}
  <input {% render_attrs widget class_name="form-control" %} id="{{ field.id_for_label }}" name="{{ widget.name }}" type="{{ widget.type }}"{% if widget.value is not None %} value="{{ widget.value }}"{% endif %} {% render_bool_attrs widget %} {% if field.field.disabled %}disabled{% endif %}>
{% endif %}