{% extends "misago/emails/base.html" %}
{% load i18n misago_absoluteurl misago_capture %}


{% block content %}
{% capture trimmed as thread_link %}
  <a href="{% absoluteurl thread.get_absolute_url %}">{{ thread }}</a>
{% endcapture %}
{% blocktrans trimmed with user=user sender=sender thread=thread_link|safe %}
{{ user }}, you are receiving this message because {{ sender }} has replied to the thread {{ thread }} that you are subscribed to.
{% endblocktrans %}
<br>
<br>
{% trans "To read this reply, click the below link:" %}
<br>
<br>
<a href="{% absoluteurl post.get_absolute_url %}">{% trans "Go to reply" %}</a>
<br>
{% endblock content %}
