{% extends "misago/admin/layout.html" %}
{% load i18n %}


{% block title %}{% trans "Settings" %} | {{ block.super }}{% endblock %}


{% block content %}
<div class="page-header">
  <h1>
    <a href="{{ active_section.link }}">
      {{ active_section.name }}
    </a>
  </h1>
</div>

<div class="card card-admin-settings-grid">
  <div class="card-body">
    <div class="row">
      {% for subnode in actions %}
        <div class="col-12 col-lg-6 col-xl-4 p-0">
          <a href="{{ subnode.link }}" class="card-admin-settings-card">
            <h5 class="list-group-item-heading">{{ subnode.name }}</h5>
            {% if subnode.description %}
              <small>{{ subnode.description }}</small>
            {% endif %}
          </a>
        </div>
      {% endfor %}
    </div>
  </div>
</div>
{% endblock content %}
