# -*- coding: utf-8 -*-
# snapshottest: v1 - https://goo.gl/zC4yUc
from __future__ import unicode_literals

from snapshottest import Snapshot


snapshots = Snapshot()

snapshots["test_strikethrough_markdown 1"] = "<p>Lorem <del>ipsum</del> dolor met!</p>"
