# Generated by Django 2.2.1 on 2019-05-18 16:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("misago_threads", "0011_remove_custom_partial_indexes")]

    operations = [
        migrations.AddIndex(
            model_name="post",
            index=models.Index(
                condition=models.Q(has_open_reports=True),
                fields=["has_open_reports"],
                name="misago_post_has_open_repo_part",
            ),
        ),
        migrations.AddIndex(
            model_name="post",
            index=models.Index(
                condition=models.Q(is_hidden=False),
                fields=["is_hidden"],
                name="misago_post_is_hidden_part",
            ),
        ),
        migrations.AddIndex(
            model_name="post",
            index=models.Index(
                condition=models.Q(is_event=True),
                fields=["is_event", "event_type"],
                name="misago_post_is_event_part",
            ),
        ),
        migrations.AddIndex(
            model_name="thread",
            index=models.Index(
                condition=models.Q(weight=2),
                fields=["weight"],
                name="misago_thread_pinned_glob_part",
            ),
        ),
        migrations.AddIndex(
            model_name="thread",
            index=models.Index(
                condition=models.Q(weight=1),
                fields=["weight"],
                name="misago_thread_pinned_loca_part",
            ),
        ),
        migrations.AddIndex(
            model_name="thread",
            index=models.Index(
                condition=models.Q(weight=0),
                fields=["weight"],
                name="misago_thread_not_pinned_part",
            ),
        ),
        migrations.AddIndex(
            model_name="thread",
            index=models.Index(
                condition=models.Q(weight__lt=2),
                fields=["weight"],
                name="misago_thread_not_global_part",
            ),
        ),
        migrations.AddIndex(
            model_name="thread",
            index=models.Index(
                condition=models.Q(has_reported_posts=True),
                fields=["has_reported_posts"],
                name="misago_thread_has_reporte_part",
            ),
        ),
        migrations.AddIndex(
            model_name="thread",
            index=models.Index(
                condition=models.Q(has_unapproved_posts=True),
                fields=["has_unapproved_posts"],
                name="misago_thread_has_unappro_part",
            ),
        ),
        migrations.AddIndex(
            model_name="thread",
            index=models.Index(
                condition=models.Q(is_hidden=False),
                fields=["is_hidden"],
                name="misago_thread_is_visible_part",
            ),
        ),
    ]
