{% extends "misago/admin/conf/form.html" %}
{% load i18n misago_admin_form %}


{% block form-body %}
<div class="form-fieldset">
  {% if settings.enable_oauth2_client %}  
    <div class="alert alert-warning" role="alert">
      <strong>{% trans "Note" %}:</strong> {% blocktrans trimmed with provider=settings.oauth2_provider %}
      OAuth2 client is enabled. On site registration, password and account deletion features have been disabled and delegated to the {{ provider }}.
      {% endblocktrans %}
    </div>
  {% endif %}
  <fieldset>
    <legend>{% trans "New accounts" %}</legend>

    {% form_row form.account_activation %}
    {% form_row form.new_inactive_accounts_delete %}

  </fieldset>
</div>
<div class="form-fieldset">
  <fieldset>
    <legend>{% trans "Usernames" %}</legend>

    {% form_row form.username_length_min %}
    {% form_row form.username_length_max %}
    {% form_row form.anonymous_username %}

  </fieldset>
</div>
<div class="form-fieldset">
  <fieldset>
    <legend>{% trans "Avatars" %}</legend>

    {% form_row form.allow_custom_avatars %}
    {% form_row form.avatar_upload_limit %}
    {% form_row form.default_avatar %}
    {% form_row form.default_gravatar_fallback %}
    {% with form.blank_avatar_delete as delete_field %}
      {% with form_settings.blank_avatar as setting %}
        {% form_image_row form.blank_avatar delete_field=delete_field size=setting.image_size dimensions=setting.image_dimensions %}
      {% endwith %}
    {% endwith %}

  </fieldset>
</div>
<div class="form-fieldset">
  <fieldset>
    <legend>{% trans "Signatures" %}</legend>

    {% form_row form.signature_length_max %}

  </fieldset>
</div>
<div class="form-fieldset">
  <fieldset>
    <legend>{% trans "Data downloads" %}</legend>

    {% form_row form.allow_data_downloads %}
    {% form_row form.data_downloads_expiration %}

  </fieldset>
</div>
<div class="form-fieldset">
  <fieldset>
    <legend>{% trans "Own account deletion" %}</legend>

    {% form_row form.allow_delete_own_account %}

  </fieldset>
</div>
<div class="form-fieldset">
  <fieldset>
    <legend>{% trans "IP addresses" %}</legend>

    {% form_row form.ip_storage_time %}

  </fieldset>
</div>
<div class="form-fieldset">
  <fieldset>
    <legend>{% trans "Default subscription preferences" %}</legend>

    {% form_row form.subscribe_start %}
    {% form_row form.subscribe_reply %}

  </fieldset>
</div>
<div class="form-fieldset">
  <fieldset>
    <legend>{% trans "Users lists" %}</legend>

    {% form_row form.users_per_page %}
    {% form_row form.users_per_page_orphans %}

  </fieldset>
</div>
<div class="form-fieldset">
  <fieldset>
    <legend>{% trans "Top posters ranking" %}</legend>

    {% form_row form.top_posters_ranking_size %}
    {% form_row form.top_posters_ranking_length %}

  </fieldset>
</div>
{% endblock form-body %}