{% extends "misago/base.html" %}
{% load i18n %}


{% block title %}{% trans "Page not found" %} | {{ block.super }}{% endblock %}


{% block meta-description %}
{% trans "The link you clicked was incorrect or the page has been moved or deleted." %}
{% endblock meta-description %}


{% block og-title %}{% trans "Page not found" %}{% endblock %}
{% block twitter-title %}{% trans "Page not found" %}{% endblock %}


{% block og-description %}
{% trans "The link you clicked was incorrect or the page has been moved or deleted." %}
{% endblock %}


{% block twitter-description %}
{% trans "The link you clicked was incorrect or the page has been moved or deleted." %}
{% endblock %}


{% block content %}
<div class="page page-error page-error-404">
  <div class="container page-container">
    <div class="message-panel">

      <div class="message-icon">
        <span class="material-icon">info_outline</span>
      </div>

      <div class="message-body">
        <p class="lead">{% trans "Requested page could not be found." %}</p>
        <p>{% trans "The link you followed was incorrect or the page has been moved or deleted." %}</p>
      </div>

    </div>
  </div>
</div>
{% endblock content %}
