{% load i18n misago_admin_form %}
<div class="form-group{% if field.errors %} has-error{% endif %}">
  <label for="{{ field.id_for_label }}" class="control-label{% if label_class %} {{ label_class }}{% endif %}">
    {{ field.label }}:
  </label>
  <div class="{% if field|is_yesno_switch_field %}controls control-yesno-switch {% elif field|is_radio_select_field %}controls control-radioselect {% endif %}{% if field|is_multiple_choice_field %}controls control-checkboxselect {% endif %}{% if field_class %}{{ field_class }}{% endif %}">
    {% form_input field %}
    {% for error in field.errors %}
      <small class="form-text text-danger font-weight-bold">{{ error }}</small>
    {% endfor %}
    {% if field.help_text %}
      <small class="form-text text-muted">{{ field.help_text }}</small>
    {% endif %}
  </div>
</div>