{% extends "misago/admin/generic/form.html" %}
{% load i18n misago_admin_form %}


{% block title %}
{% blocktrans trimmed with category=target.name %}
  Delete category: {{ category }}
{% endblocktrans %} | {{ active_link.name }} | {{ block.super }}
{% endblock title %}


{% block page-header %}
{{ block.super }}
<small>
  {{ target }}
</small>
{% endblock page-header %}


{% block form-header %}
{% trans "Delete category" %}
{% endblock %}


{% block form-body %}
<fieldset>
  <legend>{% trans "Category contents" %}</legend>

  {% if not form.instance.is_leaf_node %}
    {% form_row form.move_children_to %}
  {% endif %}
  {% form_row form.move_threads_to %}

</fieldset>
{% endblock form-body %}


{% block form-footer %}
<button class="btn btn-danger btn-sm">{% trans "Delete category" %}</button>
{% endblock %}
