{% extends "misago/emails/base.html" %}
{% load i18n misago_absoluteurl misago_capture %}


{% block content %}
{% blocktrans trimmed with user=user %}
{{ user }}, your account has been activated by forum administrator.
{% endblocktrans %}
<br>
<br>
{% capture trimmed as login_link %}
<a href="{% absoluteurl LOGIN_URL %}">{% trans "this form" %}</a>
{% endcapture %}
{% blocktrans trimmed with login_form=login_link|safe %}
You can now sign in to it using {{ login_form }}.
{% endblocktrans %}
<br>
{% endblock content %}
