{% extends "misago/admin/generic/form.html" %}
{% load i18n static misago_admin_form %}


{% block title %}
{% if target.pk %}
  {{ target }}
{% else %}
  {% trans "New CSS link" %}
{% endif %} | {{ theme }} | {{ active_link.name }} | {{ block.super }}
{% endblock title %}


{% block page-header %}
{{ block.super }}
<small>
  {{ theme }}
</small>
{% if target.pk %}
  <small>
    {{ target }}
  </small>
{% endif %}
{% endblock page-header %}


{% block form-header %}
{% if target.pk %}
  {% trans "Edit CSS link" %}
{% else %}
  {% trans "New CSS link" %}
{% endif %}
{% endblock %}


{% block form-body %}
{% form_row form.name %}
{% form_row form.url %}
{% endblock form-body %}


{% block form-footer-cancel %}
<a href="{% url 'misago:admin:themes:assets' pk=theme.pk %}" class="btn btn-light btn-sm">
  {% trans "Cancel" %}
</a>
{% endblock %}
