# Generated by Django 1.11.5 on 2017-10-07 14:49
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("misago_readtracker", "0002_postread")]

    operations = []
