{% load i18n %}
<form onSubmit="false">
  <div class="panel-body">
    <h2>{{ poll.question }}</h2>
    {% include "misago/poll/info.html" %}

    <ul class="list-unstyled poll-select-choices">
      {% for choice in poll.choices %}
        <li class="poll-select-choice">
          <button class="btn" disabled>
            <span class="material-icon">
              check_box_outline_blank
            </span>
            <strong>
              {{ choice.label }}
            </strong>
          </button>
        </li>
      {% endfor %}
    </ul>

    <ul class="list-unstyled list-inline poll-help">
      <li class="poll-help-choices-left">
        {% blocktrans trimmed count choices=poll.allowed_choices context "thread poll" %}
          You can select {{ choices }} choice.
        {% plural %}
          You can select {{ choices }} choices.
        {% endblocktrans %}
      </li>
      {% if poll.allow_revotes %}
        <li class="poll-help-allow-revotes">
          {% trans "You can change your vote later." context "thread poll" %}
        </li>
      {% else %}
        <li class="poll-help-no-revotes">
          {% trans "Votes are final." context "thread poll" %}
        </li>
      {% endif %}
    </ul>

  </div>
  <div class="panel-footer">
    <div class="row">
      <div class="col-xs-6 col-sm-4 col-md-2">
        <button class="btn btn-primary btn-block btn-sm" disabled>
          {% trans "Save your vote" context "thread poll" %}
        </button>
      </div>
      <div class="col-xs-6 col-sm-4 col-md-2">
        <button class="btn btn-default btn-block btn-sm" disabled>
          {% trans "See results" context "thread poll" %}
        </button>
      </div>
    </div>
  </div>
</form>