{% extends "misago/profile/base.html" %}
{% load i18n %}


{% block title %}{% trans "Ban details" %} | {{ block.super }}{% endblock %}


{% block og-title %}{% trans "Ban details" %} | {{ block.super }}{% endblock %}
{% block twitter-title %}{% trans "Ban details" %} | {{ block.super }}{% endblock %}


{% block section %}
<div class="profile-ban-details">
  <div class="panel panel-default">
    <div class="panel-heading">
      <h3 class="panel-title">{% trans "Ban details" %}</h3>
    </div>
    <div>

      {% if ban.user_message %}
        <div class="panel-body ban-message ban-user-message">
          <h4>{% trans "User-shown ban message" %}</h4>
          <div class="lead">
            {{ ban.user_message|force_escape|urlize|linebreaks }}
          </div>
        </div>
      {% endif %}

      {% if ban.staff_message %}
        <div class="panel-body ban-message ban-staff-message">
          <h4>{% trans "Team-shown ban message" %}</h4>
          <div class="lead">
            {{ ban.staff_message|force_escape|urlize|linebreaks }}
          </div>
        </div>
      {% endif %}

      <div class="panel-body ban-expires">
        <h4>{% trans "Ban expiration" %}</h4>

        <p class="lead">
          {% if ban.expires_on %}
            {% blocktrans trimmed with username=profile.username expires_on=ban.expires_on|date:"DATETIME_FORMAT" %}
              {{ username }}'s ban expires on {{ expires_on }}.
            {% endblocktrans %}
          {% else %}
            {% blocktrans trimmed with username=profile.username %}
              {{ username }}'s ban is permanent.
            {% endblocktrans %}
          {% endif %}
        </p>
      </div>

    </div>
  </div>
</div>
{% endblock section %}