{% extends "misago/admin/base.html" %}
{% load i18n %}


{% block title %}{% trans "Form submission rejected" %} | {{ block.super }}{% endblock %}


{% block body %}
<body class="login-error">
  <div class="login-error-container">
    <div class="card login-error-card">
      <div class="card-body">
        <h1 class="card-title login-error-title">
          {% trans "Form submission rejected" %}
        </h1>
        {% include "misago/admin/errorpages/csrf_failure_message.html" %}
      </div>
    </div>
  </div>
</body>
{% endblock body %}
