from .analytics import ChangeAnalyticsSettingsForm
from .base import ChangeSettingsForm
from .captcha import ChangeCaptchaSettingsForm
from .general import ChangeGeneralSettingsForm
from .oauth2 import ChangeOAuth2SettingsForm
from .threads import ChangeThreadsSettingsForm
from .users import ChangeUsersSettingsForm
