{% extends "misago/base.html" %}
{% load i18n %}


{% block title %}
  {% if paginator.page > 1 %}
    {{ thread.title }} ({% blocktrans with page=paginator.page %}page: {{ page }}{% endblocktrans %}) | {{ category }} | {{ block.super }}
  {% else %}
    {{ thread.title }} | {{ category }} | {{ block.super }}
  {% endif %}
{% endblock title %}


{% block content %}
<div class="page page-thread {% if category.css_class %}page-thread-{{ category.css_class }}{% endif %}">
  {% include "misago/thread/header.html" %}
  <div class="container page-container">

    {% include "misago/participants.html" with participants=thread.participants_list %}
    {% include "misago/thread/toolbar_top.html" %}
    {% include "misago/thread/posts/index.html" %}
    {% include "misago/thread/toolbar_bottom.html" %}

  </div>
</div>
{% endblock content %}
