# Generated by Django 1.11.1 on 2017-05-26 21:56
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("misago_users", "0008_ban_registration_only")]

    operations = []
