<ul class="breadcrumbs" itemscope itemtype="http://schema.org/BreadcrumbList">
  {% for node in breadcrumbs %}
    <li id="thread-breadcrumb-{{ node.id }}" class="breadcrumbs-item" itemprop="itemListElement" itemscope itemtype="http://schema.org/ListItem">
      <a href="{{ node.get_absolute_url }}">
        {% if node.special_role %}
          <span class="material-icon">chevron_right</span>
          <span class="breadcrumbs-item-name">{{ node.name }} </span>
        {% else %}
          <span class="material-icon"{% if node.color %} style="color: {{ node.color }};"{% endif %}>label</span>
          {% if node.short_name %}
            <span class="breadcrumbs-item-name hidden-sm hidden-md hidden-lg" title="{{ node.name }}">{{ node.short_name }}</span>
            <span class="breadcrumbs-item-name hidden-xs">{{ node.name }}</span>
          {% else %}
            <span class="breadcrumbs-item-name">{{ node.name }}</span>
          {% endif %}
        {% endif %}
      </a>
    </li>
  {% endfor %}
</ul>