{% load i18n %}
<div id="categories-mount">
  {% if categories %}
      <div class="categories-list">
        {% for category in categories %}
          {% include "misago/categories/category.html" with category=category %}
        {% endfor %}
      </div>
    </div>
  {% else %}
    <div class="categories-list">
      <ul class="list-group">
        <li class="list-group-item empty-message">
          <p class="lead">
            {% trans "No categories exist or you don't have permission to see them." %}
          </p>
        </li>
      </ul>
    </div>
  {% endif %}
</div>