{% extends "misago/admin/generic/form.html" %}
{% load i18n misago_admin_form %}


{% block title %}
{% if target.pk %}
  {{ target }}
{% else %}
  {% trans "New warning level" %}
{% endif %} | {{ active_link.name }} | {{ block.super }}
{% endblock title %}


{% block form-header %}
<h1>
  {% if target.pk %}
    {{ target }}
  {% else %}
    {% trans "New warning level" %}
  {% endif %}
</h1>
{% endblock %}


{% block form-extra %}
class="form-horizontal"
{% endblock form-extra %}


{% block form-body %}
<div class="form-body">
  {% with label_class="col-md-3" field_class="col-md-9" %}
    <fieldset>
      <legend>{% trans "Basic options" %}</legend>

      {% form_row form.name label_class field_class %}
      {% form_row form.length_in_minutes label_class field_class %}

    </fieldset>
    <fieldset>
      <legend>{% trans "Restrictions" %}</legend>

      {% form_row form.restricts_posting_replies label_class field_class %}
      {% form_row form.restricts_posting_threads label_class field_class %}

    </fieldset>
  {% endwith %}
</div>
{% endblock form-body %}


{% block form-footer-class %}
col-md-offset-3
{% endblock form-footer-class %}
