{% extends "misago/emails/base.html" %}
{% load i18n misago_absoluteurl %}


{% block content %}
{% blocktrans trimmed with user=user %}
{{ user }}, you are receiving this message because your data is ready for download.
{% endblocktrans %}
<br>
<br>
<a href="{% absoluteurl data_download.file.url %}">{% trans "Download data" %}</a>
<br>
<br>
{% blocktrans trimmed count expires_in=expires_in %}
This link will remain active for {{ expires_in }} hour from the time this message has been sent.
{% plural %}
This link will remain active for {{ expires_in }} hours from the time this message has been sent.
{% endblocktrans %}
<br>
{% endblock content %}
