# Generated by Django 3.2.15 on 2022-12-29 19:55

from django.db import migrations


def create_index_message_setting(apps, _):
    Setting = apps.get_model("misago_conf", "Setting")
    Setting.objects.create(setting="index_message", is_public=False)


class Migration(migrations.Migration):

    dependencies = [
        ("misago_conf", "0005_add_sso_settings"),
    ]

    operations = [migrations.RunPython(create_index_message_setting)]
