{% extends "misago/base.html" %}
{% load i18n %}


{% block title %}{% trans "Request blocked" %} | {{ block.super }}{% endblock %}


{% block meta-description %}
{% trans "This is usually caused by your browser not accepting or using outdated cookies. Check your browser configuration and try again." %}
{% endblock meta-description %}


{% block content %}
<div class="page page-error page-error-csrf">
  <div class="container page-container">
    <div class="message-panel">

      <div class="message-icon">
        <span class="material-icon">remove_circle_outline</span>
      </div>

      <div class="message-body">
        <p class="lead">{% trans "Suspicious request blocked." %}</p>
        <p>{% trans "This is usually caused by your browser not accepting or using outdated cookies." %}</p>
        <p>{% trans "Check your browser configuration and try again." %}</p>
      </div>

    </div>
  </div>
</div>
{% endblock content %}
