{% extends "misago/admin/generic/form.html" %}
{% load i18n misago_admin_form %}


{% block title %}
{% trans "Import theme" %} | {{ active_link.name }} | {{ block.super }}
{% endblock title %}


{% block form-header %}
{% trans "Import theme" %}
{% endblock %}


{% block form-body %}
{% form_row form.name %}
{% form_row form.parent %}
{% form_row form.upload %}
{% endblock form-body %}


{% block form-footer %}
<button class="btn btn-primary btn-sm">{% trans "Import" %}</button>
{% endblock %}