{% extends "misago/base.html" %}
{% load i18n %}


{% block title %}{% trans "Activation failed" %} | {{ block.super }}{% endblock %}


{% block content %}
<div class="page page-message page-message-info">
  <div class="container page-container">
    <div class="message-panel">

      <div class="message-icon">
        <span class="material-icon">info_outline</span>
      </div>

      <div class="message-body">
        <p class="lead">
          {% trans "Your account can't be activated at this time." %}
        </p>
        <p>{{ message }}</p>
      </div>

    </div>
  </div>
</div>
{% endblock content %}