# Generated by Django 1.11.15 on 2018-08-16 14:22
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("misago_legal", "0002_agreement_useragreement")]

    operations = []
