# Generated by Django 2.2.3 on 2019-11-10 14:32

from django.db import migrations


def rename_google_provider(apps, _):
    SocialAuthProvider = apps.get_model("misago_socialauth", "SocialAuthProvider")
    SocialAuthProvider.objects.filter(provider="google").update(
        provider="google-oauth2"
    )


class Migration(migrations.Migration):

    dependencies = [("misago_socialauth", "0002_cache_version")]

    operations = [migrations.RunPython(rename_google_provider)]
