{% load i18n static %}
<!doctype html>
<html lang="{{ LANGUAGE_CODE }}">
  <head>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no">
    <title>{% spaceless %}{% block title %}{% trans "Misago Administration" %}{% endblock %}{% endspaceless %}</title>
    <link rel="stylesheet" href="https://use.fontawesome.com/releases/v5.7.2/css/all.css" integrity="sha384-fnmOCqbTlWIlj8LyTjo7mOUStjsKC4pOpQbqyi7RrhN7udi9RwhKkMHpvLbHG9Sr" crossorigin="anonymous">
    <link href="{% static 'misago/admin/index.css' %}" rel="stylesheet">
    <link rel="apple-touch-icon" sizes="180x180" href="{% static 'misago/admin/apple-touch-icon.png' %}">
    <link rel="icon" type="image/png" sizes="32x32" href="{% static 'misago/admin/favicon-32.png' %}">
    <link rel="icon" type="image/png" sizes="16x16" href="{% static 'misago/admin/favicon-16.png' %}">
    <link rel="shortcut icon" href="{% static 'misago/admin/favicon.ico' %}">
    {% block extra-head %}{% endblock extra-head %}
  </head>
  {% block body %}{% endblock %}
</html>
