{% load i18n misago_poststags %}
{% if post.acl.can_reply or post.acl.can_edit or post.acl.can_see_likes or post.acl.can_like %}
  <div class="post-footer">
    {% if post.acl.can_like %}
      <button type="button" class="btn btn-{% if post.is_liked %}success{% else %}default{% endif %} pull-left" disabled="disabled">
        {% if post.is_liked %}
          {% trans "Liked" %}
        {% else %}
          {% trans "Like" %}
        {% endif %}
      </button>
    {% endif %}
    {% if post.acl.can_see_likes and post.last_likes %}
      <p class="pull-left hidden-xs">
        {% likes_label post %}
      </p>
      <p class="likes-compact pull-left visible-xs-block">
        <span class="material-icon">
          favorite
        </span>
        {{ post.likes }}
      </p>
    {% endif %}
    {% if post.acl.can_reply %}
      <button type="button" class="btn btn-primary pull-right" disabled="disabled">
        {% trans "Reply" %}
      </button>
    {% endif %}
    {% if post.acl.can_edit %}
      <button type="button" class="btn btn-default pull-right" disabled="disabled">
        {% trans "Edit" %}
      </button>
    {% endif %}
  </div>
{% endif %}