from django.db import migrations


class Migration(migrations.Migration):
    """Superseded by 0004"""

    dependencies = [("misago_acl", "0001_initial"), ("misago_core", "0001_initial")]

    operations = []
