apipatterns = []
urlpatterns = []
context_processors = []

new_registrations_validators = []

oauth2_user_data_filters = []
oauth2_validators = []

post_search_filters = []
post_validators = []

markdown_extensions = []
parsing_result_processors = []
