{% load i18n %}
{% for message in messages %}
  {% if 'info' in message.tags %}
    <p class="alert alert-info">
  {% elif 'success' in message.tags %}
    <p class="alert alert-success">
  {% elif 'warning' in message.tags %}
    <p class="alert alert-warning">
  {% elif 'error' in message.tags %}
    <p class="alert alert-danger">
  {% endif %}
    {{ message }}
  </p>
{% endfor %}
