# Generated by Django 1.11.9 on 2018-03-26 00:10
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("misago_threads", "0008_auto_20180310_2234")]

    operations = []
