{% extends "misago/admin/generic/form.html" %}
{% load i18n misago_admin_form %}


{% block title %}
{% trans "Request new data downloads" %} | {{ active_link.name }} | {{ block.super }}
{% endblock title %}


{% block form-header %}
{% trans "Request new data downloads" %}
{% endblock %}


{% block form-body %}
<fieldset>
  {% form_row form.user_identifiers %}
</fieldset>
{% endblock form-body %}
