{% load i18n %}
<p>
  {% trans "Your form submission was rejected because your browser didn't send the CSRF cookie, or the cookie sent was invalid." %}
</p>
<p>
  {% trans "This is usually a result of one of following problems:" %}
</p>
<p>
  {% trans "Your browser is configured to don't accept or send any cookies." %}
</p>
<p>
  {% trans "The form was sent after being opened in the browser for a long time (eg. few days) and cookie token has expired." %}
</p>
<p class="mb-0">
  {% trans "The site is configured to use the HTTPS, but this form was sent over the the HTTP." %}
  {% trans "If you are an owner of this site, please check if the HTTPS certificate used by the site is valid and accepted by the browser." %}
</p>