{% load i18n %}
<div id="admin-analytics">
  <div class="card card-admin-info">
    <div class="card-body">
      <div class="row align-items-center">
        <div class="col">
          <h4 class="card-title">{% trans "Analytics" %}</h4>
        </div>
      </div>
    </div>
    <div class="card-body border-top">
      <div class="text-center py-5">
        <div class="spinner-border text-light" role="status">
          <span class="sr-only">Loading...</span>
        </div>
      </div>
    </div>
  </div>
</div>