# Generated by Django 1.10.5 on 2017-02-05 14:34
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("misago_users", "0005_dj_19_update")]

    operations = []
