# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-05-11 20:51
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [("misago_users", "0017_move_bans_to_cache_version")]

    operations = [
        migrations.AlterField(
            model_name="datadownload",
            name="requested_on",
            field=models.DateTimeField(
                db_index=True, default=django.utils.timezone.now
            ),
        ),
        migrations.AlterField(
            model_name="user",
            name="joined_on",
            field=models.DateTimeField(
                db_index=True,
                default=django.utils.timezone.now,
                verbose_name="joined on",
            ),
        ),
    ]
