# Generated by Django 2.2.1 on 2019-05-18 16:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("misago_themes", "0002_create_default_theme_and_cache_version")]

    operations = [
        migrations.AlterField(
            model_name="theme",
            name="level",
            field=models.PositiveIntegerField(editable=False),
        ),
        migrations.AlterField(
            model_name="theme",
            name="lft",
            field=models.PositiveIntegerField(editable=False),
        ),
        migrations.AlterField(
            model_name="theme",
            name="rght",
            field=models.PositiveIntegerField(editable=False),
        ),
    ]
