# Generated by Django 1.11.11 on 2018-03-31 22:08
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("misago_users", "0010_user_profile_fields")]

    operations = [
        migrations.AddField(
            model_name="user",
            name="is_deleting_account",
            field=models.BooleanField(default=False),
        )
    ]
