{% extends "misago/admin/generic/form.html" %}
{% load i18n misago_admin_form %}


{% block title %}
{% if target.pk %}
{{ target }}
{% else %}
{% trans "New type" %}
{% endif %} | {{ active_link.name }} | {{ block.super }}
{% endblock title %}


{% block page-header %}
{{ block.super }}
{% if target.pk %}
  <small>
    {{ target }}
  </small>
{% endif %}
{% endblock page-header %}


{% block form-header %}
{% if target.pk %}
  {% trans "Edit attachment type" %}
{% else %}
  {% trans "New attachment type" %}
{% endif %}
{% endblock %}


{% block form-body %}
<div class="form-fieldset">
  <fieldset>
    <legend>{% trans "Basic options" %}</legend>

    {% form_row form.name %}
    {% form_row form.extensions %}
    {% form_row form.mimetypes %}
    {% form_row form.size_limit %}

  </fieldset>
</div>
<div class="form-fieldset">
  <fieldset>
    <legend>{% trans "Availability" %}</legend>

    {% form_row form.status %}
    {% form_row form.limit_uploads_to %}
    {% form_row form.limit_downloads_to %}

  </fieldset>
</div>
{% endblock form-body %}
