{% extends "misago/admin/layout.html" %}


{% block content %}
<div class="page-header row align-items-center">
  <div class="col">
    <h1>
      {% block page-header %}
        <a href="{{ active_link.link }}">
          {{ active_link.name }}
        </a>
      {% endblock page-header %}
    </h1>
  </div>
  {% block page-actions %}{% endblock %}
</div>
{% block view %}{% endblock view %}
{% endblock content %}
