# Generated by Django 1.11.1 on 2017-05-21 17:52
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("misago_threads", "0005_index_search_document")]

    operations = []
