{% extends "misago/base.html" %}
{% load i18n %}


{% block title %}{% trans "You are banned" %} | {{ block.super }}{% endblock %}


{% block meta-description %}
{% trans "You are banned from accessing this page." %}
{% endblock meta-description %}


{% block content %}
<div class="page page-error page-error-403 page-error-banned">
  <div class="container page-container">
    <div class="message-panel">

      <div class="message-icon">
        <span class="material-icon">highlight_off</span>
      </div>

      <div class="message-body">
        {% include "misago/errorpages/ban_message.html" %}
      </div>

    </div>
  </div>
</div>
{% endblock content %}
