{% extends "misago/profile/base.html" %}
{% load i18n %}


{% block title %}{% trans "Follows" %} | {{ block.super }}{% endblock %}


{% block og-title %}{% trans "Follows" %} | {{ block.super }}{% endblock %}
{% block twitter-title %}{% trans "Follows" %} | {{ block.super }}{% endblock %}


{% block section %}
<div class="profile-follows">

  <nav class="toolbar">
    <div class="toolbar-section toolbar-section-auto">
      <div class="toolbar-item">
        <h3>
          {% if is_authenticated_user %}
            {% blocktrans trimmed count users=count %}
              You are following {{ users }} user.
            {% plural %}
              You are following {{ users }} users.
            {% endblocktrans %}
          {% else %}
            {% blocktrans trimmed count users=count with username=profile %}
              {{ username }} is following {{ users }} user.
            {% plural %}
              {{ username }} is following {{ users }} users.
            {% endblocktrans %}
          {% endif %}
        </div>
      </div>
    </h3>
  </nav>

  {% if count %}
    <div class="users-cards-list ui-ready" itemscope itemtype="http://schema.org/ItemList">
      <div class="row">
        {% for user in follows %}
          <div class="col-xs-12 col-sm-4" itemprop="itemListElement" itemscope itemtype="http://schema.org/ListItem">
            {% include "misago/userslists/usercard.html" with user=user %}
            <meta itemprop="position" content="{{ forloop.counter }}" />
          </div>
        {% endfor %}
      </div>
      <meta itemprop="numberOfItems" content="{{ count }}">
      <meta itemprop="itemListOrder" content="http://schema.org/ItemListOrderAscending">
    </div>
  {% else %}
    <p class="lead">
      {% if is_authenticated_user %}
        {% trans "You are not following any users." %}
      {% else %}
        {% blocktrans trimmed with username=profile.username %}
          {{ username }} is not following any users.
        {% endblocktrans %}
      {% endif %}
    </p>
  {% endif %}

</div>
{% endblock section %}