!function(e){var t={};function n(i){if(t[i])return t[i].exports;var r=t[i]={i:i,l:!1,exports:{}};return e[i].call(r.exports,r,r.exports,n),r.l=!0,r.exports}n.m=e,n.c=t,n.d=function(e,t,i){n.o(e,t)||Object.defineProperty(e,t,{enumerable:!0,get:i})},n.r=function(e){"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})},n.t=function(e,t){if(1&t&&(e=n(e)),8&t)return e;if(4&t&&"object"==typeof e&&e&&e.__esModule)return e;var i=Object.create(null);if(n.r(i),Object.defineProperty(i,"default",{enumerable:!0,value:e}),2&t&&"string"!=typeof e)for(var r in e)n.d(i,r,function(t){return e[t]}.bind(null,r));return i},n.n=function(e){var t=e&&e.__esModule?function(){return e.default}:function(){return e};return n.d(t,"a",t),t},n.o=function(e,t){return Object.prototype.hasOwnProperty.call(e,t)},n.p="",n(n.s=111)}([function(e,t,n){"use strict";e.exports=n(80)},function(e,t,n){"use strict";n.d(t,"c",function(){return r}),n.d(t,"a",function(){return a}),n.d(t,"e",function(){return o}),n.d(t,"b",function(){return s}),n.d(t,"d",function(){return l});
/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
var i=function(e,t){return(i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n])})(e,t)};function r(e,t){function n(){this.constructor=e}i(e,t),e.prototype=null===t?Object.create(t):(n.prototype=t.prototype,new n)}var a=function(){return(a=Object.assign||function(e){for(var t,n=1,i=arguments.length;n<i;n++)for(var r in t=arguments[n])Object.prototype.hasOwnProperty.call(t,r)&&(e[r]=t[r]);return e}).apply(this,arguments)};function o(e,t){var n={};for(var i in e)Object.prototype.hasOwnProperty.call(e,i)&&t.indexOf(i)<0&&(n[i]=e[i]);if(null!=e&&"function"==typeof Object.getOwnPropertySymbols){var r=0;for(i=Object.getOwnPropertySymbols(e);r<i.length;r++)t.indexOf(i[r])<0&&(n[i[r]]=e[i[r]])}return n}function s(e,t,n,i){return new(n||(n=Promise))(function(r,a){function o(e){try{l(i.next(e))}catch(e){a(e)}}function s(e){try{l(i.throw(e))}catch(e){a(e)}}function l(e){e.done?r(e.value):new n(function(t){t(e.value)}).then(o,s)}l((i=i.apply(e,t||[])).next())})}function l(e,t){var n,i,r,a,o={label:0,sent:function(){if(1&r[0])throw r[1];return r[1]},trys:[],ops:[]};return a={next:s(0),throw:s(1),return:s(2)},"function"==typeof Symbol&&(a[Symbol.iterator]=function(){return this}),a;function s(a){return function(s){return function(a){if(n)throw new TypeError("Generator is already executing.");for(;o;)try{if(n=1,i&&(r=2&a[0]?i.return:a[0]?i.throw||((r=i.return)&&r.call(i),0):i.next)&&!(r=r.call(i,a[1])).done)return r;switch(i=0,r&&(a=[2&a[0],r.value]),a[0]){case 0:case 1:r=a;break;case 4:return o.label++,{value:a[1],done:!1};case 5:o.label++,i=a[1],a=[0];continue;case 7:a=o.ops.pop(),o.trys.pop();continue;default:if(!(r=(r=o.trys).length>0&&r[r.length-1])&&(6===a[0]||2===a[0])){o=0;continue}if(3===a[0]&&(!r||a[1]>r[0]&&a[1]<r[3])){o.label=a[1];break}if(6===a[0]&&o.label<r[1]){o.label=r[1],r=a;break}if(r&&o.label<r[2]){o.label=r[2],o.ops.push(a);break}r[2]&&o.ops.pop(),o.trys.pop();continue}a=t.call(e,o)}catch(e){a=[6,e],i=0}finally{n=r=0}if(5&a[0])throw a[1];return{value:a[0]?a[1]:void 0,done:!0}}([a,s])}}}},function(e,t,n){"use strict";(function(e){n.d(t,"h",function(){return b}),n.d(t,"F",function(){return x}),n.d(t,"s",function(){return w}),n.d(t,"r",function(){return k}),n.d(t,"j",function(){return S}),n.d(t,"l",function(){return C}),n.d(t,"m",function(){return A}),n.d(t,"n",function(){return _}),n.d(t,"i",function(){return O}),n.d(t,"o",function(){return P}),n.d(t,"k",function(){return M}),n.d(t,"f",function(){return I}),n.d(t,"g",function(){return D}),n.d(t,"a",function(){return z}),n.d(t,"D",function(){return H}),n.d(t,"d",function(){return W}),n.d(t,"C",function(){return X}),n.d(t,"G",function(){return c}),n.d(t,"p",function(){return h}),n.d(t,"b",function(){return d}),n.d(t,"E",function(){return f}),n.d(t,"u",function(){return p}),n.d(t,"w",function(){return g}),n.d(t,"v",function(){return m}),n.d(t,"H",function(){return v}),n.d(t,"x",function(){return y}),n.d(t,"c",function(){return E}),n.d(t,"e",function(){return B}),n.d(t,"y",function(){return U}),n.d(t,"z",function(){return G}),n.d(t,"I",function(){return Q}),n.d(t,"q",function(){return $}),n.d(t,"t",function(){return Z}),n.d(t,"A",function(){return J}),n.d(t,"B",function(){return ee});var i=n(5),r=n(3),a=n(1),o=n(62),s=n.n(o);function l(e,t,n,i){if(function(e){return"IntValue"===e.kind}(n)||function(e){return"FloatValue"===e.kind}(n))e[t.value]=Number(n.value);else if(function(e){return"BooleanValue"===e.kind}(n)||function(e){return"StringValue"===e.kind}(n))e[t.value]=n.value;else if(function(e){return"ObjectValue"===e.kind}(n)){var a={};n.fields.map(function(e){return l(a,e.name,e.value,i)}),e[t.value]=a}else if(function(e){return"Variable"===e.kind}(n)){var o=(i||{})[n.name.value];e[t.value]=o}else if(function(e){return"ListValue"===e.kind}(n))e[t.value]=n.values.map(function(e){var n={};return l(n,t,e,i),n[t.value]});else if(function(e){return"EnumValue"===e.kind}(n))e[t.value]=n.value;else{if(!function(e){return"NullValue"===e.kind}(n))throw new r.a;e[t.value]=null}}function c(e,t){var n=null;e.directives&&(n={},e.directives.forEach(function(e){n[e.name.value]={},e.arguments&&e.arguments.forEach(function(i){var r=i.name,a=i.value;return l(n[e.name.value],r,a,t)})}));var i=null;return e.arguments&&e.arguments.length&&(i={},e.arguments.forEach(function(e){var n=e.name,r=e.value;return l(i,n,r,t)})),h(e.name.value,i,n)}var u=["connection","include","skip","client","rest","export"];function h(e,t,n){if(n&&n.connection&&n.connection.key){if(n.connection.filter&&n.connection.filter.length>0){var i=n.connection.filter?n.connection.filter:[];i.sort();var r=t,a={};return i.forEach(function(e){a[e]=r[e]}),n.connection.key+"("+JSON.stringify(a)+")"}return n.connection.key}var o=e;if(t){var l=s()(t);o+="("+l+")"}return n&&Object.keys(n).forEach(function(e){-1===u.indexOf(e)&&(n[e]&&Object.keys(n[e]).length?o+="@"+e+"("+JSON.stringify(n[e])+")":o+="@"+e)}),o}function d(e,t){if(e.arguments&&e.arguments.length){var n={};return e.arguments.forEach(function(e){var i=e.name,r=e.value;return l(n,i,r,t)}),n}return null}function f(e){return e.alias?e.alias.value:e.name.value}function p(e){return"Field"===e.kind}function g(e){return"InlineFragment"===e.kind}function m(e){return e&&"id"===e.type&&"boolean"==typeof e.generated}function v(e,t){return void 0===t&&(t=!1),Object(a.a)({type:"id",generated:t},"string"==typeof e?{id:e,typename:void 0}:e)}function y(e){return null!=e&&"object"==typeof e&&"json"===e.type}function b(e,t){if(e.directives&&e.directives.length){var n={};return e.directives.forEach(function(e){n[e.name.value]=d(e,t)}),n}return null}function x(e,t){if(void 0===t&&(t={}),!e.directives)return!0;var n=!0;return e.directives.forEach(function(e){if("skip"===e.name.value||"include"===e.name.value){var i=e.arguments||[],a=e.name.value;Object(r.b)(1===i.length);var o=i[0];Object(r.b)(o.name&&"if"===o.name.value);var s=i[0].value,l=!1;s&&"BooleanValue"===s.kind?l=s.value:(Object(r.b)("Variable"===s.kind),l=t[s.name.value],Object(r.b)(void 0!==l)),"skip"===a&&(l=!l),l||(n=!1)}}),n}function w(e,t){return function(e){var t=[];return Object(i.b)(e,{Directive:function(e){t.push(e.name.value)}}),t}(t).some(function(t){return e.indexOf(t)>-1})}function k(e){return e&&w(["client"],e)&&w(["export"],e)}function S(e,t){var n=t,i=[];return e.definitions.forEach(function(e){if("OperationDefinition"===e.kind)throw new r.a;"FragmentDefinition"===e.kind&&i.push(e)}),void 0===n&&(Object(r.b)(1===i.length),n=i[0].name.value),Object(a.a)({},e,{definitions:[{kind:"OperationDefinition",operation:"query",selectionSet:{kind:"SelectionSet",selections:[{kind:"FragmentSpread",name:{kind:"Name",value:n}}]}}].concat(e.definitions)})}function E(e){for(var t=[],n=1;n<arguments.length;n++)t[n-1]=arguments[n];return t.forEach(function(t){null!=t&&Object.keys(t).forEach(function(n){e[n]=t[n]})}),e}function C(e){T(e);var t=e.definitions.filter(function(e){return"OperationDefinition"===e.kind&&"mutation"===e.operation})[0];return Object(r.b)(t),t}function T(e){Object(r.b)(e&&"Document"===e.kind);var t=e.definitions.filter(function(e){return"FragmentDefinition"!==e.kind}).map(function(e){if("OperationDefinition"!==e.kind)throw new r.a;return e});return Object(r.b)(t.length<=1),e}function A(e){return T(e),e.definitions.filter(function(e){return"OperationDefinition"===e.kind})[0]}function _(e){return e.definitions.filter(function(e){return"OperationDefinition"===e.kind&&e.name}).map(function(e){return e.name.value})[0]||null}function O(e){return e.definitions.filter(function(e){return"FragmentDefinition"===e.kind})}function P(e){var t=A(e);return Object(r.b)(t&&"query"===t.operation),t}function M(e){var t;T(e);for(var n=0,i=e.definitions;n<i.length;n++){var a=i[n];if("OperationDefinition"===a.kind){var o=a.operation;if("query"===o||"mutation"===o||"subscription"===o)return a}"FragmentDefinition"!==a.kind||t||(t=a)}if(t)return t;throw new r.a}function I(e){void 0===e&&(e=[]);var t={};return e.forEach(function(e){t[e.name.value]=e}),t}function D(e){if(e&&e.variableDefinitions&&e.variableDefinitions.length){var t=e.variableDefinitions.filter(function(e){return e.defaultValue}).map(function(e){var t=e.variable,n=e.defaultValue,i={};return l(i,t.name,n),i});return E.apply(void 0,[{}].concat(t))}return{}}function N(e,t,n){var i=0;return e.forEach(function(n,r){t.call(this,n,r,e)&&(e[i++]=n)},n),e.length=i,e}var L={kind:"Field",name:{kind:"Name",value:"__typename"}};function R(e){return function e(t,n){return t.selectionSet.selections.every(function(t){return"FragmentSpread"===t.kind&&e(n[t.name.value],n)})}(A(e)||function(e){Object(r.b)("Document"===e.kind),Object(r.b)(e.definitions.length<=1);var t=e.definitions[0];return Object(r.b)("FragmentDefinition"===t.kind),t}(e),I(O(e)))?null:e}function F(e){return function(t){return e.some(function(e){return e.name&&e.name===t.name.value||e.test&&e.test(t)})}}function j(e,t){var n=Object.create(null),r=[],o=Object.create(null),s=[],l=R(Object(i.b)(t,{Variable:{enter:function(e,t,i){"VariableDefinition"!==i.kind&&(n[e.name.value]=!0)}},Field:{enter:function(t){if(e&&t.directives&&(e.some(function(e){return e.remove})&&t.directives&&t.directives.some(F(e))))return t.arguments&&t.arguments.forEach(function(e){"Variable"===e.value.kind&&r.push({name:e.value.name.value})}),t.selectionSet&&function e(t){var n=[];t.selections.forEach(function(t){"Field"!==t.kind&&"InlineFragment"!==t.kind||!t.selectionSet?"FragmentSpread"===t.kind&&n.push(t):e(t.selectionSet).forEach(function(e){return n.push(e)})});return n}(t.selectionSet).forEach(function(e){s.push({name:e.name.value})}),null}},FragmentSpread:{enter:function(e){o[e.name.value]=!0}},Directive:{enter:function(t){if(F(e)(t))return null}}}));return l&&N(r,function(e){return!n[e.name]}).length&&(l=function(e,t){var n=function(e){return function(t){return e.some(function(e){return t.value&&"Variable"===t.value.kind&&t.value.name&&(e.name===t.value.name.value||e.test&&e.test(t))})}}(e);return R(Object(i.b)(t,{OperationDefinition:{enter:function(t){return Object(a.a)({},t,{variableDefinitions:t.variableDefinitions.filter(function(t){return!e.some(function(e){return e.name===t.variable.name.value})})})}},Field:{enter:function(t){var i=e.some(function(e){return e.remove});if(i){var r=0;if(t.arguments.forEach(function(e){n(e)&&(r+=1)}),1===r)return null}}},Argument:{enter:function(e){if(n(e))return null}}}))}(r,l)),l&&N(s,function(e){return!o[e.name]}).length&&(l=function(e,t){function n(t){if(e.some(function(e){return e.name===t.name.value}))return null}return R(Object(i.b)(t,{FragmentSpread:{enter:n},FragmentDefinition:{enter:n}}))}(s,l)),l}function z(e){return Object(i.b)(T(e),{SelectionSet:{enter:function(e,t,n){if(!n||"OperationDefinition"!==n.kind){var i=e.selections;if(i)if(!i.some(function(e){return"Field"===e.kind&&("__typename"===e.name.value||0===e.name.value.lastIndexOf("__",0))}))return Object(a.a)({},e,{selections:i.concat([L])})}}}})}var Y={test:function(e){var t="connection"===e.name.value;return t&&(e.arguments&&e.arguments.some(function(e){return"key"===e.name.value})||console.warn("Removing an @connection directive even though it does not have a key. You may want to use the key parameter to specify a store key.")),t}};function H(e){return j([Y],T(e))}function W(e){return"query"===M(e).operation?e:Object(i.b)(e,{OperationDefinition:{enter:function(e){return Object(a.a)({},e,{operation:"query"})}}})}function X(e){T(e);var t=j([{test:function(e){return"client"===e.name.value},remove:!0}],e);return t&&(t=Object(i.b)(t,{FragmentDefinition:{enter:function(e){if(e.selectionSet&&e.selectionSet.selections.every(function(e){return"Field"===e.kind&&"__typename"===e.name.value}))return null}}})),t}var V=Object.prototype.toString;function B(e){return function e(t,n){switch(V.call(t)){case"[object Array]":if(n.has(t))return n.get(t);var i=t.slice(0);return n.set(t,i),i.forEach(function(t,r){i[r]=e(t,n)}),i;case"[object Object]":if(n.has(t))return n.get(t);var r=Object.create(Object.getPrototypeOf(t));return n.set(t,r),Object.keys(t).forEach(function(i){r[i]=e(t[i],n)}),r;default:return t}}(e,new Map)}function q(t){return(void 0!==e?"production":"development")===t}function U(){return!0===q("production")}function G(){return!0===q("test")}function Q(e){try{return e()}catch(e){console.error&&console.error(e)}}function $(e){return e.errors&&e.errors.length}function Z(e,t){if(e===t)return!0;if(e instanceof Date&&t instanceof Date)return e.getTime()===t.getTime();if(null!=e&&"object"==typeof e&&null!=t&&"object"==typeof t){for(var n in e)if(Object.prototype.hasOwnProperty.call(e,n)){if(!Object.prototype.hasOwnProperty.call(t,n))return!1;if(!Z(e[n],t[n]))return!1}for(var n in t)if(Object.prototype.hasOwnProperty.call(t,n)&&!Object.prototype.hasOwnProperty.call(e,n))return!1;return!0}return!1}var K=Object.prototype.hasOwnProperty;function J(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return ee(e)}function ee(e){var t=e[0]||{},n=e.length;if(n>1){var i=[];t=ie(t,i);for(var r=1;r<n;++r)t=ne(t,e[r],i)}return t}function te(e){return null!==e&&"object"==typeof e}function ne(e,t,n){return te(t)&&te(e)?(Object.isExtensible&&!Object.isExtensible(e)&&(e=ie(e,n)),Object.keys(t).forEach(function(i){var r=t[i];if(K.call(e,i)){var a=e[i];r!==a&&(e[i]=ne(ie(a,n),r,n))}else e[i]=r}),e):t}function ie(e,t){return null!==e&&"object"==typeof e&&t.indexOf(e)<0&&(e=Array.isArray(e)?e.slice(0):Object(a.a)({__proto__:Object.getPrototypeOf(e)},e),t.push(e)),e}Object.create({})}).call(this,n(15))},function(e,t,n){"use strict";n.d(t,"a",function(){return s}),n.d(t,"b",function(){return l});var i=n(1),r="Invariant Violation",a=Object.setPrototypeOf,o=void 0===a?function(e,t){return e.__proto__=t,e}:a,s=function(e){function t(n){void 0===n&&(n=r);var i=e.call(this,n)||this;return i.framesToPop=1,i.name=r,o(i,t.prototype),i}return Object(i.c)(t,e),t}(Error);function l(e,t){if(!e)throw new s(t)}!function(e){e.warn=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return console.warn.apply(console,e)},e.error=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return console.error.apply(console,e)}}(l||(l={}))},function(e,t,n){e.exports=n(87)()},function(e,t,n){"use strict";n.d(t,"a",function(){return a}),n.d(t,"b",function(){return o});var i=n(26),r={Name:[],Document:["definitions"],OperationDefinition:["name","variableDefinitions","directives","selectionSet"],VariableDefinition:["variable","type","defaultValue","directives"],Variable:["name"],SelectionSet:["selections"],Field:["alias","name","arguments","directives","selectionSet"],Argument:["name","value"],FragmentSpread:["name","directives"],InlineFragment:["typeCondition","directives","selectionSet"],FragmentDefinition:["name","variableDefinitions","typeCondition","directives","selectionSet"],IntValue:[],FloatValue:[],StringValue:[],BooleanValue:[],NullValue:[],EnumValue:[],ListValue:["values"],ObjectValue:["fields"],ObjectField:["name","value"],Directive:["name","arguments"],NamedType:["name"],ListType:["type"],NonNullType:["type"],SchemaDefinition:["directives","operationTypes"],OperationTypeDefinition:["type"],ScalarTypeDefinition:["description","name","directives"],ObjectTypeDefinition:["description","name","interfaces","directives","fields"],FieldDefinition:["description","name","arguments","type","directives"],InputValueDefinition:["description","name","type","defaultValue","directives"],InterfaceTypeDefinition:["description","name","directives","fields"],UnionTypeDefinition:["description","name","directives","types"],EnumTypeDefinition:["description","name","directives","values"],EnumValueDefinition:["description","name","directives"],InputObjectTypeDefinition:["description","name","directives","fields"],DirectiveDefinition:["description","name","arguments","locations"],SchemaExtension:["directives","operationTypes"],ScalarTypeExtension:["name","directives"],ObjectTypeExtension:["name","interfaces","directives","fields"],InterfaceTypeExtension:["name","directives","fields"],UnionTypeExtension:["name","directives","types"],EnumTypeExtension:["name","directives","values"],InputObjectTypeExtension:["name","directives","fields"]},a={};function o(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:r,o=void 0,c=Array.isArray(e),u=[e],h=-1,d=[],f=void 0,p=void 0,g=void 0,m=[],v=[],y=e;do{var b=++h===u.length,x=b&&0!==d.length;if(b){if(p=0===v.length?void 0:m[m.length-1],f=g,g=v.pop(),x){if(c)f=f.slice();else{for(var w={},k=Object.keys(f),S=0;S<k.length;S++){var E=k[S];w[E]=f[E]}f=w}for(var C=0,T=0;T<d.length;T++){var A=d[T][0],_=d[T][1];c&&(A-=C),c&&null===_?(f.splice(A,1),C++):f[A]=_}}h=o.index,u=o.keys,d=o.edits,c=o.inArray,o=o.prev}else{if(p=g?c?h:u[h]:void 0,null==(f=g?g[p]:y))continue;g&&m.push(p)}var O=void 0;if(!Array.isArray(f)){if(!s(f))throw new Error("Invalid AST Node: "+Object(i.a)(f));var P=l(t,f.kind,b);if(P){if((O=P.call(t,f,p,g,m,v))===a)break;if(!1===O){if(!b){m.pop();continue}}else if(void 0!==O&&(d.push([p,O]),!b)){if(!s(O)){m.pop();continue}f=O}}}void 0===O&&x&&d.push([p,f]),b?m.pop():(o={inArray:c,index:h,keys:u,edits:d,prev:o},u=(c=Array.isArray(f))?f:n[f.kind]||[],h=-1,d=[],g&&v.push(g),g=f)}while(void 0!==o);return 0!==d.length&&(y=d[d.length-1][1]),y}function s(e){return Boolean(e&&"string"==typeof e.kind)}function l(e,t,n){var i=e[t];if(i){if(!n&&"function"==typeof i)return i;var r=n?i.leave:i.enter;if("function"==typeof r)return r}else{var a=n?e.leave:e.enter;if(a){if("function"==typeof a)return a;var o=a[t];if("function"==typeof o)return o}}}},function(e,t,n){(function(e){e.exports=function(){"use strict";var t,n;function i(){return t.apply(null,arguments)}function r(e){return e instanceof Array||"[object Array]"===Object.prototype.toString.call(e)}function a(e){return null!=e&&"[object Object]"===Object.prototype.toString.call(e)}function o(e){return void 0===e}function s(e){return"number"==typeof e||"[object Number]"===Object.prototype.toString.call(e)}function l(e){return e instanceof Date||"[object Date]"===Object.prototype.toString.call(e)}function c(e,t){var n,i=[];for(n=0;n<e.length;++n)i.push(t(e[n],n));return i}function u(e,t){return Object.prototype.hasOwnProperty.call(e,t)}function h(e,t){for(var n in t)u(t,n)&&(e[n]=t[n]);return u(t,"toString")&&(e.toString=t.toString),u(t,"valueOf")&&(e.valueOf=t.valueOf),e}function d(e,t,n,i){return At(e,t,n,i,!0).utc()}function f(e){return null==e._pf&&(e._pf={empty:!1,unusedTokens:[],unusedInput:[],overflow:-2,charsLeftOver:0,nullInput:!1,invalidMonth:null,invalidFormat:!1,userInvalidated:!1,iso:!1,parsedDateParts:[],meridiem:null,rfc2822:!1,weekdayMismatch:!1}),e._pf}function p(e){if(null==e._isValid){var t=f(e),i=n.call(t.parsedDateParts,function(e){return null!=e}),r=!isNaN(e._d.getTime())&&t.overflow<0&&!t.empty&&!t.invalidMonth&&!t.invalidWeekday&&!t.weekdayMismatch&&!t.nullInput&&!t.invalidFormat&&!t.userInvalidated&&(!t.meridiem||t.meridiem&&i);if(e._strict&&(r=r&&0===t.charsLeftOver&&0===t.unusedTokens.length&&void 0===t.bigHour),null!=Object.isFrozen&&Object.isFrozen(e))return r;e._isValid=r}return e._isValid}function g(e){var t=d(NaN);return null!=e?h(f(t),e):f(t).userInvalidated=!0,t}n=Array.prototype.some?Array.prototype.some:function(e){for(var t=Object(this),n=t.length>>>0,i=0;i<n;i++)if(i in t&&e.call(this,t[i],i,t))return!0;return!1};var m=i.momentProperties=[];function v(e,t){var n,i,r;if(o(t._isAMomentObject)||(e._isAMomentObject=t._isAMomentObject),o(t._i)||(e._i=t._i),o(t._f)||(e._f=t._f),o(t._l)||(e._l=t._l),o(t._strict)||(e._strict=t._strict),o(t._tzm)||(e._tzm=t._tzm),o(t._isUTC)||(e._isUTC=t._isUTC),o(t._offset)||(e._offset=t._offset),o(t._pf)||(e._pf=f(t)),o(t._locale)||(e._locale=t._locale),0<m.length)for(n=0;n<m.length;n++)o(r=t[i=m[n]])||(e[i]=r);return e}var y=!1;function b(e){v(this,e),this._d=new Date(null!=e._d?e._d.getTime():NaN),this.isValid()||(this._d=new Date(NaN)),!1===y&&(y=!0,i.updateOffset(this),y=!1)}function x(e){return e instanceof b||null!=e&&null!=e._isAMomentObject}function w(e){return e<0?Math.ceil(e)||0:Math.floor(e)}function k(e){var t=+e,n=0;return 0!==t&&isFinite(t)&&(n=w(t)),n}function S(e,t,n){var i,r=Math.min(e.length,t.length),a=Math.abs(e.length-t.length),o=0;for(i=0;i<r;i++)(n&&e[i]!==t[i]||!n&&k(e[i])!==k(t[i]))&&o++;return o+a}function E(e){!1===i.suppressDeprecationWarnings&&"undefined"!=typeof console&&console.warn&&console.warn("Deprecation warning: "+e)}function C(e,t){var n=!0;return h(function(){if(null!=i.deprecationHandler&&i.deprecationHandler(null,e),n){for(var r,a=[],o=0;o<arguments.length;o++){if(r="","object"==typeof arguments[o]){for(var s in r+="\n["+o+"] ",arguments[0])r+=s+": "+arguments[0][s]+", ";r=r.slice(0,-2)}else r=arguments[o];a.push(r)}E(e+"\nArguments: "+Array.prototype.slice.call(a).join("")+"\n"+(new Error).stack),n=!1}return t.apply(this,arguments)},t)}var T,A={};function _(e,t){null!=i.deprecationHandler&&i.deprecationHandler(e,t),A[e]||(E(t),A[e]=!0)}function O(e){return e instanceof Function||"[object Function]"===Object.prototype.toString.call(e)}function P(e,t){var n,i=h({},e);for(n in t)u(t,n)&&(a(e[n])&&a(t[n])?(i[n]={},h(i[n],e[n]),h(i[n],t[n])):null!=t[n]?i[n]=t[n]:delete i[n]);for(n in e)u(e,n)&&!u(t,n)&&a(e[n])&&(i[n]=h({},i[n]));return i}function M(e){null!=e&&this.set(e)}i.suppressDeprecationWarnings=!1,i.deprecationHandler=null,T=Object.keys?Object.keys:function(e){var t,n=[];for(t in e)u(e,t)&&n.push(t);return n};var I={};function D(e,t){var n=e.toLowerCase();I[n]=I[n+"s"]=I[t]=e}function N(e){return"string"==typeof e?I[e]||I[e.toLowerCase()]:void 0}function L(e){var t,n,i={};for(n in e)u(e,n)&&(t=N(n))&&(i[t]=e[n]);return i}var R={};function F(e,t){R[e]=t}function j(e,t,n){var i=""+Math.abs(e),r=t-i.length;return(0<=e?n?"+":"":"-")+Math.pow(10,Math.max(0,r)).toString().substr(1)+i}var z=/(\[[^\[]*\])|(\\)?([Hh]mm(ss)?|Mo|MM?M?M?|Do|DDDo|DD?D?D?|ddd?d?|do?|w[o|w]?|W[o|W]?|Qo?|YYYYYY|YYYYY|YYYY|YY|gg(ggg?)?|GG(GGG?)?|e|E|a|A|hh?|HH?|kk?|mm?|ss?|S{1,9}|x|X|zz?|ZZ?|.)/g,Y=/(\[[^\[]*\])|(\\)?(LTS|LT|LL?L?L?|l{1,4})/g,H={},W={};function X(e,t,n,i){var r=i;"string"==typeof i&&(r=function(){return this[i]()}),e&&(W[e]=r),t&&(W[t[0]]=function(){return j(r.apply(this,arguments),t[1],t[2])}),n&&(W[n]=function(){return this.localeData().ordinal(r.apply(this,arguments),e)})}function V(e,t){return e.isValid()?(t=B(t,e.localeData()),H[t]=H[t]||function(e){var t,n,i,r=e.match(z);for(t=0,n=r.length;t<n;t++)W[r[t]]?r[t]=W[r[t]]:r[t]=(i=r[t]).match(/\[[\s\S]/)?i.replace(/^\[|\]$/g,""):i.replace(/\\/g,"");return function(t){var i,a="";for(i=0;i<n;i++)a+=O(r[i])?r[i].call(t,e):r[i];return a}}(t),H[t](e)):e.localeData().invalidDate()}function B(e,t){var n=5;function i(e){return t.longDateFormat(e)||e}for(Y.lastIndex=0;0<=n&&Y.test(e);)e=e.replace(Y,i),Y.lastIndex=0,n-=1;return e}var q=/\d/,U=/\d\d/,G=/\d{3}/,Q=/\d{4}/,$=/[+-]?\d{6}/,Z=/\d\d?/,K=/\d\d\d\d?/,J=/\d\d\d\d\d\d?/,ee=/\d{1,3}/,te=/\d{1,4}/,ne=/[+-]?\d{1,6}/,ie=/\d+/,re=/[+-]?\d+/,ae=/Z|[+-]\d\d:?\d\d/gi,oe=/Z|[+-]\d\d(?::?\d\d)?/gi,se=/[0-9]{0,256}['a-z\u00A0-\u05FF\u0700-\uD7FF\uF900-\uFDCF\uFDF0-\uFF07\uFF10-\uFFEF]{1,256}|[\u0600-\u06FF\/]{1,256}(\s*?[\u0600-\u06FF]{1,256}){1,2}/i,le={};function ce(e,t,n){le[e]=O(t)?t:function(e,i){return e&&n?n:t}}function ue(e,t){return u(le,e)?le[e](t._strict,t._locale):new RegExp(he(e.replace("\\","").replace(/\\(\[)|\\(\])|\[([^\]\[]*)\]|\\(.)/g,function(e,t,n,i,r){return t||n||i||r})))}function he(e){return e.replace(/[-\/\\^$*+?.()|[\]{}]/g,"\\$&")}var de={};function fe(e,t){var n,i=t;for("string"==typeof e&&(e=[e]),s(t)&&(i=function(e,n){n[t]=k(e)}),n=0;n<e.length;n++)de[e[n]]=i}function pe(e,t){fe(e,function(e,n,i,r){i._w=i._w||{},t(e,i._w,i,r)})}var ge=0,me=1,ve=2,ye=3,be=4,xe=5,we=6,ke=7,Se=8;function Ee(e){return Ce(e)?366:365}function Ce(e){return e%4==0&&e%100!=0||e%400==0}X("Y",0,0,function(){var e=this.year();return e<=9999?""+e:"+"+e}),X(0,["YY",2],0,function(){return this.year()%100}),X(0,["YYYY",4],0,"year"),X(0,["YYYYY",5],0,"year"),X(0,["YYYYYY",6,!0],0,"year"),D("year","y"),F("year",1),ce("Y",re),ce("YY",Z,U),ce("YYYY",te,Q),ce("YYYYY",ne,$),ce("YYYYYY",ne,$),fe(["YYYYY","YYYYYY"],ge),fe("YYYY",function(e,t){t[ge]=2===e.length?i.parseTwoDigitYear(e):k(e)}),fe("YY",function(e,t){t[ge]=i.parseTwoDigitYear(e)}),fe("Y",function(e,t){t[ge]=parseInt(e,10)}),i.parseTwoDigitYear=function(e){return k(e)+(68<k(e)?1900:2e3)};var Te,Ae=_e("FullYear",!0);function _e(e,t){return function(n){return null!=n?(Pe(this,e,n),i.updateOffset(this,t),this):Oe(this,e)}}function Oe(e,t){return e.isValid()?e._d["get"+(e._isUTC?"UTC":"")+t]():NaN}function Pe(e,t,n){e.isValid()&&!isNaN(n)&&("FullYear"===t&&Ce(e.year())&&1===e.month()&&29===e.date()?e._d["set"+(e._isUTC?"UTC":"")+t](n,e.month(),Me(n,e.month())):e._d["set"+(e._isUTC?"UTC":"")+t](n))}function Me(e,t){if(isNaN(e)||isNaN(t))return NaN;var n=(t%12+12)%12;return e+=(t-n)/12,1===n?Ce(e)?29:28:31-n%7%2}Te=Array.prototype.indexOf?Array.prototype.indexOf:function(e){var t;for(t=0;t<this.length;++t)if(this[t]===e)return t;return-1},X("M",["MM",2],"Mo",function(){return this.month()+1}),X("MMM",0,0,function(e){return this.localeData().monthsShort(this,e)}),X("MMMM",0,0,function(e){return this.localeData().months(this,e)}),D("month","M"),F("month",8),ce("M",Z),ce("MM",Z,U),ce("MMM",function(e,t){return t.monthsShortRegex(e)}),ce("MMMM",function(e,t){return t.monthsRegex(e)}),fe(["M","MM"],function(e,t){t[me]=k(e)-1}),fe(["MMM","MMMM"],function(e,t,n,i){var r=n._locale.monthsParse(e,i,n._strict);null!=r?t[me]=r:f(n).invalidMonth=e});var Ie=/D[oD]?(\[[^\[\]]*\]|\s)+MMMM?/,De="January_February_March_April_May_June_July_August_September_October_November_December".split("_"),Ne="Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_");function Le(e,t){var n;if(!e.isValid())return e;if("string"==typeof t)if(/^\d+$/.test(t))t=k(t);else if(!s(t=e.localeData().monthsParse(t)))return e;return n=Math.min(e.date(),Me(e.year(),t)),e._d["set"+(e._isUTC?"UTC":"")+"Month"](t,n),e}function Re(e){return null!=e?(Le(this,e),i.updateOffset(this,!0),this):Oe(this,"Month")}var Fe=se,je=se;function ze(){function e(e,t){return t.length-e.length}var t,n,i=[],r=[],a=[];for(t=0;t<12;t++)n=d([2e3,t]),i.push(this.monthsShort(n,"")),r.push(this.months(n,"")),a.push(this.months(n,"")),a.push(this.monthsShort(n,""));for(i.sort(e),r.sort(e),a.sort(e),t=0;t<12;t++)i[t]=he(i[t]),r[t]=he(r[t]);for(t=0;t<24;t++)a[t]=he(a[t]);this._monthsRegex=new RegExp("^("+a.join("|")+")","i"),this._monthsShortRegex=this._monthsRegex,this._monthsStrictRegex=new RegExp("^("+r.join("|")+")","i"),this._monthsShortStrictRegex=new RegExp("^("+i.join("|")+")","i")}function Ye(e){var t;if(e<100&&0<=e){var n=Array.prototype.slice.call(arguments);n[0]=e+400,t=new Date(Date.UTC.apply(null,n)),isFinite(t.getUTCFullYear())&&t.setUTCFullYear(e)}else t=new Date(Date.UTC.apply(null,arguments));return t}function He(e,t,n){var i=7+t-n;return-(7+Ye(e,0,i).getUTCDay()-t)%7+i-1}function We(e,t,n,i,r){var a,o,s=1+7*(t-1)+(7+n-i)%7+He(e,i,r);return o=s<=0?Ee(a=e-1)+s:s>Ee(e)?(a=e+1,s-Ee(e)):(a=e,s),{year:a,dayOfYear:o}}function Xe(e,t,n){var i,r,a=He(e.year(),t,n),o=Math.floor((e.dayOfYear()-a-1)/7)+1;return o<1?i=o+Ve(r=e.year()-1,t,n):o>Ve(e.year(),t,n)?(i=o-Ve(e.year(),t,n),r=e.year()+1):(r=e.year(),i=o),{week:i,year:r}}function Ve(e,t,n){var i=He(e,t,n),r=He(e+1,t,n);return(Ee(e)-i+r)/7}function Be(e,t){return e.slice(t,7).concat(e.slice(0,t))}X("w",["ww",2],"wo","week"),X("W",["WW",2],"Wo","isoWeek"),D("week","w"),D("isoWeek","W"),F("week",5),F("isoWeek",5),ce("w",Z),ce("ww",Z,U),ce("W",Z),ce("WW",Z,U),pe(["w","ww","W","WW"],function(e,t,n,i){t[i.substr(0,1)]=k(e)}),X("d",0,"do","day"),X("dd",0,0,function(e){return this.localeData().weekdaysMin(this,e)}),X("ddd",0,0,function(e){return this.localeData().weekdaysShort(this,e)}),X("dddd",0,0,function(e){return this.localeData().weekdays(this,e)}),X("e",0,0,"weekday"),X("E",0,0,"isoWeekday"),D("day","d"),D("weekday","e"),D("isoWeekday","E"),F("day",11),F("weekday",11),F("isoWeekday",11),ce("d",Z),ce("e",Z),ce("E",Z),ce("dd",function(e,t){return t.weekdaysMinRegex(e)}),ce("ddd",function(e,t){return t.weekdaysShortRegex(e)}),ce("dddd",function(e,t){return t.weekdaysRegex(e)}),pe(["dd","ddd","dddd"],function(e,t,n,i){var r=n._locale.weekdaysParse(e,i,n._strict);null!=r?t.d=r:f(n).invalidWeekday=e}),pe(["d","e","E"],function(e,t,n,i){t[i]=k(e)});var qe="Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),Ue="Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),Ge="Su_Mo_Tu_We_Th_Fr_Sa".split("_"),Qe=se,$e=se,Ze=se;function Ke(){function e(e,t){return t.length-e.length}var t,n,i,r,a,o=[],s=[],l=[],c=[];for(t=0;t<7;t++)n=d([2e3,1]).day(t),i=this.weekdaysMin(n,""),r=this.weekdaysShort(n,""),a=this.weekdays(n,""),o.push(i),s.push(r),l.push(a),c.push(i),c.push(r),c.push(a);for(o.sort(e),s.sort(e),l.sort(e),c.sort(e),t=0;t<7;t++)s[t]=he(s[t]),l[t]=he(l[t]),c[t]=he(c[t]);this._weekdaysRegex=new RegExp("^("+c.join("|")+")","i"),this._weekdaysShortRegex=this._weekdaysRegex,this._weekdaysMinRegex=this._weekdaysRegex,this._weekdaysStrictRegex=new RegExp("^("+l.join("|")+")","i"),this._weekdaysShortStrictRegex=new RegExp("^("+s.join("|")+")","i"),this._weekdaysMinStrictRegex=new RegExp("^("+o.join("|")+")","i")}function Je(){return this.hours()%12||12}function et(e,t){X(e,0,0,function(){return this.localeData().meridiem(this.hours(),this.minutes(),t)})}function tt(e,t){return t._meridiemParse}X("H",["HH",2],0,"hour"),X("h",["hh",2],0,Je),X("k",["kk",2],0,function(){return this.hours()||24}),X("hmm",0,0,function(){return""+Je.apply(this)+j(this.minutes(),2)}),X("hmmss",0,0,function(){return""+Je.apply(this)+j(this.minutes(),2)+j(this.seconds(),2)}),X("Hmm",0,0,function(){return""+this.hours()+j(this.minutes(),2)}),X("Hmmss",0,0,function(){return""+this.hours()+j(this.minutes(),2)+j(this.seconds(),2)}),et("a",!0),et("A",!1),D("hour","h"),F("hour",13),ce("a",tt),ce("A",tt),ce("H",Z),ce("h",Z),ce("k",Z),ce("HH",Z,U),ce("hh",Z,U),ce("kk",Z,U),ce("hmm",K),ce("hmmss",J),ce("Hmm",K),ce("Hmmss",J),fe(["H","HH"],ye),fe(["k","kk"],function(e,t,n){var i=k(e);t[ye]=24===i?0:i}),fe(["a","A"],function(e,t,n){n._isPm=n._locale.isPM(e),n._meridiem=e}),fe(["h","hh"],function(e,t,n){t[ye]=k(e),f(n).bigHour=!0}),fe("hmm",function(e,t,n){var i=e.length-2;t[ye]=k(e.substr(0,i)),t[be]=k(e.substr(i)),f(n).bigHour=!0}),fe("hmmss",function(e,t,n){var i=e.length-4,r=e.length-2;t[ye]=k(e.substr(0,i)),t[be]=k(e.substr(i,2)),t[xe]=k(e.substr(r)),f(n).bigHour=!0}),fe("Hmm",function(e,t,n){var i=e.length-2;t[ye]=k(e.substr(0,i)),t[be]=k(e.substr(i))}),fe("Hmmss",function(e,t,n){var i=e.length-4,r=e.length-2;t[ye]=k(e.substr(0,i)),t[be]=k(e.substr(i,2)),t[xe]=k(e.substr(r))});var nt,it=_e("Hours",!0),rt={calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},longDateFormat:{LTS:"h:mm:ss A",LT:"h:mm A",L:"MM/DD/YYYY",LL:"MMMM D, YYYY",LLL:"MMMM D, YYYY h:mm A",LLLL:"dddd, MMMM D, YYYY h:mm A"},invalidDate:"Invalid date",ordinal:"%d",dayOfMonthOrdinalParse:/\d{1,2}/,relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},months:De,monthsShort:Ne,week:{dow:0,doy:6},weekdays:qe,weekdaysMin:Ge,weekdaysShort:Ue,meridiemParse:/[ap]\.?m?\.?/i},at={},ot={};function st(e){return e?e.toLowerCase().replace("_","-"):e}function lt(t){var n=null;if(!at[t]&&void 0!==e&&e&&e.exports)try{n=nt._abbr,function(){var e=new Error("Cannot find module 'undefined'");throw e.code="MODULE_NOT_FOUND",e}(),ct(n)}catch(t){}return at[t]}function ct(e,t){var n;return e&&((n=o(t)?ht(e):ut(e,t))?nt=n:"undefined"!=typeof console&&console.warn&&console.warn("Locale "+e+" not found. Did you forget to load it?")),nt._abbr}function ut(e,t){if(null===t)return delete at[e],null;var n,i=rt;if(t.abbr=e,null!=at[e])_("defineLocaleOverride","use moment.updateLocale(localeName, config) to change an existing locale. moment.defineLocale(localeName, config) should only be used for creating a new locale See http://momentjs.com/guides/#/warnings/define-locale/ for more info."),i=at[e]._config;else if(null!=t.parentLocale)if(null!=at[t.parentLocale])i=at[t.parentLocale]._config;else{if(null==(n=lt(t.parentLocale)))return ot[t.parentLocale]||(ot[t.parentLocale]=[]),ot[t.parentLocale].push({name:e,config:t}),null;i=n._config}return at[e]=new M(P(i,t)),ot[e]&&ot[e].forEach(function(e){ut(e.name,e.config)}),ct(e),at[e]}function ht(e){var t;if(e&&e._locale&&e._locale._abbr&&(e=e._locale._abbr),!e)return nt;if(!r(e)){if(t=lt(e))return t;e=[e]}return function(e){for(var t,n,i,r,a=0;a<e.length;){for(t=(r=st(e[a]).split("-")).length,n=(n=st(e[a+1]))?n.split("-"):null;0<t;){if(i=lt(r.slice(0,t).join("-")))return i;if(n&&n.length>=t&&S(r,n,!0)>=t-1)break;t--}a++}return nt}(e)}function dt(e){var t,n=e._a;return n&&-2===f(e).overflow&&(t=n[me]<0||11<n[me]?me:n[ve]<1||n[ve]>Me(n[ge],n[me])?ve:n[ye]<0||24<n[ye]||24===n[ye]&&(0!==n[be]||0!==n[xe]||0!==n[we])?ye:n[be]<0||59<n[be]?be:n[xe]<0||59<n[xe]?xe:n[we]<0||999<n[we]?we:-1,f(e)._overflowDayOfYear&&(t<ge||ve<t)&&(t=ve),f(e)._overflowWeeks&&-1===t&&(t=ke),f(e)._overflowWeekday&&-1===t&&(t=Se),f(e).overflow=t),e}function ft(e,t,n){return null!=e?e:null!=t?t:n}function pt(e){var t,n,r,a,o,s=[];if(!e._d){var l,c;for(l=e,c=new Date(i.now()),r=l._useUTC?[c.getUTCFullYear(),c.getUTCMonth(),c.getUTCDate()]:[c.getFullYear(),c.getMonth(),c.getDate()],e._w&&null==e._a[ve]&&null==e._a[me]&&function(e){var t,n,i,r,a,o,s,l;if(null!=(t=e._w).GG||null!=t.W||null!=t.E)a=1,o=4,n=ft(t.GG,e._a[ge],Xe(_t(),1,4).year),i=ft(t.W,1),((r=ft(t.E,1))<1||7<r)&&(l=!0);else{a=e._locale._week.dow,o=e._locale._week.doy;var c=Xe(_t(),a,o);n=ft(t.gg,e._a[ge],c.year),i=ft(t.w,c.week),null!=t.d?((r=t.d)<0||6<r)&&(l=!0):null!=t.e?(r=t.e+a,(t.e<0||6<t.e)&&(l=!0)):r=a}i<1||i>Ve(n,a,o)?f(e)._overflowWeeks=!0:null!=l?f(e)._overflowWeekday=!0:(s=We(n,i,r,a,o),e._a[ge]=s.year,e._dayOfYear=s.dayOfYear)}(e),null!=e._dayOfYear&&(o=ft(e._a[ge],r[ge]),(e._dayOfYear>Ee(o)||0===e._dayOfYear)&&(f(e)._overflowDayOfYear=!0),n=Ye(o,0,e._dayOfYear),e._a[me]=n.getUTCMonth(),e._a[ve]=n.getUTCDate()),t=0;t<3&&null==e._a[t];++t)e._a[t]=s[t]=r[t];for(;t<7;t++)e._a[t]=s[t]=null==e._a[t]?2===t?1:0:e._a[t];24===e._a[ye]&&0===e._a[be]&&0===e._a[xe]&&0===e._a[we]&&(e._nextDay=!0,e._a[ye]=0),e._d=(e._useUTC?Ye:function(e,t,n,i,r,a,o){var s;return e<100&&0<=e?(s=new Date(e+400,t,n,i,r,a,o),isFinite(s.getFullYear())&&s.setFullYear(e)):s=new Date(e,t,n,i,r,a,o),s}).apply(null,s),a=e._useUTC?e._d.getUTCDay():e._d.getDay(),null!=e._tzm&&e._d.setUTCMinutes(e._d.getUTCMinutes()-e._tzm),e._nextDay&&(e._a[ye]=24),e._w&&void 0!==e._w.d&&e._w.d!==a&&(f(e).weekdayMismatch=!0)}}var gt=/^\s*((?:[+-]\d{6}|\d{4})-(?:\d\d-\d\d|W\d\d-\d|W\d\d|\d\d\d|\d\d))(?:(T| )(\d\d(?::\d\d(?::\d\d(?:[.,]\d+)?)?)?)([\+\-]\d\d(?::?\d\d)?|\s*Z)?)?$/,mt=/^\s*((?:[+-]\d{6}|\d{4})(?:\d\d\d\d|W\d\d\d|W\d\d|\d\d\d|\d\d))(?:(T| )(\d\d(?:\d\d(?:\d\d(?:[.,]\d+)?)?)?)([\+\-]\d\d(?::?\d\d)?|\s*Z)?)?$/,vt=/Z|[+-]\d\d(?::?\d\d)?/,yt=[["YYYYYY-MM-DD",/[+-]\d{6}-\d\d-\d\d/],["YYYY-MM-DD",/\d{4}-\d\d-\d\d/],["GGGG-[W]WW-E",/\d{4}-W\d\d-\d/],["GGGG-[W]WW",/\d{4}-W\d\d/,!1],["YYYY-DDD",/\d{4}-\d{3}/],["YYYY-MM",/\d{4}-\d\d/,!1],["YYYYYYMMDD",/[+-]\d{10}/],["YYYYMMDD",/\d{8}/],["GGGG[W]WWE",/\d{4}W\d{3}/],["GGGG[W]WW",/\d{4}W\d{2}/,!1],["YYYYDDD",/\d{7}/]],bt=[["HH:mm:ss.SSSS",/\d\d:\d\d:\d\d\.\d+/],["HH:mm:ss,SSSS",/\d\d:\d\d:\d\d,\d+/],["HH:mm:ss",/\d\d:\d\d:\d\d/],["HH:mm",/\d\d:\d\d/],["HHmmss.SSSS",/\d\d\d\d\d\d\.\d+/],["HHmmss,SSSS",/\d\d\d\d\d\d,\d+/],["HHmmss",/\d\d\d\d\d\d/],["HHmm",/\d\d\d\d/],["HH",/\d\d/]],xt=/^\/?Date\((\-?\d+)/i;function wt(e){var t,n,i,r,a,o,s=e._i,l=gt.exec(s)||mt.exec(s);if(l){for(f(e).iso=!0,t=0,n=yt.length;t<n;t++)if(yt[t][1].exec(l[1])){r=yt[t][0],i=!1!==yt[t][2];break}if(null==r)return void(e._isValid=!1);if(l[3]){for(t=0,n=bt.length;t<n;t++)if(bt[t][1].exec(l[3])){a=(l[2]||" ")+bt[t][0];break}if(null==a)return void(e._isValid=!1)}if(!i&&null!=a)return void(e._isValid=!1);if(l[4]){if(!vt.exec(l[4]))return void(e._isValid=!1);o="Z"}e._f=r+(a||"")+(o||""),Ct(e)}else e._isValid=!1}var kt=/^(?:(Mon|Tue|Wed|Thu|Fri|Sat|Sun),?\s)?(\d{1,2})\s(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)\s(\d{2,4})\s(\d\d):(\d\d)(?::(\d\d))?\s(?:(UT|GMT|[ECMP][SD]T)|([Zz])|([+-]\d{4}))$/,St={UT:0,GMT:0,EDT:-240,EST:-300,CDT:-300,CST:-360,MDT:-360,MST:-420,PDT:-420,PST:-480};function Et(e){var t,n,i,r=kt.exec(e._i.replace(/\([^)]*\)|[\n\t]/g," ").replace(/(\s\s+)/g," ").replace(/^\s\s*/,"").replace(/\s\s*$/,""));if(r){var a=function(e,t,n,i,r,a){var o=[function(e){var t=parseInt(e,10);return t<=49?2e3+t:t<=999?1900+t:t}(e),Ne.indexOf(t),parseInt(n,10),parseInt(i,10),parseInt(r,10)];return a&&o.push(parseInt(a,10)),o}(r[4],r[3],r[2],r[5],r[6],r[7]);if(t=r[1],n=a,i=e,t&&Ue.indexOf(t)!==new Date(n[0],n[1],n[2]).getDay()&&(f(i).weekdayMismatch=!0,!(i._isValid=!1)))return;e._a=a,e._tzm=function(e,t,n){if(e)return St[e];if(t)return 0;var i=parseInt(n,10),r=i%100;return(i-r)/100*60+r}(r[8],r[9],r[10]),e._d=Ye.apply(null,e._a),e._d.setUTCMinutes(e._d.getUTCMinutes()-e._tzm),f(e).rfc2822=!0}else e._isValid=!1}function Ct(e){if(e._f!==i.ISO_8601)if(e._f!==i.RFC_2822){e._a=[],f(e).empty=!0;var t,n,r,a,o,s,l,c,h=""+e._i,d=h.length,p=0;for(r=B(e._f,e._locale).match(z)||[],t=0;t<r.length;t++)a=r[t],(n=(h.match(ue(a,e))||[])[0])&&(0<(o=h.substr(0,h.indexOf(n))).length&&f(e).unusedInput.push(o),h=h.slice(h.indexOf(n)+n.length),p+=n.length),W[a]?(n?f(e).empty=!1:f(e).unusedTokens.push(a),s=a,c=e,null!=(l=n)&&u(de,s)&&de[s](l,c._a,c,s)):e._strict&&!n&&f(e).unusedTokens.push(a);f(e).charsLeftOver=d-p,0<h.length&&f(e).unusedInput.push(h),e._a[ye]<=12&&!0===f(e).bigHour&&0<e._a[ye]&&(f(e).bigHour=void 0),f(e).parsedDateParts=e._a.slice(0),f(e).meridiem=e._meridiem,e._a[ye]=function(e,t,n){var i;return null==n?t:null!=e.meridiemHour?e.meridiemHour(t,n):(null!=e.isPM&&((i=e.isPM(n))&&t<12&&(t+=12),i||12!==t||(t=0)),t)}(e._locale,e._a[ye],e._meridiem),pt(e),dt(e)}else Et(e);else wt(e)}function Tt(e){var t,n,u,d,m=e._i,y=e._f;return e._locale=e._locale||ht(e._l),null===m||void 0===y&&""===m?g({nullInput:!0}):("string"==typeof m&&(e._i=m=e._locale.preparse(m)),x(m)?new b(dt(m)):(l(m)?e._d=m:r(y)?function(e){var t,n,i,r,a;if(0===e._f.length)return f(e).invalidFormat=!0,e._d=new Date(NaN);for(r=0;r<e._f.length;r++)a=0,t=v({},e),null!=e._useUTC&&(t._useUTC=e._useUTC),t._f=e._f[r],Ct(t),p(t)&&(a+=f(t).charsLeftOver,a+=10*f(t).unusedTokens.length,f(t).score=a,(null==i||a<i)&&(i=a,n=t));h(e,n||t)}(e):y?Ct(e):o(n=(t=e)._i)?t._d=new Date(i.now()):l(n)?t._d=new Date(n.valueOf()):"string"==typeof n?(u=t,null===(d=xt.exec(u._i))?(wt(u),!1===u._isValid&&(delete u._isValid,Et(u),!1===u._isValid&&(delete u._isValid,i.createFromInputFallback(u)))):u._d=new Date(+d[1])):r(n)?(t._a=c(n.slice(0),function(e){return parseInt(e,10)}),pt(t)):a(n)?function(e){if(!e._d){var t=L(e._i);e._a=c([t.year,t.month,t.day||t.date,t.hour,t.minute,t.second,t.millisecond],function(e){return e&&parseInt(e,10)}),pt(e)}}(t):s(n)?t._d=new Date(n):i.createFromInputFallback(t),p(e)||(e._d=null),e))}function At(e,t,n,i,o){var s,l={};return!0!==n&&!1!==n||(i=n,n=void 0),(a(e)&&function(e){if(Object.getOwnPropertyNames)return 0===Object.getOwnPropertyNames(e).length;var t;for(t in e)if(e.hasOwnProperty(t))return!1;return!0}(e)||r(e)&&0===e.length)&&(e=void 0),l._isAMomentObject=!0,l._useUTC=l._isUTC=o,l._l=n,l._i=e,l._f=t,l._strict=i,(s=new b(dt(Tt(l))))._nextDay&&(s.add(1,"d"),s._nextDay=void 0),s}function _t(e,t,n,i){return At(e,t,n,i,!1)}i.createFromInputFallback=C("value provided is not in a recognized RFC2822 or ISO format. moment construction falls back to js Date(), which is not reliable across all browsers and versions. Non RFC2822/ISO date formats are discouraged and will be removed in an upcoming major release. Please refer to http://momentjs.com/guides/#/warnings/js-date/ for more info.",function(e){e._d=new Date(e._i+(e._useUTC?" UTC":""))}),i.ISO_8601=function(){},i.RFC_2822=function(){};var Ot=C("moment().min is deprecated, use moment.max instead. http://momentjs.com/guides/#/warnings/min-max/",function(){var e=_t.apply(null,arguments);return this.isValid()&&e.isValid()?e<this?this:e:g()}),Pt=C("moment().max is deprecated, use moment.min instead. http://momentjs.com/guides/#/warnings/min-max/",function(){var e=_t.apply(null,arguments);return this.isValid()&&e.isValid()?this<e?this:e:g()});function Mt(e,t){var n,i;if(1===t.length&&r(t[0])&&(t=t[0]),!t.length)return _t();for(n=t[0],i=1;i<t.length;++i)t[i].isValid()&&!t[i][e](n)||(n=t[i]);return n}var It=["year","quarter","month","week","day","hour","minute","second","millisecond"];function Dt(e){var t=L(e),n=t.year||0,i=t.quarter||0,r=t.month||0,a=t.week||t.isoWeek||0,o=t.day||0,s=t.hour||0,l=t.minute||0,c=t.second||0,u=t.millisecond||0;this._isValid=function(e){for(var t in e)if(-1===Te.call(It,t)||null!=e[t]&&isNaN(e[t]))return!1;for(var n=!1,i=0;i<It.length;++i)if(e[It[i]]){if(n)return!1;parseFloat(e[It[i]])!==k(e[It[i]])&&(n=!0)}return!0}(t),this._milliseconds=+u+1e3*c+6e4*l+1e3*s*60*60,this._days=+o+7*a,this._months=+r+3*i+12*n,this._data={},this._locale=ht(),this._bubble()}function Nt(e){return e instanceof Dt}function Lt(e){return e<0?-1*Math.round(-1*e):Math.round(e)}function Rt(e,t){X(e,0,0,function(){var e=this.utcOffset(),n="+";return e<0&&(e=-e,n="-"),n+j(~~(e/60),2)+t+j(~~e%60,2)})}Rt("Z",":"),Rt("ZZ",""),ce("Z",oe),ce("ZZ",oe),fe(["Z","ZZ"],function(e,t,n){n._useUTC=!0,n._tzm=jt(oe,e)});var Ft=/([\+\-]|\d\d)/gi;function jt(e,t){var n=(t||"").match(e);if(null===n)return null;var i=((n[n.length-1]||[])+"").match(Ft)||["-",0,0],r=60*i[1]+k(i[2]);return 0===r?0:"+"===i[0]?r:-r}function zt(e,t){var n,r;return t._isUTC?(n=t.clone(),r=(x(e)||l(e)?e.valueOf():_t(e).valueOf())-n.valueOf(),n._d.setTime(n._d.valueOf()+r),i.updateOffset(n,!1),n):_t(e).local()}function Yt(e){return 15*-Math.round(e._d.getTimezoneOffset()/15)}function Ht(){return!!this.isValid()&&this._isUTC&&0===this._offset}i.updateOffset=function(){};var Wt=/^(\-|\+)?(?:(\d*)[. ])?(\d+)\:(\d+)(?:\:(\d+)(\.\d*)?)?$/,Xt=/^(-|\+)?P(?:([-+]?[0-9,.]*)Y)?(?:([-+]?[0-9,.]*)M)?(?:([-+]?[0-9,.]*)W)?(?:([-+]?[0-9,.]*)D)?(?:T(?:([-+]?[0-9,.]*)H)?(?:([-+]?[0-9,.]*)M)?(?:([-+]?[0-9,.]*)S)?)?$/;function Vt(e,t){var n,i,r,a=e,o=null;return Nt(e)?a={ms:e._milliseconds,d:e._days,M:e._months}:s(e)?(a={},t?a[t]=e:a.milliseconds=e):(o=Wt.exec(e))?(n="-"===o[1]?-1:1,a={y:0,d:k(o[ve])*n,h:k(o[ye])*n,m:k(o[be])*n,s:k(o[xe])*n,ms:k(Lt(1e3*o[we]))*n}):(o=Xt.exec(e))?(n="-"===o[1]?-1:1,a={y:Bt(o[2],n),M:Bt(o[3],n),w:Bt(o[4],n),d:Bt(o[5],n),h:Bt(o[6],n),m:Bt(o[7],n),s:Bt(o[8],n)}):null==a?a={}:"object"==typeof a&&("from"in a||"to"in a)&&(r=function(e,t){var n;return e.isValid()&&t.isValid()?(t=zt(t,e),e.isBefore(t)?n=qt(e,t):((n=qt(t,e)).milliseconds=-n.milliseconds,n.months=-n.months),n):{milliseconds:0,months:0}}(_t(a.from),_t(a.to)),(a={}).ms=r.milliseconds,a.M=r.months),i=new Dt(a),Nt(e)&&u(e,"_locale")&&(i._locale=e._locale),i}function Bt(e,t){var n=e&&parseFloat(e.replace(",","."));return(isNaN(n)?0:n)*t}function qt(e,t){var n={};return n.months=t.month()-e.month()+12*(t.year()-e.year()),e.clone().add(n.months,"M").isAfter(t)&&--n.months,n.milliseconds=+t-+e.clone().add(n.months,"M"),n}function Ut(e,t){return function(n,i){var r;return null===i||isNaN(+i)||(_(t,"moment()."+t+"(period, number) is deprecated. Please use moment()."+t+"(number, period). See http://momentjs.com/guides/#/warnings/add-inverted-param/ for more info."),r=n,n=i,i=r),Gt(this,Vt(n="string"==typeof n?+n:n,i),e),this}}function Gt(e,t,n,r){var a=t._milliseconds,o=Lt(t._days),s=Lt(t._months);e.isValid()&&(r=null==r||r,s&&Le(e,Oe(e,"Month")+s*n),o&&Pe(e,"Date",Oe(e,"Date")+o*n),a&&e._d.setTime(e._d.valueOf()+a*n),r&&i.updateOffset(e,o||s))}Vt.fn=Dt.prototype,Vt.invalid=function(){return Vt(NaN)};var Qt=Ut(1,"add"),$t=Ut(-1,"subtract");function Zt(e,t){var n=12*(t.year()-e.year())+(t.month()-e.month()),i=e.clone().add(n,"months");return-(n+(t-i<0?(t-i)/(i-e.clone().add(n-1,"months")):(t-i)/(e.clone().add(n+1,"months")-i)))||0}function Kt(e){var t;return void 0===e?this._locale._abbr:(null!=(t=ht(e))&&(this._locale=t),this)}i.defaultFormat="YYYY-MM-DDTHH:mm:ssZ",i.defaultFormatUtc="YYYY-MM-DDTHH:mm:ss[Z]";var Jt=C("moment().lang() is deprecated. Instead, use moment().localeData() to get the language configuration. Use moment().locale() to change languages.",function(e){return void 0===e?this.localeData():this.locale(e)});function en(){return this._locale}var tn=126227808e5;function nn(e,t){return(e%t+t)%t}function rn(e,t,n){return e<100&&0<=e?new Date(e+400,t,n)-tn:new Date(e,t,n).valueOf()}function an(e,t,n){return e<100&&0<=e?Date.UTC(e+400,t,n)-tn:Date.UTC(e,t,n)}function on(e,t){X(0,[e,e.length],0,t)}function sn(e,t,n,i,r){var a;return null==e?Xe(this,i,r).year:((a=Ve(e,i,r))<t&&(t=a),function(e,t,n,i,r){var a=We(e,t,n,i,r),o=Ye(a.year,0,a.dayOfYear);return this.year(o.getUTCFullYear()),this.month(o.getUTCMonth()),this.date(o.getUTCDate()),this}.call(this,e,t,n,i,r))}X(0,["gg",2],0,function(){return this.weekYear()%100}),X(0,["GG",2],0,function(){return this.isoWeekYear()%100}),on("gggg","weekYear"),on("ggggg","weekYear"),on("GGGG","isoWeekYear"),on("GGGGG","isoWeekYear"),D("weekYear","gg"),D("isoWeekYear","GG"),F("weekYear",1),F("isoWeekYear",1),ce("G",re),ce("g",re),ce("GG",Z,U),ce("gg",Z,U),ce("GGGG",te,Q),ce("gggg",te,Q),ce("GGGGG",ne,$),ce("ggggg",ne,$),pe(["gggg","ggggg","GGGG","GGGGG"],function(e,t,n,i){t[i.substr(0,2)]=k(e)}),pe(["gg","GG"],function(e,t,n,r){t[r]=i.parseTwoDigitYear(e)}),X("Q",0,"Qo","quarter"),D("quarter","Q"),F("quarter",7),ce("Q",q),fe("Q",function(e,t){t[me]=3*(k(e)-1)}),X("D",["DD",2],"Do","date"),D("date","D"),F("date",9),ce("D",Z),ce("DD",Z,U),ce("Do",function(e,t){return e?t._dayOfMonthOrdinalParse||t._ordinalParse:t._dayOfMonthOrdinalParseLenient}),fe(["D","DD"],ve),fe("Do",function(e,t){t[ve]=k(e.match(Z)[0])});var ln=_e("Date",!0);X("DDD",["DDDD",3],"DDDo","dayOfYear"),D("dayOfYear","DDD"),F("dayOfYear",4),ce("DDD",ee),ce("DDDD",G),fe(["DDD","DDDD"],function(e,t,n){n._dayOfYear=k(e)}),X("m",["mm",2],0,"minute"),D("minute","m"),F("minute",14),ce("m",Z),ce("mm",Z,U),fe(["m","mm"],be);var cn=_e("Minutes",!1);X("s",["ss",2],0,"second"),D("second","s"),F("second",15),ce("s",Z),ce("ss",Z,U),fe(["s","ss"],xe);var un,hn=_e("Seconds",!1);for(X("S",0,0,function(){return~~(this.millisecond()/100)}),X(0,["SS",2],0,function(){return~~(this.millisecond()/10)}),X(0,["SSS",3],0,"millisecond"),X(0,["SSSS",4],0,function(){return 10*this.millisecond()}),X(0,["SSSSS",5],0,function(){return 100*this.millisecond()}),X(0,["SSSSSS",6],0,function(){return 1e3*this.millisecond()}),X(0,["SSSSSSS",7],0,function(){return 1e4*this.millisecond()}),X(0,["SSSSSSSS",8],0,function(){return 1e5*this.millisecond()}),X(0,["SSSSSSSSS",9],0,function(){return 1e6*this.millisecond()}),D("millisecond","ms"),F("millisecond",16),ce("S",ee,q),ce("SS",ee,U),ce("SSS",ee,G),un="SSSS";un.length<=9;un+="S")ce(un,ie);function dn(e,t){t[we]=k(1e3*("0."+e))}for(un="S";un.length<=9;un+="S")fe(un,dn);var fn=_e("Milliseconds",!1);X("z",0,0,"zoneAbbr"),X("zz",0,0,"zoneName");var pn=b.prototype;function gn(e){return e}pn.add=Qt,pn.calendar=function(e,t){var n=e||_t(),r=zt(n,this).startOf("day"),a=i.calendarFormat(this,r)||"sameElse",o=t&&(O(t[a])?t[a].call(this,n):t[a]);return this.format(o||this.localeData().calendar(a,this,_t(n)))},pn.clone=function(){return new b(this)},pn.diff=function(e,t,n){var i,r,a;if(!this.isValid())return NaN;if(!(i=zt(e,this)).isValid())return NaN;switch(r=6e4*(i.utcOffset()-this.utcOffset()),t=N(t)){case"year":a=Zt(this,i)/12;break;case"month":a=Zt(this,i);break;case"quarter":a=Zt(this,i)/3;break;case"second":a=(this-i)/1e3;break;case"minute":a=(this-i)/6e4;break;case"hour":a=(this-i)/36e5;break;case"day":a=(this-i-r)/864e5;break;case"week":a=(this-i-r)/6048e5;break;default:a=this-i}return n?a:w(a)},pn.endOf=function(e){var t;if(void 0===(e=N(e))||"millisecond"===e||!this.isValid())return this;var n=this._isUTC?an:rn;switch(e){case"year":t=n(this.year()+1,0,1)-1;break;case"quarter":t=n(this.year(),this.month()-this.month()%3+3,1)-1;break;case"month":t=n(this.year(),this.month()+1,1)-1;break;case"week":t=n(this.year(),this.month(),this.date()-this.weekday()+7)-1;break;case"isoWeek":t=n(this.year(),this.month(),this.date()-(this.isoWeekday()-1)+7)-1;break;case"day":case"date":t=n(this.year(),this.month(),this.date()+1)-1;break;case"hour":t=this._d.valueOf(),t+=36e5-nn(t+(this._isUTC?0:6e4*this.utcOffset()),36e5)-1;break;case"minute":t=this._d.valueOf(),t+=6e4-nn(t,6e4)-1;break;case"second":t=this._d.valueOf(),t+=1e3-nn(t,1e3)-1}return this._d.setTime(t),i.updateOffset(this,!0),this},pn.format=function(e){e||(e=this.isUtc()?i.defaultFormatUtc:i.defaultFormat);var t=V(this,e);return this.localeData().postformat(t)},pn.from=function(e,t){return this.isValid()&&(x(e)&&e.isValid()||_t(e).isValid())?Vt({to:this,from:e}).locale(this.locale()).humanize(!t):this.localeData().invalidDate()},pn.fromNow=function(e){return this.from(_t(),e)},pn.to=function(e,t){return this.isValid()&&(x(e)&&e.isValid()||_t(e).isValid())?Vt({from:this,to:e}).locale(this.locale()).humanize(!t):this.localeData().invalidDate()},pn.toNow=function(e){return this.to(_t(),e)},pn.get=function(e){return O(this[e=N(e)])?this[e]():this},pn.invalidAt=function(){return f(this).overflow},pn.isAfter=function(e,t){var n=x(e)?e:_t(e);return!(!this.isValid()||!n.isValid())&&("millisecond"===(t=N(t)||"millisecond")?this.valueOf()>n.valueOf():n.valueOf()<this.clone().startOf(t).valueOf())},pn.isBefore=function(e,t){var n=x(e)?e:_t(e);return!(!this.isValid()||!n.isValid())&&("millisecond"===(t=N(t)||"millisecond")?this.valueOf()<n.valueOf():this.clone().endOf(t).valueOf()<n.valueOf())},pn.isBetween=function(e,t,n,i){var r=x(e)?e:_t(e),a=x(t)?t:_t(t);return!!(this.isValid()&&r.isValid()&&a.isValid())&&("("===(i=i||"()")[0]?this.isAfter(r,n):!this.isBefore(r,n))&&(")"===i[1]?this.isBefore(a,n):!this.isAfter(a,n))},pn.isSame=function(e,t){var n,i=x(e)?e:_t(e);return!(!this.isValid()||!i.isValid())&&("millisecond"===(t=N(t)||"millisecond")?this.valueOf()===i.valueOf():(n=i.valueOf(),this.clone().startOf(t).valueOf()<=n&&n<=this.clone().endOf(t).valueOf()))},pn.isSameOrAfter=function(e,t){return this.isSame(e,t)||this.isAfter(e,t)},pn.isSameOrBefore=function(e,t){return this.isSame(e,t)||this.isBefore(e,t)},pn.isValid=function(){return p(this)},pn.lang=Jt,pn.locale=Kt,pn.localeData=en,pn.max=Pt,pn.min=Ot,pn.parsingFlags=function(){return h({},f(this))},pn.set=function(e,t){if("object"==typeof e)for(var n=function(e){var t=[];for(var n in e)t.push({unit:n,priority:R[n]});return t.sort(function(e,t){return e.priority-t.priority}),t}(e=L(e)),i=0;i<n.length;i++)this[n[i].unit](e[n[i].unit]);else if(O(this[e=N(e)]))return this[e](t);return this},pn.startOf=function(e){var t;if(void 0===(e=N(e))||"millisecond"===e||!this.isValid())return this;var n=this._isUTC?an:rn;switch(e){case"year":t=n(this.year(),0,1);break;case"quarter":t=n(this.year(),this.month()-this.month()%3,1);break;case"month":t=n(this.year(),this.month(),1);break;case"week":t=n(this.year(),this.month(),this.date()-this.weekday());break;case"isoWeek":t=n(this.year(),this.month(),this.date()-(this.isoWeekday()-1));break;case"day":case"date":t=n(this.year(),this.month(),this.date());break;case"hour":t=this._d.valueOf(),t-=nn(t+(this._isUTC?0:6e4*this.utcOffset()),36e5);break;case"minute":t=this._d.valueOf(),t-=nn(t,6e4);break;case"second":t=this._d.valueOf(),t-=nn(t,1e3)}return this._d.setTime(t),i.updateOffset(this,!0),this},pn.subtract=$t,pn.toArray=function(){var e=this;return[e.year(),e.month(),e.date(),e.hour(),e.minute(),e.second(),e.millisecond()]},pn.toObject=function(){var e=this;return{years:e.year(),months:e.month(),date:e.date(),hours:e.hours(),minutes:e.minutes(),seconds:e.seconds(),milliseconds:e.milliseconds()}},pn.toDate=function(){return new Date(this.valueOf())},pn.toISOString=function(e){if(!this.isValid())return null;var t=!0!==e,n=t?this.clone().utc():this;return n.year()<0||9999<n.year()?V(n,t?"YYYYYY-MM-DD[T]HH:mm:ss.SSS[Z]":"YYYYYY-MM-DD[T]HH:mm:ss.SSSZ"):O(Date.prototype.toISOString)?t?this.toDate().toISOString():new Date(this.valueOf()+60*this.utcOffset()*1e3).toISOString().replace("Z",V(n,"Z")):V(n,t?"YYYY-MM-DD[T]HH:mm:ss.SSS[Z]":"YYYY-MM-DD[T]HH:mm:ss.SSSZ")},pn.inspect=function(){if(!this.isValid())return"moment.invalid(/* "+this._i+" */)";var e="moment",t="";this.isLocal()||(e=0===this.utcOffset()?"moment.utc":"moment.parseZone",t="Z");var n="["+e+'("]',i=0<=this.year()&&this.year()<=9999?"YYYY":"YYYYYY",r=t+'[")]';return this.format(n+i+"-MM-DD[T]HH:mm:ss.SSS"+r)},pn.toJSON=function(){return this.isValid()?this.toISOString():null},pn.toString=function(){return this.clone().locale("en").format("ddd MMM DD YYYY HH:mm:ss [GMT]ZZ")},pn.unix=function(){return Math.floor(this.valueOf()/1e3)},pn.valueOf=function(){return this._d.valueOf()-6e4*(this._offset||0)},pn.creationData=function(){return{input:this._i,format:this._f,locale:this._locale,isUTC:this._isUTC,strict:this._strict}},pn.year=Ae,pn.isLeapYear=function(){return Ce(this.year())},pn.weekYear=function(e){return sn.call(this,e,this.week(),this.weekday(),this.localeData()._week.dow,this.localeData()._week.doy)},pn.isoWeekYear=function(e){return sn.call(this,e,this.isoWeek(),this.isoWeekday(),1,4)},pn.quarter=pn.quarters=function(e){return null==e?Math.ceil((this.month()+1)/3):this.month(3*(e-1)+this.month()%3)},pn.month=Re,pn.daysInMonth=function(){return Me(this.year(),this.month())},pn.week=pn.weeks=function(e){var t=this.localeData().week(this);return null==e?t:this.add(7*(e-t),"d")},pn.isoWeek=pn.isoWeeks=function(e){var t=Xe(this,1,4).week;return null==e?t:this.add(7*(e-t),"d")},pn.weeksInYear=function(){var e=this.localeData()._week;return Ve(this.year(),e.dow,e.doy)},pn.isoWeeksInYear=function(){return Ve(this.year(),1,4)},pn.date=ln,pn.day=pn.days=function(e){if(!this.isValid())return null!=e?this:NaN;var t,n,i=this._isUTC?this._d.getUTCDay():this._d.getDay();return null!=e?(t=e,n=this.localeData(),e="string"!=typeof t?t:isNaN(t)?"number"==typeof(t=n.weekdaysParse(t))?t:null:parseInt(t,10),this.add(e-i,"d")):i},pn.weekday=function(e){if(!this.isValid())return null!=e?this:NaN;var t=(this.day()+7-this.localeData()._week.dow)%7;return null==e?t:this.add(e-t,"d")},pn.isoWeekday=function(e){if(!this.isValid())return null!=e?this:NaN;if(null==e)return this.day()||7;var t,n,i=(t=e,n=this.localeData(),"string"==typeof t?n.weekdaysParse(t)%7||7:isNaN(t)?null:t);return this.day(this.day()%7?i:i-7)},pn.dayOfYear=function(e){var t=Math.round((this.clone().startOf("day")-this.clone().startOf("year"))/864e5)+1;return null==e?t:this.add(e-t,"d")},pn.hour=pn.hours=it,pn.minute=pn.minutes=cn,pn.second=pn.seconds=hn,pn.millisecond=pn.milliseconds=fn,pn.utcOffset=function(e,t,n){var r,a=this._offset||0;if(!this.isValid())return null!=e?this:NaN;if(null==e)return this._isUTC?a:Yt(this);if("string"==typeof e){if(null===(e=jt(oe,e)))return this}else Math.abs(e)<16&&!n&&(e*=60);return!this._isUTC&&t&&(r=Yt(this)),this._offset=e,this._isUTC=!0,null!=r&&this.add(r,"m"),a!==e&&(!t||this._changeInProgress?Gt(this,Vt(e-a,"m"),1,!1):this._changeInProgress||(this._changeInProgress=!0,i.updateOffset(this,!0),this._changeInProgress=null)),this},pn.utc=function(e){return this.utcOffset(0,e)},pn.local=function(e){return this._isUTC&&(this.utcOffset(0,e),this._isUTC=!1,e&&this.subtract(Yt(this),"m")),this},pn.parseZone=function(){if(null!=this._tzm)this.utcOffset(this._tzm,!1,!0);else if("string"==typeof this._i){var e=jt(ae,this._i);null!=e?this.utcOffset(e):this.utcOffset(0,!0)}return this},pn.hasAlignedHourOffset=function(e){return!!this.isValid()&&(e=e?_t(e).utcOffset():0,(this.utcOffset()-e)%60==0)},pn.isDST=function(){return this.utcOffset()>this.clone().month(0).utcOffset()||this.utcOffset()>this.clone().month(5).utcOffset()},pn.isLocal=function(){return!!this.isValid()&&!this._isUTC},pn.isUtcOffset=function(){return!!this.isValid()&&this._isUTC},pn.isUtc=Ht,pn.isUTC=Ht,pn.zoneAbbr=function(){return this._isUTC?"UTC":""},pn.zoneName=function(){return this._isUTC?"Coordinated Universal Time":""},pn.dates=C("dates accessor is deprecated. Use date instead.",ln),pn.months=C("months accessor is deprecated. Use month instead",Re),pn.years=C("years accessor is deprecated. Use year instead",Ae),pn.zone=C("moment().zone is deprecated, use moment().utcOffset instead. http://momentjs.com/guides/#/warnings/zone/",function(e,t){return null!=e?("string"!=typeof e&&(e=-e),this.utcOffset(e,t),this):-this.utcOffset()}),pn.isDSTShifted=C("isDSTShifted is deprecated. See http://momentjs.com/guides/#/warnings/dst-shifted/ for more information",function(){if(!o(this._isDSTShifted))return this._isDSTShifted;var e={};if(v(e,this),(e=Tt(e))._a){var t=e._isUTC?d(e._a):_t(e._a);this._isDSTShifted=this.isValid()&&0<S(e._a,t.toArray())}else this._isDSTShifted=!1;return this._isDSTShifted});var mn=M.prototype;function vn(e,t,n,i){var r=ht(),a=d().set(i,t);return r[n](a,e)}function yn(e,t,n){if(s(e)&&(t=e,e=void 0),e=e||"",null!=t)return vn(e,t,n,"month");var i,r=[];for(i=0;i<12;i++)r[i]=vn(e,i,n,"month");return r}function bn(e,t,n,i){"boolean"==typeof e?s(t)&&(n=t,t=void 0):(t=e,e=!1,s(n=t)&&(n=t,t=void 0)),t=t||"";var r,a=ht(),o=e?a._week.dow:0;if(null!=n)return vn(t,(n+o)%7,i,"day");var l=[];for(r=0;r<7;r++)l[r]=vn(t,(r+o)%7,i,"day");return l}mn.calendar=function(e,t,n){var i=this._calendar[e]||this._calendar.sameElse;return O(i)?i.call(t,n):i},mn.longDateFormat=function(e){var t=this._longDateFormat[e],n=this._longDateFormat[e.toUpperCase()];return t||!n?t:(this._longDateFormat[e]=n.replace(/MMMM|MM|DD|dddd/g,function(e){return e.slice(1)}),this._longDateFormat[e])},mn.invalidDate=function(){return this._invalidDate},mn.ordinal=function(e){return this._ordinal.replace("%d",e)},mn.preparse=gn,mn.postformat=gn,mn.relativeTime=function(e,t,n,i){var r=this._relativeTime[n];return O(r)?r(e,t,n,i):r.replace(/%d/i,e)},mn.pastFuture=function(e,t){var n=this._relativeTime[0<e?"future":"past"];return O(n)?n(t):n.replace(/%s/i,t)},mn.set=function(e){var t,n;for(n in e)O(t=e[n])?this[n]=t:this["_"+n]=t;this._config=e,this._dayOfMonthOrdinalParseLenient=new RegExp((this._dayOfMonthOrdinalParse.source||this._ordinalParse.source)+"|"+/\d{1,2}/.source)},mn.months=function(e,t){return e?r(this._months)?this._months[e.month()]:this._months[(this._months.isFormat||Ie).test(t)?"format":"standalone"][e.month()]:r(this._months)?this._months:this._months.standalone},mn.monthsShort=function(e,t){return e?r(this._monthsShort)?this._monthsShort[e.month()]:this._monthsShort[Ie.test(t)?"format":"standalone"][e.month()]:r(this._monthsShort)?this._monthsShort:this._monthsShort.standalone},mn.monthsParse=function(e,t,n){var i,r,a;if(this._monthsParseExact)return function(e,t,n){var i,r,a,o=e.toLocaleLowerCase();if(!this._monthsParse)for(this._monthsParse=[],this._longMonthsParse=[],this._shortMonthsParse=[],i=0;i<12;++i)a=d([2e3,i]),this._shortMonthsParse[i]=this.monthsShort(a,"").toLocaleLowerCase(),this._longMonthsParse[i]=this.months(a,"").toLocaleLowerCase();return n?"MMM"===t?-1!==(r=Te.call(this._shortMonthsParse,o))?r:null:-1!==(r=Te.call(this._longMonthsParse,o))?r:null:"MMM"===t?-1!==(r=Te.call(this._shortMonthsParse,o))?r:-1!==(r=Te.call(this._longMonthsParse,o))?r:null:-1!==(r=Te.call(this._longMonthsParse,o))?r:-1!==(r=Te.call(this._shortMonthsParse,o))?r:null}.call(this,e,t,n);for(this._monthsParse||(this._monthsParse=[],this._longMonthsParse=[],this._shortMonthsParse=[]),i=0;i<12;i++){if(r=d([2e3,i]),n&&!this._longMonthsParse[i]&&(this._longMonthsParse[i]=new RegExp("^"+this.months(r,"").replace(".","")+"$","i"),this._shortMonthsParse[i]=new RegExp("^"+this.monthsShort(r,"").replace(".","")+"$","i")),n||this._monthsParse[i]||(a="^"+this.months(r,"")+"|^"+this.monthsShort(r,""),this._monthsParse[i]=new RegExp(a.replace(".",""),"i")),n&&"MMMM"===t&&this._longMonthsParse[i].test(e))return i;if(n&&"MMM"===t&&this._shortMonthsParse[i].test(e))return i;if(!n&&this._monthsParse[i].test(e))return i}},mn.monthsRegex=function(e){return this._monthsParseExact?(u(this,"_monthsRegex")||ze.call(this),e?this._monthsStrictRegex:this._monthsRegex):(u(this,"_monthsRegex")||(this._monthsRegex=je),this._monthsStrictRegex&&e?this._monthsStrictRegex:this._monthsRegex)},mn.monthsShortRegex=function(e){return this._monthsParseExact?(u(this,"_monthsRegex")||ze.call(this),e?this._monthsShortStrictRegex:this._monthsShortRegex):(u(this,"_monthsShortRegex")||(this._monthsShortRegex=Fe),this._monthsShortStrictRegex&&e?this._monthsShortStrictRegex:this._monthsShortRegex)},mn.week=function(e){return Xe(e,this._week.dow,this._week.doy).week},mn.firstDayOfYear=function(){return this._week.doy},mn.firstDayOfWeek=function(){return this._week.dow},mn.weekdays=function(e,t){var n=r(this._weekdays)?this._weekdays:this._weekdays[e&&!0!==e&&this._weekdays.isFormat.test(t)?"format":"standalone"];return!0===e?Be(n,this._week.dow):e?n[e.day()]:n},mn.weekdaysMin=function(e){return!0===e?Be(this._weekdaysMin,this._week.dow):e?this._weekdaysMin[e.day()]:this._weekdaysMin},mn.weekdaysShort=function(e){return!0===e?Be(this._weekdaysShort,this._week.dow):e?this._weekdaysShort[e.day()]:this._weekdaysShort},mn.weekdaysParse=function(e,t,n){var i,r,a;if(this._weekdaysParseExact)return function(e,t,n){var i,r,a,o=e.toLocaleLowerCase();if(!this._weekdaysParse)for(this._weekdaysParse=[],this._shortWeekdaysParse=[],this._minWeekdaysParse=[],i=0;i<7;++i)a=d([2e3,1]).day(i),this._minWeekdaysParse[i]=this.weekdaysMin(a,"").toLocaleLowerCase(),this._shortWeekdaysParse[i]=this.weekdaysShort(a,"").toLocaleLowerCase(),this._weekdaysParse[i]=this.weekdays(a,"").toLocaleLowerCase();return n?"dddd"===t?-1!==(r=Te.call(this._weekdaysParse,o))?r:null:"ddd"===t?-1!==(r=Te.call(this._shortWeekdaysParse,o))?r:null:-1!==(r=Te.call(this._minWeekdaysParse,o))?r:null:"dddd"===t?-1!==(r=Te.call(this._weekdaysParse,o))?r:-1!==(r=Te.call(this._shortWeekdaysParse,o))?r:-1!==(r=Te.call(this._minWeekdaysParse,o))?r:null:"ddd"===t?-1!==(r=Te.call(this._shortWeekdaysParse,o))?r:-1!==(r=Te.call(this._weekdaysParse,o))?r:-1!==(r=Te.call(this._minWeekdaysParse,o))?r:null:-1!==(r=Te.call(this._minWeekdaysParse,o))?r:-1!==(r=Te.call(this._weekdaysParse,o))?r:-1!==(r=Te.call(this._shortWeekdaysParse,o))?r:null}.call(this,e,t,n);for(this._weekdaysParse||(this._weekdaysParse=[],this._minWeekdaysParse=[],this._shortWeekdaysParse=[],this._fullWeekdaysParse=[]),i=0;i<7;i++){if(r=d([2e3,1]).day(i),n&&!this._fullWeekdaysParse[i]&&(this._fullWeekdaysParse[i]=new RegExp("^"+this.weekdays(r,"").replace(".","\\.?")+"$","i"),this._shortWeekdaysParse[i]=new RegExp("^"+this.weekdaysShort(r,"").replace(".","\\.?")+"$","i"),this._minWeekdaysParse[i]=new RegExp("^"+this.weekdaysMin(r,"").replace(".","\\.?")+"$","i")),this._weekdaysParse[i]||(a="^"+this.weekdays(r,"")+"|^"+this.weekdaysShort(r,"")+"|^"+this.weekdaysMin(r,""),this._weekdaysParse[i]=new RegExp(a.replace(".",""),"i")),n&&"dddd"===t&&this._fullWeekdaysParse[i].test(e))return i;if(n&&"ddd"===t&&this._shortWeekdaysParse[i].test(e))return i;if(n&&"dd"===t&&this._minWeekdaysParse[i].test(e))return i;if(!n&&this._weekdaysParse[i].test(e))return i}},mn.weekdaysRegex=function(e){return this._weekdaysParseExact?(u(this,"_weekdaysRegex")||Ke.call(this),e?this._weekdaysStrictRegex:this._weekdaysRegex):(u(this,"_weekdaysRegex")||(this._weekdaysRegex=Qe),this._weekdaysStrictRegex&&e?this._weekdaysStrictRegex:this._weekdaysRegex)},mn.weekdaysShortRegex=function(e){return this._weekdaysParseExact?(u(this,"_weekdaysRegex")||Ke.call(this),e?this._weekdaysShortStrictRegex:this._weekdaysShortRegex):(u(this,"_weekdaysShortRegex")||(this._weekdaysShortRegex=$e),this._weekdaysShortStrictRegex&&e?this._weekdaysShortStrictRegex:this._weekdaysShortRegex)},mn.weekdaysMinRegex=function(e){return this._weekdaysParseExact?(u(this,"_weekdaysRegex")||Ke.call(this),e?this._weekdaysMinStrictRegex:this._weekdaysMinRegex):(u(this,"_weekdaysMinRegex")||(this._weekdaysMinRegex=Ze),this._weekdaysMinStrictRegex&&e?this._weekdaysMinStrictRegex:this._weekdaysMinRegex)},mn.isPM=function(e){return"p"===(e+"").toLowerCase().charAt(0)},mn.meridiem=function(e,t,n){return 11<e?n?"pm":"PM":n?"am":"AM"},ct("en",{dayOfMonthOrdinalParse:/\d{1,2}(th|st|nd|rd)/,ordinal:function(e){var t=e%10;return e+(1===k(e%100/10)?"th":1===t?"st":2===t?"nd":3===t?"rd":"th")}}),i.lang=C("moment.lang is deprecated. Use moment.locale instead.",ct),i.langData=C("moment.langData is deprecated. Use moment.localeData instead.",ht);var xn=Math.abs;function wn(e,t,n,i){var r=Vt(t,n);return e._milliseconds+=i*r._milliseconds,e._days+=i*r._days,e._months+=i*r._months,e._bubble()}function kn(e){return e<0?Math.floor(e):Math.ceil(e)}function Sn(e){return 4800*e/146097}function En(e){return 146097*e/4800}function Cn(e){return function(){return this.as(e)}}var Tn=Cn("ms"),An=Cn("s"),_n=Cn("m"),On=Cn("h"),Pn=Cn("d"),Mn=Cn("w"),In=Cn("M"),Dn=Cn("Q"),Nn=Cn("y");function Ln(e){return function(){return this.isValid()?this._data[e]:NaN}}var Rn=Ln("milliseconds"),Fn=Ln("seconds"),jn=Ln("minutes"),zn=Ln("hours"),Yn=Ln("days"),Hn=Ln("months"),Wn=Ln("years"),Xn=Math.round,Vn={ss:44,s:45,m:45,h:22,d:26,M:11},Bn=Math.abs;function qn(e){return(0<e)-(e<0)||+e}function Un(){if(!this.isValid())return this.localeData().invalidDate();var e,t,n=Bn(this._milliseconds)/1e3,i=Bn(this._days),r=Bn(this._months);t=w((e=w(n/60))/60),n%=60,e%=60;var a=w(r/12),o=r%=12,s=i,l=t,c=e,u=n?n.toFixed(3).replace(/\.?0+$/,""):"",h=this.asSeconds();if(!h)return"P0D";var d=h<0?"-":"",f=qn(this._months)!==qn(h)?"-":"",p=qn(this._days)!==qn(h)?"-":"",g=qn(this._milliseconds)!==qn(h)?"-":"";return d+"P"+(a?f+a+"Y":"")+(o?f+o+"M":"")+(s?p+s+"D":"")+(l||c||u?"T":"")+(l?g+l+"H":"")+(c?g+c+"M":"")+(u?g+u+"S":"")}var Gn=Dt.prototype;return Gn.isValid=function(){return this._isValid},Gn.abs=function(){var e=this._data;return this._milliseconds=xn(this._milliseconds),this._days=xn(this._days),this._months=xn(this._months),e.milliseconds=xn(e.milliseconds),e.seconds=xn(e.seconds),e.minutes=xn(e.minutes),e.hours=xn(e.hours),e.months=xn(e.months),e.years=xn(e.years),this},Gn.add=function(e,t){return wn(this,e,t,1)},Gn.subtract=function(e,t){return wn(this,e,t,-1)},Gn.as=function(e){if(!this.isValid())return NaN;var t,n,i=this._milliseconds;if("month"===(e=N(e))||"quarter"===e||"year"===e)switch(t=this._days+i/864e5,n=this._months+Sn(t),e){case"month":return n;case"quarter":return n/3;case"year":return n/12}else switch(t=this._days+Math.round(En(this._months)),e){case"week":return t/7+i/6048e5;case"day":return t+i/864e5;case"hour":return 24*t+i/36e5;case"minute":return 1440*t+i/6e4;case"second":return 86400*t+i/1e3;case"millisecond":return Math.floor(864e5*t)+i;default:throw new Error("Unknown unit "+e)}},Gn.asMilliseconds=Tn,Gn.asSeconds=An,Gn.asMinutes=_n,Gn.asHours=On,Gn.asDays=Pn,Gn.asWeeks=Mn,Gn.asMonths=In,Gn.asQuarters=Dn,Gn.asYears=Nn,Gn.valueOf=function(){return this.isValid()?this._milliseconds+864e5*this._days+this._months%12*2592e6+31536e6*k(this._months/12):NaN},Gn._bubble=function(){var e,t,n,i,r,a=this._milliseconds,o=this._days,s=this._months,l=this._data;return 0<=a&&0<=o&&0<=s||a<=0&&o<=0&&s<=0||(a+=864e5*kn(En(s)+o),s=o=0),l.milliseconds=a%1e3,e=w(a/1e3),l.seconds=e%60,t=w(e/60),l.minutes=t%60,n=w(t/60),l.hours=n%24,s+=r=w(Sn(o+=w(n/24))),o-=kn(En(r)),i=w(s/12),s%=12,l.days=o,l.months=s,l.years=i,this},Gn.clone=function(){return Vt(this)},Gn.get=function(e){return e=N(e),this.isValid()?this[e+"s"]():NaN},Gn.milliseconds=Rn,Gn.seconds=Fn,Gn.minutes=jn,Gn.hours=zn,Gn.days=Yn,Gn.weeks=function(){return w(this.days()/7)},Gn.months=Hn,Gn.years=Wn,Gn.humanize=function(e){if(!this.isValid())return this.localeData().invalidDate();var t,n,i,r,a,o,s,l,c,u,h=this.localeData(),d=(t=!e,n=h,i=Vt(this).abs(),r=Xn(i.as("s")),a=Xn(i.as("m")),o=Xn(i.as("h")),s=Xn(i.as("d")),l=Xn(i.as("M")),c=Xn(i.as("y")),(u=r<=Vn.ss&&["s",r]||r<Vn.s&&["ss",r]||a<=1&&["m"]||a<Vn.m&&["mm",a]||o<=1&&["h"]||o<Vn.h&&["hh",o]||s<=1&&["d"]||s<Vn.d&&["dd",s]||l<=1&&["M"]||l<Vn.M&&["MM",l]||c<=1&&["y"]||["yy",c])[2]=t,u[3]=0<+this,u[4]=n,function(e,t,n,i,r){return r.relativeTime(t||1,!!n,e,i)}.apply(null,u));return e&&(d=h.pastFuture(+this,d)),h.postformat(d)},Gn.toISOString=Un,Gn.toString=Un,Gn.toJSON=Un,Gn.locale=Kt,Gn.localeData=en,Gn.toIsoString=C("toIsoString() is deprecated. Please use toISOString() instead (notice the capitals)",Un),Gn.lang=Jt,X("X",0,0,"unix"),X("x",0,0,"valueOf"),ce("x",re),ce("X",/[+-]?\d+(\.\d{1,3})?/),fe("X",function(e,t,n){n._d=new Date(1e3*parseFloat(e,10))}),fe("x",function(e,t,n){n._d=new Date(k(e))}),i.version="2.24.0",t=_t,i.fn=pn,i.min=function(){return Mt("isBefore",[].slice.call(arguments,0))},i.max=function(){return Mt("isAfter",[].slice.call(arguments,0))},i.now=function(){return Date.now?Date.now():+new Date},i.utc=d,i.unix=function(e){return _t(1e3*e)},i.months=function(e,t){return yn(e,t,"months")},i.isDate=l,i.locale=ct,i.invalid=g,i.duration=Vt,i.isMoment=x,i.weekdays=function(e,t,n){return bn(e,t,n,"weekdays")},i.parseZone=function(){return _t.apply(null,arguments).parseZone()},i.localeData=ht,i.isDuration=Nt,i.monthsShort=function(e,t){return yn(e,t,"monthsShort")},i.weekdaysMin=function(e,t,n){return bn(e,t,n,"weekdaysMin")},i.defineLocale=ut,i.updateLocale=function(e,t){if(null!=t){var n,i,r=rt;null!=(i=lt(e))&&(r=i._config),(n=new M(t=P(r,t))).parentLocale=at[e],at[e]=n,ct(e)}else null!=at[e]&&(null!=at[e].parentLocale?at[e]=at[e].parentLocale:null!=at[e]&&delete at[e]);return at[e]},i.locales=function(){return T(at)},i.weekdaysShort=function(e,t,n){return bn(e,t,n,"weekdaysShort")},i.normalizeUnits=N,i.relativeTimeRounding=function(e){return void 0===e?Xn:"function"==typeof e&&(Xn=e,!0)},i.relativeTimeThreshold=function(e,t){return void 0!==Vn[e]&&(void 0===t?Vn[e]:(Vn[e]=t,"s"===e&&(Vn.ss=t-1),!0))},i.calendarFormat=function(e,t){var n=e.diff(t,"days",!0);return n<-6?"sameElse":n<-1?"lastWeek":n<0?"lastDay":n<1?"sameDay":n<2?"nextDay":n<7?"nextWeek":"sameElse"},i.prototype=pn,i.HTML5_FMT={DATETIME_LOCAL:"YYYY-MM-DDTHH:mm",DATETIME_LOCAL_SECONDS:"YYYY-MM-DDTHH:mm:ss",DATETIME_LOCAL_MS:"YYYY-MM-DDTHH:mm:ss.SSS",DATE:"YYYY-MM-DD",TIME:"HH:mm",TIME_SECONDS:"HH:mm:ss",TIME_MS:"HH:mm:ss.SSS",WEEK:"GGGG-[W]WW",MONTH:"YYYY-MM"},i}()}).call(this,n(35)(e))},function(e,t,n){"use strict";(function(e){n.d(t,"a",function(){return s}),n.d(t,"b",function(){return l});var i=n(1),r="Invariant Violation",a=Object.setPrototypeOf,o=void 0===a?function(e,t){return e.__proto__=t,e}:a,s=function(e){function t(n){void 0===n&&(n=r);var i=e.call(this,"number"==typeof n?r+": "+n+" (see https://github.com/apollographql/invariant-packages)":n)||this;return i.framesToPop=1,i.name=r,o(i,t.prototype),i}return Object(i.c)(t,e),t}(Error);function l(e,t){if(!e)throw new s(t)}function c(e){return function(){return console[e].apply(console,arguments)}}!function(e){e.warn=c("warn"),e.error=c("error")}(l||(l={}));var u={env:{}};if("object"==typeof e)u=e;else try{Function("stub","process = stub")(u)}catch(e){}}).call(this,n(15))},function(e,t,n){var i;
/*!
 * jQuery JavaScript Library v3.4.0
 * https://jquery.com/
 *
 * Includes Sizzle.js
 * https://sizzlejs.com/
 *
 * Copyright JS Foundation and other contributors
 * Released under the MIT license
 * https://jquery.org/license
 *
 * Date: 2019-04-10T19:48Z
 */
/*!
 * jQuery JavaScript Library v3.4.0
 * https://jquery.com/
 *
 * Includes Sizzle.js
 * https://sizzlejs.com/
 *
 * Copyright JS Foundation and other contributors
 * Released under the MIT license
 * https://jquery.org/license
 *
 * Date: 2019-04-10T19:48Z
 */
!function(t,n){"use strict";"object"==typeof e.exports?e.exports=t.document?n(t,!0):function(e){if(!e.document)throw new Error("jQuery requires a window with a document");return n(e)}:n(t)}("undefined"!=typeof window?window:this,function(n,r){"use strict";var a=[],o=n.document,s=Object.getPrototypeOf,l=a.slice,c=a.concat,u=a.push,h=a.indexOf,d={},f=d.toString,p=d.hasOwnProperty,g=p.toString,m=g.call(Object),v={},y=function(e){return"function"==typeof e&&"number"!=typeof e.nodeType},b=function(e){return null!=e&&e===e.window},x={type:!0,src:!0,nonce:!0,noModule:!0};function w(e,t,n){var i,r,a=(n=n||o).createElement("script");if(a.text=e,t)for(i in x)(r=t[i]||t.getAttribute&&t.getAttribute(i))&&a.setAttribute(i,r);n.head.appendChild(a).parentNode.removeChild(a)}function k(e){return null==e?e+"":"object"==typeof e||"function"==typeof e?d[f.call(e)]||"object":typeof e}var S=function(e,t){return new S.fn.init(e,t)},E=/^[\s\uFEFF\xA0]+|[\s\uFEFF\xA0]+$/g;function C(e){var t=!!e&&"length"in e&&e.length,n=k(e);return!y(e)&&!b(e)&&("array"===n||0===t||"number"==typeof t&&t>0&&t-1 in e)}S.fn=S.prototype={jquery:"3.4.0",constructor:S,length:0,toArray:function(){return l.call(this)},get:function(e){return null==e?l.call(this):e<0?this[e+this.length]:this[e]},pushStack:function(e){var t=S.merge(this.constructor(),e);return t.prevObject=this,t},each:function(e){return S.each(this,e)},map:function(e){return this.pushStack(S.map(this,function(t,n){return e.call(t,n,t)}))},slice:function(){return this.pushStack(l.apply(this,arguments))},first:function(){return this.eq(0)},last:function(){return this.eq(-1)},eq:function(e){var t=this.length,n=+e+(e<0?t:0);return this.pushStack(n>=0&&n<t?[this[n]]:[])},end:function(){return this.prevObject||this.constructor()},push:u,sort:a.sort,splice:a.splice},S.extend=S.fn.extend=function(){var e,t,n,i,r,a,o=arguments[0]||{},s=1,l=arguments.length,c=!1;for("boolean"==typeof o&&(c=o,o=arguments[s]||{},s++),"object"==typeof o||y(o)||(o={}),s===l&&(o=this,s--);s<l;s++)if(null!=(e=arguments[s]))for(t in e)i=e[t],"__proto__"!==t&&o!==i&&(c&&i&&(S.isPlainObject(i)||(r=Array.isArray(i)))?(n=o[t],a=r&&!Array.isArray(n)?[]:r||S.isPlainObject(n)?n:{},r=!1,o[t]=S.extend(c,a,i)):void 0!==i&&(o[t]=i));return o},S.extend({expando:"jQuery"+("3.4.0"+Math.random()).replace(/\D/g,""),isReady:!0,error:function(e){throw new Error(e)},noop:function(){},isPlainObject:function(e){var t,n;return!(!e||"[object Object]"!==f.call(e))&&(!(t=s(e))||"function"==typeof(n=p.call(t,"constructor")&&t.constructor)&&g.call(n)===m)},isEmptyObject:function(e){var t;for(t in e)return!1;return!0},globalEval:function(e,t){w(e,{nonce:t&&t.nonce})},each:function(e,t){var n,i=0;if(C(e))for(n=e.length;i<n&&!1!==t.call(e[i],i,e[i]);i++);else for(i in e)if(!1===t.call(e[i],i,e[i]))break;return e},trim:function(e){return null==e?"":(e+"").replace(E,"")},makeArray:function(e,t){var n=t||[];return null!=e&&(C(Object(e))?S.merge(n,"string"==typeof e?[e]:e):u.call(n,e)),n},inArray:function(e,t,n){return null==t?-1:h.call(t,e,n)},merge:function(e,t){for(var n=+t.length,i=0,r=e.length;i<n;i++)e[r++]=t[i];return e.length=r,e},grep:function(e,t,n){for(var i=[],r=0,a=e.length,o=!n;r<a;r++)!t(e[r],r)!==o&&i.push(e[r]);return i},map:function(e,t,n){var i,r,a=0,o=[];if(C(e))for(i=e.length;a<i;a++)null!=(r=t(e[a],a,n))&&o.push(r);else for(a in e)null!=(r=t(e[a],a,n))&&o.push(r);return c.apply([],o)},guid:1,support:v}),"function"==typeof Symbol&&(S.fn[Symbol.iterator]=a[Symbol.iterator]),S.each("Boolean Number String Function Array Date RegExp Object Error Symbol".split(" "),function(e,t){d["[object "+t+"]"]=t.toLowerCase()});var T=
/*!
 * Sizzle CSS Selector Engine v2.3.4
 * https://sizzlejs.com/
 *
 * Copyright JS Foundation and other contributors
 * Released under the MIT license
 * https://js.foundation/
 *
 * Date: 2019-04-08
 */
function(e){var t,n,i,r,a,o,s,l,c,u,h,d,f,p,g,m,v,y,b,x="sizzle"+1*new Date,w=e.document,k=0,S=0,E=le(),C=le(),T=le(),A=le(),_=function(e,t){return e===t&&(h=!0),0},O={}.hasOwnProperty,P=[],M=P.pop,I=P.push,D=P.push,N=P.slice,L=function(e,t){for(var n=0,i=e.length;n<i;n++)if(e[n]===t)return n;return-1},R="checked|selected|async|autofocus|autoplay|controls|defer|disabled|hidden|ismap|loop|multiple|open|readonly|required|scoped",F="[\\x20\\t\\r\\n\\f]",j="(?:\\\\.|[\\w-]|[^\0-\\xa0])+",z="\\["+F+"*("+j+")(?:"+F+"*([*^$|!~]?=)"+F+"*(?:'((?:\\\\.|[^\\\\'])*)'|\"((?:\\\\.|[^\\\\\"])*)\"|("+j+"))|)"+F+"*\\]",Y=":("+j+")(?:\\((('((?:\\\\.|[^\\\\'])*)'|\"((?:\\\\.|[^\\\\\"])*)\")|((?:\\\\.|[^\\\\()[\\]]|"+z+")*)|.*)\\)|)",H=new RegExp(F+"+","g"),W=new RegExp("^"+F+"+|((?:^|[^\\\\])(?:\\\\.)*)"+F+"+$","g"),X=new RegExp("^"+F+"*,"+F+"*"),V=new RegExp("^"+F+"*([>+~]|"+F+")"+F+"*"),B=new RegExp(F+"|>"),q=new RegExp(Y),U=new RegExp("^"+j+"$"),G={ID:new RegExp("^#("+j+")"),CLASS:new RegExp("^\\.("+j+")"),TAG:new RegExp("^("+j+"|[*])"),ATTR:new RegExp("^"+z),PSEUDO:new RegExp("^"+Y),CHILD:new RegExp("^:(only|first|last|nth|nth-last)-(child|of-type)(?:\\("+F+"*(even|odd|(([+-]|)(\\d*)n|)"+F+"*(?:([+-]|)"+F+"*(\\d+)|))"+F+"*\\)|)","i"),bool:new RegExp("^(?:"+R+")$","i"),needsContext:new RegExp("^"+F+"*[>+~]|:(even|odd|eq|gt|lt|nth|first|last)(?:\\("+F+"*((?:-\\d)?\\d*)"+F+"*\\)|)(?=[^-]|$)","i")},Q=/HTML$/i,$=/^(?:input|select|textarea|button)$/i,Z=/^h\d$/i,K=/^[^{]+\{\s*\[native \w/,J=/^(?:#([\w-]+)|(\w+)|\.([\w-]+))$/,ee=/[+~]/,te=new RegExp("\\\\([\\da-f]{1,6}"+F+"?|("+F+")|.)","ig"),ne=function(e,t,n){var i="0x"+t-65536;return i!=i||n?t:i<0?String.fromCharCode(i+65536):String.fromCharCode(i>>10|55296,1023&i|56320)},ie=/([\0-\x1f\x7f]|^-?\d)|^-$|[^\0-\x1f\x7f-\uFFFF\w-]/g,re=function(e,t){return t?"\0"===e?"�":e.slice(0,-1)+"\\"+e.charCodeAt(e.length-1).toString(16)+" ":"\\"+e},ae=function(){d()},oe=xe(function(e){return!0===e.disabled&&"fieldset"===e.nodeName.toLowerCase()},{dir:"parentNode",next:"legend"});try{D.apply(P=N.call(w.childNodes),w.childNodes),P[w.childNodes.length].nodeType}catch(e){D={apply:P.length?function(e,t){I.apply(e,N.call(t))}:function(e,t){for(var n=e.length,i=0;e[n++]=t[i++];);e.length=n-1}}}function se(e,t,i,r){var a,s,c,u,h,p,v,y=t&&t.ownerDocument,k=t?t.nodeType:9;if(i=i||[],"string"!=typeof e||!e||1!==k&&9!==k&&11!==k)return i;if(!r&&((t?t.ownerDocument||t:w)!==f&&d(t),t=t||f,g)){if(11!==k&&(h=J.exec(e)))if(a=h[1]){if(9===k){if(!(c=t.getElementById(a)))return i;if(c.id===a)return i.push(c),i}else if(y&&(c=y.getElementById(a))&&b(t,c)&&c.id===a)return i.push(c),i}else{if(h[2])return D.apply(i,t.getElementsByTagName(e)),i;if((a=h[3])&&n.getElementsByClassName&&t.getElementsByClassName)return D.apply(i,t.getElementsByClassName(a)),i}if(n.qsa&&!A[e+" "]&&(!m||!m.test(e))&&(1!==k||"object"!==t.nodeName.toLowerCase())){if(v=e,y=t,1===k&&B.test(e)){for((u=t.getAttribute("id"))?u=u.replace(ie,re):t.setAttribute("id",u=x),s=(p=o(e)).length;s--;)p[s]="#"+u+" "+be(p[s]);v=p.join(","),y=ee.test(e)&&ve(t.parentNode)||t}try{return D.apply(i,y.querySelectorAll(v)),i}catch(t){A(e,!0)}finally{u===x&&t.removeAttribute("id")}}}return l(e.replace(W,"$1"),t,i,r)}function le(){var e=[];return function t(n,r){return e.push(n+" ")>i.cacheLength&&delete t[e.shift()],t[n+" "]=r}}function ce(e){return e[x]=!0,e}function ue(e){var t=f.createElement("fieldset");try{return!!e(t)}catch(e){return!1}finally{t.parentNode&&t.parentNode.removeChild(t),t=null}}function he(e,t){for(var n=e.split("|"),r=n.length;r--;)i.attrHandle[n[r]]=t}function de(e,t){var n=t&&e,i=n&&1===e.nodeType&&1===t.nodeType&&e.sourceIndex-t.sourceIndex;if(i)return i;if(n)for(;n=n.nextSibling;)if(n===t)return-1;return e?1:-1}function fe(e){return function(t){return"input"===t.nodeName.toLowerCase()&&t.type===e}}function pe(e){return function(t){var n=t.nodeName.toLowerCase();return("input"===n||"button"===n)&&t.type===e}}function ge(e){return function(t){return"form"in t?t.parentNode&&!1===t.disabled?"label"in t?"label"in t.parentNode?t.parentNode.disabled===e:t.disabled===e:t.isDisabled===e||t.isDisabled!==!e&&oe(t)===e:t.disabled===e:"label"in t&&t.disabled===e}}function me(e){return ce(function(t){return t=+t,ce(function(n,i){for(var r,a=e([],n.length,t),o=a.length;o--;)n[r=a[o]]&&(n[r]=!(i[r]=n[r]))})})}function ve(e){return e&&void 0!==e.getElementsByTagName&&e}for(t in n=se.support={},a=se.isXML=function(e){var t=e.namespaceURI,n=(e.ownerDocument||e).documentElement;return!Q.test(t||n&&n.nodeName||"HTML")},d=se.setDocument=function(e){var t,r,o=e?e.ownerDocument||e:w;return o!==f&&9===o.nodeType&&o.documentElement?(p=(f=o).documentElement,g=!a(f),w!==f&&(r=f.defaultView)&&r.top!==r&&(r.addEventListener?r.addEventListener("unload",ae,!1):r.attachEvent&&r.attachEvent("onunload",ae)),n.attributes=ue(function(e){return e.className="i",!e.getAttribute("className")}),n.getElementsByTagName=ue(function(e){return e.appendChild(f.createComment("")),!e.getElementsByTagName("*").length}),n.getElementsByClassName=K.test(f.getElementsByClassName),n.getById=ue(function(e){return p.appendChild(e).id=x,!f.getElementsByName||!f.getElementsByName(x).length}),n.getById?(i.filter.ID=function(e){var t=e.replace(te,ne);return function(e){return e.getAttribute("id")===t}},i.find.ID=function(e,t){if(void 0!==t.getElementById&&g){var n=t.getElementById(e);return n?[n]:[]}}):(i.filter.ID=function(e){var t=e.replace(te,ne);return function(e){var n=void 0!==e.getAttributeNode&&e.getAttributeNode("id");return n&&n.value===t}},i.find.ID=function(e,t){if(void 0!==t.getElementById&&g){var n,i,r,a=t.getElementById(e);if(a){if((n=a.getAttributeNode("id"))&&n.value===e)return[a];for(r=t.getElementsByName(e),i=0;a=r[i++];)if((n=a.getAttributeNode("id"))&&n.value===e)return[a]}return[]}}),i.find.TAG=n.getElementsByTagName?function(e,t){return void 0!==t.getElementsByTagName?t.getElementsByTagName(e):n.qsa?t.querySelectorAll(e):void 0}:function(e,t){var n,i=[],r=0,a=t.getElementsByTagName(e);if("*"===e){for(;n=a[r++];)1===n.nodeType&&i.push(n);return i}return a},i.find.CLASS=n.getElementsByClassName&&function(e,t){if(void 0!==t.getElementsByClassName&&g)return t.getElementsByClassName(e)},v=[],m=[],(n.qsa=K.test(f.querySelectorAll))&&(ue(function(e){p.appendChild(e).innerHTML="<a id='"+x+"'></a><select id='"+x+"-\r\\' msallowcapture=''><option selected=''></option></select>",e.querySelectorAll("[msallowcapture^='']").length&&m.push("[*^$]="+F+"*(?:''|\"\")"),e.querySelectorAll("[selected]").length||m.push("\\["+F+"*(?:value|"+R+")"),e.querySelectorAll("[id~="+x+"-]").length||m.push("~="),e.querySelectorAll(":checked").length||m.push(":checked"),e.querySelectorAll("a#"+x+"+*").length||m.push(".#.+[+~]")}),ue(function(e){e.innerHTML="<a href='' disabled='disabled'></a><select disabled='disabled'><option/></select>";var t=f.createElement("input");t.setAttribute("type","hidden"),e.appendChild(t).setAttribute("name","D"),e.querySelectorAll("[name=d]").length&&m.push("name"+F+"*[*^$|!~]?="),2!==e.querySelectorAll(":enabled").length&&m.push(":enabled",":disabled"),p.appendChild(e).disabled=!0,2!==e.querySelectorAll(":disabled").length&&m.push(":enabled",":disabled"),e.querySelectorAll("*,:x"),m.push(",.*:")})),(n.matchesSelector=K.test(y=p.matches||p.webkitMatchesSelector||p.mozMatchesSelector||p.oMatchesSelector||p.msMatchesSelector))&&ue(function(e){n.disconnectedMatch=y.call(e,"*"),y.call(e,"[s!='']:x"),v.push("!=",Y)}),m=m.length&&new RegExp(m.join("|")),v=v.length&&new RegExp(v.join("|")),t=K.test(p.compareDocumentPosition),b=t||K.test(p.contains)?function(e,t){var n=9===e.nodeType?e.documentElement:e,i=t&&t.parentNode;return e===i||!(!i||1!==i.nodeType||!(n.contains?n.contains(i):e.compareDocumentPosition&&16&e.compareDocumentPosition(i)))}:function(e,t){if(t)for(;t=t.parentNode;)if(t===e)return!0;return!1},_=t?function(e,t){if(e===t)return h=!0,0;var i=!e.compareDocumentPosition-!t.compareDocumentPosition;return i||(1&(i=(e.ownerDocument||e)===(t.ownerDocument||t)?e.compareDocumentPosition(t):1)||!n.sortDetached&&t.compareDocumentPosition(e)===i?e===f||e.ownerDocument===w&&b(w,e)?-1:t===f||t.ownerDocument===w&&b(w,t)?1:u?L(u,e)-L(u,t):0:4&i?-1:1)}:function(e,t){if(e===t)return h=!0,0;var n,i=0,r=e.parentNode,a=t.parentNode,o=[e],s=[t];if(!r||!a)return e===f?-1:t===f?1:r?-1:a?1:u?L(u,e)-L(u,t):0;if(r===a)return de(e,t);for(n=e;n=n.parentNode;)o.unshift(n);for(n=t;n=n.parentNode;)s.unshift(n);for(;o[i]===s[i];)i++;return i?de(o[i],s[i]):o[i]===w?-1:s[i]===w?1:0},f):f},se.matches=function(e,t){return se(e,null,null,t)},se.matchesSelector=function(e,t){if((e.ownerDocument||e)!==f&&d(e),n.matchesSelector&&g&&!A[t+" "]&&(!v||!v.test(t))&&(!m||!m.test(t)))try{var i=y.call(e,t);if(i||n.disconnectedMatch||e.document&&11!==e.document.nodeType)return i}catch(e){A(t,!0)}return se(t,f,null,[e]).length>0},se.contains=function(e,t){return(e.ownerDocument||e)!==f&&d(e),b(e,t)},se.attr=function(e,t){(e.ownerDocument||e)!==f&&d(e);var r=i.attrHandle[t.toLowerCase()],a=r&&O.call(i.attrHandle,t.toLowerCase())?r(e,t,!g):void 0;return void 0!==a?a:n.attributes||!g?e.getAttribute(t):(a=e.getAttributeNode(t))&&a.specified?a.value:null},se.escape=function(e){return(e+"").replace(ie,re)},se.error=function(e){throw new Error("Syntax error, unrecognized expression: "+e)},se.uniqueSort=function(e){var t,i=[],r=0,a=0;if(h=!n.detectDuplicates,u=!n.sortStable&&e.slice(0),e.sort(_),h){for(;t=e[a++];)t===e[a]&&(r=i.push(a));for(;r--;)e.splice(i[r],1)}return u=null,e},r=se.getText=function(e){var t,n="",i=0,a=e.nodeType;if(a){if(1===a||9===a||11===a){if("string"==typeof e.textContent)return e.textContent;for(e=e.firstChild;e;e=e.nextSibling)n+=r(e)}else if(3===a||4===a)return e.nodeValue}else for(;t=e[i++];)n+=r(t);return n},(i=se.selectors={cacheLength:50,createPseudo:ce,match:G,attrHandle:{},find:{},relative:{">":{dir:"parentNode",first:!0}," ":{dir:"parentNode"},"+":{dir:"previousSibling",first:!0},"~":{dir:"previousSibling"}},preFilter:{ATTR:function(e){return e[1]=e[1].replace(te,ne),e[3]=(e[3]||e[4]||e[5]||"").replace(te,ne),"~="===e[2]&&(e[3]=" "+e[3]+" "),e.slice(0,4)},CHILD:function(e){return e[1]=e[1].toLowerCase(),"nth"===e[1].slice(0,3)?(e[3]||se.error(e[0]),e[4]=+(e[4]?e[5]+(e[6]||1):2*("even"===e[3]||"odd"===e[3])),e[5]=+(e[7]+e[8]||"odd"===e[3])):e[3]&&se.error(e[0]),e},PSEUDO:function(e){var t,n=!e[6]&&e[2];return G.CHILD.test(e[0])?null:(e[3]?e[2]=e[4]||e[5]||"":n&&q.test(n)&&(t=o(n,!0))&&(t=n.indexOf(")",n.length-t)-n.length)&&(e[0]=e[0].slice(0,t),e[2]=n.slice(0,t)),e.slice(0,3))}},filter:{TAG:function(e){var t=e.replace(te,ne).toLowerCase();return"*"===e?function(){return!0}:function(e){return e.nodeName&&e.nodeName.toLowerCase()===t}},CLASS:function(e){var t=E[e+" "];return t||(t=new RegExp("(^|"+F+")"+e+"("+F+"|$)"))&&E(e,function(e){return t.test("string"==typeof e.className&&e.className||void 0!==e.getAttribute&&e.getAttribute("class")||"")})},ATTR:function(e,t,n){return function(i){var r=se.attr(i,e);return null==r?"!="===t:!t||(r+="","="===t?r===n:"!="===t?r!==n:"^="===t?n&&0===r.indexOf(n):"*="===t?n&&r.indexOf(n)>-1:"$="===t?n&&r.slice(-n.length)===n:"~="===t?(" "+r.replace(H," ")+" ").indexOf(n)>-1:"|="===t&&(r===n||r.slice(0,n.length+1)===n+"-"))}},CHILD:function(e,t,n,i,r){var a="nth"!==e.slice(0,3),o="last"!==e.slice(-4),s="of-type"===t;return 1===i&&0===r?function(e){return!!e.parentNode}:function(t,n,l){var c,u,h,d,f,p,g=a!==o?"nextSibling":"previousSibling",m=t.parentNode,v=s&&t.nodeName.toLowerCase(),y=!l&&!s,b=!1;if(m){if(a){for(;g;){for(d=t;d=d[g];)if(s?d.nodeName.toLowerCase()===v:1===d.nodeType)return!1;p=g="only"===e&&!p&&"nextSibling"}return!0}if(p=[o?m.firstChild:m.lastChild],o&&y){for(b=(f=(c=(u=(h=(d=m)[x]||(d[x]={}))[d.uniqueID]||(h[d.uniqueID]={}))[e]||[])[0]===k&&c[1])&&c[2],d=f&&m.childNodes[f];d=++f&&d&&d[g]||(b=f=0)||p.pop();)if(1===d.nodeType&&++b&&d===t){u[e]=[k,f,b];break}}else if(y&&(b=f=(c=(u=(h=(d=t)[x]||(d[x]={}))[d.uniqueID]||(h[d.uniqueID]={}))[e]||[])[0]===k&&c[1]),!1===b)for(;(d=++f&&d&&d[g]||(b=f=0)||p.pop())&&((s?d.nodeName.toLowerCase()!==v:1!==d.nodeType)||!++b||(y&&((u=(h=d[x]||(d[x]={}))[d.uniqueID]||(h[d.uniqueID]={}))[e]=[k,b]),d!==t)););return(b-=r)===i||b%i==0&&b/i>=0}}},PSEUDO:function(e,t){var n,r=i.pseudos[e]||i.setFilters[e.toLowerCase()]||se.error("unsupported pseudo: "+e);return r[x]?r(t):r.length>1?(n=[e,e,"",t],i.setFilters.hasOwnProperty(e.toLowerCase())?ce(function(e,n){for(var i,a=r(e,t),o=a.length;o--;)e[i=L(e,a[o])]=!(n[i]=a[o])}):function(e){return r(e,0,n)}):r}},pseudos:{not:ce(function(e){var t=[],n=[],i=s(e.replace(W,"$1"));return i[x]?ce(function(e,t,n,r){for(var a,o=i(e,null,r,[]),s=e.length;s--;)(a=o[s])&&(e[s]=!(t[s]=a))}):function(e,r,a){return t[0]=e,i(t,null,a,n),t[0]=null,!n.pop()}}),has:ce(function(e){return function(t){return se(e,t).length>0}}),contains:ce(function(e){return e=e.replace(te,ne),function(t){return(t.textContent||r(t)).indexOf(e)>-1}}),lang:ce(function(e){return U.test(e||"")||se.error("unsupported lang: "+e),e=e.replace(te,ne).toLowerCase(),function(t){var n;do{if(n=g?t.lang:t.getAttribute("xml:lang")||t.getAttribute("lang"))return(n=n.toLowerCase())===e||0===n.indexOf(e+"-")}while((t=t.parentNode)&&1===t.nodeType);return!1}}),target:function(t){var n=e.location&&e.location.hash;return n&&n.slice(1)===t.id},root:function(e){return e===p},focus:function(e){return e===f.activeElement&&(!f.hasFocus||f.hasFocus())&&!!(e.type||e.href||~e.tabIndex)},enabled:ge(!1),disabled:ge(!0),checked:function(e){var t=e.nodeName.toLowerCase();return"input"===t&&!!e.checked||"option"===t&&!!e.selected},selected:function(e){return e.parentNode&&e.parentNode.selectedIndex,!0===e.selected},empty:function(e){for(e=e.firstChild;e;e=e.nextSibling)if(e.nodeType<6)return!1;return!0},parent:function(e){return!i.pseudos.empty(e)},header:function(e){return Z.test(e.nodeName)},input:function(e){return $.test(e.nodeName)},button:function(e){var t=e.nodeName.toLowerCase();return"input"===t&&"button"===e.type||"button"===t},text:function(e){var t;return"input"===e.nodeName.toLowerCase()&&"text"===e.type&&(null==(t=e.getAttribute("type"))||"text"===t.toLowerCase())},first:me(function(){return[0]}),last:me(function(e,t){return[t-1]}),eq:me(function(e,t,n){return[n<0?n+t:n]}),even:me(function(e,t){for(var n=0;n<t;n+=2)e.push(n);return e}),odd:me(function(e,t){for(var n=1;n<t;n+=2)e.push(n);return e}),lt:me(function(e,t,n){for(var i=n<0?n+t:n>t?t:n;--i>=0;)e.push(i);return e}),gt:me(function(e,t,n){for(var i=n<0?n+t:n;++i<t;)e.push(i);return e})}}).pseudos.nth=i.pseudos.eq,{radio:!0,checkbox:!0,file:!0,password:!0,image:!0})i.pseudos[t]=fe(t);for(t in{submit:!0,reset:!0})i.pseudos[t]=pe(t);function ye(){}function be(e){for(var t=0,n=e.length,i="";t<n;t++)i+=e[t].value;return i}function xe(e,t,n){var i=t.dir,r=t.next,a=r||i,o=n&&"parentNode"===a,s=S++;return t.first?function(t,n,r){for(;t=t[i];)if(1===t.nodeType||o)return e(t,n,r);return!1}:function(t,n,l){var c,u,h,d=[k,s];if(l){for(;t=t[i];)if((1===t.nodeType||o)&&e(t,n,l))return!0}else for(;t=t[i];)if(1===t.nodeType||o)if(u=(h=t[x]||(t[x]={}))[t.uniqueID]||(h[t.uniqueID]={}),r&&r===t.nodeName.toLowerCase())t=t[i]||t;else{if((c=u[a])&&c[0]===k&&c[1]===s)return d[2]=c[2];if(u[a]=d,d[2]=e(t,n,l))return!0}return!1}}function we(e){return e.length>1?function(t,n,i){for(var r=e.length;r--;)if(!e[r](t,n,i))return!1;return!0}:e[0]}function ke(e,t,n,i,r){for(var a,o=[],s=0,l=e.length,c=null!=t;s<l;s++)(a=e[s])&&(n&&!n(a,i,r)||(o.push(a),c&&t.push(s)));return o}function Se(e,t,n,i,r,a){return i&&!i[x]&&(i=Se(i)),r&&!r[x]&&(r=Se(r,a)),ce(function(a,o,s,l){var c,u,h,d=[],f=[],p=o.length,g=a||function(e,t,n){for(var i=0,r=t.length;i<r;i++)se(e,t[i],n);return n}(t||"*",s.nodeType?[s]:s,[]),m=!e||!a&&t?g:ke(g,d,e,s,l),v=n?r||(a?e:p||i)?[]:o:m;if(n&&n(m,v,s,l),i)for(c=ke(v,f),i(c,[],s,l),u=c.length;u--;)(h=c[u])&&(v[f[u]]=!(m[f[u]]=h));if(a){if(r||e){if(r){for(c=[],u=v.length;u--;)(h=v[u])&&c.push(m[u]=h);r(null,v=[],c,l)}for(u=v.length;u--;)(h=v[u])&&(c=r?L(a,h):d[u])>-1&&(a[c]=!(o[c]=h))}}else v=ke(v===o?v.splice(p,v.length):v),r?r(null,o,v,l):D.apply(o,v)})}function Ee(e){for(var t,n,r,a=e.length,o=i.relative[e[0].type],s=o||i.relative[" "],l=o?1:0,u=xe(function(e){return e===t},s,!0),h=xe(function(e){return L(t,e)>-1},s,!0),d=[function(e,n,i){var r=!o&&(i||n!==c)||((t=n).nodeType?u(e,n,i):h(e,n,i));return t=null,r}];l<a;l++)if(n=i.relative[e[l].type])d=[xe(we(d),n)];else{if((n=i.filter[e[l].type].apply(null,e[l].matches))[x]){for(r=++l;r<a&&!i.relative[e[r].type];r++);return Se(l>1&&we(d),l>1&&be(e.slice(0,l-1).concat({value:" "===e[l-2].type?"*":""})).replace(W,"$1"),n,l<r&&Ee(e.slice(l,r)),r<a&&Ee(e=e.slice(r)),r<a&&be(e))}d.push(n)}return we(d)}return ye.prototype=i.filters=i.pseudos,i.setFilters=new ye,o=se.tokenize=function(e,t){var n,r,a,o,s,l,c,u=C[e+" "];if(u)return t?0:u.slice(0);for(s=e,l=[],c=i.preFilter;s;){for(o in n&&!(r=X.exec(s))||(r&&(s=s.slice(r[0].length)||s),l.push(a=[])),n=!1,(r=V.exec(s))&&(n=r.shift(),a.push({value:n,type:r[0].replace(W," ")}),s=s.slice(n.length)),i.filter)!(r=G[o].exec(s))||c[o]&&!(r=c[o](r))||(n=r.shift(),a.push({value:n,type:o,matches:r}),s=s.slice(n.length));if(!n)break}return t?s.length:s?se.error(e):C(e,l).slice(0)},s=se.compile=function(e,t){var n,r=[],a=[],s=T[e+" "];if(!s){for(t||(t=o(e)),n=t.length;n--;)(s=Ee(t[n]))[x]?r.push(s):a.push(s);(s=T(e,function(e,t){var n=t.length>0,r=e.length>0,a=function(a,o,s,l,u){var h,p,m,v=0,y="0",b=a&&[],x=[],w=c,S=a||r&&i.find.TAG("*",u),E=k+=null==w?1:Math.random()||.1,C=S.length;for(u&&(c=o===f||o||u);y!==C&&null!=(h=S[y]);y++){if(r&&h){for(p=0,o||h.ownerDocument===f||(d(h),s=!g);m=e[p++];)if(m(h,o||f,s)){l.push(h);break}u&&(k=E)}n&&((h=!m&&h)&&v--,a&&b.push(h))}if(v+=y,n&&y!==v){for(p=0;m=t[p++];)m(b,x,o,s);if(a){if(v>0)for(;y--;)b[y]||x[y]||(x[y]=M.call(l));x=ke(x)}D.apply(l,x),u&&!a&&x.length>0&&v+t.length>1&&se.uniqueSort(l)}return u&&(k=E,c=w),b};return n?ce(a):a}(a,r))).selector=e}return s},l=se.select=function(e,t,n,r){var a,l,c,u,h,d="function"==typeof e&&e,f=!r&&o(e=d.selector||e);if(n=n||[],1===f.length){if((l=f[0]=f[0].slice(0)).length>2&&"ID"===(c=l[0]).type&&9===t.nodeType&&g&&i.relative[l[1].type]){if(!(t=(i.find.ID(c.matches[0].replace(te,ne),t)||[])[0]))return n;d&&(t=t.parentNode),e=e.slice(l.shift().value.length)}for(a=G.needsContext.test(e)?0:l.length;a--&&(c=l[a],!i.relative[u=c.type]);)if((h=i.find[u])&&(r=h(c.matches[0].replace(te,ne),ee.test(l[0].type)&&ve(t.parentNode)||t))){if(l.splice(a,1),!(e=r.length&&be(l)))return D.apply(n,r),n;break}}return(d||s(e,f))(r,t,!g,n,!t||ee.test(e)&&ve(t.parentNode)||t),n},n.sortStable=x.split("").sort(_).join("")===x,n.detectDuplicates=!!h,d(),n.sortDetached=ue(function(e){return 1&e.compareDocumentPosition(f.createElement("fieldset"))}),ue(function(e){return e.innerHTML="<a href='#'></a>","#"===e.firstChild.getAttribute("href")})||he("type|href|height|width",function(e,t,n){if(!n)return e.getAttribute(t,"type"===t.toLowerCase()?1:2)}),n.attributes&&ue(function(e){return e.innerHTML="<input/>",e.firstChild.setAttribute("value",""),""===e.firstChild.getAttribute("value")})||he("value",function(e,t,n){if(!n&&"input"===e.nodeName.toLowerCase())return e.defaultValue}),ue(function(e){return null==e.getAttribute("disabled")})||he(R,function(e,t,n){var i;if(!n)return!0===e[t]?t.toLowerCase():(i=e.getAttributeNode(t))&&i.specified?i.value:null}),se}(n);S.find=T,S.expr=T.selectors,S.expr[":"]=S.expr.pseudos,S.uniqueSort=S.unique=T.uniqueSort,S.text=T.getText,S.isXMLDoc=T.isXML,S.contains=T.contains,S.escapeSelector=T.escape;var A=function(e,t,n){for(var i=[],r=void 0!==n;(e=e[t])&&9!==e.nodeType;)if(1===e.nodeType){if(r&&S(e).is(n))break;i.push(e)}return i},_=function(e,t){for(var n=[];e;e=e.nextSibling)1===e.nodeType&&e!==t&&n.push(e);return n},O=S.expr.match.needsContext;function P(e,t){return e.nodeName&&e.nodeName.toLowerCase()===t.toLowerCase()}var M=/^<([a-z][^\/\0>:\x20\t\r\n\f]*)[\x20\t\r\n\f]*\/?>(?:<\/\1>|)$/i;function I(e,t,n){return y(t)?S.grep(e,function(e,i){return!!t.call(e,i,e)!==n}):t.nodeType?S.grep(e,function(e){return e===t!==n}):"string"!=typeof t?S.grep(e,function(e){return h.call(t,e)>-1!==n}):S.filter(t,e,n)}S.filter=function(e,t,n){var i=t[0];return n&&(e=":not("+e+")"),1===t.length&&1===i.nodeType?S.find.matchesSelector(i,e)?[i]:[]:S.find.matches(e,S.grep(t,function(e){return 1===e.nodeType}))},S.fn.extend({find:function(e){var t,n,i=this.length,r=this;if("string"!=typeof e)return this.pushStack(S(e).filter(function(){for(t=0;t<i;t++)if(S.contains(r[t],this))return!0}));for(n=this.pushStack([]),t=0;t<i;t++)S.find(e,r[t],n);return i>1?S.uniqueSort(n):n},filter:function(e){return this.pushStack(I(this,e||[],!1))},not:function(e){return this.pushStack(I(this,e||[],!0))},is:function(e){return!!I(this,"string"==typeof e&&O.test(e)?S(e):e||[],!1).length}});var D,N=/^(?:\s*(<[\w\W]+>)[^>]*|#([\w-]+))$/;(S.fn.init=function(e,t,n){var i,r;if(!e)return this;if(n=n||D,"string"==typeof e){if(!(i="<"===e[0]&&">"===e[e.length-1]&&e.length>=3?[null,e,null]:N.exec(e))||!i[1]&&t)return!t||t.jquery?(t||n).find(e):this.constructor(t).find(e);if(i[1]){if(t=t instanceof S?t[0]:t,S.merge(this,S.parseHTML(i[1],t&&t.nodeType?t.ownerDocument||t:o,!0)),M.test(i[1])&&S.isPlainObject(t))for(i in t)y(this[i])?this[i](t[i]):this.attr(i,t[i]);return this}return(r=o.getElementById(i[2]))&&(this[0]=r,this.length=1),this}return e.nodeType?(this[0]=e,this.length=1,this):y(e)?void 0!==n.ready?n.ready(e):e(S):S.makeArray(e,this)}).prototype=S.fn,D=S(o);var L=/^(?:parents|prev(?:Until|All))/,R={children:!0,contents:!0,next:!0,prev:!0};function F(e,t){for(;(e=e[t])&&1!==e.nodeType;);return e}S.fn.extend({has:function(e){var t=S(e,this),n=t.length;return this.filter(function(){for(var e=0;e<n;e++)if(S.contains(this,t[e]))return!0})},closest:function(e,t){var n,i=0,r=this.length,a=[],o="string"!=typeof e&&S(e);if(!O.test(e))for(;i<r;i++)for(n=this[i];n&&n!==t;n=n.parentNode)if(n.nodeType<11&&(o?o.index(n)>-1:1===n.nodeType&&S.find.matchesSelector(n,e))){a.push(n);break}return this.pushStack(a.length>1?S.uniqueSort(a):a)},index:function(e){return e?"string"==typeof e?h.call(S(e),this[0]):h.call(this,e.jquery?e[0]:e):this[0]&&this[0].parentNode?this.first().prevAll().length:-1},add:function(e,t){return this.pushStack(S.uniqueSort(S.merge(this.get(),S(e,t))))},addBack:function(e){return this.add(null==e?this.prevObject:this.prevObject.filter(e))}}),S.each({parent:function(e){var t=e.parentNode;return t&&11!==t.nodeType?t:null},parents:function(e){return A(e,"parentNode")},parentsUntil:function(e,t,n){return A(e,"parentNode",n)},next:function(e){return F(e,"nextSibling")},prev:function(e){return F(e,"previousSibling")},nextAll:function(e){return A(e,"nextSibling")},prevAll:function(e){return A(e,"previousSibling")},nextUntil:function(e,t,n){return A(e,"nextSibling",n)},prevUntil:function(e,t,n){return A(e,"previousSibling",n)},siblings:function(e){return _((e.parentNode||{}).firstChild,e)},children:function(e){return _(e.firstChild)},contents:function(e){return void 0!==e.contentDocument?e.contentDocument:(P(e,"template")&&(e=e.content||e),S.merge([],e.childNodes))}},function(e,t){S.fn[e]=function(n,i){var r=S.map(this,t,n);return"Until"!==e.slice(-5)&&(i=n),i&&"string"==typeof i&&(r=S.filter(i,r)),this.length>1&&(R[e]||S.uniqueSort(r),L.test(e)&&r.reverse()),this.pushStack(r)}});var j=/[^\x20\t\r\n\f]+/g;function z(e){return e}function Y(e){throw e}function H(e,t,n,i){var r;try{e&&y(r=e.promise)?r.call(e).done(t).fail(n):e&&y(r=e.then)?r.call(e,t,n):t.apply(void 0,[e].slice(i))}catch(e){n.apply(void 0,[e])}}S.Callbacks=function(e){e="string"==typeof e?function(e){var t={};return S.each(e.match(j)||[],function(e,n){t[n]=!0}),t}(e):S.extend({},e);var t,n,i,r,a=[],o=[],s=-1,l=function(){for(r=r||e.once,i=t=!0;o.length;s=-1)for(n=o.shift();++s<a.length;)!1===a[s].apply(n[0],n[1])&&e.stopOnFalse&&(s=a.length,n=!1);e.memory||(n=!1),t=!1,r&&(a=n?[]:"")},c={add:function(){return a&&(n&&!t&&(s=a.length-1,o.push(n)),function t(n){S.each(n,function(n,i){y(i)?e.unique&&c.has(i)||a.push(i):i&&i.length&&"string"!==k(i)&&t(i)})}(arguments),n&&!t&&l()),this},remove:function(){return S.each(arguments,function(e,t){for(var n;(n=S.inArray(t,a,n))>-1;)a.splice(n,1),n<=s&&s--}),this},has:function(e){return e?S.inArray(e,a)>-1:a.length>0},empty:function(){return a&&(a=[]),this},disable:function(){return r=o=[],a=n="",this},disabled:function(){return!a},lock:function(){return r=o=[],n||t||(a=n=""),this},locked:function(){return!!r},fireWith:function(e,n){return r||(n=[e,(n=n||[]).slice?n.slice():n],o.push(n),t||l()),this},fire:function(){return c.fireWith(this,arguments),this},fired:function(){return!!i}};return c},S.extend({Deferred:function(e){var t=[["notify","progress",S.Callbacks("memory"),S.Callbacks("memory"),2],["resolve","done",S.Callbacks("once memory"),S.Callbacks("once memory"),0,"resolved"],["reject","fail",S.Callbacks("once memory"),S.Callbacks("once memory"),1,"rejected"]],i="pending",r={state:function(){return i},always:function(){return a.done(arguments).fail(arguments),this},catch:function(e){return r.then(null,e)},pipe:function(){var e=arguments;return S.Deferred(function(n){S.each(t,function(t,i){var r=y(e[i[4]])&&e[i[4]];a[i[1]](function(){var e=r&&r.apply(this,arguments);e&&y(e.promise)?e.promise().progress(n.notify).done(n.resolve).fail(n.reject):n[i[0]+"With"](this,r?[e]:arguments)})}),e=null}).promise()},then:function(e,i,r){var a=0;function o(e,t,i,r){return function(){var s=this,l=arguments,c=function(){var n,c;if(!(e<a)){if((n=i.apply(s,l))===t.promise())throw new TypeError("Thenable self-resolution");c=n&&("object"==typeof n||"function"==typeof n)&&n.then,y(c)?r?c.call(n,o(a,t,z,r),o(a,t,Y,r)):(a++,c.call(n,o(a,t,z,r),o(a,t,Y,r),o(a,t,z,t.notifyWith))):(i!==z&&(s=void 0,l=[n]),(r||t.resolveWith)(s,l))}},u=r?c:function(){try{c()}catch(n){S.Deferred.exceptionHook&&S.Deferred.exceptionHook(n,u.stackTrace),e+1>=a&&(i!==Y&&(s=void 0,l=[n]),t.rejectWith(s,l))}};e?u():(S.Deferred.getStackHook&&(u.stackTrace=S.Deferred.getStackHook()),n.setTimeout(u))}}return S.Deferred(function(n){t[0][3].add(o(0,n,y(r)?r:z,n.notifyWith)),t[1][3].add(o(0,n,y(e)?e:z)),t[2][3].add(o(0,n,y(i)?i:Y))}).promise()},promise:function(e){return null!=e?S.extend(e,r):r}},a={};return S.each(t,function(e,n){var o=n[2],s=n[5];r[n[1]]=o.add,s&&o.add(function(){i=s},t[3-e][2].disable,t[3-e][3].disable,t[0][2].lock,t[0][3].lock),o.add(n[3].fire),a[n[0]]=function(){return a[n[0]+"With"](this===a?void 0:this,arguments),this},a[n[0]+"With"]=o.fireWith}),r.promise(a),e&&e.call(a,a),a},when:function(e){var t=arguments.length,n=t,i=Array(n),r=l.call(arguments),a=S.Deferred(),o=function(e){return function(n){i[e]=this,r[e]=arguments.length>1?l.call(arguments):n,--t||a.resolveWith(i,r)}};if(t<=1&&(H(e,a.done(o(n)).resolve,a.reject,!t),"pending"===a.state()||y(r[n]&&r[n].then)))return a.then();for(;n--;)H(r[n],o(n),a.reject);return a.promise()}});var W=/^(Eval|Internal|Range|Reference|Syntax|Type|URI)Error$/;S.Deferred.exceptionHook=function(e,t){n.console&&n.console.warn&&e&&W.test(e.name)&&n.console.warn("jQuery.Deferred exception: "+e.message,e.stack,t)},S.readyException=function(e){n.setTimeout(function(){throw e})};var X=S.Deferred();function V(){o.removeEventListener("DOMContentLoaded",V),n.removeEventListener("load",V),S.ready()}S.fn.ready=function(e){return X.then(e).catch(function(e){S.readyException(e)}),this},S.extend({isReady:!1,readyWait:1,ready:function(e){(!0===e?--S.readyWait:S.isReady)||(S.isReady=!0,!0!==e&&--S.readyWait>0||X.resolveWith(o,[S]))}}),S.ready.then=X.then,"complete"===o.readyState||"loading"!==o.readyState&&!o.documentElement.doScroll?n.setTimeout(S.ready):(o.addEventListener("DOMContentLoaded",V),n.addEventListener("load",V));var B=function(e,t,n,i,r,a,o){var s=0,l=e.length,c=null==n;if("object"===k(n))for(s in r=!0,n)B(e,t,s,n[s],!0,a,o);else if(void 0!==i&&(r=!0,y(i)||(o=!0),c&&(o?(t.call(e,i),t=null):(c=t,t=function(e,t,n){return c.call(S(e),n)})),t))for(;s<l;s++)t(e[s],n,o?i:i.call(e[s],s,t(e[s],n)));return r?e:c?t.call(e):l?t(e[0],n):a},q=/^-ms-/,U=/-([a-z])/g;function G(e,t){return t.toUpperCase()}function Q(e){return e.replace(q,"ms-").replace(U,G)}var $=function(e){return 1===e.nodeType||9===e.nodeType||!+e.nodeType};function Z(){this.expando=S.expando+Z.uid++}Z.uid=1,Z.prototype={cache:function(e){var t=e[this.expando];return t||(t={},$(e)&&(e.nodeType?e[this.expando]=t:Object.defineProperty(e,this.expando,{value:t,configurable:!0}))),t},set:function(e,t,n){var i,r=this.cache(e);if("string"==typeof t)r[Q(t)]=n;else for(i in t)r[Q(i)]=t[i];return r},get:function(e,t){return void 0===t?this.cache(e):e[this.expando]&&e[this.expando][Q(t)]},access:function(e,t,n){return void 0===t||t&&"string"==typeof t&&void 0===n?this.get(e,t):(this.set(e,t,n),void 0!==n?n:t)},remove:function(e,t){var n,i=e[this.expando];if(void 0!==i){if(void 0!==t){n=(t=Array.isArray(t)?t.map(Q):(t=Q(t))in i?[t]:t.match(j)||[]).length;for(;n--;)delete i[t[n]]}(void 0===t||S.isEmptyObject(i))&&(e.nodeType?e[this.expando]=void 0:delete e[this.expando])}},hasData:function(e){var t=e[this.expando];return void 0!==t&&!S.isEmptyObject(t)}};var K=new Z,J=new Z,ee=/^(?:\{[\w\W]*\}|\[[\w\W]*\])$/,te=/[A-Z]/g;function ne(e,t,n){var i;if(void 0===n&&1===e.nodeType)if(i="data-"+t.replace(te,"-$&").toLowerCase(),"string"==typeof(n=e.getAttribute(i))){try{n=function(e){return"true"===e||"false"!==e&&("null"===e?null:e===+e+""?+e:ee.test(e)?JSON.parse(e):e)}(n)}catch(e){}J.set(e,t,n)}else n=void 0;return n}S.extend({hasData:function(e){return J.hasData(e)||K.hasData(e)},data:function(e,t,n){return J.access(e,t,n)},removeData:function(e,t){J.remove(e,t)},_data:function(e,t,n){return K.access(e,t,n)},_removeData:function(e,t){K.remove(e,t)}}),S.fn.extend({data:function(e,t){var n,i,r,a=this[0],o=a&&a.attributes;if(void 0===e){if(this.length&&(r=J.get(a),1===a.nodeType&&!K.get(a,"hasDataAttrs"))){for(n=o.length;n--;)o[n]&&0===(i=o[n].name).indexOf("data-")&&(i=Q(i.slice(5)),ne(a,i,r[i]));K.set(a,"hasDataAttrs",!0)}return r}return"object"==typeof e?this.each(function(){J.set(this,e)}):B(this,function(t){var n;if(a&&void 0===t)return void 0!==(n=J.get(a,e))?n:void 0!==(n=ne(a,e))?n:void 0;this.each(function(){J.set(this,e,t)})},null,t,arguments.length>1,null,!0)},removeData:function(e){return this.each(function(){J.remove(this,e)})}}),S.extend({queue:function(e,t,n){var i;if(e)return t=(t||"fx")+"queue",i=K.get(e,t),n&&(!i||Array.isArray(n)?i=K.access(e,t,S.makeArray(n)):i.push(n)),i||[]},dequeue:function(e,t){t=t||"fx";var n=S.queue(e,t),i=n.length,r=n.shift(),a=S._queueHooks(e,t);"inprogress"===r&&(r=n.shift(),i--),r&&("fx"===t&&n.unshift("inprogress"),delete a.stop,r.call(e,function(){S.dequeue(e,t)},a)),!i&&a&&a.empty.fire()},_queueHooks:function(e,t){var n=t+"queueHooks";return K.get(e,n)||K.access(e,n,{empty:S.Callbacks("once memory").add(function(){K.remove(e,[t+"queue",n])})})}}),S.fn.extend({queue:function(e,t){var n=2;return"string"!=typeof e&&(t=e,e="fx",n--),arguments.length<n?S.queue(this[0],e):void 0===t?this:this.each(function(){var n=S.queue(this,e,t);S._queueHooks(this,e),"fx"===e&&"inprogress"!==n[0]&&S.dequeue(this,e)})},dequeue:function(e){return this.each(function(){S.dequeue(this,e)})},clearQueue:function(e){return this.queue(e||"fx",[])},promise:function(e,t){var n,i=1,r=S.Deferred(),a=this,o=this.length,s=function(){--i||r.resolveWith(a,[a])};for("string"!=typeof e&&(t=e,e=void 0),e=e||"fx";o--;)(n=K.get(a[o],e+"queueHooks"))&&n.empty&&(i++,n.empty.add(s));return s(),r.promise(t)}});var ie=/[+-]?(?:\d*\.|)\d+(?:[eE][+-]?\d+|)/.source,re=new RegExp("^(?:([+-])=|)("+ie+")([a-z%]*)$","i"),ae=["Top","Right","Bottom","Left"],oe=o.documentElement,se=function(e){return S.contains(e.ownerDocument,e)},le={composed:!0};oe.attachShadow&&(se=function(e){return S.contains(e.ownerDocument,e)||e.getRootNode(le)===e.ownerDocument});var ce=function(e,t){return"none"===(e=t||e).style.display||""===e.style.display&&se(e)&&"none"===S.css(e,"display")},ue=function(e,t,n,i){var r,a,o={};for(a in t)o[a]=e.style[a],e.style[a]=t[a];for(a in r=n.apply(e,i||[]),t)e.style[a]=o[a];return r};function he(e,t,n,i){var r,a,o=20,s=i?function(){return i.cur()}:function(){return S.css(e,t,"")},l=s(),c=n&&n[3]||(S.cssNumber[t]?"":"px"),u=e.nodeType&&(S.cssNumber[t]||"px"!==c&&+l)&&re.exec(S.css(e,t));if(u&&u[3]!==c){for(l/=2,c=c||u[3],u=+l||1;o--;)S.style(e,t,u+c),(1-a)*(1-(a=s()/l||.5))<=0&&(o=0),u/=a;u*=2,S.style(e,t,u+c),n=n||[]}return n&&(u=+u||+l||0,r=n[1]?u+(n[1]+1)*n[2]:+n[2],i&&(i.unit=c,i.start=u,i.end=r)),r}var de={};function fe(e){var t,n=e.ownerDocument,i=e.nodeName,r=de[i];return r||(t=n.body.appendChild(n.createElement(i)),r=S.css(t,"display"),t.parentNode.removeChild(t),"none"===r&&(r="block"),de[i]=r,r)}function pe(e,t){for(var n,i,r=[],a=0,o=e.length;a<o;a++)(i=e[a]).style&&(n=i.style.display,t?("none"===n&&(r[a]=K.get(i,"display")||null,r[a]||(i.style.display="")),""===i.style.display&&ce(i)&&(r[a]=fe(i))):"none"!==n&&(r[a]="none",K.set(i,"display",n)));for(a=0;a<o;a++)null!=r[a]&&(e[a].style.display=r[a]);return e}S.fn.extend({show:function(){return pe(this,!0)},hide:function(){return pe(this)},toggle:function(e){return"boolean"==typeof e?e?this.show():this.hide():this.each(function(){ce(this)?S(this).show():S(this).hide()})}});var ge=/^(?:checkbox|radio)$/i,me=/<([a-z][^\/\0>\x20\t\r\n\f]*)/i,ve=/^$|^module$|\/(?:java|ecma)script/i,ye={option:[1,"<select multiple='multiple'>","</select>"],thead:[1,"<table>","</table>"],col:[2,"<table><colgroup>","</colgroup></table>"],tr:[2,"<table><tbody>","</tbody></table>"],td:[3,"<table><tbody><tr>","</tr></tbody></table>"],_default:[0,"",""]};function be(e,t){var n;return n=void 0!==e.getElementsByTagName?e.getElementsByTagName(t||"*"):void 0!==e.querySelectorAll?e.querySelectorAll(t||"*"):[],void 0===t||t&&P(e,t)?S.merge([e],n):n}function xe(e,t){for(var n=0,i=e.length;n<i;n++)K.set(e[n],"globalEval",!t||K.get(t[n],"globalEval"))}ye.optgroup=ye.option,ye.tbody=ye.tfoot=ye.colgroup=ye.caption=ye.thead,ye.th=ye.td;var we,ke,Se=/<|&#?\w+;/;function Ee(e,t,n,i,r){for(var a,o,s,l,c,u,h=t.createDocumentFragment(),d=[],f=0,p=e.length;f<p;f++)if((a=e[f])||0===a)if("object"===k(a))S.merge(d,a.nodeType?[a]:a);else if(Se.test(a)){for(o=o||h.appendChild(t.createElement("div")),s=(me.exec(a)||["",""])[1].toLowerCase(),l=ye[s]||ye._default,o.innerHTML=l[1]+S.htmlPrefilter(a)+l[2],u=l[0];u--;)o=o.lastChild;S.merge(d,o.childNodes),(o=h.firstChild).textContent=""}else d.push(t.createTextNode(a));for(h.textContent="",f=0;a=d[f++];)if(i&&S.inArray(a,i)>-1)r&&r.push(a);else if(c=se(a),o=be(h.appendChild(a),"script"),c&&xe(o),n)for(u=0;a=o[u++];)ve.test(a.type||"")&&n.push(a);return h}we=o.createDocumentFragment().appendChild(o.createElement("div")),(ke=o.createElement("input")).setAttribute("type","radio"),ke.setAttribute("checked","checked"),ke.setAttribute("name","t"),we.appendChild(ke),v.checkClone=we.cloneNode(!0).cloneNode(!0).lastChild.checked,we.innerHTML="<textarea>x</textarea>",v.noCloneChecked=!!we.cloneNode(!0).lastChild.defaultValue;var Ce=/^key/,Te=/^(?:mouse|pointer|contextmenu|drag|drop)|click/,Ae=/^([^.]*)(?:\.(.+)|)/;function _e(){return!0}function Oe(){return!1}function Pe(e,t){return e===function(){try{return o.activeElement}catch(e){}}()==("focus"===t)}function Me(e,t,n,i,r,a){var o,s;if("object"==typeof t){for(s in"string"!=typeof n&&(i=i||n,n=void 0),t)Me(e,s,n,i,t[s],a);return e}if(null==i&&null==r?(r=n,i=n=void 0):null==r&&("string"==typeof n?(r=i,i=void 0):(r=i,i=n,n=void 0)),!1===r)r=Oe;else if(!r)return e;return 1===a&&(o=r,(r=function(e){return S().off(e),o.apply(this,arguments)}).guid=o.guid||(o.guid=S.guid++)),e.each(function(){S.event.add(this,t,r,i,n)})}function Ie(e,t,n){n?(K.set(e,t,!1),S.event.add(e,t,{namespace:!1,handler:function(e){var i,r,a=K.get(this,t);if(1&e.isTrigger&&this[t]){if(a)(S.event.special[t]||{}).delegateType&&e.stopPropagation();else if(a=l.call(arguments),K.set(this,t,a),i=n(this,t),this[t](),a!==(r=K.get(this,t))||i?K.set(this,t,!1):r=void 0,a!==r)return e.stopImmediatePropagation(),e.preventDefault(),r}else a&&(K.set(this,t,S.event.trigger(S.extend(a.shift(),S.Event.prototype),a,this)),e.stopImmediatePropagation())}})):S.event.add(e,t,_e)}S.event={global:{},add:function(e,t,n,i,r){var a,o,s,l,c,u,h,d,f,p,g,m=K.get(e);if(m)for(n.handler&&(n=(a=n).handler,r=a.selector),r&&S.find.matchesSelector(oe,r),n.guid||(n.guid=S.guid++),(l=m.events)||(l=m.events={}),(o=m.handle)||(o=m.handle=function(t){return void 0!==S&&S.event.triggered!==t.type?S.event.dispatch.apply(e,arguments):void 0}),c=(t=(t||"").match(j)||[""]).length;c--;)f=g=(s=Ae.exec(t[c])||[])[1],p=(s[2]||"").split(".").sort(),f&&(h=S.event.special[f]||{},f=(r?h.delegateType:h.bindType)||f,h=S.event.special[f]||{},u=S.extend({type:f,origType:g,data:i,handler:n,guid:n.guid,selector:r,needsContext:r&&S.expr.match.needsContext.test(r),namespace:p.join(".")},a),(d=l[f])||((d=l[f]=[]).delegateCount=0,h.setup&&!1!==h.setup.call(e,i,p,o)||e.addEventListener&&e.addEventListener(f,o)),h.add&&(h.add.call(e,u),u.handler.guid||(u.handler.guid=n.guid)),r?d.splice(d.delegateCount++,0,u):d.push(u),S.event.global[f]=!0)},remove:function(e,t,n,i,r){var a,o,s,l,c,u,h,d,f,p,g,m=K.hasData(e)&&K.get(e);if(m&&(l=m.events)){for(c=(t=(t||"").match(j)||[""]).length;c--;)if(f=g=(s=Ae.exec(t[c])||[])[1],p=(s[2]||"").split(".").sort(),f){for(h=S.event.special[f]||{},d=l[f=(i?h.delegateType:h.bindType)||f]||[],s=s[2]&&new RegExp("(^|\\.)"+p.join("\\.(?:.*\\.|)")+"(\\.|$)"),o=a=d.length;a--;)u=d[a],!r&&g!==u.origType||n&&n.guid!==u.guid||s&&!s.test(u.namespace)||i&&i!==u.selector&&("**"!==i||!u.selector)||(d.splice(a,1),u.selector&&d.delegateCount--,h.remove&&h.remove.call(e,u));o&&!d.length&&(h.teardown&&!1!==h.teardown.call(e,p,m.handle)||S.removeEvent(e,f,m.handle),delete l[f])}else for(f in l)S.event.remove(e,f+t[c],n,i,!0);S.isEmptyObject(l)&&K.remove(e,"handle events")}},dispatch:function(e){var t,n,i,r,a,o,s=S.event.fix(e),l=new Array(arguments.length),c=(K.get(this,"events")||{})[s.type]||[],u=S.event.special[s.type]||{};for(l[0]=s,t=1;t<arguments.length;t++)l[t]=arguments[t];if(s.delegateTarget=this,!u.preDispatch||!1!==u.preDispatch.call(this,s)){for(o=S.event.handlers.call(this,s,c),t=0;(r=o[t++])&&!s.isPropagationStopped();)for(s.currentTarget=r.elem,n=0;(a=r.handlers[n++])&&!s.isImmediatePropagationStopped();)s.rnamespace&&!1!==a.namespace&&!s.rnamespace.test(a.namespace)||(s.handleObj=a,s.data=a.data,void 0!==(i=((S.event.special[a.origType]||{}).handle||a.handler).apply(r.elem,l))&&!1===(s.result=i)&&(s.preventDefault(),s.stopPropagation()));return u.postDispatch&&u.postDispatch.call(this,s),s.result}},handlers:function(e,t){var n,i,r,a,o,s=[],l=t.delegateCount,c=e.target;if(l&&c.nodeType&&!("click"===e.type&&e.button>=1))for(;c!==this;c=c.parentNode||this)if(1===c.nodeType&&("click"!==e.type||!0!==c.disabled)){for(a=[],o={},n=0;n<l;n++)void 0===o[r=(i=t[n]).selector+" "]&&(o[r]=i.needsContext?S(r,this).index(c)>-1:S.find(r,this,null,[c]).length),o[r]&&a.push(i);a.length&&s.push({elem:c,handlers:a})}return c=this,l<t.length&&s.push({elem:c,handlers:t.slice(l)}),s},addProp:function(e,t){Object.defineProperty(S.Event.prototype,e,{enumerable:!0,configurable:!0,get:y(t)?function(){if(this.originalEvent)return t(this.originalEvent)}:function(){if(this.originalEvent)return this.originalEvent[e]},set:function(t){Object.defineProperty(this,e,{enumerable:!0,configurable:!0,writable:!0,value:t})}})},fix:function(e){return e[S.expando]?e:new S.Event(e)},special:{load:{noBubble:!0},click:{setup:function(e){var t=this||e;return ge.test(t.type)&&t.click&&P(t,"input")&&void 0===K.get(t,"click")&&Ie(t,"click",_e),!1},trigger:function(e){var t=this||e;return ge.test(t.type)&&t.click&&P(t,"input")&&void 0===K.get(t,"click")&&Ie(t,"click"),!0},_default:function(e){var t=e.target;return ge.test(t.type)&&t.click&&P(t,"input")&&K.get(t,"click")||P(t,"a")}},beforeunload:{postDispatch:function(e){void 0!==e.result&&e.originalEvent&&(e.originalEvent.returnValue=e.result)}}}},S.removeEvent=function(e,t,n){e.removeEventListener&&e.removeEventListener(t,n)},S.Event=function(e,t){if(!(this instanceof S.Event))return new S.Event(e,t);e&&e.type?(this.originalEvent=e,this.type=e.type,this.isDefaultPrevented=e.defaultPrevented||void 0===e.defaultPrevented&&!1===e.returnValue?_e:Oe,this.target=e.target&&3===e.target.nodeType?e.target.parentNode:e.target,this.currentTarget=e.currentTarget,this.relatedTarget=e.relatedTarget):this.type=e,t&&S.extend(this,t),this.timeStamp=e&&e.timeStamp||Date.now(),this[S.expando]=!0},S.Event.prototype={constructor:S.Event,isDefaultPrevented:Oe,isPropagationStopped:Oe,isImmediatePropagationStopped:Oe,isSimulated:!1,preventDefault:function(){var e=this.originalEvent;this.isDefaultPrevented=_e,e&&!this.isSimulated&&e.preventDefault()},stopPropagation:function(){var e=this.originalEvent;this.isPropagationStopped=_e,e&&!this.isSimulated&&e.stopPropagation()},stopImmediatePropagation:function(){var e=this.originalEvent;this.isImmediatePropagationStopped=_e,e&&!this.isSimulated&&e.stopImmediatePropagation(),this.stopPropagation()}},S.each({altKey:!0,bubbles:!0,cancelable:!0,changedTouches:!0,ctrlKey:!0,detail:!0,eventPhase:!0,metaKey:!0,pageX:!0,pageY:!0,shiftKey:!0,view:!0,char:!0,code:!0,charCode:!0,key:!0,keyCode:!0,button:!0,buttons:!0,clientX:!0,clientY:!0,offsetX:!0,offsetY:!0,pointerId:!0,pointerType:!0,screenX:!0,screenY:!0,targetTouches:!0,toElement:!0,touches:!0,which:function(e){var t=e.button;return null==e.which&&Ce.test(e.type)?null!=e.charCode?e.charCode:e.keyCode:!e.which&&void 0!==t&&Te.test(e.type)?1&t?1:2&t?3:4&t?2:0:e.which}},S.event.addProp),S.each({focus:"focusin",blur:"focusout"},function(e,t){S.event.special[e]={setup:function(){return Ie(this,e,Pe),!1},trigger:function(){return Ie(this,e),!0},delegateType:t}}),S.each({mouseenter:"mouseover",mouseleave:"mouseout",pointerenter:"pointerover",pointerleave:"pointerout"},function(e,t){S.event.special[e]={delegateType:t,bindType:t,handle:function(e){var n,i=e.relatedTarget,r=e.handleObj;return i&&(i===this||S.contains(this,i))||(e.type=r.origType,n=r.handler.apply(this,arguments),e.type=t),n}}}),S.fn.extend({on:function(e,t,n,i){return Me(this,e,t,n,i)},one:function(e,t,n,i){return Me(this,e,t,n,i,1)},off:function(e,t,n){var i,r;if(e&&e.preventDefault&&e.handleObj)return i=e.handleObj,S(e.delegateTarget).off(i.namespace?i.origType+"."+i.namespace:i.origType,i.selector,i.handler),this;if("object"==typeof e){for(r in e)this.off(r,t,e[r]);return this}return!1!==t&&"function"!=typeof t||(n=t,t=void 0),!1===n&&(n=Oe),this.each(function(){S.event.remove(this,e,n,t)})}});var De=/<(?!area|br|col|embed|hr|img|input|link|meta|param)(([a-z][^\/\0>\x20\t\r\n\f]*)[^>]*)\/>/gi,Ne=/<script|<style|<link/i,Le=/checked\s*(?:[^=]|=\s*.checked.)/i,Re=/^\s*<!(?:\[CDATA\[|--)|(?:\]\]|--)>\s*$/g;function Fe(e,t){return P(e,"table")&&P(11!==t.nodeType?t:t.firstChild,"tr")&&S(e).children("tbody")[0]||e}function je(e){return e.type=(null!==e.getAttribute("type"))+"/"+e.type,e}function ze(e){return"true/"===(e.type||"").slice(0,5)?e.type=e.type.slice(5):e.removeAttribute("type"),e}function Ye(e,t){var n,i,r,a,o,s,l,c;if(1===t.nodeType){if(K.hasData(e)&&(a=K.access(e),o=K.set(t,a),c=a.events))for(r in delete o.handle,o.events={},c)for(n=0,i=c[r].length;n<i;n++)S.event.add(t,r,c[r][n]);J.hasData(e)&&(s=J.access(e),l=S.extend({},s),J.set(t,l))}}function He(e,t,n,i){t=c.apply([],t);var r,a,o,s,l,u,h=0,d=e.length,f=d-1,p=t[0],g=y(p);if(g||d>1&&"string"==typeof p&&!v.checkClone&&Le.test(p))return e.each(function(r){var a=e.eq(r);g&&(t[0]=p.call(this,r,a.html())),He(a,t,n,i)});if(d&&(a=(r=Ee(t,e[0].ownerDocument,!1,e,i)).firstChild,1===r.childNodes.length&&(r=a),a||i)){for(s=(o=S.map(be(r,"script"),je)).length;h<d;h++)l=r,h!==f&&(l=S.clone(l,!0,!0),s&&S.merge(o,be(l,"script"))),n.call(e[h],l,h);if(s)for(u=o[o.length-1].ownerDocument,S.map(o,ze),h=0;h<s;h++)l=o[h],ve.test(l.type||"")&&!K.access(l,"globalEval")&&S.contains(u,l)&&(l.src&&"module"!==(l.type||"").toLowerCase()?S._evalUrl&&!l.noModule&&S._evalUrl(l.src,{nonce:l.nonce||l.getAttribute("nonce")}):w(l.textContent.replace(Re,""),l,u))}return e}function We(e,t,n){for(var i,r=t?S.filter(t,e):e,a=0;null!=(i=r[a]);a++)n||1!==i.nodeType||S.cleanData(be(i)),i.parentNode&&(n&&se(i)&&xe(be(i,"script")),i.parentNode.removeChild(i));return e}S.extend({htmlPrefilter:function(e){return e.replace(De,"<$1></$2>")},clone:function(e,t,n){var i,r,a,o,s,l,c,u=e.cloneNode(!0),h=se(e);if(!(v.noCloneChecked||1!==e.nodeType&&11!==e.nodeType||S.isXMLDoc(e)))for(o=be(u),i=0,r=(a=be(e)).length;i<r;i++)s=a[i],l=o[i],c=void 0,"input"===(c=l.nodeName.toLowerCase())&&ge.test(s.type)?l.checked=s.checked:"input"!==c&&"textarea"!==c||(l.defaultValue=s.defaultValue);if(t)if(n)for(a=a||be(e),o=o||be(u),i=0,r=a.length;i<r;i++)Ye(a[i],o[i]);else Ye(e,u);return(o=be(u,"script")).length>0&&xe(o,!h&&be(e,"script")),u},cleanData:function(e){for(var t,n,i,r=S.event.special,a=0;void 0!==(n=e[a]);a++)if($(n)){if(t=n[K.expando]){if(t.events)for(i in t.events)r[i]?S.event.remove(n,i):S.removeEvent(n,i,t.handle);n[K.expando]=void 0}n[J.expando]&&(n[J.expando]=void 0)}}}),S.fn.extend({detach:function(e){return We(this,e,!0)},remove:function(e){return We(this,e)},text:function(e){return B(this,function(e){return void 0===e?S.text(this):this.empty().each(function(){1!==this.nodeType&&11!==this.nodeType&&9!==this.nodeType||(this.textContent=e)})},null,e,arguments.length)},append:function(){return He(this,arguments,function(e){1!==this.nodeType&&11!==this.nodeType&&9!==this.nodeType||Fe(this,e).appendChild(e)})},prepend:function(){return He(this,arguments,function(e){if(1===this.nodeType||11===this.nodeType||9===this.nodeType){var t=Fe(this,e);t.insertBefore(e,t.firstChild)}})},before:function(){return He(this,arguments,function(e){this.parentNode&&this.parentNode.insertBefore(e,this)})},after:function(){return He(this,arguments,function(e){this.parentNode&&this.parentNode.insertBefore(e,this.nextSibling)})},empty:function(){for(var e,t=0;null!=(e=this[t]);t++)1===e.nodeType&&(S.cleanData(be(e,!1)),e.textContent="");return this},clone:function(e,t){return e=null!=e&&e,t=null==t?e:t,this.map(function(){return S.clone(this,e,t)})},html:function(e){return B(this,function(e){var t=this[0]||{},n=0,i=this.length;if(void 0===e&&1===t.nodeType)return t.innerHTML;if("string"==typeof e&&!Ne.test(e)&&!ye[(me.exec(e)||["",""])[1].toLowerCase()]){e=S.htmlPrefilter(e);try{for(;n<i;n++)1===(t=this[n]||{}).nodeType&&(S.cleanData(be(t,!1)),t.innerHTML=e);t=0}catch(e){}}t&&this.empty().append(e)},null,e,arguments.length)},replaceWith:function(){var e=[];return He(this,arguments,function(t){var n=this.parentNode;S.inArray(this,e)<0&&(S.cleanData(be(this)),n&&n.replaceChild(t,this))},e)}}),S.each({appendTo:"append",prependTo:"prepend",insertBefore:"before",insertAfter:"after",replaceAll:"replaceWith"},function(e,t){S.fn[e]=function(e){for(var n,i=[],r=S(e),a=r.length-1,o=0;o<=a;o++)n=o===a?this:this.clone(!0),S(r[o])[t](n),u.apply(i,n.get());return this.pushStack(i)}});var Xe=new RegExp("^("+ie+")(?!px)[a-z%]+$","i"),Ve=function(e){var t=e.ownerDocument.defaultView;return t&&t.opener||(t=n),t.getComputedStyle(e)},Be=new RegExp(ae.join("|"),"i");function qe(e,t,n){var i,r,a,o,s=e.style;return(n=n||Ve(e))&&(""!==(o=n.getPropertyValue(t)||n[t])||se(e)||(o=S.style(e,t)),!v.pixelBoxStyles()&&Xe.test(o)&&Be.test(t)&&(i=s.width,r=s.minWidth,a=s.maxWidth,s.minWidth=s.maxWidth=s.width=o,o=n.width,s.width=i,s.minWidth=r,s.maxWidth=a)),void 0!==o?o+"":o}function Ue(e,t){return{get:function(){if(!e())return(this.get=t).apply(this,arguments);delete this.get}}}!function(){function e(){if(u){c.style.cssText="position:absolute;left:-11111px;width:60px;margin-top:1px;padding:0;border:0",u.style.cssText="position:relative;display:block;box-sizing:border-box;overflow:scroll;margin:auto;border:1px;padding:1px;width:60%;top:1%",oe.appendChild(c).appendChild(u);var e=n.getComputedStyle(u);i="1%"!==e.top,l=12===t(e.marginLeft),u.style.right="60%",s=36===t(e.right),r=36===t(e.width),u.style.position="absolute",a=12===t(u.offsetWidth/3),oe.removeChild(c),u=null}}function t(e){return Math.round(parseFloat(e))}var i,r,a,s,l,c=o.createElement("div"),u=o.createElement("div");u.style&&(u.style.backgroundClip="content-box",u.cloneNode(!0).style.backgroundClip="",v.clearCloneStyle="content-box"===u.style.backgroundClip,S.extend(v,{boxSizingReliable:function(){return e(),r},pixelBoxStyles:function(){return e(),s},pixelPosition:function(){return e(),i},reliableMarginLeft:function(){return e(),l},scrollboxSize:function(){return e(),a}}))}();var Ge=["Webkit","Moz","ms"],Qe=o.createElement("div").style,$e={};function Ze(e){var t=S.cssProps[e]||$e[e];return t||(e in Qe?e:$e[e]=function(e){for(var t=e[0].toUpperCase()+e.slice(1),n=Ge.length;n--;)if((e=Ge[n]+t)in Qe)return e}(e)||e)}var Ke=/^(none|table(?!-c[ea]).+)/,Je=/^--/,et={position:"absolute",visibility:"hidden",display:"block"},tt={letterSpacing:"0",fontWeight:"400"};function nt(e,t,n){var i=re.exec(t);return i?Math.max(0,i[2]-(n||0))+(i[3]||"px"):t}function it(e,t,n,i,r,a){var o="width"===t?1:0,s=0,l=0;if(n===(i?"border":"content"))return 0;for(;o<4;o+=2)"margin"===n&&(l+=S.css(e,n+ae[o],!0,r)),i?("content"===n&&(l-=S.css(e,"padding"+ae[o],!0,r)),"margin"!==n&&(l-=S.css(e,"border"+ae[o]+"Width",!0,r))):(l+=S.css(e,"padding"+ae[o],!0,r),"padding"!==n?l+=S.css(e,"border"+ae[o]+"Width",!0,r):s+=S.css(e,"border"+ae[o]+"Width",!0,r));return!i&&a>=0&&(l+=Math.max(0,Math.ceil(e["offset"+t[0].toUpperCase()+t.slice(1)]-a-l-s-.5))||0),l}function rt(e,t,n){var i=Ve(e),r=(!v.boxSizingReliable()||n)&&"border-box"===S.css(e,"boxSizing",!1,i),a=r,o=qe(e,t,i),s="offset"+t[0].toUpperCase()+t.slice(1);if(Xe.test(o)){if(!n)return o;o="auto"}return(!v.boxSizingReliable()&&r||"auto"===o||!parseFloat(o)&&"inline"===S.css(e,"display",!1,i))&&e.getClientRects().length&&(r="border-box"===S.css(e,"boxSizing",!1,i),(a=s in e)&&(o=e[s])),(o=parseFloat(o)||0)+it(e,t,n||(r?"border":"content"),a,i,o)+"px"}function at(e,t,n,i,r){return new at.prototype.init(e,t,n,i,r)}S.extend({cssHooks:{opacity:{get:function(e,t){if(t){var n=qe(e,"opacity");return""===n?"1":n}}}},cssNumber:{animationIterationCount:!0,columnCount:!0,fillOpacity:!0,flexGrow:!0,flexShrink:!0,fontWeight:!0,gridArea:!0,gridColumn:!0,gridColumnEnd:!0,gridColumnStart:!0,gridRow:!0,gridRowEnd:!0,gridRowStart:!0,lineHeight:!0,opacity:!0,order:!0,orphans:!0,widows:!0,zIndex:!0,zoom:!0},cssProps:{},style:function(e,t,n,i){if(e&&3!==e.nodeType&&8!==e.nodeType&&e.style){var r,a,o,s=Q(t),l=Je.test(t),c=e.style;if(l||(t=Ze(s)),o=S.cssHooks[t]||S.cssHooks[s],void 0===n)return o&&"get"in o&&void 0!==(r=o.get(e,!1,i))?r:c[t];"string"===(a=typeof n)&&(r=re.exec(n))&&r[1]&&(n=he(e,t,r),a="number"),null!=n&&n==n&&("number"!==a||l||(n+=r&&r[3]||(S.cssNumber[s]?"":"px")),v.clearCloneStyle||""!==n||0!==t.indexOf("background")||(c[t]="inherit"),o&&"set"in o&&void 0===(n=o.set(e,n,i))||(l?c.setProperty(t,n):c[t]=n))}},css:function(e,t,n,i){var r,a,o,s=Q(t);return Je.test(t)||(t=Ze(s)),(o=S.cssHooks[t]||S.cssHooks[s])&&"get"in o&&(r=o.get(e,!0,n)),void 0===r&&(r=qe(e,t,i)),"normal"===r&&t in tt&&(r=tt[t]),""===n||n?(a=parseFloat(r),!0===n||isFinite(a)?a||0:r):r}}),S.each(["height","width"],function(e,t){S.cssHooks[t]={get:function(e,n,i){if(n)return!Ke.test(S.css(e,"display"))||e.getClientRects().length&&e.getBoundingClientRect().width?rt(e,t,i):ue(e,et,function(){return rt(e,t,i)})},set:function(e,n,i){var r,a=Ve(e),o=!v.scrollboxSize()&&"absolute"===a.position,s=(o||i)&&"border-box"===S.css(e,"boxSizing",!1,a),l=i?it(e,t,i,s,a):0;return s&&o&&(l-=Math.ceil(e["offset"+t[0].toUpperCase()+t.slice(1)]-parseFloat(a[t])-it(e,t,"border",!1,a)-.5)),l&&(r=re.exec(n))&&"px"!==(r[3]||"px")&&(e.style[t]=n,n=S.css(e,t)),nt(0,n,l)}}}),S.cssHooks.marginLeft=Ue(v.reliableMarginLeft,function(e,t){if(t)return(parseFloat(qe(e,"marginLeft"))||e.getBoundingClientRect().left-ue(e,{marginLeft:0},function(){return e.getBoundingClientRect().left}))+"px"}),S.each({margin:"",padding:"",border:"Width"},function(e,t){S.cssHooks[e+t]={expand:function(n){for(var i=0,r={},a="string"==typeof n?n.split(" "):[n];i<4;i++)r[e+ae[i]+t]=a[i]||a[i-2]||a[0];return r}},"margin"!==e&&(S.cssHooks[e+t].set=nt)}),S.fn.extend({css:function(e,t){return B(this,function(e,t,n){var i,r,a={},o=0;if(Array.isArray(t)){for(i=Ve(e),r=t.length;o<r;o++)a[t[o]]=S.css(e,t[o],!1,i);return a}return void 0!==n?S.style(e,t,n):S.css(e,t)},e,t,arguments.length>1)}}),S.Tween=at,at.prototype={constructor:at,init:function(e,t,n,i,r,a){this.elem=e,this.prop=n,this.easing=r||S.easing._default,this.options=t,this.start=this.now=this.cur(),this.end=i,this.unit=a||(S.cssNumber[n]?"":"px")},cur:function(){var e=at.propHooks[this.prop];return e&&e.get?e.get(this):at.propHooks._default.get(this)},run:function(e){var t,n=at.propHooks[this.prop];return this.options.duration?this.pos=t=S.easing[this.easing](e,this.options.duration*e,0,1,this.options.duration):this.pos=t=e,this.now=(this.end-this.start)*t+this.start,this.options.step&&this.options.step.call(this.elem,this.now,this),n&&n.set?n.set(this):at.propHooks._default.set(this),this}},at.prototype.init.prototype=at.prototype,at.propHooks={_default:{get:function(e){var t;return 1!==e.elem.nodeType||null!=e.elem[e.prop]&&null==e.elem.style[e.prop]?e.elem[e.prop]:(t=S.css(e.elem,e.prop,""))&&"auto"!==t?t:0},set:function(e){S.fx.step[e.prop]?S.fx.step[e.prop](e):1!==e.elem.nodeType||!S.cssHooks[e.prop]&&null==e.elem.style[Ze(e.prop)]?e.elem[e.prop]=e.now:S.style(e.elem,e.prop,e.now+e.unit)}}},at.propHooks.scrollTop=at.propHooks.scrollLeft={set:function(e){e.elem.nodeType&&e.elem.parentNode&&(e.elem[e.prop]=e.now)}},S.easing={linear:function(e){return e},swing:function(e){return.5-Math.cos(e*Math.PI)/2},_default:"swing"},S.fx=at.prototype.init,S.fx.step={};var ot,st,lt=/^(?:toggle|show|hide)$/,ct=/queueHooks$/;function ut(){st&&(!1===o.hidden&&n.requestAnimationFrame?n.requestAnimationFrame(ut):n.setTimeout(ut,S.fx.interval),S.fx.tick())}function ht(){return n.setTimeout(function(){ot=void 0}),ot=Date.now()}function dt(e,t){var n,i=0,r={height:e};for(t=t?1:0;i<4;i+=2-t)r["margin"+(n=ae[i])]=r["padding"+n]=e;return t&&(r.opacity=r.width=e),r}function ft(e,t,n){for(var i,r=(pt.tweeners[t]||[]).concat(pt.tweeners["*"]),a=0,o=r.length;a<o;a++)if(i=r[a].call(n,t,e))return i}function pt(e,t,n){var i,r,a=0,o=pt.prefilters.length,s=S.Deferred().always(function(){delete l.elem}),l=function(){if(r)return!1;for(var t=ot||ht(),n=Math.max(0,c.startTime+c.duration-t),i=1-(n/c.duration||0),a=0,o=c.tweens.length;a<o;a++)c.tweens[a].run(i);return s.notifyWith(e,[c,i,n]),i<1&&o?n:(o||s.notifyWith(e,[c,1,0]),s.resolveWith(e,[c]),!1)},c=s.promise({elem:e,props:S.extend({},t),opts:S.extend(!0,{specialEasing:{},easing:S.easing._default},n),originalProperties:t,originalOptions:n,startTime:ot||ht(),duration:n.duration,tweens:[],createTween:function(t,n){var i=S.Tween(e,c.opts,t,n,c.opts.specialEasing[t]||c.opts.easing);return c.tweens.push(i),i},stop:function(t){var n=0,i=t?c.tweens.length:0;if(r)return this;for(r=!0;n<i;n++)c.tweens[n].run(1);return t?(s.notifyWith(e,[c,1,0]),s.resolveWith(e,[c,t])):s.rejectWith(e,[c,t]),this}}),u=c.props;for(!function(e,t){var n,i,r,a,o;for(n in e)if(r=t[i=Q(n)],a=e[n],Array.isArray(a)&&(r=a[1],a=e[n]=a[0]),n!==i&&(e[i]=a,delete e[n]),(o=S.cssHooks[i])&&"expand"in o)for(n in a=o.expand(a),delete e[i],a)n in e||(e[n]=a[n],t[n]=r);else t[i]=r}(u,c.opts.specialEasing);a<o;a++)if(i=pt.prefilters[a].call(c,e,u,c.opts))return y(i.stop)&&(S._queueHooks(c.elem,c.opts.queue).stop=i.stop.bind(i)),i;return S.map(u,ft,c),y(c.opts.start)&&c.opts.start.call(e,c),c.progress(c.opts.progress).done(c.opts.done,c.opts.complete).fail(c.opts.fail).always(c.opts.always),S.fx.timer(S.extend(l,{elem:e,anim:c,queue:c.opts.queue})),c}S.Animation=S.extend(pt,{tweeners:{"*":[function(e,t){var n=this.createTween(e,t);return he(n.elem,e,re.exec(t),n),n}]},tweener:function(e,t){y(e)?(t=e,e=["*"]):e=e.match(j);for(var n,i=0,r=e.length;i<r;i++)n=e[i],pt.tweeners[n]=pt.tweeners[n]||[],pt.tweeners[n].unshift(t)},prefilters:[function(e,t,n){var i,r,a,o,s,l,c,u,h="width"in t||"height"in t,d=this,f={},p=e.style,g=e.nodeType&&ce(e),m=K.get(e,"fxshow");for(i in n.queue||(null==(o=S._queueHooks(e,"fx")).unqueued&&(o.unqueued=0,s=o.empty.fire,o.empty.fire=function(){o.unqueued||s()}),o.unqueued++,d.always(function(){d.always(function(){o.unqueued--,S.queue(e,"fx").length||o.empty.fire()})})),t)if(r=t[i],lt.test(r)){if(delete t[i],a=a||"toggle"===r,r===(g?"hide":"show")){if("show"!==r||!m||void 0===m[i])continue;g=!0}f[i]=m&&m[i]||S.style(e,i)}if((l=!S.isEmptyObject(t))||!S.isEmptyObject(f))for(i in h&&1===e.nodeType&&(n.overflow=[p.overflow,p.overflowX,p.overflowY],null==(c=m&&m.display)&&(c=K.get(e,"display")),"none"===(u=S.css(e,"display"))&&(c?u=c:(pe([e],!0),c=e.style.display||c,u=S.css(e,"display"),pe([e]))),("inline"===u||"inline-block"===u&&null!=c)&&"none"===S.css(e,"float")&&(l||(d.done(function(){p.display=c}),null==c&&(u=p.display,c="none"===u?"":u)),p.display="inline-block")),n.overflow&&(p.overflow="hidden",d.always(function(){p.overflow=n.overflow[0],p.overflowX=n.overflow[1],p.overflowY=n.overflow[2]})),l=!1,f)l||(m?"hidden"in m&&(g=m.hidden):m=K.access(e,"fxshow",{display:c}),a&&(m.hidden=!g),g&&pe([e],!0),d.done(function(){for(i in g||pe([e]),K.remove(e,"fxshow"),f)S.style(e,i,f[i])})),l=ft(g?m[i]:0,i,d),i in m||(m[i]=l.start,g&&(l.end=l.start,l.start=0))}],prefilter:function(e,t){t?pt.prefilters.unshift(e):pt.prefilters.push(e)}}),S.speed=function(e,t,n){var i=e&&"object"==typeof e?S.extend({},e):{complete:n||!n&&t||y(e)&&e,duration:e,easing:n&&t||t&&!y(t)&&t};return S.fx.off?i.duration=0:"number"!=typeof i.duration&&(i.duration in S.fx.speeds?i.duration=S.fx.speeds[i.duration]:i.duration=S.fx.speeds._default),null!=i.queue&&!0!==i.queue||(i.queue="fx"),i.old=i.complete,i.complete=function(){y(i.old)&&i.old.call(this),i.queue&&S.dequeue(this,i.queue)},i},S.fn.extend({fadeTo:function(e,t,n,i){return this.filter(ce).css("opacity",0).show().end().animate({opacity:t},e,n,i)},animate:function(e,t,n,i){var r=S.isEmptyObject(e),a=S.speed(t,n,i),o=function(){var t=pt(this,S.extend({},e),a);(r||K.get(this,"finish"))&&t.stop(!0)};return o.finish=o,r||!1===a.queue?this.each(o):this.queue(a.queue,o)},stop:function(e,t,n){var i=function(e){var t=e.stop;delete e.stop,t(n)};return"string"!=typeof e&&(n=t,t=e,e=void 0),t&&!1!==e&&this.queue(e||"fx",[]),this.each(function(){var t=!0,r=null!=e&&e+"queueHooks",a=S.timers,o=K.get(this);if(r)o[r]&&o[r].stop&&i(o[r]);else for(r in o)o[r]&&o[r].stop&&ct.test(r)&&i(o[r]);for(r=a.length;r--;)a[r].elem!==this||null!=e&&a[r].queue!==e||(a[r].anim.stop(n),t=!1,a.splice(r,1));!t&&n||S.dequeue(this,e)})},finish:function(e){return!1!==e&&(e=e||"fx"),this.each(function(){var t,n=K.get(this),i=n[e+"queue"],r=n[e+"queueHooks"],a=S.timers,o=i?i.length:0;for(n.finish=!0,S.queue(this,e,[]),r&&r.stop&&r.stop.call(this,!0),t=a.length;t--;)a[t].elem===this&&a[t].queue===e&&(a[t].anim.stop(!0),a.splice(t,1));for(t=0;t<o;t++)i[t]&&i[t].finish&&i[t].finish.call(this);delete n.finish})}}),S.each(["toggle","show","hide"],function(e,t){var n=S.fn[t];S.fn[t]=function(e,i,r){return null==e||"boolean"==typeof e?n.apply(this,arguments):this.animate(dt(t,!0),e,i,r)}}),S.each({slideDown:dt("show"),slideUp:dt("hide"),slideToggle:dt("toggle"),fadeIn:{opacity:"show"},fadeOut:{opacity:"hide"},fadeToggle:{opacity:"toggle"}},function(e,t){S.fn[e]=function(e,n,i){return this.animate(t,e,n,i)}}),S.timers=[],S.fx.tick=function(){var e,t=0,n=S.timers;for(ot=Date.now();t<n.length;t++)(e=n[t])()||n[t]!==e||n.splice(t--,1);n.length||S.fx.stop(),ot=void 0},S.fx.timer=function(e){S.timers.push(e),S.fx.start()},S.fx.interval=13,S.fx.start=function(){st||(st=!0,ut())},S.fx.stop=function(){st=null},S.fx.speeds={slow:600,fast:200,_default:400},S.fn.delay=function(e,t){return e=S.fx&&S.fx.speeds[e]||e,t=t||"fx",this.queue(t,function(t,i){var r=n.setTimeout(t,e);i.stop=function(){n.clearTimeout(r)}})},function(){var e=o.createElement("input"),t=o.createElement("select").appendChild(o.createElement("option"));e.type="checkbox",v.checkOn=""!==e.value,v.optSelected=t.selected,(e=o.createElement("input")).value="t",e.type="radio",v.radioValue="t"===e.value}();var gt,mt=S.expr.attrHandle;S.fn.extend({attr:function(e,t){return B(this,S.attr,e,t,arguments.length>1)},removeAttr:function(e){return this.each(function(){S.removeAttr(this,e)})}}),S.extend({attr:function(e,t,n){var i,r,a=e.nodeType;if(3!==a&&8!==a&&2!==a)return void 0===e.getAttribute?S.prop(e,t,n):(1===a&&S.isXMLDoc(e)||(r=S.attrHooks[t.toLowerCase()]||(S.expr.match.bool.test(t)?gt:void 0)),void 0!==n?null===n?void S.removeAttr(e,t):r&&"set"in r&&void 0!==(i=r.set(e,n,t))?i:(e.setAttribute(t,n+""),n):r&&"get"in r&&null!==(i=r.get(e,t))?i:null==(i=S.find.attr(e,t))?void 0:i)},attrHooks:{type:{set:function(e,t){if(!v.radioValue&&"radio"===t&&P(e,"input")){var n=e.value;return e.setAttribute("type",t),n&&(e.value=n),t}}}},removeAttr:function(e,t){var n,i=0,r=t&&t.match(j);if(r&&1===e.nodeType)for(;n=r[i++];)e.removeAttribute(n)}}),gt={set:function(e,t,n){return!1===t?S.removeAttr(e,n):e.setAttribute(n,n),n}},S.each(S.expr.match.bool.source.match(/\w+/g),function(e,t){var n=mt[t]||S.find.attr;mt[t]=function(e,t,i){var r,a,o=t.toLowerCase();return i||(a=mt[o],mt[o]=r,r=null!=n(e,t,i)?o:null,mt[o]=a),r}});var vt=/^(?:input|select|textarea|button)$/i,yt=/^(?:a|area)$/i;function bt(e){return(e.match(j)||[]).join(" ")}function xt(e){return e.getAttribute&&e.getAttribute("class")||""}function wt(e){return Array.isArray(e)?e:"string"==typeof e&&e.match(j)||[]}S.fn.extend({prop:function(e,t){return B(this,S.prop,e,t,arguments.length>1)},removeProp:function(e){return this.each(function(){delete this[S.propFix[e]||e]})}}),S.extend({prop:function(e,t,n){var i,r,a=e.nodeType;if(3!==a&&8!==a&&2!==a)return 1===a&&S.isXMLDoc(e)||(t=S.propFix[t]||t,r=S.propHooks[t]),void 0!==n?r&&"set"in r&&void 0!==(i=r.set(e,n,t))?i:e[t]=n:r&&"get"in r&&null!==(i=r.get(e,t))?i:e[t]},propHooks:{tabIndex:{get:function(e){var t=S.find.attr(e,"tabindex");return t?parseInt(t,10):vt.test(e.nodeName)||yt.test(e.nodeName)&&e.href?0:-1}}},propFix:{for:"htmlFor",class:"className"}}),v.optSelected||(S.propHooks.selected={get:function(e){var t=e.parentNode;return t&&t.parentNode&&t.parentNode.selectedIndex,null},set:function(e){var t=e.parentNode;t&&(t.selectedIndex,t.parentNode&&t.parentNode.selectedIndex)}}),S.each(["tabIndex","readOnly","maxLength","cellSpacing","cellPadding","rowSpan","colSpan","useMap","frameBorder","contentEditable"],function(){S.propFix[this.toLowerCase()]=this}),S.fn.extend({addClass:function(e){var t,n,i,r,a,o,s,l=0;if(y(e))return this.each(function(t){S(this).addClass(e.call(this,t,xt(this)))});if((t=wt(e)).length)for(;n=this[l++];)if(r=xt(n),i=1===n.nodeType&&" "+bt(r)+" "){for(o=0;a=t[o++];)i.indexOf(" "+a+" ")<0&&(i+=a+" ");r!==(s=bt(i))&&n.setAttribute("class",s)}return this},removeClass:function(e){var t,n,i,r,a,o,s,l=0;if(y(e))return this.each(function(t){S(this).removeClass(e.call(this,t,xt(this)))});if(!arguments.length)return this.attr("class","");if((t=wt(e)).length)for(;n=this[l++];)if(r=xt(n),i=1===n.nodeType&&" "+bt(r)+" "){for(o=0;a=t[o++];)for(;i.indexOf(" "+a+" ")>-1;)i=i.replace(" "+a+" "," ");r!==(s=bt(i))&&n.setAttribute("class",s)}return this},toggleClass:function(e,t){var n=typeof e,i="string"===n||Array.isArray(e);return"boolean"==typeof t&&i?t?this.addClass(e):this.removeClass(e):y(e)?this.each(function(n){S(this).toggleClass(e.call(this,n,xt(this),t),t)}):this.each(function(){var t,r,a,o;if(i)for(r=0,a=S(this),o=wt(e);t=o[r++];)a.hasClass(t)?a.removeClass(t):a.addClass(t);else void 0!==e&&"boolean"!==n||((t=xt(this))&&K.set(this,"__className__",t),this.setAttribute&&this.setAttribute("class",t||!1===e?"":K.get(this,"__className__")||""))})},hasClass:function(e){var t,n,i=0;for(t=" "+e+" ";n=this[i++];)if(1===n.nodeType&&(" "+bt(xt(n))+" ").indexOf(t)>-1)return!0;return!1}});var kt=/\r/g;S.fn.extend({val:function(e){var t,n,i,r=this[0];return arguments.length?(i=y(e),this.each(function(n){var r;1===this.nodeType&&(null==(r=i?e.call(this,n,S(this).val()):e)?r="":"number"==typeof r?r+="":Array.isArray(r)&&(r=S.map(r,function(e){return null==e?"":e+""})),(t=S.valHooks[this.type]||S.valHooks[this.nodeName.toLowerCase()])&&"set"in t&&void 0!==t.set(this,r,"value")||(this.value=r))})):r?(t=S.valHooks[r.type]||S.valHooks[r.nodeName.toLowerCase()])&&"get"in t&&void 0!==(n=t.get(r,"value"))?n:"string"==typeof(n=r.value)?n.replace(kt,""):null==n?"":n:void 0}}),S.extend({valHooks:{option:{get:function(e){var t=S.find.attr(e,"value");return null!=t?t:bt(S.text(e))}},select:{get:function(e){var t,n,i,r=e.options,a=e.selectedIndex,o="select-one"===e.type,s=o?null:[],l=o?a+1:r.length;for(i=a<0?l:o?a:0;i<l;i++)if(((n=r[i]).selected||i===a)&&!n.disabled&&(!n.parentNode.disabled||!P(n.parentNode,"optgroup"))){if(t=S(n).val(),o)return t;s.push(t)}return s},set:function(e,t){for(var n,i,r=e.options,a=S.makeArray(t),o=r.length;o--;)((i=r[o]).selected=S.inArray(S.valHooks.option.get(i),a)>-1)&&(n=!0);return n||(e.selectedIndex=-1),a}}}}),S.each(["radio","checkbox"],function(){S.valHooks[this]={set:function(e,t){if(Array.isArray(t))return e.checked=S.inArray(S(e).val(),t)>-1}},v.checkOn||(S.valHooks[this].get=function(e){return null===e.getAttribute("value")?"on":e.value})}),v.focusin="onfocusin"in n;var St=/^(?:focusinfocus|focusoutblur)$/,Et=function(e){e.stopPropagation()};S.extend(S.event,{trigger:function(e,t,i,r){var a,s,l,c,u,h,d,f,g=[i||o],m=p.call(e,"type")?e.type:e,v=p.call(e,"namespace")?e.namespace.split("."):[];if(s=f=l=i=i||o,3!==i.nodeType&&8!==i.nodeType&&!St.test(m+S.event.triggered)&&(m.indexOf(".")>-1&&(v=m.split("."),m=v.shift(),v.sort()),u=m.indexOf(":")<0&&"on"+m,(e=e[S.expando]?e:new S.Event(m,"object"==typeof e&&e)).isTrigger=r?2:3,e.namespace=v.join("."),e.rnamespace=e.namespace?new RegExp("(^|\\.)"+v.join("\\.(?:.*\\.|)")+"(\\.|$)"):null,e.result=void 0,e.target||(e.target=i),t=null==t?[e]:S.makeArray(t,[e]),d=S.event.special[m]||{},r||!d.trigger||!1!==d.trigger.apply(i,t))){if(!r&&!d.noBubble&&!b(i)){for(c=d.delegateType||m,St.test(c+m)||(s=s.parentNode);s;s=s.parentNode)g.push(s),l=s;l===(i.ownerDocument||o)&&g.push(l.defaultView||l.parentWindow||n)}for(a=0;(s=g[a++])&&!e.isPropagationStopped();)f=s,e.type=a>1?c:d.bindType||m,(h=(K.get(s,"events")||{})[e.type]&&K.get(s,"handle"))&&h.apply(s,t),(h=u&&s[u])&&h.apply&&$(s)&&(e.result=h.apply(s,t),!1===e.result&&e.preventDefault());return e.type=m,r||e.isDefaultPrevented()||d._default&&!1!==d._default.apply(g.pop(),t)||!$(i)||u&&y(i[m])&&!b(i)&&((l=i[u])&&(i[u]=null),S.event.triggered=m,e.isPropagationStopped()&&f.addEventListener(m,Et),i[m](),e.isPropagationStopped()&&f.removeEventListener(m,Et),S.event.triggered=void 0,l&&(i[u]=l)),e.result}},simulate:function(e,t,n){var i=S.extend(new S.Event,n,{type:e,isSimulated:!0});S.event.trigger(i,null,t)}}),S.fn.extend({trigger:function(e,t){return this.each(function(){S.event.trigger(e,t,this)})},triggerHandler:function(e,t){var n=this[0];if(n)return S.event.trigger(e,t,n,!0)}}),v.focusin||S.each({focus:"focusin",blur:"focusout"},function(e,t){var n=function(e){S.event.simulate(t,e.target,S.event.fix(e))};S.event.special[t]={setup:function(){var i=this.ownerDocument||this,r=K.access(i,t);r||i.addEventListener(e,n,!0),K.access(i,t,(r||0)+1)},teardown:function(){var i=this.ownerDocument||this,r=K.access(i,t)-1;r?K.access(i,t,r):(i.removeEventListener(e,n,!0),K.remove(i,t))}}});var Ct=n.location,Tt=Date.now(),At=/\?/;S.parseXML=function(e){var t;if(!e||"string"!=typeof e)return null;try{t=(new n.DOMParser).parseFromString(e,"text/xml")}catch(e){t=void 0}return t&&!t.getElementsByTagName("parsererror").length||S.error("Invalid XML: "+e),t};var _t=/\[\]$/,Ot=/\r?\n/g,Pt=/^(?:submit|button|image|reset|file)$/i,Mt=/^(?:input|select|textarea|keygen)/i;function It(e,t,n,i){var r;if(Array.isArray(t))S.each(t,function(t,r){n||_t.test(e)?i(e,r):It(e+"["+("object"==typeof r&&null!=r?t:"")+"]",r,n,i)});else if(n||"object"!==k(t))i(e,t);else for(r in t)It(e+"["+r+"]",t[r],n,i)}S.param=function(e,t){var n,i=[],r=function(e,t){var n=y(t)?t():t;i[i.length]=encodeURIComponent(e)+"="+encodeURIComponent(null==n?"":n)};if(null==e)return"";if(Array.isArray(e)||e.jquery&&!S.isPlainObject(e))S.each(e,function(){r(this.name,this.value)});else for(n in e)It(n,e[n],t,r);return i.join("&")},S.fn.extend({serialize:function(){return S.param(this.serializeArray())},serializeArray:function(){return this.map(function(){var e=S.prop(this,"elements");return e?S.makeArray(e):this}).filter(function(){var e=this.type;return this.name&&!S(this).is(":disabled")&&Mt.test(this.nodeName)&&!Pt.test(e)&&(this.checked||!ge.test(e))}).map(function(e,t){var n=S(this).val();return null==n?null:Array.isArray(n)?S.map(n,function(e){return{name:t.name,value:e.replace(Ot,"\r\n")}}):{name:t.name,value:n.replace(Ot,"\r\n")}}).get()}});var Dt=/%20/g,Nt=/#.*$/,Lt=/([?&])_=[^&]*/,Rt=/^(.*?):[ \t]*([^\r\n]*)$/gm,Ft=/^(?:GET|HEAD)$/,jt=/^\/\//,zt={},Yt={},Ht="*/".concat("*"),Wt=o.createElement("a");function Xt(e){return function(t,n){"string"!=typeof t&&(n=t,t="*");var i,r=0,a=t.toLowerCase().match(j)||[];if(y(n))for(;i=a[r++];)"+"===i[0]?(i=i.slice(1)||"*",(e[i]=e[i]||[]).unshift(n)):(e[i]=e[i]||[]).push(n)}}function Vt(e,t,n,i){var r={},a=e===Yt;function o(s){var l;return r[s]=!0,S.each(e[s]||[],function(e,s){var c=s(t,n,i);return"string"!=typeof c||a||r[c]?a?!(l=c):void 0:(t.dataTypes.unshift(c),o(c),!1)}),l}return o(t.dataTypes[0])||!r["*"]&&o("*")}function Bt(e,t){var n,i,r=S.ajaxSettings.flatOptions||{};for(n in t)void 0!==t[n]&&((r[n]?e:i||(i={}))[n]=t[n]);return i&&S.extend(!0,e,i),e}Wt.href=Ct.href,S.extend({active:0,lastModified:{},etag:{},ajaxSettings:{url:Ct.href,type:"GET",isLocal:/^(?:about|app|app-storage|.+-extension|file|res|widget):$/.test(Ct.protocol),global:!0,processData:!0,async:!0,contentType:"application/x-www-form-urlencoded; charset=UTF-8",accepts:{"*":Ht,text:"text/plain",html:"text/html",xml:"application/xml, text/xml",json:"application/json, text/javascript"},contents:{xml:/\bxml\b/,html:/\bhtml/,json:/\bjson\b/},responseFields:{xml:"responseXML",text:"responseText",json:"responseJSON"},converters:{"* text":String,"text html":!0,"text json":JSON.parse,"text xml":S.parseXML},flatOptions:{url:!0,context:!0}},ajaxSetup:function(e,t){return t?Bt(Bt(e,S.ajaxSettings),t):Bt(S.ajaxSettings,e)},ajaxPrefilter:Xt(zt),ajaxTransport:Xt(Yt),ajax:function(e,t){"object"==typeof e&&(t=e,e=void 0),t=t||{};var i,r,a,s,l,c,u,h,d,f,p=S.ajaxSetup({},t),g=p.context||p,m=p.context&&(g.nodeType||g.jquery)?S(g):S.event,v=S.Deferred(),y=S.Callbacks("once memory"),b=p.statusCode||{},x={},w={},k="canceled",E={readyState:0,getResponseHeader:function(e){var t;if(u){if(!s)for(s={};t=Rt.exec(a);)s[t[1].toLowerCase()+" "]=(s[t[1].toLowerCase()+" "]||[]).concat(t[2]);t=s[e.toLowerCase()+" "]}return null==t?null:t.join(", ")},getAllResponseHeaders:function(){return u?a:null},setRequestHeader:function(e,t){return null==u&&(e=w[e.toLowerCase()]=w[e.toLowerCase()]||e,x[e]=t),this},overrideMimeType:function(e){return null==u&&(p.mimeType=e),this},statusCode:function(e){var t;if(e)if(u)E.always(e[E.status]);else for(t in e)b[t]=[b[t],e[t]];return this},abort:function(e){var t=e||k;return i&&i.abort(t),C(0,t),this}};if(v.promise(E),p.url=((e||p.url||Ct.href)+"").replace(jt,Ct.protocol+"//"),p.type=t.method||t.type||p.method||p.type,p.dataTypes=(p.dataType||"*").toLowerCase().match(j)||[""],null==p.crossDomain){c=o.createElement("a");try{c.href=p.url,c.href=c.href,p.crossDomain=Wt.protocol+"//"+Wt.host!=c.protocol+"//"+c.host}catch(e){p.crossDomain=!0}}if(p.data&&p.processData&&"string"!=typeof p.data&&(p.data=S.param(p.data,p.traditional)),Vt(zt,p,t,E),u)return E;for(d in(h=S.event&&p.global)&&0==S.active++&&S.event.trigger("ajaxStart"),p.type=p.type.toUpperCase(),p.hasContent=!Ft.test(p.type),r=p.url.replace(Nt,""),p.hasContent?p.data&&p.processData&&0===(p.contentType||"").indexOf("application/x-www-form-urlencoded")&&(p.data=p.data.replace(Dt,"+")):(f=p.url.slice(r.length),p.data&&(p.processData||"string"==typeof p.data)&&(r+=(At.test(r)?"&":"?")+p.data,delete p.data),!1===p.cache&&(r=r.replace(Lt,"$1"),f=(At.test(r)?"&":"?")+"_="+Tt+++f),p.url=r+f),p.ifModified&&(S.lastModified[r]&&E.setRequestHeader("If-Modified-Since",S.lastModified[r]),S.etag[r]&&E.setRequestHeader("If-None-Match",S.etag[r])),(p.data&&p.hasContent&&!1!==p.contentType||t.contentType)&&E.setRequestHeader("Content-Type",p.contentType),E.setRequestHeader("Accept",p.dataTypes[0]&&p.accepts[p.dataTypes[0]]?p.accepts[p.dataTypes[0]]+("*"!==p.dataTypes[0]?", "+Ht+"; q=0.01":""):p.accepts["*"]),p.headers)E.setRequestHeader(d,p.headers[d]);if(p.beforeSend&&(!1===p.beforeSend.call(g,E,p)||u))return E.abort();if(k="abort",y.add(p.complete),E.done(p.success),E.fail(p.error),i=Vt(Yt,p,t,E)){if(E.readyState=1,h&&m.trigger("ajaxSend",[E,p]),u)return E;p.async&&p.timeout>0&&(l=n.setTimeout(function(){E.abort("timeout")},p.timeout));try{u=!1,i.send(x,C)}catch(e){if(u)throw e;C(-1,e)}}else C(-1,"No Transport");function C(e,t,o,s){var c,d,f,x,w,k=t;u||(u=!0,l&&n.clearTimeout(l),i=void 0,a=s||"",E.readyState=e>0?4:0,c=e>=200&&e<300||304===e,o&&(x=function(e,t,n){for(var i,r,a,o,s=e.contents,l=e.dataTypes;"*"===l[0];)l.shift(),void 0===i&&(i=e.mimeType||t.getResponseHeader("Content-Type"));if(i)for(r in s)if(s[r]&&s[r].test(i)){l.unshift(r);break}if(l[0]in n)a=l[0];else{for(r in n){if(!l[0]||e.converters[r+" "+l[0]]){a=r;break}o||(o=r)}a=a||o}if(a)return a!==l[0]&&l.unshift(a),n[a]}(p,E,o)),x=function(e,t,n,i){var r,a,o,s,l,c={},u=e.dataTypes.slice();if(u[1])for(o in e.converters)c[o.toLowerCase()]=e.converters[o];for(a=u.shift();a;)if(e.responseFields[a]&&(n[e.responseFields[a]]=t),!l&&i&&e.dataFilter&&(t=e.dataFilter(t,e.dataType)),l=a,a=u.shift())if("*"===a)a=l;else if("*"!==l&&l!==a){if(!(o=c[l+" "+a]||c["* "+a]))for(r in c)if((s=r.split(" "))[1]===a&&(o=c[l+" "+s[0]]||c["* "+s[0]])){!0===o?o=c[r]:!0!==c[r]&&(a=s[0],u.unshift(s[1]));break}if(!0!==o)if(o&&e.throws)t=o(t);else try{t=o(t)}catch(e){return{state:"parsererror",error:o?e:"No conversion from "+l+" to "+a}}}return{state:"success",data:t}}(p,x,E,c),c?(p.ifModified&&((w=E.getResponseHeader("Last-Modified"))&&(S.lastModified[r]=w),(w=E.getResponseHeader("etag"))&&(S.etag[r]=w)),204===e||"HEAD"===p.type?k="nocontent":304===e?k="notmodified":(k=x.state,d=x.data,c=!(f=x.error))):(f=k,!e&&k||(k="error",e<0&&(e=0))),E.status=e,E.statusText=(t||k)+"",c?v.resolveWith(g,[d,k,E]):v.rejectWith(g,[E,k,f]),E.statusCode(b),b=void 0,h&&m.trigger(c?"ajaxSuccess":"ajaxError",[E,p,c?d:f]),y.fireWith(g,[E,k]),h&&(m.trigger("ajaxComplete",[E,p]),--S.active||S.event.trigger("ajaxStop")))}return E},getJSON:function(e,t,n){return S.get(e,t,n,"json")},getScript:function(e,t){return S.get(e,void 0,t,"script")}}),S.each(["get","post"],function(e,t){S[t]=function(e,n,i,r){return y(n)&&(r=r||i,i=n,n=void 0),S.ajax(S.extend({url:e,type:t,dataType:r,data:n,success:i},S.isPlainObject(e)&&e))}}),S._evalUrl=function(e,t){return S.ajax({url:e,type:"GET",dataType:"script",cache:!0,async:!1,global:!1,converters:{"text script":function(){}},dataFilter:function(e){S.globalEval(e,t)}})},S.fn.extend({wrapAll:function(e){var t;return this[0]&&(y(e)&&(e=e.call(this[0])),t=S(e,this[0].ownerDocument).eq(0).clone(!0),this[0].parentNode&&t.insertBefore(this[0]),t.map(function(){for(var e=this;e.firstElementChild;)e=e.firstElementChild;return e}).append(this)),this},wrapInner:function(e){return y(e)?this.each(function(t){S(this).wrapInner(e.call(this,t))}):this.each(function(){var t=S(this),n=t.contents();n.length?n.wrapAll(e):t.append(e)})},wrap:function(e){var t=y(e);return this.each(function(n){S(this).wrapAll(t?e.call(this,n):e)})},unwrap:function(e){return this.parent(e).not("body").each(function(){S(this).replaceWith(this.childNodes)}),this}}),S.expr.pseudos.hidden=function(e){return!S.expr.pseudos.visible(e)},S.expr.pseudos.visible=function(e){return!!(e.offsetWidth||e.offsetHeight||e.getClientRects().length)},S.ajaxSettings.xhr=function(){try{return new n.XMLHttpRequest}catch(e){}};var qt={0:200,1223:204},Ut=S.ajaxSettings.xhr();v.cors=!!Ut&&"withCredentials"in Ut,v.ajax=Ut=!!Ut,S.ajaxTransport(function(e){var t,i;if(v.cors||Ut&&!e.crossDomain)return{send:function(r,a){var o,s=e.xhr();if(s.open(e.type,e.url,e.async,e.username,e.password),e.xhrFields)for(o in e.xhrFields)s[o]=e.xhrFields[o];for(o in e.mimeType&&s.overrideMimeType&&s.overrideMimeType(e.mimeType),e.crossDomain||r["X-Requested-With"]||(r["X-Requested-With"]="XMLHttpRequest"),r)s.setRequestHeader(o,r[o]);t=function(e){return function(){t&&(t=i=s.onload=s.onerror=s.onabort=s.ontimeout=s.onreadystatechange=null,"abort"===e?s.abort():"error"===e?"number"!=typeof s.status?a(0,"error"):a(s.status,s.statusText):a(qt[s.status]||s.status,s.statusText,"text"!==(s.responseType||"text")||"string"!=typeof s.responseText?{binary:s.response}:{text:s.responseText},s.getAllResponseHeaders()))}},s.onload=t(),i=s.onerror=s.ontimeout=t("error"),void 0!==s.onabort?s.onabort=i:s.onreadystatechange=function(){4===s.readyState&&n.setTimeout(function(){t&&i()})},t=t("abort");try{s.send(e.hasContent&&e.data||null)}catch(e){if(t)throw e}},abort:function(){t&&t()}}}),S.ajaxPrefilter(function(e){e.crossDomain&&(e.contents.script=!1)}),S.ajaxSetup({accepts:{script:"text/javascript, application/javascript, application/ecmascript, application/x-ecmascript"},contents:{script:/\b(?:java|ecma)script\b/},converters:{"text script":function(e){return S.globalEval(e),e}}}),S.ajaxPrefilter("script",function(e){void 0===e.cache&&(e.cache=!1),e.crossDomain&&(e.type="GET")}),S.ajaxTransport("script",function(e){var t,n;if(e.crossDomain||e.scriptAttrs)return{send:function(i,r){t=S("<script>").attr(e.scriptAttrs||{}).prop({charset:e.scriptCharset,src:e.url}).on("load error",n=function(e){t.remove(),n=null,e&&r("error"===e.type?404:200,e.type)}),o.head.appendChild(t[0])},abort:function(){n&&n()}}});var Gt,Qt=[],$t=/(=)\?(?=&|$)|\?\?/;S.ajaxSetup({jsonp:"callback",jsonpCallback:function(){var e=Qt.pop()||S.expando+"_"+Tt++;return this[e]=!0,e}}),S.ajaxPrefilter("json jsonp",function(e,t,i){var r,a,o,s=!1!==e.jsonp&&($t.test(e.url)?"url":"string"==typeof e.data&&0===(e.contentType||"").indexOf("application/x-www-form-urlencoded")&&$t.test(e.data)&&"data");if(s||"jsonp"===e.dataTypes[0])return r=e.jsonpCallback=y(e.jsonpCallback)?e.jsonpCallback():e.jsonpCallback,s?e[s]=e[s].replace($t,"$1"+r):!1!==e.jsonp&&(e.url+=(At.test(e.url)?"&":"?")+e.jsonp+"="+r),e.converters["script json"]=function(){return o||S.error(r+" was not called"),o[0]},e.dataTypes[0]="json",a=n[r],n[r]=function(){o=arguments},i.always(function(){void 0===a?S(n).removeProp(r):n[r]=a,e[r]&&(e.jsonpCallback=t.jsonpCallback,Qt.push(r)),o&&y(a)&&a(o[0]),o=a=void 0}),"script"}),v.createHTMLDocument=((Gt=o.implementation.createHTMLDocument("").body).innerHTML="<form></form><form></form>",2===Gt.childNodes.length),S.parseHTML=function(e,t,n){return"string"!=typeof e?[]:("boolean"==typeof t&&(n=t,t=!1),t||(v.createHTMLDocument?((i=(t=o.implementation.createHTMLDocument("")).createElement("base")).href=o.location.href,t.head.appendChild(i)):t=o),a=!n&&[],(r=M.exec(e))?[t.createElement(r[1])]:(r=Ee([e],t,a),a&&a.length&&S(a).remove(),S.merge([],r.childNodes)));var i,r,a},S.fn.load=function(e,t,n){var i,r,a,o=this,s=e.indexOf(" ");return s>-1&&(i=bt(e.slice(s)),e=e.slice(0,s)),y(t)?(n=t,t=void 0):t&&"object"==typeof t&&(r="POST"),o.length>0&&S.ajax({url:e,type:r||"GET",dataType:"html",data:t}).done(function(e){a=arguments,o.html(i?S("<div>").append(S.parseHTML(e)).find(i):e)}).always(n&&function(e,t){o.each(function(){n.apply(this,a||[e.responseText,t,e])})}),this},S.each(["ajaxStart","ajaxStop","ajaxComplete","ajaxError","ajaxSuccess","ajaxSend"],function(e,t){S.fn[t]=function(e){return this.on(t,e)}}),S.expr.pseudos.animated=function(e){return S.grep(S.timers,function(t){return e===t.elem}).length},S.offset={setOffset:function(e,t,n){var i,r,a,o,s,l,c=S.css(e,"position"),u=S(e),h={};"static"===c&&(e.style.position="relative"),s=u.offset(),a=S.css(e,"top"),l=S.css(e,"left"),("absolute"===c||"fixed"===c)&&(a+l).indexOf("auto")>-1?(o=(i=u.position()).top,r=i.left):(o=parseFloat(a)||0,r=parseFloat(l)||0),y(t)&&(t=t.call(e,n,S.extend({},s))),null!=t.top&&(h.top=t.top-s.top+o),null!=t.left&&(h.left=t.left-s.left+r),"using"in t?t.using.call(e,h):u.css(h)}},S.fn.extend({offset:function(e){if(arguments.length)return void 0===e?this:this.each(function(t){S.offset.setOffset(this,e,t)});var t,n,i=this[0];return i?i.getClientRects().length?(t=i.getBoundingClientRect(),n=i.ownerDocument.defaultView,{top:t.top+n.pageYOffset,left:t.left+n.pageXOffset}):{top:0,left:0}:void 0},position:function(){if(this[0]){var e,t,n,i=this[0],r={top:0,left:0};if("fixed"===S.css(i,"position"))t=i.getBoundingClientRect();else{for(t=this.offset(),n=i.ownerDocument,e=i.offsetParent||n.documentElement;e&&(e===n.body||e===n.documentElement)&&"static"===S.css(e,"position");)e=e.parentNode;e&&e!==i&&1===e.nodeType&&((r=S(e).offset()).top+=S.css(e,"borderTopWidth",!0),r.left+=S.css(e,"borderLeftWidth",!0))}return{top:t.top-r.top-S.css(i,"marginTop",!0),left:t.left-r.left-S.css(i,"marginLeft",!0)}}},offsetParent:function(){return this.map(function(){for(var e=this.offsetParent;e&&"static"===S.css(e,"position");)e=e.offsetParent;return e||oe})}}),S.each({scrollLeft:"pageXOffset",scrollTop:"pageYOffset"},function(e,t){var n="pageYOffset"===t;S.fn[e]=function(i){return B(this,function(e,i,r){var a;if(b(e)?a=e:9===e.nodeType&&(a=e.defaultView),void 0===r)return a?a[t]:e[i];a?a.scrollTo(n?a.pageXOffset:r,n?r:a.pageYOffset):e[i]=r},e,i,arguments.length)}}),S.each(["top","left"],function(e,t){S.cssHooks[t]=Ue(v.pixelPosition,function(e,n){if(n)return n=qe(e,t),Xe.test(n)?S(e).position()[t]+"px":n})}),S.each({Height:"height",Width:"width"},function(e,t){S.each({padding:"inner"+e,content:t,"":"outer"+e},function(n,i){S.fn[i]=function(r,a){var o=arguments.length&&(n||"boolean"!=typeof r),s=n||(!0===r||!0===a?"margin":"border");return B(this,function(t,n,r){var a;return b(t)?0===i.indexOf("outer")?t["inner"+e]:t.document.documentElement["client"+e]:9===t.nodeType?(a=t.documentElement,Math.max(t.body["scroll"+e],a["scroll"+e],t.body["offset"+e],a["offset"+e],a["client"+e])):void 0===r?S.css(t,n,s):S.style(t,n,r,s)},t,o?r:void 0,o)}})}),S.each("blur focus focusin focusout resize scroll click dblclick mousedown mouseup mousemove mouseover mouseout mouseenter mouseleave change select submit keydown keypress keyup contextmenu".split(" "),function(e,t){S.fn[t]=function(e,n){return arguments.length>0?this.on(t,null,e,n):this.trigger(t)}}),S.fn.extend({hover:function(e,t){return this.mouseenter(e).mouseleave(t||e)}}),S.fn.extend({bind:function(e,t,n){return this.on(e,null,t,n)},unbind:function(e,t){return this.off(e,null,t)},delegate:function(e,t,n,i){return this.on(t,e,n,i)},undelegate:function(e,t,n){return 1===arguments.length?this.off(e,"**"):this.off(t,e||"**",n)}}),S.proxy=function(e,t){var n,i,r;if("string"==typeof t&&(n=e[t],t=e,e=n),y(e))return i=l.call(arguments,2),(r=function(){return e.apply(t||this,i.concat(l.call(arguments)))}).guid=e.guid=e.guid||S.guid++,r},S.holdReady=function(e){e?S.readyWait++:S.ready(!0)},S.isArray=Array.isArray,S.parseJSON=JSON.parse,S.nodeName=P,S.isFunction=y,S.isWindow=b,S.camelCase=Q,S.type=k,S.now=Date.now,S.isNumeric=function(e){var t=S.type(e);return("number"===t||"string"===t)&&!isNaN(e-parseFloat(e))},void 0===(i=function(){return S}.apply(t,[]))||(e.exports=i);var Zt=n.jQuery,Kt=n.$;return S.noConflict=function(e){return n.$===S&&(n.$=Kt),e&&n.jQuery===S&&(n.jQuery=Zt),S},r||(n.jQuery=n.$=S),S})},function(e,t,n){"use strict";!function e(){if("undefined"!=typeof __REACT_DEVTOOLS_GLOBAL_HOOK__&&"function"==typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE)try{__REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE(e)}catch(e){console.error(e)}}(),e.exports=n(81)},function(e,t,n){var i=n(30)("wks"),r=n(32),a=n(11).Symbol,o="function"==typeof a;(e.exports=function(e){return i[e]||(i[e]=o&&a[e]||(o?a:r)("Symbol."+e))}).store=i},function(e,t){var n=e.exports="undefined"!=typeof window&&window.Math==Math?window:"undefined"!=typeof self&&self.Math==Math?self:Function("return this")();"number"==typeof __g&&(__g=n)},function(e,t,n){var i=n(33),r=n(54);e.exports=n(22)?function(e,t,n){return i.f(e,t,r(1,n))}:function(e,t,n){return e[t]=n,e}},function(e,t,n){var i=n(28);e.exports=function(e){if(!i(e))throw TypeError(e+" is not an object!");return e}},function(e,t){var n;n=function(){return this}();try{n=n||new Function("return this")()}catch(e){"object"==typeof window&&(n=window)}e.exports=n},function(e,t){var n,i,r=e.exports={};function a(){throw new Error("setTimeout has not been defined")}function o(){throw new Error("clearTimeout has not been defined")}function s(e){if(n===setTimeout)return setTimeout(e,0);if((n===a||!n)&&setTimeout)return n=setTimeout,setTimeout(e,0);try{return n(e,0)}catch(t){try{return n.call(null,e,0)}catch(t){return n.call(this,e,0)}}}!function(){try{n="function"==typeof setTimeout?setTimeout:a}catch(e){n=a}try{i="function"==typeof clearTimeout?clearTimeout:o}catch(e){i=o}}();var l,c=[],u=!1,h=-1;function d(){u&&l&&(u=!1,l.length?c=l.concat(c):h=-1,c.length&&f())}function f(){if(!u){var e=s(d);u=!0;for(var t=c.length;t;){for(l=c,c=[];++h<t;)l&&l[h].run();h=-1,t=c.length}l=null,u=!1,function(e){if(i===clearTimeout)return clearTimeout(e);if((i===o||!i)&&clearTimeout)return i=clearTimeout,clearTimeout(e);try{i(e)}catch(t){try{return i.call(null,e)}catch(t){return i.call(this,e)}}}(e)}}function p(e,t){this.fun=e,this.array=t}function g(){}r.nextTick=function(e){var t=new Array(arguments.length-1);if(arguments.length>1)for(var n=1;n<arguments.length;n++)t[n-1]=arguments[n];c.push(new p(e,t)),1!==c.length||u||s(f)},p.prototype.run=function(){this.fun.apply(null,this.array)},r.title="browser",r.browser=!0,r.env={},r.argv=[],r.version="",r.versions={},r.on=g,r.addListener=g,r.once=g,r.off=g,r.removeListener=g,r.removeAllListeners=g,r.emit=g,r.prependListener=g,r.prependOnceListener=g,r.listeners=function(e){return[]},r.binding=function(e){throw new Error("process.binding is not supported")},r.cwd=function(){return"/"},r.chdir=function(e){throw new Error("process.chdir is not supported")},r.umask=function(){return 0}},function(e,t,n){for(var i=n(97),r=n(58),a=n(23),o=n(11),s=n(12),l=n(38),c=n(10),u=c("iterator"),h=c("toStringTag"),d=l.Array,f={CSSRuleList:!0,CSSStyleDeclaration:!1,CSSValueList:!1,ClientRectList:!1,DOMRectList:!1,DOMStringList:!1,DOMTokenList:!0,DataTransferItemList:!1,FileList:!1,HTMLAllCollection:!1,HTMLCollection:!1,HTMLFormElement:!1,HTMLSelectElement:!1,MediaList:!0,MimeTypeArray:!1,NamedNodeMap:!1,NodeList:!0,PaintRequestList:!1,Plugin:!1,PluginArray:!1,SVGLengthList:!1,SVGNumberList:!1,SVGPathSegList:!1,SVGPointList:!1,SVGStringList:!1,SVGTransformList:!1,SourceBufferList:!1,StyleSheetList:!0,TextTrackCueList:!1,TextTrackList:!1,TouchList:!1},p=r(f),g=0;g<p.length;g++){var m,v=p[g],y=f[v],b=o[v],x=b&&b.prototype;if(x&&(x[u]||s(x,u,d),x[h]||s(x,h,v),l[v]=d,y))for(m in i)x[m]||a(x,m,i[m],!0)}},function(e,t,n){"use strict";var i="function"==typeof Symbol?Symbol.for("nodejs.util.inspect.custom"):void 0;t.a=i},function(e,t,n){"use strict";var i=n(89).Cache,r=n(90).tuple,a=n(91).Entry,o=n(57).get;t.defaultMakeCacheKey=r,t.wrap=function(e,t){var n=!!(t=function(e){return"function"!=typeof(e=e||Object.create(null)).makeCacheKey&&(e.makeCacheKey=r),"number"!=typeof e.max&&(e.max=Math.pow(2,16)),e}(t)).disposable,s=new i({max:t.max,dispose:function(e,t){t.dispose()}});function l(e){if(n)return s.delete(e.key),!0}function c(){if(!n||o().currentParentEntry){var i=t.makeCacheKey.apply(null,arguments);if(!i)return e.apply(null,arguments);for(var r=[],c=arguments.length;c--;)r[c]=arguments[c];var u=s.get(i);u?u.args=r:(s.set(i,u=a.acquire(e,i,r)),u.subscribe=t.subscribe,n&&(u.reportOrphan=l));var h=u.recompute();return s.set(i,u),0===u.parents.size&&s.clean(),n?void 0:h}}return c.dirty=function(){var e=t.makeCacheKey.apply(null,arguments);e&&s.has(e)&&s.get(e).setDirty()},c}},function(e,t,n){"use strict";var i=n(94),r={childContextTypes:!0,contextType:!0,contextTypes:!0,defaultProps:!0,displayName:!0,getDefaultProps:!0,getDerivedStateFromError:!0,getDerivedStateFromProps:!0,mixins:!0,propTypes:!0,type:!0},a={name:!0,length:!0,prototype:!0,caller:!0,callee:!0,arguments:!0,arity:!0},o={$$typeof:!0,compare:!0,defaultProps:!0,displayName:!0,propTypes:!0,type:!0},s={};function l(e){return i.isMemo(e)?o:s[e.$$typeof]||r}s[i.ForwardRef]={$$typeof:!0,render:!0,defaultProps:!0,displayName:!0,propTypes:!0};var c=Object.defineProperty,u=Object.getOwnPropertyNames,h=Object.getOwnPropertySymbols,d=Object.getOwnPropertyDescriptor,f=Object.getPrototypeOf,p=Object.prototype;e.exports=function e(t,n,i){if("string"!=typeof n){if(p){var r=f(n);r&&r!==p&&e(t,r,i)}var o=u(n);h&&(o=o.concat(h(n)));for(var s=l(t),g=l(n),m=0;m<o.length;++m){var v=o[m];if(!(a[v]||i&&i[v]||g&&g[v]||s&&s[v])){var y=d(n,v);try{c(t,v,y)}catch(e){}}}return t}return t}},function(e,t){e.exports=function(e){if(null==e)throw TypeError("Can't call method on  "+e);return e}},function(e,t){var n=Math.ceil,i=Math.floor;e.exports=function(e){return isNaN(e=+e)?0:(e>0?i:n)(e)}},function(e,t,n){e.exports=!n(34)(function(){return 7!=Object.defineProperty({},"a",{get:function(){return 7}}).a})},function(e,t,n){var i=n(11),r=n(12),a=n(24),o=n(32)("src"),s=n(73),l=(""+s).split("toString");n(31).inspectSource=function(e){return s.call(e)},(e.exports=function(e,t,n,s){var c="function"==typeof n;c&&(a(n,"name")||r(n,"name",t)),e[t]!==n&&(c&&(a(n,o)||r(n,o,e[t]?""+e[t]:l.join(String(t)))),e===i?e[t]=n:s?e[t]?e[t]=n:r(e,t,n):(delete e[t],r(e,t,n)))})(Function.prototype,"toString",function(){return"function"==typeof this&&this[o]||s.call(this)})},function(e,t){var n={}.hasOwnProperty;e.exports=function(e,t){return n.call(e,t)}},function(e,t,n){
/*!
  * Bootstrap util.js v4.3.1 (https://getbootstrap.com/)
  * Copyright 2011-2019 The Bootstrap Authors (https://github.com/twbs/bootstrap/graphs/contributors)
  * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
  */
e.exports=function(e){"use strict";e=e&&e.hasOwnProperty("default")?e.default:e;var t="transitionend";function n(t){var n=this,r=!1;return e(this).one(i.TRANSITION_END,function(){r=!0}),setTimeout(function(){r||i.triggerTransitionEnd(n)},t),this}var i={TRANSITION_END:"bsTransitionEnd",getUID:function(e){do{e+=~~(1e6*Math.random())}while(document.getElementById(e));return e},getSelectorFromElement:function(e){var t=e.getAttribute("data-target");if(!t||"#"===t){var n=e.getAttribute("href");t=n&&"#"!==n?n.trim():""}try{return document.querySelector(t)?t:null}catch(e){return null}},getTransitionDurationFromElement:function(t){if(!t)return 0;var n=e(t).css("transition-duration"),i=e(t).css("transition-delay"),r=parseFloat(n),a=parseFloat(i);return r||a?(n=n.split(",")[0],i=i.split(",")[0],1e3*(parseFloat(n)+parseFloat(i))):0},reflow:function(e){return e.offsetHeight},triggerTransitionEnd:function(n){e(n).trigger(t)},supportsTransitionEnd:function(){return Boolean(t)},isElement:function(e){return(e[0]||e).nodeType},typeCheckConfig:function(e,t,n){for(var r in n)if(Object.prototype.hasOwnProperty.call(n,r)){var a=n[r],o=t[r],s=o&&i.isElement(o)?"element":(l=o,{}.toString.call(l).match(/\s([a-z]+)/i)[1].toLowerCase());if(!new RegExp(a).test(s))throw new Error(e.toUpperCase()+': Option "'+r+'" provided type "'+s+'" but expected type "'+a+'".')}var l},findShadowRoot:function(e){if(!document.documentElement.attachShadow)return null;if("function"==typeof e.getRootNode){var t=e.getRootNode();return t instanceof ShadowRoot?t:null}return e instanceof ShadowRoot?e:e.parentNode?i.findShadowRoot(e.parentNode):null}};return e.fn.emulateTransitionEnd=n,e.event.special[i.TRANSITION_END]={bindType:t,delegateType:t,handle:function(t){if(e(t.target).is(this))return t.handleObj.handler.apply(this,arguments)}},i}(n(8))},function(e,t,n){"use strict";n.d(t,"a",function(){return s});var i=n(17);function r(e){return(r="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}var a=10,o=2;function s(e){return l(e,[])}function l(e,t){switch(r(e)){case"string":return JSON.stringify(e);case"function":return e.name?"[function ".concat(e.name,"]"):"[function]";case"object":return function(e,t){if(-1!==t.indexOf(e))return"[Circular]";var n=[].concat(t,[e]);if(e){var r=function(e){var t=e[String(i.a)];if("function"==typeof t)return t;if("function"==typeof e.inspect)return e.inspect}(e);if(r){var s=r.call(e);if(s!==e)return"string"==typeof s?s:l(s,n)}else if(Array.isArray(e))return function(e,t){if(0===e.length)return"[]";if(t.length>o)return"[Array]";for(var n=Math.min(a,e.length),i=e.length-n,r=[],s=0;s<n;++s)r.push(l(e[s],t));1===i?r.push("... 1 more item"):i>1&&r.push("... ".concat(i," more items"));return"["+r.join(", ")+"]"}(e,n);return function(e,t){var n=Object.keys(e);if(0===n.length)return"{}";if(t.length>o)return"["+function(e){var t=Object.prototype.toString.call(e).replace(/^\[object /,"").replace(/]$/,"");if("Object"===t&&"function"==typeof e.constructor){var n=e.constructor.name;if("string"==typeof n)return n}return t}(e)+"]";return"{ "+n.map(function(n){var i=l(e[n],t);return n+": "+i}).join(", ")+" }"}(e,n)}return String(e)}(e,t);default:return String(e)}}},function(e,t,n){"use strict";function i(e){for(var t=e.split(/\r\n|[\n\r]/g),n=null,i=1;i<t.length;i++){var o=t[i],s=r(o);if(s<o.length&&(null===n||s<n)&&0===(n=s))break}if(n)for(var l=1;l<t.length;l++)t[l]=t[l].slice(n);for(;t.length>0&&a(t[0]);)t.shift();for(;t.length>0&&a(t[t.length-1]);)t.pop();return t.join("\n")}function r(e){for(var t=0;t<e.length&&(" "===e[t]||"\t"===e[t]);)t++;return t}function a(e){return r(e)===e.length}function o(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:"",n=arguments.length>2&&void 0!==arguments[2]&&arguments[2],i=-1===e.indexOf("\n"),r=" "===e[0]||"\t"===e[0],a='"'===e[e.length-1],o=!i||a||n,s="";return!o||i&&r||(s+="\n"+t),s+=t?e.replace(/\n/g,"\n"+t):e,o&&(s+="\n"),'"""'+s.replace(/"""/g,'\\"""')+'"""'}n.d(t,"a",function(){return i}),n.d(t,"b",function(){return o})},function(e,t){e.exports=function(e){return"object"==typeof e?null!==e:"function"==typeof e}},function(e,t,n){var i=n(21),r=Math.min;e.exports=function(e){return e>0?r(i(e),9007199254740991):0}},function(e,t,n){var i=n(31),r=n(11),a=r["__core-js_shared__"]||(r["__core-js_shared__"]={});(e.exports=function(e,t){return a[e]||(a[e]=void 0!==t?t:{})})("versions",[]).push({version:i.version,mode:n(49)?"pure":"global",copyright:"© 2019 Denis Pushkarev (zloirock.ru)"})},function(e,t){var n=e.exports={version:"2.6.5"};"number"==typeof __e&&(__e=n)},function(e,t){var n=0,i=Math.random();e.exports=function(e){return"Symbol(".concat(void 0===e?"":e,")_",(++n+i).toString(36))}},function(e,t,n){var i=n(13),r=n(71),a=n(72),o=Object.defineProperty;t.f=n(22)?Object.defineProperty:function(e,t,n){if(i(e),t=a(t,!0),i(n),r)try{return o(e,t,n)}catch(e){}if("get"in n||"set"in n)throw TypeError("Accessors not supported!");return"value"in n&&(e[t]=n.value),e}},function(e,t){e.exports=function(e){try{return!!e()}catch(e){return!0}}},function(e,t){e.exports=function(e){return e.webpackPolyfill||(e.deprecate=function(){},e.paths=[],e.children||(e.children=[]),Object.defineProperty(e,"loaded",{enumerable:!0,get:function(){return e.l}}),Object.defineProperty(e,"id",{enumerable:!0,get:function(){return e.i}}),e.webpackPolyfill=1),e}},function(e,t,n){"use strict";
/*
object-assign
(c) Sindre Sorhus
@license MIT
*/var i=Object.getOwnPropertySymbols,r=Object.prototype.hasOwnProperty,a=Object.prototype.propertyIsEnumerable;e.exports=function(){try{if(!Object.assign)return!1;var e=new String("abc");if(e[5]="de","5"===Object.getOwnPropertyNames(e)[0])return!1;for(var t={},n=0;n<10;n++)t["_"+String.fromCharCode(n)]=n;if("0123456789"!==Object.getOwnPropertyNames(t).map(function(e){return t[e]}).join(""))return!1;var i={};return"abcdefghijklmnopqrst".split("").forEach(function(e){i[e]=e}),"abcdefghijklmnopqrst"===Object.keys(Object.assign({},i)).join("")}catch(e){return!1}}()?Object.assign:function(e,t){for(var n,o,s=function(e){if(null==e)throw new TypeError("Object.assign cannot be called with null or undefined");return Object(e)}(e),l=1;l<arguments.length;l++){for(var c in n=Object(arguments[l]))r.call(n,c)&&(s[c]=n[c]);if(i){o=i(n);for(var u=0;u<o.length;u++)a.call(n,o[u])&&(s[o[u]]=n[o[u]])}}return s}},function(e,t,n){"use strict";e.exports=n(96)},function(e,t){e.exports={}},function(e,t,n){var i=n(100),r=n(20);e.exports=function(e){return i(r(e))}},function(e,t,n){var i=n(30)("keys"),r=n(32);e.exports=function(e){return i[e]||(i[e]=r(e))}},function(e,t,n){"use strict";(function(e){n.d(t,"a",function(){return s});var i=n(1),r="Invariant Violation",a=Object.setPrototypeOf,o=void 0===a?function(e,t){return e.__proto__=t,e}:a,s=function(e){function t(n){void 0===n&&(n=r);var i=e.call(this,"number"==typeof n?r+": "+n+" (see https://github.com/apollographql/invariant-packages)":n)||this;return i.framesToPop=1,i.name=r,o(i,t.prototype),i}return Object(i.c)(t,e),t}(Error);function l(e,t){if(!e)throw new s(t)}!function(e){e.warn=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return console.warn.apply(console,e)},e.error=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return console.error.apply(console,e)}}(l||(l={}))}).call(this,n(15))},function(e,t,n){"use strict";(function(e){n.d(t,"a",function(){return s});var i=n(1),r="Invariant Violation",a=Object.setPrototypeOf,o=void 0===a?function(e,t){return e.__proto__=t,e}:a,s=function(e){function t(n){void 0===n&&(n=r);var i=e.call(this,"number"==typeof n?r+": "+n+" (see https://github.com/apollographql/invariant-packages)":n)||this;return i.framesToPop=1,i.name=r,o(i,t.prototype),i}return Object(i.c)(t,e),t}(Error);function l(e,t){if(!e)throw new s(t)}!function(e){e.warn=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return console.warn.apply(console,e)},e.error=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return console.error.apply(console,e)}}(l||(l={}))}).call(this,n(15))},function(e,t,n){(function(e,n){var i=200,r="__lodash_hash_undefined__",a=1,o=2,s=9007199254740991,l="[object Arguments]",c="[object Array]",u="[object AsyncFunction]",h="[object Boolean]",d="[object Date]",f="[object Error]",p="[object Function]",g="[object GeneratorFunction]",m="[object Map]",v="[object Number]",y="[object Null]",b="[object Object]",x="[object Proxy]",w="[object RegExp]",k="[object Set]",S="[object String]",E="[object Symbol]",C="[object Undefined]",T="[object ArrayBuffer]",A="[object DataView]",_=/^\[object .+?Constructor\]$/,O=/^(?:0|[1-9]\d*)$/,P={};P["[object Float32Array]"]=P["[object Float64Array]"]=P["[object Int8Array]"]=P["[object Int16Array]"]=P["[object Int32Array]"]=P["[object Uint8Array]"]=P["[object Uint8ClampedArray]"]=P["[object Uint16Array]"]=P["[object Uint32Array]"]=!0,P[l]=P[c]=P[T]=P[h]=P[A]=P[d]=P[f]=P[p]=P[m]=P[v]=P[b]=P[w]=P[k]=P[S]=P["[object WeakMap]"]=!1;var M="object"==typeof e&&e&&e.Object===Object&&e,I="object"==typeof self&&self&&self.Object===Object&&self,D=M||I||Function("return this")(),N=t&&!t.nodeType&&t,L=N&&"object"==typeof n&&n&&!n.nodeType&&n,R=L&&L.exports===N,F=R&&M.process,j=function(){try{return F&&F.binding&&F.binding("util")}catch(e){}}(),z=j&&j.isTypedArray;function Y(e,t){for(var n=-1,i=null==e?0:e.length;++n<i;)if(t(e[n],n,e))return!0;return!1}function H(e){var t=-1,n=Array(e.size);return e.forEach(function(e,i){n[++t]=[i,e]}),n}function W(e){var t=-1,n=Array(e.size);return e.forEach(function(e){n[++t]=e}),n}var X,V,B,q=Array.prototype,U=Function.prototype,G=Object.prototype,Q=D["__core-js_shared__"],$=U.toString,Z=G.hasOwnProperty,K=(X=/[^.]+$/.exec(Q&&Q.keys&&Q.keys.IE_PROTO||""))?"Symbol(src)_1."+X:"",J=G.toString,ee=RegExp("^"+$.call(Z).replace(/[\\^$.*+?()[\]{}|]/g,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$"),te=R?D.Buffer:void 0,ne=D.Symbol,ie=D.Uint8Array,re=G.propertyIsEnumerable,ae=q.splice,oe=ne?ne.toStringTag:void 0,se=Object.getOwnPropertySymbols,le=te?te.isBuffer:void 0,ce=(V=Object.keys,B=Object,function(e){return V(B(e))}),ue=je(D,"DataView"),he=je(D,"Map"),de=je(D,"Promise"),fe=je(D,"Set"),pe=je(D,"WeakMap"),ge=je(Object,"create"),me=We(ue),ve=We(he),ye=We(de),be=We(fe),xe=We(pe),we=ne?ne.prototype:void 0,ke=we?we.valueOf:void 0;function Se(e){var t=-1,n=null==e?0:e.length;for(this.clear();++t<n;){var i=e[t];this.set(i[0],i[1])}}function Ee(e){var t=-1,n=null==e?0:e.length;for(this.clear();++t<n;){var i=e[t];this.set(i[0],i[1])}}function Ce(e){var t=-1,n=null==e?0:e.length;for(this.clear();++t<n;){var i=e[t];this.set(i[0],i[1])}}function Te(e){var t=-1,n=null==e?0:e.length;for(this.__data__=new Ce;++t<n;)this.add(e[t])}function Ae(e){var t=this.__data__=new Ee(e);this.size=t.size}function _e(e,t){var n=Be(e),i=!n&&Ve(e),r=!n&&!i&&qe(e),a=!n&&!i&&!r&&Ze(e),o=n||i||r||a,s=o?function(e,t){for(var n=-1,i=Array(e);++n<e;)i[n]=t(n);return i}(e.length,String):[],l=s.length;for(var c in e)!t&&!Z.call(e,c)||o&&("length"==c||r&&("offset"==c||"parent"==c)||a&&("buffer"==c||"byteLength"==c||"byteOffset"==c)||He(c,l))||s.push(c);return s}function Oe(e,t){for(var n=e.length;n--;)if(Xe(e[n][0],t))return n;return-1}function Pe(e){return null==e?void 0===e?C:y:oe&&oe in Object(e)?function(e){var t=Z.call(e,oe),n=e[oe];try{e[oe]=void 0;var i=!0}catch(e){}var r=J.call(e);i&&(t?e[oe]=n:delete e[oe]);return r}(e):function(e){return J.call(e)}(e)}function Me(e){return $e(e)&&Pe(e)==l}function Ie(e,t,n,i,r){return e===t||(null==e||null==t||!$e(e)&&!$e(t)?e!=e&&t!=t:function(e,t,n,i,r,s){var u=Be(e),p=Be(t),g=u?c:Ye(e),y=p?c:Ye(t),x=(g=g==l?b:g)==b,C=(y=y==l?b:y)==b,_=g==y;if(_&&qe(e)){if(!qe(t))return!1;u=!0,x=!1}if(_&&!x)return s||(s=new Ae),u||Ze(e)?Le(e,t,n,i,r,s):function(e,t,n,i,r,s,l){switch(n){case A:if(e.byteLength!=t.byteLength||e.byteOffset!=t.byteOffset)return!1;e=e.buffer,t=t.buffer;case T:return!(e.byteLength!=t.byteLength||!s(new ie(e),new ie(t)));case h:case d:case v:return Xe(+e,+t);case f:return e.name==t.name&&e.message==t.message;case w:case S:return e==t+"";case m:var c=H;case k:var u=i&a;if(c||(c=W),e.size!=t.size&&!u)return!1;var p=l.get(e);if(p)return p==t;i|=o,l.set(e,t);var g=Le(c(e),c(t),i,r,s,l);return l.delete(e),g;case E:if(ke)return ke.call(e)==ke.call(t)}return!1}(e,t,g,n,i,r,s);if(!(n&a)){var O=x&&Z.call(e,"__wrapped__"),P=C&&Z.call(t,"__wrapped__");if(O||P){var M=O?e.value():e,I=P?t.value():t;return s||(s=new Ae),r(M,I,n,i,s)}}if(!_)return!1;return s||(s=new Ae),function(e,t,n,i,r,o){var s=n&a,l=Re(e),c=l.length,u=Re(t).length;if(c!=u&&!s)return!1;for(var h=c;h--;){var d=l[h];if(!(s?d in t:Z.call(t,d)))return!1}var f=o.get(e);if(f&&o.get(t))return f==t;var p=!0;o.set(e,t),o.set(t,e);for(var g=s;++h<c;){d=l[h];var m=e[d],v=t[d];if(i)var y=s?i(v,m,d,t,e,o):i(m,v,d,e,t,o);if(!(void 0===y?m===v||r(m,v,n,i,o):y)){p=!1;break}g||(g="constructor"==d)}if(p&&!g){var b=e.constructor,x=t.constructor;b!=x&&"constructor"in e&&"constructor"in t&&!("function"==typeof b&&b instanceof b&&"function"==typeof x&&x instanceof x)&&(p=!1)}return o.delete(e),o.delete(t),p}(e,t,n,i,r,s)}(e,t,n,i,Ie,r))}function De(e){return!(!Qe(e)||(t=e,K&&K in t))&&(Ue(e)?ee:_).test(We(e));var t}function Ne(e){if(n=(t=e)&&t.constructor,i="function"==typeof n&&n.prototype||G,t!==i)return ce(e);var t,n,i,r=[];for(var a in Object(e))Z.call(e,a)&&"constructor"!=a&&r.push(a);return r}function Le(e,t,n,i,r,s){var l=n&a,c=e.length,u=t.length;if(c!=u&&!(l&&u>c))return!1;var h=s.get(e);if(h&&s.get(t))return h==t;var d=-1,f=!0,p=n&o?new Te:void 0;for(s.set(e,t),s.set(t,e);++d<c;){var g=e[d],m=t[d];if(i)var v=l?i(m,g,d,t,e,s):i(g,m,d,e,t,s);if(void 0!==v){if(v)continue;f=!1;break}if(p){if(!Y(t,function(e,t){if(a=t,!p.has(a)&&(g===e||r(g,e,n,i,s)))return p.push(t);var a})){f=!1;break}}else if(g!==m&&!r(g,m,n,i,s)){f=!1;break}}return s.delete(e),s.delete(t),f}function Re(e){return function(e,t,n){var i=t(e);return Be(e)?i:function(e,t){for(var n=-1,i=t.length,r=e.length;++n<i;)e[r+n]=t[n];return e}(i,n(e))}(e,Ke,ze)}function Fe(e,t){var n,i,r=e.__data__;return("string"==(i=typeof(n=t))||"number"==i||"symbol"==i||"boolean"==i?"__proto__"!==n:null===n)?r["string"==typeof t?"string":"hash"]:r.map}function je(e,t){var n=function(e,t){return null==e?void 0:e[t]}(e,t);return De(n)?n:void 0}Se.prototype.clear=function(){this.__data__=ge?ge(null):{},this.size=0},Se.prototype.delete=function(e){var t=this.has(e)&&delete this.__data__[e];return this.size-=t?1:0,t},Se.prototype.get=function(e){var t=this.__data__;if(ge){var n=t[e];return n===r?void 0:n}return Z.call(t,e)?t[e]:void 0},Se.prototype.has=function(e){var t=this.__data__;return ge?void 0!==t[e]:Z.call(t,e)},Se.prototype.set=function(e,t){var n=this.__data__;return this.size+=this.has(e)?0:1,n[e]=ge&&void 0===t?r:t,this},Ee.prototype.clear=function(){this.__data__=[],this.size=0},Ee.prototype.delete=function(e){var t=this.__data__,n=Oe(t,e);return!(n<0||(n==t.length-1?t.pop():ae.call(t,n,1),--this.size,0))},Ee.prototype.get=function(e){var t=this.__data__,n=Oe(t,e);return n<0?void 0:t[n][1]},Ee.prototype.has=function(e){return Oe(this.__data__,e)>-1},Ee.prototype.set=function(e,t){var n=this.__data__,i=Oe(n,e);return i<0?(++this.size,n.push([e,t])):n[i][1]=t,this},Ce.prototype.clear=function(){this.size=0,this.__data__={hash:new Se,map:new(he||Ee),string:new Se}},Ce.prototype.delete=function(e){var t=Fe(this,e).delete(e);return this.size-=t?1:0,t},Ce.prototype.get=function(e){return Fe(this,e).get(e)},Ce.prototype.has=function(e){return Fe(this,e).has(e)},Ce.prototype.set=function(e,t){var n=Fe(this,e),i=n.size;return n.set(e,t),this.size+=n.size==i?0:1,this},Te.prototype.add=Te.prototype.push=function(e){return this.__data__.set(e,r),this},Te.prototype.has=function(e){return this.__data__.has(e)},Ae.prototype.clear=function(){this.__data__=new Ee,this.size=0},Ae.prototype.delete=function(e){var t=this.__data__,n=t.delete(e);return this.size=t.size,n},Ae.prototype.get=function(e){return this.__data__.get(e)},Ae.prototype.has=function(e){return this.__data__.has(e)},Ae.prototype.set=function(e,t){var n=this.__data__;if(n instanceof Ee){var r=n.__data__;if(!he||r.length<i-1)return r.push([e,t]),this.size=++n.size,this;n=this.__data__=new Ce(r)}return n.set(e,t),this.size=n.size,this};var ze=se?function(e){return null==e?[]:(e=Object(e),function(e,t){for(var n=-1,i=null==e?0:e.length,r=0,a=[];++n<i;){var o=e[n];t(o,n,e)&&(a[r++]=o)}return a}(se(e),function(t){return re.call(e,t)}))}:function(){return[]},Ye=Pe;function He(e,t){return!!(t=null==t?s:t)&&("number"==typeof e||O.test(e))&&e>-1&&e%1==0&&e<t}function We(e){if(null!=e){try{return $.call(e)}catch(e){}try{return e+""}catch(e){}}return""}function Xe(e,t){return e===t||e!=e&&t!=t}(ue&&Ye(new ue(new ArrayBuffer(1)))!=A||he&&Ye(new he)!=m||de&&"[object Promise]"!=Ye(de.resolve())||fe&&Ye(new fe)!=k||pe&&"[object WeakMap]"!=Ye(new pe))&&(Ye=function(e){var t=Pe(e),n=t==b?e.constructor:void 0,i=n?We(n):"";if(i)switch(i){case me:return A;case ve:return m;case ye:return"[object Promise]";case be:return k;case xe:return"[object WeakMap]"}return t});var Ve=Me(function(){return arguments}())?Me:function(e){return $e(e)&&Z.call(e,"callee")&&!re.call(e,"callee")},Be=Array.isArray;var qe=le||function(){return!1};function Ue(e){if(!Qe(e))return!1;var t=Pe(e);return t==p||t==g||t==u||t==x}function Ge(e){return"number"==typeof e&&e>-1&&e%1==0&&e<=s}function Qe(e){var t=typeof e;return null!=e&&("object"==t||"function"==t)}function $e(e){return null!=e&&"object"==typeof e}var Ze=z?function(e){return function(t){return e(t)}}(z):function(e){return $e(e)&&Ge(e.length)&&!!P[Pe(e)]};function Ke(e){return null!=(t=e)&&Ge(t.length)&&!Ue(t)?_e(e):Ne(e);var t}n.exports=function(e,t){return Ie(e,t)}}).call(this,n(14),n(35)(e))},function(e,t,n){"use strict";var i=n(13),r=n(45),a=n(29),o=n(21),s=n(46),l=n(47),c=Math.max,u=Math.min,h=Math.floor,d=/\$([$&`']|\d\d?|<[^>]*>)/g,f=/\$([$&`']|\d\d?)/g;n(50)("replace",2,function(e,t,n,p){return[function(i,r){var a=e(this),o=null==i?void 0:i[t];return void 0!==o?o.call(i,a,r):n.call(String(a),i,r)},function(e,t){var r=p(n,e,this,t);if(r.done)return r.value;var h=i(e),d=String(this),f="function"==typeof t;f||(t=String(t));var m=h.global;if(m){var v=h.unicode;h.lastIndex=0}for(var y=[];;){var b=l(h,d);if(null===b)break;if(y.push(b),!m)break;""===String(b[0])&&(h.lastIndex=s(d,a(h.lastIndex),v))}for(var x,w="",k=0,S=0;S<y.length;S++){b=y[S];for(var E=String(b[0]),C=c(u(o(b.index),d.length),0),T=[],A=1;A<b.length;A++)T.push(void 0===(x=b[A])?x:String(x));var _=b.groups;if(f){var O=[E].concat(T,C,d);void 0!==_&&O.push(_);var P=String(t.apply(void 0,O))}else P=g(E,d,C,T,_,t);C>=k&&(w+=d.slice(k,C)+P,k=C+E.length)}return w+d.slice(k)}];function g(e,t,i,a,o,s){var l=i+e.length,c=a.length,u=f;return void 0!==o&&(o=r(o),u=d),n.call(s,u,function(n,r){var s;switch(r.charAt(0)){case"$":return"$";case"&":return e;case"`":return t.slice(0,i);case"'":return t.slice(l);case"<":s=o[r.slice(1,-1)];break;default:var u=+r;if(0===u)return n;if(u>c){var d=h(u/10);return 0===d?n:d<=c?void 0===a[d-1]?r.charAt(1):a[d-1]+r.charAt(1):n}s=a[u-1]}return void 0===s?"":s})}})},function(e,t,n){var i=n(20);e.exports=function(e){return Object(i(e))}},function(e,t,n){"use strict";var i=n(67)(!0);e.exports=function(e,t,n){return t+(n?i(e,t).length:1)}},function(e,t,n){"use strict";var i=n(68),r=RegExp.prototype.exec;e.exports=function(e,t){var n=e.exec;if("function"==typeof n){var a=n.call(e,t);if("object"!=typeof a)throw new TypeError("RegExp exec method returned something other than an Object or null");return a}if("RegExp"!==i(e))throw new TypeError("RegExp#exec called on incompatible receiver");return r.call(e,t)}},function(e,t){var n={}.toString;e.exports=function(e){return n.call(e).slice(8,-1)}},function(e,t){e.exports=!1},function(e,t,n){"use strict";n(69);var i=n(23),r=n(12),a=n(34),o=n(20),s=n(10),l=n(51),c=s("species"),u=!a(function(){var e=/./;return e.exec=function(){var e=[];return e.groups={a:"7"},e},"7"!=="".replace(e,"$<a>")}),h=function(){var e=/(?:)/,t=e.exec;e.exec=function(){return t.apply(this,arguments)};var n="ab".split(e);return 2===n.length&&"a"===n[0]&&"b"===n[1]}();e.exports=function(e,t,n){var d=s(e),f=!a(function(){var t={};return t[d]=function(){return 7},7!=""[e](t)}),p=f?!a(function(){var t=!1,n=/a/;return n.exec=function(){return t=!0,null},"split"===e&&(n.constructor={},n.constructor[c]=function(){return n}),n[d](""),!t}):void 0;if(!f||!p||"replace"===e&&!u||"split"===e&&!h){var g=/./[d],m=n(o,d,""[e],function(e,t,n,i,r){return t.exec===l?f&&!r?{done:!0,value:g.call(t,n,i)}:{done:!0,value:e.call(n,t,i)}:{done:!1}}),v=m[0],y=m[1];i(String.prototype,e,v),r(RegExp.prototype,d,2==t?function(e,t){return y.call(e,this,t)}:function(e){return y.call(e,this)})}}},function(e,t,n){"use strict";var i,r,a=n(70),o=RegExp.prototype.exec,s=String.prototype.replace,l=o,c=(i=/a/,r=/b*/g,o.call(i,"a"),o.call(r,"a"),0!==i.lastIndex||0!==r.lastIndex),u=void 0!==/()??/.exec("")[1];(c||u)&&(l=function(e){var t,n,i,r,l=this;return u&&(n=new RegExp("^"+l.source+"$(?!\\s)",a.call(l))),c&&(t=l.lastIndex),i=o.call(l,e),c&&i&&(l.lastIndex=l.global?i.index+i[0].length:t),u&&i&&i.length>1&&s.call(i[0],n,function(){for(r=1;r<arguments.length-2;r++)void 0===arguments[r]&&(i[r]=void 0)}),i}),e.exports=l},function(e,t,n){var i=n(11),r=n(31),a=n(12),o=n(23),s=n(74),l=function(e,t,n){var c,u,h,d,f=e&l.F,p=e&l.G,g=e&l.S,m=e&l.P,v=e&l.B,y=p?i:g?i[t]||(i[t]={}):(i[t]||{}).prototype,b=p?r:r[t]||(r[t]={}),x=b.prototype||(b.prototype={});for(c in p&&(n=t),n)h=((u=!f&&y&&void 0!==y[c])?y:n)[c],d=v&&u?s(h,i):m&&"function"==typeof h?s(Function.call,h):h,y&&o(y,c,h,e&l.U),b[c]!=h&&a(b,c,d),m&&x[c]!=h&&(x[c]=h)};i.core=r,l.F=1,l.G=2,l.S=4,l.P=8,l.B=16,l.W=32,l.U=64,l.R=128,e.exports=l},function(e,t,n){var i=n(28),r=n(11).document,a=i(r)&&i(r.createElement);e.exports=function(e){return a?r.createElement(e):{}}},function(e,t){e.exports=function(e,t){return{enumerable:!(1&e),configurable:!(2&e),writable:!(4&e),value:t}}},function(e,t,n){"use strict";n.r(t),function(e){for(
/**!
 * @fileOverview Kickass library to create and place poppers near their reference elements.
 * @version 1.14.7
 * @license
 * Copyright (c) 2016 Federico Zivolo and contributors
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
var n="undefined"!=typeof window&&"undefined"!=typeof document,i=["Edge","Trident","Firefox"],r=0,a=0;a<i.length;a+=1)if(n&&navigator.userAgent.indexOf(i[a])>=0){r=1;break}var o=n&&window.Promise?function(e){var t=!1;return function(){t||(t=!0,window.Promise.resolve().then(function(){t=!1,e()}))}}:function(e){var t=!1;return function(){t||(t=!0,setTimeout(function(){t=!1,e()},r))}};function s(e){return e&&"[object Function]"==={}.toString.call(e)}function l(e,t){if(1!==e.nodeType)return[];var n=e.ownerDocument.defaultView.getComputedStyle(e,null);return t?n[t]:n}function c(e){return"HTML"===e.nodeName?e:e.parentNode||e.host}function u(e){if(!e)return document.body;switch(e.nodeName){case"HTML":case"BODY":return e.ownerDocument.body;case"#document":return e.body}var t=l(e),n=t.overflow,i=t.overflowX,r=t.overflowY;return/(auto|scroll|overlay)/.test(n+r+i)?e:u(c(e))}var h=n&&!(!window.MSInputMethodContext||!document.documentMode),d=n&&/MSIE 10/.test(navigator.userAgent);function f(e){return 11===e?h:10===e?d:h||d}function p(e){if(!e)return document.documentElement;for(var t=f(10)?document.body:null,n=e.offsetParent||null;n===t&&e.nextElementSibling;)n=(e=e.nextElementSibling).offsetParent;var i=n&&n.nodeName;return i&&"BODY"!==i&&"HTML"!==i?-1!==["TH","TD","TABLE"].indexOf(n.nodeName)&&"static"===l(n,"position")?p(n):n:e?e.ownerDocument.documentElement:document.documentElement}function g(e){return null!==e.parentNode?g(e.parentNode):e}function m(e,t){if(!(e&&e.nodeType&&t&&t.nodeType))return document.documentElement;var n=e.compareDocumentPosition(t)&Node.DOCUMENT_POSITION_FOLLOWING,i=n?e:t,r=n?t:e,a=document.createRange();a.setStart(i,0),a.setEnd(r,0);var o,s,l=a.commonAncestorContainer;if(e!==l&&t!==l||i.contains(r))return"BODY"===(s=(o=l).nodeName)||"HTML"!==s&&p(o.firstElementChild)!==o?p(l):l;var c=g(e);return c.host?m(c.host,t):m(e,g(t).host)}function v(e){var t="top"===(arguments.length>1&&void 0!==arguments[1]?arguments[1]:"top")?"scrollTop":"scrollLeft",n=e.nodeName;if("BODY"===n||"HTML"===n){var i=e.ownerDocument.documentElement;return(e.ownerDocument.scrollingElement||i)[t]}return e[t]}function y(e,t){var n="x"===t?"Left":"Top",i="Left"===n?"Right":"Bottom";return parseFloat(e["border"+n+"Width"],10)+parseFloat(e["border"+i+"Width"],10)}function b(e,t,n,i){return Math.max(t["offset"+e],t["scroll"+e],n["client"+e],n["offset"+e],n["scroll"+e],f(10)?parseInt(n["offset"+e])+parseInt(i["margin"+("Height"===e?"Top":"Left")])+parseInt(i["margin"+("Height"===e?"Bottom":"Right")]):0)}function x(e){var t=e.body,n=e.documentElement,i=f(10)&&getComputedStyle(n);return{height:b("Height",t,n,i),width:b("Width",t,n,i)}}var w=function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")},k=function(){function e(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}return function(t,n,i){return n&&e(t.prototype,n),i&&e(t,i),t}}(),S=function(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e},E=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var i in n)Object.prototype.hasOwnProperty.call(n,i)&&(e[i]=n[i])}return e};function C(e){return E({},e,{right:e.left+e.width,bottom:e.top+e.height})}function T(e){var t={};try{if(f(10)){t=e.getBoundingClientRect();var n=v(e,"top"),i=v(e,"left");t.top+=n,t.left+=i,t.bottom+=n,t.right+=i}else t=e.getBoundingClientRect()}catch(e){}var r={left:t.left,top:t.top,width:t.right-t.left,height:t.bottom-t.top},a="HTML"===e.nodeName?x(e.ownerDocument):{},o=a.width||e.clientWidth||r.right-r.left,s=a.height||e.clientHeight||r.bottom-r.top,c=e.offsetWidth-o,u=e.offsetHeight-s;if(c||u){var h=l(e);c-=y(h,"x"),u-=y(h,"y"),r.width-=c,r.height-=u}return C(r)}function A(e,t){var n=arguments.length>2&&void 0!==arguments[2]&&arguments[2],i=f(10),r="HTML"===t.nodeName,a=T(e),o=T(t),s=u(e),c=l(t),h=parseFloat(c.borderTopWidth,10),d=parseFloat(c.borderLeftWidth,10);n&&r&&(o.top=Math.max(o.top,0),o.left=Math.max(o.left,0));var p=C({top:a.top-o.top-h,left:a.left-o.left-d,width:a.width,height:a.height});if(p.marginTop=0,p.marginLeft=0,!i&&r){var g=parseFloat(c.marginTop,10),m=parseFloat(c.marginLeft,10);p.top-=h-g,p.bottom-=h-g,p.left-=d-m,p.right-=d-m,p.marginTop=g,p.marginLeft=m}return(i&&!n?t.contains(s):t===s&&"BODY"!==s.nodeName)&&(p=function(e,t){var n=arguments.length>2&&void 0!==arguments[2]&&arguments[2],i=v(t,"top"),r=v(t,"left"),a=n?-1:1;return e.top+=i*a,e.bottom+=i*a,e.left+=r*a,e.right+=r*a,e}(p,t)),p}function _(e){if(!e||!e.parentElement||f())return document.documentElement;for(var t=e.parentElement;t&&"none"===l(t,"transform");)t=t.parentElement;return t||document.documentElement}function O(e,t,n,i){var r=arguments.length>4&&void 0!==arguments[4]&&arguments[4],a={top:0,left:0},o=r?_(e):m(e,t);if("viewport"===i)a=function(e){var t=arguments.length>1&&void 0!==arguments[1]&&arguments[1],n=e.ownerDocument.documentElement,i=A(e,n),r=Math.max(n.clientWidth,window.innerWidth||0),a=Math.max(n.clientHeight,window.innerHeight||0),o=t?0:v(n),s=t?0:v(n,"left");return C({top:o-i.top+i.marginTop,left:s-i.left+i.marginLeft,width:r,height:a})}(o,r);else{var s=void 0;"scrollParent"===i?"BODY"===(s=u(c(t))).nodeName&&(s=e.ownerDocument.documentElement):s="window"===i?e.ownerDocument.documentElement:i;var h=A(s,o,r);if("HTML"!==s.nodeName||function e(t){var n=t.nodeName;if("BODY"===n||"HTML"===n)return!1;if("fixed"===l(t,"position"))return!0;var i=c(t);return!!i&&e(i)}(o))a=h;else{var d=x(e.ownerDocument),f=d.height,p=d.width;a.top+=h.top-h.marginTop,a.bottom=f+h.top,a.left+=h.left-h.marginLeft,a.right=p+h.left}}var g="number"==typeof(n=n||0);return a.left+=g?n:n.left||0,a.top+=g?n:n.top||0,a.right-=g?n:n.right||0,a.bottom-=g?n:n.bottom||0,a}function P(e,t,n,i,r){var a=arguments.length>5&&void 0!==arguments[5]?arguments[5]:0;if(-1===e.indexOf("auto"))return e;var o=O(n,i,a,r),s={top:{width:o.width,height:t.top-o.top},right:{width:o.right-t.right,height:o.height},bottom:{width:o.width,height:o.bottom-t.bottom},left:{width:t.left-o.left,height:o.height}},l=Object.keys(s).map(function(e){return E({key:e},s[e],{area:(t=s[e],t.width*t.height)});var t}).sort(function(e,t){return t.area-e.area}),c=l.filter(function(e){var t=e.width,i=e.height;return t>=n.clientWidth&&i>=n.clientHeight}),u=c.length>0?c[0].key:l[0].key,h=e.split("-")[1];return u+(h?"-"+h:"")}function M(e,t,n){var i=arguments.length>3&&void 0!==arguments[3]?arguments[3]:null;return A(n,i?_(t):m(t,n),i)}function I(e){var t=e.ownerDocument.defaultView.getComputedStyle(e),n=parseFloat(t.marginTop||0)+parseFloat(t.marginBottom||0),i=parseFloat(t.marginLeft||0)+parseFloat(t.marginRight||0);return{width:e.offsetWidth+i,height:e.offsetHeight+n}}function D(e){var t={left:"right",right:"left",bottom:"top",top:"bottom"};return e.replace(/left|right|bottom|top/g,function(e){return t[e]})}function N(e,t,n){n=n.split("-")[0];var i=I(e),r={width:i.width,height:i.height},a=-1!==["right","left"].indexOf(n),o=a?"top":"left",s=a?"left":"top",l=a?"height":"width",c=a?"width":"height";return r[o]=t[o]+t[l]/2-i[l]/2,r[s]=n===s?t[s]-i[c]:t[D(s)],r}function L(e,t){return Array.prototype.find?e.find(t):e.filter(t)[0]}function R(e,t,n){return(void 0===n?e:e.slice(0,function(e,t,n){if(Array.prototype.findIndex)return e.findIndex(function(e){return e[t]===n});var i=L(e,function(e){return e[t]===n});return e.indexOf(i)}(e,"name",n))).forEach(function(e){e.function&&console.warn("`modifier.function` is deprecated, use `modifier.fn`!");var n=e.function||e.fn;e.enabled&&s(n)&&(t.offsets.popper=C(t.offsets.popper),t.offsets.reference=C(t.offsets.reference),t=n(t,e))}),t}function F(e,t){return e.some(function(e){var n=e.name;return e.enabled&&n===t})}function j(e){for(var t=[!1,"ms","Webkit","Moz","O"],n=e.charAt(0).toUpperCase()+e.slice(1),i=0;i<t.length;i++){var r=t[i],a=r?""+r+n:e;if(void 0!==document.body.style[a])return a}return null}function z(e){var t=e.ownerDocument;return t?t.defaultView:window}function Y(e,t,n,i){n.updateBound=i,z(e).addEventListener("resize",n.updateBound,{passive:!0});var r=u(e);return function e(t,n,i,r){var a="BODY"===t.nodeName,o=a?t.ownerDocument.defaultView:t;o.addEventListener(n,i,{passive:!0}),a||e(u(o.parentNode),n,i,r),r.push(o)}(r,"scroll",n.updateBound,n.scrollParents),n.scrollElement=r,n.eventsEnabled=!0,n}function H(){var e,t;this.state.eventsEnabled&&(cancelAnimationFrame(this.scheduleUpdate),this.state=(e=this.reference,t=this.state,z(e).removeEventListener("resize",t.updateBound),t.scrollParents.forEach(function(e){e.removeEventListener("scroll",t.updateBound)}),t.updateBound=null,t.scrollParents=[],t.scrollElement=null,t.eventsEnabled=!1,t))}function W(e){return""!==e&&!isNaN(parseFloat(e))&&isFinite(e)}function X(e,t){Object.keys(t).forEach(function(n){var i="";-1!==["width","height","top","right","bottom","left"].indexOf(n)&&W(t[n])&&(i="px"),e.style[n]=t[n]+i})}var V=n&&/Firefox/i.test(navigator.userAgent);function B(e,t,n){var i=L(e,function(e){return e.name===t}),r=!!i&&e.some(function(e){return e.name===n&&e.enabled&&e.order<i.order});if(!r){var a="`"+t+"`",o="`"+n+"`";console.warn(o+" modifier is required by "+a+" modifier in order to work, be sure to include it before "+a+"!")}return r}var q=["auto-start","auto","auto-end","top-start","top","top-end","right-start","right","right-end","bottom-end","bottom","bottom-start","left-end","left","left-start"],U=q.slice(3);function G(e){var t=arguments.length>1&&void 0!==arguments[1]&&arguments[1],n=U.indexOf(e),i=U.slice(n+1).concat(U.slice(0,n));return t?i.reverse():i}var Q={FLIP:"flip",CLOCKWISE:"clockwise",COUNTERCLOCKWISE:"counterclockwise"};function $(e,t,n,i){var r=[0,0],a=-1!==["right","left"].indexOf(i),o=e.split(/(\+|\-)/).map(function(e){return e.trim()}),s=o.indexOf(L(o,function(e){return-1!==e.search(/,|\s/)}));o[s]&&-1===o[s].indexOf(",")&&console.warn("Offsets separated by white space(s) are deprecated, use a comma (,) instead.");var l=/\s*,\s*|\s+/,c=-1!==s?[o.slice(0,s).concat([o[s].split(l)[0]]),[o[s].split(l)[1]].concat(o.slice(s+1))]:[o];return(c=c.map(function(e,i){var r=(1===i?!a:a)?"height":"width",o=!1;return e.reduce(function(e,t){return""===e[e.length-1]&&-1!==["+","-"].indexOf(t)?(e[e.length-1]=t,o=!0,e):o?(e[e.length-1]+=t,o=!1,e):e.concat(t)},[]).map(function(e){return function(e,t,n,i){var r=e.match(/((?:\-|\+)?\d*\.?\d*)(.*)/),a=+r[1],o=r[2];if(!a)return e;if(0===o.indexOf("%")){var s=void 0;switch(o){case"%p":s=n;break;case"%":case"%r":default:s=i}return C(s)[t]/100*a}if("vh"===o||"vw"===o)return("vh"===o?Math.max(document.documentElement.clientHeight,window.innerHeight||0):Math.max(document.documentElement.clientWidth,window.innerWidth||0))/100*a;return a}(e,r,t,n)})})).forEach(function(e,t){e.forEach(function(n,i){W(n)&&(r[t]+=n*("-"===e[i-1]?-1:1))})}),r}var Z={placement:"bottom",positionFixed:!1,eventsEnabled:!0,removeOnDestroy:!1,onCreate:function(){},onUpdate:function(){},modifiers:{shift:{order:100,enabled:!0,fn:function(e){var t=e.placement,n=t.split("-")[0],i=t.split("-")[1];if(i){var r=e.offsets,a=r.reference,o=r.popper,s=-1!==["bottom","top"].indexOf(n),l=s?"left":"top",c=s?"width":"height",u={start:S({},l,a[l]),end:S({},l,a[l]+a[c]-o[c])};e.offsets.popper=E({},o,u[i])}return e}},offset:{order:200,enabled:!0,fn:function(e,t){var n=t.offset,i=e.placement,r=e.offsets,a=r.popper,o=r.reference,s=i.split("-")[0],l=void 0;return l=W(+n)?[+n,0]:$(n,a,o,s),"left"===s?(a.top+=l[0],a.left-=l[1]):"right"===s?(a.top+=l[0],a.left+=l[1]):"top"===s?(a.left+=l[0],a.top-=l[1]):"bottom"===s&&(a.left+=l[0],a.top+=l[1]),e.popper=a,e},offset:0},preventOverflow:{order:300,enabled:!0,fn:function(e,t){var n=t.boundariesElement||p(e.instance.popper);e.instance.reference===n&&(n=p(n));var i=j("transform"),r=e.instance.popper.style,a=r.top,o=r.left,s=r[i];r.top="",r.left="",r[i]="";var l=O(e.instance.popper,e.instance.reference,t.padding,n,e.positionFixed);r.top=a,r.left=o,r[i]=s,t.boundaries=l;var c=t.priority,u=e.offsets.popper,h={primary:function(e){var n=u[e];return u[e]<l[e]&&!t.escapeWithReference&&(n=Math.max(u[e],l[e])),S({},e,n)},secondary:function(e){var n="right"===e?"left":"top",i=u[n];return u[e]>l[e]&&!t.escapeWithReference&&(i=Math.min(u[n],l[e]-("right"===e?u.width:u.height))),S({},n,i)}};return c.forEach(function(e){var t=-1!==["left","top"].indexOf(e)?"primary":"secondary";u=E({},u,h[t](e))}),e.offsets.popper=u,e},priority:["left","right","top","bottom"],padding:5,boundariesElement:"scrollParent"},keepTogether:{order:400,enabled:!0,fn:function(e){var t=e.offsets,n=t.popper,i=t.reference,r=e.placement.split("-")[0],a=Math.floor,o=-1!==["top","bottom"].indexOf(r),s=o?"right":"bottom",l=o?"left":"top",c=o?"width":"height";return n[s]<a(i[l])&&(e.offsets.popper[l]=a(i[l])-n[c]),n[l]>a(i[s])&&(e.offsets.popper[l]=a(i[s])),e}},arrow:{order:500,enabled:!0,fn:function(e,t){var n;if(!B(e.instance.modifiers,"arrow","keepTogether"))return e;var i=t.element;if("string"==typeof i){if(!(i=e.instance.popper.querySelector(i)))return e}else if(!e.instance.popper.contains(i))return console.warn("WARNING: `arrow.element` must be child of its popper element!"),e;var r=e.placement.split("-")[0],a=e.offsets,o=a.popper,s=a.reference,c=-1!==["left","right"].indexOf(r),u=c?"height":"width",h=c?"Top":"Left",d=h.toLowerCase(),f=c?"left":"top",p=c?"bottom":"right",g=I(i)[u];s[p]-g<o[d]&&(e.offsets.popper[d]-=o[d]-(s[p]-g)),s[d]+g>o[p]&&(e.offsets.popper[d]+=s[d]+g-o[p]),e.offsets.popper=C(e.offsets.popper);var m=s[d]+s[u]/2-g/2,v=l(e.instance.popper),y=parseFloat(v["margin"+h],10),b=parseFloat(v["border"+h+"Width"],10),x=m-e.offsets.popper[d]-y-b;return x=Math.max(Math.min(o[u]-g,x),0),e.arrowElement=i,e.offsets.arrow=(S(n={},d,Math.round(x)),S(n,f,""),n),e},element:"[x-arrow]"},flip:{order:600,enabled:!0,fn:function(e,t){if(F(e.instance.modifiers,"inner"))return e;if(e.flipped&&e.placement===e.originalPlacement)return e;var n=O(e.instance.popper,e.instance.reference,t.padding,t.boundariesElement,e.positionFixed),i=e.placement.split("-")[0],r=D(i),a=e.placement.split("-")[1]||"",o=[];switch(t.behavior){case Q.FLIP:o=[i,r];break;case Q.CLOCKWISE:o=G(i);break;case Q.COUNTERCLOCKWISE:o=G(i,!0);break;default:o=t.behavior}return o.forEach(function(s,l){if(i!==s||o.length===l+1)return e;i=e.placement.split("-")[0],r=D(i);var c=e.offsets.popper,u=e.offsets.reference,h=Math.floor,d="left"===i&&h(c.right)>h(u.left)||"right"===i&&h(c.left)<h(u.right)||"top"===i&&h(c.bottom)>h(u.top)||"bottom"===i&&h(c.top)<h(u.bottom),f=h(c.left)<h(n.left),p=h(c.right)>h(n.right),g=h(c.top)<h(n.top),m=h(c.bottom)>h(n.bottom),v="left"===i&&f||"right"===i&&p||"top"===i&&g||"bottom"===i&&m,y=-1!==["top","bottom"].indexOf(i),b=!!t.flipVariations&&(y&&"start"===a&&f||y&&"end"===a&&p||!y&&"start"===a&&g||!y&&"end"===a&&m);(d||v||b)&&(e.flipped=!0,(d||v)&&(i=o[l+1]),b&&(a=function(e){return"end"===e?"start":"start"===e?"end":e}(a)),e.placement=i+(a?"-"+a:""),e.offsets.popper=E({},e.offsets.popper,N(e.instance.popper,e.offsets.reference,e.placement)),e=R(e.instance.modifiers,e,"flip"))}),e},behavior:"flip",padding:5,boundariesElement:"viewport"},inner:{order:700,enabled:!1,fn:function(e){var t=e.placement,n=t.split("-")[0],i=e.offsets,r=i.popper,a=i.reference,o=-1!==["left","right"].indexOf(n),s=-1===["top","left"].indexOf(n);return r[o?"left":"top"]=a[n]-(s?r[o?"width":"height"]:0),e.placement=D(t),e.offsets.popper=C(r),e}},hide:{order:800,enabled:!0,fn:function(e){if(!B(e.instance.modifiers,"hide","preventOverflow"))return e;var t=e.offsets.reference,n=L(e.instance.modifiers,function(e){return"preventOverflow"===e.name}).boundaries;if(t.bottom<n.top||t.left>n.right||t.top>n.bottom||t.right<n.left){if(!0===e.hide)return e;e.hide=!0,e.attributes["x-out-of-boundaries"]=""}else{if(!1===e.hide)return e;e.hide=!1,e.attributes["x-out-of-boundaries"]=!1}return e}},computeStyle:{order:850,enabled:!0,fn:function(e,t){var n=t.x,i=t.y,r=e.offsets.popper,a=L(e.instance.modifiers,function(e){return"applyStyle"===e.name}).gpuAcceleration;void 0!==a&&console.warn("WARNING: `gpuAcceleration` option moved to `computeStyle` modifier and will not be supported in future versions of Popper.js!");var o=void 0!==a?a:t.gpuAcceleration,s=p(e.instance.popper),l=T(s),c={position:r.position},u=function(e,t){var n=e.offsets,i=n.popper,r=n.reference,a=Math.round,o=Math.floor,s=function(e){return e},l=a(r.width),c=a(i.width),u=-1!==["left","right"].indexOf(e.placement),h=-1!==e.placement.indexOf("-"),d=t?u||h||l%2==c%2?a:o:s,f=t?a:s;return{left:d(l%2==1&&c%2==1&&!h&&t?i.left-1:i.left),top:f(i.top),bottom:f(i.bottom),right:d(i.right)}}(e,window.devicePixelRatio<2||!V),h="bottom"===n?"top":"bottom",d="right"===i?"left":"right",f=j("transform"),g=void 0,m=void 0;if(m="bottom"===h?"HTML"===s.nodeName?-s.clientHeight+u.bottom:-l.height+u.bottom:u.top,g="right"===d?"HTML"===s.nodeName?-s.clientWidth+u.right:-l.width+u.right:u.left,o&&f)c[f]="translate3d("+g+"px, "+m+"px, 0)",c[h]=0,c[d]=0,c.willChange="transform";else{var v="bottom"===h?-1:1,y="right"===d?-1:1;c[h]=m*v,c[d]=g*y,c.willChange=h+", "+d}var b={"x-placement":e.placement};return e.attributes=E({},b,e.attributes),e.styles=E({},c,e.styles),e.arrowStyles=E({},e.offsets.arrow,e.arrowStyles),e},gpuAcceleration:!0,x:"bottom",y:"right"},applyStyle:{order:900,enabled:!0,fn:function(e){var t,n;return X(e.instance.popper,e.styles),t=e.instance.popper,n=e.attributes,Object.keys(n).forEach(function(e){!1!==n[e]?t.setAttribute(e,n[e]):t.removeAttribute(e)}),e.arrowElement&&Object.keys(e.arrowStyles).length&&X(e.arrowElement,e.arrowStyles),e},onLoad:function(e,t,n,i,r){var a=M(r,t,e,n.positionFixed),o=P(n.placement,a,t,e,n.modifiers.flip.boundariesElement,n.modifiers.flip.padding);return t.setAttribute("x-placement",o),X(t,{position:n.positionFixed?"fixed":"absolute"}),n},gpuAcceleration:void 0}}},K=function(){function e(t,n){var i=this,r=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};w(this,e),this.scheduleUpdate=function(){return requestAnimationFrame(i.update)},this.update=o(this.update.bind(this)),this.options=E({},e.Defaults,r),this.state={isDestroyed:!1,isCreated:!1,scrollParents:[]},this.reference=t&&t.jquery?t[0]:t,this.popper=n&&n.jquery?n[0]:n,this.options.modifiers={},Object.keys(E({},e.Defaults.modifiers,r.modifiers)).forEach(function(t){i.options.modifiers[t]=E({},e.Defaults.modifiers[t]||{},r.modifiers?r.modifiers[t]:{})}),this.modifiers=Object.keys(this.options.modifiers).map(function(e){return E({name:e},i.options.modifiers[e])}).sort(function(e,t){return e.order-t.order}),this.modifiers.forEach(function(e){e.enabled&&s(e.onLoad)&&e.onLoad(i.reference,i.popper,i.options,e,i.state)}),this.update();var a=this.options.eventsEnabled;a&&this.enableEventListeners(),this.state.eventsEnabled=a}return k(e,[{key:"update",value:function(){return function(){if(!this.state.isDestroyed){var e={instance:this,styles:{},arrowStyles:{},attributes:{},flipped:!1,offsets:{}};e.offsets.reference=M(this.state,this.popper,this.reference,this.options.positionFixed),e.placement=P(this.options.placement,e.offsets.reference,this.popper,this.reference,this.options.modifiers.flip.boundariesElement,this.options.modifiers.flip.padding),e.originalPlacement=e.placement,e.positionFixed=this.options.positionFixed,e.offsets.popper=N(this.popper,e.offsets.reference,e.placement),e.offsets.popper.position=this.options.positionFixed?"fixed":"absolute",e=R(this.modifiers,e),this.state.isCreated?this.options.onUpdate(e):(this.state.isCreated=!0,this.options.onCreate(e))}}.call(this)}},{key:"destroy",value:function(){return function(){return this.state.isDestroyed=!0,F(this.modifiers,"applyStyle")&&(this.popper.removeAttribute("x-placement"),this.popper.style.position="",this.popper.style.top="",this.popper.style.left="",this.popper.style.right="",this.popper.style.bottom="",this.popper.style.willChange="",this.popper.style[j("transform")]=""),this.disableEventListeners(),this.options.removeOnDestroy&&this.popper.parentNode.removeChild(this.popper),this}.call(this)}},{key:"enableEventListeners",value:function(){return function(){this.state.eventsEnabled||(this.state=Y(this.reference,this.options,this.state,this.scheduleUpdate))}.call(this)}},{key:"disableEventListeners",value:function(){return H.call(this)}}]),e}();K.Utils=("undefined"!=typeof window?window:e).PopperUtils,K.placements=q,K.Defaults=Z,t.default=K}.call(this,n(14))},function(e,t,n){var i=n(112).parse;function r(e){return e.replace(/[\s,]+/g," ").trim()}var a={},o={};var s=!0;var l=!1;function c(e){var t=r(e);if(a[t])return a[t];var n=i(e,{experimentalFragmentVariables:l});if(!n||"Document"!==n.kind)throw new Error("Not a valid GraphQL document.");return n=function e(t,n){var i=Object.prototype.toString.call(t);if("[object Array]"===i)return t.map(function(t){return e(t,n)});if("[object Object]"!==i)throw new Error("Unexpected input.");n&&t.loc&&delete t.loc,t.loc&&(delete t.loc.startToken,delete t.loc.endToken);var r,a,o,s=Object.keys(t);for(r in s)s.hasOwnProperty(r)&&(a=t[s[r]],"[object Object]"!==(o=Object.prototype.toString.call(a))&&"[object Array]"!==o||(t[s[r]]=e(a,!0)));return t}(n=function(e){for(var t,n={},i=[],a=0;a<e.definitions.length;a++){var l=e.definitions[a];if("FragmentDefinition"===l.kind){var c=l.name.value,u=r((t=l.loc).source.body.substring(t.start,t.end));o.hasOwnProperty(c)&&!o[c][u]?(s&&console.warn("Warning: fragment with name "+c+" already exists.\ngraphql-tag enforces all fragment names across your application to be unique; read more about\nthis in the docs: http://dev.apollodata.com/core/fragments.html#unique-names"),o[c][u]=!0):o.hasOwnProperty(c)||(o[c]={},o[c][u]=!0),n[u]||(n[u]=!0,i.push(l))}else i.push(l)}return e.definitions=i,e}(n),!1),a[t]=n,n}function u(){for(var e=Array.prototype.slice.call(arguments),t=e[0],n="string"==typeof t?t:t[0],i=1;i<e.length;i++)e[i]&&e[i].kind&&"Document"===e[i].kind?n+=e[i].loc.source.body:n+=e[i],n+=t[i];return c(n)}u.default=u,u.resetCaches=function(){a={},o={}},u.disableFragmentWarnings=function(){s=!1},u.enableExperimentalFragmentVariables=function(){l=!0},u.disableExperimentalFragmentVariables=function(){l=!1},e.exports=u},function(e,t,n){"use strict";(function(e){var n=new function(){};function i(){return n}try{var r=e["eriuqer".split("").reverse().join("")]("fibers");i=function(){return r.current||n}}catch(e){}t.get=function(){var e=i();return e._optimism_local||(e._optimism_local=Object.create(null))}}).call(this,n(35)(e))},function(e,t,n){var i=n(105),r=n(59);e.exports=Object.keys||function(e){return i(e,r)}},function(e,t){e.exports="constructor,hasOwnProperty,isPrototypeOf,propertyIsEnumerable,toLocaleString,toString,valueOf".split(",")},function(e,t,n){var i=n(33).f,r=n(24),a=n(10)("toStringTag");e.exports=function(e,t,n){e&&!r(e=n?e:e.prototype,a)&&i(e,a,{configurable:!0,value:t})}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var i="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e},r=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var i in n)Object.prototype.hasOwnProperty.call(n,i)&&(e[i]=n[i])}return e},a=function(){function e(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}return function(t,n,i){return n&&e(t.prototype,n),i&&e(t,i),t}}(),o=u(n(84)),s=n(0),l=u(s),c=u(n(4));function u(e){return e&&e.__esModule?e:{default:e}}window.ApexCharts=o.default;var h=function(e){function t(e){!function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,t);var n=function(e,t){if(!e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return!t||"object"!=typeof t&&"function"!=typeof t?e:t}(this,(t.__proto__||Object.getPrototypeOf(t)).call(this,e));return l.default.createRef?n.chartRef=l.default.createRef():n.setRef=function(e){return n.chartRef=e},n.chart=null,n}return function(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function, not "+typeof t);e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,enumerable:!1,writable:!0,configurable:!0}}),t&&(Object.setPrototypeOf?Object.setPrototypeOf(e,t):e.__proto__=t)}(t,s.Component),a(t,[{key:"render",value:function(){var e=this.props,t=(e.type,e.width,e.height,e.series,e.options,function(e,t){var n={};for(var i in e)0<=t.indexOf(i)||Object.prototype.hasOwnProperty.call(e,i)&&(n[i]=e[i]);return n}(e,["type","width","height","series","options"]));return l.default.createElement("div",r({ref:l.default.createRef?this.chartRef:this.setRef},t))}},{key:"componentDidMount",value:function(){var e=l.default.createRef?this.chartRef.current:this.chartRef;this.chart=new o.default(e,this.getConfig()),this.chart.render()}},{key:"getConfig",value:function(){var e=this.props,t=e.type,n=e.height,i=e.width,r=e.series,a=e.options,o={chart:{type:t,height:n,width:i},series:r};return this.extend(a,o)}},{key:"isObject",value:function(e){return e&&"object"===(void 0===e?"undefined":i(e))&&!Array.isArray(e)&&null!=e}},{key:"extend",value:function(e,t){var n=this;"function"!=typeof Object.assign&&(Object.assign=function(e){if(null==e)throw new TypeError("Cannot convert undefined or null to object");for(var t=Object(e),n=1;n<arguments.length;n++){var i=arguments[n];if(null!=i)for(var r in i)i.hasOwnProperty(r)&&(t[r]=i[r])}return t});var i=Object.assign({},e);return this.isObject(e)&&this.isObject(t)&&Object.keys(t).forEach(function(r){n.isObject(t[r])&&r in e?i[r]=n.extend(e[r],t[r]):Object.assign(i,function(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}({},r,t[r]))}),i}},{key:"componentDidUpdate",value:function(e){if(!this.chart)return null;var t=this.props,n=t.options,i=t.series,r=JSON.stringify(e.options),a=JSON.stringify(e.series),o=JSON.stringify(n),s=JSON.stringify(i);r===o&&a===s||(a===s?this.chart.updateOptions(this.getConfig()):r===o?this.chart.updateSeries(i):this.chart.updateOptions(this.getConfig()))}},{key:"componentWillUnmount",value:function(){this.chart&&"function"==typeof this.chart.destroy&&this.chart.destroy()}}]),t}();(t.default=h).propTypes={type:c.default.string.isRequired,width:c.default.any,height:c.default.any,series:c.default.array.isRequired,options:c.default.object.isRequired},h.defaultProps={type:"line",width:"100%",height:"auto"}},function(e,t,n){"use strict";e.exports=function(e,t){t||(t={}),"function"==typeof t&&(t={cmp:t});var n,i="boolean"==typeof t.cycles&&t.cycles,r=t.cmp&&(n=t.cmp,function(e){return function(t,i){var r={key:t,value:e[t]},a={key:i,value:e[i]};return n(r,a)}}),a=[];return function e(t){if(t&&t.toJSON&&"function"==typeof t.toJSON&&(t=t.toJSON()),void 0!==t){if("number"==typeof t)return isFinite(t)?""+t:"null";if("object"!=typeof t)return JSON.stringify(t);var n,o;if(Array.isArray(t)){for(o="[",n=0;n<t.length;n++)n&&(o+=","),o+=e(t[n])||"null";return o+"]"}if(null===t)return"null";if(-1!==a.indexOf(t)){if(i)return JSON.stringify("__cycle__");throw new TypeError("Converting circular structure to JSON")}var s=a.push(t)-1,l=Object.keys(t).sort(r&&r(t));for(o="",n=0;n<l.length;n++){var c=l[n],u=e(t[c]);u&&(o&&(o+=","),o+=JSON.stringify(c)+":"+u)}return a.splice(s,1),"{"+o+"}"}}(e)}},function(e,t,n){e.exports=n(92).Observable},function(e,t,n){"use strict";(function(e,i){var r,a=n(65);r="undefined"!=typeof self?self:"undefined"!=typeof window?window:void 0!==e?e:i;var o=Object(a.a)(r);t.a=o}).call(this,n(14),n(93)(e))},function(e,t,n){"use strict";function i(e){var t,n=e.Symbol;return"function"==typeof n?n.observable?t=n.observable:(t=n("observable"),n.observable=t):t="@@observable",t}n.d(t,"a",function(){return i})},,function(e,t,n){var i=n(21),r=n(20);e.exports=function(e){return function(t,n){var a,o,s=String(r(t)),l=i(n),c=s.length;return l<0||l>=c?e?"":void 0:(a=s.charCodeAt(l))<55296||a>56319||l+1===c||(o=s.charCodeAt(l+1))<56320||o>57343?e?s.charAt(l):a:e?s.slice(l,l+2):o-56320+(a-55296<<10)+65536}}},function(e,t,n){var i=n(48),r=n(10)("toStringTag"),a="Arguments"==i(function(){return arguments}());e.exports=function(e){var t,n,o;return void 0===e?"Undefined":null===e?"Null":"string"==typeof(n=function(e,t){try{return e[t]}catch(e){}}(t=Object(e),r))?n:a?i(t):"Object"==(o=i(t))&&"function"==typeof t.callee?"Arguments":o}},function(e,t,n){"use strict";var i=n(51);n(52)({target:"RegExp",proto:!0,forced:i!==/./.exec},{exec:i})},function(e,t,n){"use strict";var i=n(13);e.exports=function(){var e=i(this),t="";return e.global&&(t+="g"),e.ignoreCase&&(t+="i"),e.multiline&&(t+="m"),e.unicode&&(t+="u"),e.sticky&&(t+="y"),t}},function(e,t,n){e.exports=!n(22)&&!n(34)(function(){return 7!=Object.defineProperty(n(53)("div"),"a",{get:function(){return 7}}).a})},function(e,t,n){var i=n(28);e.exports=function(e,t){if(!i(e))return e;var n,r;if(t&&"function"==typeof(n=e.toString)&&!i(r=n.call(e)))return r;if("function"==typeof(n=e.valueOf)&&!i(r=n.call(e)))return r;if(!t&&"function"==typeof(n=e.toString)&&!i(r=n.call(e)))return r;throw TypeError("Can't convert object to primitive value")}},function(e,t,n){e.exports=n(30)("native-function-to-string",Function.toString)},function(e,t,n){var i=n(75);e.exports=function(e,t,n){if(i(e),void 0===t)return e;switch(n){case 1:return function(n){return e.call(t,n)};case 2:return function(n,i){return e.call(t,n,i)};case 3:return function(n,i,r){return e.call(t,n,i,r)}}return function(){return e.apply(t,arguments)}}},function(e,t){e.exports=function(e){if("function"!=typeof e)throw TypeError(e+" is not a function!");return e}},function(e,t,n){
/*!
  * Bootstrap dropdown.js v4.3.1 (https://getbootstrap.com/)
  * Copyright 2011-2019 The Bootstrap Authors (https://github.com/twbs/bootstrap/graphs/contributors)
  * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
  */
e.exports=function(e,t,n){"use strict";function i(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function a(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{},i=Object.keys(n);"function"==typeof Object.getOwnPropertySymbols&&(i=i.concat(Object.getOwnPropertySymbols(n).filter(function(e){return Object.getOwnPropertyDescriptor(n,e).enumerable}))),i.forEach(function(t){r(e,t,n[t])})}return e}e=e&&e.hasOwnProperty("default")?e.default:e,t=t&&t.hasOwnProperty("default")?t.default:t,n=n&&n.hasOwnProperty("default")?n.default:n;var o="dropdown",s="bs.dropdown",l="."+s,c=e.fn[o],u=new RegExp("38|40|27"),h={HIDE:"hide"+l,HIDDEN:"hidden"+l,SHOW:"show"+l,SHOWN:"shown"+l,CLICK:"click"+l,CLICK_DATA_API:"click.bs.dropdown.data-api",KEYDOWN_DATA_API:"keydown.bs.dropdown.data-api",KEYUP_DATA_API:"keyup.bs.dropdown.data-api"},d={DISABLED:"disabled",SHOW:"show",DROPUP:"dropup",DROPRIGHT:"dropright",DROPLEFT:"dropleft",MENURIGHT:"dropdown-menu-right",MENULEFT:"dropdown-menu-left",POSITION_STATIC:"position-static"},f={DATA_TOGGLE:'[data-toggle="dropdown"]',FORM_CHILD:".dropdown form",MENU:".dropdown-menu",NAVBAR_NAV:".navbar-nav",VISIBLE_ITEMS:".dropdown-menu .dropdown-item:not(.disabled):not(:disabled)"},p={TOP:"top-start",TOPEND:"top-end",BOTTOM:"bottom-start",BOTTOMEND:"bottom-end",RIGHT:"right-start",RIGHTEND:"right-end",LEFT:"left-start",LEFTEND:"left-end"},g={offset:0,flip:!0,boundary:"scrollParent",reference:"toggle",display:"dynamic"},m={offset:"(number|string|function)",flip:"boolean",boundary:"(string|element)",reference:"(string|element)",display:"string"},v=function(){function r(e,t){this._element=e,this._popper=null,this._config=this._getConfig(t),this._menu=this._getMenuElement(),this._inNavbar=this._detectNavbar(),this._addEventListeners()}var c,v,y,b=r.prototype;return b.toggle=function(){if(!this._element.disabled&&!e(this._element).hasClass(d.DISABLED)){var i=r._getParentFromElement(this._element),a=e(this._menu).hasClass(d.SHOW);if(r._clearMenus(),!a){var o={relatedTarget:this._element},s=e.Event(h.SHOW,o);if(e(i).trigger(s),!s.isDefaultPrevented()){if(!this._inNavbar){if(void 0===t)throw new TypeError("Bootstrap's dropdowns require Popper.js (https://popper.js.org/)");var l=this._element;"parent"===this._config.reference?l=i:n.isElement(this._config.reference)&&(l=this._config.reference,void 0!==this._config.reference.jquery&&(l=this._config.reference[0])),"scrollParent"!==this._config.boundary&&e(i).addClass(d.POSITION_STATIC),this._popper=new t(l,this._menu,this._getPopperConfig())}"ontouchstart"in document.documentElement&&0===e(i).closest(f.NAVBAR_NAV).length&&e(document.body).children().on("mouseover",null,e.noop),this._element.focus(),this._element.setAttribute("aria-expanded",!0),e(this._menu).toggleClass(d.SHOW),e(i).toggleClass(d.SHOW).trigger(e.Event(h.SHOWN,o))}}}},b.show=function(){if(!(this._element.disabled||e(this._element).hasClass(d.DISABLED)||e(this._menu).hasClass(d.SHOW))){var t={relatedTarget:this._element},n=e.Event(h.SHOW,t),i=r._getParentFromElement(this._element);e(i).trigger(n),n.isDefaultPrevented()||(e(this._menu).toggleClass(d.SHOW),e(i).toggleClass(d.SHOW).trigger(e.Event(h.SHOWN,t)))}},b.hide=function(){if(!this._element.disabled&&!e(this._element).hasClass(d.DISABLED)&&e(this._menu).hasClass(d.SHOW)){var t={relatedTarget:this._element},n=e.Event(h.HIDE,t),i=r._getParentFromElement(this._element);e(i).trigger(n),n.isDefaultPrevented()||(e(this._menu).toggleClass(d.SHOW),e(i).toggleClass(d.SHOW).trigger(e.Event(h.HIDDEN,t)))}},b.dispose=function(){e.removeData(this._element,s),e(this._element).off(l),this._element=null,this._menu=null,null!==this._popper&&(this._popper.destroy(),this._popper=null)},b.update=function(){this._inNavbar=this._detectNavbar(),null!==this._popper&&this._popper.scheduleUpdate()},b._addEventListeners=function(){var t=this;e(this._element).on(h.CLICK,function(e){e.preventDefault(),e.stopPropagation(),t.toggle()})},b._getConfig=function(t){return t=a({},this.constructor.Default,e(this._element).data(),t),n.typeCheckConfig(o,t,this.constructor.DefaultType),t},b._getMenuElement=function(){if(!this._menu){var e=r._getParentFromElement(this._element);e&&(this._menu=e.querySelector(f.MENU))}return this._menu},b._getPlacement=function(){var t=e(this._element.parentNode),n=p.BOTTOM;return t.hasClass(d.DROPUP)?(n=p.TOP,e(this._menu).hasClass(d.MENURIGHT)&&(n=p.TOPEND)):t.hasClass(d.DROPRIGHT)?n=p.RIGHT:t.hasClass(d.DROPLEFT)?n=p.LEFT:e(this._menu).hasClass(d.MENURIGHT)&&(n=p.BOTTOMEND),n},b._detectNavbar=function(){return e(this._element).closest(".navbar").length>0},b._getOffset=function(){var e=this,t={};return"function"==typeof this._config.offset?t.fn=function(t){return t.offsets=a({},t.offsets,e._config.offset(t.offsets,e._element)||{}),t}:t.offset=this._config.offset,t},b._getPopperConfig=function(){var e={placement:this._getPlacement(),modifiers:{offset:this._getOffset(),flip:{enabled:this._config.flip},preventOverflow:{boundariesElement:this._config.boundary}}};return"static"===this._config.display&&(e.modifiers.applyStyle={enabled:!1}),e},r._jQueryInterface=function(t){return this.each(function(){var n=e(this).data(s),i="object"==typeof t?t:null;if(n||(n=new r(this,i),e(this).data(s,n)),"string"==typeof t){if(void 0===n[t])throw new TypeError('No method named "'+t+'"');n[t]()}})},r._clearMenus=function(t){if(!t||3!==t.which&&("keyup"!==t.type||9===t.which))for(var n=[].slice.call(document.querySelectorAll(f.DATA_TOGGLE)),i=0,a=n.length;i<a;i++){var o=r._getParentFromElement(n[i]),l=e(n[i]).data(s),c={relatedTarget:n[i]};if(t&&"click"===t.type&&(c.clickEvent=t),l){var u=l._menu;if(e(o).hasClass(d.SHOW)&&!(t&&("click"===t.type&&/input|textarea/i.test(t.target.tagName)||"keyup"===t.type&&9===t.which)&&e.contains(o,t.target))){var p=e.Event(h.HIDE,c);e(o).trigger(p),p.isDefaultPrevented()||("ontouchstart"in document.documentElement&&e(document.body).children().off("mouseover",null,e.noop),n[i].setAttribute("aria-expanded","false"),e(u).removeClass(d.SHOW),e(o).removeClass(d.SHOW).trigger(e.Event(h.HIDDEN,c)))}}}},r._getParentFromElement=function(e){var t,i=n.getSelectorFromElement(e);return i&&(t=document.querySelector(i)),t||e.parentNode},r._dataApiKeydownHandler=function(t){if((/input|textarea/i.test(t.target.tagName)?!(32===t.which||27!==t.which&&(40!==t.which&&38!==t.which||e(t.target).closest(f.MENU).length)):u.test(t.which))&&(t.preventDefault(),t.stopPropagation(),!this.disabled&&!e(this).hasClass(d.DISABLED))){var n=r._getParentFromElement(this),i=e(n).hasClass(d.SHOW);if(i&&(!i||27!==t.which&&32!==t.which)){var a=[].slice.call(n.querySelectorAll(f.VISIBLE_ITEMS));if(0!==a.length){var o=a.indexOf(t.target);38===t.which&&o>0&&o--,40===t.which&&o<a.length-1&&o++,o<0&&(o=0),a[o].focus()}}else{if(27===t.which){var s=n.querySelector(f.DATA_TOGGLE);e(s).trigger("focus")}e(this).trigger("click")}}},c=r,y=[{key:"VERSION",get:function(){return"4.3.1"}},{key:"Default",get:function(){return g}},{key:"DefaultType",get:function(){return m}}],(v=null)&&i(c.prototype,v),y&&i(c,y),r}();return e(document).on(h.KEYDOWN_DATA_API,f.DATA_TOGGLE,v._dataApiKeydownHandler).on(h.KEYDOWN_DATA_API,f.MENU,v._dataApiKeydownHandler).on(h.CLICK_DATA_API+" "+h.KEYUP_DATA_API,v._clearMenus).on(h.CLICK_DATA_API,f.DATA_TOGGLE,function(t){t.preventDefault(),t.stopPropagation(),v._jQueryInterface.call(e(this),"toggle")}).on(h.CLICK_DATA_API,f.FORM_CHILD,function(e){e.stopPropagation()}),e.fn[o]=v._jQueryInterface,e.fn[o].Constructor=v,e.fn[o].noConflict=function(){return e.fn[o]=c,v._jQueryInterface},v}(n(8),n(55),n(25))},function(e,t,n){
/*!
  * Bootstrap tooltip.js v4.3.1 (https://getbootstrap.com/)
  * Copyright 2011-2019 The Bootstrap Authors (https://github.com/twbs/bootstrap/graphs/contributors)
  * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
  */
e.exports=function(e,t,n){"use strict";function i(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function r(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function a(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{},i=Object.keys(n);"function"==typeof Object.getOwnPropertySymbols&&(i=i.concat(Object.getOwnPropertySymbols(n).filter(function(e){return Object.getOwnPropertyDescriptor(n,e).enumerable}))),i.forEach(function(t){r(e,t,n[t])})}return e}e=e&&e.hasOwnProperty("default")?e.default:e,t=t&&t.hasOwnProperty("default")?t.default:t,n=n&&n.hasOwnProperty("default")?n.default:n;var o=["background","cite","href","itemtype","longdesc","poster","src","xlink:href"],s={"*":["class","dir","id","lang","role",/^aria-[\w-]*$/i],a:["target","href","title","rel"],area:[],b:[],br:[],col:[],code:[],div:[],em:[],hr:[],h1:[],h2:[],h3:[],h4:[],h5:[],h6:[],i:[],img:["src","alt","title","width","height"],li:[],ol:[],p:[],pre:[],s:[],small:[],span:[],sub:[],sup:[],strong:[],u:[],ul:[]},l=/^(?:(?:https?|mailto|ftp|tel|file):|[^&:\/?#]*(?:[\/?#]|$))/gi,c=/^data:(?:image\/(?:bmp|gif|jpeg|jpg|png|tiff|webp)|video\/(?:mpeg|mp4|ogg|webm)|audio\/(?:mp3|oga|ogg|opus));base64,[a-z0-9+\/]+=*$/i;function u(e,t,n){if(0===e.length)return e;if(n&&"function"==typeof n)return n(e);for(var i=new window.DOMParser,r=i.parseFromString(e,"text/html"),a=Object.keys(t),s=[].slice.call(r.body.querySelectorAll("*")),u=function(e,n){var i=s[e],r=i.nodeName.toLowerCase();if(-1===a.indexOf(i.nodeName.toLowerCase()))return i.parentNode.removeChild(i),"continue";var u=[].slice.call(i.attributes),h=[].concat(t["*"]||[],t[r]||[]);u.forEach(function(e){(function(e,t){var n=e.nodeName.toLowerCase();if(-1!==t.indexOf(n))return-1===o.indexOf(n)||Boolean(e.nodeValue.match(l)||e.nodeValue.match(c));for(var i=t.filter(function(e){return e instanceof RegExp}),r=0,a=i.length;r<a;r++)if(n.match(i[r]))return!0;return!1})(e,h)||i.removeAttribute(e.nodeName)})},h=0,d=s.length;h<d;h++)u(h);return r.body.innerHTML}var h="tooltip",d=".bs.tooltip",f=e.fn[h],p=new RegExp("(^|\\s)bs-tooltip\\S+","g"),g=["sanitize","whiteList","sanitizeFn"],m={animation:"boolean",template:"string",title:"(string|element|function)",trigger:"string",delay:"(number|object)",html:"boolean",selector:"(string|boolean)",placement:"(string|function)",offset:"(number|string|function)",container:"(string|element|boolean)",fallbackPlacement:"(string|array)",boundary:"(string|element)",sanitize:"boolean",sanitizeFn:"(null|function)",whiteList:"object"},v={AUTO:"auto",TOP:"top",RIGHT:"right",BOTTOM:"bottom",LEFT:"left"},y={animation:!0,template:'<div class="tooltip" role="tooltip"><div class="arrow"></div><div class="tooltip-inner"></div></div>',trigger:"hover focus",title:"",delay:0,html:!1,selector:!1,placement:"top",offset:0,container:!1,fallbackPlacement:"flip",boundary:"scrollParent",sanitize:!0,sanitizeFn:null,whiteList:s},b={SHOW:"show",OUT:"out"},x={HIDE:"hide"+d,HIDDEN:"hidden"+d,SHOW:"show"+d,SHOWN:"shown"+d,INSERTED:"inserted"+d,CLICK:"click"+d,FOCUSIN:"focusin"+d,FOCUSOUT:"focusout"+d,MOUSEENTER:"mouseenter"+d,MOUSELEAVE:"mouseleave"+d},w={FADE:"fade",SHOW:"show"},k={TOOLTIP:".tooltip",TOOLTIP_INNER:".tooltip-inner",ARROW:".arrow"},S={HOVER:"hover",FOCUS:"focus",CLICK:"click",MANUAL:"manual"},E=function(){function r(e,n){if(void 0===t)throw new TypeError("Bootstrap's tooltips require Popper.js (https://popper.js.org/)");this._isEnabled=!0,this._timeout=0,this._hoverState="",this._activeTrigger={},this._popper=null,this.element=e,this.config=this._getConfig(n),this.tip=null,this._setListeners()}var o,s,l,c=r.prototype;return c.enable=function(){this._isEnabled=!0},c.disable=function(){this._isEnabled=!1},c.toggleEnabled=function(){this._isEnabled=!this._isEnabled},c.toggle=function(t){if(this._isEnabled)if(t){var n=this.constructor.DATA_KEY,i=e(t.currentTarget).data(n);i||(i=new this.constructor(t.currentTarget,this._getDelegateConfig()),e(t.currentTarget).data(n,i)),i._activeTrigger.click=!i._activeTrigger.click,i._isWithActiveTrigger()?i._enter(null,i):i._leave(null,i)}else{if(e(this.getTipElement()).hasClass(w.SHOW))return void this._leave(null,this);this._enter(null,this)}},c.dispose=function(){clearTimeout(this._timeout),e.removeData(this.element,this.constructor.DATA_KEY),e(this.element).off(this.constructor.EVENT_KEY),e(this.element).closest(".modal").off("hide.bs.modal"),this.tip&&e(this.tip).remove(),this._isEnabled=null,this._timeout=null,this._hoverState=null,this._activeTrigger=null,null!==this._popper&&this._popper.destroy(),this._popper=null,this.element=null,this.config=null,this.tip=null},c.show=function(){var i=this;if("none"===e(this.element).css("display"))throw new Error("Please use show on visible elements");var r=e.Event(this.constructor.Event.SHOW);if(this.isWithContent()&&this._isEnabled){e(this.element).trigger(r);var a=n.findShadowRoot(this.element),o=e.contains(null!==a?a:this.element.ownerDocument.documentElement,this.element);if(r.isDefaultPrevented()||!o)return;var s=this.getTipElement(),l=n.getUID(this.constructor.NAME);s.setAttribute("id",l),this.element.setAttribute("aria-describedby",l),this.setContent(),this.config.animation&&e(s).addClass(w.FADE);var c="function"==typeof this.config.placement?this.config.placement.call(this,s,this.element):this.config.placement,u=this._getAttachment(c);this.addAttachmentClass(u);var h=this._getContainer();e(s).data(this.constructor.DATA_KEY,this),e.contains(this.element.ownerDocument.documentElement,this.tip)||e(s).appendTo(h),e(this.element).trigger(this.constructor.Event.INSERTED),this._popper=new t(this.element,s,{placement:u,modifiers:{offset:this._getOffset(),flip:{behavior:this.config.fallbackPlacement},arrow:{element:k.ARROW},preventOverflow:{boundariesElement:this.config.boundary}},onCreate:function(e){e.originalPlacement!==e.placement&&i._handlePopperPlacementChange(e)},onUpdate:function(e){return i._handlePopperPlacementChange(e)}}),e(s).addClass(w.SHOW),"ontouchstart"in document.documentElement&&e(document.body).children().on("mouseover",null,e.noop);var d=function(){i.config.animation&&i._fixTransition();var t=i._hoverState;i._hoverState=null,e(i.element).trigger(i.constructor.Event.SHOWN),t===b.OUT&&i._leave(null,i)};if(e(this.tip).hasClass(w.FADE)){var f=n.getTransitionDurationFromElement(this.tip);e(this.tip).one(n.TRANSITION_END,d).emulateTransitionEnd(f)}else d()}},c.hide=function(t){var i=this,r=this.getTipElement(),a=e.Event(this.constructor.Event.HIDE),o=function(){i._hoverState!==b.SHOW&&r.parentNode&&r.parentNode.removeChild(r),i._cleanTipClass(),i.element.removeAttribute("aria-describedby"),e(i.element).trigger(i.constructor.Event.HIDDEN),null!==i._popper&&i._popper.destroy(),t&&t()};if(e(this.element).trigger(a),!a.isDefaultPrevented()){if(e(r).removeClass(w.SHOW),"ontouchstart"in document.documentElement&&e(document.body).children().off("mouseover",null,e.noop),this._activeTrigger[S.CLICK]=!1,this._activeTrigger[S.FOCUS]=!1,this._activeTrigger[S.HOVER]=!1,e(this.tip).hasClass(w.FADE)){var s=n.getTransitionDurationFromElement(r);e(r).one(n.TRANSITION_END,o).emulateTransitionEnd(s)}else o();this._hoverState=""}},c.update=function(){null!==this._popper&&this._popper.scheduleUpdate()},c.isWithContent=function(){return Boolean(this.getTitle())},c.addAttachmentClass=function(t){e(this.getTipElement()).addClass("bs-tooltip-"+t)},c.getTipElement=function(){return this.tip=this.tip||e(this.config.template)[0],this.tip},c.setContent=function(){var t=this.getTipElement();this.setElementContent(e(t.querySelectorAll(k.TOOLTIP_INNER)),this.getTitle()),e(t).removeClass(w.FADE+" "+w.SHOW)},c.setElementContent=function(t,n){"object"!=typeof n||!n.nodeType&&!n.jquery?this.config.html?(this.config.sanitize&&(n=u(n,this.config.whiteList,this.config.sanitizeFn)),t.html(n)):t.text(n):this.config.html?e(n).parent().is(t)||t.empty().append(n):t.text(e(n).text())},c.getTitle=function(){var e=this.element.getAttribute("data-original-title");return e||(e="function"==typeof this.config.title?this.config.title.call(this.element):this.config.title),e},c._getOffset=function(){var e=this,t={};return"function"==typeof this.config.offset?t.fn=function(t){return t.offsets=a({},t.offsets,e.config.offset(t.offsets,e.element)||{}),t}:t.offset=this.config.offset,t},c._getContainer=function(){return!1===this.config.container?document.body:n.isElement(this.config.container)?e(this.config.container):e(document).find(this.config.container)},c._getAttachment=function(e){return v[e.toUpperCase()]},c._setListeners=function(){var t=this,n=this.config.trigger.split(" ");n.forEach(function(n){if("click"===n)e(t.element).on(t.constructor.Event.CLICK,t.config.selector,function(e){return t.toggle(e)});else if(n!==S.MANUAL){var i=n===S.HOVER?t.constructor.Event.MOUSEENTER:t.constructor.Event.FOCUSIN,r=n===S.HOVER?t.constructor.Event.MOUSELEAVE:t.constructor.Event.FOCUSOUT;e(t.element).on(i,t.config.selector,function(e){return t._enter(e)}).on(r,t.config.selector,function(e){return t._leave(e)})}}),e(this.element).closest(".modal").on("hide.bs.modal",function(){t.element&&t.hide()}),this.config.selector?this.config=a({},this.config,{trigger:"manual",selector:""}):this._fixTitle()},c._fixTitle=function(){var e=typeof this.element.getAttribute("data-original-title");(this.element.getAttribute("title")||"string"!==e)&&(this.element.setAttribute("data-original-title",this.element.getAttribute("title")||""),this.element.setAttribute("title",""))},c._enter=function(t,n){var i=this.constructor.DATA_KEY;(n=n||e(t.currentTarget).data(i))||(n=new this.constructor(t.currentTarget,this._getDelegateConfig()),e(t.currentTarget).data(i,n)),t&&(n._activeTrigger["focusin"===t.type?S.FOCUS:S.HOVER]=!0),e(n.getTipElement()).hasClass(w.SHOW)||n._hoverState===b.SHOW?n._hoverState=b.SHOW:(clearTimeout(n._timeout),n._hoverState=b.SHOW,n.config.delay&&n.config.delay.show?n._timeout=setTimeout(function(){n._hoverState===b.SHOW&&n.show()},n.config.delay.show):n.show())},c._leave=function(t,n){var i=this.constructor.DATA_KEY;(n=n||e(t.currentTarget).data(i))||(n=new this.constructor(t.currentTarget,this._getDelegateConfig()),e(t.currentTarget).data(i,n)),t&&(n._activeTrigger["focusout"===t.type?S.FOCUS:S.HOVER]=!1),n._isWithActiveTrigger()||(clearTimeout(n._timeout),n._hoverState=b.OUT,n.config.delay&&n.config.delay.hide?n._timeout=setTimeout(function(){n._hoverState===b.OUT&&n.hide()},n.config.delay.hide):n.hide())},c._isWithActiveTrigger=function(){for(var e in this._activeTrigger)if(this._activeTrigger[e])return!0;return!1},c._getConfig=function(t){var i=e(this.element).data();return Object.keys(i).forEach(function(e){-1!==g.indexOf(e)&&delete i[e]}),"number"==typeof(t=a({},this.constructor.Default,i,"object"==typeof t&&t?t:{})).delay&&(t.delay={show:t.delay,hide:t.delay}),"number"==typeof t.title&&(t.title=t.title.toString()),"number"==typeof t.content&&(t.content=t.content.toString()),n.typeCheckConfig(h,t,this.constructor.DefaultType),t.sanitize&&(t.template=u(t.template,t.whiteList,t.sanitizeFn)),t},c._getDelegateConfig=function(){var e={};if(this.config)for(var t in this.config)this.constructor.Default[t]!==this.config[t]&&(e[t]=this.config[t]);return e},c._cleanTipClass=function(){var t=e(this.getTipElement()),n=t.attr("class").match(p);null!==n&&n.length&&t.removeClass(n.join(""))},c._handlePopperPlacementChange=function(e){var t=e.instance;this.tip=t.popper,this._cleanTipClass(),this.addAttachmentClass(this._getAttachment(e.placement))},c._fixTransition=function(){var t=this.getTipElement(),n=this.config.animation;null===t.getAttribute("x-placement")&&(e(t).removeClass(w.FADE),this.config.animation=!1,this.hide(),this.show(),this.config.animation=n)},r._jQueryInterface=function(t){return this.each(function(){var n=e(this).data("bs.tooltip"),i="object"==typeof t&&t;if((n||!/dispose|hide/.test(t))&&(n||(n=new r(this,i),e(this).data("bs.tooltip",n)),"string"==typeof t)){if(void 0===n[t])throw new TypeError('No method named "'+t+'"');n[t]()}})},o=r,l=[{key:"VERSION",get:function(){return"4.3.1"}},{key:"Default",get:function(){return y}},{key:"NAME",get:function(){return h}},{key:"DATA_KEY",get:function(){return"bs.tooltip"}},{key:"Event",get:function(){return x}},{key:"EVENT_KEY",get:function(){return d}},{key:"DefaultType",get:function(){return m}}],(s=null)&&i(o.prototype,s),l&&i(o,l),r}();return e.fn[h]=E._jQueryInterface,e.fn[h].Constructor=E,e.fn[h].noConflict=function(){return e.fn[h]=f,E._jQueryInterface},E}(n(8),n(55),n(25))},function(e,t,n){
/*!
  * Bootstrap modal.js v4.3.1 (https://getbootstrap.com/)
  * Copyright 2011-2019 The Bootstrap Authors (https://github.com/twbs/bootstrap/graphs/contributors)
  * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
  */
e.exports=function(e,t){"use strict";function n(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function i(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function r(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{},r=Object.keys(n);"function"==typeof Object.getOwnPropertySymbols&&(r=r.concat(Object.getOwnPropertySymbols(n).filter(function(e){return Object.getOwnPropertyDescriptor(n,e).enumerable}))),r.forEach(function(t){i(e,t,n[t])})}return e}e=e&&e.hasOwnProperty("default")?e.default:e,t=t&&t.hasOwnProperty("default")?t.default:t;var a="modal",o=".bs.modal",s=e.fn.modal,l={backdrop:!0,keyboard:!0,focus:!0,show:!0},c={backdrop:"(boolean|string)",keyboard:"boolean",focus:"boolean",show:"boolean"},u={HIDE:"hide.bs.modal",HIDDEN:"hidden.bs.modal",SHOW:"show.bs.modal",SHOWN:"shown.bs.modal",FOCUSIN:"focusin.bs.modal",RESIZE:"resize.bs.modal",CLICK_DISMISS:"click.dismiss.bs.modal",KEYDOWN_DISMISS:"keydown.dismiss.bs.modal",MOUSEUP_DISMISS:"mouseup.dismiss.bs.modal",MOUSEDOWN_DISMISS:"mousedown.dismiss.bs.modal",CLICK_DATA_API:"click.bs.modal.data-api"},h={SCROLLABLE:"modal-dialog-scrollable",SCROLLBAR_MEASURER:"modal-scrollbar-measure",BACKDROP:"modal-backdrop",OPEN:"modal-open",FADE:"fade",SHOW:"show"},d={DIALOG:".modal-dialog",MODAL_BODY:".modal-body",DATA_TOGGLE:'[data-toggle="modal"]',DATA_DISMISS:'[data-dismiss="modal"]',FIXED_CONTENT:".fixed-top, .fixed-bottom, .is-fixed, .sticky-top",STICKY_CONTENT:".sticky-top"},f=function(){function i(e,t){this._config=this._getConfig(t),this._element=e,this._dialog=e.querySelector(d.DIALOG),this._backdrop=null,this._isShown=!1,this._isBodyOverflowing=!1,this._ignoreBackdropClick=!1,this._isTransitioning=!1,this._scrollbarWidth=0}var s,f,p,g=i.prototype;return g.toggle=function(e){return this._isShown?this.hide():this.show(e)},g.show=function(t){var n=this;if(!this._isShown&&!this._isTransitioning){e(this._element).hasClass(h.FADE)&&(this._isTransitioning=!0);var i=e.Event(u.SHOW,{relatedTarget:t});e(this._element).trigger(i),this._isShown||i.isDefaultPrevented()||(this._isShown=!0,this._checkScrollbar(),this._setScrollbar(),this._adjustDialog(),this._setEscapeEvent(),this._setResizeEvent(),e(this._element).on(u.CLICK_DISMISS,d.DATA_DISMISS,function(e){return n.hide(e)}),e(this._dialog).on(u.MOUSEDOWN_DISMISS,function(){e(n._element).one(u.MOUSEUP_DISMISS,function(t){e(t.target).is(n._element)&&(n._ignoreBackdropClick=!0)})}),this._showBackdrop(function(){return n._showElement(t)}))}},g.hide=function(n){var i=this;if(n&&n.preventDefault(),this._isShown&&!this._isTransitioning){var r=e.Event(u.HIDE);if(e(this._element).trigger(r),this._isShown&&!r.isDefaultPrevented()){this._isShown=!1;var a=e(this._element).hasClass(h.FADE);if(a&&(this._isTransitioning=!0),this._setEscapeEvent(),this._setResizeEvent(),e(document).off(u.FOCUSIN),e(this._element).removeClass(h.SHOW),e(this._element).off(u.CLICK_DISMISS),e(this._dialog).off(u.MOUSEDOWN_DISMISS),a){var o=t.getTransitionDurationFromElement(this._element);e(this._element).one(t.TRANSITION_END,function(e){return i._hideModal(e)}).emulateTransitionEnd(o)}else this._hideModal()}}},g.dispose=function(){[window,this._element,this._dialog].forEach(function(t){return e(t).off(o)}),e(document).off(u.FOCUSIN),e.removeData(this._element,"bs.modal"),this._config=null,this._element=null,this._dialog=null,this._backdrop=null,this._isShown=null,this._isBodyOverflowing=null,this._ignoreBackdropClick=null,this._isTransitioning=null,this._scrollbarWidth=null},g.handleUpdate=function(){this._adjustDialog()},g._getConfig=function(e){return e=r({},l,e),t.typeCheckConfig(a,e,c),e},g._showElement=function(n){var i=this,r=e(this._element).hasClass(h.FADE);this._element.parentNode&&this._element.parentNode.nodeType===Node.ELEMENT_NODE||document.body.appendChild(this._element),this._element.style.display="block",this._element.removeAttribute("aria-hidden"),this._element.setAttribute("aria-modal",!0),e(this._dialog).hasClass(h.SCROLLABLE)?this._dialog.querySelector(d.MODAL_BODY).scrollTop=0:this._element.scrollTop=0,r&&t.reflow(this._element),e(this._element).addClass(h.SHOW),this._config.focus&&this._enforceFocus();var a=e.Event(u.SHOWN,{relatedTarget:n}),o=function(){i._config.focus&&i._element.focus(),i._isTransitioning=!1,e(i._element).trigger(a)};if(r){var s=t.getTransitionDurationFromElement(this._dialog);e(this._dialog).one(t.TRANSITION_END,o).emulateTransitionEnd(s)}else o()},g._enforceFocus=function(){var t=this;e(document).off(u.FOCUSIN).on(u.FOCUSIN,function(n){document!==n.target&&t._element!==n.target&&0===e(t._element).has(n.target).length&&t._element.focus()})},g._setEscapeEvent=function(){var t=this;this._isShown&&this._config.keyboard?e(this._element).on(u.KEYDOWN_DISMISS,function(e){27===e.which&&(e.preventDefault(),t.hide())}):this._isShown||e(this._element).off(u.KEYDOWN_DISMISS)},g._setResizeEvent=function(){var t=this;this._isShown?e(window).on(u.RESIZE,function(e){return t.handleUpdate(e)}):e(window).off(u.RESIZE)},g._hideModal=function(){var t=this;this._element.style.display="none",this._element.setAttribute("aria-hidden",!0),this._element.removeAttribute("aria-modal"),this._isTransitioning=!1,this._showBackdrop(function(){e(document.body).removeClass(h.OPEN),t._resetAdjustments(),t._resetScrollbar(),e(t._element).trigger(u.HIDDEN)})},g._removeBackdrop=function(){this._backdrop&&(e(this._backdrop).remove(),this._backdrop=null)},g._showBackdrop=function(n){var i=this,r=e(this._element).hasClass(h.FADE)?h.FADE:"";if(this._isShown&&this._config.backdrop){if(this._backdrop=document.createElement("div"),this._backdrop.className=h.BACKDROP,r&&this._backdrop.classList.add(r),e(this._backdrop).appendTo(document.body),e(this._element).on(u.CLICK_DISMISS,function(e){i._ignoreBackdropClick?i._ignoreBackdropClick=!1:e.target===e.currentTarget&&("static"===i._config.backdrop?i._element.focus():i.hide())}),r&&t.reflow(this._backdrop),e(this._backdrop).addClass(h.SHOW),!n)return;if(!r)return void n();var a=t.getTransitionDurationFromElement(this._backdrop);e(this._backdrop).one(t.TRANSITION_END,n).emulateTransitionEnd(a)}else if(!this._isShown&&this._backdrop){e(this._backdrop).removeClass(h.SHOW);var o=function(){i._removeBackdrop(),n&&n()};if(e(this._element).hasClass(h.FADE)){var s=t.getTransitionDurationFromElement(this._backdrop);e(this._backdrop).one(t.TRANSITION_END,o).emulateTransitionEnd(s)}else o()}else n&&n()},g._adjustDialog=function(){var e=this._element.scrollHeight>document.documentElement.clientHeight;!this._isBodyOverflowing&&e&&(this._element.style.paddingLeft=this._scrollbarWidth+"px"),this._isBodyOverflowing&&!e&&(this._element.style.paddingRight=this._scrollbarWidth+"px")},g._resetAdjustments=function(){this._element.style.paddingLeft="",this._element.style.paddingRight=""},g._checkScrollbar=function(){var e=document.body.getBoundingClientRect();this._isBodyOverflowing=e.left+e.right<window.innerWidth,this._scrollbarWidth=this._getScrollbarWidth()},g._setScrollbar=function(){var t=this;if(this._isBodyOverflowing){var n=[].slice.call(document.querySelectorAll(d.FIXED_CONTENT)),i=[].slice.call(document.querySelectorAll(d.STICKY_CONTENT));e(n).each(function(n,i){var r=i.style.paddingRight,a=e(i).css("padding-right");e(i).data("padding-right",r).css("padding-right",parseFloat(a)+t._scrollbarWidth+"px")}),e(i).each(function(n,i){var r=i.style.marginRight,a=e(i).css("margin-right");e(i).data("margin-right",r).css("margin-right",parseFloat(a)-t._scrollbarWidth+"px")});var r=document.body.style.paddingRight,a=e(document.body).css("padding-right");e(document.body).data("padding-right",r).css("padding-right",parseFloat(a)+this._scrollbarWidth+"px")}e(document.body).addClass(h.OPEN)},g._resetScrollbar=function(){var t=[].slice.call(document.querySelectorAll(d.FIXED_CONTENT));e(t).each(function(t,n){var i=e(n).data("padding-right");e(n).removeData("padding-right"),n.style.paddingRight=i||""});var n=[].slice.call(document.querySelectorAll(""+d.STICKY_CONTENT));e(n).each(function(t,n){var i=e(n).data("margin-right");void 0!==i&&e(n).css("margin-right",i).removeData("margin-right")});var i=e(document.body).data("padding-right");e(document.body).removeData("padding-right"),document.body.style.paddingRight=i||""},g._getScrollbarWidth=function(){var e=document.createElement("div");e.className=h.SCROLLBAR_MEASURER,document.body.appendChild(e);var t=e.getBoundingClientRect().width-e.clientWidth;return document.body.removeChild(e),t},i._jQueryInterface=function(t,n){return this.each(function(){var a=e(this).data("bs.modal"),o=r({},l,e(this).data(),"object"==typeof t&&t?t:{});if(a||(a=new i(this,o),e(this).data("bs.modal",a)),"string"==typeof t){if(void 0===a[t])throw new TypeError('No method named "'+t+'"');a[t](n)}else o.show&&a.show(n)})},s=i,p=[{key:"VERSION",get:function(){return"4.3.1"}},{key:"Default",get:function(){return l}}],(f=null)&&n(s.prototype,f),p&&n(s,p),i}();return e(document).on(u.CLICK_DATA_API,d.DATA_TOGGLE,function(n){var i,a=this,o=t.getSelectorFromElement(this);o&&(i=document.querySelector(o));var s=e(i).data("bs.modal")?"toggle":r({},e(i).data(),e(this).data());"A"!==this.tagName&&"AREA"!==this.tagName||n.preventDefault();var l=e(i).one(u.SHOW,function(t){t.isDefaultPrevented()||l.one(u.HIDDEN,function(){e(a).is(":visible")&&a.focus()})});f._jQueryInterface.call(e(i),s,this)}),e.fn.modal=f._jQueryInterface,e.fn.modal.Constructor=f,e.fn.modal.noConflict=function(){return e.fn.modal=s,f._jQueryInterface},f}(n(8),n(25))},function(e,t,n){},function(e,t,n){"use strict";
/** @license React v16.8.6
 * react.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var i=n(36),r="function"==typeof Symbol&&Symbol.for,a=r?Symbol.for("react.element"):60103,o=r?Symbol.for("react.portal"):60106,s=r?Symbol.for("react.fragment"):60107,l=r?Symbol.for("react.strict_mode"):60108,c=r?Symbol.for("react.profiler"):60114,u=r?Symbol.for("react.provider"):60109,h=r?Symbol.for("react.context"):60110,d=r?Symbol.for("react.concurrent_mode"):60111,f=r?Symbol.for("react.forward_ref"):60112,p=r?Symbol.for("react.suspense"):60113,g=r?Symbol.for("react.memo"):60115,m=r?Symbol.for("react.lazy"):60116,v="function"==typeof Symbol&&Symbol.iterator;function y(e){for(var t=arguments.length-1,n="https://reactjs.org/docs/error-decoder.html?invariant="+e,i=0;i<t;i++)n+="&args[]="+encodeURIComponent(arguments[i+1]);!function(e,t,n,i,r,a,o,s){if(!e){if(e=void 0,void 0===t)e=Error("Minified exception occurred; use the non-minified dev environment for the full error message and additional helpful warnings.");else{var l=[n,i,r,a,o,s],c=0;(e=Error(t.replace(/%s/g,function(){return l[c++]}))).name="Invariant Violation"}throw e.framesToPop=1,e}}(!1,"Minified React error #"+e+"; visit %s for the full message or use the non-minified dev environment for full errors and additional helpful warnings. ",n)}var b={isMounted:function(){return!1},enqueueForceUpdate:function(){},enqueueReplaceState:function(){},enqueueSetState:function(){}},x={};function w(e,t,n){this.props=e,this.context=t,this.refs=x,this.updater=n||b}function k(){}function S(e,t,n){this.props=e,this.context=t,this.refs=x,this.updater=n||b}w.prototype.isReactComponent={},w.prototype.setState=function(e,t){"object"!=typeof e&&"function"!=typeof e&&null!=e&&y("85"),this.updater.enqueueSetState(this,e,t,"setState")},w.prototype.forceUpdate=function(e){this.updater.enqueueForceUpdate(this,e,"forceUpdate")},k.prototype=w.prototype;var E=S.prototype=new k;E.constructor=S,i(E,w.prototype),E.isPureReactComponent=!0;var C={current:null},T={current:null},A=Object.prototype.hasOwnProperty,_={key:!0,ref:!0,__self:!0,__source:!0};function O(e,t,n){var i=void 0,r={},o=null,s=null;if(null!=t)for(i in void 0!==t.ref&&(s=t.ref),void 0!==t.key&&(o=""+t.key),t)A.call(t,i)&&!_.hasOwnProperty(i)&&(r[i]=t[i]);var l=arguments.length-2;if(1===l)r.children=n;else if(1<l){for(var c=Array(l),u=0;u<l;u++)c[u]=arguments[u+2];r.children=c}if(e&&e.defaultProps)for(i in l=e.defaultProps)void 0===r[i]&&(r[i]=l[i]);return{$$typeof:a,type:e,key:o,ref:s,props:r,_owner:T.current}}function P(e){return"object"==typeof e&&null!==e&&e.$$typeof===a}var M=/\/+/g,I=[];function D(e,t,n,i){if(I.length){var r=I.pop();return r.result=e,r.keyPrefix=t,r.func=n,r.context=i,r.count=0,r}return{result:e,keyPrefix:t,func:n,context:i,count:0}}function N(e){e.result=null,e.keyPrefix=null,e.func=null,e.context=null,e.count=0,10>I.length&&I.push(e)}function L(e,t,n){return null==e?0:function e(t,n,i,r){var s=typeof t;"undefined"!==s&&"boolean"!==s||(t=null);var l=!1;if(null===t)l=!0;else switch(s){case"string":case"number":l=!0;break;case"object":switch(t.$$typeof){case a:case o:l=!0}}if(l)return i(r,t,""===n?"."+R(t,0):n),1;if(l=0,n=""===n?".":n+":",Array.isArray(t))for(var c=0;c<t.length;c++){var u=n+R(s=t[c],c);l+=e(s,u,i,r)}else if(u=null===t||"object"!=typeof t?null:"function"==typeof(u=v&&t[v]||t["@@iterator"])?u:null,"function"==typeof u)for(t=u.call(t),c=0;!(s=t.next()).done;)l+=e(s=s.value,u=n+R(s,c++),i,r);else"object"===s&&y("31","[object Object]"==(i=""+t)?"object with keys {"+Object.keys(t).join(", ")+"}":i,"");return l}(e,"",t,n)}function R(e,t){return"object"==typeof e&&null!==e&&null!=e.key?function(e){var t={"=":"=0",":":"=2"};return"$"+(""+e).replace(/[=:]/g,function(e){return t[e]})}(e.key):t.toString(36)}function F(e,t){e.func.call(e.context,t,e.count++)}function j(e,t,n){var i=e.result,r=e.keyPrefix;e=e.func.call(e.context,t,e.count++),Array.isArray(e)?z(e,i,n,function(e){return e}):null!=e&&(P(e)&&(e=function(e,t){return{$$typeof:a,type:e.type,key:t,ref:e.ref,props:e.props,_owner:e._owner}}(e,r+(!e.key||t&&t.key===e.key?"":(""+e.key).replace(M,"$&/")+"/")+n)),i.push(e))}function z(e,t,n,i,r){var a="";null!=n&&(a=(""+n).replace(M,"$&/")+"/"),L(e,j,t=D(t,a,i,r)),N(t)}function Y(){var e=C.current;return null===e&&y("321"),e}var H={Children:{map:function(e,t,n){if(null==e)return e;var i=[];return z(e,i,null,t,n),i},forEach:function(e,t,n){if(null==e)return e;L(e,F,t=D(null,null,t,n)),N(t)},count:function(e){return L(e,function(){return null},null)},toArray:function(e){var t=[];return z(e,t,null,function(e){return e}),t},only:function(e){return P(e)||y("143"),e}},createRef:function(){return{current:null}},Component:w,PureComponent:S,createContext:function(e,t){return void 0===t&&(t=null),(e={$$typeof:h,_calculateChangedBits:t,_currentValue:e,_currentValue2:e,_threadCount:0,Provider:null,Consumer:null}).Provider={$$typeof:u,_context:e},e.Consumer=e},forwardRef:function(e){return{$$typeof:f,render:e}},lazy:function(e){return{$$typeof:m,_ctor:e,_status:-1,_result:null}},memo:function(e,t){return{$$typeof:g,type:e,compare:void 0===t?null:t}},useCallback:function(e,t){return Y().useCallback(e,t)},useContext:function(e,t){return Y().useContext(e,t)},useEffect:function(e,t){return Y().useEffect(e,t)},useImperativeHandle:function(e,t,n){return Y().useImperativeHandle(e,t,n)},useDebugValue:function(){},useLayoutEffect:function(e,t){return Y().useLayoutEffect(e,t)},useMemo:function(e,t){return Y().useMemo(e,t)},useReducer:function(e,t,n){return Y().useReducer(e,t,n)},useRef:function(e){return Y().useRef(e)},useState:function(e){return Y().useState(e)},Fragment:s,StrictMode:l,Suspense:p,createElement:O,cloneElement:function(e,t,n){null==e&&y("267",e);var r=void 0,o=i({},e.props),s=e.key,l=e.ref,c=e._owner;if(null!=t){void 0!==t.ref&&(l=t.ref,c=T.current),void 0!==t.key&&(s=""+t.key);var u=void 0;for(r in e.type&&e.type.defaultProps&&(u=e.type.defaultProps),t)A.call(t,r)&&!_.hasOwnProperty(r)&&(o[r]=void 0===t[r]&&void 0!==u?u[r]:t[r])}if(1===(r=arguments.length-2))o.children=n;else if(1<r){u=Array(r);for(var h=0;h<r;h++)u[h]=arguments[h+2];o.children=u}return{$$typeof:a,type:e.type,key:s,ref:l,props:o,_owner:c}},createFactory:function(e){var t=O.bind(null,e);return t.type=e,t},isValidElement:P,version:"16.8.6",unstable_ConcurrentMode:d,unstable_Profiler:c,__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED:{ReactCurrentDispatcher:C,ReactCurrentOwner:T,assign:i}},W={default:H},X=W&&H||W;e.exports=X.default||X},function(e,t,n){"use strict";
/** @license React v16.8.6
 * react-dom.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var i=n(0),r=n(36),a=n(82);function o(e){for(var t=arguments.length-1,n="https://reactjs.org/docs/error-decoder.html?invariant="+e,i=0;i<t;i++)n+="&args[]="+encodeURIComponent(arguments[i+1]);!function(e,t,n,i,r,a,o,s){if(!e){if(e=void 0,void 0===t)e=Error("Minified exception occurred; use the non-minified dev environment for the full error message and additional helpful warnings.");else{var l=[n,i,r,a,o,s],c=0;(e=Error(t.replace(/%s/g,function(){return l[c++]}))).name="Invariant Violation"}throw e.framesToPop=1,e}}(!1,"Minified React error #"+e+"; visit %s for the full message or use the non-minified dev environment for full errors and additional helpful warnings. ",n)}i||o("227");var s=!1,l=null,c=!1,u=null,h={onError:function(e){s=!0,l=e}};function d(e,t,n,i,r,a,o,c,u){s=!1,l=null,function(e,t,n,i,r,a,o,s,l){var c=Array.prototype.slice.call(arguments,3);try{t.apply(n,c)}catch(e){this.onError(e)}}.apply(h,arguments)}var f=null,p={};function g(){if(f)for(var e in p){var t=p[e],n=f.indexOf(e);if(-1<n||o("96",e),!v[n])for(var i in t.extractEvents||o("97",e),v[n]=t,n=t.eventTypes){var r=void 0,a=n[i],s=t,l=i;y.hasOwnProperty(l)&&o("99",l),y[l]=a;var c=a.phasedRegistrationNames;if(c){for(r in c)c.hasOwnProperty(r)&&m(c[r],s,l);r=!0}else a.registrationName?(m(a.registrationName,s,l),r=!0):r=!1;r||o("98",i,e)}}}function m(e,t,n){b[e]&&o("100",e),b[e]=t,x[e]=t.eventTypes[n].dependencies}var v=[],y={},b={},x={},w=null,k=null,S=null;function E(e,t,n){var i=e.type||"unknown-event";e.currentTarget=S(n),function(e,t,n,i,r,a,h,f,p){if(d.apply(this,arguments),s){if(s){var g=l;s=!1,l=null}else o("198"),g=void 0;c||(c=!0,u=g)}}(i,t,void 0,e),e.currentTarget=null}function C(e,t){return null==t&&o("30"),null==e?t:Array.isArray(e)?Array.isArray(t)?(e.push.apply(e,t),e):(e.push(t),e):Array.isArray(t)?[e].concat(t):[e,t]}function T(e,t,n){Array.isArray(e)?e.forEach(t,n):e&&t.call(n,e)}var A=null;function _(e){if(e){var t=e._dispatchListeners,n=e._dispatchInstances;if(Array.isArray(t))for(var i=0;i<t.length&&!e.isPropagationStopped();i++)E(e,t[i],n[i]);else t&&E(e,t,n);e._dispatchListeners=null,e._dispatchInstances=null,e.isPersistent()||e.constructor.release(e)}}var O={injectEventPluginOrder:function(e){f&&o("101"),f=Array.prototype.slice.call(e),g()},injectEventPluginsByName:function(e){var t,n=!1;for(t in e)if(e.hasOwnProperty(t)){var i=e[t];p.hasOwnProperty(t)&&p[t]===i||(p[t]&&o("102",t),p[t]=i,n=!0)}n&&g()}};function P(e,t){var n=e.stateNode;if(!n)return null;var i=w(n);if(!i)return null;n=i[t];e:switch(t){case"onClick":case"onClickCapture":case"onDoubleClick":case"onDoubleClickCapture":case"onMouseDown":case"onMouseDownCapture":case"onMouseMove":case"onMouseMoveCapture":case"onMouseUp":case"onMouseUpCapture":(i=!i.disabled)||(i=!("button"===(e=e.type)||"input"===e||"select"===e||"textarea"===e)),e=!i;break e;default:e=!1}return e?null:(n&&"function"!=typeof n&&o("231",t,typeof n),n)}function M(e){if(null!==e&&(A=C(A,e)),e=A,A=null,e&&(T(e,_),A&&o("95"),c))throw e=u,c=!1,u=null,e}var I=Math.random().toString(36).slice(2),D="__reactInternalInstance$"+I,N="__reactEventHandlers$"+I;function L(e){if(e[D])return e[D];for(;!e[D];){if(!e.parentNode)return null;e=e.parentNode}return 5===(e=e[D]).tag||6===e.tag?e:null}function R(e){return!(e=e[D])||5!==e.tag&&6!==e.tag?null:e}function F(e){if(5===e.tag||6===e.tag)return e.stateNode;o("33")}function j(e){return e[N]||null}function z(e){do{e=e.return}while(e&&5!==e.tag);return e||null}function Y(e,t,n){(t=P(e,n.dispatchConfig.phasedRegistrationNames[t]))&&(n._dispatchListeners=C(n._dispatchListeners,t),n._dispatchInstances=C(n._dispatchInstances,e))}function H(e){if(e&&e.dispatchConfig.phasedRegistrationNames){for(var t=e._targetInst,n=[];t;)n.push(t),t=z(t);for(t=n.length;0<t--;)Y(n[t],"captured",e);for(t=0;t<n.length;t++)Y(n[t],"bubbled",e)}}function W(e,t,n){e&&n&&n.dispatchConfig.registrationName&&(t=P(e,n.dispatchConfig.registrationName))&&(n._dispatchListeners=C(n._dispatchListeners,t),n._dispatchInstances=C(n._dispatchInstances,e))}function X(e){e&&e.dispatchConfig.registrationName&&W(e._targetInst,null,e)}function V(e){T(e,H)}var B=!("undefined"==typeof window||!window.document||!window.document.createElement);function q(e,t){var n={};return n[e.toLowerCase()]=t.toLowerCase(),n["Webkit"+e]="webkit"+t,n["Moz"+e]="moz"+t,n}var U={animationend:q("Animation","AnimationEnd"),animationiteration:q("Animation","AnimationIteration"),animationstart:q("Animation","AnimationStart"),transitionend:q("Transition","TransitionEnd")},G={},Q={};function $(e){if(G[e])return G[e];if(!U[e])return e;var t,n=U[e];for(t in n)if(n.hasOwnProperty(t)&&t in Q)return G[e]=n[t];return e}B&&(Q=document.createElement("div").style,"AnimationEvent"in window||(delete U.animationend.animation,delete U.animationiteration.animation,delete U.animationstart.animation),"TransitionEvent"in window||delete U.transitionend.transition);var Z=$("animationend"),K=$("animationiteration"),J=$("animationstart"),ee=$("transitionend"),te="abort canplay canplaythrough durationchange emptied encrypted ended error loadeddata loadedmetadata loadstart pause play playing progress ratechange seeked seeking stalled suspend timeupdate volumechange waiting".split(" "),ne=null,ie=null,re=null;function ae(){if(re)return re;var e,t,n=ie,i=n.length,r="value"in ne?ne.value:ne.textContent,a=r.length;for(e=0;e<i&&n[e]===r[e];e++);var o=i-e;for(t=1;t<=o&&n[i-t]===r[a-t];t++);return re=r.slice(e,1<t?1-t:void 0)}function oe(){return!0}function se(){return!1}function le(e,t,n,i){for(var r in this.dispatchConfig=e,this._targetInst=t,this.nativeEvent=n,e=this.constructor.Interface)e.hasOwnProperty(r)&&((t=e[r])?this[r]=t(n):"target"===r?this.target=i:this[r]=n[r]);return this.isDefaultPrevented=(null!=n.defaultPrevented?n.defaultPrevented:!1===n.returnValue)?oe:se,this.isPropagationStopped=se,this}function ce(e,t,n,i){if(this.eventPool.length){var r=this.eventPool.pop();return this.call(r,e,t,n,i),r}return new this(e,t,n,i)}function ue(e){e instanceof this||o("279"),e.destructor(),10>this.eventPool.length&&this.eventPool.push(e)}function he(e){e.eventPool=[],e.getPooled=ce,e.release=ue}r(le.prototype,{preventDefault:function(){this.defaultPrevented=!0;var e=this.nativeEvent;e&&(e.preventDefault?e.preventDefault():"unknown"!=typeof e.returnValue&&(e.returnValue=!1),this.isDefaultPrevented=oe)},stopPropagation:function(){var e=this.nativeEvent;e&&(e.stopPropagation?e.stopPropagation():"unknown"!=typeof e.cancelBubble&&(e.cancelBubble=!0),this.isPropagationStopped=oe)},persist:function(){this.isPersistent=oe},isPersistent:se,destructor:function(){var e,t=this.constructor.Interface;for(e in t)this[e]=null;this.nativeEvent=this._targetInst=this.dispatchConfig=null,this.isPropagationStopped=this.isDefaultPrevented=se,this._dispatchInstances=this._dispatchListeners=null}}),le.Interface={type:null,target:null,currentTarget:function(){return null},eventPhase:null,bubbles:null,cancelable:null,timeStamp:function(e){return e.timeStamp||Date.now()},defaultPrevented:null,isTrusted:null},le.extend=function(e){function t(){}function n(){return i.apply(this,arguments)}var i=this;t.prototype=i.prototype;var a=new t;return r(a,n.prototype),n.prototype=a,n.prototype.constructor=n,n.Interface=r({},i.Interface,e),n.extend=i.extend,he(n),n},he(le);var de=le.extend({data:null}),fe=le.extend({data:null}),pe=[9,13,27,32],ge=B&&"CompositionEvent"in window,me=null;B&&"documentMode"in document&&(me=document.documentMode);var ve=B&&"TextEvent"in window&&!me,ye=B&&(!ge||me&&8<me&&11>=me),be=String.fromCharCode(32),xe={beforeInput:{phasedRegistrationNames:{bubbled:"onBeforeInput",captured:"onBeforeInputCapture"},dependencies:["compositionend","keypress","textInput","paste"]},compositionEnd:{phasedRegistrationNames:{bubbled:"onCompositionEnd",captured:"onCompositionEndCapture"},dependencies:"blur compositionend keydown keypress keyup mousedown".split(" ")},compositionStart:{phasedRegistrationNames:{bubbled:"onCompositionStart",captured:"onCompositionStartCapture"},dependencies:"blur compositionstart keydown keypress keyup mousedown".split(" ")},compositionUpdate:{phasedRegistrationNames:{bubbled:"onCompositionUpdate",captured:"onCompositionUpdateCapture"},dependencies:"blur compositionupdate keydown keypress keyup mousedown".split(" ")}},we=!1;function ke(e,t){switch(e){case"keyup":return-1!==pe.indexOf(t.keyCode);case"keydown":return 229!==t.keyCode;case"keypress":case"mousedown":case"blur":return!0;default:return!1}}function Se(e){return"object"==typeof(e=e.detail)&&"data"in e?e.data:null}var Ee=!1;var Ce={eventTypes:xe,extractEvents:function(e,t,n,i){var r=void 0,a=void 0;if(ge)e:{switch(e){case"compositionstart":r=xe.compositionStart;break e;case"compositionend":r=xe.compositionEnd;break e;case"compositionupdate":r=xe.compositionUpdate;break e}r=void 0}else Ee?ke(e,n)&&(r=xe.compositionEnd):"keydown"===e&&229===n.keyCode&&(r=xe.compositionStart);return r?(ye&&"ko"!==n.locale&&(Ee||r!==xe.compositionStart?r===xe.compositionEnd&&Ee&&(a=ae()):(ie="value"in(ne=i)?ne.value:ne.textContent,Ee=!0)),r=de.getPooled(r,t,n,i),a?r.data=a:null!==(a=Se(n))&&(r.data=a),V(r),a=r):a=null,(e=ve?function(e,t){switch(e){case"compositionend":return Se(t);case"keypress":return 32!==t.which?null:(we=!0,be);case"textInput":return(e=t.data)===be&&we?null:e;default:return null}}(e,n):function(e,t){if(Ee)return"compositionend"===e||!ge&&ke(e,t)?(e=ae(),re=ie=ne=null,Ee=!1,e):null;switch(e){case"paste":return null;case"keypress":if(!(t.ctrlKey||t.altKey||t.metaKey)||t.ctrlKey&&t.altKey){if(t.char&&1<t.char.length)return t.char;if(t.which)return String.fromCharCode(t.which)}return null;case"compositionend":return ye&&"ko"!==t.locale?null:t.data;default:return null}}(e,n))?((t=fe.getPooled(xe.beforeInput,t,n,i)).data=e,V(t)):t=null,null===a?t:null===t?a:[a,t]}},Te=null,Ae=null,_e=null;function Oe(e){if(e=k(e)){"function"!=typeof Te&&o("280");var t=w(e.stateNode);Te(e.stateNode,e.type,t)}}function Pe(e){Ae?_e?_e.push(e):_e=[e]:Ae=e}function Me(){if(Ae){var e=Ae,t=_e;if(_e=Ae=null,Oe(e),t)for(e=0;e<t.length;e++)Oe(t[e])}}function Ie(e,t){return e(t)}function De(e,t,n){return e(t,n)}function Ne(){}var Le=!1;function Re(e,t){if(Le)return e(t);Le=!0;try{return Ie(e,t)}finally{Le=!1,(null!==Ae||null!==_e)&&(Ne(),Me())}}var Fe={color:!0,date:!0,datetime:!0,"datetime-local":!0,email:!0,month:!0,number:!0,password:!0,range:!0,search:!0,tel:!0,text:!0,time:!0,url:!0,week:!0};function je(e){var t=e&&e.nodeName&&e.nodeName.toLowerCase();return"input"===t?!!Fe[e.type]:"textarea"===t}function ze(e){return(e=e.target||e.srcElement||window).correspondingUseElement&&(e=e.correspondingUseElement),3===e.nodeType?e.parentNode:e}function Ye(e){if(!B)return!1;var t=(e="on"+e)in document;return t||((t=document.createElement("div")).setAttribute(e,"return;"),t="function"==typeof t[e]),t}function He(e){var t=e.type;return(e=e.nodeName)&&"input"===e.toLowerCase()&&("checkbox"===t||"radio"===t)}function We(e){e._valueTracker||(e._valueTracker=function(e){var t=He(e)?"checked":"value",n=Object.getOwnPropertyDescriptor(e.constructor.prototype,t),i=""+e[t];if(!e.hasOwnProperty(t)&&void 0!==n&&"function"==typeof n.get&&"function"==typeof n.set){var r=n.get,a=n.set;return Object.defineProperty(e,t,{configurable:!0,get:function(){return r.call(this)},set:function(e){i=""+e,a.call(this,e)}}),Object.defineProperty(e,t,{enumerable:n.enumerable}),{getValue:function(){return i},setValue:function(e){i=""+e},stopTracking:function(){e._valueTracker=null,delete e[t]}}}}(e))}function Xe(e){if(!e)return!1;var t=e._valueTracker;if(!t)return!0;var n=t.getValue(),i="";return e&&(i=He(e)?e.checked?"true":"false":e.value),(e=i)!==n&&(t.setValue(e),!0)}var Ve=i.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED;Ve.hasOwnProperty("ReactCurrentDispatcher")||(Ve.ReactCurrentDispatcher={current:null});var Be=/^(.*)[\\\/]/,qe="function"==typeof Symbol&&Symbol.for,Ue=qe?Symbol.for("react.element"):60103,Ge=qe?Symbol.for("react.portal"):60106,Qe=qe?Symbol.for("react.fragment"):60107,$e=qe?Symbol.for("react.strict_mode"):60108,Ze=qe?Symbol.for("react.profiler"):60114,Ke=qe?Symbol.for("react.provider"):60109,Je=qe?Symbol.for("react.context"):60110,et=qe?Symbol.for("react.concurrent_mode"):60111,tt=qe?Symbol.for("react.forward_ref"):60112,nt=qe?Symbol.for("react.suspense"):60113,it=qe?Symbol.for("react.memo"):60115,rt=qe?Symbol.for("react.lazy"):60116,at="function"==typeof Symbol&&Symbol.iterator;function ot(e){return null===e||"object"!=typeof e?null:"function"==typeof(e=at&&e[at]||e["@@iterator"])?e:null}function st(e){if(null==e)return null;if("function"==typeof e)return e.displayName||e.name||null;if("string"==typeof e)return e;switch(e){case et:return"ConcurrentMode";case Qe:return"Fragment";case Ge:return"Portal";case Ze:return"Profiler";case $e:return"StrictMode";case nt:return"Suspense"}if("object"==typeof e)switch(e.$$typeof){case Je:return"Context.Consumer";case Ke:return"Context.Provider";case tt:var t=e.render;return t=t.displayName||t.name||"",e.displayName||(""!==t?"ForwardRef("+t+")":"ForwardRef");case it:return st(e.type);case rt:if(e=1===e._status?e._result:null)return st(e)}return null}function lt(e){var t="";do{e:switch(e.tag){case 3:case 4:case 6:case 7:case 10:case 9:var n="";break e;default:var i=e._debugOwner,r=e._debugSource,a=st(e.type);n=null,i&&(n=st(i.type)),i=a,a="",r?a=" (at "+r.fileName.replace(Be,"")+":"+r.lineNumber+")":n&&(a=" (created by "+n+")"),n="\n    in "+(i||"Unknown")+a}t+=n,e=e.return}while(e);return t}var ct=/^[:A-Z_a-z\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u02FF\u0370-\u037D\u037F-\u1FFF\u200C-\u200D\u2070-\u218F\u2C00-\u2FEF\u3001-\uD7FF\uF900-\uFDCF\uFDF0-\uFFFD][:A-Z_a-z\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u02FF\u0370-\u037D\u037F-\u1FFF\u200C-\u200D\u2070-\u218F\u2C00-\u2FEF\u3001-\uD7FF\uF900-\uFDCF\uFDF0-\uFFFD\-.0-9\u00B7\u0300-\u036F\u203F-\u2040]*$/,ut=Object.prototype.hasOwnProperty,ht={},dt={};function ft(e,t,n,i,r){this.acceptsBooleans=2===t||3===t||4===t,this.attributeName=i,this.attributeNamespace=r,this.mustUseProperty=n,this.propertyName=e,this.type=t}var pt={};"children dangerouslySetInnerHTML defaultValue defaultChecked innerHTML suppressContentEditableWarning suppressHydrationWarning style".split(" ").forEach(function(e){pt[e]=new ft(e,0,!1,e,null)}),[["acceptCharset","accept-charset"],["className","class"],["htmlFor","for"],["httpEquiv","http-equiv"]].forEach(function(e){var t=e[0];pt[t]=new ft(t,1,!1,e[1],null)}),["contentEditable","draggable","spellCheck","value"].forEach(function(e){pt[e]=new ft(e,2,!1,e.toLowerCase(),null)}),["autoReverse","externalResourcesRequired","focusable","preserveAlpha"].forEach(function(e){pt[e]=new ft(e,2,!1,e,null)}),"allowFullScreen async autoFocus autoPlay controls default defer disabled formNoValidate hidden loop noModule noValidate open playsInline readOnly required reversed scoped seamless itemScope".split(" ").forEach(function(e){pt[e]=new ft(e,3,!1,e.toLowerCase(),null)}),["checked","multiple","muted","selected"].forEach(function(e){pt[e]=new ft(e,3,!0,e,null)}),["capture","download"].forEach(function(e){pt[e]=new ft(e,4,!1,e,null)}),["cols","rows","size","span"].forEach(function(e){pt[e]=new ft(e,6,!1,e,null)}),["rowSpan","start"].forEach(function(e){pt[e]=new ft(e,5,!1,e.toLowerCase(),null)});var gt=/[\-:]([a-z])/g;function mt(e){return e[1].toUpperCase()}function vt(e,t,n,i){var r=pt.hasOwnProperty(t)?pt[t]:null;(null!==r?0===r.type:!i&&(2<t.length&&("o"===t[0]||"O"===t[0])&&("n"===t[1]||"N"===t[1])))||(function(e,t,n,i){if(null==t||function(e,t,n,i){if(null!==n&&0===n.type)return!1;switch(typeof t){case"function":case"symbol":return!0;case"boolean":return!i&&(null!==n?!n.acceptsBooleans:"data-"!==(e=e.toLowerCase().slice(0,5))&&"aria-"!==e);default:return!1}}(e,t,n,i))return!0;if(i)return!1;if(null!==n)switch(n.type){case 3:return!t;case 4:return!1===t;case 5:return isNaN(t);case 6:return isNaN(t)||1>t}return!1}(t,n,r,i)&&(n=null),i||null===r?function(e){return!!ut.call(dt,e)||!ut.call(ht,e)&&(ct.test(e)?dt[e]=!0:(ht[e]=!0,!1))}(t)&&(null===n?e.removeAttribute(t):e.setAttribute(t,""+n)):r.mustUseProperty?e[r.propertyName]=null===n?3!==r.type&&"":n:(t=r.attributeName,i=r.attributeNamespace,null===n?e.removeAttribute(t):(n=3===(r=r.type)||4===r&&!0===n?"":""+n,i?e.setAttributeNS(i,t,n):e.setAttribute(t,n))))}function yt(e){switch(typeof e){case"boolean":case"number":case"object":case"string":case"undefined":return e;default:return""}}function bt(e,t){var n=t.checked;return r({},t,{defaultChecked:void 0,defaultValue:void 0,value:void 0,checked:null!=n?n:e._wrapperState.initialChecked})}function xt(e,t){var n=null==t.defaultValue?"":t.defaultValue,i=null!=t.checked?t.checked:t.defaultChecked;n=yt(null!=t.value?t.value:n),e._wrapperState={initialChecked:i,initialValue:n,controlled:"checkbox"===t.type||"radio"===t.type?null!=t.checked:null!=t.value}}function wt(e,t){null!=(t=t.checked)&&vt(e,"checked",t,!1)}function kt(e,t){wt(e,t);var n=yt(t.value),i=t.type;if(null!=n)"number"===i?(0===n&&""===e.value||e.value!=n)&&(e.value=""+n):e.value!==""+n&&(e.value=""+n);else if("submit"===i||"reset"===i)return void e.removeAttribute("value");t.hasOwnProperty("value")?Et(e,t.type,n):t.hasOwnProperty("defaultValue")&&Et(e,t.type,yt(t.defaultValue)),null==t.checked&&null!=t.defaultChecked&&(e.defaultChecked=!!t.defaultChecked)}function St(e,t,n){if(t.hasOwnProperty("value")||t.hasOwnProperty("defaultValue")){var i=t.type;if(!("submit"!==i&&"reset"!==i||void 0!==t.value&&null!==t.value))return;t=""+e._wrapperState.initialValue,n||t===e.value||(e.value=t),e.defaultValue=t}""!==(n=e.name)&&(e.name=""),e.defaultChecked=!e.defaultChecked,e.defaultChecked=!!e._wrapperState.initialChecked,""!==n&&(e.name=n)}function Et(e,t,n){"number"===t&&e.ownerDocument.activeElement===e||(null==n?e.defaultValue=""+e._wrapperState.initialValue:e.defaultValue!==""+n&&(e.defaultValue=""+n))}"accent-height alignment-baseline arabic-form baseline-shift cap-height clip-path clip-rule color-interpolation color-interpolation-filters color-profile color-rendering dominant-baseline enable-background fill-opacity fill-rule flood-color flood-opacity font-family font-size font-size-adjust font-stretch font-style font-variant font-weight glyph-name glyph-orientation-horizontal glyph-orientation-vertical horiz-adv-x horiz-origin-x image-rendering letter-spacing lighting-color marker-end marker-mid marker-start overline-position overline-thickness paint-order panose-1 pointer-events rendering-intent shape-rendering stop-color stop-opacity strikethrough-position strikethrough-thickness stroke-dasharray stroke-dashoffset stroke-linecap stroke-linejoin stroke-miterlimit stroke-opacity stroke-width text-anchor text-decoration text-rendering underline-position underline-thickness unicode-bidi unicode-range units-per-em v-alphabetic v-hanging v-ideographic v-mathematical vector-effect vert-adv-y vert-origin-x vert-origin-y word-spacing writing-mode xmlns:xlink x-height".split(" ").forEach(function(e){var t=e.replace(gt,mt);pt[t]=new ft(t,1,!1,e,null)}),"xlink:actuate xlink:arcrole xlink:href xlink:role xlink:show xlink:title xlink:type".split(" ").forEach(function(e){var t=e.replace(gt,mt);pt[t]=new ft(t,1,!1,e,"http://www.w3.org/1999/xlink")}),["xml:base","xml:lang","xml:space"].forEach(function(e){var t=e.replace(gt,mt);pt[t]=new ft(t,1,!1,e,"http://www.w3.org/XML/1998/namespace")}),["tabIndex","crossOrigin"].forEach(function(e){pt[e]=new ft(e,1,!1,e.toLowerCase(),null)});var Ct={change:{phasedRegistrationNames:{bubbled:"onChange",captured:"onChangeCapture"},dependencies:"blur change click focus input keydown keyup selectionchange".split(" ")}};function Tt(e,t,n){return(e=le.getPooled(Ct.change,e,t,n)).type="change",Pe(n),V(e),e}var At=null,_t=null;function Ot(e){M(e)}function Pt(e){if(Xe(F(e)))return e}function Mt(e,t){if("change"===e)return t}var It=!1;function Dt(){At&&(At.detachEvent("onpropertychange",Nt),_t=At=null)}function Nt(e){"value"===e.propertyName&&Pt(_t)&&Re(Ot,e=Tt(_t,e,ze(e)))}function Lt(e,t,n){"focus"===e?(Dt(),_t=n,(At=t).attachEvent("onpropertychange",Nt)):"blur"===e&&Dt()}function Rt(e){if("selectionchange"===e||"keyup"===e||"keydown"===e)return Pt(_t)}function Ft(e,t){if("click"===e)return Pt(t)}function jt(e,t){if("input"===e||"change"===e)return Pt(t)}B&&(It=Ye("input")&&(!document.documentMode||9<document.documentMode));var zt={eventTypes:Ct,_isInputEventSupported:It,extractEvents:function(e,t,n,i){var r=t?F(t):window,a=void 0,o=void 0,s=r.nodeName&&r.nodeName.toLowerCase();if("select"===s||"input"===s&&"file"===r.type?a=Mt:je(r)?It?a=jt:(a=Rt,o=Lt):(s=r.nodeName)&&"input"===s.toLowerCase()&&("checkbox"===r.type||"radio"===r.type)&&(a=Ft),a&&(a=a(e,t)))return Tt(a,n,i);o&&o(e,r,t),"blur"===e&&(e=r._wrapperState)&&e.controlled&&"number"===r.type&&Et(r,"number",r.value)}},Yt=le.extend({view:null,detail:null}),Ht={Alt:"altKey",Control:"ctrlKey",Meta:"metaKey",Shift:"shiftKey"};function Wt(e){var t=this.nativeEvent;return t.getModifierState?t.getModifierState(e):!!(e=Ht[e])&&!!t[e]}function Xt(){return Wt}var Vt=0,Bt=0,qt=!1,Ut=!1,Gt=Yt.extend({screenX:null,screenY:null,clientX:null,clientY:null,pageX:null,pageY:null,ctrlKey:null,shiftKey:null,altKey:null,metaKey:null,getModifierState:Xt,button:null,buttons:null,relatedTarget:function(e){return e.relatedTarget||(e.fromElement===e.srcElement?e.toElement:e.fromElement)},movementX:function(e){if("movementX"in e)return e.movementX;var t=Vt;return Vt=e.screenX,qt?"mousemove"===e.type?e.screenX-t:0:(qt=!0,0)},movementY:function(e){if("movementY"in e)return e.movementY;var t=Bt;return Bt=e.screenY,Ut?"mousemove"===e.type?e.screenY-t:0:(Ut=!0,0)}}),Qt=Gt.extend({pointerId:null,width:null,height:null,pressure:null,tangentialPressure:null,tiltX:null,tiltY:null,twist:null,pointerType:null,isPrimary:null}),$t={mouseEnter:{registrationName:"onMouseEnter",dependencies:["mouseout","mouseover"]},mouseLeave:{registrationName:"onMouseLeave",dependencies:["mouseout","mouseover"]},pointerEnter:{registrationName:"onPointerEnter",dependencies:["pointerout","pointerover"]},pointerLeave:{registrationName:"onPointerLeave",dependencies:["pointerout","pointerover"]}},Zt={eventTypes:$t,extractEvents:function(e,t,n,i){var r="mouseover"===e||"pointerover"===e,a="mouseout"===e||"pointerout"===e;if(r&&(n.relatedTarget||n.fromElement)||!a&&!r)return null;if(r=i.window===i?i:(r=i.ownerDocument)?r.defaultView||r.parentWindow:window,a?(a=t,t=(t=n.relatedTarget||n.toElement)?L(t):null):a=null,a===t)return null;var o=void 0,s=void 0,l=void 0,c=void 0;"mouseout"===e||"mouseover"===e?(o=Gt,s=$t.mouseLeave,l=$t.mouseEnter,c="mouse"):"pointerout"!==e&&"pointerover"!==e||(o=Qt,s=$t.pointerLeave,l=$t.pointerEnter,c="pointer");var u=null==a?r:F(a);if(r=null==t?r:F(t),(e=o.getPooled(s,a,n,i)).type=c+"leave",e.target=u,e.relatedTarget=r,(n=o.getPooled(l,t,n,i)).type=c+"enter",n.target=r,n.relatedTarget=u,i=t,a&&i)e:{for(r=i,c=0,o=t=a;o;o=z(o))c++;for(o=0,l=r;l;l=z(l))o++;for(;0<c-o;)t=z(t),c--;for(;0<o-c;)r=z(r),o--;for(;c--;){if(t===r||t===r.alternate)break e;t=z(t),r=z(r)}t=null}else t=null;for(r=t,t=[];a&&a!==r&&(null===(c=a.alternate)||c!==r);)t.push(a),a=z(a);for(a=[];i&&i!==r&&(null===(c=i.alternate)||c!==r);)a.push(i),i=z(i);for(i=0;i<t.length;i++)W(t[i],"bubbled",e);for(i=a.length;0<i--;)W(a[i],"captured",n);return[e,n]}};function Kt(e,t){return e===t&&(0!==e||1/e==1/t)||e!=e&&t!=t}var Jt=Object.prototype.hasOwnProperty;function en(e,t){if(Kt(e,t))return!0;if("object"!=typeof e||null===e||"object"!=typeof t||null===t)return!1;var n=Object.keys(e),i=Object.keys(t);if(n.length!==i.length)return!1;for(i=0;i<n.length;i++)if(!Jt.call(t,n[i])||!Kt(e[n[i]],t[n[i]]))return!1;return!0}function tn(e){var t=e;if(e.alternate)for(;t.return;)t=t.return;else{if(0!=(2&t.effectTag))return 1;for(;t.return;)if(0!=(2&(t=t.return).effectTag))return 1}return 3===t.tag?2:3}function nn(e){2!==tn(e)&&o("188")}function rn(e){if(!(e=function(e){var t=e.alternate;if(!t)return 3===(t=tn(e))&&o("188"),1===t?null:e;for(var n=e,i=t;;){var r=n.return,a=r?r.alternate:null;if(!r||!a)break;if(r.child===a.child){for(var s=r.child;s;){if(s===n)return nn(r),e;if(s===i)return nn(r),t;s=s.sibling}o("188")}if(n.return!==i.return)n=r,i=a;else{s=!1;for(var l=r.child;l;){if(l===n){s=!0,n=r,i=a;break}if(l===i){s=!0,i=r,n=a;break}l=l.sibling}if(!s){for(l=a.child;l;){if(l===n){s=!0,n=a,i=r;break}if(l===i){s=!0,i=a,n=r;break}l=l.sibling}s||o("189")}}n.alternate!==i&&o("190")}return 3!==n.tag&&o("188"),n.stateNode.current===n?e:t}(e)))return null;for(var t=e;;){if(5===t.tag||6===t.tag)return t;if(t.child)t.child.return=t,t=t.child;else{if(t===e)break;for(;!t.sibling;){if(!t.return||t.return===e)return null;t=t.return}t.sibling.return=t.return,t=t.sibling}}return null}var an=le.extend({animationName:null,elapsedTime:null,pseudoElement:null}),on=le.extend({clipboardData:function(e){return"clipboardData"in e?e.clipboardData:window.clipboardData}}),sn=Yt.extend({relatedTarget:null});function ln(e){var t=e.keyCode;return"charCode"in e?0===(e=e.charCode)&&13===t&&(e=13):e=t,10===e&&(e=13),32<=e||13===e?e:0}var cn={Esc:"Escape",Spacebar:" ",Left:"ArrowLeft",Up:"ArrowUp",Right:"ArrowRight",Down:"ArrowDown",Del:"Delete",Win:"OS",Menu:"ContextMenu",Apps:"ContextMenu",Scroll:"ScrollLock",MozPrintableKey:"Unidentified"},un={8:"Backspace",9:"Tab",12:"Clear",13:"Enter",16:"Shift",17:"Control",18:"Alt",19:"Pause",20:"CapsLock",27:"Escape",32:" ",33:"PageUp",34:"PageDown",35:"End",36:"Home",37:"ArrowLeft",38:"ArrowUp",39:"ArrowRight",40:"ArrowDown",45:"Insert",46:"Delete",112:"F1",113:"F2",114:"F3",115:"F4",116:"F5",117:"F6",118:"F7",119:"F8",120:"F9",121:"F10",122:"F11",123:"F12",144:"NumLock",145:"ScrollLock",224:"Meta"},hn=Yt.extend({key:function(e){if(e.key){var t=cn[e.key]||e.key;if("Unidentified"!==t)return t}return"keypress"===e.type?13===(e=ln(e))?"Enter":String.fromCharCode(e):"keydown"===e.type||"keyup"===e.type?un[e.keyCode]||"Unidentified":""},location:null,ctrlKey:null,shiftKey:null,altKey:null,metaKey:null,repeat:null,locale:null,getModifierState:Xt,charCode:function(e){return"keypress"===e.type?ln(e):0},keyCode:function(e){return"keydown"===e.type||"keyup"===e.type?e.keyCode:0},which:function(e){return"keypress"===e.type?ln(e):"keydown"===e.type||"keyup"===e.type?e.keyCode:0}}),dn=Gt.extend({dataTransfer:null}),fn=Yt.extend({touches:null,targetTouches:null,changedTouches:null,altKey:null,metaKey:null,ctrlKey:null,shiftKey:null,getModifierState:Xt}),pn=le.extend({propertyName:null,elapsedTime:null,pseudoElement:null}),gn=Gt.extend({deltaX:function(e){return"deltaX"in e?e.deltaX:"wheelDeltaX"in e?-e.wheelDeltaX:0},deltaY:function(e){return"deltaY"in e?e.deltaY:"wheelDeltaY"in e?-e.wheelDeltaY:"wheelDelta"in e?-e.wheelDelta:0},deltaZ:null,deltaMode:null}),mn=[["abort","abort"],[Z,"animationEnd"],[K,"animationIteration"],[J,"animationStart"],["canplay","canPlay"],["canplaythrough","canPlayThrough"],["drag","drag"],["dragenter","dragEnter"],["dragexit","dragExit"],["dragleave","dragLeave"],["dragover","dragOver"],["durationchange","durationChange"],["emptied","emptied"],["encrypted","encrypted"],["ended","ended"],["error","error"],["gotpointercapture","gotPointerCapture"],["load","load"],["loadeddata","loadedData"],["loadedmetadata","loadedMetadata"],["loadstart","loadStart"],["lostpointercapture","lostPointerCapture"],["mousemove","mouseMove"],["mouseout","mouseOut"],["mouseover","mouseOver"],["playing","playing"],["pointermove","pointerMove"],["pointerout","pointerOut"],["pointerover","pointerOver"],["progress","progress"],["scroll","scroll"],["seeking","seeking"],["stalled","stalled"],["suspend","suspend"],["timeupdate","timeUpdate"],["toggle","toggle"],["touchmove","touchMove"],[ee,"transitionEnd"],["waiting","waiting"],["wheel","wheel"]],vn={},yn={};function bn(e,t){var n=e[0],i="on"+((e=e[1])[0].toUpperCase()+e.slice(1));t={phasedRegistrationNames:{bubbled:i,captured:i+"Capture"},dependencies:[n],isInteractive:t},vn[e]=t,yn[n]=t}[["blur","blur"],["cancel","cancel"],["click","click"],["close","close"],["contextmenu","contextMenu"],["copy","copy"],["cut","cut"],["auxclick","auxClick"],["dblclick","doubleClick"],["dragend","dragEnd"],["dragstart","dragStart"],["drop","drop"],["focus","focus"],["input","input"],["invalid","invalid"],["keydown","keyDown"],["keypress","keyPress"],["keyup","keyUp"],["mousedown","mouseDown"],["mouseup","mouseUp"],["paste","paste"],["pause","pause"],["play","play"],["pointercancel","pointerCancel"],["pointerdown","pointerDown"],["pointerup","pointerUp"],["ratechange","rateChange"],["reset","reset"],["seeked","seeked"],["submit","submit"],["touchcancel","touchCancel"],["touchend","touchEnd"],["touchstart","touchStart"],["volumechange","volumeChange"]].forEach(function(e){bn(e,!0)}),mn.forEach(function(e){bn(e,!1)});var xn={eventTypes:vn,isInteractiveTopLevelEventType:function(e){return void 0!==(e=yn[e])&&!0===e.isInteractive},extractEvents:function(e,t,n,i){var r=yn[e];if(!r)return null;switch(e){case"keypress":if(0===ln(n))return null;case"keydown":case"keyup":e=hn;break;case"blur":case"focus":e=sn;break;case"click":if(2===n.button)return null;case"auxclick":case"dblclick":case"mousedown":case"mousemove":case"mouseup":case"mouseout":case"mouseover":case"contextmenu":e=Gt;break;case"drag":case"dragend":case"dragenter":case"dragexit":case"dragleave":case"dragover":case"dragstart":case"drop":e=dn;break;case"touchcancel":case"touchend":case"touchmove":case"touchstart":e=fn;break;case Z:case K:case J:e=an;break;case ee:e=pn;break;case"scroll":e=Yt;break;case"wheel":e=gn;break;case"copy":case"cut":case"paste":e=on;break;case"gotpointercapture":case"lostpointercapture":case"pointercancel":case"pointerdown":case"pointermove":case"pointerout":case"pointerover":case"pointerup":e=Qt;break;default:e=le}return V(t=e.getPooled(r,t,n,i)),t}},wn=xn.isInteractiveTopLevelEventType,kn=[];function Sn(e){var t=e.targetInst,n=t;do{if(!n){e.ancestors.push(n);break}var i;for(i=n;i.return;)i=i.return;if(!(i=3!==i.tag?null:i.stateNode.containerInfo))break;e.ancestors.push(n),n=L(i)}while(n);for(n=0;n<e.ancestors.length;n++){t=e.ancestors[n];var r=ze(e.nativeEvent);i=e.topLevelType;for(var a=e.nativeEvent,o=null,s=0;s<v.length;s++){var l=v[s];l&&(l=l.extractEvents(i,t,a,r))&&(o=C(o,l))}M(o)}}var En=!0;function Cn(e,t){if(!t)return null;var n=(wn(e)?An:_n).bind(null,e);t.addEventListener(e,n,!1)}function Tn(e,t){if(!t)return null;var n=(wn(e)?An:_n).bind(null,e);t.addEventListener(e,n,!0)}function An(e,t){De(_n,e,t)}function _n(e,t){if(En){var n=ze(t);if(null===(n=L(n))||"number"!=typeof n.tag||2===tn(n)||(n=null),kn.length){var i=kn.pop();i.topLevelType=e,i.nativeEvent=t,i.targetInst=n,e=i}else e={topLevelType:e,nativeEvent:t,targetInst:n,ancestors:[]};try{Re(Sn,e)}finally{e.topLevelType=null,e.nativeEvent=null,e.targetInst=null,e.ancestors.length=0,10>kn.length&&kn.push(e)}}}var On={},Pn=0,Mn="_reactListenersID"+(""+Math.random()).slice(2);function In(e){return Object.prototype.hasOwnProperty.call(e,Mn)||(e[Mn]=Pn++,On[e[Mn]]={}),On[e[Mn]]}function Dn(e){if(void 0===(e=e||("undefined"!=typeof document?document:void 0)))return null;try{return e.activeElement||e.body}catch(t){return e.body}}function Nn(e){for(;e&&e.firstChild;)e=e.firstChild;return e}function Ln(e,t){var n,i=Nn(e);for(e=0;i;){if(3===i.nodeType){if(n=e+i.textContent.length,e<=t&&n>=t)return{node:i,offset:t-e};e=n}e:{for(;i;){if(i.nextSibling){i=i.nextSibling;break e}i=i.parentNode}i=void 0}i=Nn(i)}}function Rn(){for(var e=window,t=Dn();t instanceof e.HTMLIFrameElement;){try{var n="string"==typeof t.contentWindow.location.href}catch(e){n=!1}if(!n)break;t=Dn((e=t.contentWindow).document)}return t}function Fn(e){var t=e&&e.nodeName&&e.nodeName.toLowerCase();return t&&("input"===t&&("text"===e.type||"search"===e.type||"tel"===e.type||"url"===e.type||"password"===e.type)||"textarea"===t||"true"===e.contentEditable)}function jn(e){var t=Rn(),n=e.focusedElem,i=e.selectionRange;if(t!==n&&n&&n.ownerDocument&&function e(t,n){return!(!t||!n)&&(t===n||(!t||3!==t.nodeType)&&(n&&3===n.nodeType?e(t,n.parentNode):"contains"in t?t.contains(n):!!t.compareDocumentPosition&&!!(16&t.compareDocumentPosition(n))))}(n.ownerDocument.documentElement,n)){if(null!==i&&Fn(n))if(t=i.start,void 0===(e=i.end)&&(e=t),"selectionStart"in n)n.selectionStart=t,n.selectionEnd=Math.min(e,n.value.length);else if((e=(t=n.ownerDocument||document)&&t.defaultView||window).getSelection){e=e.getSelection();var r=n.textContent.length,a=Math.min(i.start,r);i=void 0===i.end?a:Math.min(i.end,r),!e.extend&&a>i&&(r=i,i=a,a=r),r=Ln(n,a);var o=Ln(n,i);r&&o&&(1!==e.rangeCount||e.anchorNode!==r.node||e.anchorOffset!==r.offset||e.focusNode!==o.node||e.focusOffset!==o.offset)&&((t=t.createRange()).setStart(r.node,r.offset),e.removeAllRanges(),a>i?(e.addRange(t),e.extend(o.node,o.offset)):(t.setEnd(o.node,o.offset),e.addRange(t)))}for(t=[],e=n;e=e.parentNode;)1===e.nodeType&&t.push({element:e,left:e.scrollLeft,top:e.scrollTop});for("function"==typeof n.focus&&n.focus(),n=0;n<t.length;n++)(e=t[n]).element.scrollLeft=e.left,e.element.scrollTop=e.top}}var zn=B&&"documentMode"in document&&11>=document.documentMode,Yn={select:{phasedRegistrationNames:{bubbled:"onSelect",captured:"onSelectCapture"},dependencies:"blur contextmenu dragend focus keydown keyup mousedown mouseup selectionchange".split(" ")}},Hn=null,Wn=null,Xn=null,Vn=!1;function Bn(e,t){var n=t.window===t?t.document:9===t.nodeType?t:t.ownerDocument;return Vn||null==Hn||Hn!==Dn(n)?null:("selectionStart"in(n=Hn)&&Fn(n)?n={start:n.selectionStart,end:n.selectionEnd}:n={anchorNode:(n=(n.ownerDocument&&n.ownerDocument.defaultView||window).getSelection()).anchorNode,anchorOffset:n.anchorOffset,focusNode:n.focusNode,focusOffset:n.focusOffset},Xn&&en(Xn,n)?null:(Xn=n,(e=le.getPooled(Yn.select,Wn,e,t)).type="select",e.target=Hn,V(e),e))}var qn={eventTypes:Yn,extractEvents:function(e,t,n,i){var r,a=i.window===i?i.document:9===i.nodeType?i:i.ownerDocument;if(!(r=!a)){e:{a=In(a),r=x.onSelect;for(var o=0;o<r.length;o++){var s=r[o];if(!a.hasOwnProperty(s)||!a[s]){a=!1;break e}}a=!0}r=!a}if(r)return null;switch(a=t?F(t):window,e){case"focus":(je(a)||"true"===a.contentEditable)&&(Hn=a,Wn=t,Xn=null);break;case"blur":Xn=Wn=Hn=null;break;case"mousedown":Vn=!0;break;case"contextmenu":case"mouseup":case"dragend":return Vn=!1,Bn(n,i);case"selectionchange":if(zn)break;case"keydown":case"keyup":return Bn(n,i)}return null}};function Un(e,t){return e=r({children:void 0},t),(t=function(e){var t="";return i.Children.forEach(e,function(e){null!=e&&(t+=e)}),t}(t.children))&&(e.children=t),e}function Gn(e,t,n,i){if(e=e.options,t){t={};for(var r=0;r<n.length;r++)t["$"+n[r]]=!0;for(n=0;n<e.length;n++)r=t.hasOwnProperty("$"+e[n].value),e[n].selected!==r&&(e[n].selected=r),r&&i&&(e[n].defaultSelected=!0)}else{for(n=""+yt(n),t=null,r=0;r<e.length;r++){if(e[r].value===n)return e[r].selected=!0,void(i&&(e[r].defaultSelected=!0));null!==t||e[r].disabled||(t=e[r])}null!==t&&(t.selected=!0)}}function Qn(e,t){return null!=t.dangerouslySetInnerHTML&&o("91"),r({},t,{value:void 0,defaultValue:void 0,children:""+e._wrapperState.initialValue})}function $n(e,t){var n=t.value;null==n&&(n=t.defaultValue,null!=(t=t.children)&&(null!=n&&o("92"),Array.isArray(t)&&(1>=t.length||o("93"),t=t[0]),n=t),null==n&&(n="")),e._wrapperState={initialValue:yt(n)}}function Zn(e,t){var n=yt(t.value),i=yt(t.defaultValue);null!=n&&((n=""+n)!==e.value&&(e.value=n),null==t.defaultValue&&e.defaultValue!==n&&(e.defaultValue=n)),null!=i&&(e.defaultValue=""+i)}function Kn(e){var t=e.textContent;t===e._wrapperState.initialValue&&(e.value=t)}O.injectEventPluginOrder("ResponderEventPlugin SimpleEventPlugin EnterLeaveEventPlugin ChangeEventPlugin SelectEventPlugin BeforeInputEventPlugin".split(" ")),w=j,k=R,S=F,O.injectEventPluginsByName({SimpleEventPlugin:xn,EnterLeaveEventPlugin:Zt,ChangeEventPlugin:zt,SelectEventPlugin:qn,BeforeInputEventPlugin:Ce});var Jn={html:"http://www.w3.org/1999/xhtml",mathml:"http://www.w3.org/1998/Math/MathML",svg:"http://www.w3.org/2000/svg"};function ei(e){switch(e){case"svg":return"http://www.w3.org/2000/svg";case"math":return"http://www.w3.org/1998/Math/MathML";default:return"http://www.w3.org/1999/xhtml"}}function ti(e,t){return null==e||"http://www.w3.org/1999/xhtml"===e?ei(t):"http://www.w3.org/2000/svg"===e&&"foreignObject"===t?"http://www.w3.org/1999/xhtml":e}var ni,ii=void 0,ri=(ni=function(e,t){if(e.namespaceURI!==Jn.svg||"innerHTML"in e)e.innerHTML=t;else{for((ii=ii||document.createElement("div")).innerHTML="<svg>"+t+"</svg>",t=ii.firstChild;e.firstChild;)e.removeChild(e.firstChild);for(;t.firstChild;)e.appendChild(t.firstChild)}},"undefined"!=typeof MSApp&&MSApp.execUnsafeLocalFunction?function(e,t,n,i){MSApp.execUnsafeLocalFunction(function(){return ni(e,t)})}:ni);function ai(e,t){if(t){var n=e.firstChild;if(n&&n===e.lastChild&&3===n.nodeType)return void(n.nodeValue=t)}e.textContent=t}var oi={animationIterationCount:!0,borderImageOutset:!0,borderImageSlice:!0,borderImageWidth:!0,boxFlex:!0,boxFlexGroup:!0,boxOrdinalGroup:!0,columnCount:!0,columns:!0,flex:!0,flexGrow:!0,flexPositive:!0,flexShrink:!0,flexNegative:!0,flexOrder:!0,gridArea:!0,gridRow:!0,gridRowEnd:!0,gridRowSpan:!0,gridRowStart:!0,gridColumn:!0,gridColumnEnd:!0,gridColumnSpan:!0,gridColumnStart:!0,fontWeight:!0,lineClamp:!0,lineHeight:!0,opacity:!0,order:!0,orphans:!0,tabSize:!0,widows:!0,zIndex:!0,zoom:!0,fillOpacity:!0,floodOpacity:!0,stopOpacity:!0,strokeDasharray:!0,strokeDashoffset:!0,strokeMiterlimit:!0,strokeOpacity:!0,strokeWidth:!0},si=["Webkit","ms","Moz","O"];function li(e,t,n){return null==t||"boolean"==typeof t||""===t?"":n||"number"!=typeof t||0===t||oi.hasOwnProperty(e)&&oi[e]?(""+t).trim():t+"px"}function ci(e,t){for(var n in e=e.style,t)if(t.hasOwnProperty(n)){var i=0===n.indexOf("--"),r=li(n,t[n],i);"float"===n&&(n="cssFloat"),i?e.setProperty(n,r):e[n]=r}}Object.keys(oi).forEach(function(e){si.forEach(function(t){t=t+e.charAt(0).toUpperCase()+e.substring(1),oi[t]=oi[e]})});var ui=r({menuitem:!0},{area:!0,base:!0,br:!0,col:!0,embed:!0,hr:!0,img:!0,input:!0,keygen:!0,link:!0,meta:!0,param:!0,source:!0,track:!0,wbr:!0});function hi(e,t){t&&(ui[e]&&(null!=t.children||null!=t.dangerouslySetInnerHTML)&&o("137",e,""),null!=t.dangerouslySetInnerHTML&&(null!=t.children&&o("60"),"object"==typeof t.dangerouslySetInnerHTML&&"__html"in t.dangerouslySetInnerHTML||o("61")),null!=t.style&&"object"!=typeof t.style&&o("62",""))}function di(e,t){if(-1===e.indexOf("-"))return"string"==typeof t.is;switch(e){case"annotation-xml":case"color-profile":case"font-face":case"font-face-src":case"font-face-uri":case"font-face-format":case"font-face-name":case"missing-glyph":return!1;default:return!0}}function fi(e,t){var n=In(e=9===e.nodeType||11===e.nodeType?e:e.ownerDocument);t=x[t];for(var i=0;i<t.length;i++){var r=t[i];if(!n.hasOwnProperty(r)||!n[r]){switch(r){case"scroll":Tn("scroll",e);break;case"focus":case"blur":Tn("focus",e),Tn("blur",e),n.blur=!0,n.focus=!0;break;case"cancel":case"close":Ye(r)&&Tn(r,e);break;case"invalid":case"submit":case"reset":break;default:-1===te.indexOf(r)&&Cn(r,e)}n[r]=!0}}}function pi(){}var gi=null,mi=null;function vi(e,t){switch(e){case"button":case"input":case"select":case"textarea":return!!t.autoFocus}return!1}function yi(e,t){return"textarea"===e||"option"===e||"noscript"===e||"string"==typeof t.children||"number"==typeof t.children||"object"==typeof t.dangerouslySetInnerHTML&&null!==t.dangerouslySetInnerHTML&&null!=t.dangerouslySetInnerHTML.__html}var bi="function"==typeof setTimeout?setTimeout:void 0,xi="function"==typeof clearTimeout?clearTimeout:void 0,wi=a.unstable_scheduleCallback,ki=a.unstable_cancelCallback;function Si(e){for(e=e.nextSibling;e&&1!==e.nodeType&&3!==e.nodeType;)e=e.nextSibling;return e}function Ei(e){for(e=e.firstChild;e&&1!==e.nodeType&&3!==e.nodeType;)e=e.nextSibling;return e}new Set;var Ci=[],Ti=-1;function Ai(e){0>Ti||(e.current=Ci[Ti],Ci[Ti]=null,Ti--)}function _i(e,t){Ci[++Ti]=e.current,e.current=t}var Oi={},Pi={current:Oi},Mi={current:!1},Ii=Oi;function Di(e,t){var n=e.type.contextTypes;if(!n)return Oi;var i=e.stateNode;if(i&&i.__reactInternalMemoizedUnmaskedChildContext===t)return i.__reactInternalMemoizedMaskedChildContext;var r,a={};for(r in n)a[r]=t[r];return i&&((e=e.stateNode).__reactInternalMemoizedUnmaskedChildContext=t,e.__reactInternalMemoizedMaskedChildContext=a),a}function Ni(e){return null!=(e=e.childContextTypes)}function Li(e){Ai(Mi),Ai(Pi)}function Ri(e){Ai(Mi),Ai(Pi)}function Fi(e,t,n){Pi.current!==Oi&&o("168"),_i(Pi,t),_i(Mi,n)}function ji(e,t,n){var i=e.stateNode;if(e=t.childContextTypes,"function"!=typeof i.getChildContext)return n;for(var a in i=i.getChildContext())a in e||o("108",st(t)||"Unknown",a);return r({},n,i)}function zi(e){var t=e.stateNode;return t=t&&t.__reactInternalMemoizedMergedChildContext||Oi,Ii=Pi.current,_i(Pi,t),_i(Mi,Mi.current),!0}function Yi(e,t,n){var i=e.stateNode;i||o("169"),n?(t=ji(e,t,Ii),i.__reactInternalMemoizedMergedChildContext=t,Ai(Mi),Ai(Pi),_i(Pi,t)):Ai(Mi),_i(Mi,n)}var Hi=null,Wi=null;function Xi(e){return function(t){try{return e(t)}catch(e){}}}function Vi(e,t,n,i){this.tag=e,this.key=n,this.sibling=this.child=this.return=this.stateNode=this.type=this.elementType=null,this.index=0,this.ref=null,this.pendingProps=t,this.contextDependencies=this.memoizedState=this.updateQueue=this.memoizedProps=null,this.mode=i,this.effectTag=0,this.lastEffect=this.firstEffect=this.nextEffect=null,this.childExpirationTime=this.expirationTime=0,this.alternate=null}function Bi(e,t,n,i){return new Vi(e,t,n,i)}function qi(e){return!(!(e=e.prototype)||!e.isReactComponent)}function Ui(e,t){var n=e.alternate;return null===n?((n=Bi(e.tag,t,e.key,e.mode)).elementType=e.elementType,n.type=e.type,n.stateNode=e.stateNode,n.alternate=e,e.alternate=n):(n.pendingProps=t,n.effectTag=0,n.nextEffect=null,n.firstEffect=null,n.lastEffect=null),n.childExpirationTime=e.childExpirationTime,n.expirationTime=e.expirationTime,n.child=e.child,n.memoizedProps=e.memoizedProps,n.memoizedState=e.memoizedState,n.updateQueue=e.updateQueue,n.contextDependencies=e.contextDependencies,n.sibling=e.sibling,n.index=e.index,n.ref=e.ref,n}function Gi(e,t,n,i,r,a){var s=2;if(i=e,"function"==typeof e)qi(e)&&(s=1);else if("string"==typeof e)s=5;else e:switch(e){case Qe:return Qi(n.children,r,a,t);case et:return $i(n,3|r,a,t);case $e:return $i(n,2|r,a,t);case Ze:return(e=Bi(12,n,t,4|r)).elementType=Ze,e.type=Ze,e.expirationTime=a,e;case nt:return(e=Bi(13,n,t,r)).elementType=nt,e.type=nt,e.expirationTime=a,e;default:if("object"==typeof e&&null!==e)switch(e.$$typeof){case Ke:s=10;break e;case Je:s=9;break e;case tt:s=11;break e;case it:s=14;break e;case rt:s=16,i=null;break e}o("130",null==e?e:typeof e,"")}return(t=Bi(s,n,t,r)).elementType=e,t.type=i,t.expirationTime=a,t}function Qi(e,t,n,i){return(e=Bi(7,e,i,t)).expirationTime=n,e}function $i(e,t,n,i){return e=Bi(8,e,i,t),t=0==(1&t)?$e:et,e.elementType=t,e.type=t,e.expirationTime=n,e}function Zi(e,t,n){return(e=Bi(6,e,null,t)).expirationTime=n,e}function Ki(e,t,n){return(t=Bi(4,null!==e.children?e.children:[],e.key,t)).expirationTime=n,t.stateNode={containerInfo:e.containerInfo,pendingChildren:null,implementation:e.implementation},t}function Ji(e,t){e.didError=!1;var n=e.earliestPendingTime;0===n?e.earliestPendingTime=e.latestPendingTime=t:n<t?e.earliestPendingTime=t:e.latestPendingTime>t&&(e.latestPendingTime=t),nr(t,e)}function er(e,t){e.didError=!1,e.latestPingedTime>=t&&(e.latestPingedTime=0);var n=e.earliestPendingTime,i=e.latestPendingTime;n===t?e.earliestPendingTime=i===t?e.latestPendingTime=0:i:i===t&&(e.latestPendingTime=n),n=e.earliestSuspendedTime,i=e.latestSuspendedTime,0===n?e.earliestSuspendedTime=e.latestSuspendedTime=t:n<t?e.earliestSuspendedTime=t:i>t&&(e.latestSuspendedTime=t),nr(t,e)}function tr(e,t){var n=e.earliestPendingTime;return n>t&&(t=n),(e=e.earliestSuspendedTime)>t&&(t=e),t}function nr(e,t){var n=t.earliestSuspendedTime,i=t.latestSuspendedTime,r=t.earliestPendingTime,a=t.latestPingedTime;0===(r=0!==r?r:a)&&(0===e||i<e)&&(r=i),0!==(e=r)&&n>e&&(e=n),t.nextExpirationTimeToWorkOn=r,t.expirationTime=e}function ir(e,t){if(e&&e.defaultProps)for(var n in t=r({},t),e=e.defaultProps)void 0===t[n]&&(t[n]=e[n]);return t}var rr=(new i.Component).refs;function ar(e,t,n,i){n=null==(n=n(i,t=e.memoizedState))?t:r({},t,n),e.memoizedState=n,null!==(i=e.updateQueue)&&0===e.expirationTime&&(i.baseState=n)}var or={isMounted:function(e){return!!(e=e._reactInternalFiber)&&2===tn(e)},enqueueSetState:function(e,t,n){e=e._reactInternalFiber;var i=ks(),r=Qa(i=Qo(i,e));r.payload=t,null!=n&&(r.callback=n),Xo(),Za(e,r),Ko(e,i)},enqueueReplaceState:function(e,t,n){e=e._reactInternalFiber;var i=ks(),r=Qa(i=Qo(i,e));r.tag=Xa,r.payload=t,null!=n&&(r.callback=n),Xo(),Za(e,r),Ko(e,i)},enqueueForceUpdate:function(e,t){e=e._reactInternalFiber;var n=ks(),i=Qa(n=Qo(n,e));i.tag=Va,null!=t&&(i.callback=t),Xo(),Za(e,i),Ko(e,n)}};function sr(e,t,n,i,r,a,o){return"function"==typeof(e=e.stateNode).shouldComponentUpdate?e.shouldComponentUpdate(i,a,o):!t.prototype||!t.prototype.isPureReactComponent||(!en(n,i)||!en(r,a))}function lr(e,t,n){var i=!1,r=Oi,a=t.contextType;return"object"==typeof a&&null!==a?a=Ha(a):(r=Ni(t)?Ii:Pi.current,a=(i=null!=(i=t.contextTypes))?Di(e,r):Oi),t=new t(n,a),e.memoizedState=null!==t.state&&void 0!==t.state?t.state:null,t.updater=or,e.stateNode=t,t._reactInternalFiber=e,i&&((e=e.stateNode).__reactInternalMemoizedUnmaskedChildContext=r,e.__reactInternalMemoizedMaskedChildContext=a),t}function cr(e,t,n,i){e=t.state,"function"==typeof t.componentWillReceiveProps&&t.componentWillReceiveProps(n,i),"function"==typeof t.UNSAFE_componentWillReceiveProps&&t.UNSAFE_componentWillReceiveProps(n,i),t.state!==e&&or.enqueueReplaceState(t,t.state,null)}function ur(e,t,n,i){var r=e.stateNode;r.props=n,r.state=e.memoizedState,r.refs=rr;var a=t.contextType;"object"==typeof a&&null!==a?r.context=Ha(a):(a=Ni(t)?Ii:Pi.current,r.context=Di(e,a)),null!==(a=e.updateQueue)&&(to(e,a,n,r,i),r.state=e.memoizedState),"function"==typeof(a=t.getDerivedStateFromProps)&&(ar(e,t,a,n),r.state=e.memoizedState),"function"==typeof t.getDerivedStateFromProps||"function"==typeof r.getSnapshotBeforeUpdate||"function"!=typeof r.UNSAFE_componentWillMount&&"function"!=typeof r.componentWillMount||(t=r.state,"function"==typeof r.componentWillMount&&r.componentWillMount(),"function"==typeof r.UNSAFE_componentWillMount&&r.UNSAFE_componentWillMount(),t!==r.state&&or.enqueueReplaceState(r,r.state,null),null!==(a=e.updateQueue)&&(to(e,a,n,r,i),r.state=e.memoizedState)),"function"==typeof r.componentDidMount&&(e.effectTag|=4)}var hr=Array.isArray;function dr(e,t,n){if(null!==(e=n.ref)&&"function"!=typeof e&&"object"!=typeof e){if(n._owner){n=n._owner;var i=void 0;n&&(1!==n.tag&&o("309"),i=n.stateNode),i||o("147",e);var r=""+e;return null!==t&&null!==t.ref&&"function"==typeof t.ref&&t.ref._stringRef===r?t.ref:((t=function(e){var t=i.refs;t===rr&&(t=i.refs={}),null===e?delete t[r]:t[r]=e})._stringRef=r,t)}"string"!=typeof e&&o("284"),n._owner||o("290",e)}return e}function fr(e,t){"textarea"!==e.type&&o("31","[object Object]"===Object.prototype.toString.call(t)?"object with keys {"+Object.keys(t).join(", ")+"}":t,"")}function pr(e){function t(t,n){if(e){var i=t.lastEffect;null!==i?(i.nextEffect=n,t.lastEffect=n):t.firstEffect=t.lastEffect=n,n.nextEffect=null,n.effectTag=8}}function n(n,i){if(!e)return null;for(;null!==i;)t(n,i),i=i.sibling;return null}function i(e,t){for(e=new Map;null!==t;)null!==t.key?e.set(t.key,t):e.set(t.index,t),t=t.sibling;return e}function r(e,t,n){return(e=Ui(e,t)).index=0,e.sibling=null,e}function a(t,n,i){return t.index=i,e?null!==(i=t.alternate)?(i=i.index)<n?(t.effectTag=2,n):i:(t.effectTag=2,n):n}function s(t){return e&&null===t.alternate&&(t.effectTag=2),t}function l(e,t,n,i){return null===t||6!==t.tag?((t=Zi(n,e.mode,i)).return=e,t):((t=r(t,n)).return=e,t)}function c(e,t,n,i){return null!==t&&t.elementType===n.type?((i=r(t,n.props)).ref=dr(e,t,n),i.return=e,i):((i=Gi(n.type,n.key,n.props,null,e.mode,i)).ref=dr(e,t,n),i.return=e,i)}function u(e,t,n,i){return null===t||4!==t.tag||t.stateNode.containerInfo!==n.containerInfo||t.stateNode.implementation!==n.implementation?((t=Ki(n,e.mode,i)).return=e,t):((t=r(t,n.children||[])).return=e,t)}function h(e,t,n,i,a){return null===t||7!==t.tag?((t=Qi(n,e.mode,i,a)).return=e,t):((t=r(t,n)).return=e,t)}function d(e,t,n){if("string"==typeof t||"number"==typeof t)return(t=Zi(""+t,e.mode,n)).return=e,t;if("object"==typeof t&&null!==t){switch(t.$$typeof){case Ue:return(n=Gi(t.type,t.key,t.props,null,e.mode,n)).ref=dr(e,null,t),n.return=e,n;case Ge:return(t=Ki(t,e.mode,n)).return=e,t}if(hr(t)||ot(t))return(t=Qi(t,e.mode,n,null)).return=e,t;fr(e,t)}return null}function f(e,t,n,i){var r=null!==t?t.key:null;if("string"==typeof n||"number"==typeof n)return null!==r?null:l(e,t,""+n,i);if("object"==typeof n&&null!==n){switch(n.$$typeof){case Ue:return n.key===r?n.type===Qe?h(e,t,n.props.children,i,r):c(e,t,n,i):null;case Ge:return n.key===r?u(e,t,n,i):null}if(hr(n)||ot(n))return null!==r?null:h(e,t,n,i,null);fr(e,n)}return null}function p(e,t,n,i,r){if("string"==typeof i||"number"==typeof i)return l(t,e=e.get(n)||null,""+i,r);if("object"==typeof i&&null!==i){switch(i.$$typeof){case Ue:return e=e.get(null===i.key?n:i.key)||null,i.type===Qe?h(t,e,i.props.children,r,i.key):c(t,e,i,r);case Ge:return u(t,e=e.get(null===i.key?n:i.key)||null,i,r)}if(hr(i)||ot(i))return h(t,e=e.get(n)||null,i,r,null);fr(t,i)}return null}function g(r,o,s,l){for(var c=null,u=null,h=o,g=o=0,m=null;null!==h&&g<s.length;g++){h.index>g?(m=h,h=null):m=h.sibling;var v=f(r,h,s[g],l);if(null===v){null===h&&(h=m);break}e&&h&&null===v.alternate&&t(r,h),o=a(v,o,g),null===u?c=v:u.sibling=v,u=v,h=m}if(g===s.length)return n(r,h),c;if(null===h){for(;g<s.length;g++)(h=d(r,s[g],l))&&(o=a(h,o,g),null===u?c=h:u.sibling=h,u=h);return c}for(h=i(r,h);g<s.length;g++)(m=p(h,r,g,s[g],l))&&(e&&null!==m.alternate&&h.delete(null===m.key?g:m.key),o=a(m,o,g),null===u?c=m:u.sibling=m,u=m);return e&&h.forEach(function(e){return t(r,e)}),c}function m(r,s,l,c){var u=ot(l);"function"!=typeof u&&o("150"),null==(l=u.call(l))&&o("151");for(var h=u=null,g=s,m=s=0,v=null,y=l.next();null!==g&&!y.done;m++,y=l.next()){g.index>m?(v=g,g=null):v=g.sibling;var b=f(r,g,y.value,c);if(null===b){g||(g=v);break}e&&g&&null===b.alternate&&t(r,g),s=a(b,s,m),null===h?u=b:h.sibling=b,h=b,g=v}if(y.done)return n(r,g),u;if(null===g){for(;!y.done;m++,y=l.next())null!==(y=d(r,y.value,c))&&(s=a(y,s,m),null===h?u=y:h.sibling=y,h=y);return u}for(g=i(r,g);!y.done;m++,y=l.next())null!==(y=p(g,r,m,y.value,c))&&(e&&null!==y.alternate&&g.delete(null===y.key?m:y.key),s=a(y,s,m),null===h?u=y:h.sibling=y,h=y);return e&&g.forEach(function(e){return t(r,e)}),u}return function(e,i,a,l){var c="object"==typeof a&&null!==a&&a.type===Qe&&null===a.key;c&&(a=a.props.children);var u="object"==typeof a&&null!==a;if(u)switch(a.$$typeof){case Ue:e:{for(u=a.key,c=i;null!==c;){if(c.key===u){if(7===c.tag?a.type===Qe:c.elementType===a.type){n(e,c.sibling),(i=r(c,a.type===Qe?a.props.children:a.props)).ref=dr(e,c,a),i.return=e,e=i;break e}n(e,c);break}t(e,c),c=c.sibling}a.type===Qe?((i=Qi(a.props.children,e.mode,l,a.key)).return=e,e=i):((l=Gi(a.type,a.key,a.props,null,e.mode,l)).ref=dr(e,i,a),l.return=e,e=l)}return s(e);case Ge:e:{for(c=a.key;null!==i;){if(i.key===c){if(4===i.tag&&i.stateNode.containerInfo===a.containerInfo&&i.stateNode.implementation===a.implementation){n(e,i.sibling),(i=r(i,a.children||[])).return=e,e=i;break e}n(e,i);break}t(e,i),i=i.sibling}(i=Ki(a,e.mode,l)).return=e,e=i}return s(e)}if("string"==typeof a||"number"==typeof a)return a=""+a,null!==i&&6===i.tag?(n(e,i.sibling),(i=r(i,a)).return=e,e=i):(n(e,i),(i=Zi(a,e.mode,l)).return=e,e=i),s(e);if(hr(a))return g(e,i,a,l);if(ot(a))return m(e,i,a,l);if(u&&fr(e,a),void 0===a&&!c)switch(e.tag){case 1:case 0:o("152",(l=e.type).displayName||l.name||"Component")}return n(e,i)}}var gr=pr(!0),mr=pr(!1),vr={},yr={current:vr},br={current:vr},xr={current:vr};function wr(e){return e===vr&&o("174"),e}function kr(e,t){_i(xr,t),_i(br,e),_i(yr,vr);var n=t.nodeType;switch(n){case 9:case 11:t=(t=t.documentElement)?t.namespaceURI:ti(null,"");break;default:t=ti(t=(n=8===n?t.parentNode:t).namespaceURI||null,n=n.tagName)}Ai(yr),_i(yr,t)}function Sr(e){Ai(yr),Ai(br),Ai(xr)}function Er(e){wr(xr.current);var t=wr(yr.current),n=ti(t,e.type);t!==n&&(_i(br,e),_i(yr,n))}function Cr(e){br.current===e&&(Ai(yr),Ai(br))}var Tr=0,Ar=2,_r=4,Or=8,Pr=16,Mr=32,Ir=64,Dr=128,Nr=Ve.ReactCurrentDispatcher,Lr=0,Rr=null,Fr=null,jr=null,zr=null,Yr=null,Hr=null,Wr=0,Xr=null,Vr=0,Br=!1,qr=null,Ur=0;function Gr(){o("321")}function Qr(e,t){if(null===t)return!1;for(var n=0;n<t.length&&n<e.length;n++)if(!Kt(e[n],t[n]))return!1;return!0}function $r(e,t,n,i,r,a){if(Lr=a,Rr=t,jr=null!==e?e.memoizedState:null,Nr.current=null===jr?ca:ua,t=n(i,r),Br){do{Br=!1,Ur+=1,jr=null!==e?e.memoizedState:null,Hr=zr,Xr=Yr=Fr=null,Nr.current=ua,t=n(i,r)}while(Br);qr=null,Ur=0}return Nr.current=la,(e=Rr).memoizedState=zr,e.expirationTime=Wr,e.updateQueue=Xr,e.effectTag|=Vr,e=null!==Fr&&null!==Fr.next,Lr=0,Hr=Yr=zr=jr=Fr=Rr=null,Wr=0,Xr=null,Vr=0,e&&o("300"),t}function Zr(){Nr.current=la,Lr=0,Hr=Yr=zr=jr=Fr=Rr=null,Wr=0,Xr=null,Vr=0,Br=!1,qr=null,Ur=0}function Kr(){var e={memoizedState:null,baseState:null,queue:null,baseUpdate:null,next:null};return null===Yr?zr=Yr=e:Yr=Yr.next=e,Yr}function Jr(){if(null!==Hr)Hr=(Yr=Hr).next,jr=null!==(Fr=jr)?Fr.next:null;else{null===jr&&o("310");var e={memoizedState:(Fr=jr).memoizedState,baseState:Fr.baseState,queue:Fr.queue,baseUpdate:Fr.baseUpdate,next:null};Yr=null===Yr?zr=e:Yr.next=e,jr=Fr.next}return Yr}function ea(e,t){return"function"==typeof t?t(e):t}function ta(e){var t=Jr(),n=t.queue;if(null===n&&o("311"),n.lastRenderedReducer=e,0<Ur){var i=n.dispatch;if(null!==qr){var r=qr.get(n);if(void 0!==r){qr.delete(n);var a=t.memoizedState;do{a=e(a,r.action),r=r.next}while(null!==r);return Kt(a,t.memoizedState)||(wa=!0),t.memoizedState=a,t.baseUpdate===n.last&&(t.baseState=a),n.lastRenderedState=a,[a,i]}}return[t.memoizedState,i]}i=n.last;var s=t.baseUpdate;if(a=t.baseState,null!==s?(null!==i&&(i.next=null),i=s.next):i=null!==i?i.next:null,null!==i){var l=r=null,c=i,u=!1;do{var h=c.expirationTime;h<Lr?(u||(u=!0,l=s,r=a),h>Wr&&(Wr=h)):a=c.eagerReducer===e?c.eagerState:e(a,c.action),s=c,c=c.next}while(null!==c&&c!==i);u||(l=s,r=a),Kt(a,t.memoizedState)||(wa=!0),t.memoizedState=a,t.baseUpdate=l,t.baseState=r,n.lastRenderedState=a}return[t.memoizedState,n.dispatch]}function na(e,t,n,i){return e={tag:e,create:t,destroy:n,deps:i,next:null},null===Xr?(Xr={lastEffect:null}).lastEffect=e.next=e:null===(t=Xr.lastEffect)?Xr.lastEffect=e.next=e:(n=t.next,t.next=e,e.next=n,Xr.lastEffect=e),e}function ia(e,t,n,i){var r=Kr();Vr|=e,r.memoizedState=na(t,n,void 0,void 0===i?null:i)}function ra(e,t,n,i){var r=Jr();i=void 0===i?null:i;var a=void 0;if(null!==Fr){var o=Fr.memoizedState;if(a=o.destroy,null!==i&&Qr(i,o.deps))return void na(Tr,n,a,i)}Vr|=e,r.memoizedState=na(t,n,a,i)}function aa(e,t){return"function"==typeof t?(e=e(),t(e),function(){t(null)}):null!=t?(e=e(),t.current=e,function(){t.current=null}):void 0}function oa(){}function sa(e,t,n){25>Ur||o("301");var i=e.alternate;if(e===Rr||null!==i&&i===Rr)if(Br=!0,e={expirationTime:Lr,action:n,eagerReducer:null,eagerState:null,next:null},null===qr&&(qr=new Map),void 0===(n=qr.get(t)))qr.set(t,e);else{for(t=n;null!==t.next;)t=t.next;t.next=e}else{Xo();var r=ks(),a={expirationTime:r=Qo(r,e),action:n,eagerReducer:null,eagerState:null,next:null},s=t.last;if(null===s)a.next=a;else{var l=s.next;null!==l&&(a.next=l),s.next=a}if(t.last=a,0===e.expirationTime&&(null===i||0===i.expirationTime)&&null!==(i=t.lastRenderedReducer))try{var c=t.lastRenderedState,u=i(c,n);if(a.eagerReducer=i,a.eagerState=u,Kt(u,c))return}catch(e){}Ko(e,r)}}var la={readContext:Ha,useCallback:Gr,useContext:Gr,useEffect:Gr,useImperativeHandle:Gr,useLayoutEffect:Gr,useMemo:Gr,useReducer:Gr,useRef:Gr,useState:Gr,useDebugValue:Gr},ca={readContext:Ha,useCallback:function(e,t){return Kr().memoizedState=[e,void 0===t?null:t],e},useContext:Ha,useEffect:function(e,t){return ia(516,Dr|Ir,e,t)},useImperativeHandle:function(e,t,n){return n=null!=n?n.concat([e]):null,ia(4,_r|Mr,aa.bind(null,t,e),n)},useLayoutEffect:function(e,t){return ia(4,_r|Mr,e,t)},useMemo:function(e,t){var n=Kr();return t=void 0===t?null:t,e=e(),n.memoizedState=[e,t],e},useReducer:function(e,t,n){var i=Kr();return t=void 0!==n?n(t):t,i.memoizedState=i.baseState=t,e=(e=i.queue={last:null,dispatch:null,lastRenderedReducer:e,lastRenderedState:t}).dispatch=sa.bind(null,Rr,e),[i.memoizedState,e]},useRef:function(e){return e={current:e},Kr().memoizedState=e},useState:function(e){var t=Kr();return"function"==typeof e&&(e=e()),t.memoizedState=t.baseState=e,e=(e=t.queue={last:null,dispatch:null,lastRenderedReducer:ea,lastRenderedState:e}).dispatch=sa.bind(null,Rr,e),[t.memoizedState,e]},useDebugValue:oa},ua={readContext:Ha,useCallback:function(e,t){var n=Jr();t=void 0===t?null:t;var i=n.memoizedState;return null!==i&&null!==t&&Qr(t,i[1])?i[0]:(n.memoizedState=[e,t],e)},useContext:Ha,useEffect:function(e,t){return ra(516,Dr|Ir,e,t)},useImperativeHandle:function(e,t,n){return n=null!=n?n.concat([e]):null,ra(4,_r|Mr,aa.bind(null,t,e),n)},useLayoutEffect:function(e,t){return ra(4,_r|Mr,e,t)},useMemo:function(e,t){var n=Jr();t=void 0===t?null:t;var i=n.memoizedState;return null!==i&&null!==t&&Qr(t,i[1])?i[0]:(e=e(),n.memoizedState=[e,t],e)},useReducer:ta,useRef:function(){return Jr().memoizedState},useState:function(e){return ta(ea)},useDebugValue:oa},ha=null,da=null,fa=!1;function pa(e,t){var n=Bi(5,null,null,0);n.elementType="DELETED",n.type="DELETED",n.stateNode=t,n.return=e,n.effectTag=8,null!==e.lastEffect?(e.lastEffect.nextEffect=n,e.lastEffect=n):e.firstEffect=e.lastEffect=n}function ga(e,t){switch(e.tag){case 5:var n=e.type;return null!==(t=1!==t.nodeType||n.toLowerCase()!==t.nodeName.toLowerCase()?null:t)&&(e.stateNode=t,!0);case 6:return null!==(t=""===e.pendingProps||3!==t.nodeType?null:t)&&(e.stateNode=t,!0);case 13:default:return!1}}function ma(e){if(fa){var t=da;if(t){var n=t;if(!ga(e,t)){if(!(t=Si(n))||!ga(e,t))return e.effectTag|=2,fa=!1,void(ha=e);pa(ha,n)}ha=e,da=Ei(t)}else e.effectTag|=2,fa=!1,ha=e}}function va(e){for(e=e.return;null!==e&&5!==e.tag&&3!==e.tag&&18!==e.tag;)e=e.return;ha=e}function ya(e){if(e!==ha)return!1;if(!fa)return va(e),fa=!0,!1;var t=e.type;if(5!==e.tag||"head"!==t&&"body"!==t&&!yi(t,e.memoizedProps))for(t=da;t;)pa(e,t),t=Si(t);return va(e),da=ha?Si(e.stateNode):null,!0}function ba(){da=ha=null,fa=!1}var xa=Ve.ReactCurrentOwner,wa=!1;function ka(e,t,n,i){t.child=null===e?mr(t,null,n,i):gr(t,e.child,n,i)}function Sa(e,t,n,i,r){n=n.render;var a=t.ref;return Ya(t,r),i=$r(e,t,n,i,a,r),null===e||wa?(t.effectTag|=1,ka(e,t,i,r),t.child):(t.updateQueue=e.updateQueue,t.effectTag&=-517,e.expirationTime<=r&&(e.expirationTime=0),Ia(e,t,r))}function Ea(e,t,n,i,r,a){if(null===e){var o=n.type;return"function"!=typeof o||qi(o)||void 0!==o.defaultProps||null!==n.compare||void 0!==n.defaultProps?((e=Gi(n.type,null,i,null,t.mode,a)).ref=t.ref,e.return=t,t.child=e):(t.tag=15,t.type=o,Ca(e,t,o,i,r,a))}return o=e.child,r<a&&(r=o.memoizedProps,(n=null!==(n=n.compare)?n:en)(r,i)&&e.ref===t.ref)?Ia(e,t,a):(t.effectTag|=1,(e=Ui(o,i)).ref=t.ref,e.return=t,t.child=e)}function Ca(e,t,n,i,r,a){return null!==e&&en(e.memoizedProps,i)&&e.ref===t.ref&&(wa=!1,r<a)?Ia(e,t,a):Aa(e,t,n,i,a)}function Ta(e,t){var n=t.ref;(null===e&&null!==n||null!==e&&e.ref!==n)&&(t.effectTag|=128)}function Aa(e,t,n,i,r){var a=Ni(n)?Ii:Pi.current;return a=Di(t,a),Ya(t,r),n=$r(e,t,n,i,a,r),null===e||wa?(t.effectTag|=1,ka(e,t,n,r),t.child):(t.updateQueue=e.updateQueue,t.effectTag&=-517,e.expirationTime<=r&&(e.expirationTime=0),Ia(e,t,r))}function _a(e,t,n,i,r){if(Ni(n)){var a=!0;zi(t)}else a=!1;if(Ya(t,r),null===t.stateNode)null!==e&&(e.alternate=null,t.alternate=null,t.effectTag|=2),lr(t,n,i),ur(t,n,i,r),i=!0;else if(null===e){var o=t.stateNode,s=t.memoizedProps;o.props=s;var l=o.context,c=n.contextType;"object"==typeof c&&null!==c?c=Ha(c):c=Di(t,c=Ni(n)?Ii:Pi.current);var u=n.getDerivedStateFromProps,h="function"==typeof u||"function"==typeof o.getSnapshotBeforeUpdate;h||"function"!=typeof o.UNSAFE_componentWillReceiveProps&&"function"!=typeof o.componentWillReceiveProps||(s!==i||l!==c)&&cr(t,o,i,c),qa=!1;var d=t.memoizedState;l=o.state=d;var f=t.updateQueue;null!==f&&(to(t,f,i,o,r),l=t.memoizedState),s!==i||d!==l||Mi.current||qa?("function"==typeof u&&(ar(t,n,u,i),l=t.memoizedState),(s=qa||sr(t,n,s,i,d,l,c))?(h||"function"!=typeof o.UNSAFE_componentWillMount&&"function"!=typeof o.componentWillMount||("function"==typeof o.componentWillMount&&o.componentWillMount(),"function"==typeof o.UNSAFE_componentWillMount&&o.UNSAFE_componentWillMount()),"function"==typeof o.componentDidMount&&(t.effectTag|=4)):("function"==typeof o.componentDidMount&&(t.effectTag|=4),t.memoizedProps=i,t.memoizedState=l),o.props=i,o.state=l,o.context=c,i=s):("function"==typeof o.componentDidMount&&(t.effectTag|=4),i=!1)}else o=t.stateNode,s=t.memoizedProps,o.props=t.type===t.elementType?s:ir(t.type,s),l=o.context,"object"==typeof(c=n.contextType)&&null!==c?c=Ha(c):c=Di(t,c=Ni(n)?Ii:Pi.current),(h="function"==typeof(u=n.getDerivedStateFromProps)||"function"==typeof o.getSnapshotBeforeUpdate)||"function"!=typeof o.UNSAFE_componentWillReceiveProps&&"function"!=typeof o.componentWillReceiveProps||(s!==i||l!==c)&&cr(t,o,i,c),qa=!1,l=t.memoizedState,d=o.state=l,null!==(f=t.updateQueue)&&(to(t,f,i,o,r),d=t.memoizedState),s!==i||l!==d||Mi.current||qa?("function"==typeof u&&(ar(t,n,u,i),d=t.memoizedState),(u=qa||sr(t,n,s,i,l,d,c))?(h||"function"!=typeof o.UNSAFE_componentWillUpdate&&"function"!=typeof o.componentWillUpdate||("function"==typeof o.componentWillUpdate&&o.componentWillUpdate(i,d,c),"function"==typeof o.UNSAFE_componentWillUpdate&&o.UNSAFE_componentWillUpdate(i,d,c)),"function"==typeof o.componentDidUpdate&&(t.effectTag|=4),"function"==typeof o.getSnapshotBeforeUpdate&&(t.effectTag|=256)):("function"!=typeof o.componentDidUpdate||s===e.memoizedProps&&l===e.memoizedState||(t.effectTag|=4),"function"!=typeof o.getSnapshotBeforeUpdate||s===e.memoizedProps&&l===e.memoizedState||(t.effectTag|=256),t.memoizedProps=i,t.memoizedState=d),o.props=i,o.state=d,o.context=c,i=u):("function"!=typeof o.componentDidUpdate||s===e.memoizedProps&&l===e.memoizedState||(t.effectTag|=4),"function"!=typeof o.getSnapshotBeforeUpdate||s===e.memoizedProps&&l===e.memoizedState||(t.effectTag|=256),i=!1);return Oa(e,t,n,i,a,r)}function Oa(e,t,n,i,r,a){Ta(e,t);var o=0!=(64&t.effectTag);if(!i&&!o)return r&&Yi(t,n,!1),Ia(e,t,a);i=t.stateNode,xa.current=t;var s=o&&"function"!=typeof n.getDerivedStateFromError?null:i.render();return t.effectTag|=1,null!==e&&o?(t.child=gr(t,e.child,null,a),t.child=gr(t,null,s,a)):ka(e,t,s,a),t.memoizedState=i.state,r&&Yi(t,n,!0),t.child}function Pa(e){var t=e.stateNode;t.pendingContext?Fi(0,t.pendingContext,t.pendingContext!==t.context):t.context&&Fi(0,t.context,!1),kr(e,t.containerInfo)}function Ma(e,t,n){var i=t.mode,r=t.pendingProps,a=t.memoizedState;if(0==(64&t.effectTag)){a=null;var o=!1}else a={timedOutAt:null!==a?a.timedOutAt:0},o=!0,t.effectTag&=-65;if(null===e)if(o){var s=r.fallback;e=Qi(null,i,0,null),0==(1&t.mode)&&(e.child=null!==t.memoizedState?t.child.child:t.child),i=Qi(s,i,n,null),e.sibling=i,(n=e).return=i.return=t}else n=i=mr(t,null,r.children,n);else null!==e.memoizedState?(s=(i=e.child).sibling,o?(n=r.fallback,r=Ui(i,i.pendingProps),0==(1&t.mode)&&((o=null!==t.memoizedState?t.child.child:t.child)!==i.child&&(r.child=o)),i=r.sibling=Ui(s,n,s.expirationTime),n=r,r.childExpirationTime=0,n.return=i.return=t):n=i=gr(t,i.child,r.children,n)):(s=e.child,o?(o=r.fallback,(r=Qi(null,i,0,null)).child=s,0==(1&t.mode)&&(r.child=null!==t.memoizedState?t.child.child:t.child),(i=r.sibling=Qi(o,i,n,null)).effectTag|=2,n=r,r.childExpirationTime=0,n.return=i.return=t):i=n=gr(t,s,r.children,n)),t.stateNode=e.stateNode;return t.memoizedState=a,t.child=n,i}function Ia(e,t,n){if(null!==e&&(t.contextDependencies=e.contextDependencies),t.childExpirationTime<n)return null;if(null!==e&&t.child!==e.child&&o("153"),null!==t.child){for(n=Ui(e=t.child,e.pendingProps,e.expirationTime),t.child=n,n.return=t;null!==e.sibling;)e=e.sibling,(n=n.sibling=Ui(e,e.pendingProps,e.expirationTime)).return=t;n.sibling=null}return t.child}function Da(e,t,n){var i=t.expirationTime;if(null!==e){if(e.memoizedProps!==t.pendingProps||Mi.current)wa=!0;else if(i<n){switch(wa=!1,t.tag){case 3:Pa(t),ba();break;case 5:Er(t);break;case 1:Ni(t.type)&&zi(t);break;case 4:kr(t,t.stateNode.containerInfo);break;case 10:ja(t,t.memoizedProps.value);break;case 13:if(null!==t.memoizedState)return 0!==(i=t.child.childExpirationTime)&&i>=n?Ma(e,t,n):null!==(t=Ia(e,t,n))?t.sibling:null}return Ia(e,t,n)}}else wa=!1;switch(t.expirationTime=0,t.tag){case 2:i=t.elementType,null!==e&&(e.alternate=null,t.alternate=null,t.effectTag|=2),e=t.pendingProps;var r=Di(t,Pi.current);if(Ya(t,n),r=$r(null,t,i,e,r,n),t.effectTag|=1,"object"==typeof r&&null!==r&&"function"==typeof r.render&&void 0===r.$$typeof){if(t.tag=1,Zr(),Ni(i)){var a=!0;zi(t)}else a=!1;t.memoizedState=null!==r.state&&void 0!==r.state?r.state:null;var s=i.getDerivedStateFromProps;"function"==typeof s&&ar(t,i,s,e),r.updater=or,t.stateNode=r,r._reactInternalFiber=t,ur(t,i,e,n),t=Oa(null,t,i,!0,a,n)}else t.tag=0,ka(null,t,r,n),t=t.child;return t;case 16:switch(r=t.elementType,null!==e&&(e.alternate=null,t.alternate=null,t.effectTag|=2),a=t.pendingProps,e=function(e){var t=e._result;switch(e._status){case 1:return t;case 2:case 0:throw t;default:switch(e._status=0,(t=(t=e._ctor)()).then(function(t){0===e._status&&(t=t.default,e._status=1,e._result=t)},function(t){0===e._status&&(e._status=2,e._result=t)}),e._status){case 1:return e._result;case 2:throw e._result}throw e._result=t,t}}(r),t.type=e,r=t.tag=function(e){if("function"==typeof e)return qi(e)?1:0;if(null!=e){if((e=e.$$typeof)===tt)return 11;if(e===it)return 14}return 2}(e),a=ir(e,a),s=void 0,r){case 0:s=Aa(null,t,e,a,n);break;case 1:s=_a(null,t,e,a,n);break;case 11:s=Sa(null,t,e,a,n);break;case 14:s=Ea(null,t,e,ir(e.type,a),i,n);break;default:o("306",e,"")}return s;case 0:return i=t.type,r=t.pendingProps,Aa(e,t,i,r=t.elementType===i?r:ir(i,r),n);case 1:return i=t.type,r=t.pendingProps,_a(e,t,i,r=t.elementType===i?r:ir(i,r),n);case 3:return Pa(t),null===(i=t.updateQueue)&&o("282"),r=null!==(r=t.memoizedState)?r.element:null,to(t,i,t.pendingProps,null,n),(i=t.memoizedState.element)===r?(ba(),t=Ia(e,t,n)):(r=t.stateNode,(r=(null===e||null===e.child)&&r.hydrate)&&(da=Ei(t.stateNode.containerInfo),ha=t,r=fa=!0),r?(t.effectTag|=2,t.child=mr(t,null,i,n)):(ka(e,t,i,n),ba()),t=t.child),t;case 5:return Er(t),null===e&&ma(t),i=t.type,r=t.pendingProps,a=null!==e?e.memoizedProps:null,s=r.children,yi(i,r)?s=null:null!==a&&yi(i,a)&&(t.effectTag|=16),Ta(e,t),1!==n&&1&t.mode&&r.hidden?(t.expirationTime=t.childExpirationTime=1,t=null):(ka(e,t,s,n),t=t.child),t;case 6:return null===e&&ma(t),null;case 13:return Ma(e,t,n);case 4:return kr(t,t.stateNode.containerInfo),i=t.pendingProps,null===e?t.child=gr(t,null,i,n):ka(e,t,i,n),t.child;case 11:return i=t.type,r=t.pendingProps,Sa(e,t,i,r=t.elementType===i?r:ir(i,r),n);case 7:return ka(e,t,t.pendingProps,n),t.child;case 8:case 12:return ka(e,t,t.pendingProps.children,n),t.child;case 10:e:{if(i=t.type._context,r=t.pendingProps,s=t.memoizedProps,ja(t,a=r.value),null!==s){var l=s.value;if(0===(a=Kt(l,a)?0:0|("function"==typeof i._calculateChangedBits?i._calculateChangedBits(l,a):1073741823))){if(s.children===r.children&&!Mi.current){t=Ia(e,t,n);break e}}else for(null!==(l=t.child)&&(l.return=t);null!==l;){var c=l.contextDependencies;if(null!==c){s=l.child;for(var u=c.first;null!==u;){if(u.context===i&&0!=(u.observedBits&a)){1===l.tag&&((u=Qa(n)).tag=Va,Za(l,u)),l.expirationTime<n&&(l.expirationTime=n),null!==(u=l.alternate)&&u.expirationTime<n&&(u.expirationTime=n),u=n;for(var h=l.return;null!==h;){var d=h.alternate;if(h.childExpirationTime<u)h.childExpirationTime=u,null!==d&&d.childExpirationTime<u&&(d.childExpirationTime=u);else{if(!(null!==d&&d.childExpirationTime<u))break;d.childExpirationTime=u}h=h.return}c.expirationTime<n&&(c.expirationTime=n);break}u=u.next}}else s=10===l.tag&&l.type===t.type?null:l.child;if(null!==s)s.return=l;else for(s=l;null!==s;){if(s===t){s=null;break}if(null!==(l=s.sibling)){l.return=s.return,s=l;break}s=s.return}l=s}}ka(e,t,r.children,n),t=t.child}return t;case 9:return r=t.type,i=(a=t.pendingProps).children,Ya(t,n),i=i(r=Ha(r,a.unstable_observedBits)),t.effectTag|=1,ka(e,t,i,n),t.child;case 14:return a=ir(r=t.type,t.pendingProps),Ea(e,t,r,a=ir(r.type,a),i,n);case 15:return Ca(e,t,t.type,t.pendingProps,i,n);case 17:return i=t.type,r=t.pendingProps,r=t.elementType===i?r:ir(i,r),null!==e&&(e.alternate=null,t.alternate=null,t.effectTag|=2),t.tag=1,Ni(i)?(e=!0,zi(t)):e=!1,Ya(t,n),lr(t,i,r),ur(t,i,r,n),Oa(null,t,i,!0,e,n)}o("156")}var Na={current:null},La=null,Ra=null,Fa=null;function ja(e,t){var n=e.type._context;_i(Na,n._currentValue),n._currentValue=t}function za(e){var t=Na.current;Ai(Na),e.type._context._currentValue=t}function Ya(e,t){La=e,Fa=Ra=null;var n=e.contextDependencies;null!==n&&n.expirationTime>=t&&(wa=!0),e.contextDependencies=null}function Ha(e,t){return Fa!==e&&!1!==t&&0!==t&&("number"==typeof t&&1073741823!==t||(Fa=e,t=1073741823),t={context:e,observedBits:t,next:null},null===Ra?(null===La&&o("308"),Ra=t,La.contextDependencies={first:t,expirationTime:0}):Ra=Ra.next=t),e._currentValue}var Wa=0,Xa=1,Va=2,Ba=3,qa=!1;function Ua(e){return{baseState:e,firstUpdate:null,lastUpdate:null,firstCapturedUpdate:null,lastCapturedUpdate:null,firstEffect:null,lastEffect:null,firstCapturedEffect:null,lastCapturedEffect:null}}function Ga(e){return{baseState:e.baseState,firstUpdate:e.firstUpdate,lastUpdate:e.lastUpdate,firstCapturedUpdate:null,lastCapturedUpdate:null,firstEffect:null,lastEffect:null,firstCapturedEffect:null,lastCapturedEffect:null}}function Qa(e){return{expirationTime:e,tag:Wa,payload:null,callback:null,next:null,nextEffect:null}}function $a(e,t){null===e.lastUpdate?e.firstUpdate=e.lastUpdate=t:(e.lastUpdate.next=t,e.lastUpdate=t)}function Za(e,t){var n=e.alternate;if(null===n){var i=e.updateQueue,r=null;null===i&&(i=e.updateQueue=Ua(e.memoizedState))}else i=e.updateQueue,r=n.updateQueue,null===i?null===r?(i=e.updateQueue=Ua(e.memoizedState),r=n.updateQueue=Ua(n.memoizedState)):i=e.updateQueue=Ga(r):null===r&&(r=n.updateQueue=Ga(i));null===r||i===r?$a(i,t):null===i.lastUpdate||null===r.lastUpdate?($a(i,t),$a(r,t)):($a(i,t),r.lastUpdate=t)}function Ka(e,t){var n=e.updateQueue;null===(n=null===n?e.updateQueue=Ua(e.memoizedState):Ja(e,n)).lastCapturedUpdate?n.firstCapturedUpdate=n.lastCapturedUpdate=t:(n.lastCapturedUpdate.next=t,n.lastCapturedUpdate=t)}function Ja(e,t){var n=e.alternate;return null!==n&&t===n.updateQueue&&(t=e.updateQueue=Ga(t)),t}function eo(e,t,n,i,a,o){switch(n.tag){case Xa:return"function"==typeof(e=n.payload)?e.call(o,i,a):e;case Ba:e.effectTag=-2049&e.effectTag|64;case Wa:if(null==(a="function"==typeof(e=n.payload)?e.call(o,i,a):e))break;return r({},i,a);case Va:qa=!0}return i}function to(e,t,n,i,r){qa=!1;for(var a=(t=Ja(e,t)).baseState,o=null,s=0,l=t.firstUpdate,c=a;null!==l;){var u=l.expirationTime;u<r?(null===o&&(o=l,a=c),s<u&&(s=u)):(c=eo(e,0,l,c,n,i),null!==l.callback&&(e.effectTag|=32,l.nextEffect=null,null===t.lastEffect?t.firstEffect=t.lastEffect=l:(t.lastEffect.nextEffect=l,t.lastEffect=l))),l=l.next}for(u=null,l=t.firstCapturedUpdate;null!==l;){var h=l.expirationTime;h<r?(null===u&&(u=l,null===o&&(a=c)),s<h&&(s=h)):(c=eo(e,0,l,c,n,i),null!==l.callback&&(e.effectTag|=32,l.nextEffect=null,null===t.lastCapturedEffect?t.firstCapturedEffect=t.lastCapturedEffect=l:(t.lastCapturedEffect.nextEffect=l,t.lastCapturedEffect=l))),l=l.next}null===o&&(t.lastUpdate=null),null===u?t.lastCapturedUpdate=null:e.effectTag|=32,null===o&&null===u&&(a=c),t.baseState=a,t.firstUpdate=o,t.firstCapturedUpdate=u,e.expirationTime=s,e.memoizedState=c}function no(e,t,n){null!==t.firstCapturedUpdate&&(null!==t.lastUpdate&&(t.lastUpdate.next=t.firstCapturedUpdate,t.lastUpdate=t.lastCapturedUpdate),t.firstCapturedUpdate=t.lastCapturedUpdate=null),io(t.firstEffect,n),t.firstEffect=t.lastEffect=null,io(t.firstCapturedEffect,n),t.firstCapturedEffect=t.lastCapturedEffect=null}function io(e,t){for(;null!==e;){var n=e.callback;if(null!==n){e.callback=null;var i=t;"function"!=typeof n&&o("191",n),n.call(i)}e=e.nextEffect}}function ro(e,t){return{value:e,source:t,stack:lt(t)}}function ao(e){e.effectTag|=4}var oo=void 0,so=void 0,lo=void 0,co=void 0;oo=function(e,t){for(var n=t.child;null!==n;){if(5===n.tag||6===n.tag)e.appendChild(n.stateNode);else if(4!==n.tag&&null!==n.child){n.child.return=n,n=n.child;continue}if(n===t)break;for(;null===n.sibling;){if(null===n.return||n.return===t)return;n=n.return}n.sibling.return=n.return,n=n.sibling}},so=function(){},lo=function(e,t,n,i,a){var o=e.memoizedProps;if(o!==i){var s=t.stateNode;switch(wr(yr.current),e=null,n){case"input":o=bt(s,o),i=bt(s,i),e=[];break;case"option":o=Un(s,o),i=Un(s,i),e=[];break;case"select":o=r({},o,{value:void 0}),i=r({},i,{value:void 0}),e=[];break;case"textarea":o=Qn(s,o),i=Qn(s,i),e=[];break;default:"function"!=typeof o.onClick&&"function"==typeof i.onClick&&(s.onclick=pi)}hi(n,i),s=n=void 0;var l=null;for(n in o)if(!i.hasOwnProperty(n)&&o.hasOwnProperty(n)&&null!=o[n])if("style"===n){var c=o[n];for(s in c)c.hasOwnProperty(s)&&(l||(l={}),l[s]="")}else"dangerouslySetInnerHTML"!==n&&"children"!==n&&"suppressContentEditableWarning"!==n&&"suppressHydrationWarning"!==n&&"autoFocus"!==n&&(b.hasOwnProperty(n)?e||(e=[]):(e=e||[]).push(n,null));for(n in i){var u=i[n];if(c=null!=o?o[n]:void 0,i.hasOwnProperty(n)&&u!==c&&(null!=u||null!=c))if("style"===n)if(c){for(s in c)!c.hasOwnProperty(s)||u&&u.hasOwnProperty(s)||(l||(l={}),l[s]="");for(s in u)u.hasOwnProperty(s)&&c[s]!==u[s]&&(l||(l={}),l[s]=u[s])}else l||(e||(e=[]),e.push(n,l)),l=u;else"dangerouslySetInnerHTML"===n?(u=u?u.__html:void 0,c=c?c.__html:void 0,null!=u&&c!==u&&(e=e||[]).push(n,""+u)):"children"===n?c===u||"string"!=typeof u&&"number"!=typeof u||(e=e||[]).push(n,""+u):"suppressContentEditableWarning"!==n&&"suppressHydrationWarning"!==n&&(b.hasOwnProperty(n)?(null!=u&&fi(a,n),e||c===u||(e=[])):(e=e||[]).push(n,u))}l&&(e=e||[]).push("style",l),a=e,(t.updateQueue=a)&&ao(t)}},co=function(e,t,n,i){n!==i&&ao(t)};var uo="function"==typeof WeakSet?WeakSet:Set;function ho(e,t){var n=t.source,i=t.stack;null===i&&null!==n&&(i=lt(n)),null!==n&&st(n.type),t=t.value,null!==e&&1===e.tag&&st(e.type);try{console.error(t)}catch(e){setTimeout(function(){throw e})}}function fo(e){var t=e.ref;if(null!==t)if("function"==typeof t)try{t(null)}catch(t){Go(e,t)}else t.current=null}function po(e,t,n){if(null!==(n=null!==(n=n.updateQueue)?n.lastEffect:null)){var i=n=n.next;do{if((i.tag&e)!==Tr){var r=i.destroy;i.destroy=void 0,void 0!==r&&r()}(i.tag&t)!==Tr&&(r=i.create,i.destroy=r()),i=i.next}while(i!==n)}}function go(e){switch("function"==typeof Wi&&Wi(e),e.tag){case 0:case 11:case 14:case 15:var t=e.updateQueue;if(null!==t&&null!==(t=t.lastEffect)){var n=t=t.next;do{var i=n.destroy;if(void 0!==i){var r=e;try{i()}catch(e){Go(r,e)}}n=n.next}while(n!==t)}break;case 1:if(fo(e),"function"==typeof(t=e.stateNode).componentWillUnmount)try{t.props=e.memoizedProps,t.state=e.memoizedState,t.componentWillUnmount()}catch(t){Go(e,t)}break;case 5:fo(e);break;case 4:yo(e)}}function mo(e){return 5===e.tag||3===e.tag||4===e.tag}function vo(e){e:{for(var t=e.return;null!==t;){if(mo(t)){var n=t;break e}t=t.return}o("160"),n=void 0}var i=t=void 0;switch(n.tag){case 5:t=n.stateNode,i=!1;break;case 3:case 4:t=n.stateNode.containerInfo,i=!0;break;default:o("161")}16&n.effectTag&&(ai(t,""),n.effectTag&=-17);e:t:for(n=e;;){for(;null===n.sibling;){if(null===n.return||mo(n.return)){n=null;break e}n=n.return}for(n.sibling.return=n.return,n=n.sibling;5!==n.tag&&6!==n.tag&&18!==n.tag;){if(2&n.effectTag)continue t;if(null===n.child||4===n.tag)continue t;n.child.return=n,n=n.child}if(!(2&n.effectTag)){n=n.stateNode;break e}}for(var r=e;;){if(5===r.tag||6===r.tag)if(n)if(i){var a=t,s=r.stateNode,l=n;8===a.nodeType?a.parentNode.insertBefore(s,l):a.insertBefore(s,l)}else t.insertBefore(r.stateNode,n);else i?(s=t,l=r.stateNode,8===s.nodeType?(a=s.parentNode).insertBefore(l,s):(a=s).appendChild(l),null!=(s=s._reactRootContainer)||null!==a.onclick||(a.onclick=pi)):t.appendChild(r.stateNode);else if(4!==r.tag&&null!==r.child){r.child.return=r,r=r.child;continue}if(r===e)break;for(;null===r.sibling;){if(null===r.return||r.return===e)return;r=r.return}r.sibling.return=r.return,r=r.sibling}}function yo(e){for(var t=e,n=!1,i=void 0,r=void 0;;){if(!n){n=t.return;e:for(;;){switch(null===n&&o("160"),n.tag){case 5:i=n.stateNode,r=!1;break e;case 3:case 4:i=n.stateNode.containerInfo,r=!0;break e}n=n.return}n=!0}if(5===t.tag||6===t.tag){e:for(var a=t,s=a;;)if(go(s),null!==s.child&&4!==s.tag)s.child.return=s,s=s.child;else{if(s===a)break;for(;null===s.sibling;){if(null===s.return||s.return===a)break e;s=s.return}s.sibling.return=s.return,s=s.sibling}r?(a=i,s=t.stateNode,8===a.nodeType?a.parentNode.removeChild(s):a.removeChild(s)):i.removeChild(t.stateNode)}else if(4===t.tag){if(null!==t.child){i=t.stateNode.containerInfo,r=!0,t.child.return=t,t=t.child;continue}}else if(go(t),null!==t.child){t.child.return=t,t=t.child;continue}if(t===e)break;for(;null===t.sibling;){if(null===t.return||t.return===e)return;4===(t=t.return).tag&&(n=!1)}t.sibling.return=t.return,t=t.sibling}}function bo(e,t){switch(t.tag){case 0:case 11:case 14:case 15:po(_r,Or,t);break;case 1:break;case 5:var n=t.stateNode;if(null!=n){var i=t.memoizedProps;e=null!==e?e.memoizedProps:i;var r=t.type,a=t.updateQueue;t.updateQueue=null,null!==a&&function(e,t,n,i,r){e[N]=r,"input"===n&&"radio"===r.type&&null!=r.name&&wt(e,r),di(n,i),i=di(n,r);for(var a=0;a<t.length;a+=2){var o=t[a],s=t[a+1];"style"===o?ci(e,s):"dangerouslySetInnerHTML"===o?ri(e,s):"children"===o?ai(e,s):vt(e,o,s,i)}switch(n){case"input":kt(e,r);break;case"textarea":Zn(e,r);break;case"select":t=e._wrapperState.wasMultiple,e._wrapperState.wasMultiple=!!r.multiple,null!=(n=r.value)?Gn(e,!!r.multiple,n,!1):t!==!!r.multiple&&(null!=r.defaultValue?Gn(e,!!r.multiple,r.defaultValue,!0):Gn(e,!!r.multiple,r.multiple?[]:"",!1))}}(n,a,r,e,i)}break;case 6:null===t.stateNode&&o("162"),t.stateNode.nodeValue=t.memoizedProps;break;case 3:case 12:break;case 13:if(n=t.memoizedState,i=void 0,e=t,null===n?i=!1:(i=!0,e=t.child,0===n.timedOutAt&&(n.timedOutAt=ks())),null!==e&&function(e,t){for(var n=e;;){if(5===n.tag){var i=n.stateNode;if(t)i.style.display="none";else{i=n.stateNode;var r=n.memoizedProps.style;r=null!=r&&r.hasOwnProperty("display")?r.display:null,i.style.display=li("display",r)}}else if(6===n.tag)n.stateNode.nodeValue=t?"":n.memoizedProps;else{if(13===n.tag&&null!==n.memoizedState){(i=n.child.sibling).return=n,n=i;continue}if(null!==n.child){n.child.return=n,n=n.child;continue}}if(n===e)break;for(;null===n.sibling;){if(null===n.return||n.return===e)return;n=n.return}n.sibling.return=n.return,n=n.sibling}}(e,i),null!==(n=t.updateQueue)){t.updateQueue=null;var s=t.stateNode;null===s&&(s=t.stateNode=new uo),n.forEach(function(e){var n=function(e,t){var n=e.stateNode;null!==n&&n.delete(t),t=Qo(t=ks(),e),null!==(e=Zo(e,t))&&(Ji(e,t),0!==(t=e.expirationTime)&&Ss(e,t))}.bind(null,t,e);s.has(e)||(s.add(e),e.then(n,n))})}break;case 17:break;default:o("163")}}var xo="function"==typeof WeakMap?WeakMap:Map;function wo(e,t,n){(n=Qa(n)).tag=Ba,n.payload={element:null};var i=t.value;return n.callback=function(){Is(i),ho(e,t)},n}function ko(e,t,n){(n=Qa(n)).tag=Ba;var i=e.type.getDerivedStateFromError;if("function"==typeof i){var r=t.value;n.payload=function(){return i(r)}}var a=e.stateNode;return null!==a&&"function"==typeof a.componentDidCatch&&(n.callback=function(){"function"!=typeof i&&(null===jo?jo=new Set([this]):jo.add(this));var n=t.value,r=t.stack;ho(e,t),this.componentDidCatch(n,{componentStack:null!==r?r:""})}),n}function So(e){switch(e.tag){case 1:Ni(e.type)&&Li();var t=e.effectTag;return 2048&t?(e.effectTag=-2049&t|64,e):null;case 3:return Sr(),Ri(),0!=(64&(t=e.effectTag))&&o("285"),e.effectTag=-2049&t|64,e;case 5:return Cr(e),null;case 13:return 2048&(t=e.effectTag)?(e.effectTag=-2049&t|64,e):null;case 18:return null;case 4:return Sr(),null;case 10:return za(e),null;default:return null}}var Eo=Ve.ReactCurrentDispatcher,Co=Ve.ReactCurrentOwner,To=1073741822,Ao=!1,_o=null,Oo=null,Po=0,Mo=-1,Io=!1,Do=null,No=!1,Lo=null,Ro=null,Fo=null,jo=null;function zo(){if(null!==_o)for(var e=_o.return;null!==e;){var t=e;switch(t.tag){case 1:var n=t.type.childContextTypes;null!=n&&Li();break;case 3:Sr(),Ri();break;case 5:Cr(t);break;case 4:Sr();break;case 10:za(t)}e=e.return}Oo=null,Po=0,Mo=-1,Io=!1,_o=null}function Yo(){for(;null!==Do;){var e=Do.effectTag;if(16&e&&ai(Do.stateNode,""),128&e){var t=Do.alternate;null!==t&&(null!==(t=t.ref)&&("function"==typeof t?t(null):t.current=null))}switch(14&e){case 2:vo(Do),Do.effectTag&=-3;break;case 6:vo(Do),Do.effectTag&=-3,bo(Do.alternate,Do);break;case 4:bo(Do.alternate,Do);break;case 8:yo(e=Do),e.return=null,e.child=null,e.memoizedState=null,e.updateQueue=null,null!==(e=e.alternate)&&(e.return=null,e.child=null,e.memoizedState=null,e.updateQueue=null)}Do=Do.nextEffect}}function Ho(){for(;null!==Do;){if(256&Do.effectTag)e:{var e=Do.alternate,t=Do;switch(t.tag){case 0:case 11:case 15:po(Ar,Tr,t);break e;case 1:if(256&t.effectTag&&null!==e){var n=e.memoizedProps,i=e.memoizedState;t=(e=t.stateNode).getSnapshotBeforeUpdate(t.elementType===t.type?n:ir(t.type,n),i),e.__reactInternalSnapshotBeforeUpdate=t}break e;case 3:case 5:case 6:case 4:case 17:break e;default:o("163")}}Do=Do.nextEffect}}function Wo(e,t){for(;null!==Do;){var n=Do.effectTag;if(36&n){var i=Do.alternate,r=Do,a=t;switch(r.tag){case 0:case 11:case 15:po(Pr,Mr,r);break;case 1:var s=r.stateNode;if(4&r.effectTag)if(null===i)s.componentDidMount();else{var l=r.elementType===r.type?i.memoizedProps:ir(r.type,i.memoizedProps);s.componentDidUpdate(l,i.memoizedState,s.__reactInternalSnapshotBeforeUpdate)}null!==(i=r.updateQueue)&&no(0,i,s);break;case 3:if(null!==(i=r.updateQueue)){if(s=null,null!==r.child)switch(r.child.tag){case 5:s=r.child.stateNode;break;case 1:s=r.child.stateNode}no(0,i,s)}break;case 5:a=r.stateNode,null===i&&4&r.effectTag&&vi(r.type,r.memoizedProps)&&a.focus();break;case 6:case 4:case 12:case 13:case 17:break;default:o("163")}}128&n&&(null!==(r=Do.ref)&&(a=Do.stateNode,"function"==typeof r?r(a):r.current=a)),512&n&&(Lo=e),Do=Do.nextEffect}}function Xo(){null!==Ro&&ki(Ro),null!==Fo&&Fo()}function Vo(e,t){No=Ao=!0,e.current===t&&o("177");var n=e.pendingCommitExpirationTime;0===n&&o("261"),e.pendingCommitExpirationTime=0;var i=t.expirationTime,r=t.childExpirationTime;for(function(e,t){if(e.didError=!1,0===t)e.earliestPendingTime=0,e.latestPendingTime=0,e.earliestSuspendedTime=0,e.latestSuspendedTime=0,e.latestPingedTime=0;else{t<e.latestPingedTime&&(e.latestPingedTime=0);var n=e.latestPendingTime;0!==n&&(n>t?e.earliestPendingTime=e.latestPendingTime=0:e.earliestPendingTime>t&&(e.earliestPendingTime=e.latestPendingTime)),0===(n=e.earliestSuspendedTime)?Ji(e,t):t<e.latestSuspendedTime?(e.earliestSuspendedTime=0,e.latestSuspendedTime=0,e.latestPingedTime=0,Ji(e,t)):t>n&&Ji(e,t)}nr(0,e)}(e,r>i?r:i),Co.current=null,i=void 0,1<t.effectTag?null!==t.lastEffect?(t.lastEffect.nextEffect=t,i=t.firstEffect):i=t:i=t.firstEffect,gi=En,mi=function(){var e=Rn();if(Fn(e)){if("selectionStart"in e)var t={start:e.selectionStart,end:e.selectionEnd};else e:{var n=(t=(t=e.ownerDocument)&&t.defaultView||window).getSelection&&t.getSelection();if(n&&0!==n.rangeCount){t=n.anchorNode;var i=n.anchorOffset,r=n.focusNode;n=n.focusOffset;try{t.nodeType,r.nodeType}catch(e){t=null;break e}var a=0,o=-1,s=-1,l=0,c=0,u=e,h=null;t:for(;;){for(var d;u!==t||0!==i&&3!==u.nodeType||(o=a+i),u!==r||0!==n&&3!==u.nodeType||(s=a+n),3===u.nodeType&&(a+=u.nodeValue.length),null!==(d=u.firstChild);)h=u,u=d;for(;;){if(u===e)break t;if(h===t&&++l===i&&(o=a),h===r&&++c===n&&(s=a),null!==(d=u.nextSibling))break;h=(u=h).parentNode}u=d}t=-1===o||-1===s?null:{start:o,end:s}}else t=null}t=t||{start:0,end:0}}else t=null;return{focusedElem:e,selectionRange:t}}(),En=!1,Do=i;null!==Do;){r=!1;var s=void 0;try{Ho()}catch(e){r=!0,s=e}r&&(null===Do&&o("178"),Go(Do,s),null!==Do&&(Do=Do.nextEffect))}for(Do=i;null!==Do;){r=!1,s=void 0;try{Yo()}catch(e){r=!0,s=e}r&&(null===Do&&o("178"),Go(Do,s),null!==Do&&(Do=Do.nextEffect))}for(jn(mi),mi=null,En=!!gi,gi=null,e.current=t,Do=i;null!==Do;){r=!1,s=void 0;try{Wo(e,n)}catch(e){r=!0,s=e}r&&(null===Do&&o("178"),Go(Do,s),null!==Do&&(Do=Do.nextEffect))}if(null!==i&&null!==Lo){var l=function(e,t){Fo=Ro=Lo=null;var n=rs;rs=!0;do{if(512&t.effectTag){var i=!1,r=void 0;try{var a=t;po(Dr,Tr,a),po(Tr,Ir,a)}catch(e){i=!0,r=e}i&&Go(t,r)}t=t.nextEffect}while(null!==t);rs=n,0!==(n=e.expirationTime)&&Ss(e,n),us||rs||_s(1073741823,!1)}.bind(null,e,i);Ro=a.unstable_runWithPriority(a.unstable_NormalPriority,function(){return wi(l)}),Fo=l}Ao=No=!1,"function"==typeof Hi&&Hi(t.stateNode),n=t.expirationTime,0===(t=(t=t.childExpirationTime)>n?t:n)&&(jo=null),function(e,t){e.expirationTime=t,e.finishedWork=null}(e,t)}function Bo(e){for(;;){var t=e.alternate,n=e.return,i=e.sibling;if(0==(1024&e.effectTag)){_o=e;e:{var a=t,s=Po,l=(t=e).pendingProps;switch(t.tag){case 2:case 16:break;case 15:case 0:break;case 1:Ni(t.type)&&Li();break;case 3:Sr(),Ri(),(l=t.stateNode).pendingContext&&(l.context=l.pendingContext,l.pendingContext=null),null!==a&&null!==a.child||(ya(t),t.effectTag&=-3),so(t);break;case 5:Cr(t);var c=wr(xr.current);if(s=t.type,null!==a&&null!=t.stateNode)lo(a,t,s,l,c),a.ref!==t.ref&&(t.effectTag|=128);else if(l){var u=wr(yr.current);if(ya(t)){a=(l=t).stateNode;var h=l.type,d=l.memoizedProps,f=c;switch(a[D]=l,a[N]=d,s=void 0,c=h){case"iframe":case"object":Cn("load",a);break;case"video":case"audio":for(h=0;h<te.length;h++)Cn(te[h],a);break;case"source":Cn("error",a);break;case"img":case"image":case"link":Cn("error",a),Cn("load",a);break;case"form":Cn("reset",a),Cn("submit",a);break;case"details":Cn("toggle",a);break;case"input":xt(a,d),Cn("invalid",a),fi(f,"onChange");break;case"select":a._wrapperState={wasMultiple:!!d.multiple},Cn("invalid",a),fi(f,"onChange");break;case"textarea":$n(a,d),Cn("invalid",a),fi(f,"onChange")}for(s in hi(c,d),h=null,d)d.hasOwnProperty(s)&&(u=d[s],"children"===s?"string"==typeof u?a.textContent!==u&&(h=["children",u]):"number"==typeof u&&a.textContent!==""+u&&(h=["children",""+u]):b.hasOwnProperty(s)&&null!=u&&fi(f,s));switch(c){case"input":We(a),St(a,d,!0);break;case"textarea":We(a),Kn(a);break;case"select":case"option":break;default:"function"==typeof d.onClick&&(a.onclick=pi)}s=h,l.updateQueue=s,(l=null!==s)&&ao(t)}else{d=t,f=s,a=l,h=9===c.nodeType?c:c.ownerDocument,u===Jn.html&&(u=ei(f)),u===Jn.html?"script"===f?((a=h.createElement("div")).innerHTML="<script><\/script>",h=a.removeChild(a.firstChild)):"string"==typeof a.is?h=h.createElement(f,{is:a.is}):(h=h.createElement(f),"select"===f&&(f=h,a.multiple?f.multiple=!0:a.size&&(f.size=a.size))):h=h.createElementNS(u,f),(a=h)[D]=d,a[N]=l,oo(a,t,!1,!1),f=a;var p=c,g=di(h=s,d=l);switch(h){case"iframe":case"object":Cn("load",f),c=d;break;case"video":case"audio":for(c=0;c<te.length;c++)Cn(te[c],f);c=d;break;case"source":Cn("error",f),c=d;break;case"img":case"image":case"link":Cn("error",f),Cn("load",f),c=d;break;case"form":Cn("reset",f),Cn("submit",f),c=d;break;case"details":Cn("toggle",f),c=d;break;case"input":xt(f,d),c=bt(f,d),Cn("invalid",f),fi(p,"onChange");break;case"option":c=Un(f,d);break;case"select":f._wrapperState={wasMultiple:!!d.multiple},c=r({},d,{value:void 0}),Cn("invalid",f),fi(p,"onChange");break;case"textarea":$n(f,d),c=Qn(f,d),Cn("invalid",f),fi(p,"onChange");break;default:c=d}hi(h,c),u=void 0;var m=h,v=f,y=c;for(u in y)if(y.hasOwnProperty(u)){var x=y[u];"style"===u?ci(v,x):"dangerouslySetInnerHTML"===u?null!=(x=x?x.__html:void 0)&&ri(v,x):"children"===u?"string"==typeof x?("textarea"!==m||""!==x)&&ai(v,x):"number"==typeof x&&ai(v,""+x):"suppressContentEditableWarning"!==u&&"suppressHydrationWarning"!==u&&"autoFocus"!==u&&(b.hasOwnProperty(u)?null!=x&&fi(p,u):null!=x&&vt(v,u,x,g))}switch(h){case"input":We(f),St(f,d,!1);break;case"textarea":We(f),Kn(f);break;case"option":null!=d.value&&f.setAttribute("value",""+yt(d.value));break;case"select":(c=f).multiple=!!d.multiple,null!=(f=d.value)?Gn(c,!!d.multiple,f,!1):null!=d.defaultValue&&Gn(c,!!d.multiple,d.defaultValue,!0);break;default:"function"==typeof c.onClick&&(f.onclick=pi)}(l=vi(s,l))&&ao(t),t.stateNode=a}null!==t.ref&&(t.effectTag|=128)}else null===t.stateNode&&o("166");break;case 6:a&&null!=t.stateNode?co(a,t,a.memoizedProps,l):("string"!=typeof l&&(null===t.stateNode&&o("166")),a=wr(xr.current),wr(yr.current),ya(t)?(s=(l=t).stateNode,a=l.memoizedProps,s[D]=l,(l=s.nodeValue!==a)&&ao(t)):(s=t,(l=(9===a.nodeType?a:a.ownerDocument).createTextNode(l))[D]=t,s.stateNode=l));break;case 11:break;case 13:if(l=t.memoizedState,0!=(64&t.effectTag)){t.expirationTime=s,_o=t;break e}l=null!==l,s=null!==a&&null!==a.memoizedState,null!==a&&!l&&s&&(null!==(a=a.child.sibling)&&(null!==(c=t.firstEffect)?(t.firstEffect=a,a.nextEffect=c):(t.firstEffect=t.lastEffect=a,a.nextEffect=null),a.effectTag=8)),(l||s)&&(t.effectTag|=4);break;case 7:case 8:case 12:break;case 4:Sr(),so(t);break;case 10:za(t);break;case 9:case 14:break;case 17:Ni(t.type)&&Li();break;case 18:break;default:o("156")}_o=null}if(t=e,1===Po||1!==t.childExpirationTime){for(l=0,s=t.child;null!==s;)(a=s.expirationTime)>l&&(l=a),(c=s.childExpirationTime)>l&&(l=c),s=s.sibling;t.childExpirationTime=l}if(null!==_o)return _o;null!==n&&0==(1024&n.effectTag)&&(null===n.firstEffect&&(n.firstEffect=e.firstEffect),null!==e.lastEffect&&(null!==n.lastEffect&&(n.lastEffect.nextEffect=e.firstEffect),n.lastEffect=e.lastEffect),1<e.effectTag&&(null!==n.lastEffect?n.lastEffect.nextEffect=e:n.firstEffect=e,n.lastEffect=e))}else{if(null!==(e=So(e)))return e.effectTag&=1023,e;null!==n&&(n.firstEffect=n.lastEffect=null,n.effectTag|=1024)}if(null!==i)return i;if(null===n)break;e=n}return null}function qo(e){var t=Da(e.alternate,e,Po);return e.memoizedProps=e.pendingProps,null===t&&(t=Bo(e)),Co.current=null,t}function Uo(e,t){Ao&&o("243"),Xo(),Ao=!0;var n=Eo.current;Eo.current=la;var i=e.nextExpirationTimeToWorkOn;i===Po&&e===Oo&&null!==_o||(zo(),Po=i,_o=Ui((Oo=e).current,null),e.pendingCommitExpirationTime=0);for(var r=!1;;){try{if(t)for(;null!==_o&&!Ts();)_o=qo(_o);else for(;null!==_o;)_o=qo(_o)}catch(t){if(Fa=Ra=La=null,Zr(),null===_o)r=!0,Is(t);else{null===_o&&o("271");var a=_o,s=a.return;if(null!==s){e:{var l=e,c=s,u=a,h=t;if(s=Po,u.effectTag|=1024,u.firstEffect=u.lastEffect=null,null!==h&&"object"==typeof h&&"function"==typeof h.then){var d=h;h=c;var f=-1,p=-1;do{if(13===h.tag){var g=h.alternate;if(null!==g&&null!==(g=g.memoizedState)){p=10*(1073741822-g.timedOutAt);break}"number"==typeof(g=h.pendingProps.maxDuration)&&(0>=g?f=0:(-1===f||g<f)&&(f=g))}h=h.return}while(null!==h);h=c;do{if((g=13===h.tag)&&(g=void 0!==h.memoizedProps.fallback&&null===h.memoizedState),g){if(null===(c=h.updateQueue)?((c=new Set).add(d),h.updateQueue=c):c.add(d),0==(1&h.mode)){h.effectTag|=64,u.effectTag&=-1957,1===u.tag&&(null===u.alternate?u.tag=17:((s=Qa(1073741823)).tag=Va,Za(u,s))),u.expirationTime=1073741823;break e}c=s;var m=(u=l).pingCache;null===m?(m=u.pingCache=new xo,g=new Set,m.set(d,g)):void 0===(g=m.get(d))&&(g=new Set,m.set(d,g)),g.has(c)||(g.add(c),u=$o.bind(null,u,d,c),d.then(u,u)),-1===f?l=1073741823:(-1===p&&(p=10*(1073741822-tr(l,s))-5e3),l=p+f),0<=l&&Mo<l&&(Mo=l),h.effectTag|=2048,h.expirationTime=s;break e}h=h.return}while(null!==h);h=Error((st(u.type)||"A React component")+" suspended while rendering, but no fallback UI was specified.\n\nAdd a <Suspense fallback=...> component higher in the tree to provide a loading indicator or placeholder to display."+lt(u))}Io=!0,h=ro(h,u),l=c;do{switch(l.tag){case 3:l.effectTag|=2048,l.expirationTime=s,Ka(l,s=wo(l,h,s));break e;case 1:if(f=h,p=l.type,u=l.stateNode,0==(64&l.effectTag)&&("function"==typeof p.getDerivedStateFromError||null!==u&&"function"==typeof u.componentDidCatch&&(null===jo||!jo.has(u)))){l.effectTag|=2048,l.expirationTime=s,Ka(l,s=ko(l,f,s));break e}}l=l.return}while(null!==l)}_o=Bo(a);continue}r=!0,Is(t)}}break}if(Ao=!1,Eo.current=n,Fa=Ra=La=null,Zr(),r)Oo=null,e.finishedWork=null;else if(null!==_o)e.finishedWork=null;else{if(null===(n=e.current.alternate)&&o("281"),Oo=null,Io){if(r=e.latestPendingTime,a=e.latestSuspendedTime,s=e.latestPingedTime,0!==r&&r<i||0!==a&&a<i||0!==s&&s<i)return er(e,i),void ws(e,n,i,e.expirationTime,-1);if(!e.didError&&t)return e.didError=!0,i=e.nextExpirationTimeToWorkOn=i,t=e.expirationTime=1073741823,void ws(e,n,i,t,-1)}t&&-1!==Mo?(er(e,i),(t=10*(1073741822-tr(e,i)))<Mo&&(Mo=t),t=10*(1073741822-ks()),t=Mo-t,ws(e,n,i,e.expirationTime,0>t?0:t)):(e.pendingCommitExpirationTime=i,e.finishedWork=n)}}function Go(e,t){for(var n=e.return;null!==n;){switch(n.tag){case 1:var i=n.stateNode;if("function"==typeof n.type.getDerivedStateFromError||"function"==typeof i.componentDidCatch&&(null===jo||!jo.has(i)))return Za(n,e=ko(n,e=ro(t,e),1073741823)),void Ko(n,1073741823);break;case 3:return Za(n,e=wo(n,e=ro(t,e),1073741823)),void Ko(n,1073741823)}n=n.return}3===e.tag&&(Za(e,n=wo(e,n=ro(t,e),1073741823)),Ko(e,1073741823))}function Qo(e,t){var n=a.unstable_getCurrentPriorityLevel(),i=void 0;if(0==(1&t.mode))i=1073741823;else if(Ao&&!No)i=Po;else{switch(n){case a.unstable_ImmediatePriority:i=1073741823;break;case a.unstable_UserBlockingPriority:i=1073741822-10*(1+((1073741822-e+15)/10|0));break;case a.unstable_NormalPriority:i=1073741822-25*(1+((1073741822-e+500)/25|0));break;case a.unstable_LowPriority:case a.unstable_IdlePriority:i=1;break;default:o("313")}null!==Oo&&i===Po&&--i}return n===a.unstable_UserBlockingPriority&&(0===ss||i<ss)&&(ss=i),i}function $o(e,t,n){var i=e.pingCache;null!==i&&i.delete(t),null!==Oo&&Po===n?Oo=null:(t=e.earliestSuspendedTime,i=e.latestSuspendedTime,0!==t&&n<=t&&n>=i&&(e.didError=!1,(0===(t=e.latestPingedTime)||t>n)&&(e.latestPingedTime=n),nr(n,e),0!==(n=e.expirationTime)&&Ss(e,n)))}function Zo(e,t){e.expirationTime<t&&(e.expirationTime=t);var n=e.alternate;null!==n&&n.expirationTime<t&&(n.expirationTime=t);var i=e.return,r=null;if(null===i&&3===e.tag)r=e.stateNode;else for(;null!==i;){if(n=i.alternate,i.childExpirationTime<t&&(i.childExpirationTime=t),null!==n&&n.childExpirationTime<t&&(n.childExpirationTime=t),null===i.return&&3===i.tag){r=i.stateNode;break}i=i.return}return r}function Ko(e,t){null!==(e=Zo(e,t))&&(!Ao&&0!==Po&&t>Po&&zo(),Ji(e,t),Ao&&!No&&Oo===e||Ss(e,e.expirationTime),vs>ms&&(vs=0,o("185")))}function Jo(e,t,n,i,r){return a.unstable_runWithPriority(a.unstable_ImmediatePriority,function(){return e(t,n,i,r)})}var es=null,ts=null,ns=0,is=void 0,rs=!1,as=null,os=0,ss=0,ls=!1,cs=null,us=!1,hs=!1,ds=null,fs=a.unstable_now(),ps=1073741822-(fs/10|0),gs=ps,ms=50,vs=0,ys=null;function bs(){ps=1073741822-((a.unstable_now()-fs)/10|0)}function xs(e,t){if(0!==ns){if(t<ns)return;null!==is&&a.unstable_cancelCallback(is)}ns=t,e=a.unstable_now()-fs,is=a.unstable_scheduleCallback(As,{timeout:10*(1073741822-t)-e})}function ws(e,t,n,i,r){e.expirationTime=i,0!==r||Ts()?0<r&&(e.timeoutHandle=bi(function(e,t,n){e.pendingCommitExpirationTime=n,e.finishedWork=t,bs(),gs=ps,Os(e,n)}.bind(null,e,t,n),r)):(e.pendingCommitExpirationTime=n,e.finishedWork=t)}function ks(){return rs?gs:(Es(),0!==os&&1!==os||(bs(),gs=ps),gs)}function Ss(e,t){null===e.nextScheduledRoot?(e.expirationTime=t,null===ts?(es=ts=e,e.nextScheduledRoot=e):(ts=ts.nextScheduledRoot=e).nextScheduledRoot=es):t>e.expirationTime&&(e.expirationTime=t),rs||(us?hs&&(as=e,os=1073741823,Ps(e,1073741823,!1)):1073741823===t?_s(1073741823,!1):xs(e,t))}function Es(){var e=0,t=null;if(null!==ts)for(var n=ts,i=es;null!==i;){var r=i.expirationTime;if(0===r){if((null===n||null===ts)&&o("244"),i===i.nextScheduledRoot){es=ts=i.nextScheduledRoot=null;break}if(i===es)es=r=i.nextScheduledRoot,ts.nextScheduledRoot=r,i.nextScheduledRoot=null;else{if(i===ts){(ts=n).nextScheduledRoot=es,i.nextScheduledRoot=null;break}n.nextScheduledRoot=i.nextScheduledRoot,i.nextScheduledRoot=null}i=n.nextScheduledRoot}else{if(r>e&&(e=r,t=i),i===ts)break;if(1073741823===e)break;n=i,i=i.nextScheduledRoot}}as=t,os=e}var Cs=!1;function Ts(){return!!Cs||!!a.unstable_shouldYield()&&(Cs=!0)}function As(){try{if(!Ts()&&null!==es){bs();var e=es;do{var t=e.expirationTime;0!==t&&ps<=t&&(e.nextExpirationTimeToWorkOn=ps),e=e.nextScheduledRoot}while(e!==es)}_s(0,!0)}finally{Cs=!1}}function _s(e,t){if(Es(),t)for(bs(),gs=ps;null!==as&&0!==os&&e<=os&&!(Cs&&ps>os);)Ps(as,os,ps>os),Es(),bs(),gs=ps;else for(;null!==as&&0!==os&&e<=os;)Ps(as,os,!1),Es();if(t&&(ns=0,is=null),0!==os&&xs(as,os),vs=0,ys=null,null!==ds)for(e=ds,ds=null,t=0;t<e.length;t++){var n=e[t];try{n._onComplete()}catch(e){ls||(ls=!0,cs=e)}}if(ls)throw e=cs,cs=null,ls=!1,e}function Os(e,t){rs&&o("253"),as=e,os=t,Ps(e,t,!1),_s(1073741823,!1)}function Ps(e,t,n){if(rs&&o("245"),rs=!0,n){var i=e.finishedWork;null!==i?Ms(e,i,t):(e.finishedWork=null,-1!==(i=e.timeoutHandle)&&(e.timeoutHandle=-1,xi(i)),Uo(e,n),null!==(i=e.finishedWork)&&(Ts()?e.finishedWork=i:Ms(e,i,t)))}else null!==(i=e.finishedWork)?Ms(e,i,t):(e.finishedWork=null,-1!==(i=e.timeoutHandle)&&(e.timeoutHandle=-1,xi(i)),Uo(e,n),null!==(i=e.finishedWork)&&Ms(e,i,t));rs=!1}function Ms(e,t,n){var i=e.firstBatch;if(null!==i&&i._expirationTime>=n&&(null===ds?ds=[i]:ds.push(i),i._defer))return e.finishedWork=t,void(e.expirationTime=0);e.finishedWork=null,e===ys?vs++:(ys=e,vs=0),a.unstable_runWithPriority(a.unstable_ImmediatePriority,function(){Vo(e,t)})}function Is(e){null===as&&o("246"),as.expirationTime=0,ls||(ls=!0,cs=e)}function Ds(e,t){var n=us;us=!0;try{return e(t)}finally{(us=n)||rs||_s(1073741823,!1)}}function Ns(e,t){if(us&&!hs){hs=!0;try{return e(t)}finally{hs=!1}}return e(t)}function Ls(e,t,n){us||rs||0===ss||(_s(ss,!1),ss=0);var i=us;us=!0;try{return a.unstable_runWithPriority(a.unstable_UserBlockingPriority,function(){return e(t,n)})}finally{(us=i)||rs||_s(1073741823,!1)}}function Rs(e,t,n,i,r){var a=t.current;e:if(n){t:{2===tn(n=n._reactInternalFiber)&&1===n.tag||o("170");var s=n;do{switch(s.tag){case 3:s=s.stateNode.context;break t;case 1:if(Ni(s.type)){s=s.stateNode.__reactInternalMemoizedMergedChildContext;break t}}s=s.return}while(null!==s);o("171"),s=void 0}if(1===n.tag){var l=n.type;if(Ni(l)){n=ji(n,l,s);break e}}n=s}else n=Oi;return null===t.context?t.context=n:t.pendingContext=n,t=r,(r=Qa(i)).payload={element:e},null!==(t=void 0===t?null:t)&&(r.callback=t),Xo(),Za(a,r),Ko(a,i),i}function Fs(e,t,n,i){var r=t.current;return Rs(e,t,n,r=Qo(ks(),r),i)}function js(e){if(!(e=e.current).child)return null;switch(e.child.tag){case 5:default:return e.child.stateNode}}function zs(e){var t=1073741822-25*(1+((1073741822-ks()+500)/25|0));t>=To&&(t=To-1),this._expirationTime=To=t,this._root=e,this._callbacks=this._next=null,this._hasChildren=this._didComplete=!1,this._children=null,this._defer=!0}function Ys(){this._callbacks=null,this._didCommit=!1,this._onCommit=this._onCommit.bind(this)}function Hs(e,t,n){e={current:t=Bi(3,null,null,t?3:0),containerInfo:e,pendingChildren:null,pingCache:null,earliestPendingTime:0,latestPendingTime:0,earliestSuspendedTime:0,latestSuspendedTime:0,latestPingedTime:0,didError:!1,pendingCommitExpirationTime:0,finishedWork:null,timeoutHandle:-1,context:null,pendingContext:null,hydrate:n,nextExpirationTimeToWorkOn:0,expirationTime:0,firstBatch:null,nextScheduledRoot:null},this._internalRoot=t.stateNode=e}function Ws(e){return!(!e||1!==e.nodeType&&9!==e.nodeType&&11!==e.nodeType&&(8!==e.nodeType||" react-mount-point-unstable "!==e.nodeValue))}function Xs(e,t,n,i,r){var a=n._reactRootContainer;if(a){if("function"==typeof r){var o=r;r=function(){var e=js(a._internalRoot);o.call(e)}}null!=e?a.legacy_renderSubtreeIntoContainer(e,t,r):a.render(t,r)}else{if(a=n._reactRootContainer=function(e,t){if(t||(t=!(!(t=e?9===e.nodeType?e.documentElement:e.firstChild:null)||1!==t.nodeType||!t.hasAttribute("data-reactroot"))),!t)for(var n;n=e.lastChild;)e.removeChild(n);return new Hs(e,!1,t)}(n,i),"function"==typeof r){var s=r;r=function(){var e=js(a._internalRoot);s.call(e)}}Ns(function(){null!=e?a.legacy_renderSubtreeIntoContainer(e,t,r):a.render(t,r)})}return js(a._internalRoot)}function Vs(e,t){var n=2<arguments.length&&void 0!==arguments[2]?arguments[2]:null;return Ws(t)||o("200"),function(e,t,n){var i=3<arguments.length&&void 0!==arguments[3]?arguments[3]:null;return{$$typeof:Ge,key:null==i?null:""+i,children:e,containerInfo:t,implementation:n}}(e,t,null,n)}Te=function(e,t,n){switch(t){case"input":if(kt(e,n),t=n.name,"radio"===n.type&&null!=t){for(n=e;n.parentNode;)n=n.parentNode;for(n=n.querySelectorAll("input[name="+JSON.stringify(""+t)+'][type="radio"]'),t=0;t<n.length;t++){var i=n[t];if(i!==e&&i.form===e.form){var r=j(i);r||o("90"),Xe(i),kt(i,r)}}}break;case"textarea":Zn(e,n);break;case"select":null!=(t=n.value)&&Gn(e,!!n.multiple,t,!1)}},zs.prototype.render=function(e){this._defer||o("250"),this._hasChildren=!0,this._children=e;var t=this._root._internalRoot,n=this._expirationTime,i=new Ys;return Rs(e,t,null,n,i._onCommit),i},zs.prototype.then=function(e){if(this._didComplete)e();else{var t=this._callbacks;null===t&&(t=this._callbacks=[]),t.push(e)}},zs.prototype.commit=function(){var e=this._root._internalRoot,t=e.firstBatch;if(this._defer&&null!==t||o("251"),this._hasChildren){var n=this._expirationTime;if(t!==this){this._hasChildren&&(n=this._expirationTime=t._expirationTime,this.render(this._children));for(var i=null,r=t;r!==this;)i=r,r=r._next;null===i&&o("251"),i._next=r._next,this._next=t,e.firstBatch=this}this._defer=!1,Os(e,n),t=this._next,this._next=null,null!==(t=e.firstBatch=t)&&t._hasChildren&&t.render(t._children)}else this._next=null,this._defer=!1},zs.prototype._onComplete=function(){if(!this._didComplete){this._didComplete=!0;var e=this._callbacks;if(null!==e)for(var t=0;t<e.length;t++)(0,e[t])()}},Ys.prototype.then=function(e){if(this._didCommit)e();else{var t=this._callbacks;null===t&&(t=this._callbacks=[]),t.push(e)}},Ys.prototype._onCommit=function(){if(!this._didCommit){this._didCommit=!0;var e=this._callbacks;if(null!==e)for(var t=0;t<e.length;t++){var n=e[t];"function"!=typeof n&&o("191",n),n()}}},Hs.prototype.render=function(e,t){var n=this._internalRoot,i=new Ys;return null!==(t=void 0===t?null:t)&&i.then(t),Fs(e,n,null,i._onCommit),i},Hs.prototype.unmount=function(e){var t=this._internalRoot,n=new Ys;return null!==(e=void 0===e?null:e)&&n.then(e),Fs(null,t,null,n._onCommit),n},Hs.prototype.legacy_renderSubtreeIntoContainer=function(e,t,n){var i=this._internalRoot,r=new Ys;return null!==(n=void 0===n?null:n)&&r.then(n),Fs(t,i,e,r._onCommit),r},Hs.prototype.createBatch=function(){var e=new zs(this),t=e._expirationTime,n=this._internalRoot,i=n.firstBatch;if(null===i)n.firstBatch=e,e._next=null;else{for(n=null;null!==i&&i._expirationTime>=t;)n=i,i=i._next;e._next=i,null!==n&&(n._next=e)}return e},Ie=Ds,De=Ls,Ne=function(){rs||0===ss||(_s(ss,!1),ss=0)};var Bs={createPortal:Vs,findDOMNode:function(e){if(null==e)return null;if(1===e.nodeType)return e;var t=e._reactInternalFiber;return void 0===t&&("function"==typeof e.render?o("188"):o("268",Object.keys(e))),e=null===(e=rn(t))?null:e.stateNode},hydrate:function(e,t,n){return Ws(t)||o("200"),Xs(null,e,t,!0,n)},render:function(e,t,n){return Ws(t)||o("200"),Xs(null,e,t,!1,n)},unstable_renderSubtreeIntoContainer:function(e,t,n,i){return Ws(n)||o("200"),(null==e||void 0===e._reactInternalFiber)&&o("38"),Xs(e,t,n,!1,i)},unmountComponentAtNode:function(e){return Ws(e)||o("40"),!!e._reactRootContainer&&(Ns(function(){Xs(null,null,e,!1,function(){e._reactRootContainer=null})}),!0)},unstable_createPortal:function(){return Vs.apply(void 0,arguments)},unstable_batchedUpdates:Ds,unstable_interactiveUpdates:Ls,flushSync:function(e,t){rs&&o("187");var n=us;us=!0;try{return Jo(e,t)}finally{us=n,_s(1073741823,!1)}},unstable_createRoot:function(e,t){return Ws(e)||o("299","unstable_createRoot"),new Hs(e,!0,null!=t&&!0===t.hydrate)},unstable_flushControlled:function(e){var t=us;us=!0;try{Jo(e)}finally{(us=t)||rs||_s(1073741823,!1)}},__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED:{Events:[R,F,j,O.injectEventPluginsByName,y,V,function(e){T(e,X)},Pe,Me,_n,M]}};!function(e){var t=e.findFiberByHostInstance;(function(e){if("undefined"==typeof __REACT_DEVTOOLS_GLOBAL_HOOK__)return!1;var t=__REACT_DEVTOOLS_GLOBAL_HOOK__;if(t.isDisabled||!t.supportsFiber)return!0;try{var n=t.inject(e);Hi=Xi(function(e){return t.onCommitFiberRoot(n,e)}),Wi=Xi(function(e){return t.onCommitFiberUnmount(n,e)})}catch(e){}})(r({},e,{overrideProps:null,currentDispatcherRef:Ve.ReactCurrentDispatcher,findHostInstanceByFiber:function(e){return null===(e=rn(e))?null:e.stateNode},findFiberByHostInstance:function(e){return t?t(e):null}}))}({findFiberByHostInstance:L,bundleType:0,version:"16.8.6",rendererPackageName:"react-dom"});var qs={default:Bs},Us=qs&&Bs||qs;e.exports=Us.default||Us},function(e,t,n){"use strict";e.exports=n(83)},function(e,t,n){"use strict";(function(e){
/** @license React v0.13.6
 * scheduler.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
Object.defineProperty(t,"__esModule",{value:!0});var n=null,i=!1,r=3,a=-1,o=-1,s=!1,l=!1;function c(){if(!s){var e=n.expirationTime;l?S():l=!0,k(d,e)}}function u(){var e=n,t=n.next;if(n===t)n=null;else{var i=n.previous;n=i.next=t,t.previous=i}e.next=e.previous=null,i=e.callback,t=e.expirationTime,e=e.priorityLevel;var a=r,s=o;r=e,o=t;try{var l=i()}finally{r=a,o=s}if("function"==typeof l)if(l={callback:l,priorityLevel:e,expirationTime:t,next:null,previous:null},null===n)n=l.next=l.previous=l;else{i=null,e=n;do{if(e.expirationTime>=t){i=e;break}e=e.next}while(e!==n);null===i?i=n:i===n&&(n=l,c()),(t=i.previous).next=i.previous=l,l.next=i,l.previous=t}}function h(){if(-1===a&&null!==n&&1===n.priorityLevel){s=!0;try{do{u()}while(null!==n&&1===n.priorityLevel)}finally{s=!1,null!==n?c():l=!1}}}function d(e){s=!0;var r=i;i=e;try{if(e)for(;null!==n;){var a=t.unstable_now();if(!(n.expirationTime<=a))break;do{u()}while(null!==n&&n.expirationTime<=a)}else if(null!==n)do{u()}while(null!==n&&!E())}finally{s=!1,i=r,null!==n?c():l=!1,h()}}var f,p,g=Date,m="function"==typeof setTimeout?setTimeout:void 0,v="function"==typeof clearTimeout?clearTimeout:void 0,y="function"==typeof requestAnimationFrame?requestAnimationFrame:void 0,b="function"==typeof cancelAnimationFrame?cancelAnimationFrame:void 0;function x(e){f=y(function(t){v(p),e(t)}),p=m(function(){b(f),e(t.unstable_now())},100)}if("object"==typeof performance&&"function"==typeof performance.now){var w=performance;t.unstable_now=function(){return w.now()}}else t.unstable_now=function(){return g.now()};var k,S,E,C=null;if("undefined"!=typeof window?C=window:void 0!==e&&(C=e),C&&C._schedMock){var T=C._schedMock;k=T[0],S=T[1],E=T[2],t.unstable_now=T[3]}else if("undefined"==typeof window||"function"!=typeof MessageChannel){var A=null,_=function(e){if(null!==A)try{A(e)}finally{A=null}};k=function(e){null!==A?setTimeout(k,0,e):(A=e,setTimeout(_,0,!1))},S=function(){A=null},E=function(){return!1}}else{"undefined"!=typeof console&&("function"!=typeof y&&console.error("This browser doesn't support requestAnimationFrame. Make sure that you load a polyfill in older browsers. https://fb.me/react-polyfills"),"function"!=typeof b&&console.error("This browser doesn't support cancelAnimationFrame. Make sure that you load a polyfill in older browsers. https://fb.me/react-polyfills"));var O=null,P=!1,M=-1,I=!1,D=!1,N=0,L=33,R=33;E=function(){return N<=t.unstable_now()};var F=new MessageChannel,j=F.port2;F.port1.onmessage=function(){P=!1;var e=O,n=M;O=null,M=-1;var i=t.unstable_now(),r=!1;if(0>=N-i){if(!(-1!==n&&n<=i))return I||(I=!0,x(z)),O=e,void(M=n);r=!0}if(null!==e){D=!0;try{e(r)}finally{D=!1}}};var z=function(e){if(null!==O){x(z);var t=e-N+R;t<R&&L<R?(8>t&&(t=8),R=t<L?L:t):L=t,N=e+R,P||(P=!0,j.postMessage(void 0))}else I=!1};k=function(e,t){O=e,M=t,D||0>t?j.postMessage(void 0):I||(I=!0,x(z))},S=function(){O=null,P=!1,M=-1}}t.unstable_ImmediatePriority=1,t.unstable_UserBlockingPriority=2,t.unstable_NormalPriority=3,t.unstable_IdlePriority=5,t.unstable_LowPriority=4,t.unstable_runWithPriority=function(e,n){switch(e){case 1:case 2:case 3:case 4:case 5:break;default:e=3}var i=r,o=a;r=e,a=t.unstable_now();try{return n()}finally{r=i,a=o,h()}},t.unstable_next=function(e){switch(r){case 1:case 2:case 3:var n=3;break;default:n=r}var i=r,o=a;r=n,a=t.unstable_now();try{return e()}finally{r=i,a=o,h()}},t.unstable_scheduleCallback=function(e,i){var o=-1!==a?a:t.unstable_now();if("object"==typeof i&&null!==i&&"number"==typeof i.timeout)i=o+i.timeout;else switch(r){case 1:i=o+-1;break;case 2:i=o+250;break;case 5:i=o+1073741823;break;case 4:i=o+1e4;break;default:i=o+5e3}if(e={callback:e,priorityLevel:r,expirationTime:i,next:null,previous:null},null===n)n=e.next=e.previous=e,c();else{o=null;var s=n;do{if(s.expirationTime>i){o=s;break}s=s.next}while(s!==n);null===o?o=n:o===n&&(n=e,c()),(i=o.previous).next=o.previous=e,e.next=o,e.previous=i}return e},t.unstable_cancelCallback=function(e){var t=e.next;if(null!==t){if(t===e)n=null;else{e===n&&(n=t);var i=e.previous;i.next=t,t.previous=i}e.next=e.previous=null}},t.unstable_wrapCallback=function(e){var n=r;return function(){var i=r,o=a;r=n,a=t.unstable_now();try{return e.apply(this,arguments)}finally{r=i,a=o,h()}}},t.unstable_getCurrentPriorityLevel=function(){return r},t.unstable_shouldYield=function(){return!i&&(null!==n&&n.expirationTime<o||E())},t.unstable_continueExecution=function(){null!==n&&c()},t.unstable_pauseExecution=function(){},t.unstable_getFirstCallbackNode=function(){return n}}).call(this,n(14))},function(e,t,n){"use strict";(function(i){var r;
/*!
 * ApexCharts v3.6.10
 * (c) 2018-2019 Juned Chhipa
 * Released under the MIT License.
 */function a(e){return(a="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function s(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function l(e,t,n){return t&&s(e.prototype,t),n&&s(e,n),e}function c(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function u(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{},i=Object.keys(n);"function"==typeof Object.getOwnPropertySymbols&&(i=i.concat(Object.getOwnPropertySymbols(n).filter(function(e){return Object.getOwnPropertyDescriptor(n,e).enumerable}))),i.forEach(function(t){c(e,t,n[t])})}return e}function h(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&f(e,t)}function d(e){return(d=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}function f(e,t){return(f=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function p(e,t){return!t||"object"!=typeof t&&"function"!=typeof t?function(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}(e):t}function g(e){return function(e){if(Array.isArray(e)){for(var t=0,n=new Array(e.length);t<e.length;t++)n[t]=e[t];return n}}(e)||function(e){if(Symbol.iterator in Object(e)||"[object Arguments]"===Object.prototype.toString.call(e))return Array.from(e)}(e)||function(){throw new TypeError("Invalid attempt to spread non-iterable instance")}()}var m=function(){function e(){o(this,e)}return l(e,[{key:"shadeRGBColor",value:function(e,t){var n=t.split(","),i=e<0?0:255,r=e<0?-1*e:e,a=parseInt(n[0].slice(4)),o=parseInt(n[1]),s=parseInt(n[2]);return"rgb("+(Math.round((i-a)*r)+a)+","+(Math.round((i-o)*r)+o)+","+(Math.round((i-s)*r)+s)+")"}},{key:"shadeHexColor",value:function(e,t){var n=parseInt(t.slice(1),16),i=e<0?0:255,r=e<0?-1*e:e,a=n>>16,o=n>>8&255,s=255&n;return"#"+(16777216+65536*(Math.round((i-a)*r)+a)+256*(Math.round((i-o)*r)+o)+(Math.round((i-s)*r)+s)).toString(16).slice(1)}},{key:"shadeColor",value:function(e,t){return t.length>7?this.shadeRGBColor(e,t):this.shadeHexColor(e,t)}}],[{key:"bind",value:function(e,t){return function(){return e.apply(t,arguments)}}},{key:"isObject",value:function(e){return e&&"object"===a(e)&&!Array.isArray(e)&&null!=e}},{key:"listToArray",value:function(e){var t,n=[];for(t=0;t<e.length;t++)n[t]=e[t];return n}},{key:"extend",value:function(e,t){var n=this;"function"!=typeof Object.assign&&(Object.assign=function(e){if(null==e)throw new TypeError("Cannot convert undefined or null to object");for(var t=Object(e),n=1;n<arguments.length;n++){var i=arguments[n];if(null!=i)for(var r in i)i.hasOwnProperty(r)&&(t[r]=i[r])}return t});var i=Object.assign({},e);return this.isObject(e)&&this.isObject(t)&&Object.keys(t).forEach(function(r){n.isObject(t[r])&&r in e?i[r]=n.extend(e[r],t[r]):Object.assign(i,c({},r,t[r]))}),i}},{key:"extendArray",value:function(t,n){var i=[];return t.map(function(t){i.push(e.extend(n,t))}),t=i}},{key:"monthMod",value:function(e){return e%12}},{key:"addProps",value:function(e,t,n){"string"==typeof t&&(t=t.split(".")),e[t[0]]=e[t[0]]||{};var i=e[t[0]];return t.length>1?(t.shift(),this.addProps(i,t,n)):e[t[0]]=n,e}},{key:"clone",value:function(e){if("[object Array]"===Object.prototype.toString.call(e)){for(var t=[],n=0;n<e.length;n++)t[n]=this.clone(e[n]);return t}if("object"===a(e)){var i={};for(var r in e)e.hasOwnProperty(r)&&(i[r]=this.clone(e[r]));return i}return e}},{key:"log10",value:function(e){return Math.log(e)/Math.LN10}},{key:"roundToBase10",value:function(e){return Math.pow(10,Math.floor(Math.log10(e)))}},{key:"roundToBase",value:function(e,t){return Math.pow(t,Math.floor(Math.log(e)/Math.log(t)))}},{key:"parseNumber",value:function(e){return null===e?e:parseFloat(e)}},{key:"noExponents",value:function(e){var t=String(e).split(/[eE]/);if(1==t.length)return t[0];var n="",i=e<0?"-":"",r=t[0].replace(".",""),a=Number(t[1])+1;if(a<0){for(n=i+"0.";a++;)n+="0";return n+r.replace(/^\-/,"")}for(a-=r.length;a--;)n+="0";return r+n}},{key:"getDimensions",value:function(e){var t=getComputedStyle(e),n=[],i=e.clientHeight,r=e.clientWidth;return i-=parseFloat(t.paddingTop)+parseFloat(t.paddingBottom),r-=parseFloat(t.paddingLeft)+parseFloat(t.paddingRight),n.push(r),n.push(i),n}},{key:"getBoundingClientRect",value:function(e){var t=e.getBoundingClientRect();return{top:t.top,right:t.right,bottom:t.bottom,left:t.left,width:t.width,height:t.height,x:t.x,y:t.y}}},{key:"hexToRgba",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:"#999999",t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:.6;"#"!==e.substring(0,1)&&(e="#999999");var n=e.replace("#","");n=n.match(new RegExp("(.{"+n.length/3+"})","g"));for(var i=0;i<n.length;i++)n[i]=parseInt(1===n[i].length?n[i]+n[i]:n[i],16);return void 0!==t&&n.push(t),"rgba("+n.join(",")+")"}},{key:"getOpacityFromRGBA",value:function(e){return(e=e.match(/^rgba?[\s+]?\([\s+]?(\d+)[\s+]?,[\s+]?(\d+)[\s+]?,[\s+]?(\d+)[\s+]?/i))[3]}},{key:"rgb2hex",value:function(e){return(e=e.match(/^rgba?[\s+]?\([\s+]?(\d+)[\s+]?,[\s+]?(\d+)[\s+]?,[\s+]?(\d+)[\s+]?/i))&&4===e.length?"#"+("0"+parseInt(e[1],10).toString(16)).slice(-2)+("0"+parseInt(e[2],10).toString(16)).slice(-2)+("0"+parseInt(e[3],10).toString(16)).slice(-2):""}},{key:"isColorHex",value:function(e){return/(^#[0-9A-F]{6}$)|(^#[0-9A-F]{3}$)/i.test(e)}},{key:"polarToCartesian",value:function(e,t,n,i){var r=(i-90)*Math.PI/180;return{x:e+n*Math.cos(r),y:t+n*Math.sin(r)}}},{key:"escapeString",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:"x",n=e.toString().slice();return n=n.replace(/[` ~!@#$%^&*()_|+\-=?;:'",.<>\{\}\[\]\\\/]/gi,t)}},{key:"negToZero",value:function(e){return e<0?0:e}},{key:"moveIndexInArray",value:function(e,t,n){if(n>=e.length)for(var i=n-e.length+1;i--;)e.push(void 0);return e.splice(n,0,e.splice(t,1)[0]),e}},{key:"extractNumber",value:function(e){return parseFloat(e.replace(/[^\d\.]*/g,""))}},{key:"randomString",value:function(e){for(var t="",n="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz",i=0;i<e;i++)t+=n.charAt(Math.floor(Math.random()*n.length));return t}},{key:"findAncestor",value:function(e,t){for(;(e=e.parentElement)&&!e.classList.contains(t););return e}},{key:"setELstyles",value:function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e.style.key=t[n])}},{key:"isNumber",value:function(e){return!isNaN(e)&&parseFloat(Number(e))===e&&!isNaN(parseInt(e,10))}},{key:"isFloat",value:function(e){return Number(e)===e&&e%1!=0}},{key:"isSafari",value:function(){return/^((?!chrome|android).)*safari/i.test(navigator.userAgent)}},{key:"isFirefox",value:function(){return navigator.userAgent.toLowerCase().indexOf("firefox")>-1}},{key:"isIE11",value:function(){if(-1!==window.navigator.userAgent.indexOf("MSIE")||window.navigator.appVersion.indexOf("Trident/")>-1)return!0}},{key:"isIE",value:function(){var e=window.navigator.userAgent,t=e.indexOf("MSIE ");if(t>0)return parseInt(e.substring(t+5,e.indexOf(".",t)),10);if(e.indexOf("Trident/")>0){var n=e.indexOf("rv:");return parseInt(e.substring(n+3,e.indexOf(".",n)),10)}var i=e.indexOf("Edge/");return i>0&&parseInt(e.substring(i+5,e.indexOf(".",i)),10)}}]),e}(),v=function(){function e(t){o(this,e),this.ctx=t,this.w=t.w}return l(e,[{key:"getDefaultFilter",value:function(e,t){var n=this.w;e.unfilter(!0),(new window.SVG.Filter).size("120%","180%","-5%","-40%"),"none"!==n.config.states.normal.filter?this.applyFilter(e,t,n.config.states.normal.filter.type,n.config.states.normal.filter.value):n.config.chart.dropShadow.enabled&&this.dropShadow(e,n.config.chart.dropShadow,t)}},{key:"addNormalFilter",value:function(e,t){var n=this.w;n.config.chart.dropShadow.enabled&&this.dropShadow(e,n.config.chart.dropShadow,t)}},{key:"addLightenFilter",value:function(e,t,n){var i=this,r=this.w,a=n.intensity;if(!m.isFirefox()){e.unfilter(!0);var o=new window.SVG.Filter;o.size("120%","180%","-5%","-40%"),e.filter(function(e){var n=r.config.chart.dropShadow;(o=n.enabled?i.addShadow(e,t,n):e).componentTransfer({rgb:{type:"linear",slope:1.5,intercept:a}})}),e.filterer.node.setAttribute("filterUnits","userSpaceOnUse")}}},{key:"addDarkenFilter",value:function(e,t,n){var i=this,r=this.w,a=n.intensity;if(!m.isFirefox()){e.unfilter(!0);var o=new window.SVG.Filter;o.size("120%","180%","-5%","-40%"),e.filter(function(e){var n=r.config.chart.dropShadow;(o=n.enabled?i.addShadow(e,t,n):e).componentTransfer({rgb:{type:"linear",slope:a}})}),e.filterer.node.setAttribute("filterUnits","userSpaceOnUse")}}},{key:"applyFilter",value:function(e,t,n){var i=arguments.length>3&&void 0!==arguments[3]?arguments[3]:.5;switch(n){case"none":this.addNormalFilter(e,t);break;case"lighten":this.addLightenFilter(e,t,{intensity:i});break;case"darken":this.addDarkenFilter(e,t,{intensity:i})}}},{key:"addShadow",value:function(e,t,n){var i=n.blur,r=n.top,a=n.left,o=n.color,s=n.opacity,l=e.flood(Array.isArray(o)?o[t]:o,s).composite(e.sourceAlpha,"in").offset(a,r).gaussianBlur(i).merge(e.source);return e.blend(e.source,l)}},{key:"dropShadow",value:function(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:0,i=t.top,r=t.left,a=t.blur,o=t.color,s=t.opacity,l=t.noUserSpaceOnUse;return e.unfilter(!0),o=Array.isArray(o)?o[n]:o,(new window.SVG.Filter).size("120%","180%","-5%","-40%"),e.filter(function(e){var t=null;t=m.isSafari()||m.isFirefox()||m.isIE()?e.flood(o,s).composite(e.sourceAlpha,"in").offset(r,i).gaussianBlur(a):e.flood(o,s).composite(e.sourceAlpha,"in").offset(r,i).gaussianBlur(a).merge(e.source),e.blend(e.source,t)}),l||e.filterer.node.setAttribute("filterUnits","userSpaceOnUse"),e}},{key:"setSelectionFilter",value:function(e,t,n){var i=this.w;if(void 0!==i.globals.selectedDataPoints[t]&&i.globals.selectedDataPoints[t].indexOf(n)>-1){e.node.setAttribute("selected",!0);var r=i.config.states.active.filter;"none"!==r&&this.applyFilter(e,t,r.type,r.value)}}}]),e}(),y=function(){function e(t){o(this,e),this.ctx=t,this.w=t.w,this.setEasingFunctions()}return l(e,[{key:"setEasingFunctions",value:function(){var e;switch(this.w.config.chart.animations.easing){case"linear":e="-";break;case"easein":e="<";break;case"easeout":e=">";break;case"easeinout":e="<>";break;case"swing":e=function(e){var t=1.70158;return(e-=1)*e*((t+1)*e+t)+1};break;case"bounce":e=function(e){return e<1/2.75?7.5625*e*e:e<2/2.75?7.5625*(e-=1.5/2.75)*e+.75:e<2.5/2.75?7.5625*(e-=2.25/2.75)*e+.9375:7.5625*(e-=2.625/2.75)*e+.984375};break;case"elastic":e=function(e){return e===!!e?e:Math.pow(2,-10*e)*Math.sin((e-.075)*(2*Math.PI)/.3)+1};break;default:e="<>"}this.w.globals.easing=e}},{key:"animateLine",value:function(e,t,n,i){e.attr(t).animate(i).attr(n)}},{key:"animateCircleRadius",value:function(e,t,n,i,r){t||(t=0),e.attr({r:t}).animate(i,r).attr({r:n})}},{key:"animateCircle",value:function(e,t,n,i,r){e.attr({r:t.r,cx:t.cx,cy:t.cy}).animate(i,r).attr({r:n.r,cx:n.cx,cy:n.cy})}},{key:"animateRect",value:function(e,t,n,i,r){e.attr(t).animate(i).attr(n).afterAll(function(){r()})}},{key:"animatePathsGradually",value:function(e){var t=e.el,n=e.j,i=e.pathFrom,r=e.pathTo,a=e.speed,o=e.delay,s=e.strokeWidth,l=this.w,c=0;l.config.chart.animations.animateGradually.enabled&&(c=l.config.chart.animations.animateGradually.delay),l.config.chart.animations.dynamicAnimation.enabled&&l.globals.dataChanged&&(c=0),this.morphSVG(t,n,i,r,a,s,o*c)}},{key:"showDelayedElements",value:function(){this.w.globals.delayedElements.forEach(function(e){e.el.classList.remove("hidden")})}},{key:"morphSVG",value:function(e,t,n,i,r,a,o){var s=this,l=this.w;n||(n=e.attr("pathFrom")),i||(i=e.attr("pathTo")),(!n||n.indexOf("undefined")>-1||n.indexOf("NaN")>-1)&&(n="M 0 ".concat(l.globals.gridHeight),r=1),(i.indexOf("undefined")>-1||i.indexOf("NaN")>-1)&&(i="M 0 ".concat(l.globals.gridHeight),r=1),l.globals.shouldAnimate||(r=1),e.plot(n).animate(1,l.globals.easing,o).plot(n).animate(r,l.globals.easing,o).plot(i).afterAll(function(){m.isNumber(t)?t===l.globals.series[l.globals.maxValsInArrayIndex].length-2&&l.globals.shouldAnimate&&(l.globals.animationEnded=!0):l.globals.shouldAnimate&&(l.globals.animationEnded=!0,"function"==typeof l.config.chart.events.animationEnd&&l.config.chart.events.animationEnd(s.ctx,l)),s.showDelayedElements()})}}]),e}(),b=function(){function e(t){o(this,e),this.ctx=t,this.w=t.w}return l(e,[{key:"drawLine",value:function(e,t,n,i){var r=arguments.length>4&&void 0!==arguments[4]?arguments[4]:"#a8a8a8",a=arguments.length>5&&void 0!==arguments[5]?arguments[5]:0,o=arguments.length>6&&void 0!==arguments[6]?arguments[6]:null;return this.w.globals.dom.Paper.line().attr({x1:e,y1:t,x2:n,y2:i,stroke:r,"stroke-dasharray":a,"stroke-width":o})}},{key:"drawRect",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:0,t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:0,n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:0,i=arguments.length>3&&void 0!==arguments[3]?arguments[3]:0,r=arguments.length>4&&void 0!==arguments[4]?arguments[4]:0,a=arguments.length>5&&void 0!==arguments[5]?arguments[5]:"#fefefe",o=arguments.length>6&&void 0!==arguments[6]?arguments[6]:1,s=arguments.length>7&&void 0!==arguments[7]?arguments[7]:null,l=arguments.length>8&&void 0!==arguments[8]?arguments[8]:null,c=arguments.length>9&&void 0!==arguments[9]?arguments[9]:0,u=this.w.globals.dom.Paper.rect();return u.attr({x:e,y:t,width:n>0?n:0,height:i>0?i:0,rx:r,ry:r,fill:a,opacity:o,"stroke-width":null!==s?s:0,stroke:null!==l?l:"none","stroke-dasharray":c}),u}},{key:"drawPolygon",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:"#e1e1e1",n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:"none";return this.w.globals.dom.Paper.polygon(e).attr({fill:n,stroke:t})}},{key:"drawCircle",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:null,n=this.w.globals.dom.Paper.circle(2*e);return null!==t&&n.attr(t),n}},{key:"drawPath",value:function(e){var t=e.d,n=void 0===t?"":t,i=e.stroke,r=void 0===i?"#a8a8a8":i,a=e.strokeWidth,o=void 0===a?1:a,s=e.fill,l=e.fillOpacity,c=void 0===l?1:l,u=e.strokeOpacity,h=void 0===u?1:u,d=e.classes,f=e.strokeLinecap,p=void 0===f?null:f,g=e.strokeDashArray,m=void 0===g?0:g,v=this.w;return null===p&&(p=v.config.stroke.lineCap),(n.indexOf("undefined")>-1||n.indexOf("NaN")>-1)&&(n="M 0 ".concat(v.globals.gridHeight)),v.globals.dom.Paper.path(n).attr({fill:s,"fill-opacity":c,stroke:r,"stroke-opacity":h,"stroke-linecap":p,"stroke-width":o,"stroke-dasharray":m,class:d})}},{key:"group",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:null,t=this.w.globals.dom.Paper.group();return null!==e&&t.attr(e),t}},{key:"move",value:function(e,t){var n=["M",e,t].join(" ");return n}},{key:"line",value:function(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:null,i=null;return null===n?i=["L",e,t].join(" "):"H"===n?i=["H",e].join(" "):"V"===n&&(i=["V",t].join(" ")),i}},{key:"curve",value:function(e,t,n,i,r,a){var o=["C",e,t,n,i,r,a].join(" ");return o}},{key:"quadraticCurve",value:function(e,t,n,i){return["Q",e,t,n,i].join(" ")}},{key:"arc",value:function(e,t,n,i,r,a,o){var s="A";arguments.length>7&&void 0!==arguments[7]&&arguments[7]&&(s="a");var l=[s,e,t,n,i,r,a,o].join(" ");return l}},{key:"renderPaths",value:function(e){var t,n=e.i,i=e.j,r=e.realIndex,a=e.pathFrom,o=e.pathTo,s=e.stroke,l=e.strokeWidth,c=e.strokeLinecap,h=e.fill,d=e.animationDelay,f=e.initialSpeed,p=e.dataChangeSpeed,g=e.className,m=e.id,b=e.shouldClipToGrid,x=void 0===b||b,w=e.bindEventsOnPaths,k=void 0===w||w,S=e.drawShadow,E=void 0===S||S,C=this.w,T=new v(this.ctx),A=new y(this.ctx),_=this.w.config.chart.animations.enabled,O=_&&this.w.config.chart.animations.dynamicAnimation.enabled,P=!!(_&&!C.globals.resized||O&&C.globals.dataChanged&&C.globals.shouldAnimate);P?t=a:(t=o,this.w.globals.animationEnded=!0);var M=C.config.stroke.dashArray,I=0;I=Array.isArray(M)?M[r]:C.config.stroke.dashArray;var D=this.drawPath({d:t,stroke:s,strokeWidth:l,fill:h,fillOpacity:1,classes:g,strokeLinecap:c,strokeDashArray:I});if(D.attr("id","".concat(m,"-").concat(n)),D.attr("index",r),x&&D.attr({"clip-path":"url(#gridRectMask".concat(C.globals.cuid,")")}),"none"!==C.config.states.normal.filter.type)T.getDefaultFilter(D,r);else if(C.config.chart.dropShadow.enabled&&E&&(!C.config.chart.dropShadow.enabledSeries||C.config.chart.dropShadow.enabledSeries&&-1!==C.config.chart.dropShadow.enabledSeries.indexOf(r))){var N=C.config.chart.dropShadow;T.dropShadow(D,N,r)}k&&(D.node.addEventListener("mouseenter",this.pathMouseEnter.bind(this,D)),D.node.addEventListener("mouseleave",this.pathMouseLeave.bind(this,D)),D.node.addEventListener("mousedown",this.pathMouseDown.bind(this,D))),D.attr({pathTo:o,pathFrom:a});var L={el:D,j:i,pathFrom:a,pathTo:o,strokeWidth:l};return!_||C.globals.resized||C.globals.dataChanged?!C.globals.resized&&C.globals.dataChanged||A.showDelayedElements():A.animatePathsGradually(u({},L,{speed:f,delay:d})),C.globals.dataChanged&&O&&P&&A.animatePathsGradually(u({},L,{speed:p})),D}},{key:"drawPattern",value:function(e,t,n){var i=arguments.length>3&&void 0!==arguments[3]?arguments[3]:"#a8a8a8",r=arguments.length>4&&void 0!==arguments[4]?arguments[4]:0;return this.w.globals.dom.Paper.pattern(t,n,function(a){"horizontalLines"===e?a.line(0,0,n,0).stroke({color:i,width:r+1}):"verticalLines"===e?a.line(0,0,0,t).stroke({color:i,width:r+1}):"slantedLines"===e?a.line(0,0,t,n).stroke({color:i,width:r}):"squares"===e?a.rect(t,n).fill("none").stroke({color:i,width:r}):"circles"===e&&a.circle(t).fill("none").stroke({color:i,width:r})})}},{key:"drawGradient",value:function(e,t,n,i,r){var a,o=arguments.length>5&&void 0!==arguments[5]?arguments[5]:null,s=arguments.length>6&&void 0!==arguments[6]?arguments[6]:null,l=arguments.length>7&&void 0!==arguments[7]?arguments[7]:null,c=arguments.length>8&&void 0!==arguments[8]?arguments[8]:0,u=this.w;t=m.hexToRgba(t,i),n=m.hexToRgba(n,r);var h=0,d=1,f=1,p=null;null!==s&&(h=void 0!==s[0]?s[0]/100:0,d=void 0!==s[1]?s[1]/100:1,f=void 0!==s[2]?s[2]/100:1,p=void 0!==s[3]?s[3]/100:null);var g=!("donut"!==u.config.chart.type&&"pie"!==u.config.chart.type&&"bubble"!==u.config.chart.type);if(a=null===l||0===l.length?u.globals.dom.Paper.gradient(g?"radial":"linear",function(e){e.at(h,t,i),e.at(d,n,r),e.at(f,n,r),null!==p&&e.at(p,t,i)}):u.globals.dom.Paper.gradient(g?"radial":"linear",function(e){(Array.isArray(l[c])?l[c]:l).forEach(function(t){e.at(t.offset/100,t.color,t.opacity)})}),g){var v=u.globals.gridWidth/2,y=u.globals.gridHeight/2;"bubble"!==u.config.chart.type?a.attr({gradientUnits:"userSpaceOnUse",cx:v,cy:y,r:o}):a.attr({cx:.5,cy:.5,r:.8,fx:.2,fy:.2})}else"vertical"===e?a.from(0,0).to(0,1):"diagonal"===e?a.from(0,0).to(1,1):"horizontal"===e?a.from(0,1).to(1,1):"diagonal2"===e&&a.from(0,1).to(2,2);return a}},{key:"drawText",value:function(e){var t,n=this.w,i=e.x,r=e.y,a=e.text,o=e.textAnchor,s=e.fontSize,l=e.fontFamily,c=e.foreColor,u=e.opacity;return void 0===a&&(a=""),o||(o="start"),c||(c=n.config.chart.foreColor),l=l||n.config.chart.fontFamily,(t=Array.isArray(a)?n.globals.dom.Paper.text(function(e){for(var t=0;t<a.length;t++)e.tspan(a[t])}):n.globals.dom.Paper.plain(a)).attr({x:i,y:r,"text-anchor":o,"dominant-baseline":"auto","font-size":s,"font-family":l,fill:c,class:e.cssClass}),t.node.style.fontFamily=l,t.node.style.opacity=u,t}},{key:"addTspan",value:function(e,t,n){var i=e.tspan(t);n||(n=this.w.config.chart.fontFamily),i.node.style.fontFamily=n}},{key:"drawMarker",value:function(e,t,n){e=e||0;var i=n.pSize||0,r=null;if("square"===n.shape){var a=void 0===n.pRadius?i/2:n.pRadius;null===t&&(i=0,a=0);var o=1.2*i+a,s=this.drawRect(o,o,o,o,a);s.attr({x:e-o/2,y:t-o/2,cx:e,cy:t,class:n.class?n.class:"",fill:n.pointFillColor,"fill-opacity":n.pointFillOpacity?n.pointFillOpacity:1,stroke:n.pointStrokeColor,"stroke-width":n.pWidth?n.pWidth:0,"stroke-opacity":n.pointStrokeOpacity?n.pointStrokeOpacity:1}),r=s}else"circle"===n.shape&&(m.isNumber(t)||(i=0,t=0),r=this.drawCircle(i,{cx:e,cy:t,class:n.class?n.class:"",stroke:n.pointStrokeColor,fill:n.pointFillColor,"fill-opacity":n.pointFillOpacity?n.pointFillOpacity:1,"stroke-width":n.pWidth?n.pWidth:0,"stroke-opacity":n.pointStrokeOpacity?n.pointStrokeOpacity:1}));return r}},{key:"pathMouseEnter",value:function(e,t){var n=this.w,i=new v(this.ctx),r=parseInt(e.node.getAttribute("index")),a=parseInt(e.node.getAttribute("j"));if("function"==typeof n.config.chart.events.dataPointMouseEnter&&n.config.chart.events.dataPointMouseEnter(t,this.ctx,{seriesIndex:r,dataPointIndex:a,w:n}),this.ctx.fireEvent("dataPointMouseEnter",[t,this.ctx,{seriesIndex:r,dataPointIndex:a,w:n}]),("none"===n.config.states.active.filter.type||"true"!==e.node.getAttribute("selected"))&&"none"!==n.config.states.hover.filter.type&&"none"!==n.config.states.active.filter.type&&!n.globals.isTouchDevice){var o=n.config.states.hover.filter;i.applyFilter(e,r,o.type,o.value)}}},{key:"pathMouseLeave",value:function(e,t){var n=this.w,i=new v(this.ctx),r=parseInt(e.node.getAttribute("index")),a=parseInt(e.node.getAttribute("j"));"function"==typeof n.config.chart.events.dataPointMouseLeave&&n.config.chart.events.dataPointMouseLeave(t,this.ctx,{seriesIndex:r,dataPointIndex:a,w:n}),this.ctx.fireEvent("dataPointMouseLeave",[t,this.ctx,{seriesIndex:r,dataPointIndex:a,w:n}]),"none"!==n.config.states.active.filter.type&&"true"===e.node.getAttribute("selected")||"none"!==n.config.states.hover.filter.type&&i.getDefaultFilter(e,r)}},{key:"pathMouseDown",value:function(e,t){var n=this.w,i=new v(this.ctx),r=parseInt(e.node.getAttribute("index")),a=parseInt(e.node.getAttribute("j")),o="false";if("true"===e.node.getAttribute("selected")){if(e.node.setAttribute("selected","false"),n.globals.selectedDataPoints[r].indexOf(a)>-1){var s=n.globals.selectedDataPoints[r].indexOf(a);n.globals.selectedDataPoints[r].splice(s,1)}}else{if(!n.config.states.active.allowMultipleDataPointsSelection&&n.globals.selectedDataPoints.length>0){n.globals.selectedDataPoints=[];var l=n.globals.dom.Paper.select(".apexcharts-series path").members,c=n.globals.dom.Paper.select(".apexcharts-series circle, .apexcharts-series rect").members;l.forEach(function(e){e.node.setAttribute("selected","false"),i.getDefaultFilter(e,r)}),c.forEach(function(e){e.node.setAttribute("selected","false"),i.getDefaultFilter(e,r)})}e.node.setAttribute("selected","true"),o="true",void 0===n.globals.selectedDataPoints[r]&&(n.globals.selectedDataPoints[r]=[]),n.globals.selectedDataPoints[r].push(a)}if("true"===o){var u=n.config.states.active.filter;"none"!==u&&i.applyFilter(e,r,u.type,u.value)}else"none"!==n.config.states.active.filter.type&&i.getDefaultFilter(e,r);"function"==typeof n.config.chart.events.dataPointSelection&&n.config.chart.events.dataPointSelection(t,this.ctx,{selectedDataPoints:n.globals.selectedDataPoints,seriesIndex:r,dataPointIndex:a,w:n}),this.ctx.fireEvent("dataPointSelection",[t,this.ctx,{selectedDataPoints:n.globals.selectedDataPoints,seriesIndex:r,dataPointIndex:a,w:n}])}},{key:"rotateAroundCenter",value:function(e){var t=e.getBBox();return{x:t.x+t.width/2,y:t.y+t.height/2}}},{key:"getTextRects",value:function(e,t,n,i){var r=!(arguments.length>4&&void 0!==arguments[4])||arguments[4],a=this.w,o=this.drawText({x:-200,y:-200,text:e,textAnchor:"start",fontSize:t,fontFamily:n,foreColor:"#fff",opacity:0});i&&o.attr("transform",i),a.globals.dom.Paper.add(o);var s=o.bbox();return r||(s=o.node.getBoundingClientRect()),o.remove(),{width:s.width,height:s.height}}},{key:"placeTextWithEllipsis",value:function(e,t,n){if(e.textContent=t,t.length>0&&e.getSubStringLength(0,t.length)>=n){for(var i=t.length-3;i>0;i-=3)if(e.getSubStringLength(0,i)<=n)return void(e.textContent=t.substring(0,i)+"...");e.textContent="..."}}}],[{key:"setAttrs",value:function(e,t){for(var n in t)t.hasOwnProperty(n)&&e.setAttribute(n,t[n])}}]),e}();var x={name:"en",options:{months:["January","February","March","April","May","June","July","August","September","October","November","December"],shortMonths:["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"],days:["Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"],shortDays:["Sun","Mon","Tue","Wed","Thu","Fri","Sat"],toolbar:{exportToSVG:"Download SVG",exportToPNG:"Download PNG",menu:"Menu",selection:"Selection",selectionZoom:"Selection Zoom",zoomIn:"Zoom In",zoomOut:"Zoom Out",pan:"Panning",reset:"Reset Zoom"}}},w=function(){function e(){o(this,e),this.yAxis={show:!0,showAlways:!1,seriesName:void 0,opposite:!1,reversed:!1,logarithmic:!1,tickAmount:void 0,forceNiceScale:!1,max:void 0,min:void 0,floating:!1,decimalsInFloat:void 0,labels:{show:!0,minWidth:0,maxWidth:160,offsetX:0,offsetY:0,align:void 0,rotate:0,padding:20,style:{colors:[],fontSize:"11px",fontFamily:void 0,cssClass:""},formatter:void 0},axisBorder:{show:!1,color:"#78909C",offsetX:0,offsetY:0},axisTicks:{show:!1,color:"#78909C",width:6,offsetX:0,offsetY:0},title:{text:void 0,rotate:90,offsetY:0,offsetX:0,style:{color:void 0,fontSize:"11px",fontFamily:void 0,cssClass:""}},tooltip:{enabled:!1,offsetX:0},crosshairs:{show:!0,position:"front",stroke:{color:"#b6b6b6",width:1,dashArray:0}}},this.xAxisAnnotation={x:0,x2:null,strokeDashArray:1,fillColor:"#c2c2c2",borderColor:"#c2c2c2",opacity:.3,offsetX:0,offsetY:0,label:{borderColor:"#c2c2c2",borderWidth:1,text:void 0,textAnchor:"middle",orientation:"vertical",position:"top",offsetX:0,offsetY:0,style:{background:"#fff",color:void 0,fontSize:"11px",fontFamily:void 0,cssClass:"",padding:{left:5,right:5,top:2,bottom:2}}}},this.yAxisAnnotation={y:0,y2:null,strokeDashArray:1,fillColor:"#c2c2c2",borderColor:"#c2c2c2",opacity:.3,offsetX:0,offsetY:0,yAxisIndex:0,label:{borderColor:"#c2c2c2",borderWidth:1,text:void 0,textAnchor:"end",position:"right",offsetX:0,offsetY:-3,style:{background:"#fff",color:void 0,fontSize:"11px",fontFamily:void 0,cssClass:"",padding:{left:5,right:5,top:0,bottom:2}}}},this.pointAnnotation={x:0,y:null,yAxisIndex:0,seriesIndex:0,marker:{size:0,fillColor:"#fff",strokeWidth:2,strokeColor:"#333",shape:"circle",offsetX:0,offsetY:0,radius:2,cssClass:""},label:{borderColor:"#c2c2c2",borderWidth:1,text:void 0,textAnchor:"middle",offsetX:0,offsetY:-15,style:{background:"#fff",color:void 0,fontSize:"11px",fontFamily:void 0,cssClass:"",padding:{left:5,right:5,top:0,bottom:2}}},customSVG:{SVG:void 0,cssClass:void 0,offsetX:0,offsetY:0}}}return l(e,[{key:"init",value:function(){return{annotations:{position:"front",yaxis:[this.yAxisAnnotation],xaxis:[this.xAxisAnnotation],points:[this.pointAnnotation]},chart:{animations:{enabled:!0,easing:"easeinout",speed:800,animateGradually:{delay:150,enabled:!0},dynamicAnimation:{enabled:!0,speed:350}},background:"transparent",locales:[x],defaultLocale:"en",dropShadow:{enabled:!1,enabledSeries:void 0,top:2,left:2,blur:4,color:"#000",opacity:.35},events:{animationEnd:void 0,beforeMount:void 0,mounted:void 0,updated:void 0,click:void 0,legendClick:void 0,markerClick:void 0,selection:void 0,dataPointSelection:void 0,dataPointMouseEnter:void 0,dataPointMouseLeave:void 0,beforeZoom:void 0,zoomed:void 0,scrolled:void 0},foreColor:"#373d3f",fontFamily:"Helvetica, Arial, sans-serif",height:"auto",parentHeightOffset:15,id:void 0,group:void 0,offsetX:0,offsetY:0,selection:{enabled:!1,type:"x",fill:{color:"#24292e",opacity:.1},stroke:{width:1,color:"#24292e",opacity:.4,dashArray:3},xaxis:{min:void 0,max:void 0},yaxis:{min:void 0,max:void 0}},sparkline:{enabled:!1},brush:{enabled:!1,autoScaleYaxis:!1,target:void 0},stacked:!1,stackType:"normal",toolbar:{show:!0,tools:{download:!0,selection:!0,zoom:!0,zoomin:!0,zoomout:!0,pan:!0,reset:!0,customIcons:[]},autoSelected:"zoom"},type:"line",width:"100%",zoom:{enabled:!0,type:"x",zoomedArea:{fill:{color:"#90CAF9",opacity:.4},stroke:{color:"#0D47A1",opacity:.4,width:1}}}},plotOptions:{bar:{horizontal:!1,columnWidth:"70%",barHeight:"70%",distributed:!1,endingShape:"flat",colors:{ranges:[],backgroundBarColors:[],backgroundBarOpacity:1},dataLabels:{maxItems:100,hideOverflowingLabels:!0,position:"top"}},candlestick:{colors:{upward:"#00B746",downward:"#EF403C"},wick:{useFillColor:!0}},heatmap:{radius:2,enableShades:!0,shadeIntensity:.5,distributed:!1,colorScale:{inverse:!1,ranges:[],min:void 0,max:void 0}},radialBar:{size:void 0,inverseOrder:!1,startAngle:0,endAngle:360,offsetX:0,offsetY:0,hollow:{margin:5,size:"50%",background:"transparent",image:void 0,imageWidth:150,imageHeight:150,imageOffsetX:0,imageOffsetY:0,imageClipped:!0,position:"front",dropShadow:{enabled:!1,top:0,left:0,blur:3,color:"#000",opacity:.5}},track:{show:!0,startAngle:void 0,endAngle:void 0,background:"#f2f2f2",strokeWidth:"97%",opacity:1,margin:5,dropShadow:{enabled:!1,top:0,left:0,blur:3,color:"#000",opacity:.5}},dataLabels:{show:!0,name:{show:!0,fontSize:"16px",fontFamily:void 0,color:void 0,offsetY:0},value:{show:!0,fontSize:"14px",fontFamily:void 0,color:void 0,offsetY:16,formatter:function(e){return e+"%"}},total:{show:!1,label:"Total",color:void 0,formatter:function(e){return e.globals.seriesTotals.reduce(function(e,t){return e+t},0)/e.globals.series.length+"%"}}}},rangeBar:{},pie:{size:void 0,customScale:1,offsetX:0,offsetY:0,expandOnClick:!0,dataLabels:{offset:0,minAngleToShowLabel:10},donut:{size:"65%",background:"transparent",labels:{show:!1,name:{show:!0,fontSize:"16px",fontFamily:void 0,color:void 0,offsetY:-10},value:{show:!0,fontSize:"20px",fontFamily:void 0,color:void 0,offsetY:10,formatter:function(e){return e}},total:{show:!1,label:"Total",color:void 0,formatter:function(e){return e.globals.seriesTotals.reduce(function(e,t){return e+t},0)}}}}},radar:{size:void 0,offsetX:0,offsetY:0,polygons:{strokeColors:"#e8e8e8",connectorColors:"#e8e8e8",fill:{colors:void 0}}}},colors:void 0,dataLabels:{enabled:!0,enabledOnSeries:void 0,formatter:function(e){return e},textAnchor:"middle",offsetX:0,offsetY:0,style:{fontSize:"12px",fontFamily:void 0,colors:void 0},dropShadow:{enabled:!1,top:1,left:1,blur:1,color:"#000",opacity:.45}},fill:{type:"solid",colors:void 0,opacity:.85,gradient:{shade:"dark",type:"horizontal",shadeIntensity:.5,gradientToColors:void 0,inverseColors:!0,opacityFrom:1,opacityTo:1,stops:[0,50,100],colorStops:[]},image:{src:[],width:void 0,height:void 0},pattern:{style:"sqaures",width:6,height:6,strokeWidth:2}},grid:{show:!0,borderColor:"#e0e0e0",strokeDashArray:0,position:"back",xaxis:{lines:{show:!1,animate:!1}},yaxis:{lines:{show:!0,animate:!1}},row:{colors:void 0,opacity:.5},column:{colors:void 0,opacity:.5},padding:{top:0,right:10,bottom:0,left:12}},labels:[],legend:{show:!0,showForSingleSeries:!1,showForNullSeries:!0,showForZeroSeries:!0,floating:!1,position:"bottom",horizontalAlign:"center",fontSize:"12px",fontFamily:void 0,width:void 0,height:void 0,formatter:void 0,offsetX:-20,offsetY:0,labels:{colors:void 0,useSeriesColors:!1},markers:{width:12,height:12,strokeWidth:0,strokeColor:"#fff",radius:12,customHTML:void 0,offsetX:0,offsetY:0,onClick:void 0},itemMargin:{horizontal:0,vertical:5},onItemClick:{toggleDataSeries:!0},onItemHover:{highlightDataSeries:!0}},markers:{discrete:[],size:0,colors:void 0,strokeColors:"#fff",strokeWidth:2,strokeOpacity:.9,fillOpacity:1,shape:"circle",radius:2,offsetX:0,offsetY:0,hover:{size:void 0,sizeOffset:3}},noData:{text:void 0,align:"center",verticalAlign:"middle",offsetX:0,offsetY:0,style:{color:void 0,fontSize:"14px",fontFamily:void 0}},responsive:[],series:void 0,states:{normal:{filter:{type:"none",value:0}},hover:{filter:{type:"lighten",value:.15}},active:{allowMultipleDataPointsSelection:!1,filter:{type:"darken",value:.65}}},title:{text:void 0,align:"left",margin:10,offsetX:0,offsetY:0,floating:!1,style:{fontSize:"14px",fontFamily:void 0,color:void 0}},subtitle:{text:void 0,align:"left",margin:10,offsetX:0,offsetY:30,floating:!1,style:{fontSize:"12px",fontFamily:void 0,color:void 0}},stroke:{show:!0,curve:"smooth",lineCap:"butt",width:2,colors:void 0,dashArray:0},tooltip:{enabled:!0,shared:!0,followCursor:!1,intersect:!1,inverseOrder:!1,custom:void 0,fillSeriesColor:!1,theme:"light",style:{fontSize:"12px",fontFamily:void 0},onDatasetHover:{highlightDataSeries:!1},x:{show:!0,format:"dd MMM",formatter:void 0},y:{formatter:void 0,title:{formatter:function(e){return e}}},z:{formatter:void 0,title:"Size: "},marker:{show:!0},items:{display:"flex"},fixed:{enabled:!1,position:"topRight",offsetX:0,offsetY:0}},xaxis:{type:"category",categories:[],offsetX:0,offsetY:0,labels:{show:!0,rotate:-45,rotateAlways:!1,hideOverlappingLabels:!0,trim:!0,minHeight:void 0,maxHeight:120,showDuplicates:!0,style:{colors:[],fontSize:"12px",fontFamily:void 0,cssClass:""},offsetX:0,offsetY:0,format:void 0,formatter:void 0,datetimeFormatter:{year:"yyyy",month:"MMM 'yy",day:"dd MMM",hour:"HH:mm",minute:"HH:mm:ss"}},axisBorder:{show:!0,color:"#78909C",width:"100%",height:1,offsetX:0,offsetY:0},axisTicks:{show:!0,color:"#78909C",height:6,offsetX:0,offsetY:0},tickAmount:void 0,tickPlacement:"on",min:void 0,max:void 0,range:void 0,floating:!1,position:"bottom",title:{text:void 0,offsetX:0,offsetY:0,style:{color:void 0,fontSize:"12px",fontFamily:void 0,cssClass:""}},crosshairs:{show:!0,width:1,position:"back",opacity:.9,stroke:{color:"#b6b6b6",width:1,dashArray:3},fill:{type:"solid",color:"#B1B9C4",gradient:{colorFrom:"#D8E3F0",colorTo:"#BED1E6",stops:[0,100],opacityFrom:.4,opacityTo:.5}},dropShadow:{enabled:!1,left:0,top:0,blur:1,opacity:.4}},tooltip:{enabled:!0,offsetY:0,formatter:void 0,style:{fontSize:"12px",fontFamily:void 0}}},yaxis:this.yAxis,theme:{mode:"light",palette:"palette1",monochrome:{enabled:!1,color:"#008FFB",shadeTo:"light",shadeIntensity:.65}}}}}]),e}(),k=function(){function e(t){o(this,e),this.ctx=t,this.w=t.w,this.graphics=new b(this.ctx),this.w.globals.isBarHorizontal&&(this.invertAxis=!0),this.xDivision=this.w.globals.gridWidth/this.w.globals.dataPoints}return l(e,[{key:"drawAnnotations",value:function(){var e=this.w;if(e.globals.axisCharts){for(var t=this.drawYAxisAnnotations(),n=this.drawXAxisAnnotations(),i=this.drawPointAnnotations(),r=e.config.chart.animations.enabled,a=[t,n,i],o=[n.node,t.node,i.node],s=0;s<3;s++)e.globals.dom.elGraphical.add(a[s]),!r||e.globals.resized||e.globals.dataChanged||o[s].classList.add("hidden"),e.globals.delayedElements.push({el:o[s],index:0});this.setOrientations(e.config.annotations.xaxis),this.annotationsBackground()}}},{key:"addXaxisAnnotation",value:function(e,t,n){var i=this.w,r=this.invertAxis?i.globals.minY:i.globals.minX,a=this.invertAxis?i.globals.yRange[0]:i.globals.xRange,o=(e.x-r)/(a/i.globals.gridWidth),s=e.label.text;if("category"===i.config.xaxis.type||i.config.xaxis.convertedCatToNumeric){var l=i.globals.labels.indexOf(e.x),c=i.globals.dom.baseEl.querySelector(".apexcharts-xaxis-texts-g text:nth-child("+(l+1)+")");c&&(o=parseFloat(c.getAttribute("x")))}var u=e.strokeDashArray;if(!(o<0||o>i.globals.gridWidth)){if(null===e.x2){var h=this.graphics.drawLine(o+e.offsetX,0+e.offsetY,o+e.offsetX,i.globals.gridHeight+e.offsetY,e.borderColor,u);t.appendChild(h.node)}else{var d=(e.x2-r)/(a/i.globals.gridWidth);if(d<o){var f=o;o=d,d=f}if(s){var p=this.graphics.drawRect(o+e.offsetX,0+e.offsetY,d-o,i.globals.gridHeight+e.offsetY,0,e.fillColor,e.opacity,1,e.borderColor,u);t.appendChild(p.node)}}var g="top"===e.label.position?-3:i.globals.gridHeight,m=this.graphics.drawText({x:o+e.label.offsetX,y:g+e.label.offsetY,text:s,textAnchor:e.label.textAnchor,fontSize:e.label.style.fontSize,fontFamily:e.label.style.fontFamily,foreColor:e.label.style.color,cssClass:"apexcharts-xaxis-annotation-label "+e.label.style.cssClass});m.attr({rel:n}),t.appendChild(m.node)}}},{key:"drawXAxisAnnotations",value:function(){var e=this,t=this.w,n=this.graphics.group({class:"apexcharts-xaxis-annotations"});return t.config.annotations.xaxis.map(function(t,i){e.addXaxisAnnotation(t,n.node,i)}),n}},{key:"addYaxisAnnotation",value:function(e,t,n){var i,r,a=this.w,o=e.strokeDashArray;if(this.invertAxis){var s=a.globals.labels.indexOf(e.y),l=a.globals.dom.baseEl.querySelector(".apexcharts-yaxis-texts-g text:nth-child("+(s+1)+")");l&&(i=parseFloat(l.getAttribute("y")))}else i=a.globals.gridHeight-(e.y-a.globals.minYArr[e.yAxisIndex])/(a.globals.yRange[e.yAxisIndex]/a.globals.gridHeight),a.config.yaxis[e.yAxisIndex]&&a.config.yaxis[e.yAxisIndex].reversed&&(i=(e.y-a.globals.minYArr[e.yAxisIndex])/(a.globals.yRange[e.yAxisIndex]/a.globals.gridHeight));var c=e.label.text;if(null===e.y2){var u=this.graphics.drawLine(0+e.offsetX,i+e.offsetY,a.globals.gridWidth+e.offsetX,i+e.offsetY,e.borderColor,o);t.appendChild(u.node)}else{if(this.invertAxis){var h=a.globals.labels.indexOf(e.y2),d=a.globals.dom.baseEl.querySelector(".apexcharts-yaxis-texts-g text:nth-child("+(h+1)+")");d&&(r=parseFloat(d.getAttribute("y")))}else r=a.globals.gridHeight-(e.y2-a.globals.minYArr[e.yAxisIndex])/(a.globals.yRange[e.yAxisIndex]/a.globals.gridHeight),a.config.yaxis[e.yAxisIndex]&&a.config.yaxis[e.yAxisIndex].reversed&&(r=(e.y2-a.globals.minYArr[e.yAxisIndex])/(a.globals.yRange[e.yAxisIndex]/a.globals.gridHeight));if(r>i){var f=i;i=r,r=f}if(c){var p=this.graphics.drawRect(0+e.offsetX,r+e.offsetY,a.globals.gridWidth+e.offsetX,i-r,0,e.fillColor,e.opacity,1,e.borderColor,o);t.appendChild(p.node)}}var g="right"===e.label.position?a.globals.gridWidth:0,m=this.graphics.drawText({x:g+e.label.offsetX,y:(r||i)+e.label.offsetY-3,text:c,textAnchor:e.label.textAnchor,fontSize:e.label.style.fontSize,fontFamily:e.label.style.fontFamily,foreColor:e.label.style.color,cssClass:"apexcharts-yaxis-annotation-label "+e.label.style.cssClass});m.attr({rel:n}),t.appendChild(m.node)}},{key:"drawYAxisAnnotations",value:function(){var e=this,t=this.w,n=this.graphics.group({class:"apexcharts-yaxis-annotations"});return t.config.annotations.yaxis.map(function(t,i){e.addYaxisAnnotation(t,n.node,i)}),n}},{key:"clearAnnotations",value:function(e){var t=e.w.globals.dom.baseEl.querySelectorAll(".apexcharts-yaxis-annotations, .apexcharts-xaxis-annotations, .apexcharts-point-annotations");(t=m.listToArray(t)).forEach(function(e){for(;e.firstChild;)e.removeChild(e.firstChild)})}},{key:"addPointAnnotation",value:function(e,t,n){var i=this.w,r=0,a=0,o=0;if(this.invertAxis&&console.warn("Point annotation is not supported in horizontal bar charts."),"string"==typeof e.x){var s=i.globals.labels.indexOf(e.x),l=i.globals.dom.baseEl.querySelector(".apexcharts-xaxis-texts-g text:nth-child("+(s+1)+")");r=parseFloat(l.getAttribute("x"));var c=e.y;null===e.y&&(c=i.globals.series[e.seriesIndex][s]),a=i.globals.gridHeight-(c-i.globals.minYArr[e.yAxisIndex])/(i.globals.yRange[e.yAxisIndex]/i.globals.gridHeight)-parseInt(e.label.style.fontSize)-e.marker.size,o=i.globals.gridHeight-(c-i.globals.minYArr[e.yAxisIndex])/(i.globals.yRange[e.yAxisIndex]/i.globals.gridHeight),i.config.yaxis[e.yAxisIndex]&&i.config.yaxis[e.yAxisIndex].reversed&&(a=(c-i.globals.minYArr[e.yAxisIndex])/(i.globals.yRange[e.yAxisIndex]/i.globals.gridHeight)+parseInt(e.label.style.fontSize)+e.marker.size,o=(c-i.globals.minYArr[e.yAxisIndex])/(i.globals.yRange[e.yAxisIndex]/i.globals.gridHeight))}else r=(e.x-i.globals.minX)/(i.globals.xRange/i.globals.gridWidth),a=i.globals.gridHeight-(parseFloat(e.y)-i.globals.minYArr[e.yAxisIndex])/(i.globals.yRange[e.yAxisIndex]/i.globals.gridHeight)-parseInt(e.label.style.fontSize)-e.marker.size,o=i.globals.gridHeight-(e.y-i.globals.minYArr[e.yAxisIndex])/(i.globals.yRange[e.yAxisIndex]/i.globals.gridHeight),i.config.yaxis[e.yAxisIndex]&&i.config.yaxis[e.yAxisIndex].reversed&&(a=(parseFloat(e.y)-i.globals.minYArr[e.yAxisIndex])/(i.globals.yRange[e.yAxisIndex]/i.globals.gridHeight)-parseInt(e.label.style.fontSize)-e.marker.size,o=(e.y-i.globals.minYArr[e.yAxisIndex])/(i.globals.yRange[e.yAxisIndex]/i.globals.gridHeight));if(!(r<0||r>i.globals.gridWidth)){var u={pSize:e.marker.size,pWidth:e.marker.strokeWidth,pointFillColor:e.marker.fillColor,pointStrokeColor:e.marker.strokeColor,shape:e.marker.shape,radius:e.marker.radius,class:"apexcharts-point-annotation-marker "+e.marker.cssClass},h=this.graphics.drawMarker(r+e.marker.offsetX,o+e.marker.offsetY,u);t.appendChild(h.node);var d=e.label.text?e.label.text:"",f=this.graphics.drawText({x:r+e.label.offsetX,y:a+e.label.offsetY,text:d,textAnchor:e.label.textAnchor,fontSize:e.label.style.fontSize,fontFamily:e.label.style.fontFamily,foreColor:e.label.style.color,cssClass:"apexcharts-point-annotation-label "+e.label.style.cssClass});if(f.attr({rel:n}),t.appendChild(f.node),e.customSVG.SVG){var p=this.graphics.group({class:"apexcharts-point-annotations-custom-svg "+e.customSVG.cssClass});p.attr({transform:"translate(".concat(r+e.customSVG.offsetX,", ").concat(a+e.customSVG.offsetY,")")}),p.node.innerHTML=e.customSVG.SVG,t.appendChild(p.node)}}}},{key:"drawPointAnnotations",value:function(){var e=this,t=this.w,n=this.graphics.group({class:"apexcharts-point-annotations"});return t.config.annotations.points.map(function(t,i){e.addPointAnnotation(t,n.node,i)}),n}},{key:"setOrientations",value:function(e){var t=this,n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:null,i=this.w;e.map(function(e,r){if("vertical"===e.label.orientation){var a=null!==n?n:r,o=i.globals.dom.baseEl.querySelector(".apexcharts-xaxis-annotations .apexcharts-xaxis-annotation-label[rel='".concat(a,"']"));if(null!==o){var s=o.getBoundingClientRect();o.setAttribute("x",parseFloat(o.getAttribute("x"))-s.height+4),"top"===e.label.position?o.setAttribute("y",parseFloat(o.getAttribute("y"))+s.width):o.setAttribute("y",parseFloat(o.getAttribute("y"))-s.width);var l=t.graphics.rotateAroundCenter(o),c=l.x,u=l.y;o.setAttribute("transform","rotate(-90 ".concat(c," ").concat(u,")"))}}})}},{key:"addBackgroundToAnno",value:function(e,t){var n=this.w;if(!t.label.text)return null;var i=n.globals.dom.baseEl.querySelector(".apexcharts-grid").getBoundingClientRect(),r=e.getBoundingClientRect(),a=t.label.style.padding.left,o=t.label.style.padding.right,s=t.label.style.padding.top,l=t.label.style.padding.bottom;"vertical"===t.label.orientation&&(s=t.label.style.padding.left,l=t.label.style.padding.right,a=t.label.style.padding.top,o=t.label.style.padding.bottom);var c=r.left-i.left-a,u=r.top-i.top-s;return this.graphics.drawRect(c,u,r.width+a+o,r.height+s+l,0,t.label.style.background,1,t.label.borderWidth,t.label.borderColor,0)}},{key:"annotationsBackground",value:function(){var e=this,t=this.w,n=function(n,i,r){var a=t.globals.dom.baseEl.querySelector(".apexcharts-".concat(r,"-annotations .apexcharts-").concat(r,"-annotation-label[rel='").concat(i,"']"));if(a){var o=a.parentNode,s=e.addBackgroundToAnno(a,n);s&&o.insertBefore(s.node,a)}};t.config.annotations.xaxis.map(function(e,t){n(e,t,"xaxis")}),t.config.annotations.yaxis.map(function(e,t){n(e,t,"yaxis")}),t.config.annotations.points.map(function(e,t){n(e,t,"point")})}},{key:"addText",value:function(e,t,n){var i=e.x,r=e.y,a=e.text,o=e.textAnchor,s=e.appendTo,l=void 0===s?".apexcharts-inner":s,c=e.foreColor,u=e.fontSize,h=e.fontFamily,d=e.cssClass,f=e.backgroundColor,p=e.borderWidth,g=e.strokeDashArray,m=e.radius,v=e.borderColor,y=e.paddingLeft,b=void 0===y?4:y,x=e.paddingRight,w=void 0===x?4:x,k=e.paddingBottom,S=void 0===k?2:k,E=e.paddingTop,C=void 0===E?2:E,T=n,A=T.w,_=A.globals.dom.baseEl.querySelector(l),O=this.graphics.drawText({x:i,y:r,text:a,textAnchor:o||"start",fontSize:u||"12px",fontFamily:h||A.config.chart.fontFamily,foreColor:c||A.config.chart.foreColor,cssClass:d});_.appendChild(O.node);var P=O.bbox();if(a){var M=this.graphics.drawRect(P.x-b,P.y-C,P.width+b+w,P.height+S+C,m,f,1,p,v,g);O.before(M)}return t&&A.globals.memory.methodsToExec.push({context:T,method:T.addText,params:{x:i,y:r,text:a,textAnchor:o,appendTo:l,foreColor:c,fontSize:u,cssClass:d,backgroundColor:f,borderWidth:p,strokeDashArray:g,radius:m,borderColor:v,paddingLeft:b,paddingRight:w,paddingBottom:S,paddingTop:C}}),n}},{key:"addPointAnnotationExternal",value:function(e,t,n){return this.addAnnotationExternal({params:e,pushToMemory:t,context:n,type:"point",contextMethod:n.addPointAnnotation}),n}},{key:"addYaxisAnnotationExternal",value:function(e,t,n){return this.addAnnotationExternal({params:e,pushToMemory:t,context:n,type:"yaxis",contextMethod:n.addYaxisAnnotation}),n}},{key:"addXaxisAnnotationExternal",value:function(e,t,n){return this.addAnnotationExternal({params:e,pushToMemory:t,context:n,type:"xaxis",contextMethod:n.addXaxisAnnotation}),n}},{key:"addAnnotationExternal",value:function(e){var t=e.params,n=e.pushToMemory,i=e.context,r=e.type,a=e.contextMethod,o=i,s=o.w,l=s.globals.dom.baseEl.querySelector(".apexcharts-".concat(r,"-annotations")),c=l.childNodes.length+1,u=new w,h=Object.assign({},"xaxis"===r?u.xAxisAnnotation:"yaxis"===r?u.yAxisAnnotation:u.pointAnnotation),d=m.extend(h,t);switch(r){case"xaxis":this.addXaxisAnnotation(d,l,c);break;case"yaxis":this.addYaxisAnnotation(d,l,c);break;case"point":this.addPointAnnotation(d,l,c)}var f=s.globals.dom.baseEl.querySelector(".apexcharts-".concat(r,"-annotations .apexcharts-").concat(r,"-annotation-label[rel='").concat(c,"']")),p=this.addBackgroundToAnno(f,d);return p&&l.insertBefore(p.node,f),n&&s.globals.memory.methodsToExec.push({context:o,method:a,params:t}),i}}]),e}(),S=function(){function e(t){o(this,e),this.ctx=t,this.w=t.w,this.months31=[1,3,5,7,8,10,12],this.months30=[2,4,6,9,11],this.daysCntOfYear=[0,31,59,90,120,151,181,212,243,273,304,334]}return l(e,[{key:"isValidDate",value:function(e){return!isNaN(this.parseDate(e))}},{key:"getUTCTimeStamp",value:function(e){return Date.parse(e)?new Date(new Date(e).toISOString().substr(0,25)).getTime():e}},{key:"parseDate",value:function(e){var t=Date.parse(e);if(!isNaN(t))return this.getUTCTimeStamp(e);var n=Date.parse(e.replace(/-/g,"/").replace(/[a-z]+/gi," "));return n=this.getUTCTimeStamp(n)}},{key:"treatAsUtc",value:function(e){var t=new Date(e);return t.setMinutes(t.getMinutes()-t.getTimezoneOffset()),t}},{key:"formatDate",value:function(e,t){var n=!(arguments.length>2&&void 0!==arguments[2])||arguments[2],i=!(arguments.length>3&&void 0!==arguments[3])||arguments[3],r=this.w.globals.locale,a=["\0"].concat(g(r.months)),o=[""].concat(g(r.shortMonths)),s=[""].concat(g(r.days)),l=[""].concat(g(r.shortDays));function c(e,t){var n=e+"";for(t=t||2;n.length<t;)n="0"+n;return n}i&&(e=this.treatAsUtc(e));var u=n?e.getUTCFullYear():e.getFullYear();t=(t=(t=t.replace(/(^|[^\\])yyyy+/g,"$1"+u)).replace(/(^|[^\\])yy/g,"$1"+u.toString().substr(2,2))).replace(/(^|[^\\])y/g,"$1"+u);var h=(n?e.getUTCMonth():e.getMonth())+1;t=(t=(t=(t=t.replace(/(^|[^\\])MMMM+/g,"$1"+a[0])).replace(/(^|[^\\])MMM/g,"$1"+o[0])).replace(/(^|[^\\])MM/g,"$1"+c(h))).replace(/(^|[^\\])M/g,"$1"+h);var d=n?e.getUTCDate():e.getDate();t=(t=(t=(t=t.replace(/(^|[^\\])dddd+/g,"$1"+s[0])).replace(/(^|[^\\])ddd/g,"$1"+l[0])).replace(/(^|[^\\])dd/g,"$1"+c(d))).replace(/(^|[^\\])d/g,"$1"+d);var f=n?e.getUTCHours():e.getHours(),p=f>12?f-12:0===f?12:f;t=(t=(t=(t=t.replace(/(^|[^\\])HH+/g,"$1"+c(f))).replace(/(^|[^\\])H/g,"$1"+f)).replace(/(^|[^\\])hh+/g,"$1"+c(p))).replace(/(^|[^\\])h/g,"$1"+p);var m=n?e.getUTCMinutes():e.getMinutes();t=(t=t.replace(/(^|[^\\])mm+/g,"$1"+c(m))).replace(/(^|[^\\])m/g,"$1"+m);var v=n?e.getUTCSeconds():e.getSeconds();t=(t=t.replace(/(^|[^\\])ss+/g,"$1"+c(v))).replace(/(^|[^\\])s/g,"$1"+v);var y=n?e.getUTCMilliseconds():e.getMilliseconds();t=t.replace(/(^|[^\\])fff+/g,"$1"+c(y,3)),y=Math.round(y/10),t=t.replace(/(^|[^\\])ff/g,"$1"+c(y)),y=Math.round(y/10);var b=f<12?"AM":"PM";t=(t=(t=t.replace(/(^|[^\\])f/g,"$1"+y)).replace(/(^|[^\\])TT+/g,"$1"+b)).replace(/(^|[^\\])T/g,"$1"+b.charAt(0));var x=b.toLowerCase();t=(t=t.replace(/(^|[^\\])tt+/g,"$1"+x)).replace(/(^|[^\\])t/g,"$1"+x.charAt(0));var w=-e.getTimezoneOffset(),k=n||!w?"Z":w>0?"+":"-";if(!n){var S=(w=Math.abs(w))%60;k+=c(Math.floor(w/60))+":"+c(S)}t=t.replace(/(^|[^\\])K/g,"$1"+k);var E=(n?e.getUTCDay():e.getDay())+1;return t=(t=(t=(t=(t=t.replace(new RegExp(s[0],"g"),s[E])).replace(new RegExp(l[0],"g"),l[E])).replace(new RegExp(a[0],"g"),a[h])).replace(new RegExp(o[0],"g"),o[h])).replace(/\\(.)/g,"$1")}},{key:"getTimeUnitsfromTimestamp",value:function(e,t){var n=this.w;void 0!==n.config.xaxis.min&&(e=n.config.xaxis.min),void 0!==n.config.xaxis.max&&(t=n.config.xaxis.max);var i=new Date(e).getFullYear(),r=new Date(t).getFullYear(),a=new Date(e).getMonth(),o=new Date(t).getMonth(),s=new Date(e).getDate(),l=new Date(t).getDate(),c=new Date(e).getHours(),u=new Date(t).getHours();return{minMinute:new Date(e).getMinutes(),maxMinute:new Date(t).getMinutes(),minHour:c,maxHour:u,minDate:s,maxDate:l,minMonth:a,maxMonth:o,minYear:i,maxYear:r}}},{key:"isLeapYear",value:function(e){return e%4==0&&e%100!=0||e%400==0}},{key:"calculcateLastDaysOfMonth",value:function(e,t,n){return this.determineDaysOfMonths(e,t)-n}},{key:"determineDaysOfYear",value:function(e){var t=365;return this.isLeapYear(e)&&(t=366),t}},{key:"determineRemainingDaysOfYear",value:function(e,t,n){var i=this.daysCntOfYear[t]+n;return t>1&&this.isLeapYear()&&i++,i}},{key:"determineDaysOfMonths",value:function(e,t){var n=30;switch(e=m.monthMod(e),!0){case this.months30.indexOf(e)>-1:2===e&&(n=this.isLeapYear(t)?29:28);break;case this.months31.indexOf(e)>-1:default:n=31}return n}}]),e}(),E=function(){function e(t){o(this,e),this.opts=t}return l(e,[{key:"line",value:function(){return{chart:{animations:{easing:"swing"}},dataLabels:{enabled:!1},stroke:{width:5,curve:"straight"},markers:{size:0,hover:{sizeOffset:6}},xaxis:{crosshairs:{width:1}}}}},{key:"sparkline",value:function(e){this.opts.yaxis[0].labels.show=!1,this.opts.yaxis[0].floating=!0;return m.extend(e,{grid:{show:!1,padding:{left:0,right:0,top:0,bottom:0}},legend:{show:!1},xaxis:{labels:{show:!1},tooltip:{enabled:!1},axisBorder:{show:!1}},chart:{toolbar:{show:!1},zoom:{enabled:!1}},dataLabels:{enabled:!1}})}},{key:"bar",value:function(){return{chart:{stacked:!1,animations:{easing:"swing"}},plotOptions:{bar:{dataLabels:{position:"center"}}},dataLabels:{style:{colors:["#fff"]}},stroke:{width:0},fill:{opacity:.85},legend:{markers:{shape:"square",radius:2,size:8}},tooltip:{shared:!1},xaxis:{tooltip:{enabled:!1},crosshairs:{width:"barWidth",position:"back",fill:{type:"gradient"},dropShadow:{enabled:!1},stroke:{width:0}}}}}},{key:"candlestick",value:function(){return{stroke:{width:1,colors:["#333"]},dataLabels:{enabled:!1},tooltip:{shared:!0,custom:function(e){var t=e.seriesIndex,n=e.dataPointIndex,i=e.w;return'<div class="apexcharts-tooltip-candlestick"><div>Open: <span class="value">'+i.globals.seriesCandleO[t][n]+'</span></div><div>High: <span class="value">'+i.globals.seriesCandleH[t][n]+'</span></div><div>Low: <span class="value">'+i.globals.seriesCandleL[t][n]+'</span></div><div>Close: <span class="value">'+i.globals.seriesCandleC[t][n]+"</span></div></div>"}},states:{active:{filter:{type:"none"}}},xaxis:{crosshairs:{width:1}}}}},{key:"rangeBar",value:function(){return{stroke:{width:0},plotOptions:{bar:{dataLabels:{position:"center"}}},dataLabels:{enabled:!1,formatter:function(e,t){t.ctx;var n=t.seriesIndex,i=t.dataPointIndex,r=t.w,a=r.globals.seriesRangeStart[n][i];return r.globals.seriesRangeEnd[n][i]-a},style:{colors:["#fff"]}},tooltip:{shared:!1,followCursor:!0,custom:function(e){var t=e.ctx,n=e.seriesIndex,i=e.dataPointIndex,r=e.w,a=r.globals.seriesRangeStart[n][i],o=r.globals.seriesRangeEnd[n][i],s="",l="",c=r.globals.colors[n];if(void 0===r.config.tooltip.x.formatter)if("datetime"===r.config.xaxis.type){var u=new S(t);s=u.formatDate(new Date(a),r.config.tooltip.x.format,!0,!0),l=u.formatDate(new Date(o),r.config.tooltip.x.format,!0,!0)}else s=a,l=o;else s=r.config.tooltip.x.formatter(a),l=r.config.tooltip.x.formatter(o);var h=r.globals.labels[i];return'<div class="apexcharts-tooltip-rangebar"><div> <span class="series-name" style="color: '+c+'">'+(r.config.series[n].name?r.config.series[n].name:"")+'</span></div><div> <span class="category">'+h+': </span> <span class="value start-value">'+s+'</span> <span class="separator">-</span> <span class="value end-value">'+l+"</span></div></div>"}},xaxis:{tooltip:{enabled:!1},crosshairs:{stroke:{width:0}}}}}},{key:"area",value:function(){return{stroke:{width:4},fill:{type:"gradient",gradient:{inverseColors:!1,shade:"light",type:"vertical",opacityFrom:.65,opacityTo:.5,stops:[0,100,100]}},markers:{size:0,hover:{sizeOffset:6}},tooltip:{followCursor:!1}}}},{key:"brush",value:function(e){return m.extend(e,{chart:{toolbar:{autoSelected:"selection",show:!1},zoom:{enabled:!1}},dataLabels:{enabled:!1},stroke:{width:1},tooltip:{enabled:!1},xaxis:{tooltip:{enabled:!1}}})}},{key:"stacked100",value:function(){var e=this;this.opts.dataLabels=this.opts.dataLabels||{},this.opts.dataLabels.formatter=this.opts.dataLabels.formatter||void 0;var t=this.opts.dataLabels.formatter;this.opts.yaxis.forEach(function(t,n){e.opts.yaxis[n].min=0,e.opts.yaxis[n].max=100}),"bar"===this.opts.chart.type&&(this.opts.dataLabels.formatter=t||function(e){return"number"==typeof e&&e?e.toFixed(0)+"%":e})}},{key:"bubble",value:function(){return{dataLabels:{style:{colors:["#fff"]}},tooltip:{shared:!1,intersect:!0},xaxis:{crosshairs:{width:0}},fill:{type:"solid",gradient:{shade:"light",inverse:!0,shadeIntensity:.55,opacityFrom:.4,opacityTo:.8}}}}},{key:"scatter",value:function(){return{dataLabels:{enabled:!1},tooltip:{shared:!1,intersect:!0},markers:{size:6,strokeWidth:2,hover:{sizeOffset:2}}}}},{key:"heatmap",value:function(){return{chart:{stacked:!1,zoom:{enabled:!1}},fill:{opacity:1},dataLabels:{style:{colors:["#fff"]}},stroke:{colors:["#fff"]},tooltip:{followCursor:!0,marker:{show:!1},x:{show:!1}},legend:{position:"top",markers:{shape:"square",size:10,offsetY:2}},grid:{padding:{right:20}}}}},{key:"pie",value:function(){return{chart:{toolbar:{show:!1}},plotOptions:{pie:{donut:{labels:{show:!1}}}},dataLabels:{formatter:function(e){return e.toFixed(1)+"%"},style:{colors:["#fff"]},dropShadow:{enabled:!0}},stroke:{colors:["#fff"]},fill:{opacity:1,gradient:{shade:"dark",shadeIntensity:.35,inverseColors:!1,stops:[0,100,100]}},padding:{right:0,left:0},tooltip:{theme:"dark",fillSeriesColor:!0},legend:{position:"right"}}}},{key:"donut",value:function(){return{chart:{toolbar:{show:!1}},dataLabels:{formatter:function(e){return e.toFixed(1)+"%"},style:{colors:["#fff"]},dropShadow:{enabled:!0}},stroke:{colors:["#fff"]},fill:{opacity:1,gradient:{shade:"dark",shadeIntensity:.4,inverseColors:!1,type:"vertical",opacityFrom:1,opacityTo:1,stops:[70,98,100]}},padding:{right:0,left:0},tooltip:{theme:"dark",fillSeriesColor:!0},legend:{position:"right"}}}},{key:"radar",value:function(){return this.opts.yaxis[0].labels.style.fontSize="13px",this.opts.yaxis[0].labels.offsetY=6,{dataLabels:{enabled:!0,style:{colors:["#a8a8a8"],fontSize:"11px"}},stroke:{width:2},markers:{size:3,strokeWidth:1,strokeOpacity:1},fill:{opacity:.2},tooltip:{shared:!1,intersect:!0,followCursor:!0},grid:{show:!1},xaxis:{tooltip:{enabled:!1},crosshairs:{show:!1}}}}},{key:"radialBar",value:function(){return{chart:{animations:{dynamicAnimation:{enabled:!0,speed:800}},toolbar:{show:!1}},fill:{gradient:{shade:"dark",shadeIntensity:.4,inverseColors:!1,type:"diagonal2",opacityFrom:1,opacityTo:1,stops:[70,98,100]}},padding:{right:0,left:0},legend:{show:!1,position:"right"},tooltip:{enabled:!1,fillSeriesColor:!0}}}}],[{key:"convertCatToNumeric",value:function(e){e.xaxis.type="numeric",e.xaxis.convertedCatToNumeric=!0,e.xaxis.labels=e.xaxis.labels||{},e.xaxis.labels.formatter=e.xaxis.labels.formatter||function(e){return e},e.chart=e.chart||{},e.chart.zoom=e.chart.zoom||window.Apex.chart&&window.Apex.chart.zoom||{};var t=e.xaxis.labels.formatter,n=e.xaxis.categories&&e.xaxis.categories.length?e.xaxis.categories:e.labels;return n&&n.length&&(e.xaxis.labels.formatter=function(e){return t(n[e-1])}),e.xaxis.categories=[],e.labels=[],e.chart.zoom.enabled=e.chart.zoom.enabled||!1,e}}]),e}(),C=function(){function e(t){o(this,e),this.ctx=t,this.w=t.w}return l(e,[{key:"getStackedSeriesTotals",value:function(){for(var e=this.w,t=[],n=0;n<e.globals.series[e.globals.maxValsInArrayIndex].length;n++){for(var i=0,r=0;r<e.globals.series.length;r++)i+=e.globals.series[r][n];t.push(i)}return e.globals.stackedSeriesTotals=t,t}},{key:"getSeriesTotalByIndex",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:null;return null===e?this.w.config.series.reduce(function(e,t){return e+t},0):this.w.globals.series[e].reduce(function(e,t){return e+t},0)}},{key:"isSeriesNull",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:null;return 0===(null===e?this.w.config.series.filter(function(e){return null!==e}):this.w.globals.series[e].filter(function(e){return null!==e})).length}},{key:"seriesHaveSameValues",value:function(e){return this.w.globals.series[e].every(function(e,t,n){return e===n[0]})}},{key:"getLargestSeries",value:function(){var e=this.w;e.globals.maxValsInArrayIndex=e.globals.series.map(function(e){return e.length}).indexOf(Math.max.apply(Math,e.globals.series.map(function(e){return e.length})))}},{key:"getLargestMarkerSize",value:function(){var e=this.w,t=0;return e.globals.markers.size.forEach(function(e){t=Math.max(t,e)}),e.globals.markers.largestSize=t,t}},{key:"getSeriesTotals",value:function(){var e=this.w;e.globals.seriesTotals=e.globals.series.map(function(e,t){var n=0;if(Array.isArray(e))for(var i=0;i<e.length;i++)n+=e[i];else n+=e;return n})}},{key:"getSeriesTotalsXRange",value:function(e,t){var n=this.w;return n.globals.series.map(function(i,r){for(var a=0,o=0;o<i.length;o++)n.globals.seriesX[r][o]>e&&n.globals.seriesX[r][o]<t&&(a+=i[o]);return a})}},{key:"getPercentSeries",value:function(){var e=this.w;e.globals.seriesPercent=e.globals.series.map(function(t,n){var i=[];if(Array.isArray(t))for(var r=0;r<t.length;r++){var a=e.globals.stackedSeriesTotals[r],o=100*t[r]/a;i.push(o)}else{var s=100*t/e.globals.seriesTotals.reduce(function(e,t){return e+t},0);i.push(s)}return i})}},{key:"getCalculatedRatios",value:function(){var e,t,n,i,r,a=this.w.globals,o=[],s=[],l=.1,c=0;if(a.yRange=[],a.isMultipleYAxis)for(var u=0;u<a.minYArr.length;u++)a.yRange.push(Math.abs(a.minYArr[u]-a.maxYArr[u])),s.push(0);else a.yRange.push(Math.abs(a.minY-a.maxY));a.xRange=Math.abs(a.maxX-a.minX),a.zRange=Math.abs(a.maxZ-a.minZ);for(var h=0;h<a.yRange.length;h++)o.push(a.yRange[h]/a.gridHeight);if(t=a.xRange/a.gridWidth,n=Math.abs(a.initialmaxX-a.initialminX)/a.gridWidth,e=a.yRange/a.gridWidth,i=a.xRange/a.gridHeight,r=a.zRange/a.gridHeight*16,a.minY!==Number.MIN_VALUE&&0!==Math.abs(a.minY)&&(a.hasNegs=!0),a.isMultipleYAxis){s=[];for(var d=0;d<o.length;d++)s.push(-a.minYArr[d]/o[d])}else s.push(-a.minY/o[0]),a.minY!==Number.MIN_VALUE&&0!==Math.abs(a.minY)&&(l=-a.minY/e,c=a.minX/t);return{yRatio:o,invertedYRatio:e,zRatio:r,xRatio:t,initialXRatio:n,invertedXRatio:i,baseLineInvertedY:l,baseLineY:s,baseLineX:c}}},{key:"getLogSeries",value:function(e){var t=this.w;return t.globals.seriesLog=e.map(function(e,n){return t.config.yaxis[n]&&t.config.yaxis[n].logarithmic?e.map(function(e){return null===e?null:(Math.log(e)-Math.log(t.globals.minYArr[n]))/(Math.log(t.globals.maxYArr[n])-Math.log(t.globals.minYArr[n]))}):e}),t.globals.seriesLog}},{key:"getLogYRatios",value:function(e){var t=this,n=this.w,i=this.w.globals;return i.yLogRatio=e.slice(),i.logYRange=i.yRange.map(function(e,r){if(n.config.yaxis[r]&&t.w.config.yaxis[r].logarithmic){var a,o=-Number.MAX_VALUE,s=Number.MIN_VALUE;return i.seriesLog.forEach(function(e,t){e.forEach(function(e){n.config.yaxis[t]&&n.config.yaxis[t].logarithmic&&(o=Math.max(e,o),s=Math.min(e,s))})}),a=Math.pow(i.yRange[r],Math.abs(s-o)/i.yRange[r]),i.yLogRatio[r]=a/i.gridHeight,a}}),i.yLogRatio}}],[{key:"checkComboSeries",value:function(e){var t=!1,n=!1;return e.length&&void 0!==e[0].type&&(t=!0,e.forEach(function(e){"bar"!==e.type&&"column"!==e.type||(n=!0)})),{comboCharts:t,comboChartsHasBars:n}}},{key:"extendArrayProps",value:function(e,t){return t.yaxis&&(t=e.extendYAxis(t)),t.annotations&&(t.annotations.yaxis&&(t=e.extendYAxisAnnotations(t)),t.annotations.xaxis&&(t=e.extendXAxisAnnotations(t)),t.annotations.points&&(t=e.extendPointAnnotations(t))),t}}]),e}(),T=function(){function e(t){o(this,e),this.opts=t}return l(e,[{key:"init",value:function(){var e=this.opts,t=new w,n=new E(e);this.chartType=e.chart.type,"histogram"===this.chartType&&(e.chart.type="bar",e=m.extend({plotOptions:{bar:{columnWidth:"99.99%"}}},e)),e.series=this.checkEmptySeries(e.series),e=this.extendYAxis(e),e=this.extendAnnotations(e);var i=t.init(),r={};if(e&&"object"===a(e)){var o={};switch(this.chartType){case"line":o=n.line();break;case"area":o=n.area();break;case"bar":o=n.bar();break;case"candlestick":o=n.candlestick();break;case"rangeBar":o=n.rangeBar();break;case"histogram":o=n.bar();break;case"bubble":o=n.bubble();break;case"scatter":o=n.scatter();break;case"heatmap":o=n.heatmap();break;case"pie":o=n.pie();break;case"donut":o=n.donut();break;case"radar":o=n.radar();break;case"radialBar":o=n.radialBar();break;default:o=n.line()}e.chart.brush&&e.chart.brush.enabled&&(o=n.brush(o)),e.chart.stacked&&"100%"===e.chart.stackType&&n.stacked100(),this.checkForDarkTheme(window.Apex),this.checkForDarkTheme(e),e.xaxis=e.xaxis||window.Apex.xaxis||{};var s=C.checkComboSeries(e.series);"line"!==e.chart.type&&"area"!==e.chart.type&&"scatter"!==e.chart.type||s.comboChartsHasBars||"datetime"===e.xaxis.type||"numeric"===e.xaxis.type||"between"===e.xaxis.tickPlacement||(e=E.convertCatToNumeric(e)),(e.chart.sparkline&&e.chart.sparkline.enabled||window.Apex.chart&&window.Apex.chart.sparkline&&window.Apex.chart.sparkline.enabled)&&(o=n.sparkline(o)),r=m.extend(i,o)}var l=m.extend(r,window.Apex);return i=m.extend(l,e),i=this.handleUserInputErrors(i)}},{key:"extendYAxis",value:function(e){var t=new w;return void 0===e.yaxis&&(e.yaxis={}),e.yaxis.constructor!==Array&&window.Apex.yaxis&&window.Apex.yaxis.constructor!==Array&&(e.yaxis=m.extend(e.yaxis,window.Apex.yaxis)),e.yaxis.constructor!==Array?e.yaxis=[m.extend(t.yAxis,e.yaxis)]:e.yaxis=m.extendArray(e.yaxis,t.yAxis),e}},{key:"extendAnnotations",value:function(e){return void 0===e.annotations&&(e.annotations={},e.annotations.yaxis=[],e.annotations.xaxis=[],e.annotations.points=[]),e=this.extendYAxisAnnotations(e),e=this.extendXAxisAnnotations(e),e=this.extendPointAnnotations(e)}},{key:"extendYAxisAnnotations",value:function(e){var t=new w;return e.annotations.yaxis=m.extendArray(void 0!==e.annotations.yaxis?e.annotations.yaxis:[],t.yAxisAnnotation),e}},{key:"extendXAxisAnnotations",value:function(e){var t=new w;return e.annotations.xaxis=m.extendArray(void 0!==e.annotations.xaxis?e.annotations.xaxis:[],t.xAxisAnnotation),e}},{key:"extendPointAnnotations",value:function(e){var t=new w;return e.annotations.points=m.extendArray(void 0!==e.annotations.points?e.annotations.points:[],t.pointAnnotation),e}},{key:"checkForDarkTheme",value:function(e){e.theme&&"dark"===e.theme.mode&&(e.tooltip||(e.tooltip={}),"light"!==e.tooltip.theme&&(e.tooltip.theme="dark"),e.chart.foreColor||(e.chart.foreColor="#f6f7f8"),e.theme.palette||(e.theme.palette="palette4"))}},{key:"checkEmptySeries",value:function(e){return 0===e.length?[{data:[]}]:e}},{key:"handleUserInputErrors",value:function(e){var t=e;if(t.tooltip.shared&&t.tooltip.intersect)throw new Error("tooltip.shared cannot be enabled when tooltip.intersect is true. Turn off any other option by setting it to false.");if(t.chart.scroller&&console.warn("Scroller has been deprecated since v2.0.0. Please remove the configuration for chart.scroller"),("bar"===t.chart.type||"rangeBar"===t.chart.type)&&t.plotOptions.bar.horizontal){if(t.yaxis.length>1)throw new Error("Multiple Y Axis for bars are not supported. Switch to column chart by setting plotOptions.bar.horizontal=false");t.yaxis[0].reversed&&(t.yaxis[0].opposite=!0),t.xaxis.tooltip.enabled=!1,t.yaxis[0].tooltip.enabled=!1,t.chart.zoom.enabled=!1}return"bar"!==t.chart.type&&"rangeBar"!==t.chart.type||t.tooltip.shared&&("barWidth"===t.xaxis.crosshairs.width&&t.series.length>1&&(console.warn('crosshairs.width = "barWidth" is only supported in single series, not in a multi-series barChart.'),t.xaxis.crosshairs.width="tickWidth"),t.plotOptions.bar.horizontal&&(t.states.hover.type="none",t.tooltip.shared=!1),t.tooltip.followCursor||(console.warn("followCursor option in shared columns cannot be turned off. Please set %ctooltip.followCursor: true","color: blue;"),t.tooltip.followCursor=!0)),"candlestick"===t.chart.type&&t.yaxis[0].reversed&&(console.warn("Reversed y-axis in candlestick chart is not supported."),t.yaxis[0].reversed=!1),t.chart.group&&0===t.yaxis[0].labels.minWidth&&console.warn("It looks like you have multiple charts in synchronization. You must provide yaxis.labels.minWidth which must be EQUAL for all grouped charts to prevent incorrect behaviour."),Array.isArray(t.stroke.width)&&"line"!==t.chart.type&&"area"!==t.chart.type&&(console.warn("stroke.width option accepts array only for line and area charts. Reverted back to Number"),t.stroke.width=t.stroke.width[0]),t}}]),e}(),A=function(){function e(){o(this,e)}return l(e,[{key:"globalVars",value:function(e){return{chartID:null,cuid:null,events:{beforeMount:[],mounted:[],updated:[],clicked:[],selection:[],dataPointSelection:[],zoomed:[],scrolled:[]},colors:[],clientX:null,clientY:null,fill:{colors:[]},stroke:{colors:[]},dataLabels:{style:{colors:[]}},radarPolygons:{fill:{colors:[]}},markers:{colors:[],size:e.markers.size,largestSize:0},animationEnded:!1,isTouchDevice:"ontouchstart"in window||navigator.msMaxTouchPoints,isDirty:!1,initialConfig:null,lastXAxis:[],lastYAxis:[],series:[],seriesRangeStart:[],seriesRangeEnd:[],seriesPercent:[],seriesTotals:[],stackedSeriesTotals:[],seriesX:[],seriesZ:[],labels:[],timelineLabels:[],invertedTimelineLabels:[],seriesNames:[],noLabelsProvided:!1,allSeriesCollapsed:!1,collapsedSeries:[],collapsedSeriesIndices:[],ancillaryCollapsedSeries:[],ancillaryCollapsedSeriesIndices:[],risingSeries:[],dataFormatXNumeric:!1,selectedDataPoints:[],ignoreYAxisIndexes:[],padHorizontal:0,maxValsInArrayIndex:0,zoomEnabled:"zoom"===e.chart.toolbar.autoSelected&&e.chart.toolbar.tools.zoom&&e.chart.zoom.enabled,panEnabled:"pan"===e.chart.toolbar.autoSelected&&e.chart.toolbar.tools.pan,selectionEnabled:"selection"===e.chart.toolbar.autoSelected&&e.chart.toolbar.tools.selection,yaxis:null,minY:Number.MIN_VALUE,maxY:-Number.MAX_VALUE,minYArr:[],maxYArr:[],maxX:-Number.MAX_VALUE,initialmaxX:-Number.MAX_VALUE,minX:Number.MIN_VALUE,initialminX:Number.MIN_VALUE,minZ:Number.MIN_VALUE,maxZ:-Number.MAX_VALUE,minXDiff:Number.MAX_VALUE,mousedown:!1,lastClientPosition:{},visibleXRange:void 0,yRange:[],zRange:0,xRange:0,yValueDecimal:0,total:0,SVGNS:"http://www.w3.org/2000/svg",svgWidth:0,svgHeight:0,noData:!1,locale:{},dom:{},memory:{methodsToExec:[]},shouldAnimate:!0,skipLastTimelinelabel:!1,delayedElements:[],axisCharts:!0,isXNumeric:!1,isDataXYZ:!1,resized:!1,resizeTimer:null,comboCharts:!1,comboChartsHasBars:!1,dataChanged:!1,previousPaths:[],seriesXvalues:[],seriesYvalues:[],seriesCandleO:[],seriesCandleH:[],seriesCandleL:[],seriesCandleC:[],allSeriesHasEqualX:!0,dataPoints:0,pointsArray:[],dataLabelsRects:[],lastDrawnDataLabelsIndexes:[],hasNullValues:!1,easing:null,zoomed:!1,gridWidth:0,gridHeight:0,yAxisScale:[],xAxisScale:null,xAxisTicksPositions:[],timescaleTicks:[],rotateXLabels:!1,defaultLabels:!1,xLabelFormatter:void 0,yLabelFormatters:[],xaxisTooltipFormatter:void 0,ttKeyFormatter:void 0,ttVal:void 0,ttZFormatter:void 0,LINE_HEIGHT_RATIO:1.618,xAxisLabelsHeight:0,yAxisLabelsWidth:0,scaleX:1,scaleY:1,translateX:0,translateY:0,translateYAxisX:[],yLabelsCoords:[],yTitleCoords:[],yAxisWidths:[],translateXAxisY:0,translateXAxisX:0,tooltip:null,tooltipOpts:null}}},{key:"init",value:function(e){var t=this.globalVars(e);return t.initialConfig=m.extend({},e),t.initialSeries=JSON.parse(JSON.stringify(t.initialConfig.series)),t.lastXAxis=JSON.parse(JSON.stringify(t.initialConfig.xaxis)),t.lastYAxis=JSON.parse(JSON.stringify(t.initialConfig.yaxis)),t}}]),e}(),_=function(){function e(t){o(this,e),this.opts=t}return l(e,[{key:"init",value:function(){var e=new T(this.opts).init();return{config:e,globals:(new A).init(e)}}}]),e}(),O=function(){function e(t){o(this,e),this.ctx=t,this.w=t.w,this.opts=null,this.seriesIndex=0}return l(e,[{key:"clippedImgArea",value:function(e){var t=this.w,n=t.config,i=parseInt(t.globals.gridWidth),r=parseInt(t.globals.gridHeight),a=i>r?i:r,o=e.image,s=0,l=0;void 0===e.width&&void 0===e.height?void 0!==n.fill.image.width&&void 0!==n.fill.image.height?(s=n.fill.image.width+1,l=n.fill.image.height):(s=a+1,l=a):(s=e.width,l=e.height);var c=document.createElementNS(t.globals.SVGNS,"pattern");b.setAttrs(c,{id:e.patternID,patternUnits:e.patternUnits?e.patternUnits:"userSpaceOnUse",width:s+"px",height:l+"px"});var u=document.createElementNS(t.globals.SVGNS,"image");c.appendChild(u),u.setAttributeNS("http://www.w3.org/1999/xlink","href",o),b.setAttrs(u,{x:0,y:0,preserveAspectRatio:"none",width:s+"px",height:l+"px"}),u.style.opacity=e.opacity,t.globals.dom.elDefs.node.appendChild(c)}},{key:"getSeriesIndex",value:function(e){var t=this.w;return"bar"===t.config.chart.type&&t.config.plotOptions.bar.distributed||"heatmap"===t.config.chart.type?this.seriesIndex=e.seriesNumber:this.seriesIndex=e.seriesNumber%t.globals.series.length,this.seriesIndex}},{key:"fillPath",value:function(e){var t=this.w;this.opts=e;var n,i,r,a=this.w.config;this.seriesIndex=this.getSeriesIndex(e);var o=this.getFillColors(),s=o[this.seriesIndex];"function"==typeof s&&(s=s({seriesIndex:this.seriesIndex,value:e.value,w:t}));var l=this.getFillType(this.seriesIndex),c=Array.isArray(a.fill.opacity)?a.fill.opacity[this.seriesIndex]:a.fill.opacity,u=s;return e.color&&(s=e.color),-1===s.indexOf("rgb")?u=m.hexToRgba(s,c):s.indexOf("rgba")>-1&&(c="0."+m.getOpacityFromRGBA(o[this.seriesIndex])),"pattern"===l&&(i=this.handlePatternFill(i,s,c,u)),"gradient"===l&&(r=this.handleGradientFill(r,s,c,this.seriesIndex)),a.fill.image.src.length>0&&"image"===l?e.seriesNumber<a.fill.image.src.length?(this.clippedImgArea({opacity:c,image:a.fill.image.src[e.seriesNumber],patternUnits:e.patternUnits,patternID:"pattern".concat(t.globals.cuid).concat(e.seriesNumber+1)}),n="url(#pattern".concat(t.globals.cuid).concat(e.seriesNumber+1,")")):n=u:n="gradient"===l?r:"pattern"===l?i:u,e.solid&&(n=u),n}},{key:"getFillType",value:function(e){var t=this.w;return Array.isArray(t.config.fill.type)?t.config.fill.type[e]:t.config.fill.type}},{key:"getFillColors",value:function(){var e=this.w,t=e.config,n=this.opts,i=[];return e.globals.comboCharts?"line"===e.config.series[this.seriesIndex].type?e.globals.stroke.colors instanceof Array?i=e.globals.stroke.colors:i.push(e.globals.stroke.colors):e.globals.fill.colors instanceof Array?i=e.globals.fill.colors:i.push(e.globals.fill.colors):"line"===t.chart.type?e.globals.stroke.colors instanceof Array?i=e.globals.stroke.colors:i.push(e.globals.stroke.colors):e.globals.fill.colors instanceof Array?i=e.globals.fill.colors:i.push(e.globals.fill.colors),void 0!==n.fillColors&&(i=[],n.fillColors instanceof Array?i=n.fillColors.slice():i.push(n.fillColors)),i}},{key:"handlePatternFill",value:function(e,t,n,i){var r=this.w.config,a=this.opts,o=new b(this.ctx),s=void 0===r.fill.pattern.strokeWidth?Array.isArray(r.stroke.width)?r.stroke.width[this.seriesIndex]:r.stroke.width:Array.isArray(r.fill.pattern.strokeWidth)?r.fill.pattern.strokeWidth[this.seriesIndex]:r.fill.pattern.strokeWidth,l=t;r.fill.pattern.style instanceof Array?e=void 0!==r.fill.pattern.style[a.seriesNumber]?o.drawPattern(r.fill.pattern.style[a.seriesNumber],r.fill.pattern.width,r.fill.pattern.height,l,s,n):i:e=o.drawPattern(r.fill.pattern.style,r.fill.pattern.width,r.fill.pattern.height,l,s,n);return e}},{key:"handleGradientFill",value:function(e,t,n,i){var r,a,o=this.w.config,s=this.opts,l=new b(this.ctx),c=new m,u=o.fill.gradient.type,h=void 0===o.fill.gradient.opacityFrom?n:Array.isArray(o.fill.gradient.opacityFrom)?o.fill.gradient.opacityFrom[i]:o.fill.gradient.opacityFrom,d=void 0===o.fill.gradient.opacityTo?n:Array.isArray(o.fill.gradient.opacityTo)?o.fill.gradient.opacityTo[i]:o.fill.gradient.opacityTo;if(r=t,a=void 0===o.fill.gradient.gradientToColors||0===o.fill.gradient.gradientToColors.length?"dark"===o.fill.gradient.shade?c.shadeColor(-1*parseFloat(o.fill.gradient.shadeIntensity),t):c.shadeColor(parseFloat(o.fill.gradient.shadeIntensity),t):o.fill.gradient.gradientToColors[s.seriesNumber],o.fill.gradient.inverseColors){var f=r;r=a,a=f}return l.drawGradient(u,r,a,h,d,s.size,o.fill.gradient.stops,o.fill.gradient.colorStops,i)}}]),e}(),P=function(){function e(t,n){o(this,e),this.ctx=t,this.w=t.w}return l(e,[{key:"setGlobalMarkerSize",value:function(){var e=this.w;if(e.globals.markers.size=Array.isArray(e.config.markers.size)?e.config.markers.size:[e.config.markers.size],e.globals.markers.size.length>0){if(e.globals.markers.size.length<e.globals.series.length+1)for(var t=0;t<=e.globals.series.length;t++)void 0===e.globals.markers.size[t]&&e.globals.markers.size.push(e.globals.markers.size[0])}else e.globals.markers.size=e.config.series.map(function(t){return e.config.markers.size})}},{key:"plotChartMarkers",value:function(e,t,n){var i,r=this,a=this.w,o=t,s=e,l=null,c=new b(this.ctx);if(a.globals.markers.size[t]>0&&(l=c.group({class:"apexcharts-series-markers"})).attr("clip-path","url(#gridRectMarkerMask".concat(a.globals.cuid,")")),s.x instanceof Array)for(var u=function(e){var u=n;1===n&&0===e&&(u=0),1===n&&1===e&&(u=1);var h="apexcharts-marker";if("line"!==a.config.chart.type&&"area"!==a.config.chart.type||a.globals.comboCharts||a.config.tooltip.intersect||(h+=" no-pointer-events"),Array.isArray(a.config.markers.size)?a.globals.markers.size[t]>0:a.config.markers.size>0){m.isNumber(s.y[e])?h+=" w".concat((Math.random()+1).toString(36).substring(4)):h="apexcharts-nullpoint";var d=r.getMarkerConfig(h,t);a.config.markers.discrete.map(function(e){e.seriesIndex===t&&e.dataPointIndex===u&&(d.pointStrokeColor=e.strokeColor,d.pointFillColor=e.fillColor,d.pSize=e.size)}),a.config.series[o].data[n]&&(a.config.series[o].data[n].fillColor&&(d.pointFillColor=a.config.series[o].data[n].fillColor),a.config.series[o].data[n].strokeColor&&(d.pointStrokeColor=a.config.series[o].data[n].strokeColor)),(i=c.drawMarker(s.x[e],s.y[e],d)).attr("rel",u),i.attr("j",u),i.attr("index",t),i.node.setAttribute("default-marker-size",d.pSize),new v(r.ctx).setSelectionFilter(i,t,u),r.addEvents(i),l&&l.add(i)}else void 0===a.globals.pointsArray[t]&&(a.globals.pointsArray[t]=[]),a.globals.pointsArray[t].push([s.x[e],s.y[e]])},h=0;h<s.x.length;h++)u(h);return l}},{key:"getMarkerConfig",value:function(e,t){var n=this.w,i=this.getMarkerStyle(t);return{pSize:n.globals.markers.size[t],pRadius:n.config.markers.radius,pWidth:n.config.markers.strokeWidth,pointStrokeColor:i.pointStrokeColor,pointFillColor:i.pointFillColor,shape:n.config.markers.shape instanceof Array?n.config.markers.shape[t]:n.config.markers.shape,class:e,pointStrokeOpacity:n.config.markers.strokeOpacity,pointFillOpacity:n.config.markers.fillOpacity,seriesIndex:t}}},{key:"addEvents",value:function(e){var t=new b(this.ctx);e.node.addEventListener("mouseenter",t.pathMouseEnter.bind(this.ctx,e)),e.node.addEventListener("mouseleave",t.pathMouseLeave.bind(this.ctx,e)),e.node.addEventListener("mousedown",t.pathMouseDown.bind(this.ctx,e)),e.node.addEventListener("touchstart",t.pathMouseDown.bind(this.ctx,e),{passive:!0})}},{key:"getMarkerStyle",value:function(e){var t=this.w,n=t.globals.markers.colors,i=t.config.markers.strokeColor||t.config.markers.strokeColors;return{pointStrokeColor:i instanceof Array?i[e]:i,pointFillColor:n instanceof Array?n[e]:n}}}]),e}(),M=function(){function e(t){o(this,e),this.ctx=t,this.w=t.w,this.initialAnim=this.w.config.chart.animations.enabled,this.dynamicAnim=this.initialAnim&&this.w.config.chart.animations.dynamicAnimation.enabled,this.radiusSizes=[]}return l(e,[{key:"draw",value:function(e,t,n){var i=this.w,r=new b(this.ctx),a=n.realIndex,o=n.pointsPos,s=n.zRatio,l=n.elParent,c=r.group({class:"apexcharts-series-markers apexcharts-series-".concat(i.config.chart.type)});if(c.attr("clip-path","url(#gridRectMarkerMask".concat(i.globals.cuid,")")),o.x instanceof Array)for(var u=0;u<o.x.length;u++){var h=t+1,d=!0;0===t&&0===u&&(h=0),0===t&&1===u&&(h=1);var f=0,p=i.globals.markers.size[a];s!==1/0&&(p=i.globals.seriesZ[a][h]/s,void 0===this.radiusSizes[a]&&this.radiusSizes.push([]),this.radiusSizes[a].push(p)),i.config.chart.animations.enabled||(f=p);var g=o.x[u],m=o.y[u];if(f=f||0,(0===g&&0===m||null===m||void 0===i.globals.series[a][h])&&(d=!1),d){var v=this.drawPoint(g,m,f,p,a,h,t);c.add(v)}l.add(c)}}},{key:"drawPoint",value:function(e,t,n,i,r,a,o){var s=this.w,l=r,c=new y(this.ctx),u=new v(this.ctx),h=new O(this.ctx),d=new P(this.ctx),f=new b(this.ctx),p=d.getMarkerConfig("apexcharts-marker",l),g=h.fillPath({seriesNumber:r,patternUnits:"objectBoundingBox",value:s.globals.series[r][o]}),m=f.drawCircle(n);if(s.config.series[l].data[a]&&s.config.series[l].data[a].fillColor&&(g=s.config.series[l].data[a].fillColor),m.attr({cx:e,cy:t,fill:g,stroke:p.pointStrokeColor,strokeWidth:p.pWidth}),s.config.chart.dropShadow.enabled){var x=s.config.chart.dropShadow;u.dropShadow(m,x,r)}if(this.initialAnim&&!s.globals.dataChanged){var w=1;s.globals.resized||(w=s.config.chart.animations.speed),c.animateCircleRadius(m,0,i,w,s.globals.easing)}if(s.globals.dataChanged)if(this.dynamicAnim){var k,S,E,C,T=s.config.chart.animations.dynamicAnimation.speed;null!=(C=s.globals.previousPaths[r]&&s.globals.previousPaths[r][o])&&(k=C.x,S=C.y,E=void 0!==C.r?C.r:i);for(var A=0;A<s.globals.collapsedSeries.length;A++)s.globals.collapsedSeries[A].index===r&&(T=1,i=0);0===e&&0===t&&(i=0),c.animateCircle(m,{cx:k,cy:S,r:E},{cx:e,cy:t,r:i},T,s.globals.easing)}else m.attr({r:i});return m.attr({rel:a,j:a,index:r,"default-marker-size":i}),u.setSelectionFilter(m,r,a),d.addEvents(m),m.node.classList.add("apexcharts-marker"),m}},{key:"centerTextInBubble",value:function(e){var t=this.w;return{y:e+=parseInt(t.config.dataLabels.style.fontSize)/4}}}]),e}(),I=function(){function e(t){o(this,e),this.ctx=t,this.w=t.w}return l(e,[{key:"dataLabelsCorrection",value:function(e,t,n,i,r,a,o){var s=this.w,l=!1,c=new b(this.ctx).getTextRects(n,o),u=c.width,h=c.height;void 0===s.globals.dataLabelsRects[i]&&(s.globals.dataLabelsRects[i]=[]),s.globals.dataLabelsRects[i].push({x:e,y:t,width:u,height:h});var d=s.globals.dataLabelsRects[i].length-2,f=void 0!==s.globals.lastDrawnDataLabelsIndexes[i]?s.globals.lastDrawnDataLabelsIndexes[i][s.globals.lastDrawnDataLabelsIndexes[i].length-1]:0;if(void 0!==s.globals.dataLabelsRects[i][d]){var p=s.globals.dataLabelsRects[i][f];(e>p.x+p.width+2||t>p.y+p.height+2||e+u<p.x)&&(l=!0)}return(0===r||a)&&(l=!0),{x:e,y:t,drawnextLabel:l}}},{key:"drawDataLabel",value:function(e,t,n){var i=arguments.length>4&&void 0!==arguments[4]?arguments[4]:"top",r=this.w,a=new b(this.ctx),o=r.config.dataLabels,s=0,l=0,c=n,u=null;if(!o.enabled||e.x instanceof Array!=!0)return u;(u=a.group({class:"apexcharts-data-labels"})).attr("clip-path","url(#gridRectMarkerMask".concat(r.globals.cuid,")"));for(var h=0;h<e.x.length;h++)if(s=e.x[h]+o.offsetX,l=e.y[h]+o.offsetY-r.globals.markers.size[t]-5,"bottom"===i&&(l=l+2*r.globals.markers.size[t]+1.4*parseInt(o.style.fontSize)),!isNaN(s)){1===n&&0===h&&(c=0),1===n&&1===h&&(c=1);var d=r.globals.series[t][c],f="";if("bubble"===r.config.chart.type)f=r.globals.seriesZ[t][c],l=e.y[h]+r.config.dataLabels.offsetY,l=new M(this.ctx).centerTextInBubble(l,t,c).y;else null!=d&&(f=r.config.dataLabels.formatter(d,{ctx:this.ctx,seriesIndex:t,dataPointIndex:c,w:r}));this.plotDataLabelsText({x:s,y:l,text:f,i:t,j:c,parent:u,offsetCorrection:!0,dataLabelsConfig:r.config.dataLabels})}return u}},{key:"plotDataLabelsText",value:function(e){var t=this.w,n=new b(this.ctx),i=e.x,r=e.y,a=e.i,o=e.j,s=e.text,l=e.textAnchor,c=e.parent,u=e.dataLabelsConfig,h=e.alwaysDrawDataLabel,d=e.offsetCorrection;if(!(Array.isArray(t.config.dataLabels.enabledOnSeries)&&t.config.dataLabels.enabledOnSeries.indexOf(a)>-1)){var f={x:i,y:r,drawnextLabel:!0};if(d&&(f=this.dataLabelsCorrection(i,r,s,a,o,h,parseInt(u.style.fontSize))),t.globals.zoomed||(i=f.x,r=f.y),f.drawnextLabel){var p=n.drawText({width:100,height:parseInt(u.style.fontSize),x:i,y:r,foreColor:t.globals.dataLabels.style.colors[a],textAnchor:l||u.textAnchor,text:s,fontSize:u.style.fontSize,fontFamily:u.style.fontFamily});if(p.attr({class:"apexcharts-datalabel",cx:i,cy:r}),u.dropShadow.enabled){var g=u.dropShadow;new v(this.ctx).dropShadow(p,g)}c.add(p),void 0===t.globals.lastDrawnDataLabelsIndexes[a]&&(t.globals.lastDrawnDataLabelsIndexes[a]=[]),t.globals.lastDrawnDataLabelsIndexes[a].push(o)}}}}]),e}(),D=function(){function e(t,n){o(this,e),this.ctx=t,this.w=t.w;var i=this.w;this.barOptions=i.config.plotOptions.bar,this.isHorizontal=this.barOptions.horizontal,this.strokeWidth=i.config.stroke.width,this.isNullValue=!1,this.xyRatios=n,null!==this.xyRatios&&(this.xRatio=n.xRatio,this.yRatio=n.yRatio,this.invertedXRatio=n.invertedXRatio,this.invertedYRatio=n.invertedYRatio,this.baseLineY=n.baseLineY,this.baseLineInvertedY=n.baseLineInvertedY),this.yaxisIndex=0,this.seriesLen=0}return l(e,[{key:"draw",value:function(e,t){var n=this.w,i=new b(this.ctx),r=new C(this.ctx,n);e=r.getLogSeries(e),this.series=e,this.yRatio=r.getLogYRatios(this.yRatio),this.initVariables(e);var a=i.group({class:"apexcharts-bar-series apexcharts-plot-series"});n.config.dataLabels.enabled&&this.totalItems>n.config.plotOptions.bar.dataLabels.maxItems&&console.warn("WARNING: DataLabels are enabled but there are too many to display. This may cause performance issue when rendering.");for(var o=0,s=0;o<e.length;o++,s++){var l,c,u,h,d=void 0,f=void 0,p=void 0,g=void 0,v=[],y=[],x=n.globals.comboCharts?t[o]:o,w=i.group({class:"apexcharts-series ".concat(m.escapeString(n.globals.seriesNames[x])),rel:o+1,"data:realIndex":x});this.ctx.series.addCollapsedClassToSeries(w,x),e[o].length>0&&(this.visibleI=this.visibleI+1);var k=0,S=0,E=0;this.yRatio.length>1&&(this.yaxisIndex=x),this.isReversed=n.config.yaxis[this.yaxisIndex]&&n.config.yaxis[this.yaxisIndex].reversed;var T=this.initialPositions();g=T.y,S=T.barHeight,c=T.yDivision,h=T.zeroW,p=T.x,E=T.barWidth,l=T.xDivision,u=T.zeroH,this.horizontal||y.push(p+E/2);for(var A=i.group({class:"apexcharts-datalabels"}),_=0,O=n.globals.dataPoints;_<n.globals.dataPoints;_++,O--){void 0===this.series[o][_]||null===e[o][_]?this.isNullValue=!0:this.isNullValue=!1,n.config.stroke.show&&(k=this.isNullValue?0:Array.isArray(this.strokeWidth)?this.strokeWidth[x]:this.strokeWidth);var P=null;this.isHorizontal?(P=this.drawBarPaths({indexes:{i:o,j:_,realIndex:x,bc:s},barHeight:S,strokeWidth:k,pathTo:d,pathFrom:f,zeroW:h,x:p,y:g,yDivision:c,elSeries:w}),E=this.series[o][_]/this.invertedYRatio):(P=this.drawColumnPaths({indexes:{i:o,j:_,realIndex:x,bc:s},x:p,y:g,xDivision:l,pathTo:d,pathFrom:f,barWidth:E,zeroH:u,strokeWidth:k,elSeries:w}),S=this.series[o][_]/this.yRatio[this.yaxisIndex]),d=P.pathTo,f=P.pathFrom,g=P.y,p=P.x,_>0&&y.push(p+E/2),v.push(g);var M=this.getPathFillColor(e,o,_,x);w=this.renderSeries({realIndex:x,pathFill:M,j:_,i:o,pathFrom:f,pathTo:d,strokeWidth:k,elSeries:w,x:p,y:g,series:e,barHeight:S,barWidth:E,elDataLabelsWrap:A,visibleSeries:this.visibleI,type:"bar"})}n.globals.seriesXvalues[x]=y,n.globals.seriesYvalues[x]=v,a.add(w)}return a}},{key:"getPathFillColor",value:function(e,t,n,i){var r=this.w,a=new O(this.ctx),o=null,s=this.barOptions.distributed?n:t;this.barOptions.colors.ranges.length>0&&this.barOptions.colors.ranges.map(function(i){e[t][n]>=i.from&&e[t][n]<=i.to&&(o=i.color)});return r.config.series[t].data[n]&&r.config.series[t].data[n].fillColor&&(o=r.config.series[t].data[n].fillColor),a.fillPath({seriesNumber:this.barOptions.distributed?s:i,color:o,value:e[t][n]})}},{key:"renderSeries",value:function(e){var t=e.realIndex,n=e.pathFill,i=e.lineFill,r=e.j,a=e.i,o=e.pathFrom,s=e.pathTo,l=e.strokeWidth,c=e.elSeries,u=e.x,h=e.y,d=e.series,f=e.barHeight,p=e.barWidth,g=e.elDataLabelsWrap,m=e.visibleSeries,y=e.type,x=this.w,w=new b(this.ctx);i||(i=this.barOptions.distributed?x.globals.stroke.colors[r]:x.globals.stroke.colors[t]),x.config.series[a].data[r]&&x.config.series[a].data[r].strokeColor&&(i=x.config.series[a].data[r].strokeColor),this.isNullValue&&(n="none");var k=r/x.config.chart.animations.animateGradually.delay*(x.config.chart.animations.speed/x.globals.dataPoints)/2.4,S=w.renderPaths({i:a,j:r,realIndex:t,pathFrom:o,pathTo:s,stroke:i,strokeWidth:l,strokeLineCap:x.config.stroke.lineCap,fill:n,animationDelay:k,initialSpeed:x.config.chart.animations.speed,dataChangeSpeed:x.config.chart.animations.dynamicAnimation.speed,className:"apexcharts-".concat(y,"-area"),id:"apexcharts-".concat(y,"-area")});S.attr("clip-path","url(#gridRectMask".concat(x.globals.cuid,")")),new v(this.ctx).setSelectionFilter(S,t,r),c.add(S);var E=this.calculateDataLabelsPos({x:u,y:h,i:a,j:r,series:d,realIndex:t,barHeight:f,barWidth:p,renderedPath:S,visibleSeries:m});return null!==E&&g.add(E),c.add(g),c}},{key:"initVariables",value:function(e){var t=this.w;this.series=e,this.totalItems=0,this.seriesLen=0,this.visibleI=-1,this.visibleItems=1;for(var n=0;n<e.length;n++)if(e[n].length>0&&(this.seriesLen=this.seriesLen+1,this.totalItems+=e[n].length),t.globals.isXNumeric)for(var i=0;i<e[n].length;i++)t.globals.seriesX[n][i]>t.globals.minX&&t.globals.seriesX[n][i]<t.globals.maxX&&this.visibleItems++;else this.visibleItems=t.globals.dataPoints;0===this.seriesLen&&(this.seriesLen=1)}},{key:"initialPositions",value:function(){var e,t,n,i,r,a,o,s,l=this.w;return this.isHorizontal?(r=(n=l.globals.gridHeight/l.globals.dataPoints)/this.seriesLen,l.globals.isXNumeric&&(r=(n=l.globals.gridHeight/this.totalItems)/this.seriesLen),r=r*parseInt(this.barOptions.barHeight)/100,s=this.baseLineInvertedY+l.globals.padHorizontal+(this.isReversed?l.globals.gridWidth:0)-(this.isReversed?2*this.baseLineInvertedY:0),t=(n-r*this.seriesLen)/2):(a=(i=l.globals.gridWidth/this.visibleItems)/this.seriesLen*parseInt(this.barOptions.columnWidth)/100,l.globals.isXNumeric&&(a=(i=l.globals.minXDiff/this.xRatio)/this.seriesLen*parseInt(this.barOptions.columnWidth)/100),o=l.globals.gridHeight-this.baseLineY[this.yaxisIndex]-(this.isReversed?l.globals.gridHeight:0)+(this.isReversed?2*this.baseLineY[this.yaxisIndex]:0),e=l.globals.padHorizontal+(i-a*this.seriesLen)/2),{x:e,y:t,yDivision:n,xDivision:i,barHeight:r,barWidth:a,zeroH:o,zeroW:s}}},{key:"drawBarPaths",value:function(e){var t=e.indexes,n=e.barHeight,i=e.strokeWidth,r=e.pathTo,a=e.pathFrom,o=e.zeroW,s=e.x,l=e.y,c=e.yDivision,u=e.elSeries,h=this.w,d=new b(this.ctx),f=t.i,p=t.j,g=t.realIndex,m=t.bc;h.globals.isXNumeric&&(l=(h.globals.seriesX[f][p]-h.globals.minX)/this.invertedXRatio-n);var v=l+n*this.visibleI;r=d.move(o,v),a=d.move(o,v),h.globals.previousPaths.length>0&&(a=this.getPathFrom(g,p));var y={barHeight:n,strokeWidth:i,barYPosition:v,x:s=void 0===this.series[f][p]||null===this.series[f][p]?o:o+this.series[f][p]/this.invertedYRatio-2*(this.isReversed?this.series[f][p]/this.invertedYRatio:0),zeroW:o},x=this.barEndingShape(h,y,this.series,f,p);if(r=r+d.line(x.newX,v)+x.path+d.line(o,v+n-i)+d.line(o,v),a=a+d.line(o,v)+x.ending_p_from+d.line(o,v+n-i)+d.line(o,v+n-i)+d.line(o,v),h.globals.isXNumeric||(l+=c),this.barOptions.colors.backgroundBarColors.length>0&&0===f){m>=this.barOptions.colors.backgroundBarColors.length&&(m=0);var w=this.barOptions.colors.backgroundBarColors[m],k=d.drawRect(0,v-n*this.visibleI,h.globals.gridWidth,n*this.seriesLen,0,w,this.barOptions.colors.backgroundBarOpacity);u.add(k),k.node.classList.add("apexcharts-backgroundBar")}return{pathTo:r,pathFrom:a,x:s,y:l,barYPosition:v}}},{key:"drawColumnPaths",value:function(e){var t=e.indexes,n=e.x,i=e.y,r=e.xDivision,a=e.pathTo,o=e.pathFrom,s=e.barWidth,l=e.zeroH,c=e.strokeWidth,u=e.elSeries,h=this.w,d=new b(this.ctx),f=t.i,p=t.j,g=t.realIndex,m=t.bc;h.globals.isXNumeric&&(n=(h.globals.seriesX[f][p]-h.globals.minX)/this.xRatio-s/2);var v=n+s*this.visibleI;a=d.move(v,l),o=d.move(v,l),h.globals.previousPaths.length>0&&(o=this.getPathFrom(g,p));var y={barWidth:s,strokeWidth:c,barXPosition:v,y:i=void 0===this.series[f][p]||null===this.series[f][p]?l:l-this.series[f][p]/this.yRatio[this.yaxisIndex]+2*(this.isReversed?this.series[f][p]/this.yRatio[this.yaxisIndex]:0),zeroH:l},x=this.barEndingShape(h,y,this.series,f,p);if(a=a+d.line(v,x.newY)+x.path+d.line(v+s-c,l)+d.line(v-c/2,l),o=o+d.line(v,l)+x.ending_p_from+d.line(v+s-c,l)+d.line(v+s-c,l)+d.line(v-c/2,l),h.globals.isXNumeric||(n+=r),this.barOptions.colors.backgroundBarColors.length>0&&0===f){m>=this.barOptions.colors.backgroundBarColors.length&&(m=0);var w=this.barOptions.colors.backgroundBarColors[m],k=d.drawRect(v-s*this.visibleI,0,s*this.seriesLen,h.globals.gridHeight,0,w,this.barOptions.colors.backgroundBarOpacity);u.add(k),k.node.classList.add("apexcharts-backgroundBar")}return{pathTo:a,pathFrom:o,x:n,y:i,barXPosition:v}}},{key:"getPathFrom",value:function(e,t){for(var n,i=this.w,r=0;r<i.globals.previousPaths.length;r++){var a=i.globals.previousPaths[r];a.paths.length>0&&parseInt(a.realIndex)===parseInt(e)&&void 0!==i.globals.previousPaths[r].paths[t]&&(n=i.globals.previousPaths[r].paths[t].d)}return n}},{key:"calculateDataLabelsPos",value:function(e){var t=e.x,n=e.y,i=e.i,r=e.j,a=e.realIndex,o=e.series,s=e.barHeight,l=e.barWidth,c=e.visibleSeries,u=e.renderedPath,h=this.w,d=new b(this.ctx),f=Array.isArray(this.strokeWidth)?this.strokeWidth[a]:this.strokeWidth,p=t+parseFloat(l*c),g=n+parseFloat(s*c);h.globals.isXNumeric&&!h.globals.isBarHorizontal&&(p=t+parseFloat(l*(c+1))-f,g=n+parseFloat(s*(c+1))-f);var m=t,v=n,y={},x=h.config.dataLabels,w=this.barOptions.dataLabels,k=x.offsetX,S=x.offsetY,E={width:0,height:0};return h.config.dataLabels.enabled&&(E=d.getTextRects(h.globals.yLabelFormatters[0](h.globals.maxY),parseInt(x.style.fontSize))),y=this.isHorizontal?this.calculateBarsDataLabelsPosition({x:t,y:n,i:i,j:r,renderedPath:u,bcy:g,barHeight:s,barWidth:l,textRects:E,strokeWidth:f,dataLabelsX:m,dataLabelsY:v,barDataLabelsConfig:w,offX:k,offY:S}):this.calculateColumnsDataLabelsPosition({x:t,y:n,i:i,j:r,renderedPath:u,realIndex:a,bcx:p,bcy:g,barHeight:s,barWidth:l,textRects:E,strokeWidth:f,dataLabelsY:v,barDataLabelsConfig:w,offX:k,offY:S}),u.attr({cy:y.bcy,cx:y.bcx,j:r,val:o[i][r],barHeight:s,barWidth:l}),this.drawCalculatedDataLabels({x:y.dataLabelsX,y:y.dataLabelsY,val:o[i][r],i:a,j:r,barWidth:l,barHeight:s,textRects:E,dataLabelsConfig:x})}},{key:"calculateColumnsDataLabelsPosition",value:function(e){var t,n=this.w,i=e.i,r=e.j,a=e.y,o=e.bcx,s=e.barWidth,l=e.barHeight,c=e.textRects,u=e.dataLabelsY,h=e.barDataLabelsConfig,d=e.strokeWidth,f=e.offX,p=e.offY,g=n.globals.gridWidth/n.globals.dataPoints;o-=d/2,t=n.globals.isXNumeric?o-s/2+f:o-g+s/2+f;var m=this.series[i][r]<=0;switch(this.isReversed&&(a-=l),h.position){case"center":u=m?a+l/2+c.height/2+p:a+l/2+c.height/2-p;break;case"bottom":u=m?a+l+c.height+d+p:a+l-c.height/2+d-p;break;case"top":u=m?a-c.height/2-p:a+c.height+p}return n.config.chart.stacked||(u<0?u=0+d:u+c.height/3>n.globals.gridHeight&&(u=n.globals.gridHeight-d)),{bcx:o,bcy:a,dataLabelsX:t,dataLabelsY:u}}},{key:"calculateBarsDataLabelsPosition",value:function(e){var t=this.w,n=e.x,i=e.i,r=e.j,a=e.bcy,o=e.barHeight,s=e.barWidth,l=e.textRects,c=e.dataLabelsX,u=e.strokeWidth,h=e.barDataLabelsConfig,d=e.offX,f=e.offY,p=a-t.globals.gridHeight/t.globals.dataPoints+o/2+l.height/2+f-3,g=this.series[i][r]<=0;switch(this.isReversed&&(n+=s),h.position){case"center":c=g?n-s/2-d:n-s/2+d;break;case"bottom":c=g?n-s-u-Math.round(l.width/2)-d:n-s+u+Math.round(l.width/2)+d;break;case"top":c=g?n-u+Math.round(l.width/2)-d:n-u-Math.round(l.width/2)+d}return t.config.chart.stacked||(c<0?c=c+l.width+u:c+l.width/2>t.globals.gridWidth&&(c=t.globals.gridWidth-l.width-u)),{bcx:n,bcy:a,dataLabelsX:c,dataLabelsY:p}}},{key:"drawCalculatedDataLabels",value:function(e){var t=e.x,n=e.y,i=e.val,r=e.i,a=e.j,o=e.textRects,s=e.barHeight,l=e.barWidth,c=e.dataLabelsConfig,u=this.w,h=new I(this.ctx),d=new b(this.ctx),f=c.formatter,p=null,g=u.globals.collapsedSeriesIndices.indexOf(r)>-1;if(c.enabled&&!g){p=d.group({class:"apexcharts-data-labels"});var m="";null!=i&&(m=f(i,{seriesIndex:r,dataPointIndex:a,w:u})),0===i&&u.config.chart.stacked&&(m=""),u.config.chart.stacked&&this.barOptions.dataLabels.hideOverflowingLabels&&(this.isHorizontal?(l=this.series[r][a]/this.yRatio[this.yaxisIndex],o.width/1.6>l&&(m="")):(s=this.series[r][a]/this.yRatio[this.yaxisIndex],o.height/1.6>s&&(m=""))),h.plotDataLabelsText({x:t,y:n,text:m,i:r,j:a,parent:p,dataLabelsConfig:c,alwaysDrawDataLabel:!0,offsetCorrection:!0})}return p}},{key:"barEndingShape",value:function(e,t,n,i,r){var a=new b(this.ctx);if(this.isHorizontal){var o=null,s=t.x;if(void 0!==n[i][r]||null!==n[i][r]){var l=n[i][r]<0,c=t.barHeight/2-t.strokeWidth;switch(l&&(c=-t.barHeight/2-t.strokeWidth),e.config.chart.stacked||"rounded"===this.barOptions.endingShape&&(s=t.x-c/2),this.barOptions.endingShape){case"flat":o=a.line(s,t.barYPosition+t.barHeight-t.strokeWidth);break;case"rounded":o=a.quadraticCurve(s+c,t.barYPosition+(t.barHeight-t.strokeWidth)/2,s,t.barYPosition+t.barHeight-t.strokeWidth)}}return{path:o,ending_p_from:"",newX:s}}var u=null,h=t.y;if(void 0!==n[i][r]||null!==n[i][r]){var d=n[i][r]<0,f=t.barWidth/2-t.strokeWidth;switch(d&&(f=-t.barWidth/2-t.strokeWidth),e.config.chart.stacked||"rounded"===this.barOptions.endingShape&&(h+=f/2),this.barOptions.endingShape){case"flat":u=a.line(t.barXPosition+t.barWidth-t.strokeWidth,h);break;case"rounded":u=a.quadraticCurve(t.barXPosition+(t.barWidth-t.strokeWidth)/2,h-f,t.barXPosition+t.barWidth-t.strokeWidth,h)}}return{path:u,ending_p_from:"",newY:h}}}]),e}(),N=function(e){function t(){return o(this,t),p(this,d(t).apply(this,arguments))}return h(t,D),l(t,[{key:"draw",value:function(e,t){var n=this.w;this.graphics=new b(this.ctx),this.fill=new O(this.ctx),this.bar=new D(this.ctx,this.xyRatios);var i=new C(this.ctx,n);e=i.getLogSeries(e),this.yRatio=i.getLogYRatios(this.yRatio),this.initVariables(e),"100%"===n.config.chart.stackType&&(e=n.globals.seriesPercent.slice()),this.series=e,this.totalItems=0,this.prevY=[],this.prevX=[],this.prevYF=[],this.prevXF=[],this.prevYVal=[],this.prevXVal=[],this.xArrj=[],this.xArrjF=[],this.xArrjVal=[],this.yArrj=[],this.yArrjF=[],this.yArrjVal=[];for(var r=0;r<e.length;r++)e[r].length>0&&(this.totalItems+=e[r].length);for(var a=this.graphics.group({class:"apexcharts-bar-series apexcharts-plot-series"}),o=0,s=0,l=0,c=0;l<e.length;l++,c++){var u=void 0,h=void 0,d=void 0,f=void 0,p=void 0,g=void 0,v=[],y=[],x=n.globals.comboCharts?t[l]:l;this.yRatio.length>1&&(this.yaxisIndex=x),this.isReversed=n.config.yaxis[this.yaxisIndex]&&n.config.yaxis[this.yaxisIndex].reversed;var w=this.graphics.group({class:"apexcharts-series ".concat(m.escapeString(n.globals.seriesNames[x])),rel:l+1,"data:realIndex":x}),k=this.graphics.group({class:"apexcharts-datalabels"}),S=0,E=0,T=0,A=this.initialPositions(o,s,d,f,p,g);s=A.y,E=A.barHeight,f=A.yDivision,g=A.zeroW,o=A.x,T=A.barWidth,d=A.xDivision,p=A.zeroH,this.yArrj=[],this.yArrjF=[],this.yArrjVal=[],this.xArrj=[],this.xArrjF=[],this.xArrjVal=[];for(var _=0;_<n.globals.dataPoints;_++){n.config.stroke.show&&(S=this.isNullValue?0:Array.isArray(this.strokeWidth)?this.strokeWidth[x]:this.strokeWidth);var P=null;this.isHorizontal?(P=this.drawBarPaths({indexes:{i:l,j:_,realIndex:x,bc:c},barHeight:E,strokeWidth:S,pathTo:u,pathFrom:h,zeroW:g,x:o,y:s,yDivision:f,elSeries:w}),T=this.series[l][_]/this.invertedYRatio):(P=this.drawColumnPaths({indexes:{i:l,j:_,realIndex:x,bc:c},x:o,y:s,xDivision:d,pathTo:u,pathFrom:h,barWidth:T,zeroH:p,strokeWidth:S,elSeries:w}),E=this.series[l][_]/this.yRatio[this.yaxisIndex]),u=P.pathTo,h=P.pathFrom,s=P.y,o=P.x,v.push(o),y.push(s);var M=this.bar.getPathFillColor(e,l,_,x);w=this.renderSeries({realIndex:x,pathFill:M,j:_,i:l,pathFrom:h,pathTo:u,strokeWidth:S,elSeries:w,x:o,y:s,series:e,barHeight:E,barWidth:T,elDataLabelsWrap:k,type:"bar",visibleSeries:0})}n.globals.seriesXvalues[x]=v,n.globals.seriesYvalues[x]=y,this.prevY.push(this.yArrj),this.prevYF.push(this.yArrjF),this.prevYVal.push(this.yArrjVal),this.prevX.push(this.xArrj),this.prevXF.push(this.xArrjF),this.prevXVal.push(this.xArrjVal),a.add(w)}return a}},{key:"initialPositions",value:function(e,t,n,i,r,a){var o,s,l=this.w;return this.isHorizontal?(o=(o=i=l.globals.gridHeight/l.globals.dataPoints)*parseInt(l.config.plotOptions.bar.barHeight)/100,a=this.baseLineInvertedY+l.globals.padHorizontal+(this.isReversed?l.globals.gridWidth:0)-(this.isReversed?2*this.baseLineInvertedY:0),t=(i-o)/2):(s=n=l.globals.gridWidth/l.globals.dataPoints,s=l.globals.isXNumeric?(n=l.globals.minXDiff/this.xRatio)*parseInt(this.barOptions.columnWidth)/100:s*parseInt(l.config.plotOptions.bar.columnWidth)/100,r=this.baseLineY[this.yaxisIndex]+(this.isReversed?l.globals.gridHeight:0)-(this.isReversed?2*this.baseLineY[this.yaxisIndex]:0),e=l.globals.padHorizontal+(n-s)/2),{x:e,y:t,yDivision:i,xDivision:n,barHeight:o,barWidth:s,zeroH:r,zeroW:a}}},{key:"drawBarPaths",value:function(e){for(var t,n=e.indexes,i=e.barHeight,r=e.strokeWidth,a=e.pathTo,o=e.pathFrom,s=e.zeroW,l=e.x,c=e.y,u=e.yDivision,h=e.elSeries,d=this.w,f=c,p=n.i,g=n.j,m=n.realIndex,v=n.bc,y=0,b=0;b<this.prevXF.length;b++)y+=this.prevXF[b][g];if(p>0){var x=s;this.prevXVal[p-1][g]<0?x=this.series[p][g]>=0?this.prevX[p-1][g]+y-2*(this.isReversed?y:0):this.prevX[p-1][g]:this.prevXVal[p-1][g]>=0&&(x=this.series[p][g]>=0?this.prevX[p-1][g]:this.prevX[p-1][g]-y+2*(this.isReversed?y:0)),t=x}else t=s;l=null===this.series[p][g]?t:t+this.series[p][g]/this.invertedYRatio-2*(this.isReversed?this.series[p][g]/this.invertedYRatio:0);var w={barHeight:i,strokeWidth:r,invertedYRatio:this.invertedYRatio,barYPosition:f,x:l},k=this.bar.barEndingShape(d,w,this.series,p,g);if(this.series.length>1&&p!==this.endingShapeOnSeriesNumber&&(k.path=this.graphics.line(k.newX,f+i-r)),this.xArrj.push(k.newX),this.xArrjF.push(Math.abs(t-k.newX)),this.xArrjVal.push(this.series[p][g]),a=this.graphics.move(t,f),o=this.graphics.move(t,f),d.globals.previousPaths.length>0&&(o=this.bar.getPathFrom(m,g,!1)),a=a+this.graphics.line(k.newX,f)+k.path+this.graphics.line(t,f+i-r)+this.graphics.line(t,f),o=o+this.graphics.line(t,f)+this.graphics.line(t,f+i-r)+this.graphics.line(t,f+i-r)+this.graphics.line(t,f+i-r)+this.graphics.line(t,f),d.config.plotOptions.bar.colors.backgroundBarColors.length>0&&0===p){v>=d.config.plotOptions.bar.colors.backgroundBarColors.length&&(v=0);var S=d.config.plotOptions.bar.colors.backgroundBarColors[v],E=this.graphics.drawRect(0,f,d.globals.gridWidth,i,0,S,d.config.plotOptions.bar.colors.backgroundBarOpacity);h.add(E),E.node.classList.add("apexcharts-backgroundBar")}return{pathTo:a,pathFrom:o,x:l,y:c+=u}}},{key:"drawColumnPaths",value:function(e){var t=e.indexes,n=e.x,i=e.y,r=e.xDivision,a=e.pathTo,o=e.pathFrom,s=e.barWidth,l=e.zeroH,c=e.strokeWidth,u=e.elSeries,h=this.w,d=t.i,f=t.j,p=t.realIndex,g=t.bc;if(h.globals.isXNumeric){var m=h.globals.seriesX[d][f];m||(m=0),n=(m-h.globals.minX)/this.xRatio-s/2}for(var v,y=n,b=0,x=0;x<this.prevYF.length;x++)b+=this.prevYF[x][f];if(d>0&&!h.globals.isXNumeric||d>0&&h.globals.isXNumeric&&h.globals.seriesX[d-1][f]===h.globals.seriesX[d][f]){var w=this.prevY[d-1][f];v=this.prevYVal[d-1][f]<0?this.series[d][f]>=0?w-b+2*(this.isReversed?b:0):w:this.series[d][f]>=0?w:w+b-2*(this.isReversed?b:0)}else v=h.globals.gridHeight-l;i=v-this.series[d][f]/this.yRatio[this.yaxisIndex]+2*(this.isReversed?this.series[d][f]/this.yRatio[this.yaxisIndex]:0);var k={barWidth:s,strokeWidth:c,yRatio:this.yRatio[this.yaxisIndex],barXPosition:y,y:i},S=this.bar.barEndingShape(h,k,this.series,d,f);if(this.yArrj.push(S.newY),this.yArrjF.push(Math.abs(v-S.newY)),this.yArrjVal.push(this.series[d][f]),a=this.graphics.move(y,v),o=this.graphics.move(y,v),h.globals.previousPaths.length>0&&(o=this.bar.getPathFrom(p,f,!1)),a=a+this.graphics.line(y,S.newY)+S.path+this.graphics.line(y+s-c,v)+this.graphics.line(y-c/2,v),o=o+this.graphics.line(y,v)+this.graphics.line(y+s-c,v)+this.graphics.line(y+s-c,v)+this.graphics.line(y+s-c,v)+this.graphics.line(y-c/2,v),h.config.plotOptions.bar.colors.backgroundBarColors.length>0&&0===d){g>=h.config.plotOptions.bar.colors.backgroundBarColors.length&&(g=0);var E=h.config.plotOptions.bar.colors.backgroundBarColors[g],C=this.graphics.drawRect(y,0,s,h.globals.gridHeight,0,E,h.config.plotOptions.bar.colors.backgroundBarOpacity);u.add(C),C.node.classList.add("apexcharts-backgroundBar")}return n+=r,{pathTo:a,pathFrom:o,x:h.globals.isXNumeric?n-r:n,y:i}}},{key:"checkZeroSeries",value:function(e){for(var t=e.series,n=this.w,i=0;i<t.length;i++){for(var r=0,a=0;a<t[n.globals.maxValsInArrayIndex].length;a++)r+=t[i][a];0===r&&this.zeroSerieses.push(i)}for(var o=t.length-1;o>=0;o--)this.zeroSerieses.indexOf(o)>-1&&o===this.endingShapeOnSeriesNumber&&(this.endingShapeOnSeriesNumber-=1)}}]),t}(),L=function(e){function t(){return o(this,t),p(this,d(t).apply(this,arguments))}return h(t,D),l(t,[{key:"draw",value:function(e,t){var n=this.w,i=new b(this.ctx),r=new O(this.ctx);this.candlestickOptions=this.w.config.plotOptions.candlestick;var a=new C(this.ctx,n);e=a.getLogSeries(e),this.series=e,this.yRatio=a.getLogYRatios(this.yRatio),this.initVariables(e);for(var o=i.group({class:"apexcharts-candlestick-series apexcharts-plot-series"}),s=0,l=0;s<e.length;s++,l++){var c,u,h=void 0,d=void 0,f=void 0,p=void 0,g=[],v=[],y=n.globals.comboCharts?t[s]:s,x=i.group({class:"apexcharts-series ".concat(m.escapeString(n.globals.seriesNames[y])),rel:s+1,"data:realIndex":y});e[s].length>0&&(this.visibleI=this.visibleI+1);var w,k,S=0;this.yRatio.length>1&&(this.yaxisIndex=y);var E=this.initialPositions();p=E.y,w=E.barHeight,f=E.x,k=E.barWidth,c=E.xDivision,u=E.zeroH,v.push(f+k/2);for(var T=i.group({class:"apexcharts-datalabels"}),A=0,_=n.globals.dataPoints;A<n.globals.dataPoints;A++,_--){void 0===this.series[s][A]||null===e[s][A]?this.isNullValue=!0:this.isNullValue=!1,n.config.stroke.show&&(S=this.isNullValue?0:Array.isArray(this.strokeWidth)?this.strokeWidth[y]:this.strokeWidth);var P,M=this.drawCandleStickPaths({indexes:{i:s,j:A,realIndex:y,bc:l},x:f,y:p,xDivision:c,pathTo:h,pathFrom:d,barWidth:k,zeroH:u,strokeWidth:S,elSeries:x});h=M.pathTo,d=M.pathFrom,p=M.y,f=M.x,P=M.color,A>0&&v.push(f+k/2),g.push(p);var I=r.fillPath({seriesNumber:y,color:P,value:e[s][A]}),D=this.candlestickOptions.wick.useFillColor?P:void 0;x=this.renderSeries({realIndex:y,pathFill:I,lineFill:D,j:A,i:s,pathFrom:d,pathTo:h,strokeWidth:S,elSeries:x,x:f,y:p,series:e,barHeight:w,barWidth:k,elDataLabelsWrap:T,visibleSeries:this.visibleI,type:"candlestick"})}n.globals.seriesXvalues[y]=v,n.globals.seriesYvalues[y]=g,o.add(x)}return o}},{key:"drawCandleStickPaths",value:function(e){var t=e.indexes,n=e.x,i=(e.y,e.xDivision),r=e.pathTo,a=e.pathFrom,o=e.barWidth,s=e.zeroH,l=e.strokeWidth,c=this.w,u=new b(this.ctx),h=t.i,d=t.j,f=!0,p=c.config.plotOptions.candlestick.colors.upward,g=c.config.plotOptions.candlestick.colors.downward,m=this.yRatio[this.yaxisIndex],v=t.realIndex,y=this.getOHLCValue(v,d),x=s,w=s;y.o>y.c&&(f=!1);var k=Math.min(y.o,y.c),S=Math.max(y.o,y.c);c.globals.isXNumeric&&(n=(c.globals.seriesX[h][d]-c.globals.minX)/this.xRatio-o/2);var E=n+o*this.visibleI;return void 0===this.series[h][d]||null===this.series[h][d]?k=s:(k=s-k/m,S=s-S/m,x=s-y.h/m,w=s-y.l/m),u.move(E,s),a=u.move(E,k),c.globals.previousPaths.length>0&&(a=this.getPathFrom(v,d,!0)),r=u.move(E,S)+u.line(E+o/2,S)+u.line(E+o/2,x)+u.line(E+o/2,S)+u.line(E+o,S)+u.line(E+o,k)+u.line(E+o/2,k)+u.line(E+o/2,w)+u.line(E+o/2,k)+u.line(E,k)+u.line(E,S-l/2),a+=u.move(E,k),c.globals.isXNumeric||(n+=i),{pathTo:r,pathFrom:a,x:n,y:S,barXPosition:E,color:f?p:g}}},{key:"getOHLCValue",value:function(e,t){var n=this.w;return{o:n.globals.seriesCandleO[e][t],h:n.globals.seriesCandleH[e][t],l:n.globals.seriesCandleL[e][t],c:n.globals.seriesCandleC[e][t]}}}]),t}(),R=function(){function e(t){o(this,e),this.ctx=t,this.w=t.w}return l(e,[{key:"drawXCrosshairs",value:function(){var e=this.w,t=new b(this.ctx),n=new v(this.ctx),i=e.config.xaxis.crosshairs.fill.gradient,r=e.config.xaxis.crosshairs.dropShadow,a=e.config.xaxis.crosshairs.fill.type,o=i.colorFrom,s=i.colorTo,l=i.opacityFrom,c=i.opacityTo,u=i.stops,h=r.enabled,d=r.left,f=r.top,p=r.blur,g=r.color,y=r.opacity,x=e.config.xaxis.crosshairs.fill.color;if(e.config.xaxis.crosshairs.show){"gradient"===a&&(x=t.drawGradient("vertical",o,s,l,c,null,u,null));var w=t.drawRect();1===e.config.xaxis.crosshairs.width&&(w=t.drawLine()),w.attr({class:"apexcharts-xcrosshairs",x:0,y:0,y2:e.globals.gridHeight,width:m.isNumber(e.config.xaxis.crosshairs.width)?e.config.xaxis.crosshairs.width:0,height:e.globals.gridHeight,fill:x,filter:"none","fill-opacity":e.config.xaxis.crosshairs.opacity,stroke:e.config.xaxis.crosshairs.stroke.color,"stroke-width":e.config.xaxis.crosshairs.stroke.width,"stroke-dasharray":e.config.xaxis.crosshairs.stroke.dashArray}),h&&(w=n.dropShadow(w,{left:d,top:f,blur:p,color:g,opacity:y})),e.globals.dom.elGraphical.add(w)}}},{key:"drawYCrosshairs",value:function(){var e=this.w,t=new b(this.ctx),n=e.config.yaxis[0].crosshairs;if(e.config.yaxis[0].crosshairs.show){var i=t.drawLine(0,0,e.globals.gridWidth,0,n.stroke.color,n.stroke.dashArray,n.stroke.width);i.attr({class:"apexcharts-ycrosshairs"}),e.globals.dom.elGraphical.add(i)}var r=t.drawLine(0,0,e.globals.gridWidth,0,n.stroke.color,0,0);r.attr({class:"apexcharts-ycrosshairs-hidden"}),e.globals.dom.elGraphical.add(r)}}]),e}(),F=function(){function e(t,n){o(this,e),this.ctx=t,this.w=t.w,this.xRatio=n.xRatio,this.yRatio=n.yRatio,this.negRange=!1,this.dynamicAnim=this.w.config.chart.animations.dynamicAnimation,this.rectRadius=this.w.config.plotOptions.heatmap.radius,this.strokeWidth=this.w.config.stroke.width}return l(e,[{key:"draw",value:function(e){var t=this.w,n=new b(this.ctx),i=n.group({class:"apexcharts-heatmap"});i.attr("clip-path","url(#gridRectMask".concat(t.globals.cuid,")"));var r=t.globals.gridWidth/t.globals.dataPoints,a=t.globals.gridHeight/t.globals.series.length,o=0,s=!1;this.checkColorRange();var l=e.slice();t.config.yaxis[0].reversed&&(s=!0,l.reverse());for(var c=s?0:l.length-1;s?c<l.length:c>=0;s?c++:c--){var u=n.group({class:"apexcharts-series apexcharts-heatmap-series ".concat(m.escapeString(t.globals.seriesNames[c])),rel:c+1,"data:realIndex":c});if(t.config.chart.dropShadow.enabled){var h=t.config.chart.dropShadow;new v(this.ctx).dropShadow(u,h,c)}for(var d=0,f=0;f<l[c].length;f++){var p=1,g=this.determineHeatColor(c,f);if(t.globals.hasNegs||this.negRange){var y=t.config.plotOptions.heatmap.shadeIntensity;p=g.percent<0?1-(1+g.percent/100)*y:(1-g.percent/100)*y}else p=1-g.percent/100;var x=g.color;if(t.config.plotOptions.heatmap.enableShades){var w=new m;x=m.hexToRgba(w.shadeColor(p,g.color),t.config.fill.opacity)}var k=this.rectRadius,S=n.drawRect(d,o,r,a,k);if(S.attr({cx:d,cy:o}),S.node.classList.add("apexcharts-heatmap-rect"),u.add(S),S.attr({fill:x,i:c,index:c,j:f,val:l[c][f],"stroke-width":this.strokeWidth,stroke:t.globals.stroke.colors[0],color:x}),S.node.addEventListener("mouseenter",n.pathMouseEnter.bind(this,S)),S.node.addEventListener("mouseleave",n.pathMouseLeave.bind(this,S)),S.node.addEventListener("mousedown",n.pathMouseDown.bind(this,S)),t.config.chart.animations.enabled&&!t.globals.dataChanged){var E=1;t.globals.resized||(E=t.config.chart.animations.speed),this.animateHeatMap(S,d,o,r,a,E)}if(t.globals.dataChanged){var C=1;if(this.dynamicAnim.enabled&&t.globals.shouldAnimate){C=this.dynamicAnim.speed;var T=t.globals.previousPaths[c]&&t.globals.previousPaths[c][f]&&t.globals.previousPaths[c][f].color;T||(T="rgba(255, 255, 255, 0)"),this.animateHeatColor(S,m.isColorHex(T)?T:m.rgb2hex(T),m.isColorHex(x)?x:m.rgb2hex(x),C)}}var A=this.calculateHeatmapDataLabels({x:d,y:o,i:c,j:f,series:l,rectHeight:a,rectWidth:r});null!==A&&u.add(A),d+=r}o+=a,i.add(u)}var _=t.globals.yAxisScale[0].result.slice();t.config.yaxis[0].reversed?_.unshift(""):_.push(""),t.globals.yAxisScale[0].result=_;var O=t.globals.gridHeight/t.globals.series.length;return t.config.yaxis[0].labels.offsetY=-O/2,i}},{key:"checkColorRange",value:function(){var e=this,t=this.w.config.plotOptions.heatmap;t.colorScale.ranges.length>0&&t.colorScale.ranges.map(function(t,n){t.from<0&&(e.negRange=!0)})}},{key:"determineHeatColor",value:function(e,t){var n=this.w,i=n.globals.series[e][t],r=n.config.plotOptions.heatmap,a=r.colorScale.inverse?t:e,o=n.globals.colors[a],s=Math.min.apply(Math,g(n.globals.series[e])),l=Math.max.apply(Math,g(n.globals.series[e]));r.distributed||(s=n.globals.minY,l=n.globals.maxY),void 0!==r.colorScale.min&&(s=r.colorScale.min<n.globals.minY?r.colorScale.min:n.globals.minY,l=r.colorScale.max>n.globals.maxY?r.colorScale.max:n.globals.maxY);var c=Math.abs(l)+Math.abs(s),u=100*i/(0===c?c-1e-6:c);r.colorScale.ranges.length>0&&r.colorScale.ranges.map(function(e,t){if(i>=e.from&&i<=e.to){o=e.color,s=e.from,l=e.to;var n=Math.abs(l)+Math.abs(s);u=100*i/(0===n?n-1e-6:n)}});return{color:o,percent:u}}},{key:"calculateHeatmapDataLabels",value:function(e){var t=e.x,n=e.y,i=e.i,r=e.j,a=(e.series,e.rectHeight),o=e.rectWidth,s=this.w,l=s.config.dataLabels,c=new b(this.ctx),u=new I(this.ctx),h=l.formatter,d=null;if(l.enabled){d=c.group({class:"apexcharts-data-labels"});var f=l.offsetX,p=l.offsetY,g=t+o/2+f,m=n+a/2+parseInt(l.style.fontSize)/3+p,v=h(s.globals.series[i][r],{seriesIndex:i,dataPointIndex:r,w:s});u.plotDataLabelsText({x:g,y:m,text:v,i:i,j:r,parent:d,dataLabelsConfig:l})}return d}},{key:"animateHeatMap",value:function(e,t,n,i,r,a){var o=this;new y(this.ctx).animateRect(e,{x:t+i/2,y:n+r/2,width:0,height:0},{x:t,y:n,width:i,height:r},a,function(){o.w.globals.animationEnded=!0})}},{key:"animateHeatColor",value:function(e,t,n,i){e.attr({fill:t}).animate(i).attr({fill:n})}}]),e}(),j=function(){function e(t){o(this,e),this.ctx=t,this.w=t.w,this.chartType=this.w.config.chart.type,this.initialAnim=this.w.config.chart.animations.enabled,this.dynamicAnim=this.initialAnim&&this.w.config.chart.animations.dynamicAnimation.enabled,this.animBeginArr=[0],this.animDur=0,this.donutDataLabels=this.w.config.plotOptions.pie.donut.labels;var n=this.w;this.lineColorArr=void 0!==n.globals.stroke.colors?n.globals.stroke.colors:n.globals.colors,this.defaultSize=n.globals.svgHeight<n.globals.svgWidth?n.globals.svgHeight-35:n.globals.gridWidth,this.centerY=this.defaultSize/2,this.centerX=n.globals.gridWidth/2,this.fullAngle=360,this.size=0,this.donutSize=0,this.sliceLabels=[],this.prevSectorAngleArr=[]}return l(e,[{key:"draw",value:function(e){for(var t=this.w,n=new b(this.ctx),i=n.group({class:"apexcharts-pie"}),r=0,a=0;a<e.length;a++)r+=m.negToZero(e[a]);var o=[],s=n.group();0===r&&(r=1e-5);for(var l=0;l<e.length;l++){var c=this.fullAngle*m.negToZero(e[l])/r;o.push(c)}if(t.globals.dataChanged){for(var u,h=0,d=0;d<t.globals.previousPaths.length;d++)h+=m.negToZero(t.globals.previousPaths[d]);for(var f=0;f<t.globals.previousPaths.length;f++)u=this.fullAngle*m.negToZero(t.globals.previousPaths[f])/h,this.prevSectorAngleArr.push(u)}this.size=this.defaultSize/2.05-t.config.stroke.width-t.config.chart.dropShadow.blur,void 0!==t.config.plotOptions.pie.size&&(this.size=t.config.plotOptions.pie.size),this.donutSize=this.size*parseInt(t.config.plotOptions.pie.donut.size)/100;var p=t.config.plotOptions.pie.customScale,g=t.globals.gridWidth/2,v=t.globals.gridHeight/2,y=g-t.globals.gridWidth/2*p,x=v-t.globals.gridHeight/2*p;if(this.donutDataLabels.show){var w=this.renderInnerDataLabels(this.donutDataLabels,{hollowSize:this.donutSize,centerX:this.centerX,centerY:this.centerY,opacity:this.donutDataLabels.show,translateX:y,translateY:x});i.add(w)}if("donut"===t.config.chart.type){var k=n.drawCircle(this.donutSize);k.attr({cx:this.centerX,cy:this.centerY,fill:t.config.plotOptions.pie.donut.background}),s.add(k)}var S=this.drawArcs(o,e);return this.sliceLabels.forEach(function(e){S.add(e)}),s.attr({transform:"translate(".concat(y,", ").concat(x-5,") scale(").concat(p,")")}),i.attr({"data:innerTranslateX":y,"data:innerTranslateY":x-25}),s.add(S),i.add(s),i}},{key:"drawArcs",value:function(e,t){var n=this.w,i=new v(this.ctx),r=new b(this.ctx),a=new O(this.ctx),o=r.group(),s=0,l=0,c=0,u=0;this.strokeWidth=n.config.stroke.show?n.config.stroke.width:0;for(var h=0;h<e.length;h++){var d=r.group({class:"apexcharts-series apexcharts-pie-series ".concat(m.escapeString(n.globals.seriesNames[h])),id:"apexcharts-series-"+h,rel:h+1});o.add(d),l=u,c=(s=c)+e[h],u=l+this.prevSectorAngleArr[h];var f=c-s,p=a.fillPath({seriesNumber:h,size:this.size,value:t[h]}),g=this.getChangedPath(l,u),y=r.drawPath({d:g,stroke:this.lineColorArr instanceof Array?this.lineColorArr[h]:this.lineColorArr,strokeWidth:this.strokeWidth,fill:p,fillOpacity:n.config.fill.opacity,classes:"apexcharts-pie-area"});if(y.attr({id:"apexcharts-".concat(n.config.chart.type,"-slice-").concat(h),index:0,j:h}),n.config.chart.dropShadow.enabled){var x=n.config.chart.dropShadow;i.dropShadow(y,x,h)}this.addListeners(y,this.donutDataLabels),b.setAttrs(y.node,{"data:angle":f,"data:startAngle":s,"data:strokeWidth":this.strokeWidth,"data:value":t[h]});var w={x:0,y:0};"pie"===n.config.chart.type?w=m.polarToCartesian(this.centerX,this.centerY,this.size/1.25+n.config.plotOptions.pie.dataLabels.offset,s+(c-s)/2):"donut"===n.config.chart.type&&(w=m.polarToCartesian(this.centerX,this.centerY,(this.size+this.donutSize)/2+n.config.plotOptions.pie.dataLabels.offset,s+(c-s)/2)),d.add(y);var k=0;if(!this.initialAnim||n.globals.resized||n.globals.dataChanged?this.animBeginArr.push(0):(k=(c-s)/this.fullAngle*n.config.chart.animations.speed,this.animDur=k+this.animDur,this.animBeginArr.push(this.animDur)),this.dynamicAnim&&n.globals.dataChanged?this.animatePaths(y,{endAngle:c,startAngle:s,prevStartAngle:l,prevEndAngle:u,animateStartingPos:!0,i:h,animBeginArr:this.animBeginArr,dur:n.config.chart.animations.dynamicAnimation.speed}):this.animatePaths(y,{endAngle:c,startAngle:s,i:h,totalItems:e.length-1,animBeginArr:this.animBeginArr,dur:k}),n.config.plotOptions.pie.expandOnClick&&y.click(this.pieClicked.bind(this,h)),n.config.dataLabels.enabled){var S=w.x,E=w.y,C=100*(c-s)/360+"%";if(0!==f&&n.config.plotOptions.pie.dataLabels.minAngleToShowLabel<e[h]){var T=n.config.dataLabels.formatter;void 0!==T&&(C=T(n.globals.seriesPercent[h][0],{seriesIndex:h,w:n}));var A=n.globals.dataLabels.style.colors[h],_=r.drawText({x:S,y:E,text:C,textAnchor:"middle",fontSize:n.config.dataLabels.style.fontSize,fontFamily:n.config.dataLabels.style.fontFamily,foreColor:A});if(n.config.dataLabels.dropShadow.enabled){var P=n.config.dataLabels.dropShadow;new v(this.ctx).dropShadow(_,P)}_.node.classList.add("apexcharts-pie-label"),n.config.chart.animations.animate&&!1===n.globals.resized&&(_.node.classList.add("apexcharts-pie-label-delay"),_.node.style.animationDelay=n.config.chart.animations.speed/940+"s"),this.sliceLabels.push(_)}}}return o}},{key:"addListeners",value:function(e,t){var n=new b(this.ctx);e.node.addEventListener("mouseenter",n.pathMouseEnter.bind(this,e)),e.node.addEventListener("mouseenter",this.printDataLabelsInner.bind(this,e.node,t)),e.node.addEventListener("mouseleave",n.pathMouseLeave.bind(this,e)),e.node.addEventListener("mouseleave",this.revertDataLabelsInner.bind(this,e.node,t)),e.node.addEventListener("mousedown",n.pathMouseDown.bind(this,e)),e.node.addEventListener("mousedown",this.printDataLabelsInner.bind(this,e.node,t))}},{key:"animatePaths",value:function(e,t){var n=this.w,i=t.endAngle-t.startAngle,r=i,a=t.startAngle,o=t.startAngle;void 0!==t.prevStartAngle&&void 0!==t.prevEndAngle&&(a=t.prevEndAngle,r=t.prevEndAngle-t.prevStartAngle),t.i===n.config.series.length-1&&(i+o>this.fullAngle?t.endAngle=t.endAngle-(i+o):i+o<this.fullAngle&&(t.endAngle=t.endAngle+(this.fullAngle-(i+o)))),i===this.fullAngle&&(i=this.fullAngle-.01),this.animateArc(e,a,o,i,r,t)}},{key:"animateArc",value:function(e,t,n,i,r,a){var o,s=this,l=this.w,c=s.size;c||(c=a.size),(isNaN(t)||isNaN(r))&&(t=n,r=i,a.dur=0);var u=i,h=n,d=t-n;l.globals.dataChanged&&a.shouldSetPrevPaths&&(o=s.getPiePath({me:s,startAngle:h,angle:r,size:c}),e.attr({d:o})),0!==a.dur?e.animate(a.dur,l.globals.easing,a.animBeginArr[a.i]).afterAll(function(){"pie"!==l.config.chart.type&&"donut"!==l.config.chart.type||this.animate(300).attr({"stroke-width":l.config.stroke.width}),l.globals.animationEnded=!0}).during(function(l){u=d+(i-d)*l,a.animateStartingPos&&(u=r+(i-r)*l,h=t-r+(n-(t-r))*l),o=s.getPiePath({me:s,startAngle:h,angle:u,size:c}),e.node.setAttribute("data:pathOrig",o),e.attr({d:o})}):(o=s.getPiePath({me:s,startAngle:h,angle:i,size:c}),a.isTrack||(l.globals.animationEnded=!0),e.node.setAttribute("data:pathOrig",o),e.attr({d:o}))}},{key:"pieClicked",value:function(e){var t,n=this.w,i=this.size+4,r=n.globals.dom.Paper.select("#apexcharts-".concat(n.config.chart.type.toLowerCase(),"-slice-").concat(e)).members[0],a=r.attr("d");if("true"!==r.attr("data:pieClicked")){var o=n.globals.dom.baseEl.querySelectorAll(".apexcharts-pie-area");Array.prototype.forEach.call(o,function(e){e.setAttribute("data:pieClicked","false");var t=e.getAttribute("data:pathOrig");e.setAttribute("d",t)}),r.attr("data:pieClicked","true");var s=parseInt(r.attr("data:startAngle")),l=parseInt(r.attr("data:angle"));t=this.getPiePath({me:this,startAngle:s,angle:l,size:i}),360!==l&&r.plot(t).animate(1).plot(a).animate(100).plot(t)}else{r.attr({"data:pieClicked":"false"}),this.revertDataLabelsInner(r.node,this.donutDataLabels);var c=r.attr("data:pathOrig");r.attr({d:c})}}},{key:"getChangedPath",value:function(e,t){var n="";return this.dynamicAnim&&this.w.globals.dataChanged&&(n=this.getPiePath({me:this,startAngle:e,angle:t-e,size:this.size})),n}},{key:"getPiePath",value:function(e){var t=e.me,n=e.startAngle,i=e.angle,r=e.size,a=this.w,o=n,s=Math.PI*(o-90)/180,l=i+n;Math.ceil(l)>=360&&(l=359.99);var c=Math.PI*(l-90)/180,u=t.centerX+r*Math.cos(s),h=t.centerY+r*Math.sin(s),d=t.centerX+r*Math.cos(c),f=t.centerY+r*Math.sin(c),p=m.polarToCartesian(t.centerX,t.centerY,t.donutSize,l),g=m.polarToCartesian(t.centerX,t.centerY,t.donutSize,o),v=i>180?1:0;return"donut"===a.config.chart.type?["M",u,h,"A",r,r,0,v,1,d,f,"L",p.x,p.y,"A",t.donutSize,t.donutSize,0,v,0,g.x,g.y,"L",u,h,"z"].join(" "):"pie"===a.config.chart.type?["M",u,h,"A",r,r,0,v,1,d,f,"L",t.centerX,t.centerY,"L",u,h].join(" "):["M",u,h,"A",r,r,0,v,1,d,f].join(" ")}},{key:"renderInnerDataLabels",value:function(e,t){var n=this.w,i=new b(this.ctx),r=i.group({class:"apexcharts-datalabels-group",transform:"translate(".concat(t.translateX?t.translateX:0,", ").concat(t.translateY?t.translateY:0,")")}),a=e.total.show;r.node.style.opacity=t.opacity;var o,s,l=t.centerX,c=t.centerY;o=void 0===e.name.color?n.globals.colors[0]:e.name.color,s=void 0===e.value.color?n.config.chart.foreColor:e.value.color;var u=e.value.formatter,h="",d="";if(a?(o=e.total.color,d=e.total.label,h=e.total.formatter(n)):1===n.globals.series.length&&(h=u(n.globals.series[0],n),d=n.globals.seriesNames[0]),e.name.show){var f=i.drawText({x:l,y:c+parseInt(e.name.offsetY),text:d,textAnchor:"middle",foreColor:o,fontSize:e.name.fontSize,fontFamily:e.name.fontFamily});f.node.classList.add("apexcharts-datalabel-label"),r.add(f)}if(e.value.show){var p=e.name.show?parseInt(e.value.offsetY)+16:e.value.offsetY,g=i.drawText({x:l,y:c+p,text:h,textAnchor:"middle",foreColor:s,fontSize:e.value.fontSize,fontFamily:e.value.fontFamily});g.node.classList.add("apexcharts-datalabel-value"),r.add(g)}return r}},{key:"printInnerLabels",value:function(e,t,n,i){var r,a=this.w;i?r=void 0===e.name.color?a.globals.colors[parseInt(i.parentNode.getAttribute("rel"))-1]:e.name.color:a.globals.series.length>1&&e.total.show&&(r=e.total.color);var o=a.globals.dom.baseEl.querySelector(".apexcharts-datalabel-label"),s=a.globals.dom.baseEl.querySelector(".apexcharts-datalabel-value");n=(0,e.value.formatter)(n,a),i||"function"!=typeof e.total.formatter||(n=e.total.formatter(a)),null!==o&&(o.textContent=t),null!==s&&(s.textContent=n),null!==o&&(o.style.fill=r)}},{key:"printDataLabelsInner",value:function(e,t){var n=this.w,i=e.getAttribute("data:value"),r=n.globals.seriesNames[parseInt(e.parentNode.getAttribute("rel"))-1];n.globals.series.length>1&&this.printInnerLabels(t,r,i,e);var a=n.globals.dom.baseEl.querySelector(".apexcharts-datalabels-group");null!==a&&(a.style.opacity=1)}},{key:"revertDataLabelsInner",value:function(t,n,i){var r=this,a=this.w,o=a.globals.dom.baseEl.querySelector(".apexcharts-datalabels-group");if(n.total.show&&a.globals.series.length>1){new e(this.ctx).printInnerLabels(n,n.total.label,n.total.formatter(a))}else{var s=document.querySelectorAll(".apexcharts-pie-area"),l=!1;if(s.forEach(function(e){"true"===e.getAttribute("data:pieClicked")&&(l=!0,r.printDataLabelsInner(e,n))}),!l)if(a.globals.selectedDataPoints.length&&a.globals.series.length>1)if(a.globals.selectedDataPoints[0].length>0){var c=a.globals.selectedDataPoints[0],u=a.globals.dom.baseEl.querySelector("#apexcharts-".concat(a.config.chart.type.toLowerCase(),"-slice-").concat(c));this.printDataLabelsInner(u,n)}else o&&a.globals.selectedDataPoints.length&&0===a.globals.selectedDataPoints[0].length&&(o.style.opacity=0);else o&&a.globals.series.length>1&&(o.style.opacity=0)}}}]),e}(),z=function(){function e(t){o(this,e),this.ctx=t,this.w=t.w,this.chartType=this.w.config.chart.type,this.initialAnim=this.w.config.chart.animations.enabled,this.dynamicAnim=this.initialAnim&&this.w.config.chart.animations.dynamicAnimation.enabled,this.animDur=0;var n=this.w;this.graphics=new b(this.ctx),this.lineColorArr=void 0!==n.globals.stroke.colors?n.globals.stroke.colors:n.globals.colors,this.defaultSize=n.globals.svgHeight<n.globals.svgWidth?n.globals.svgHeight-35:n.globals.gridWidth,this.maxValue=this.w.globals.maxY,this.polygons=n.config.plotOptions.radar.polygons,this.maxLabelWidth=20;var i=n.globals.labels.slice().sort(function(e,t){return t.length-e.length})[0],r=this.graphics.getTextRects(i,n.config.dataLabels.style.fontSize);this.size=this.defaultSize/2.1-n.config.stroke.width-n.config.chart.dropShadow.blur-r.width/1.75,void 0!==n.config.plotOptions.radar.size&&(this.size=n.config.plotOptions.radar.size),this.dataRadiusOfPercent=[],this.dataRadius=[],this.angleArr=[],this.yaxisLabelsTextsPos=[]}return l(e,[{key:"draw",value:function(e){var t=this,n=this.w,i=new O(this.ctx),r=[];this.dataPointsLen=e[n.globals.maxValsInArrayIndex].length,this.disAngle=2*Math.PI/this.dataPointsLen;var a=n.globals.gridWidth/2,o=n.globals.gridHeight/2,s=this.graphics.group({class:"apexcharts-radar-series","data:innerTranslateX":a,"data:innerTranslateY":o-25,transform:"translate(".concat(a||0,", ").concat(o||0,")")}),l=[],c=null;if(this.yaxisLabels=this.graphics.group({class:"apexcharts-yaxis"}),e.forEach(function(e,a){var o=t.graphics.group().attr({class:"apexcharts-series ".concat(m.escapeString(n.globals.seriesNames[a])),rel:a+1,"data:realIndex":a});t.dataRadiusOfPercent[a]=[],t.dataRadius[a]=[],t.angleArr[a]=[],e.forEach(function(e,n){t.dataRadiusOfPercent[a][n]=e/t.maxValue,t.dataRadius[a][n]=t.dataRadiusOfPercent[a][n]*t.size,t.angleArr[a][n]=n*t.disAngle}),l=t.getDataPointsPos(t.dataRadius[a],t.angleArr[a]);var s=t.createPaths(l,{x:0,y:0});c=t.graphics.group({class:"apexcharts-series-markers-wrap hidden"}),n.globals.delayedElements.push({el:c.node,index:a});var h={i:a,realIndex:a,animationDelay:a,initialSpeed:n.config.chart.animations.speed,dataChangeSpeed:n.config.chart.animations.dynamicAnimation.speed,className:"apexcharts-radar",id:"apexcharts-radar",shouldClipToGrid:!1,bindEventsOnPaths:!1,stroke:n.globals.stroke.colors[a],strokeLineCap:n.config.stroke.lineCap},d=null;n.globals.previousPaths.length>0&&(d=t.getPathFrom(a));for(var f=0;f<s.linePathsTo.length;f++){var p=t.graphics.renderPaths(u({},h,{pathFrom:null===d?s.linePathsFrom[f]:d,pathTo:s.linePathsTo[f],strokeWidth:Array.isArray(n.config.stroke.width)?n.config.stroke.width[a]:n.config.stroke.width,fill:"none",drawShadow:!1}));o.add(p);var g=i.fillPath({seriesNumber:a}),y=t.graphics.renderPaths(u({},h,{pathFrom:null===d?s.areaPathsFrom[f]:d,pathTo:s.areaPathsTo[f],strokeWidth:0,fill:g,drawShadow:!1}));if(n.config.chart.dropShadow.enabled){var b=new v(t.ctx),x=n.config.chart.dropShadow;b.dropShadow(y,Object.assign({},x,{noUserSpaceOnUse:!0}),a)}o.add(y)}e.forEach(function(e,n){var i=new P(t.ctx).getMarkerConfig("apexcharts-marker",a),r=t.graphics.drawMarker(l[n].x,l[n].y,i);r.attr("rel",n),r.attr("j",n),r.attr("index",a),r.node.setAttribute("default-marker-size",i.pSize);var s=t.graphics.group({class:"apexcharts-series-markers"});s&&s.add(r),c.add(s),o.add(c)}),r.push(o)}),this.drawPolygons({parent:s}),n.config.dataLabels.enabled){var h=this.drawLabels();s.add(h)}return s.add(this.yaxisLabels),r.forEach(function(e){s.add(e)}),s}},{key:"drawPolygons",value:function(e){for(var t=this,n=this.w,i=e.parent,r=n.globals.yAxisScale[0].result.reverse(),a=r.length,o=[],s=this.size/(a-1),l=0;l<a;l++)o[l]=s*l;o.reverse();var c=[],u=[];o.forEach(function(e,n){var i=t.getPolygonPos(e),r="";i.forEach(function(e,i){if(0===n){var a=t.graphics.drawLine(e.x,e.y,0,0,Array.isArray(t.polygons.connectorColors)?t.polygons.connectorColors[i]:t.polygons.connectorColors);u.push(a)}0===i&&t.yaxisLabelsTextsPos.push({x:e.x,y:e.y}),r+=e.x+","+e.y+" "}),c.push(r)}),c.forEach(function(e,r){var a=t.polygons.strokeColors,o=t.graphics.drawPolygon(e,Array.isArray(a)?a[r]:a,n.globals.radarPolygons.fill.colors[r]);i.add(o)}),u.forEach(function(e){i.add(e)}),n.config.yaxis[0].show&&this.yaxisLabelsTextsPos.forEach(function(e,n){var i=t.drawYAxisText(e.x,e.y,n,r[n]);t.yaxisLabels.add(i)})}},{key:"drawYAxisText",value:function(e,t,n,i){var r=this.w,a=r.config.yaxis[0],o=r.globals.yLabelFormatters[0];return this.graphics.drawText({x:e+a.labels.offsetX,y:t+a.labels.offsetY,text:o(i,n),textAnchor:"middle",fontSize:a.labels.style.fontSize,fontFamily:a.labels.style.fontFamily,foreColor:a.labels.style.color})}},{key:"drawLabels",value:function(){var e=this,t=this.w,n="middle",i=t.config.dataLabels,r=this.graphics.group({class:"apexcharts-datalabels"}),a=this.getPolygonPos(this.size),o=0,s=0;return t.globals.labels.forEach(function(l,c){var u=i.formatter,h=new I(e.ctx);if(a[c]){o=a[c].x,s=a[c].y,Math.abs(a[c].x)>=10?a[c].x>0?(n="start",o+=10):a[c].x<0&&(n="end",o-=10):n="middle",Math.abs(a[c].y)>=e.size-10&&(a[c].y<0?s-=10:a[c].y>0&&(s+=10));var d=u(l,{seriesIndex:-1,dataPointIndex:c,w:t});h.plotDataLabelsText({x:o,y:s,text:d,textAnchor:n,i:c,j:c,parent:r,dataLabelsConfig:i,offsetCorrection:!1})}}),r}},{key:"createPaths",value:function(e,t){var n=this,i=[],r=[],a=[],o=[];if(e.length){r=[this.graphics.move(t.x,t.y)],o=[this.graphics.move(t.x,t.y)];var s=this.graphics.move(e[0].x,e[0].y),l=this.graphics.move(e[0].x,e[0].y);e.forEach(function(t,i){s+=n.graphics.line(t.x,t.y),l+=n.graphics.line(t.x,t.y),i===e.length-1&&(s+="Z",l+="Z")}),i.push(s),a.push(l)}return{linePathsFrom:r,linePathsTo:i,areaPathsFrom:o,areaPathsTo:a}}},{key:"getPathFrom",value:function(e){for(var t=this.w,n=null,i=0;i<t.globals.previousPaths.length;i++){var r=t.globals.previousPaths[i];r.paths.length>0&&parseInt(r.realIndex)===parseInt(e)&&void 0!==t.globals.previousPaths[i].paths[0]&&(n=t.globals.previousPaths[i].paths[0].d)}return n}},{key:"getDataPointsPos",value:function(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:this.dataPointsLen;e=e||[],t=t||[];for(var i=[],r=0;r<n;r++){var a={};a.x=e[r]*Math.sin(t[r]),a.y=-e[r]*Math.cos(t[r]),i.push(a)}return i}},{key:"getPolygonPos",value:function(e){for(var t=[],n=2*Math.PI/this.dataPointsLen,i=0;i<this.dataPointsLen;i++){var r={};r.x=e*Math.sin(i*n),r.y=-e*Math.cos(i*n),t.push(r)}return t}}]),e}(),Y=function(e){function t(e){var n;o(this,t),(n=p(this,d(t).call(this,e))).ctx=e,n.w=e.w,n.animBeginArr=[0],n.animDur=0;var i=n.w;return n.startAngle=i.config.plotOptions.radialBar.startAngle,n.endAngle=i.config.plotOptions.radialBar.endAngle,n.trackStartAngle=i.config.plotOptions.radialBar.track.startAngle,n.trackEndAngle=i.config.plotOptions.radialBar.track.endAngle,n.radialDataLabels=i.config.plotOptions.radialBar.dataLabels,n.trackStartAngle||(n.trackStartAngle=n.startAngle),n.trackEndAngle||(n.trackEndAngle=n.endAngle),360===n.endAngle&&(n.endAngle=359.99),n.fullAngle=360-i.config.plotOptions.radialBar.endAngle-i.config.plotOptions.radialBar.startAngle,n.margin=parseInt(i.config.plotOptions.radialBar.track.margin),n}return h(t,j),l(t,[{key:"draw",value:function(e){var t=this.w,n=new b(this.ctx),i=n.group({class:"apexcharts-radialbar"}),r=n.group(),a=this.defaultSize/2,o=t.globals.gridWidth/2,s=this.defaultSize/2.05-t.config.stroke.width-t.config.chart.dropShadow.blur;void 0!==t.config.plotOptions.radialBar.size&&(s=t.config.plotOptions.radialBar.size);var l=t.globals.fill.colors;if(t.config.plotOptions.radialBar.track.show){var c=this.drawTracks({size:s,centerX:o,centerY:a,colorArr:l,series:e});r.add(c)}var u=this.drawArcs({size:s,centerX:o,centerY:a,colorArr:l,series:e});return r.add(u.g),"front"===t.config.plotOptions.radialBar.hollow.position&&(u.g.add(u.elHollow),u.dataLabels&&u.g.add(u.dataLabels)),i.add(r),i}},{key:"drawTracks",value:function(e){var t=this.w,n=new b(this.ctx),i=n.group(),r=new v(this.ctx),a=new O(this.ctx),o=this.getStrokeWidth(e);e.size=e.size-o/2;for(var s=0;s<e.series.length;s++){var l=n.group({class:"apexcharts-radialbar-track apexcharts-track"});i.add(l),l.attr({id:"apexcharts-track-"+s,rel:s+1}),e.size=e.size-o-this.margin;var c=t.config.plotOptions.radialBar.track,u=a.fillPath({seriesNumber:0,size:e.size,fillColors:Array.isArray(c.background)?c.background[s]:c.background,solid:!0}),h=this.trackStartAngle,d=this.trackEndAngle;Math.abs(d)+Math.abs(h)>=360&&(d=360-Math.abs(this.startAngle)-.1);var f=n.drawPath({d:"",stroke:u,strokeWidth:o*parseInt(c.strokeWidth)/100,fill:"none",strokeOpacity:c.opacity,classes:"apexcharts-radialbar-area"});if(c.dropShadow.enabled){var p=c.dropShadow;r.dropShadow(f,p)}l.add(f),f.attr("id","apexcharts-radialbarTrack-"+s),new j(this.ctx).animatePaths(f,{centerX:e.centerX,centerY:e.centerY,endAngle:d,startAngle:h,size:e.size,i:s,totalItems:2,animBeginArr:0,dur:0,isTrack:!0,easing:t.globals.easing})}return i}},{key:"drawArcs",value:function(e){var t=this.w,n=new b(this.ctx),i=new O(this.ctx),r=new v(this.ctx),a=n.group(),o=this.getStrokeWidth(e);e.size=e.size-o/2;var s=t.config.plotOptions.radialBar.hollow.background,l=e.size-o*e.series.length-this.margin*e.series.length-o*parseInt(t.config.plotOptions.radialBar.track.strokeWidth)/100/2,c=l-t.config.plotOptions.radialBar.hollow.margin;void 0!==t.config.plotOptions.radialBar.hollow.image&&(s=this.drawHollowImage(e,a,l,s));var u=this.drawHollow({size:c,centerX:e.centerX,centerY:e.centerY,fill:s});if(t.config.plotOptions.radialBar.hollow.dropShadow.enabled){var h=t.config.plotOptions.radialBar.hollow.dropShadow;r.dropShadow(u,h)}var d=1;!this.radialDataLabels.total.show&&t.globals.series.length>1&&(d=0);var f=new j(this.ctx),p=null;this.radialDataLabels.show&&(p=f.renderInnerDataLabels(this.radialDataLabels,{hollowSize:l,centerX:e.centerX,centerY:e.centerY,opacity:d})),"back"===t.config.plotOptions.radialBar.hollow.position&&(a.add(u),p&&a.add(p));var g=!1;t.config.plotOptions.radialBar.inverseOrder&&(g=!0);for(var y=g?e.series.length-1:0;g?y>=0:y<e.series.length;g?y--:y++){var x=n.group({class:"apexcharts-series apexcharts-radial-series ".concat(m.escapeString(t.globals.seriesNames[y]))});a.add(x),x.attr({id:"apexcharts-series-"+y,rel:y+1}),this.ctx.series.addCollapsedClassToSeries(x,y),e.size=e.size-o-this.margin;var w=i.fillPath({seriesNumber:y,size:e.size,value:e.series[y]}),k=this.startAngle,S=void 0,E=Math.abs(t.config.plotOptions.radialBar.endAngle-t.config.plotOptions.radialBar.startAngle),C=m.negToZero(e.series[y]>100?100:e.series[y])/100,T=Math.round(E*C)+this.startAngle,A=void 0;t.globals.dataChanged&&(S=this.startAngle,A=Math.round(E*m.negToZero(t.globals.previousPaths[y])/100)+S),Math.abs(T)+Math.abs(k)>=360&&(T-=.01),Math.abs(A)+Math.abs(S)>=360&&(A-=.01);var _=T-k,P=Array.isArray(t.config.stroke.dashArray)?t.config.stroke.dashArray[y]:t.config.stroke.dashArray,M=n.drawPath({d:"",stroke:w,strokeWidth:o,fill:"none",fillOpacity:t.config.fill.opacity,classes:"apexcharts-radialbar-area",strokeDashArray:P});if(b.setAttrs(M.node,{"data:angle":_,"data:value":e.series[y]}),t.config.chart.dropShadow.enabled){var I=t.config.chart.dropShadow;r.dropShadow(M,I,y)}this.addListeners(M,this.radialDataLabels);var D=new j(this.ctx);x.add(M),M.attr({id:"apexcharts-radialbar-slice-"+y,index:0,j:y});var N=0;!D.initialAnim||t.globals.resized||t.globals.dataChanged||(N=(T-k)/360*t.config.chart.animations.speed,this.animDur=N/(1.2*e.series.length)+this.animDur,this.animBeginArr.push(this.animDur)),t.globals.dataChanged&&(N=(T-k)/360*t.config.chart.animations.dynamicAnimation.speed,this.animDur=N/(1.2*e.series.length)+this.animDur,this.animBeginArr.push(this.animDur)),D.animatePaths(M,{centerX:e.centerX,centerY:e.centerY,endAngle:T,startAngle:k,prevEndAngle:A,prevStartAngle:S,size:e.size,i:y,totalItems:2,animBeginArr:this.animBeginArr,dur:N,shouldSetPrevPaths:!0,easing:t.globals.easing})}return{g:a,elHollow:u,dataLabels:p}}},{key:"drawHollow",value:function(e){var t=new b(this.ctx).drawCircle(2*e.size);return t.attr({class:"apexcharts-radialbar-hollow",cx:e.centerX,cy:e.centerY,r:e.size,fill:e.fill}),t}},{key:"drawHollowImage",value:function(e,t,n,i){var r=this.w,a=new O(this.ctx),o=(Math.random()+1).toString(36).substring(4),s=r.config.plotOptions.radialBar.hollow.image;if(r.config.plotOptions.radialBar.hollow.imageClipped)a.clippedImgArea({width:n,height:n,image:s,patternID:"pattern".concat(r.globals.cuid).concat(o)}),i="url(#pattern".concat(r.globals.cuid).concat(o,")");else{var l=r.config.plotOptions.radialBar.hollow.imageWidth,c=r.config.plotOptions.radialBar.hollow.imageHeight;if(void 0===l&&void 0===c){var u=r.globals.dom.Paper.image(s).loaded(function(t){this.move(e.centerX-t.width/2+r.config.plotOptions.radialBar.hollow.imageOffsetX,e.centerY-t.height/2+r.config.plotOptions.radialBar.hollow.imageOffsetY)});t.add(u)}else{var h=r.globals.dom.Paper.image(s).loaded(function(t){this.move(e.centerX-l/2+r.config.plotOptions.radialBar.hollow.imageOffsetX,e.centerY-c/2+r.config.plotOptions.radialBar.hollow.imageOffsetY),this.size(l,c)});t.add(h)}}return i}},{key:"getStrokeWidth",value:function(e){var t=this.w;return e.size*(100-parseInt(t.config.plotOptions.radialBar.hollow.size))/100/(e.series.length+1)-this.margin}}]),t}(),H=function(e){function t(){return o(this,t),p(this,d(t).apply(this,arguments))}return h(t,D),l(t,[{key:"draw",value:function(e,t){var n=this.w,i=new b(this.ctx),r=new O(this.ctx);this.rangeBarOptions=this.w.config.plotOptions.rangeBar,this.series=e,this.seriesRangeStart=n.globals.seriesRangeStart,this.seriesRangeEnd=n.globals.seriesRangeEnd,this.initVariables(e);for(var a=i.group({class:"apexcharts-rangebar-series apexcharts-plot-series"}),o=0,s=0;o<e.length;o++,s++){var l,c,u,h,d=void 0,f=void 0,p=void 0,g=void 0,v=[],y=[],x=n.globals.comboCharts?t[o]:o,w=i.group({class:"apexcharts-series ".concat(m.escapeString(n.globals.seriesNames[x])),rel:o+1,"data:realIndex":x});e[o].length>0&&(this.visibleI=this.visibleI+1);var k=0,S=0,E=0;this.yRatio.length>1&&(this.yaxisIndex=x);var C=this.initialPositions();g=C.y,c=C.yDivision,S=C.barHeight,h=C.zeroW,p=C.x,E=C.barWidth,l=C.xDivision,u=C.zeroH,y.push(p+E/2);for(var T=i.group({class:"apexcharts-datalabels"}),A=0,_=n.globals.dataPoints;A<n.globals.dataPoints;A++,_--){void 0===this.series[o][A]||null===e[o][A]?this.isNullValue=!0:this.isNullValue=!1,n.config.stroke.show&&(k=this.isNullValue?0:Array.isArray(this.strokeWidth)?this.strokeWidth[x]:this.strokeWidth);var P=null;this.isHorizontal?E=(P=this.drawRangeBarPaths({indexes:{i:o,j:A,realIndex:x,bc:s},barHeight:S,strokeWidth:k,pathTo:d,pathFrom:f,zeroW:h,x:p,y:g,yDivision:c,elSeries:w})).barWidth:S=(P=this.drawRangeColumnPaths({indexes:{i:o,j:A,realIndex:x,bc:s},x:p,y:g,xDivision:l,pathTo:d,pathFrom:f,barWidth:E,zeroH:u,strokeWidth:k,elSeries:w})).barHeight,d=P.pathTo,f=P.pathFrom,g=P.y,p=P.x,A>0&&y.push(p+E/2),v.push(g);var M=r.fillPath({seriesNumber:x}),I=n.globals.stroke.colors[x];w=this.renderSeries({realIndex:x,pathFill:M,lineFill:I,j:A,i:o,pathFrom:f,pathTo:d,strokeWidth:k,elSeries:w,x:p,y:g,series:e,barHeight:S,barWidth:E,elDataLabelsWrap:T,visibleSeries:this.visibleI,type:"rangebar"})}n.globals.seriesXvalues[x]=y,n.globals.seriesYvalues[x]=v,a.add(w)}return a}},{key:"drawRangeColumnPaths",value:function(e){var t=e.indexes,n=e.x,i=(e.y,e.strokeWidth),r=e.xDivision,a=e.pathTo,o=e.pathFrom,s=e.barWidth,l=e.zeroH,c=this.w,u=new b(this.ctx),h=t.i,d=t.j,f=this.yRatio[this.yaxisIndex],p=t.realIndex,g=this.getRangeValue(p,d),m=Math.min(g.start,g.end),v=Math.max(g.start,g.end);c.globals.isXNumeric&&(n=(c.globals.seriesX[h][d]-c.globals.minX)/this.xRatio-s/2);var y=n+s*this.visibleI;void 0===this.series[h][d]||null===this.series[h][d]?m=l:(m=l-m/f,v=l-v/f);var x=Math.abs(v-m);return u.move(y,l),o=u.move(y,m),c.globals.previousPaths.length>0&&(o=this.getPathFrom(p,d,!0)),a=u.move(y,v)+u.line(y+s,v)+u.line(y+s,m)+u.line(y,m)+u.line(y,v-i/2),o=o+u.move(y,m)+u.line(y+s,m)+u.line(y+s,m)+u.line(y,m),c.globals.isXNumeric||(n+=r),{pathTo:a,pathFrom:o,barHeight:x,x:n,y:v,barXPosition:y}}},{key:"drawRangeBarPaths",value:function(e){var t=e.indexes,n=(e.x,e.y),i=e.yDivision,r=e.pathTo,a=e.pathFrom,o=e.barHeight,s=e.zeroW,l=this.w,c=new b(this.ctx),u=t.i,h=t.j,d=t.realIndex,f=s,p=s;l.globals.isXNumeric&&(n=(l.globals.seriesX[u][h]-l.globals.minX)/this.invertedXRatio-o);var g=n+o*this.visibleI;void 0!==this.series[u][h]&&null!==this.series[u][h]&&(f=s+this.seriesRangeStart[u][h]/this.invertedYRatio,p=s+this.seriesRangeEnd[u][h]/this.invertedYRatio),c.move(s,g),a=c.move(s,g),l.globals.previousPaths.length>0&&(a=this.getPathFrom(d,h));var m=Math.abs(p-f);return r=c.move(f,g)+c.line(p,g)+c.line(p,g+o)+c.line(f,g+o)+c.line(f,g),a=a+c.line(f,g)+c.line(f,g+o)+c.line(f,g+o)+c.line(f,g),l.globals.isXNumeric||(n+=i),{pathTo:r,pathFrom:a,barWidth:m,x:p,y:n,barYPosition:g}}},{key:"getRangeValue",value:function(e,t){var n=this.w;return{start:n.globals.seriesRangeStart[e][t],end:n.globals.seriesRangeEnd[e][t]}}}]),t}(),W=function(){function e(t,n,i){o(this,e),this.ctx=t,this.w=t.w,this.xyRatios=n,this.pointsChart=!("bubble"!==this.w.config.chart.type&&"scatter"!==this.w.config.chart.type)||i,this.scatter=new M(this.ctx),this.noNegatives=this.w.globals.minX===Number.MAX_VALUE,this.yaxisIndex=0}return l(e,[{key:"draw",value:function(e,t,n){var i=this.w,r=new b(this.ctx),a=new O(this.ctx),o=i.globals.comboCharts?t:i.config.chart.type,s=r.group({class:"apexcharts-".concat(o,"-series apexcharts-plot-series")}),l=new C(this.ctx,i);e=l.getLogSeries(e);var c=this.xyRatios.yRatio;c=l.getLogYRatios(c);for(var h=this.xyRatios.zRatio,d=this.xyRatios.xRatio,f=this.xyRatios.baseLineY,p=[],g=[],v=0,y=0;y<e.length;y++){if("line"===o&&("gradient"===i.config.fill.type||"gradient"===i.config.fill.type[y])&&l.seriesHaveSameValues(y)){var x=e[y].slice();x[x.length-1]=x[x.length-1]+1e-6,e[y]=x}var w=i.globals.gridWidth/i.globals.dataPoints,k=i.globals.comboCharts?n[y]:y;c.length>1&&(this.yaxisIndex=k),this.isReversed=i.config.yaxis[this.yaxisIndex]&&i.config.yaxis[this.yaxisIndex].reversed;var S=[],E=[],T=i.globals.gridHeight-f[this.yaxisIndex]-(this.isReversed?i.globals.gridHeight:0)+(this.isReversed?2*f[this.yaxisIndex]:0),A=T;T>i.globals.gridHeight&&(A=i.globals.gridHeight),v=w/2;var _=i.globals.padHorizontal+v,M=1;i.globals.isXNumeric&&i.globals.seriesX.length>0&&(_=(i.globals.seriesX[k][0]-i.globals.minX)/d),E.push(_);var D=void 0,N=void 0,L=void 0,R=void 0,F=[],j=[],z=r.group({class:"apexcharts-series ".concat(m.escapeString(i.globals.seriesNames[k]))}),Y=r.group({class:"apexcharts-series-markers-wrap"}),H=r.group({class:"apexcharts-datalabels"});this.ctx.series.addCollapsedClassToSeries(z,k);var W=e[y].length===i.globals.dataPoints;z.attr({"data:longestSeries":W,rel:y+1,"data:realIndex":k}),this.appendPathFrom=!0;var X=_,V=void 0,B=X,q=T,U=0;if(q=this.determineFirstPrevY({i:y,series:e,yRatio:c[this.yaxisIndex],zeroY:T,prevY:q,prevSeriesY:g,lineYPosition:U}).prevY,S.push(q),V=q,null===e[y][0]){for(var G=0;G<e[y].length;G++)if(null!==e[y][G]){B=w*G,q=T-e[y][G]/c[this.yaxisIndex],D=r.move(B,q),N=r.move(B,A);break}}else D=r.move(B,q),N=r.move(B,A)+r.line(B,q);if(L=r.move(-1,T)+r.line(-1,T),R=r.move(-1,T)+r.line(-1,T),i.globals.previousPaths.length>0){var Q=this.checkPreviousPaths({pathFromLine:L,pathFromArea:R,realIndex:k});L=Q.pathFromLine,R=Q.pathFromArea}for(var $=i.globals.dataPoints>1?i.globals.dataPoints-1:i.globals.dataPoints,Z=0;Z<$;Z++){if(i.globals.isXNumeric){var K=i.globals.seriesX[k][Z+1];void 0===i.globals.seriesX[k][Z+1]&&(K=i.globals.seriesX[k][$-1]),_=(K-i.globals.minX)/d}else _+=w;var J=m.isNumber(i.globals.minYArr[k])?i.globals.minYArr[k]:i.globals.minY;i.config.chart.stacked?(U=y>0&&i.globals.collapsedSeries.length<i.config.series.length-1?g[y-1][Z+1]:T,M=void 0===e[y][Z+1]||null===e[y][Z+1]?U-J/c[this.yaxisIndex]+2*(this.isReversed?J/c[this.yaxisIndex]:0):U-e[y][Z+1]/c[this.yaxisIndex]+2*(this.isReversed?e[y][Z+1]/c[this.yaxisIndex]:0)):M=void 0===e[y][Z+1]||null===e[y][Z+1]?T-J/c[this.yaxisIndex]+2*(this.isReversed?J/c[this.yaxisIndex]:0):T-e[y][Z+1]/c[this.yaxisIndex]+2*(this.isReversed?e[y][Z+1]/c[this.yaxisIndex]:0),E.push(_),S.push(M);var ee=this.createPaths({series:e,i:y,j:Z,x:_,y:M,xDivision:w,pX:X,pY:V,areaBottomY:A,linePath:D,areaPath:N,linePaths:F,areaPaths:j,seriesIndex:n});j=ee.areaPaths,F=ee.linePaths,X=ee.pX,V=ee.pY,N=ee.areaPath,D=ee.linePath,this.appendPathFrom&&(L+=r.line(_,T),R+=r.line(_,T));var te=this.calculatePoints({series:e,x:_,y:M,realIndex:k,i:y,j:Z,prevY:q,categoryAxisCorrection:v,xRatio:d});if(this.pointsChart)this.scatter.draw(z,Z,{realIndex:k,pointsPos:te,zRatio:h,elParent:Y});else{var ne=new P(this.ctx);i.globals.dataPoints>1&&Y.node.classList.add("hidden");var ie=ne.plotChartMarkers(te,k,Z+1);null!==ie&&Y.add(ie)}var re=!e[y][Z+1]||e[y][Z+1]>e[y][Z]?"top":"bottom",ae=new I(this.ctx).drawDataLabel(te,k,Z+1,null,re);null!==ae&&H.add(ae)}g.push(S),i.globals.seriesXvalues[k]=E,i.globals.seriesYvalues[k]=S,this.pointsChart||i.globals.delayedElements.push({el:Y.node,index:k});var oe={i:y,realIndex:k,animationDelay:y,initialSpeed:i.config.chart.animations.speed,dataChangeSpeed:i.config.chart.animations.dynamicAnimation.speed,className:"apexcharts-".concat(o),id:"apexcharts-".concat(o)};if("area"===o)for(var se=a.fillPath({seriesNumber:k}),le=0;le<j.length;le++){var ce=r.renderPaths(u({},oe,{pathFrom:R,pathTo:j[le],stroke:"none",strokeWidth:0,strokeLineCap:null,fill:se}));z.add(ce)}if(i.config.stroke.show&&!this.pointsChart){var ue=null;ue="line"===o?a.fillPath({seriesNumber:k,i:y}):i.globals.stroke.colors[k];for(var he=0;he<F.length;he++){var de=r.renderPaths(u({},oe,{pathFrom:L,pathTo:F[he],stroke:ue,strokeWidth:Array.isArray(i.config.stroke.width)?i.config.stroke.width[k]:i.config.stroke.width,strokeLineCap:i.config.stroke.lineCap,fill:"none"}));z.add(de)}}z.add(Y),z.add(H),p.push(z)}for(var fe=p.length;fe>0;fe--)s.add(p[fe-1]);return s}},{key:"createPaths",value:function(e){var t=e.series,n=e.i,i=e.j,r=e.x,a=e.y,o=e.pX,s=e.pY,l=e.xDivision,c=e.areaBottomY,u=e.linePath,h=e.areaPath,d=e.linePaths,f=e.areaPaths,p=e.seriesIndex,g=this.w,m=new b(this.ctx),v=g.config.stroke.curve;if(Array.isArray(g.config.stroke.curve)&&(v=Array.isArray(p)?g.config.stroke.curve[p[n]]:g.config.stroke.curve[n]),"smooth"===v){var y=.35*(r-o);g.globals.hasNullValues?(null!==t[n][i]&&(null!==t[n][i+1]?(u=m.move(o,s)+m.curve(o+y,s,r-y,a,r+1,a),h=m.move(o+1,s)+m.curve(o+y,s,r-y,a,r+1,a)+m.line(r,c)+m.line(o,c)+"z"):(u=m.move(o,s),h=m.move(o,s)+"z")),d.push(u),f.push(h)):(u+=m.curve(o+y,s,r-y,a,r,a),h+=m.curve(o+y,s,r-y,a,r,a)),o=r,s=a,i===t[n].length-2&&(h=h+m.curve(o,s,r,a,r,c)+m.move(r,a)+"z",g.globals.hasNullValues||(d.push(u),f.push(h)))}else null===t[n][i+1]&&(u+=m.move(r,a),h=h+m.line(r-l,c)+m.move(r,a)),null===t[n][i]&&(u+=m.move(r,a),h+=m.move(r,c)),"stepline"===v?(u=u+m.line(r,null,"H")+m.line(null,a,"V"),h=h+m.line(r,null,"H")+m.line(null,a,"V")):"straight"===v&&(u+=m.line(r,a),h+=m.line(r,a)),i===t[n].length-2&&(h=h+m.line(r,c)+m.move(r,a)+"z",d.push(u),f.push(h));return{linePaths:d,areaPaths:f,pX:o,pY:s,linePath:u,areaPath:h}}},{key:"calculatePoints",value:function(e){var t=e.series,n=e.realIndex,i=e.x,r=e.y,a=e.i,o=e.j,s=e.prevY,l=e.categoryAxisCorrection,c=e.xRatio,u=this.w,h=[],d=[];if(0===o){var f=l+u.config.markers.offsetX;u.globals.isXNumeric&&(f=(u.globals.seriesX[n][0]-u.globals.minX)/c+u.config.markers.offsetX),h.push(f),d.push(m.isNumber(t[a][0])?s+u.config.markers.offsetY:null),h.push(i+u.config.markers.offsetX),d.push(m.isNumber(t[a][o+1])?r+u.config.markers.offsetY:null)}else h.push(i+u.config.markers.offsetX),d.push(m.isNumber(t[a][o+1])?r+u.config.markers.offsetY:null);return{x:h,y:d}}},{key:"checkPreviousPaths",value:function(e){for(var t=e.pathFromLine,n=e.pathFromArea,i=e.realIndex,r=this.w,a=0;a<r.globals.previousPaths.length;a++){var o=r.globals.previousPaths[a];("line"===o.type||"area"===o.type)&&o.paths.length>0&&parseInt(o.realIndex)===parseInt(i)&&("line"===o.type?(this.appendPathFrom=!1,t=r.globals.previousPaths[a].paths[0].d):"area"===o.type&&(this.appendPathFrom=!1,n=r.globals.previousPaths[a].paths[0].d,r.config.stroke.show&&(t=r.globals.previousPaths[a].paths[1].d)))}return{pathFromLine:t,pathFromArea:n}}},{key:"determineFirstPrevY",value:function(e){var t=e.i,n=e.series,i=e.yRatio,r=e.zeroY,a=e.prevY,o=e.prevSeriesY,s=e.lineYPosition,l=this.w;if(void 0!==n[t][0])a=l.config.chart.stacked?(s=t>0?o[t-1][0]:r)-n[t][0]/i+2*(this.isReversed?n[t][0]/i:0):r-n[t][0]/i+2*(this.isReversed?n[t][0]/i:0);else if(l.config.chart.stacked&&t>0&&void 0===n[t][0])for(var c=t-1;c>=0;c--)if(null!==n[c][0]&&void 0!==n[c][0]){a=s=o[c][0];break}return{prevY:a,lineYPosition:s}}}]),e}(),X=function(){function e(t){o(this,e),this.ctx=t,this.w=t.w,this.tooltipKeyFormat="dd MMM"}return l(e,[{key:"xLabelFormat",value:function(e,t){var n=this.w;if("datetime"===n.config.xaxis.type&&void 0===n.config.tooltip.x.formatter)return new S(this.ctx).formatDate(new Date(t),n.config.tooltip.x.format,!0,!0);return e(t)}},{key:"setLabelFormatters",value:function(){var e=this.w;return e.globals.xLabelFormatter=function(e){return e},e.globals.xaxisTooltipFormatter=function(e){return e},e.globals.ttKeyFormatter=function(e){return e},e.globals.ttZFormatter=function(e){return e},e.globals.legendFormatter=function(e){return e},"function"==typeof e.config.tooltip.x.formatter&&(e.globals.ttKeyFormatter=e.config.tooltip.x.formatter),"function"==typeof e.config.xaxis.tooltip.formatter&&(e.globals.xaxisTooltipFormatter=e.config.xaxis.tooltip.formatter),Array.isArray(e.config.tooltip.y)?e.globals.ttVal=e.config.tooltip.y:void 0!==e.config.tooltip.y.formatter&&(e.globals.ttVal=e.config.tooltip.y),void 0!==e.config.tooltip.z.formatter&&(e.globals.ttZFormatter=e.config.tooltip.z.formatter),void 0!==e.config.legend.formatter&&(e.globals.legendFormatter=e.config.legend.formatter),void 0!==e.config.xaxis.labels.formatter?e.globals.xLabelFormatter=e.config.xaxis.labels.formatter:e.globals.xLabelFormatter=function(t){return m.isNumber(t)?"numeric"===e.config.xaxis.type&&e.globals.dataPoints<50?t.toFixed(1):t.toFixed(0):t},e.config.yaxis.forEach(function(t,n){void 0!==t.labels.formatter?e.globals.yLabelFormatters[n]=t.labels.formatter:e.globals.yLabelFormatters[n]=function(n){return m.isNumber(n)?0!==e.globals.yValueDecimal||e.globals.maxY-e.globals.minY<4?n.toFixed(void 0!==t.decimalsInFloat?t.decimalsInFloat:e.globals.yValueDecimal):n.toFixed(0):n}}),e.globals}},{key:"heatmapLabelFormatters",value:function(){var e=this.w;if("heatmap"===e.config.chart.type){e.globals.yAxisScale[0].result=e.globals.seriesNames.slice();var t=e.globals.seriesNames.reduce(function(e,t){return e.length>t.length?e:t},0);e.globals.yAxisScale[0].niceMax=t,e.globals.yAxisScale[0].niceMin=t}}}]),e}(),V=function(){function e(t){o(this,e),this.ctx=t,this.w=t.w}return l(e,[{key:"getLabel",value:function(e,t,n,i){var r,a=arguments.length>4&&void 0!==arguments[4]?arguments[4]:[],o=this.w,s=void 0===e[i]?"":e[i],l=o.globals.xLabelFormatter,c=o.config.xaxis.labels.formatter;return r=new X(this.ctx).xLabelFormat(l,s),void 0!==c&&(r=c(s,e[i],i)),t.length>0?(n=t[i].position,r=t[i].value):"datetime"===o.config.xaxis.type&&void 0===c&&(r=""),void 0===r&&(r=""),(0===(r=r.toString()).indexOf("NaN")||0===r.toLowerCase().indexOf("invalid")||r.toLowerCase().indexOf("infinity")>=0||a.indexOf(r)>=0&&!o.config.xaxis.labels.showDuplicates)&&(r=""),{x:n,text:r}}},{key:"drawYAxisTicks",value:function(e,t,n,i,r,a,o){var s=this.w,l=new b(this.ctx),c=s.globals.translateY;if(i.show){!0===s.config.yaxis[r].opposite&&(e+=i.width);for(var u=t;u>=0;u--){var h=c+t/10+s.config.yaxis[r].labels.offsetY-1;s.globals.isBarHorizontal&&(h=a*u);var d=l.drawLine(e+n.offsetX-i.width+i.offsetX,h+i.offsetY,e+n.offsetX+i.offsetX,h+i.offsetY,n.color);o.add(d),c+=a}}}}]),e}(),B=function(){function e(t){o(this,e),this.ctx=t,this.w=t.w;var n=this.w;this.xaxisLabels=n.globals.labels.slice(),n.globals.timelineLabels.length>0&&(this.xaxisLabels=n.globals.timelineLabels.slice()),this.drawnLabels=[],"top"===n.config.xaxis.position?this.offY=0:this.offY=n.globals.gridHeight+1,this.offY=this.offY+n.config.xaxis.axisBorder.offsetY,this.xaxisFontSize=n.config.xaxis.labels.style.fontSize,this.xaxisFontFamily=n.config.xaxis.labels.style.fontFamily,this.xaxisForeColors=n.config.xaxis.labels.style.colors,this.xaxisBorderWidth=n.config.xaxis.axisBorder.width,this.xaxisBorderWidth.indexOf("%")>-1?this.xaxisBorderWidth=n.globals.gridWidth*parseInt(this.xaxisBorderWidth)/100:this.xaxisBorderWidth=parseInt(this.xaxisBorderWidth),this.xaxisBorderHeight=n.config.xaxis.axisBorder.height,this.yaxis=n.config.yaxis[0],this.axesUtils=new V(t)}return l(e,[{key:"drawXaxis",value:function(){var e,t=this.w,n=new b(this.ctx),i=n.group({class:"apexcharts-xaxis",transform:"translate(".concat(t.config.xaxis.offsetX,", ").concat(t.config.xaxis.offsetY,")")}),r=n.group({class:"apexcharts-xaxis-texts-g",transform:"translate(".concat(t.globals.translateXAxisX,", ").concat(t.globals.translateXAxisY,")")});i.add(r);for(var a=t.globals.padHorizontal,o=[],s=0;s<this.xaxisLabels.length;s++)o.push(this.xaxisLabels[s]);a=t.globals.isXNumeric?a+(e=t.globals.gridWidth/(o.length-1))/2+t.config.xaxis.labels.offsetX:a+(e=t.globals.gridWidth/o.length)+t.config.xaxis.labels.offsetX;var l=o.length;if(t.config.xaxis.labels.show)for(var c=0;c<=l-1;c++){var u=a-e/2+t.config.xaxis.labels.offsetX,h=this.axesUtils.getLabel(o,t.globals.timelineLabels,u,c,this.drawnLabels);this.drawnLabels.push(h.text);var d=28;t.globals.rotateXLabels&&(d=22);var f=n.drawText({x:h.x,y:this.offY+t.config.xaxis.labels.offsetY+d,text:"",textAnchor:"middle",fontSize:this.xaxisFontSize,fontFamily:this.xaxisFontFamily,foreColor:Array.isArray(this.xaxisForeColors)?this.xaxisForeColors[c]:this.xaxisForeColors,cssClass:"apexcharts-xaxis-label "+t.config.xaxis.labels.style.cssClass});c===l-1&&t.globals.skipLastTimelinelabel&&(h.text=""),r.add(f),n.addTspan(f,h.text,this.xaxisFontFamily);var p=document.createElementNS(t.globals.SVGNS,"title");p.textContent=h.text,f.node.appendChild(p),a+=e}if(void 0!==t.config.xaxis.title.text){var g=n.group({class:"apexcharts-xaxis-title"}),m=n.drawText({x:t.globals.gridWidth/2+t.config.xaxis.title.offsetX,y:this.offY-parseInt(this.xaxisFontSize)+t.globals.xAxisLabelsHeight+t.config.xaxis.title.offsetY,text:t.config.xaxis.title.text,textAnchor:"middle",fontSize:t.config.xaxis.title.style.fontSize,fontFamily:t.config.xaxis.title.style.fontFamily,foreColor:t.config.xaxis.title.style.color,cssClass:"apexcharts-xaxis-title-text "+t.config.xaxis.title.style.cssClass});g.add(m),i.add(g)}if(t.config.xaxis.axisBorder.show){var v=0;"bar"===t.config.chart.type&&t.globals.isXNumeric&&(v-=15);var y=n.drawLine(t.globals.padHorizontal+v+t.config.xaxis.axisBorder.offsetX,this.offY,this.xaxisBorderWidth,this.offY,t.config.xaxis.axisBorder.color,0,this.xaxisBorderHeight);i.add(y)}return i}},{key:"drawXaxisInversed",value:function(e){var t,n,i=this.w,r=new b(this.ctx),a=i.config.yaxis[0].opposite?i.globals.translateYAxisX[e]:0,o=r.group({class:"apexcharts-yaxis apexcharts-xaxis-inversed",rel:e}),s=r.group({class:"apexcharts-yaxis-texts-g apexcharts-xaxis-inversed-texts-g",transform:"translate("+a+", 0)"});o.add(s);for(var l=[],c=0;c<this.xaxisLabels.length;c++)l.push(this.xaxisLabels[c]);n=-(t=i.globals.gridHeight/l.length)/2.2;var u=i.globals.yLabelFormatters[0],h=i.config.yaxis[0].labels;if(h.show)for(var d=0;d<=l.length-1;d++){var f=void 0===l[d]?"":l[d];f=u(f);var p=r.drawText({x:h.offsetX-15,y:n+t+h.offsetY,text:f,textAnchor:this.yaxis.opposite?"start":"end",foreColor:h.style.color?h.style.color:h.style.colors[d],fontSize:h.style.fontSize,fontFamily:h.style.fontFamily,cssClass:"apexcharts-yaxis-label "+h.style.cssClass});if(s.add(p),0!==i.config.yaxis[e].labels.rotate){var g=r.rotateAroundCenter(p.node);p.node.setAttribute("transform","rotate(".concat(i.config.yaxis[e].labels.rotate," ").concat(g.x," ").concat(g.y,")"))}n+=t}if(void 0!==i.config.yaxis[0].title.text){var m=r.group({class:"apexcharts-yaxis-title apexcharts-xaxis-title-inversed",transform:"translate("+a+", 0)"}),v=r.drawText({x:0,y:i.globals.gridHeight/2,text:i.config.yaxis[0].title.text,textAnchor:"middle",foreColor:i.config.yaxis[0].title.style.color,fontSize:i.config.yaxis[0].title.style.fontSize,fontFamily:i.config.yaxis[0].title.style.fontFamily,cssClass:"apexcharts-yaxis-title-text "+i.config.yaxis[0].title.style.cssClass});m.add(v),o.add(m)}if(i.config.xaxis.axisBorder.show){var y=r.drawLine(i.globals.padHorizontal+i.config.xaxis.axisBorder.offsetX,this.offY,this.xaxisBorderWidth,this.offY,this.yaxis.axisBorder.color,0,this.xaxisBorderHeight);o.add(y),this.axesUtils.drawYAxisTicks(0,l.length,i.config.yaxis[0].axisBorder,i.config.yaxis[0].axisTicks,0,t,o)}return o}},{key:"drawXaxisTicks",value:function(e,t){var n=this.w,i=e;if(!(e<0||e>n.globals.gridWidth)){var r=this.offY+n.config.xaxis.axisTicks.offsetY,a=r+n.config.xaxis.axisTicks.height;if(n.config.xaxis.axisTicks.show){var o=new b(this.ctx).drawLine(e+n.config.xaxis.axisTicks.offsetX,r+n.config.xaxis.offsetY,i+n.config.xaxis.axisTicks.offsetX,a+n.config.xaxis.offsetY,n.config.xaxis.axisTicks.color);t.add(o),o.node.classList.add("apexcharts-xaxis-tick")}}}},{key:"getXAxisTicksPositions",value:function(){var e=this.w,t=[],n=this.xaxisLabels.length,i=e.globals.padHorizontal;if(e.globals.timelineLabels.length>0)for(var r=0;r<n;r++)i=this.xaxisLabels[r].position,t.push(i);else for(var a=n,o=0;o<a;o++){var s=a;e.globals.isXNumeric&&"bar"!==e.config.chart.type&&(s-=1),i+=e.globals.gridWidth/s,t.push(i)}return t}},{key:"xAxisLabelCorrections",value:function(){var e=this.w,t=new b(this.ctx),n=e.globals.dom.baseEl.querySelector(".apexcharts-xaxis-texts-g"),i=e.globals.dom.baseEl.querySelectorAll(".apexcharts-xaxis-texts-g text"),r=e.globals.dom.baseEl.querySelectorAll(".apexcharts-yaxis-inversed text"),a=e.globals.dom.baseEl.querySelectorAll(".apexcharts-xaxis-inversed-texts-g text");if(e.globals.rotateXLabels||e.config.xaxis.labels.rotateAlways)for(var o=0;o<i.length;o++){var s=t.rotateAroundCenter(i[o]);s.y=s.y-1,s.x=s.x+1,i[o].setAttribute("transform","rotate(".concat(e.config.xaxis.labels.rotate," ").concat(s.x," ").concat(s.y,")")),i[o].setAttribute("text-anchor","end");n.setAttribute("transform","translate(0, ".concat(-10,")"));var l=i[o].childNodes;e.config.xaxis.labels.trim&&t.placeTextWithEllipsis(l[0],l[0].textContent,e.config.xaxis.labels.maxHeight-40)}else for(var c=e.globals.gridWidth/e.globals.labels.length,u=0;u<i.length;u++){var h=i[u].childNodes;e.config.xaxis.labels.trim&&"datetime"!==e.config.xaxis.type&&t.placeTextWithEllipsis(h[0],h[0].textContent,c)}if(r.length>0){var d=r[r.length-1].getBBox(),f=r[0].getBBox();d.x<-20&&r[r.length-1].parentNode.removeChild(r[r.length-1]),f.x+f.width>e.globals.gridWidth&&r[0].parentNode.removeChild(r[0]);for(var p=0;p<a.length;p++)t.placeTextWithEllipsis(a[p],a[p].textContent,e.config.yaxis[0].labels.maxWidth-2*parseInt(e.config.yaxis[0].title.style.fontSize)-20)}}}]),e}(),q=function(){function e(t){o(this,e),this.ctx=t,this.w=t.w;var n=this.w;this.xaxisFontSize=n.config.xaxis.labels.style.fontSize,this.axisFontFamily=n.config.xaxis.labels.style.fontFamily,this.xaxisForeColors=n.config.xaxis.labels.style.colors,this.xAxisoffX=0,"bottom"===n.config.xaxis.position&&(this.xAxisoffX=n.globals.gridHeight),this.drawnLabels=[],this.axesUtils=new V(t)}return l(e,[{key:"drawYaxis",value:function(e){var t=this.w,n=new b(this.ctx),i=t.config.yaxis[e].labels.style.fontSize,r=t.config.yaxis[e].labels.style.fontFamily,a=n.group({class:"apexcharts-yaxis",rel:e,transform:"translate("+t.globals.translateYAxisX[e]+", 0)"});if(!t.config.yaxis[e].show)return a;var o=n.group({class:"apexcharts-yaxis-texts-g"});a.add(o);var s=t.globals.yAxisScale[e].result.length-1,l=t.globals.gridHeight/s+.1,c=t.globals.translateY,u=t.globals.yLabelFormatters[e],h=t.globals.yAxisScale[e].result.slice();if(t.config.yaxis[e]&&t.config.yaxis[e].reversed&&h.reverse(),t.config.yaxis[e].labels.show)for(var d=s;d>=0;d--){var f=h[d];f=u(f,d);var p=t.config.yaxis[e].labels.padding;t.config.yaxis[e].opposite&&0!==t.config.yaxis.length&&(p*=-1);var g=n.drawText({x:p,y:c+s/10+t.config.yaxis[e].labels.offsetY+1,text:f,textAnchor:t.config.yaxis[e].opposite?"start":"end",fontSize:i,fontFamily:r,foreColor:t.config.yaxis[e].labels.style.color,cssClass:"apexcharts-yaxis-label "+t.config.yaxis[e].labels.style.cssClass});o.add(g);var m=n.rotateAroundCenter(g.node);0!==t.config.yaxis[e].labels.rotate&&g.node.setAttribute("transform","rotate(".concat(t.config.yaxis[e].labels.rotate," ").concat(m.x," ").concat(m.y,")")),c+=l}if(void 0!==t.config.yaxis[e].title.text){var v=n.group({class:"apexcharts-yaxis-title"}),y=0;t.config.yaxis[e].opposite&&(y=t.globals.translateYAxisX[e]);var x=n.drawText({x:y,y:t.globals.gridHeight/2+t.globals.translateY,text:t.config.yaxis[e].title.text,textAnchor:"end",foreColor:t.config.yaxis[e].title.style.color,fontSize:t.config.yaxis[e].title.style.fontSize,fontFamily:t.config.yaxis[e].title.style.fontFamily,cssClass:"apexcharts-yaxis-title-text "+t.config.yaxis[e].title.style.cssClass});v.add(x),a.add(v)}var w=t.config.yaxis[e].axisBorder;if(w.show){var k=31+w.offsetX;t.config.yaxis[e].opposite&&(k=-31-w.offsetX);var S=n.drawLine(k,t.globals.translateY+w.offsetY-2,k,t.globals.gridHeight+t.globals.translateY+w.offsetY+2,w.color);a.add(S),this.axesUtils.drawYAxisTicks(k,s,w,t.config.yaxis[e].axisTicks,e,l,a)}return a}},{key:"drawYaxisInversed",value:function(e){var t=this.w,n=new b(this.ctx),i=n.group({class:"apexcharts-xaxis apexcharts-yaxis-inversed"}),r=n.group({class:"apexcharts-xaxis-texts-g",transform:"translate(".concat(t.globals.translateXAxisX,", ").concat(t.globals.translateXAxisY,")")});i.add(r);var a=t.globals.yAxisScale[e].result.length-1,o=t.globals.gridWidth/a+.1,s=o+t.config.xaxis.labels.offsetX,l=t.globals.xLabelFormatter,c=t.globals.yAxisScale[e].result.slice(),u=t.globals.invertedTimelineLabels;u.length>0&&(this.xaxisLabels=u.slice(),a=(c=u.slice()).length),t.config.yaxis[e]&&t.config.yaxis[e].reversed&&c.reverse();var h=u.length;if(t.config.xaxis.labels.show)for(var d=h?0:a;h?d<h-1:d>=0;h?d++:d--){var f=c[d];f=l(f,d);var p=t.globals.gridWidth+t.globals.padHorizontal-(s-o+t.config.xaxis.labels.offsetX);if(u.length){var g=this.axesUtils.getLabel(c,u,p,d,this.drawnLabels);p=g.x,f=g.text,this.drawnLabels.push(g.text)}var m=n.drawText({x:p,y:this.xAxisoffX+t.config.xaxis.labels.offsetY+30,text:"",textAnchor:"middle",foreColor:Array.isArray(this.xaxisForeColors)?this.xaxisForeColors[e]:this.xaxisForeColors,fontSize:this.xaxisFontSize,fontFamily:this.xaxisFontFamily,cssClass:"apexcharts-xaxis-label "+t.config.xaxis.labels.style.cssClass});r.add(m),m.tspan(f);var v=document.createElementNS(t.globals.SVGNS,"title");v.textContent=f,m.node.appendChild(v),s+=o}if(void 0!==t.config.xaxis.title.text){var y=n.group({class:"apexcharts-xaxis-title apexcharts-yaxis-title-inversed"}),x=n.drawText({x:t.globals.gridWidth/2,y:this.xAxisoffX+parseInt(this.xaxisFontSize)+parseInt(t.config.xaxis.title.style.fontSize)+20,text:t.config.xaxis.title.text,textAnchor:"middle",fontSize:t.config.xaxis.title.style.fontSize,fontFamily:t.config.xaxis.title.style.fontFamily,cssClass:"apexcharts-xaxis-title-text "+t.config.xaxis.title.style.cssClass});y.add(x),i.add(y)}var w=t.config.yaxis[e].axisBorder;if(w.show){var k=n.drawLine(t.globals.padHorizontal+w.offsetX,1+w.offsetY,t.globals.padHorizontal+w.offsetX,t.globals.gridHeight+w.offsetY,w.color);i.add(k)}return i}},{key:"yAxisTitleRotate",value:function(e,t){var n=this.w,i=new b(this.ctx),r={width:0,height:0},a={width:0,height:0},o=n.globals.dom.baseEl.querySelector(" .apexcharts-yaxis[rel='".concat(e,"'] .apexcharts-yaxis-texts-g"));null!==o&&(r=o.getBoundingClientRect());var s=n.globals.dom.baseEl.querySelector(".apexcharts-yaxis[rel='".concat(e,"'] .apexcharts-yaxis-title text"));if(null!==s&&(a=s.getBoundingClientRect()),null!==s){var l=this.xPaddingForYAxisTitle(e,r,a,t);s.setAttribute("x",l.xPos-(t?10:0))}if(null!==s){var c=i.rotateAroundCenter(s);t?s.setAttribute("transform","rotate(".concat(n.config.yaxis[e].title.rotate," ").concat(c.x," ").concat(c.y,")")):s.setAttribute("transform","rotate(-".concat(n.config.yaxis[e].title.rotate," ").concat(c.x," ").concat(c.y,")"))}}},{key:"xPaddingForYAxisTitle",value:function(e,t,n,i){var r=this.w,a=0,o=0,s=10;return void 0===r.config.yaxis[e].title.text||e<0?{xPos:o,padd:0}:(i?(o=t.width+r.config.yaxis[e].title.offsetX+n.width/2+s/2,0===(a+=1)&&(o-=s/2)):(o=-1*t.width+r.config.yaxis[e].title.offsetX+s/2+n.width/2,r.globals.isBarHorizontal&&(s=25,o=-1*t.width-r.config.yaxis[e].title.offsetX-s)),{xPos:o,padd:s})}},{key:"setYAxisXPosition",value:function(e,t){var n=this.w,i=0,r=0,a=21,o=1;n.config.yaxis.length>1&&(this.multipleYs=!0),n.config.yaxis.map(function(s,l){var c=n.globals.ignoreYAxisIndexes.indexOf(l)>-1||!s.show||s.floating||0===e[l].width,u=e[l].width+t[l].width;s.opposite?n.globals.isBarHorizontal?(r=n.globals.gridWidth+n.globals.translateX-1,n.globals.translateYAxisX[l]=r-s.labels.offsetX):(r=n.globals.gridWidth+n.globals.translateX+o,c||(o=o+u+20),n.globals.translateYAxisX[l]=r-s.labels.offsetX+20):(i=n.globals.translateX-a,c||(a=a+u+20),n.globals.translateYAxisX[l]=i+s.labels.offsetX)})}},{key:"setYAxisTextAlignments",value:function(){var e=this.w,t=e.globals.dom.baseEl.querySelectorAll(".apexcharts-yaxis");(t=m.listToArray(t)).forEach(function(t,n){var i=e.config.yaxis[n];if(void 0!==i.labels.align){var r=e.globals.dom.baseEl.querySelector(".apexcharts-yaxis[rel='".concat(n,"'] .apexcharts-yaxis-texts-g")),a=e.globals.dom.baseEl.querySelectorAll(".apexcharts-yaxis[rel='".concat(n,"'] .apexcharts-yaxis-label"));a=m.listToArray(a);var o=r.getBoundingClientRect();"left"===i.labels.align?(a.forEach(function(e,t){e.setAttribute("text-anchor","start")}),i.opposite||r.setAttribute("transform","translate(-".concat(o.width,", 0)"))):"center"===i.labels.align?(a.forEach(function(e,t){e.setAttribute("text-anchor","middle")}),r.setAttribute("transform","translate(".concat(o.width/2*(i.opposite?1:-1),", 0)"))):"right"===i.labels.align&&(a.forEach(function(e,t){e.setAttribute("text-anchor","end")}),i.opposite&&r.setAttribute("transform","translate(".concat(o.width,", 0)")))}})}}]),e}(),U=function(){function e(t){o(this,e),this.ctx=t,this.w=t.w}return l(e,[{key:"niceScale",value:function(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:0,i=arguments.length>3&&void 0!==arguments[3]?arguments[3]:10,r=this.w,a=void 0===this.w.config.yaxis[n].max&&void 0===this.w.config.yaxis[n].min||this.w.config.yaxis[n].forceNiceScale;if(e===Number.MIN_VALUE&&0===t||!m.isNumber(e)&&!m.isNumber(t)||e===Number.MIN_VALUE&&t===-Number.MAX_VALUE)return e=0,t=i,this.linearScale(e,t,i);e>t?(console.warn("yaxis.min cannot be greater than yaxis.max"),t=e+.1):e===t&&(e=0===e?0:e-.5,t=0===t?2:t+.5);var o=[],s=t-e;s<1&&a&&("candlestick"===r.config.chart.type||"candlestick"===r.config.series[n].type||r.globals.isRangeData)&&(t*=1.01),s<1e-5&&a&&(t*=1.05);var l=i+1;l<2?l=2:l>2&&(l-=2);for(var c=s/l,u=Math.floor(m.log10(c)),h=Math.pow(10,u),d=parseInt(c/h)*h,f=d*Math.floor(e/d),p=d*Math.ceil(t/d),g=f;o.push(g),!((g+=d)>p););if(a)return{result:o,niceMin:o[0],niceMax:o[o.length-1]};var v=e;(o=[]).push(v);for(var y=Math.abs(t-e)/i,b=0;b<=i-1;b++)v+=y,o.push(v);return{result:o,niceMin:o[0],niceMax:o[o.length-1]}}},{key:"linearScale",value:function(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:10,i=Math.abs(t-e)/n;n===Number.MAX_VALUE&&(n=10,i=1);for(var r=[],a=e;n>=0;)r.push(a),a+=i,n-=1;return{result:r,niceMin:r[0],niceMax:r[r.length-1]}}},{key:"logarithmicScale",value:function(e,t,n,i){(t<0||t===Number.MIN_VALUE)&&(t=.01);for(var r=Math.log(t)/Math.log(10),a=Math.log(n)/Math.log(10),o=Math.abs(n-t)/i,s=[],l=t;i>=0;)s.push(l),l+=o,i-=1;var c=s.map(function(e,i){e<=0&&(e=.01);var o=(a-r)/(n-t),s=Math.pow(10,r+o*(e-r));return Math.round(s/m.roundToBase(s,10))*m.roundToBase(s,10)});return 0===c[0]&&(c[0]=1),{result:c,niceMin:c[0],niceMax:c[c.length-1]}}},{key:"setYScaleForIndex",value:function(e,t,n){var i=this.w.globals,r=this.w.config,a=i.isBarHorizontal?r.xaxis:r.yaxis[e];void 0===i.yAxisScale[e]&&(i.yAxisScale[e]=[]),a.logarithmic?(i.allSeriesCollapsed=!1,i.yAxisScale[e]=this.logarithmicScale(e,t,n,a.tickAmount?a.tickAmount:Math.floor(Math.log10(n)))):n!==-Number.MAX_VALUE&&m.isNumber(n)?(i.allSeriesCollapsed=!1,void 0===a.min&&void 0===a.max||a.forceNiceScale?i.yAxisScale[e]=this.niceScale(t,n,e,a.tickAmount?a.tickAmount:n<5&&n>1?n+1:5):i.yAxisScale[e]=this.linearScale(t,n,a.tickAmount)):i.yAxisScale[e]=this.linearScale(0,5,5)}},{key:"setMultipleYScales",value:function(){var e=this,t=this.w.globals,n=this.w.config,i=t.minYArr.concat([]),r=t.maxYArr.concat([]),a=[];n.yaxis.forEach(function(o,s){var l=s;n.series.forEach(function(e,n){e.name===o.seriesName&&-1===t.collapsedSeriesIndices.indexOf(n)&&(l=n,s!==n?a.push({index:n,similarIndex:s,alreadyExists:!0}):a.push({index:n}))});var c=i[l],u=r[l];e.setYScaleForIndex(s,c,u)}),this.sameScaleInMultipleAxes(i,r,a)}},{key:"sameScaleInMultipleAxes",value:function(e,t,n){var i=this,r=this.w.config,a=this.w.globals,o=[];n.forEach(function(e){e.alreadyExists&&(void 0===o[e.index]&&(o[e.index]=[]),o[e.index].push(e.index),o[e.index].push(e.similarIndex))}),o.forEach(function(e,t){o.forEach(function(n,i){var r,a;t!==i&&(r=e,a=n,r.filter(function(e){return-1!==a.indexOf(e)})).length>0&&(o[t]=o[t].concat(o[i]))})});var s=o.map(function(e){return e.filter(function(t,n){return e.indexOf(t)===n})}).map(function(e){return e.sort()});o=o.filter(function(e){return!!e});var l=s.slice(),c=l.map(function(e){return JSON.stringify(e)});l=l.filter(function(e,t){return c.indexOf(JSON.stringify(e))===t});var u=[],h=[];e.forEach(function(e,n){l.forEach(function(i,r){i.indexOf(n)>-1&&(void 0===u[r]&&(u[r]=[],h[r]=[]),u[r].push({key:n,value:e}),h[r].push({key:n,value:t[n]}))})});var d=Array.apply(null,Array(l.length)).map(Number.prototype.valueOf,Number.MIN_VALUE),f=Array.apply(null,Array(l.length)).map(Number.prototype.valueOf,-Number.MAX_VALUE);u.forEach(function(e,t){e.forEach(function(e,n){d[t]=Math.min(e.value,d[t])})}),h.forEach(function(e,t){e.forEach(function(e,n){f[t]=Math.max(e.value,f[t])})}),e.forEach(function(e,t){h.forEach(function(e,n){var o=d[n],s=f[n];e.forEach(function(n,l){e[l].key===t&&(void 0!==r.yaxis[t].min&&(o="function"==typeof r.yaxis[t].min?r.yaxis[t].min(a.minY):r.yaxis[t].min),void 0!==r.yaxis[t].max&&(s="function"==typeof r.yaxis[t].max?r.yaxis[t].max(a.maxY):r.yaxis[t].max),i.setYScaleForIndex(t,o,s))})})})}},{key:"autoScaleY",value:function(e,t){e||(e=this);var n=[];return e.w.config.series.forEach(function(e){var i,r,a=e.data.find(function(e){return e[0]>=t.xaxis.min})[1];r=i=a,e.data.forEach(function(e){e[0]<=t.xaxis.max&&e[0]>=t.xaxis.min&&(e[1]>r&&null!==e[1]&&(r=e[1]),e[1]<i&&null!==e[1]&&(i=e[1]))}),i*=.95,r*=1.05,n.push({min:i,max:r})}),n}}]),e}(),G=function(){function e(t){o(this,e),this.ctx=t,this.w=t.w,this.scales=new U(t)}return l(e,[{key:"init",value:function(){this.setYRange(),this.setXRange(),this.setZRange()}},{key:"getMinYMaxY",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:Number.MAX_VALUE,n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:-Number.MAX_VALUE,i=arguments.length>3&&void 0!==arguments[3]?arguments[3]:null,r=this.w.globals,a=-Number.MAX_VALUE,o=Number.MIN_VALUE;null===i&&(i=e+1);var s=r.series,l=s,c=s;"candlestick"===this.w.config.chart.type?(l=r.seriesCandleL,c=r.seriesCandleH):r.isRangeData&&(l=r.seriesRangeStart,c=r.seriesRangeEnd);for(var u=e;u<i;u++){r.dataPoints=Math.max(r.dataPoints,s[u].length);for(var h=0;h<r.series[u].length;h++){var d=s[u][h];null!==d&&m.isNumber(d)?(a=Math.max(a,c[u][h]),t=Math.min(t,l[u][h]),n=Math.max(n,l[u][h]),"candlestick"===this.w.config.chart.type&&(a=Math.max(a,r.seriesCandleO[u][h]),a=Math.max(a,r.seriesCandleH[u][h]),a=Math.max(a,r.seriesCandleL[u][h]),n=a=Math.max(a,r.seriesCandleC[u][h])),m.isFloat(d)&&(d=m.noExponents(d),r.yValueDecimal=Math.max(r.yValueDecimal,d.toString().split(".")[1].length)),o>l[u][h]&&l[u][h]<0&&(o=l[u][h])):r.hasNullValues=!0}}return{minY:o,maxY:a,lowestY:t,highestY:n}}},{key:"setYRange",value:function(){var e=this.w.globals,t=this.w.config;e.maxY=-Number.MAX_VALUE,e.minY=Number.MIN_VALUE;var n=Number.MAX_VALUE;if(e.isMultipleYAxis)for(var i=0;i<e.series.length;i++){var r=this.getMinYMaxY(i,n,null,i+1);e.minYArr.push(r.minY),e.maxYArr.push(r.maxY),n=r.lowestY}var a=this.getMinYMaxY(0,n,null,e.series.length);if(e.minY=a.minY,e.maxY=a.maxY,n=a.lowestY,t.chart.stacked){for(var o=[],s=[],l=0;l<e.series[e.maxValsInArrayIndex].length;l++)for(var c=0,u=0,h=0;h<e.series.length;h++)null!==e.series[h][l]&&m.isNumber(e.series[h][l])&&(e.series[h][l]>0?c=c+parseFloat(e.series[h][l])+1e-4:u+=parseFloat(e.series[h][l])),h===e.series.length-1&&(o.push(c),s.push(u));for(var d=0;d<o.length;d++)e.maxY=Math.max(e.maxY,o[d]),e.minY=Math.min(e.minY,s[d])}if(("line"===t.chart.type||"area"===t.chart.type||"candlestick"===t.chart.type)&&e.minY===Number.MIN_VALUE&&n!==-Number.MAX_VALUE&&n!==e.maxY){var f=e.maxY-n;n>=0&&n<=10&&(f=0),e.minY=n-5*f/100,e.maxY=e.maxY+5*f/100}return t.yaxis.map(function(t,n){void 0!==t.max&&("number"==typeof t.max?e.maxYArr[n]=t.max:"function"==typeof t.max&&(e.maxYArr[n]=t.max(e.maxY)),e.maxY=e.maxYArr[n]),void 0!==t.min&&("number"==typeof t.min?e.minYArr[n]=t.min:"function"==typeof t.min&&(e.minYArr[n]=t.min(e.minY)),e.minY=e.minYArr[n])}),e.isBarHorizontal&&(void 0!==t.xaxis.min&&"number"==typeof t.xaxis.min&&(e.minY=t.xaxis.min),void 0!==t.xaxis.max&&"number"==typeof t.xaxis.max&&(e.maxY=t.xaxis.max)),e.isMultipleYAxis?(this.scales.setMultipleYScales(),e.minY=n,e.yAxisScale.forEach(function(t,n){e.minYArr[n]=t.niceMin,e.maxYArr[n]=t.niceMax})):(this.scales.setYScaleForIndex(0,e.minY,e.maxY),e.minY=e.yAxisScale[0].niceMin,e.maxY=e.yAxisScale[0].niceMax,e.minYArr[0]=e.yAxisScale[0].niceMin,e.maxYArr[0]=e.yAxisScale[0].niceMax),{minY:e.minY,maxY:e.maxY,minYArr:e.minYArr,maxYArr:e.maxYArr}}},{key:"setXRange",value:function(){var e,t=this.w.globals,n=this.w.config,i="numeric"===n.xaxis.type||"datetime"===n.xaxis.type||"category"===n.xaxis.type&&!t.noLabelsProvided||t.noLabelsProvided||t.isXNumeric;if(t.isXNumeric)for(var r=0;r<t.series.length;r++)if(t.labels[r])for(var a=0;a<t.labels[r].length;a++)null!==t.labels[r][a]&&m.isNumber(t.labels[r][a])&&(t.maxX=Math.max(t.maxX,t.labels[r][a]),t.initialmaxX=Math.max(t.maxX,t.labels[r][a]),t.minX=Math.min(t.minX,t.labels[r][a]),t.initialminX=Math.min(t.minX,t.labels[r][a]));if(t.noLabelsProvided&&0===n.xaxis.categories.length&&(t.maxX=t.labels[t.labels.length-1],t.initialmaxX=t.labels[t.labels.length-1],t.minX=1,t.initialminX=1),(t.comboChartsHasBars||"candlestick"===n.chart.type||"bar"===n.chart.type&&"category"!==n.xaxis.type)&&"category"!==n.xaxis.type){var o=t.minX-t.svgWidth/t.dataPoints*(Math.abs(t.maxX-t.minX)/t.svgWidth)/2;t.minX=o,t.initialminX=o;var s=t.maxX+t.svgWidth/t.dataPoints*(Math.abs(t.maxX-t.minX)/t.svgWidth)/2;t.maxX=s,t.initialmaxX=s}!t.isXNumeric&&!t.noLabelsProvided||n.xaxis.convertedCatToNumeric&&!t.dataFormatXNumeric||(void 0===n.xaxis.tickAmount?(e=Math.round(t.svgWidth/150),"numeric"===n.xaxis.type&&t.dataPoints<20&&(e=t.dataPoints-1),e>t.dataPoints&&0!==t.dataPoints&&(e=t.dataPoints-1)):e="dataPoints"===n.xaxis.tickAmount?t.series[t.maxValsInArrayIndex].length-1:n.xaxis.tickAmount,void 0!==n.xaxis.max&&"number"==typeof n.xaxis.max&&(t.maxX=n.xaxis.max),void 0!==n.xaxis.min&&"number"==typeof n.xaxis.min&&(t.minX=n.xaxis.min),void 0!==n.xaxis.range&&(t.minX=t.maxX-n.xaxis.range),t.minX!==Number.MAX_VALUE&&t.maxX!==-Number.MAX_VALUE?t.xAxisScale=this.scales.linearScale(t.minX,t.maxX,e):(t.xAxisScale=this.scales.linearScale(1,e,e),t.noLabelsProvided&&t.labels.length>0&&(t.xAxisScale=this.scales.linearScale(1,t.labels.length,e-1),t.seriesX=t.labels.slice())),i&&(t.labels=t.xAxisScale.result.slice()));if(t.minX===t.maxX)if("datetime"===n.xaxis.type){var l=new Date(t.minX);l.setDate(l.getDate()-2),t.minX=new Date(l).getTime();var c=new Date(t.maxX);c.setDate(c.getDate()+2),t.maxX=new Date(c).getTime()}else("numeric"===n.xaxis.type||"category"===n.xaxis.type&&!t.noLabelsProvided)&&(t.minX=t.minX-2,t.maxX=t.maxX+2);return t.isXNumeric&&(t.seriesX.forEach(function(e,n){e.forEach(function(e,i){if(i>0){var r=e-t.seriesX[n][i-1];t.minXDiff=Math.min(r,t.minXDiff)}})}),this.calcMinXDiffForTinySeries()),{minX:t.minX,maxX:t.maxX}}},{key:"calcMinXDiffForTinySeries",value:function(){var e=this.w,t=e.globals.labels.length;return 1===e.globals.labels.length?e.globals.minXDiff=(e.globals.maxX-e.globals.minX)/t/3:e.globals.minXDiff===Number.MAX_VALUE&&(e.globals.timelineLabels.length>0&&(t=e.globals.timelineLabels.length),t<3&&(t=3),e.globals.minXDiff=(e.globals.maxX-e.globals.minX)/t),e.globals.minXDiff}},{key:"setZRange",value:function(){var e=this.w.globals;if(e.isDataXYZ)for(var t=0;t<e.series.length;t++)if(void 0!==e.seriesZ[t])for(var n=0;n<e.seriesZ[t].length;n++)null!==e.seriesZ[t][n]&&m.isNumber(e.seriesZ[t][n])&&(e.maxZ=Math.max(e.maxZ,e.seriesZ[t][n]),e.minZ=Math.min(e.minZ,e.seriesZ[t][n]))}}]),e}(),Q=function(){function e(t){o(this,e),this.ctx=t,this.w=t.w}return l(e,[{key:"getAllSeriesEls",value:function(){return this.w.globals.dom.baseEl.querySelectorAll(".apexcharts-series")}},{key:"getSeriesByName",value:function(e){return this.w.globals.dom.baseEl.querySelector(".apexcharts-series.".concat(m.escapeString(e)))}},{key:"addCollapsedClassToSeries",value:function(e,t){for(var n=this.w,i=0;i<n.globals.collapsedSeries.length;i++)n.globals.collapsedSeries[i].index===t&&e.node.classList.add("apexcharts-series-collapsed")}},{key:"toggleSeriesOnHover",value:function(e,t){var n=this.w,i=n.globals.dom.baseEl.querySelectorAll(".apexcharts-series");if("mousemove"===e.type){var r=parseInt(t.getAttribute("rel"))-1,a=null;a=n.globals.axisCharts||"radialBar"===n.config.chart.type?n.globals.axisCharts?n.globals.dom.baseEl.querySelector(".apexcharts-series[data\\:realIndex='".concat(r,"']")):n.globals.dom.baseEl.querySelector(".apexcharts-series[rel='".concat(r+1,"']")):n.globals.dom.baseEl.querySelector(".apexcharts-series[rel='".concat(r+1,"'] path"));for(var o=0;o<i.length;o++)i[o].classList.add("legend-mouseover-inactive");null!==a&&(n.globals.axisCharts||a.parentNode.classList.remove("legend-mouseover-inactive"),a.classList.remove("legend-mouseover-inactive"))}else if("mouseout"===e.type)for(var s=0;s<i.length;s++)i[s].classList.remove("legend-mouseover-inactive")}},{key:"highlightRangeInSeries",value:function(e,t){var n=this.w,i=n.globals.dom.baseEl.querySelectorAll(".apexcharts-heatmap-rect"),r=function(){for(var e=0;e<i.length;e++)i[e].classList.remove("legend-mouseover-inactive")};if("mousemove"===e.type){var a=parseInt(t.getAttribute("rel"))-1;r(),function(){for(var e=0;e<i.length;e++)i[e].classList.add("legend-mouseover-inactive")}(),function(e){for(var t=0;t<i.length;t++){var n=parseInt(i[t].getAttribute("val"));n>=e.from&&n<=e.to&&i[t].classList.remove("legend-mouseover-inactive")}}(n.config.plotOptions.heatmap.colorScale.ranges[a])}else"mouseout"===e.type&&r()}},{key:"getActiveSeriesIndex",value:function(){var e=this.w,t=0;if(e.globals.series.length>1)for(var n=e.globals.series.map(function(t,n){return t.length>0&&"bar"!==e.config.series[n].type&&"column"!==e.config.series[n].type?n:-1}),i=0;i<n.length;i++)if(-1!==n[i]){t=n[i];break}return t}},{key:"getActiveConfigSeriesIndex",value:function(){var e=this.w,t=0;if(e.config.series.length>1)for(var n=e.config.series.map(function(e,t){return e.data&&e.data.length>0?t:-1}),i=0;i<n.length;i++)if(-1!==n[i]){t=n[i];break}return t}},{key:"getPreviousPaths",value:function(){var e=this.w;function t(t,n,i){for(var r=t[n].childNodes,a={type:i,paths:[],realIndex:t[n].getAttribute("data:realIndex")},o=0;o<r.length;o++)if(r[o].hasAttribute("pathTo")){var s=r[o].getAttribute("pathTo");a.paths.push({d:s})}e.globals.previousPaths.push(a)}e.globals.previousPaths=[];var n=e.globals.dom.baseEl.querySelectorAll(".apexcharts-line-series .apexcharts-series");if(n.length>0)for(var i=n.length-1;i>=0;i--)t(n,i,"line");var r=e.globals.dom.baseEl.querySelectorAll(".apexcharts-area-series .apexcharts-series");if(r.length>0)for(var a=r.length-1;a>=0;a--)t(r,a,"area");var o=e.globals.dom.baseEl.querySelectorAll(".apexcharts-bar-series .apexcharts-series");if(o.length>0)for(var s=0;s<o.length;s++)t(o,s,"bar");var l=e.globals.dom.baseEl.querySelectorAll(".apexcharts-candlestick-series .apexcharts-series");if(l.length>0)for(var c=0;c<l.length;c++)t(l,c,"candlestick");var u=e.globals.dom.baseEl.querySelectorAll(".apexcharts-radar-series .apexcharts-series");if(u.length>0)for(var h=0;h<u.length;h++)t(u,h,"radar");var d=e.globals.dom.baseEl.querySelectorAll(".apexcharts-bubble-series .apexcharts-series");if(d.length>0)for(var f=0;f<d.length;f++){for(var p=e.globals.dom.baseEl.querySelectorAll(".apexcharts-bubble-series .apexcharts-series[data\\:realIndex='".concat(f,"'] circle")),g=[],m=0;m<p.length;m++)g.push({x:p[m].getAttribute("cx"),y:p[m].getAttribute("cy"),r:p[m].getAttribute("r")});e.globals.previousPaths.push(g)}var v=e.globals.dom.baseEl.querySelectorAll(".apexcharts-scatter-series .apexcharts-series");if(v.length>0)for(var y=0;y<v.length;y++){for(var b=e.globals.dom.baseEl.querySelectorAll(".apexcharts-scatter-series .apexcharts-series[data\\:realIndex='".concat(y,"'] circle")),x=[],w=0;w<b.length;w++)x.push({x:b[w].getAttribute("cx"),y:b[w].getAttribute("cy"),r:b[w].getAttribute("r")});e.globals.previousPaths.push(x)}var k=e.globals.dom.baseEl.querySelectorAll(".apexcharts-heatmap .apexcharts-series");if(k.length>0)for(var S=0;S<k.length;S++){for(var E=e.globals.dom.baseEl.querySelectorAll(".apexcharts-heatmap .apexcharts-series[data\\:realIndex='".concat(S,"'] rect")),C=[],T=0;T<E.length;T++)C.push({color:E[T].getAttribute("color")});e.globals.previousPaths.push(C)}e.globals.axisCharts||(e.globals.previousPaths=e.globals.series)}},{key:"handleNoData",value:function(){var e=this.w,t=e.config.noData,n=new b(this.ctx),i=e.globals.svgWidth/2,r=e.globals.svgHeight/2,a="middle";if(e.globals.noData=!0,e.globals.animationEnded=!0,"left"===t.align?(i=10,a="start"):"right"===t.align&&(i=e.globals.svgWidth-10,a="end"),"top"===t.verticalAlign?r=50:"bottom"===t.verticalAlign&&(r=e.globals.svgHeight-50),i+=t.offsetX,r=r+parseInt(t.style.fontSize)+2,void 0!==t.text&&""!==t.text){var o=n.drawText({x:i,y:r,text:t.text,textAnchor:a,fontSize:t.style.fontSize,fontFamily:t.style.fontFamily,foreColor:t.style.color,opacity:1,class:"apexcharts-text-nodata"});o.node.setAttribute("class","apexcharts-title-text"),e.globals.dom.Paper.add(o)}}},{key:"setNullSeriesToZeroValues",value:function(e){for(var t=this.w,n=0;n<e.length;n++)if(0===e[n].length)for(var i=0;i<e[t.globals.maxValsInArrayIndex].length;i++)e[n].push(0);return e}},{key:"hasAllSeriesEqualX",value:function(){for(var e=!0,t=this.w,n=this.filteredSeriesX(),i=0;i<n.length-1;i++)if(n[i][0]!==n[i+1][0]){e=!1;break}return t.globals.allSeriesHasEqualX=e,e}},{key:"filteredSeriesX",value:function(){var e=this.w.globals.seriesX.map(function(e,t){return e.length>0?e:[]});return e}}]),e}(),$=function(){function e(t){o(this,e),this.ctx=t,this.w=t.w,this.lgRect={},this.yAxisWidth=0,this.xAxisHeight=0,this.isSparkline=this.w.config.chart.sparkline.enabled,this.xPadRight=0,this.xPadLeft=0}return l(e,[{key:"plotCoords",value:function(){var e=this.w,t=e.globals,n=this.getLegendsRect();t.axisCharts?this.setGridCoordsForAxisCharts(n):this.setGridCoordsForNonAxisCharts(n),this.titleSubtitleOffset(),t.gridHeight=t.gridHeight-e.config.grid.padding.top-e.config.grid.padding.bottom,t.gridWidth=t.gridWidth-e.config.grid.padding.left-e.config.grid.padding.right-this.xPadRight-this.xPadLeft,t.translateX=t.translateX+e.config.grid.padding.left+this.xPadLeft,t.translateY=t.translateY+e.config.grid.padding.top}},{key:"conditionalChecksForAxisCoords",value:function(e,t){var n=this.w;this.xAxisHeight=(e.height+t.height)*n.globals.LINE_HEIGHT_RATIO+15,this.xAxisWidth=e.width,this.xAxisHeight-t.height>n.config.xaxis.labels.maxHeight&&(this.xAxisHeight=n.config.xaxis.labels.maxHeight),n.config.xaxis.labels.minHeight&&this.xAxisHeight<n.config.xaxis.labels.minHeight&&(this.xAxisHeight=n.config.xaxis.labels.minHeight),n.config.xaxis.floating&&(this.xAxisHeight=0),n.globals.isBarHorizontal?this.yAxisWidth=n.globals.yLabelsCoords[0].width+n.globals.yTitleCoords[0].width+15:this.yAxisWidth=this.getTotalYAxisWidth();var i=0,r=0;n.config.yaxis.forEach(function(e){i+=e.labels.minWidth,r+=e.labels.maxWidth}),this.yAxisWidth<i&&(this.yAxisWidth=i),this.yAxisWidth>r&&(this.yAxisWidth=r)}},{key:"setGridCoordsForAxisCharts",value:function(e){var t=this.w,n=t.globals,i=this.getyAxisLabelsCoords(),r=this.getxAxisLabelsCoords(),a=this.getyAxisTitleCoords(),o=this.getxAxisTitleCoords();t.globals.yLabelsCoords=[],t.globals.yTitleCoords=[],t.config.yaxis.map(function(e,n){t.globals.yLabelsCoords.push({width:i[n].width,index:n}),t.globals.yTitleCoords.push({width:a[n].width,index:n})}),this.conditionalChecksForAxisCoords(r,o),n.translateXAxisY=t.globals.rotateXLabels?this.xAxisHeight/8:-4,n.translateXAxisX=t.globals.rotateXLabels&&t.globals.isXNumeric&&t.config.xaxis.labels.rotate<=-45?-this.xAxisWidth/4:0,t.globals.isBarHorizontal&&(n.rotateXLabels=!1,n.translateXAxisY=parseInt(t.config.xaxis.labels.style.fontSize)/1.5*-1),n.translateXAxisY=n.translateXAxisY+t.config.xaxis.labels.offsetY,n.translateXAxisX=n.translateXAxisX+t.config.xaxis.labels.offsetX;var s=this.yAxisWidth,l=this.xAxisHeight;n.xAxisLabelsHeight=this.xAxisHeight,n.xAxisHeight=this.xAxisHeight;var c=10;switch(t.config.grid.show&&"radar"!==t.config.chart.type||(s=0,l=35),this.isSparkline&&(e={height:0,width:0},l=0,s=0,c=0),this.additionalPaddingXLabels(r),t.config.legend.position){case"bottom":n.translateY=c,n.translateX=s,n.gridHeight=n.svgHeight-e.height-l-(this.isSparkline?0:t.globals.rotateXLabels?10:15),n.gridWidth=n.svgWidth-s;break;case"top":n.translateY=e.height+c,n.translateX=s,n.gridHeight=n.svgHeight-e.height-l-(this.isSparkline?0:t.globals.rotateXLabels?10:15),n.gridWidth=n.svgWidth-s;break;case"left":n.translateY=c,n.translateX=e.width+s,n.gridHeight=n.svgHeight-l-12,n.gridWidth=n.svgWidth-e.width-s;break;case"right":n.translateY=c,n.translateX=s,n.gridHeight=n.svgHeight-l-12,n.gridWidth=n.svgWidth-e.width-s-5;break;default:throw new Error("Legend position not supported")}this.setGridXPosForDualYAxis(a,i),new q(this.ctx).setYAxisXPosition(i,a)}},{key:"setGridCoordsForNonAxisCharts",value:function(e){var t=this.w,n=t.globals,i=0;t.config.legend.show&&!t.config.legend.floating&&(i=20);var r=10,a=0;if("pie"===t.config.chart.type||"donut"===t.config.chart.type?(r+=t.config.plotOptions.pie.offsetY,a+=t.config.plotOptions.pie.offsetX):"radialBar"===t.config.chart.type&&(r+=t.config.plotOptions.radialBar.offsetY,a+=t.config.plotOptions.radialBar.offsetX),!t.config.legend.show)return n.gridHeight=n.svgHeight-35,n.gridWidth=n.gridHeight,n.translateY=r-10,void(n.translateX=a+(n.svgWidth-n.gridWidth)/2);switch(t.config.legend.position){case"bottom":n.gridHeight=n.svgHeight-e.height-35,n.gridWidth=n.gridHeight,n.translateY=r-20,n.translateX=a+(n.svgWidth-n.gridWidth)/2;break;case"top":n.gridHeight=n.svgHeight-e.height-35,n.gridWidth=n.gridHeight,n.translateY=e.height+r,n.translateX=a+(n.svgWidth-n.gridWidth)/2;break;case"left":n.gridWidth=n.svgWidth-e.width-i,n.gridHeight=n.gridWidth,n.translateY=r,n.translateX=a+e.width+i;break;case"right":n.gridWidth=n.svgWidth-e.width-i-5,n.gridHeight=n.gridWidth,n.translateY=r,n.translateX=a+10;break;default:throw new Error("Legend position not supported")}}},{key:"setGridXPosForDualYAxis",value:function(e,t){var n=this.w;n.config.yaxis.map(function(i,r){-1===n.globals.ignoreYAxisIndexes.indexOf(r)&&!n.config.yaxis[r].floating&&n.config.yaxis[r].show&&i.opposite&&(n.globals.translateX=n.globals.translateX-(t[r].width+e[r].width)-parseInt(n.config.yaxis[r].labels.style.fontSize)/1.2-12)})}},{key:"additionalPaddingXLabels",value:function(e){var t=this,n=this.w;if("category"===n.config.xaxis.type&&n.globals.isBarHorizontal||"numeric"===n.config.xaxis.type||"datetime"===n.config.xaxis.type){var i=n.globals.isXNumeric;n.config.yaxis.forEach(function(r,a){var o;(!r.show||r.floating||-1!==n.globals.collapsedSeriesIndices.indexOf(a)||i||r.opposite&&n.globals.isBarHorizontal)&&((i&&n.globals.isMultipleYAxis&&-1!==n.globals.collapsedSeriesIndices.indexOf(a)||n.globals.isBarHorizontal&&r.opposite)&&(o=e,n.config.grid.padding.left<o.width&&(t.xPadLeft=o.width/2+1)),(!n.globals.isBarHorizontal&&r.opposite&&-1!==n.globals.collapsedSeriesIndices.indexOf(a)||i&&!n.globals.isMultipleYAxis)&&function(e){t.timescaleLabels?t.timescaleLabels[t.timescaleLabels.length-1].position+e.width>n.globals.gridWidth?n.globals.skipLastTimelinelabel=!0:n.globals.skipLastTimelinelabel=!1:"datetime"!==n.config.xaxis.type&&n.config.grid.padding.right<e.width&&(t.xPadRight=e.width/2+1)}(e))})}}},{key:"titleSubtitleOffset",value:function(){var e=this.w,t=e.globals,n=this.isSparkline||!e.globals.axisCharts?0:10;void 0!==e.config.title.text?n+=e.config.title.margin:n+=this.isSparkline||!e.globals.axisCharts?0:5,void 0!==e.config.subtitle.text?n+=e.config.subtitle.margin:n+=this.isSparkline||!e.globals.axisCharts?0:5,e.config.legend.show&&"bottom"===e.config.legend.position&&!e.config.legend.floating&&e.config.series.length>1&&(n+=10);var i=this.getTitleSubtitleCoords("title"),r=this.getTitleSubtitleCoords("subtitle");t.gridHeight=t.gridHeight-i.height-r.height-n,t.translateY=t.translateY+i.height+r.height+n}},{key:"getTotalYAxisWidth",value:function(){var e=this.w,t=0,n=10,i=function(t){return e.globals.ignoreYAxisIndexes.indexOf(t)>-1};return e.globals.yLabelsCoords.map(function(r,a){var o=e.config.yaxis[a].floating;r.width>0&&!o?(t=t+r.width+n,i(a)&&(t=t-r.width-n)):t+=o||!e.config.yaxis[a].show?0:5}),e.globals.yTitleCoords.map(function(r,a){var o=e.config.yaxis[a].floating;n=parseInt(e.config.yaxis[a].title.style.fontSize),r.width>0&&!o?(t=t+r.width+n,i(a)&&(t=t-r.width-n)):t+=o||!e.config.yaxis[a].show?0:5}),t}},{key:"getxAxisTimeScaleLabelsCoords",value:function(){var e,t=this.w;this.timescaleLabels=t.globals.timelineLabels.slice(),t.globals.isBarHorizontal&&"datetime"===t.config.xaxis.type&&(this.timescaleLabels=t.globals.invertedTimelineLabels.slice());var n=this.timescaleLabels.map(function(e){return e.value}),i=n.reduce(function(e,t){return void 0===e?(console.error("You have possibly supplied invalid Date format. Please supply a valid JavaScript Date"),0):e.length>t.length?e:t},0);return 1.05*(e=new b(this.ctx).getTextRects(i,t.config.xaxis.labels.style.fontSize)).width*n.length>t.globals.gridWidth&&0!==t.config.xaxis.labels.rotate&&(t.globals.overlappingXLabels=!0),e}},{key:"getxAxisLabelsCoords",value:function(){var e,t=this.w,n=t.globals.labels.slice();if(t.globals.timelineLabels.length>0){var i=this.getxAxisTimeScaleLabelsCoords();e={width:i.width,height:i.height}}else{var r="left"!==t.config.legend.position||"right"!==t.config.legend.position||t.config.legend.floating?0:this.lgRect.width,a=t.globals.xLabelFormatter,o=n.reduce(function(e,t){return e.length>t.length?e:t},0);t.globals.isBarHorizontal&&(o=t.globals.yAxisScale[0].result.reduce(function(e,t){return e.length>t.length?e:t},0)),o=new X(this.ctx).xLabelFormat(a,o);var s=new b(this.ctx),l=s.getTextRects(o,t.config.xaxis.labels.style.fontSize);(e={width:l.width,height:l.height}).width*n.length>t.globals.svgWidth-r-this.yAxisWidth&&0!==t.config.xaxis.labels.rotate?t.globals.isBarHorizontal||(t.globals.rotateXLabels=!0,l=s.getTextRects(o,t.config.xaxis.labels.style.fontSize,t.config.xaxis.labels.style.fontFamily,"rotate(".concat(t.config.xaxis.labels.rotate," 0 0)"),!1),e.height=l.height/1.66):t.globals.rotateXLabels=!1}return t.config.xaxis.labels.show||(e={width:0,height:0}),{width:e.width,height:e.height}}},{key:"getyAxisLabelsCoords",value:function(){var e=this,t=this.w,n=[],i=10;return t.config.yaxis.map(function(r,a){if(r.show&&r.labels.show&&t.globals.yAxisScale[a].result.length){var o=t.globals.yLabelFormatters[a],s=o(t.globals.yAxisScale[a].niceMax,-1);if(void 0!==s&&0!==s.length||(s=t.globals.yAxisScale[a].niceMax),t.globals.isBarHorizontal)i=0,s=o(s=t.globals.labels.slice().reduce(function(e,t){return e.length>t.length?e:t},0),-1);var l=new b(e.ctx).getTextRects(s,r.labels.style.fontSize);n.push({width:l.width+i,height:l.height})}else n.push({width:0,height:0})}),n}},{key:"getxAxisTitleCoords",value:function(){var e=this.w,t=0,n=0;if(void 0!==e.config.xaxis.title.text){var i=new b(this.ctx).getTextRects(e.config.xaxis.title.text,e.config.xaxis.title.style.fontSize);t=i.width,n=i.height}return{width:t,height:n}}},{key:"getyAxisTitleCoords",value:function(){var e=this,t=this.w,n=[];return t.config.yaxis.map(function(t,i){if(t.show&&void 0!==t.title.text){var r=new b(e.ctx).getTextRects(t.title.text,t.title.style.fontSize,t.title.style.fontFamily,"rotate(-90 0 0)",!1);n.push({width:r.width,height:r.height})}else n.push({width:0,height:0})}),n}},{key:"getTitleSubtitleCoords",value:function(e){var t=this.w,n=0,i=0,r="title"===e?t.config.title.floating:t.config.subtitle.floating,a=t.globals.dom.baseEl.querySelector(".apexcharts-".concat(e,"-text"));if(null!==a&&!r){var o=a.getBoundingClientRect();n=o.width,i=t.globals.axisCharts?o.height+5:o.height}return{width:n,height:i}}},{key:"getLegendsRect",value:function(){var e=this.w,t=e.globals.dom.baseEl.querySelector(".apexcharts-legend"),n=Object.assign({},m.getBoundingClientRect(t));return null!==t&&!e.config.legend.floating&&e.config.legend.show?this.lgRect={x:n.x,y:n.y,height:n.height,width:0===n.height?0:n.width}:this.lgRect={x:0,y:0,height:0,width:0},this.lgRect}}]),e}(),Z=function(){function e(t){o(this,e),this.ctx=t,this.w=t.w,this.timeScaleArray=[]}return l(e,[{key:"calculateTimeScaleTicks",value:function(e,t){var n=this,i=this.w;if(i.globals.allSeriesCollapsed)return i.globals.labels=[],i.globals.timelineLabels=[],[];var r=new S(this.ctx),a=(t-e)/864e5;this.determineInterval(a),i.globals.disableZoomIn=!1,i.globals.disableZoomOut=!1,a<.005?i.globals.disableZoomIn=!0:a>5e4&&(i.globals.disableZoomOut=!0);var o=r.getTimeUnitsfromTimestamp(e,t),s=i.globals.gridWidth/a,l=s/24,c=l/60,h=Math.floor(24*a),d=Math.floor(24*a*60),f=Math.floor(a),p=Math.floor(a/30),g=Math.floor(a/365),m={minMinute:o.minMinute,minHour:o.minHour,minDate:o.minDate,minMonth:o.minMonth,minYear:o.minYear},v={firstVal:m,currentMinute:m.minMinute,currentHour:m.minHour,currentMonthDate:m.minDate,currentDate:m.minDate,currentMonth:m.minMonth,currentYear:m.minYear,daysWidthOnXAxis:s,hoursWidthOnXAxis:l,minutesWidthOnXAxis:c,numberOfMinutes:d,numberOfHours:h,numberOfDays:f,numberOfMonths:p,numberOfYears:g};switch(this.tickInterval){case"years":this.generateYearScale(v);break;case"months":case"half_year":this.generateMonthScale(v);break;case"months_days":case"months_fortnight":case"days":case"week_days":this.generateDayScale(v);break;case"hours":this.generateHourScale(v);break;case"minutes":this.generateMinuteScale(v)}var y=this.timeScaleArray.map(function(e){var t={position:e.position,unit:e.unit,year:e.year,day:e.day?e.day:1,hour:e.hour?e.hour:0,month:e.month+1};return"month"===e.unit?u({},t,{value:e.value+1}):"day"===e.unit||"hour"===e.unit?u({},t,{value:e.value}):"minute"===e.unit?u({},t,{value:e.value,minute:e.value}):e});return y.filter(function(e){var t=1,r=Math.ceil(i.globals.gridWidth/120),a=e.value;void 0!==i.config.xaxis.tickAmount&&(r=i.config.xaxis.tickAmount),y.length>r&&(t=Math.floor(y.length/r));var o=!1,s=!1;switch(n.tickInterval){case"half_year":t=7,"year"===e.unit&&(o=!0);break;case"months":t=1,"year"===e.unit&&(o=!0);break;case"months_fortnight":t=15,"year"!==e.unit&&"month"!==e.unit||(o=!0),30===a&&(s=!0);break;case"months_days":t=10,"month"===e.unit&&(o=!0),30===a&&(s=!0);break;case"week_days":t=8,"month"===e.unit&&(o=!0);break;case"days":t=1,"month"===e.unit&&(o=!0);break;case"hours":"day"===e.unit&&(o=!0);break;case"minutes":a%5!=0&&(s=!0)}if("minutes"===n.tickInterval||"hours"===n.tickInterval){if(!s)return!0}else if((a%t==0||o)&&!s)return!0})}},{key:"recalcDimensionsBasedOnFormat",value:function(e,t){var n=this.w,i=this.formatDates(e),r=this.removeOverlappingTS(i);t?n.globals.invertedTimelineLabels=r.slice():n.globals.timelineLabels=r.slice(),new $(this.ctx).plotCoords()}},{key:"determineInterval",value:function(e){switch(!0){case e>1825:this.tickInterval="years";break;case e>800&&e<=1825:this.tickInterval="half_year";break;case e>180&&e<=800:this.tickInterval="months";break;case e>90&&e<=180:this.tickInterval="months_fortnight";break;case e>60&&e<=90:this.tickInterval="months_days";break;case e>30&&e<=60:this.tickInterval="week_days";break;case e>2&&e<=30:this.tickInterval="days";break;case e>.1&&e<=2:this.tickInterval="hours";break;case e<.1:this.tickInterval="minutes";break;default:this.tickInterval="days"}}},{key:"generateYearScale",value:function(e){var t=e.firstVal,n=e.currentMonth,i=e.currentYear,r=e.daysWidthOnXAxis,a=e.numberOfYears,o=t.minYear,s=0,l=new S(this.ctx);if(t.minDate>1&&t.minMonth>0){var c=l.determineRemainingDaysOfYear(t.minYear,t.minMonth,t.minDate);s=(l.determineDaysOfYear(t.minYear)-c+1)*r,o=t.minYear+1,this.timeScaleArray.push({position:s,value:o,unit:"year",year:o,month:m.monthMod(n+1)})}else 1===t.minDate&&0===t.minMonth&&this.timeScaleArray.push({position:s,value:o,unit:"year",year:i,month:m.monthMod(n+1)});for(var u=o,h=s,d=0;d<a;d++)u++,h=l.determineDaysOfYear(u-1)*r+h,this.timeScaleArray.push({position:h,value:u,unit:"year",year:u,month:1})}},{key:"generateMonthScale",value:function(e){var t=e.firstVal,n=e.currentMonthDate,i=e.currentMonth,r=e.currentYear,a=e.daysWidthOnXAxis,o=e.numberOfMonths,s=i,l=0,c=new S(this.ctx),u="month",h=0;if(t.minDate>1){l=(c.determineDaysOfMonths(i+1,t.minYear)-n+1)*a,s=m.monthMod(i+1);var d=r+h,f=m.monthMod(s),p=s;0===s&&(u="year",p=d,f=1,d+=h+=1),this.timeScaleArray.push({position:l,value:p,unit:u,year:d,month:f})}else this.timeScaleArray.push({position:l,value:s,unit:u,year:r,month:m.monthMod(i)});for(var g=s+1,v=l,y=0,b=1;y<o;y++,b++){0===(g=m.monthMod(g))?(u="year",h+=1):u="month";var x=r+Math.floor(g/12)+h;v=c.determineDaysOfMonths(g,x)*a+v;var w=0===g?x:g;this.timeScaleArray.push({position:v,value:w,unit:u,year:x,month:0===g?1:g}),g++}}},{key:"generateDayScale",value:function(e){var t=e.firstVal,n=e.currentMonth,i=e.currentYear,r=e.hoursWidthOnXAxis,a=e.numberOfDays,o=new S(this.ctx),s="day",l=(24-t.minHour)*r,c=t.minDate+1,u=c,h=function(e,t,n){return e>o.determineDaysOfMonths(t+1,n)?(d=1,s="month",u=t+=1,t):t},d=c,f=h(d,n,i);this.timeScaleArray.push({position:l,value:u,unit:s,year:i,month:m.monthMod(f),day:d});for(var p=l,g=0;g<a;g++){s="day",f=h(d+=1,f,i+Math.floor(f/12)+0);var v=i+Math.floor(f/12)+0;p=24*r+p;var y=1===d?m.monthMod(f):d;this.timeScaleArray.push({position:p,value:y,unit:s,year:v,month:m.monthMod(f),day:y})}}},{key:"generateHourScale",value:function(e){var t=e.firstVal,n=e.currentDate,i=e.currentMonth,r=e.currentYear,a=e.minutesWidthOnXAxis,o=e.numberOfHours,s=new S(this.ctx),l="hour",c=function(e,t){return e>s.determineDaysOfMonths(t+1,r)&&(g=1,t+=1),{month:t,date:g}},u=function(e,t){return e>s.determineDaysOfMonths(t+1,r)?t+=1:t},h=60-t.minMinute,d=h*a,f=t.minHour+1,p=f+1;60===h&&(d=0,p=(f=t.minHour)+1);var g=n,v=u(g,i);this.timeScaleArray.push({position:d,value:f,unit:l,day:g,hour:p,year:r,month:m.monthMod(v)});for(var y=d,b=0;b<o;b++){if(l="hour",p>=24)p=0,l="day",v=c(g+=1,v).month,v=u(g,v);var x=r+Math.floor(v/12)+0;y=0===p&&0===b?h*a:60*a+y;var w=0===p?g:p;this.timeScaleArray.push({position:y,value:w,unit:l,hour:p,day:g,year:x,month:m.monthMod(v)}),p++}}},{key:"generateMinuteScale",value:function(e){var t=e.firstVal,n=e.currentMinute,i=e.currentHour,r=e.currentDate,a=e.currentMonth,o=e.currentYear,s=e.minutesWidthOnXAxis,l=e.numberOfMinutes,c=s-(n-t.minMinute),u=t.minMinute+1,h=u+1,d=r,f=a,p=o,g=i;this.timeScaleArray.push({position:c,value:u,unit:"minute",day:d,hour:g,minute:h,year:p,month:m.monthMod(f)});for(var v=c,y=0;y<l;y++){h>=60&&(h=0,24===(g+=1)&&(g=0));var b=o+Math.floor(f/12)+0;v=s+v;var x=h;this.timeScaleArray.push({position:v,value:x,unit:"minute",hour:g,minute:h,day:d,year:b,month:m.monthMod(f)}),h++}}},{key:"createRawDateString",value:function(e,t){var n=e.year;return n+="-"+("0"+e.month.toString()).slice(-2),"day"===e.unit?n+="day"===e.unit?"-"+("0"+t).slice(-2):"-01":n+="-"+("0"+(e.day?e.day:"1")).slice(-2),"hour"===e.unit?n+="hour"===e.unit?"T"+("0"+t).slice(-2):"T00":n+="T"+("0"+(e.hour?e.hour:"0")).slice(-2),n+="minute"===e.unit?":"+("0"+t).slice(-2)+":00.000Z":":00:00.000Z"}},{key:"formatDates",value:function(e){var t=this,n=this.w;return e.map(function(e){var i=e.value.toString(),r=new S(t.ctx),a=t.createRawDateString(e,i),o=new Date(Date.parse(a));if(void 0===n.config.xaxis.labels.format){var s="dd MMM",l=n.config.xaxis.labels.datetimeFormatter;"year"===e.unit&&(s=l.year),"month"===e.unit&&(s=l.month),"day"===e.unit&&(s=l.day),"hour"===e.unit&&(s=l.hour),"minute"===e.unit&&(s=l.minute),i=r.formatDate(o,s,!0,!1)}else i=r.formatDate(o,n.config.xaxis.labels.format);return{dateString:a,position:e.position,value:i,unit:e.unit,year:e.year,month:e.month}})}},{key:"removeOverlappingTS",value:function(e){var t=this,n=new b(this.ctx),i=0,r=e.map(function(r,a){if(a>0&&t.w.config.xaxis.labels.hideOverlappingLabels){var o=n.getTextRects(e[i].value).width,s=e[i].position;return r.position>s+o+10?(i=a,r):null}return r});return r=r.filter(function(e){return null!==e})}}]),e}(),K=function(){function e(t,n){o(this,e),this.ctx=n,this.w=n.w,this.el=t,this.coreUtils=new C(this.ctx),this.twoDSeries=[],this.threeDSeries=[],this.twoDSeriesX=[]}return l(e,[{key:"setupElements",value:function(){var e=this.w.globals,t=this.w.config,n=t.chart.type;e.axisCharts=["line","area","bar","rangeBar","candlestick","radar","scatter","bubble","heatmap"].indexOf(n)>-1,e.xyCharts=["line","area","bar","rangeBar","candlestick","scatter","bubble"].indexOf(n)>-1,e.isBarHorizontal=("bar"===t.chart.type||"rangeBar"===t.chart.type)&&t.plotOptions.bar.horizontal,e.chartClass=".apexcharts"+e.cuid,e.dom.baseEl=this.el,e.dom.elWrap=document.createElement("div"),b.setAttrs(e.dom.elWrap,{id:e.chartClass.substring(1),class:"apexcharts-canvas "+e.chartClass.substring(1)}),this.el.appendChild(e.dom.elWrap),e.dom.Paper=new window.SVG.Doc(e.dom.elWrap),e.dom.Paper.attr({class:"apexcharts-svg","xmlns:data":"ApexChartsNS",transform:"translate(".concat(t.chart.offsetX,", ").concat(t.chart.offsetY,")")}),e.dom.Paper.node.style.background=t.chart.background,this.setSVGDimensions(),e.dom.elGraphical=e.dom.Paper.group().attr({class:"apexcharts-inner apexcharts-graphical"}),e.dom.elDefs=e.dom.Paper.defs(),e.dom.elLegendWrap=document.createElement("div"),e.dom.elLegendWrap.classList.add("apexcharts-legend"),e.dom.elWrap.appendChild(e.dom.elLegendWrap),e.dom.Paper.add(e.dom.elGraphical),e.dom.elGraphical.add(e.dom.elDefs)}},{key:"plotChartType",value:function(e,t){var n=this.w,i=n.config,r=n.globals,a={series:[],i:[]},o={series:[],i:[]},s={series:[],i:[]},l={series:[],i:[]},c={series:[],i:[]};r.series.map(function(t,i){void 0!==e[i].type?("column"===e[i].type||"bar"===e[i].type?(n.config.plotOptions.bar.horizontal=!1,l.series.push(t),l.i.push(i)):"area"===e[i].type?(o.series.push(t),o.i.push(i)):"line"===e[i].type?(a.series.push(t),a.i.push(i)):"scatter"===e[i].type?(s.series.push(t),s.i.push(i)):"bubble"===e[i].type||("candlestick"===e[i].type?(c.series.push(t),c.i.push(i)):console.warn("You have specified an unrecognized chart type. Available types for this propery are line/area/column/bar/scatter/bubble")),r.comboCharts=!0):(a.series.push(t),a.i.push(i))});var u=new W(this.ctx,t),h=new L(this.ctx,t),d=new j(this.ctx),f=new Y(this.ctx),p=new H(this.ctx,t),g=new z(this.ctx),m=[];if(r.comboCharts){if(o.series.length>0&&m.push(u.draw(o.series,"area",o.i)),l.series.length>0)if(n.config.chart.stacked){var v=new N(this.ctx,t);m.push(v.draw(l.series,l.i))}else{var y=new D(this.ctx,t);m.push(y.draw(l.series,l.i))}if(a.series.length>0&&m.push(u.draw(a.series,"line",a.i)),c.series.length>0&&m.push(h.draw(c.series,c.i)),s.series.length>0){var b=new W(this.ctx,t,!0);m.push(b.draw(s.series,"scatter",s.i))}}else switch(i.chart.type){case"line":m=u.draw(r.series,"line");break;case"area":m=u.draw(r.series,"area");break;case"bar":if(i.chart.stacked)m=new N(this.ctx,t).draw(r.series);else m=new D(this.ctx,t).draw(r.series);break;case"candlestick":m=new L(this.ctx,t).draw(r.series);break;case"rangeBar":m=p.draw(r.series);break;case"heatmap":m=new F(this.ctx,t).draw(r.series);break;case"pie":case"donut":m=d.draw(r.series);break;case"radialBar":m=f.draw(r.series);break;case"radar":m=g.draw(r.series);break;default:m=u.draw(r.series)}return m}},{key:"setSVGDimensions",value:function(){var e=this.w.globals,t=this.w.config;e.svgWidth=t.chart.width,e.svgHeight=t.chart.height;var n=m.getDimensions(this.el),i=t.chart.width.toString().split(/[0-9]+/g).pop();if("%"===i?m.isNumber(n[0])&&(0===n[0].width&&(n=m.getDimensions(this.el.parentNode)),e.svgWidth=n[0]*parseInt(t.chart.width)/100):"px"!==i&&""!==i||(e.svgWidth=parseInt(t.chart.width)),"auto"!==e.svgHeight&&""!==e.svgHeight)if("%"===t.chart.height.toString().split(/[0-9]+/g).pop()){var r=m.getDimensions(this.el.parentNode);e.svgHeight=r[1]*parseInt(t.chart.height)/100}else e.svgHeight=parseInt(t.chart.height);else e.axisCharts?e.svgHeight=e.svgWidth/1.61:e.svgHeight=e.svgWidth;b.setAttrs(e.dom.Paper.node,{width:e.svgWidth,height:e.svgHeight});var a=t.chart.sparkline.enabled?0:e.axisCharts?t.chart.parentHeightOffset:0;e.dom.Paper.node.parentNode.parentNode.style.minHeight=e.svgHeight+a+"px",e.dom.elWrap.style.width=e.svgWidth+"px",e.dom.elWrap.style.height=e.svgHeight+"px"}},{key:"shiftGraphPosition",value:function(){var e=this.w.globals,t=e.translateY,n={transform:"translate("+e.translateX+", "+t+")"};b.setAttrs(e.dom.elGraphical.node,n)}},{key:"coreCalculations",value:function(){new G(this.ctx).init()}},{key:"resetGlobals",value:function(){var e=this,t=this.w.globals;t.series=[],t.seriesCandleO=[],t.seriesCandleH=[],t.seriesCandleL=[],t.seriesCandleC=[],t.seriesRangeStart=[],t.seriesRangeEnd=[],t.seriesPercent=[],t.seriesX=[],t.seriesZ=[],t.seriesNames=[],t.seriesTotals=[],t.stackedSeriesTotals=[],t.labels=[],t.timelineLabels=[],t.noLabelsProvided=!1,t.timescaleTicks=[],t.resizeTimer=null,t.selectionResizeTimer=null,t.seriesXvalues=e.w.config.series.map(function(e){return[]}),t.seriesYvalues=e.w.config.series.map(function(e){return[]}),t.delayedElements=[],t.pointsArray=[],t.dataLabelsRects=[],t.isXNumeric=!1,t.isDataXYZ=!1,t.maxY=-Number.MAX_VALUE,t.minY=Number.MIN_VALUE,t.minYArr=[],t.maxYArr=[],t.maxX=-Number.MAX_VALUE,t.minX=Number.MAX_VALUE,t.initialmaxX=-Number.MAX_VALUE,t.initialminX=Number.MAX_VALUE,t.maxDate=0,t.minDate=Number.MAX_VALUE,t.minZ=Number.MAX_VALUE,t.maxZ=-Number.MAX_VALUE,t.minXDiff=Number.MAX_VALUE,t.yAxisScale=[],t.xAxisScale=null,t.xAxisTicksPositions=[],t.yLabelsCoords=[],t.yTitleCoords=[],t.xRange=0,t.yRange=[],t.zRange=0,t.dataPoints=0}},{key:"isMultipleY",value:function(){if(this.w.config.yaxis.constructor===Array&&this.w.config.yaxis.length>1)return this.w.config.chart.stacked=!1,this.w.globals.isMultipleYAxis=!0,!0}},{key:"excludeCollapsedSeriesInYAxis",value:function(){var e=this,t=this.w;t.globals.ignoreYAxisIndexes=t.globals.collapsedSeries.map(function(t,n){if(e.w.globals.isMultipleYAxis)return t.index})}},{key:"isMultiFormat",value:function(){return this.isFormatXY()||this.isFormat2DArray()}},{key:"isFormatXY",value:function(){var e=this.w.config.series.slice(),t=new Q(this.ctx);if(this.activeSeriesIndex=t.getActiveConfigSeriesIndex(),void 0!==e[this.activeSeriesIndex].data&&e[this.activeSeriesIndex].data.length>0&&null!==e[this.activeSeriesIndex].data[0]&&void 0!==e[this.activeSeriesIndex].data[0].x&&null!==e[this.activeSeriesIndex].data[0])return!0}},{key:"isFormat2DArray",value:function(){var e=this.w.config.series.slice(),t=new Q(this.ctx);if(this.activeSeriesIndex=t.getActiveConfigSeriesIndex(),void 0!==e[this.activeSeriesIndex].data&&e[this.activeSeriesIndex].data.length>0&&void 0!==e[this.activeSeriesIndex].data[0]&&null!==e[this.activeSeriesIndex].data[0]&&e[this.activeSeriesIndex].data[0].constructor===Array)return!0}},{key:"handleFormat2DArray",value:function(e,t){for(var n=this.w.config,i=this.w.globals,r=0;r<e[t].data.length;r++)if(void 0!==e[t].data[r][1]&&(Array.isArray(e[t].data[r][1])&&4===e[t].data[r][1].length?this.twoDSeries.push(m.parseNumber(e[t].data[r][1][3])):this.twoDSeries.push(m.parseNumber(e[t].data[r][1])),i.dataFormatXNumeric=!0),"datetime"===n.xaxis.type){var a=new Date(e[t].data[r][0]);a=new Date(a).getTime(),this.twoDSeriesX.push(a)}else this.twoDSeriesX.push(e[t].data[r][0]);for(var o=0;o<e[t].data.length;o++)void 0!==e[t].data[o][2]&&(this.threeDSeries.push(e[t].data[o][2]),i.isDataXYZ=!0)}},{key:"handleFormatXY",value:function(e,t){var n=this.w.config,i=this.w.globals,r=new S(this.ctx),a=t;i.collapsedSeriesIndices.indexOf(t)>-1&&(a=this.activeSeriesIndex);for(var o=0;o<e[t].data.length;o++)void 0!==e[t].data[o].y&&(Array.isArray(e[t].data[o].y)?this.twoDSeries.push(m.parseNumber(e[t].data[o].y[e[t].data[o].y.length-1])):this.twoDSeries.push(m.parseNumber(e[t].data[o].y)));for(var s=0;s<e[a].data.length;s++){var l="string"==typeof e[a].data[s].x,c=!!r.isValidDate(e[a].data[s].x.toString());l||c?l?"datetime"!==n.xaxis.type||i.isRangeData?(this.fallbackToCategory=!0,this.twoDSeriesX.push(e[a].data[s].x)):this.twoDSeriesX.push(r.parseDate(e[a].data[s].x)):"datetime"===n.xaxis.type?this.twoDSeriesX.push(r.parseDate(e[a].data[s].x.toString())):(i.dataFormatXNumeric=!0,i.isXNumeric=!0,this.twoDSeriesX.push(parseFloat(e[a].data[s].x))):(i.isXNumeric=!0,i.dataFormatXNumeric=!0,this.twoDSeriesX.push(e[a].data[s].x))}if(e[t].data[0]&&void 0!==e[t].data[0].z){for(var u=0;u<e[t].data.length;u++)this.threeDSeries.push(e[t].data[u].z);i.isDataXYZ=!0}}},{key:"handleRangeData",value:function(e,t){var n=this.w.globals,i={};return this.isFormat2DArray()?i=this.handleRangeDataFormat("array",e,t):this.isFormatXY()&&(i=this.handleRangeDataFormat("xy",e,t)),n.seriesRangeStart.push(i.start),n.seriesRangeEnd.push(i.end),i}},{key:"handleCandleStickData",value:function(e,t){var n=this.w.globals,i={};return this.isFormat2DArray()?i=this.handleCandleStickDataFormat("array",e,t):this.isFormatXY()&&(i=this.handleCandleStickDataFormat("xy",e,t)),n.seriesCandleO.push(i.o),n.seriesCandleH.push(i.h),n.seriesCandleL.push(i.l),n.seriesCandleC.push(i.c),i}},{key:"handleRangeDataFormat",value:function(e,t,n){var i=[],r=[],a="Please provide [Start, End] values in valid format. Read more https://apexcharts.com/docs/series/#rangecharts",o=new Q(this.ctx).getActiveConfigSeriesIndex();if("array"===e){if(2!==t[o].data[0][1].length)throw new Error(a);for(var s=0;s<t[n].data.length;s++)i.push(t[n].data[s][1][0]),r.push(t[n].data[s][1][1])}else if("xy"===e){if(2!==t[o].data[0].y.length)throw new Error(a);for(var l=0;l<t[n].data.length;l++)i.push(t[n].data[l].y[0]),r.push(t[n].data[l].y[1])}return{start:i,end:r}}},{key:"handleCandleStickDataFormat",value:function(e,t,n){var i=[],r=[],a=[],o=[],s="Please provide [Open, High, Low and Close] values in valid format. Read more https://apexcharts.com/docs/series/#candlestick";if("array"===e){if(4!==t[n].data[0][1].length)throw new Error(s);for(var l=0;l<t[n].data.length;l++)i.push(t[n].data[l][1][0]),r.push(t[n].data[l][1][1]),a.push(t[n].data[l][1][2]),o.push(t[n].data[l][1][3])}else if("xy"===e){if(4!==t[n].data[0].y.length)throw new Error(s);for(var c=0;c<t[n].data.length;c++)i.push(t[n].data[c].y[0]),r.push(t[n].data[c].y[1]),a.push(t[n].data[c].y[2]),o.push(t[n].data[c].y[3])}return{o:i,h:r,l:a,c:o}}},{key:"parseDataAxisCharts",value:function(e){for(var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:this.ctx,n=this.w.config,i=this.w.globals,r=new S(t),a=0;a<e.length;a++){if(this.twoDSeries=[],this.twoDSeriesX=[],this.threeDSeries=[],void 0===e[a].data)return void console.error("It is a possibility that you may have not included 'data' property in series.");if("rangeBar"!==n.chart.type&&"rangeArea"!==n.chart.type&&"rangeBar"!==e[a].type&&"rangeArea"!==e[a].type||(i.isRangeData=!0,this.handleRangeData(e,a)),this.isMultiFormat())this.isFormat2DArray()?this.handleFormat2DArray(e,a):this.isFormatXY()&&this.handleFormatXY(e,a),"candlestick"!==n.chart.type&&"candlestick"!==e[a].type||this.handleCandleStickData(e,a),i.series.push(this.twoDSeries),i.labels.push(this.twoDSeriesX),i.seriesX.push(this.twoDSeriesX),this.fallbackToCategory||(i.isXNumeric=!0);else{if("datetime"===n.xaxis.type){i.isXNumeric=!0;for(var o=n.labels.length>0?n.labels.slice():n.xaxis.categories.slice(),s=0;s<o.length;s++)if("string"==typeof o[s]){if(!r.isValidDate(o[s]))throw new Error("You have provided invalid Date format. Please provide a valid JavaScript Date");this.twoDSeriesX.push(r.parseDate(o[s]))}i.seriesX.push(this.twoDSeriesX)}else if("numeric"===n.xaxis.type){i.isXNumeric=!0;var l=n.labels.length>0?n.labels.slice():n.xaxis.categories.slice();l.length>0&&(this.twoDSeriesX=l,i.seriesX.push(this.twoDSeriesX))}i.labels.push(this.twoDSeriesX);var c=e[a].data.map(function(e){return m.parseNumber(e)});i.series.push(c)}i.seriesZ.push(this.threeDSeries),void 0!==e[a].name?i.seriesNames.push(e[a].name):i.seriesNames.push("series-"+parseInt(a+1))}return this.w}},{key:"parseDataNonAxisCharts",value:function(e){var t=this.w.globals,n=this.w.config;t.series=e.slice(),t.seriesNames=n.labels.slice();for(var i=0;i<t.series.length;i++)void 0===t.seriesNames[i]&&t.seriesNames.push("series-"+(i+1));return this.w}},{key:"handleExternalLabelsData",value:function(e){var t=this.w.config,n=this.w.globals;if(t.xaxis.categories.length>0)n.labels=t.xaxis.categories;else if(t.labels.length>0)n.labels=t.labels.slice();else if(this.fallbackToCategory)n.labels=n.labels[0];else{var i=[];if(n.axisCharts){for(var r=0;r<n.series[n.maxValsInArrayIndex].length;r++)i.push(r+1);for(var a=0;a<e.length;a++)n.seriesX.push(i);n.isXNumeric=!0}if(0===i.length){i=[0,10];for(var o=0;o<e.length;o++)n.seriesX.push(i)}n.labels=i,n.noLabelsProvided=!0}}},{key:"parseData",value:function(e){var t=this.w,n=t.config,i=t.globals;if(this.excludeCollapsedSeriesInYAxis(),this.fallbackToCategory=!1,this.resetGlobals(),this.isMultipleY(),i.axisCharts?this.parseDataAxisCharts(e):this.parseDataNonAxisCharts(e),this.coreUtils.getLargestSeries(),"bar"===n.chart.type&&n.chart.stacked){var r=new Q(this.ctx);i.series=r.setNullSeriesToZeroValues(i.series)}this.coreUtils.getSeriesTotals(),i.axisCharts&&this.coreUtils.getStackedSeriesTotals(),this.coreUtils.getPercentSeries(),i.dataFormatXNumeric||i.isXNumeric&&("numeric"!==n.xaxis.type||0!==n.labels.length||0!==n.xaxis.categories.length)||this.handleExternalLabelsData(e)}},{key:"xySettings",value:function(){var e=null,t=this.w;if(t.globals.axisCharts){if("back"===t.config.xaxis.crosshairs.position)new R(this.ctx).drawXCrosshairs();if("back"===t.config.yaxis[0].crosshairs.position)new R(this.ctx).drawYCrosshairs();if(e=this.coreUtils.getCalculatedRatios(),"datetime"===t.config.xaxis.type&&void 0===t.config.xaxis.labels.formatter){var n,i=new Z(this.ctx);isFinite(t.globals.minX)&&isFinite(t.globals.maxX)&&!t.globals.isBarHorizontal?(n=i.calculateTimeScaleTicks(t.globals.minX,t.globals.maxX),i.recalcDimensionsBasedOnFormat(n,!1)):t.globals.isBarHorizontal&&(n=i.calculateTimeScaleTicks(t.globals.minY,t.globals.maxY),i.recalcDimensionsBasedOnFormat(n,!0))}}return e}},{key:"drawAxis",value:function(e,t){var n,i,r=this.w.globals,a=this.w.config,o=new B(this.ctx),s=new q(this.ctx);r.axisCharts&&"radar"!==e&&(r.isBarHorizontal?(i=s.drawYaxisInversed(0),n=o.drawXaxisInversed(0),r.dom.elGraphical.add(n),r.dom.elGraphical.add(i)):(n=o.drawXaxis(),r.dom.elGraphical.add(n),a.yaxis.map(function(e,t){-1===r.ignoreYAxisIndexes.indexOf(t)&&(i=s.drawYaxis(t),r.dom.Paper.add(i))})));a.yaxis.map(function(e,t){-1===r.ignoreYAxisIndexes.indexOf(t)&&s.yAxisTitleRotate(t,e.opposite)})}},{key:"setupBrushHandler",value:function(){var e=this,t=this.w;t.config.chart.brush.enabled&&("function"!=typeof t.config.chart.events.selection&&(t.config.chart.brush.targets||[t.config.chart.brush.target]).forEach(function(n){var i=ApexCharts.getChartByID(n);i.w.globals.brushSource=e.ctx;var r=function(){e.ctx._updateOptions({chart:{selection:{xaxis:{min:i.w.globals.minX,max:i.w.globals.maxX}}}},!1,!1)};"function"!=typeof i.w.config.chart.events.zoomed&&(i.w.config.chart.events.zoomed=function(){r()}),"function"!=typeof i.w.config.chart.events.scrolled&&(i.w.config.chart.events.scrolled=function(){r()}),t.config.chart.events.selection=function(e,n){var r=m.clone(t.config.yaxis);t.config.chart.brush.autoScaleYaxis&&(r=new U(i).autoScaleY(i,n));i._updateOptions({xaxis:{min:n.xaxis.min,max:n.xaxis.max},yaxis:r},!1,!1,!1)}}))}}]),e}();var J=setTimeout;function ee(){}function te(e){if(!(this instanceof te))throw new TypeError("Promises must be constructed via new");if("function"!=typeof e)throw new TypeError("not a function");this._state=0,this._handled=!1,this._value=void 0,this._deferreds=[],se(e,this)}function ne(e,t){for(;3===e._state;)e=e._value;0!==e._state?(e._handled=!0,te._immediateFn(function(){var n=1===e._state?t.onFulfilled:t.onRejected;if(null!==n){var i;try{i=n(e._value)}catch(e){return void re(t.promise,e)}ie(t.promise,i)}else(1===e._state?ie:re)(t.promise,e._value)})):e._deferreds.push(t)}function ie(e,t){try{if(t===e)throw new TypeError("A promise cannot be resolved with itself.");if(t&&("object"==typeof t||"function"==typeof t)){var n=t.then;if(t instanceof te)return e._state=3,e._value=t,void ae(e);if("function"==typeof n)return void se((i=n,r=t,function(){i.apply(r,arguments)}),e)}e._state=1,e._value=t,ae(e)}catch(t){re(e,t)}var i,r}function re(e,t){e._state=2,e._value=t,ae(e)}function ae(e){2===e._state&&0===e._deferreds.length&&te._immediateFn(function(){e._handled||te._unhandledRejectionFn(e._value)});for(var t=0,n=e._deferreds.length;t<n;t++)ne(e,e._deferreds[t]);e._deferreds=null}function oe(e,t,n){this.onFulfilled="function"==typeof e?e:null,this.onRejected="function"==typeof t?t:null,this.promise=n}function se(e,t){var n=!1;try{e(function(e){n||(n=!0,ie(t,e))},function(e){n||(n=!0,re(t,e))})}catch(e){if(n)return;n=!0,re(t,e)}}te.prototype.catch=function(e){return this.then(null,e)},te.prototype.then=function(e,t){var n=new this.constructor(ee);return ne(this,new oe(e,t,n)),n},te.prototype.finally=function(e){var t=this.constructor;return this.then(function(n){return t.resolve(e()).then(function(){return n})},function(n){return t.resolve(e()).then(function(){return t.reject(n)})})},te.all=function(e){return new te(function(t,n){if(!e||void 0===e.length)throw new TypeError("Promise.all accepts an array");var i=Array.prototype.slice.call(e);if(0===i.length)return t([]);var r=i.length;function a(e,o){try{if(o&&("object"==typeof o||"function"==typeof o)){var s=o.then;if("function"==typeof s)return void s.call(o,function(t){a(e,t)},n)}i[e]=o,0==--r&&t(i)}catch(e){n(e)}}for(var o=0;o<i.length;o++)a(o,i[o])})},te.resolve=function(e){return e&&"object"==typeof e&&e.constructor===te?e:new te(function(t){t(e)})},te.reject=function(e){return new te(function(t,n){n(e)})},te.race=function(e){return new te(function(t,n){for(var i=0,r=e.length;i<r;i++)e[i].then(t,n)})},te._immediateFn="function"==typeof i&&function(e){i(e)}||function(e){J(e,0)},te._unhandledRejectionFn=function(e){"undefined"!=typeof console&&console&&console.warn("Possible Unhandled Promise Rejection:",e)};var le,ce=function(){function e(t){o(this,e),this.ctx=t,this.w=t.w}return l(e,[{key:"getSvgString",value:function(){return this.w.globals.dom.Paper.svg()}},{key:"cleanup",value:function(){var e=this.w,t=e.globals.dom.baseEl.querySelector(".apexcharts-xcrosshairs"),n=e.globals.dom.baseEl.querySelector(".apexcharts-ycrosshairs");t&&t.setAttribute("x",-500),n&&(n.setAttribute("y1",-100),n.setAttribute("y2",-100))}},{key:"svgUrl",value:function(){this.cleanup();var e=this.getSvgString(),t=new Blob([e],{type:"image/svg+xml;charset=utf-8"});return URL.createObjectURL(t)}},{key:"dataURI",value:function(){var e=this;return new te(function(t){var n=e.w;e.cleanup();var i=document.createElement("canvas");i.width=n.globals.svgWidth,i.height=n.globals.svgHeight;var r="transparent"===n.config.chart.background?"#fff":n.config.chart.background,a=i.getContext("2d");a.fillStyle=r,a.fillRect(0,0,i.width,i.height);var o=window.URL||window.webkitURL||window,s=new Image;s.crossOrigin="anonymous";var l=e.getSvgString(),c="data:image/svg+xml,"+encodeURIComponent(l);s.onload=function(){a.drawImage(s,0,0),o.revokeObjectURL(c);var e=i.toDataURL("image/png");t(e)},s.src=c})}},{key:"exportToSVG",value:function(){this.triggerDownload(this.svgUrl(),".svg")}},{key:"exportToPng",value:function(){var e=this;this.dataURI().then(function(t){e.triggerDownload(t,".png")})}},{key:"triggerDownload",value:function(e,t){var n=document.createElement("a");n.href=e,n.download=this.w.globals.chartID+t,document.body.appendChild(n),n.click(),document.body.removeChild(n)}}]),e}(),ue=function(){function e(t){o(this,e),this.ctx=t,this.w=t.w;var n=this.w;this.anim=new y(this.ctx),this.xaxisLabels=n.globals.labels.slice(),this.animX=n.config.grid.xaxis.lines.animate&&n.config.chart.animations.enabled,this.animY=n.config.grid.yaxis.lines.animate&&n.config.chart.animations.enabled,n.globals.timelineLabels.length>0&&(this.xaxisLabels=n.globals.timelineLabels.slice())}return l(e,[{key:"drawGridArea",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:null,t=this.w,n=new b(this.ctx);null===e&&(e=n.group({class:"apexcharts-grid"}));var i=n.drawLine(t.globals.padHorizontal,1,t.globals.padHorizontal,t.globals.gridHeight,"transparent"),r=n.drawLine(t.globals.padHorizontal,t.globals.gridHeight,t.globals.gridWidth,t.globals.gridHeight,"transparent");return e.add(r),e.add(i),e}},{key:"drawGrid",value:function(){var e=this.w,t=new B(this.ctx),n=new q(this.ctx),i=this.w.globals,r=null;if(i.axisCharts){if(e.config.grid.show)r=this.renderGrid(),i.dom.elGraphical.add(r.el),this.drawGridArea(r.el);else{var a=this.drawGridArea();i.dom.elGraphical.add(a)}null!==r&&t.xAxisLabelCorrections(r.xAxisTickWidth),n.setYAxisTextAlignments()}}},{key:"createGridMask",value:function(){var e=this.w,t=e.globals,n=new b(this.ctx),i=Array.isArray(e.config.stroke.width)?0:e.config.stroke.width;if(Array.isArray(e.config.stroke.width)){var r=0;e.config.stroke.width.forEach(function(e){r=Math.max(r,e)}),i=r}t.dom.elGridRectMask=document.createElementNS(t.SVGNS,"clipPath"),t.dom.elGridRectMask.setAttribute("id","gridRectMask".concat(t.cuid)),t.dom.elGridRectMarkerMask=document.createElementNS(t.SVGNS,"clipPath"),t.dom.elGridRectMarkerMask.setAttribute("id","gridRectMarkerMask".concat(t.cuid)),t.dom.elGridRect=n.drawRect(-i/2,-i/2,t.gridWidth+i,t.gridHeight+i,0,"#fff"),new C(this).getLargestMarkerSize();var a=e.globals.markers.largestSize+e.config.markers.hover.sizeOffset+1;a<10&&(a=10),t.dom.elGridRectMarker=n.drawRect(2*-a,2*-a,t.gridWidth+4*a,t.gridHeight+4*a,0,"#fff"),t.dom.elGridRectMask.appendChild(t.dom.elGridRect.node),t.dom.elGridRectMarkerMask.appendChild(t.dom.elGridRectMarker.node);var o=t.dom.baseEl.querySelector("defs");o.appendChild(t.dom.elGridRectMask),o.appendChild(t.dom.elGridRectMarkerMask)}},{key:"renderGrid",value:function(){var e=this.w,t=new b(this.ctx),n=e.config.grid.strokeDashArray,i=t.group({class:"apexcharts-grid"}),r=t.group({class:"apexcharts-gridlines-horizontal"}),a=t.group({class:"apexcharts-gridlines-vertical"});i.add(r),i.add(a);for(var o,s=8,l=0;l<e.globals.series.length&&(void 0!==e.globals.yAxisScale[l]&&(s=e.globals.yAxisScale[l].result.length-1),!(s>2));l++);if(e.globals.isBarHorizontal){if(o=s,e.config.grid.xaxis.lines.show||e.config.xaxis.axisTicks.show)for(var c,u=e.globals.padHorizontal,h=e.globals.gridHeight,d=0;d<o+1&&(c=u=u+e.globals.gridWidth/o+.3,d!==o-1);d++){if(e.config.grid.xaxis.lines.show){var f=t.drawLine(u,0,c,h,e.config.grid.borderColor,n);f.node.classList.add("apexcharts-gridline"),a.add(f),this.animX&&this.animateLine(f,{x1:0,x2:0},{x1:u,x2:c})}new B(this.ctx).drawXaxisTicks(u,i)}if(e.config.grid.yaxis.lines.show)for(var p=0,g=0,m=e.globals.gridWidth,v=0;v<e.globals.dataPoints+1;v++){var y=t.drawLine(0,p,m,g,e.config.grid.borderColor,n);r.add(y),y.node.classList.add("apexcharts-gridline"),this.animY&&this.animateLine(y,{y1:p+20,y2:g+20},{y1:p,y2:g}),g=p+=e.globals.gridHeight/e.globals.dataPoints}}else{if(o=this.xaxisLabels.length,e.config.grid.xaxis.lines.show||e.config.xaxis.axisTicks.show){var x,w=e.globals.padHorizontal,k=e.globals.gridHeight;if(e.globals.timelineLabels.length>0)for(var S=0;S<o;S++){if(w=this.xaxisLabels[S].position,x=this.xaxisLabels[S].position,e.config.grid.xaxis.lines.show&&w>0&&w<e.globals.gridWidth){var E=t.drawLine(w,0,x,k,e.config.grid.borderColor,n);E.node.classList.add("apexcharts-gridline"),a.add(E),this.animX&&this.animateLine(E,{x1:0,x2:0},{x1:w,x2:x})}var C=new B(this.ctx);S===o-1&&e.globals.skipLastTimelinelabel||C.drawXaxisTicks(w,i)}else for(var T=o,A=0;A<T;A++){var _=T;if(e.globals.isXNumeric&&"bar"!==e.config.chart.type&&(_-=1),x=w+=e.globals.gridWidth/_,A===_-1)break;if(e.config.grid.xaxis.lines.show){var O=t.drawLine(w,0,x,k,e.config.grid.borderColor,n);O.node.classList.add("apexcharts-gridline"),a.add(O),this.animX&&this.animateLine(O,{x1:0,x2:0},{x1:w,x2:x})}new B(this.ctx).drawXaxisTicks(w,i)}}if(e.config.grid.yaxis.lines.show)for(var P=0,M=0,I=e.globals.gridWidth,D=0;D<s+1;D++){var N=t.drawLine(0,P,I,M,e.config.grid.borderColor,n);r.add(N),N.node.classList.add("apexcharts-gridline"),this.animY&&this.animateLine(N,{y1:P+20,y2:M+20},{y1:P,y2:M}),M=P+=e.globals.gridHeight/s}}return this.drawGridBands(i,o,s),{el:i,xAxisTickWidth:e.globals.gridWidth/o}}},{key:"drawGridBands",value:function(e,t,n){var i=this.w,r=new b(this.ctx);if(void 0!==i.config.grid.row.colors&&i.config.grid.row.colors.length>0)for(var a=0,o=i.globals.gridHeight/n,s=i.globals.gridWidth,l=0,c=0;l<n;l++,c++){c>=i.config.grid.row.colors.length&&(c=0);var u=i.config.grid.row.colors[c],h=r.drawRect(0,a,s,o,0,u,i.config.grid.row.opacity);e.add(h),h.node.classList.add("apexcharts-gridRow"),a+=i.globals.gridHeight/n}if(void 0!==i.config.grid.column.colors&&i.config.grid.column.colors.length>0)for(var d=i.globals.padHorizontal,f=i.globals.padHorizontal+i.globals.gridWidth/t,p=i.globals.gridHeight,g=0,m=0;g<t;g++,m++){m>=i.config.grid.column.colors.length&&(m=0);var v=i.config.grid.column.colors[m],y=r.drawRect(d,0,f,p,0,v,i.config.grid.column.opacity);y.node.classList.add("apexcharts-gridColumn"),e.add(y),d+=i.globals.gridWidth/t}}},{key:"animateLine",value:function(e,t,n){var i=this.w,r=i.config.chart.animations;if(r&&!i.globals.resized&&!i.globals.dataChanged){var a=r.speed;this.anim.animateLine(e,t,n,a)}}}]),e}(),he=function(){function e(t,n){o(this,e),this.ctx=t,this.w=t.w,this.onLegendClick=this.onLegendClick.bind(this),this.onLegendHovered=this.onLegendHovered.bind(this)}return l(e,[{key:"init",value:function(){var e=this.w,t=e.globals,n=e.config;if((n.legend.showForSingleSeries&&1===t.series.length||t.series.length>1||!t.axisCharts)&&n.legend.show){for(;t.dom.elLegendWrap.firstChild;)t.dom.elLegendWrap.removeChild(t.dom.elLegendWrap.firstChild);this.drawLegends(),m.isIE11()?document.getElementsByTagName("head")[0].appendChild(this.getLegendStyles()):this.appendToForeignObject(),"bottom"===n.legend.position||"top"===n.legend.position?this.legendAlignHorizontal():"right"!==n.legend.position&&"left"!==n.legend.position||this.legendAlignVertical()}}},{key:"appendToForeignObject",value:function(){var e=this.w.globals,t=document.createElementNS(e.SVGNS,"foreignObject");t.setAttribute("x",0),t.setAttribute("y",0),t.setAttribute("width",e.svgWidth),t.setAttribute("height",e.svgHeight),e.dom.elLegendWrap.setAttribute("xmlns","http://www.w3.org/1999/xhtml"),t.appendChild(e.dom.elLegendWrap),t.appendChild(this.getLegendStyles()),e.dom.Paper.node.insertBefore(t,e.dom.elGraphical.node)}},{key:"drawLegends",value:function(){var e=this.w,t=e.config.legend.fontFamily,n=e.globals.seriesNames,i=e.globals.colors.slice();if("heatmap"===e.config.chart.type){var r=e.config.plotOptions.heatmap.colorScale.ranges;n=r.map(function(e){return e.name?e.name:e.from+" - "+e.to}),i=r.map(function(e){return e.color})}for(var a=e.globals.legendFormatter,o=0;o<=n.length-1;o++){var s=a(n[o],{seriesIndex:o,w:e}),l=!1,c=!1;if(e.globals.collapsedSeries.length>0)for(var u=0;u<e.globals.collapsedSeries.length;u++)e.globals.collapsedSeries[u].index===o&&(l=!0);if(e.globals.ancillaryCollapsedSeriesIndices.length>0)for(var h=0;h<e.globals.ancillaryCollapsedSeriesIndices.length;h++)e.globals.ancillaryCollapsedSeriesIndices[h]===o&&(c=!0);var d=document.createElement("span");d.classList.add("apexcharts-legend-marker");var f=e.config.legend.markers.offsetX,p=e.config.legend.markers.offsetY,g=e.config.legend.markers.height,m=e.config.legend.markers.width,v=e.config.legend.markers.strokeWidth,y=e.config.legend.markers.strokeColor,x=e.config.legend.markers.radius,w=d.style;w.background=i[o],w.color=i[o],w.height=Array.isArray(g)?parseFloat(g[o])+"px":parseFloat(g)+"px",w.width=Array.isArray(m)?parseFloat(m[o])+"px":parseFloat(m)+"px",w.left=Array.isArray(f)?f[o]:f,w.top=Array.isArray(p)?p[o]:p,w.borderWidth=Array.isArray(v)?v[o]:v,w.borderColor=Array.isArray(y)?y[o]:y,w.borderRadius=Array.isArray(x)?parseFloat(x[o])+"px":parseFloat(x)+"px",e.config.legend.markers.customHTML&&(Array.isArray(e.config.legend.markers.customHTML)?d.innerHTML=e.config.legend.markers.customHTML[o]():d.innerHTML=e.config.legend.markers.customHTML()),b.setAttrs(d,{rel:o+1,"data:collapsed":l||c}),(l||c)&&d.classList.add("inactive-legend");var k=document.createElement("div"),S=document.createElement("span");S.classList.add("apexcharts-legend-text"),S.innerHTML=s;var E=e.config.legend.labels.useSeriesColors?e.globals.colors[o]:e.config.legend.labels.colors;E||(E=e.config.chart.foreColor),S.style.color=E,S.style.fontSize=parseFloat(e.config.legend.fontSize)+"px",S.style.fontFamily=t||e.config.chart.fontFamily,b.setAttrs(S,{rel:o+1,"data:collapsed":l||c}),k.appendChild(d),k.appendChild(S);var T=new C(this.ctx);if(!e.config.legend.showForZeroSeries)0===T.getSeriesTotalByIndex(o)&&T.seriesHaveSameValues(o)&&!T.isSeriesNull(o)&&-1===e.globals.collapsedSeriesIndices.indexOf(o)&&-1===e.globals.ancillaryCollapsedSeriesIndices.indexOf(o)&&k.classList.add("apexcharts-hidden-zero-series");e.config.legend.showForNullSeries||T.isSeriesNull(o)&&-1===e.globals.collapsedSeriesIndices.indexOf(o)&&-1===e.globals.ancillaryCollapsedSeriesIndices.indexOf(o)&&k.classList.add("apexcharts-hidden-null-series"),e.globals.dom.elLegendWrap.appendChild(k),e.globals.dom.elLegendWrap.classList.add(e.config.legend.horizontalAlign),e.globals.dom.elLegendWrap.classList.add("position-"+e.config.legend.position),k.classList.add("apexcharts-legend-series"),k.style.margin="".concat(e.config.legend.itemMargin.horizontal,"px ").concat(e.config.legend.itemMargin.vertical,"px"),e.globals.dom.elLegendWrap.style.width=e.config.legend.width?e.config.legend.width+"px":"",e.globals.dom.elLegendWrap.style.height=e.config.legend.height?e.config.legend.height+"px":"",b.setAttrs(k,{rel:o+1,"data:collapsed":l||c}),(l||c)&&k.classList.add("inactive-legend"),e.config.legend.onItemClick.toggleDataSeries||k.classList.add("no-click")}"heatmap"!==e.config.chart.type&&e.config.legend.onItemClick.toggleDataSeries&&e.globals.dom.elWrap.addEventListener("click",this.onLegendClick,!0),e.config.legend.onItemHover.highlightDataSeries&&(e.globals.dom.elWrap.addEventListener("mousemove",this.onLegendHovered,!0),e.globals.dom.elWrap.addEventListener("mouseout",this.onLegendHovered,!0))}},{key:"getLegendBBox",value:function(){var e=this.w.globals.dom.baseEl.querySelector(".apexcharts-legend").getBoundingClientRect(),t=e.width;return{clwh:e.height,clww:t}}},{key:"setLegendWrapXY",value:function(e,t){var n=this.w,i=n.globals.dom.baseEl.querySelector(".apexcharts-legend"),r=i.getBoundingClientRect(),a=0,o=0;if("bottom"===n.config.legend.position)o+=n.globals.svgHeight-r.height/2;else if("top"===n.config.legend.position){var s=new $(this.ctx),l=s.getTitleSubtitleCoords("title").height,c=s.getTitleSubtitleCoords("subtitle").height;o=o+(l>0?l-10:0)+(c>0?c-10:0)}i.style.position="absolute",a=a+e+n.config.legend.offsetX,o=o+t+n.config.legend.offsetY,i.style.left=a+"px",i.style.top=o+"px","bottom"===n.config.legend.position?(i.style.top="auto",i.style.bottom=10+n.config.legend.offsetY+"px"):"right"===n.config.legend.position&&(i.style.left="auto",i.style.right=25+n.config.legend.offsetX+"px"),i.style.width&&(i.style.width=parseInt(n.config.legend.width)+"px"),i.style.height&&(i.style.height=parseInt(n.config.legend.height)+"px")}},{key:"legendAlignHorizontal",value:function(){var e=this.w;e.globals.dom.baseEl.querySelector(".apexcharts-legend").style.right=0;var t=this.getLegendBBox(),n=new $(this.ctx),i=n.getTitleSubtitleCoords("title"),r=n.getTitleSubtitleCoords("subtitle"),a=0;"bottom"===e.config.legend.position?a=-t.clwh/1.8:"top"===e.config.legend.position&&(a=i.height+r.height+e.config.title.margin+e.config.subtitle.margin-15),this.setLegendWrapXY(20,a)}},{key:"legendAlignVertical",value:function(){var e=this.w,t=this.getLegendBBox(),n=0;"left"===e.config.legend.position&&(n=20),"right"===e.config.legend.position&&(n=e.globals.svgWidth-t.clww-10),this.setLegendWrapXY(n,20)}},{key:"onLegendHovered",value:function(e){var t=this.w,n=e.target.classList.contains("apexcharts-legend-text")||e.target.classList.contains("apexcharts-legend-marker");if("heatmap"!==t.config.chart.type)!e.target.classList.contains("inactive-legend")&&n&&new Q(this.ctx).toggleSeriesOnHover(e,e.target);else if(n){var i=parseInt(e.target.getAttribute("rel"))-1;this.ctx.fireEvent("legendHover",[this.ctx,i,this.w]),new Q(this.ctx).highlightRangeInSeries(e,e.target)}}},{key:"onLegendClick",value:function(e){if(e.target.classList.contains("apexcharts-legend-text")||e.target.classList.contains("apexcharts-legend-marker")){var t=parseInt(e.target.getAttribute("rel"))-1,n="true"===e.target.getAttribute("data:collapsed"),i=this.w.config.chart.events.legendClick;"function"==typeof i&&i(this.ctx,t,this.w),this.ctx.fireEvent("legendClick",[this.ctx,t,this.w]);var r=this.w.config.legend.markers.onClick;"function"==typeof r&&e.target.classList.contains("apexcharts-legend-marker")&&(r(this.ctx,t,this.w),this.ctx.fireEvent("legendMarkerClick",[this.ctx,t,this.w])),this.toggleDataSeries(t,n)}}},{key:"getLegendStyles",value:function(){var e=document.createElement("style");e.setAttribute("type","text/css");var t=document.createTextNode("\n    \n      .apexcharts-legend {\n        display: flex;\n        overflow: auto;\n        padding: 0 10px;\n      }\n\n      .apexcharts-legend.position-bottom, .apexcharts-legend.position-top {\n        flex-wrap: wrap\n      }\n      .apexcharts-legend.position-right, .apexcharts-legend.position-left {\n        flex-direction: column;\n        bottom: 0;\n      }\n\n      .apexcharts-legend.position-bottom.left, .apexcharts-legend.position-top.left, .apexcharts-legend.position-right, .apexcharts-legend.position-left {\n        justify-content: flex-start;\n      }\n\n      .apexcharts-legend.position-bottom.center, .apexcharts-legend.position-top.center {\n        justify-content: center;  \n      }\n\n      .apexcharts-legend.position-bottom.right, .apexcharts-legend.position-top.right {\n        justify-content: flex-end;\n      }\n\n      .apexcharts-legend-series {\n        cursor: pointer;\n        line-height: normal;\n      }\n\n      .apexcharts-legend.position-bottom .apexcharts-legend-series, .apexcharts-legend.position-top .apexcharts-legend-series{\n        display: flex;\n        align-items: center;\n      }\n\n      .apexcharts-legend-text {\n        position: relative;\n        font-size: 14px;\n      }\n\n      .apexcharts-legend-text *, .apexcharts-legend-marker * {\n        pointer-events: none;\n      }\n\n      .apexcharts-legend-marker {\n        position: relative;\n        display: inline-block;\n        cursor: pointer;\n        margin-right: 3px;\n      }\n      \n      .apexcharts-legend.right .apexcharts-legend-series, .apexcharts-legend.left .apexcharts-legend-series{\n        display: inline-block;\n      }\n\n      .apexcharts-legend-series.no-click {\n        cursor: auto;\n      }\n\n      .apexcharts-legend .apexcharts-hidden-zero-series, .apexcharts-legend .apexcharts-hidden-null-series {\n        display: none !important;\n      }\n\n      .inactive-legend {\n        opacity: 0.45;\n      }");return e.appendChild(t),e}},{key:"resetToggleDataSeries",value:function(){var e=this.w,t=null,n=[];if(e.globals.axisCharts?(t=e.globals.dom.baseEl.querySelectorAll(".apexcharts-series[data\\:realIndex]"),(t=m.listToArray(t)).forEach(function(e){n.push(parseInt(e.getAttribute("data:realIndex")))})):(t=e.globals.dom.baseEl.querySelectorAll(".apexcharts-series[rel]"),(t=m.listToArray(t)).forEach(function(e){n.push(parseInt(e.getAttribute("rel"))-1)})),n.sort(),e.globals.collapsedSeries.length>0){for(var i=e.globals.risingSeries.slice(),r=e.config.series.slice(),a=0;a<e.globals.collapsedSeries.length;a++){var o=n.indexOf(e.globals.collapsedSeries[a].index);-1!==o&&(e.globals.axisCharts?r[o].data=e.globals.collapsedSeries.slice()[a].data.slice():r[o]=e.globals.collapsedSeries.slice()[a].data,i.push(o))}e.globals.collapsedSeries=[],e.globals.ancillaryCollapsedSeries=[],e.globals.collapsedSeriesIndices=[],e.globals.ancillaryCollapsedSeriesIndices=[],e.globals.risingSeries=i,e.config.series=r,this.ctx._updateSeries(e.config.series,e.config.chart.animations.dynamicAnimation.enabled)}}},{key:"toggleDataSeries",value:function(e,t){var n=this.w;if(n.globals.axisCharts||"radialBar"===n.config.chart.type){n.globals.resized=!0;var i=null,r=null;if(n.globals.risingSeries=[],n.globals.axisCharts?(i=n.globals.dom.baseEl.querySelector(".apexcharts-series[data\\:realIndex='".concat(e,"']")),r=parseInt(i.getAttribute("data:realIndex"))):(i=n.globals.dom.baseEl.querySelector(".apexcharts-series[rel='".concat(e+1,"']")),r=parseInt(i.getAttribute("rel"))-1),t)this.riseCollapsedSeries(n.globals.collapsedSeries,n.globals.collapsedSeriesIndices,r),this.riseCollapsedSeries(n.globals.ancillaryCollapsedSeries,n.globals.ancillaryCollapsedSeriesIndices,r);else{if(n.globals.axisCharts){var a=!1;if(n.config.yaxis[r]&&n.config.yaxis[r].show&&n.config.yaxis[r].showAlways&&(a=!0,n.globals.ancillaryCollapsedSeriesIndices.indexOf(r)<0&&(n.globals.ancillaryCollapsedSeries.push({index:r,data:n.config.series[r].data.slice(),type:i.parentNode.className.baseVal.split("-")[1]}),n.globals.ancillaryCollapsedSeriesIndices.push(r))),!a){n.globals.collapsedSeries.push({index:r,data:n.config.series[r].data.slice(),type:i.parentNode.className.baseVal.split("-")[1]}),n.globals.collapsedSeriesIndices.push(r);var o=n.globals.risingSeries.indexOf(r);n.globals.risingSeries.splice(o,1)}n.config.series[r].data=[]}else n.globals.collapsedSeries.push({index:r,data:n.config.series[r]}),n.globals.collapsedSeriesIndices.push(r),n.config.series[r]=0;for(var s=i.childNodes,l=0;l<s.length;l++)s[l].classList.contains("apexcharts-series-markers-wrap")&&(s[l].classList.contains("apexcharts-hide")?s[l].classList.remove("apexcharts-hide"):s[l].classList.add("apexcharts-hide"));n.globals.allSeriesCollapsed=n.globals.collapsedSeries.length===n.globals.series.length,this.ctx._updateSeries(n.config.series,n.config.chart.animations.dynamicAnimation.enabled)}}else{var c=n.globals.dom.Paper.select(" .apexcharts-series[rel='".concat(e+1,"'] path")),u=n.config.chart.type;if("pie"===u||"donut"===u){var h=n.config.plotOptions.pie.donut.labels,d=new b(this.ctx),f=new j(this.ctx);d.pathMouseDown(c.members[0],null),f.printDataLabelsInner(c.members[0].node,h)}c.fire("click")}}},{key:"riseCollapsedSeries",value:function(e,t,n){var i=this.w;if(e.length>0)for(var r=0;r<e.length;r++)e[r].index===n&&(i.globals.axisCharts?(i.config.series[n].data=e[r].data.slice(),e.splice(r,1),t.splice(r,1),i.globals.risingSeries.push(n)):(i.config.series[n]=e[r].data,e.splice(r,1),t.splice(r,1),i.globals.risingSeries.push(n)),this.ctx._updateSeries(i.config.series,i.config.chart.animations.dynamicAnimation.enabled))}}]),e}(),de=function(){function e(t){o(this,e),this.ctx=t,this.w=t.w}return l(e,[{key:"checkResponsiveConfig",value:function(e){var t=this,n=this.w,i=n.config;if(0!==i.responsive.length){var r=i.responsive.slice();r.sort(function(e,t){return e.breakpoint>t.breakpoint?1:t.breakpoint>e.breakpoint?-1:0}).reverse();var a=new T({}),o=function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},i=r[0].breakpoint,o=window.innerWidth>0?window.innerWidth:screen.width;if(o>i){var s=C.extendArrayProps(a,n.globals.initialConfig);e=m.extend(s,e),e=m.extend(n.config,e),t.overrideResponsiveOptions(e)}else for(var l=0;l<r.length;l++)o<r[l].breakpoint&&(e=C.extendArrayProps(a,r[l].options),e=m.extend(n.config,e),t.overrideResponsiveOptions(e))};if(e){var s=C.extendArrayProps(a,e);s=m.extend(n.config,s),o(s=m.extend(s,e))}else o({})}}},{key:"overrideResponsiveOptions",value:function(e){var t=new T(e).init();this.w.config=t}}]),e}(),fe=function(){function e(t){o(this,e),this.ctx=t,this.w=t.w,this.colors=[]}return l(e,[{key:"init",value:function(){this.setDefaultColors()}},{key:"setDefaultColors",value:function(){var e=this.w,t=new m;if(e.globals.dom.elWrap.classList.add(e.config.theme.mode),void 0===e.config.colors?e.globals.colors=this.predefined():e.globals.colors=e.config.colors,e.config.theme.monochrome.enabled){var n=[],i=e.globals.series.length;e.config.plotOptions.bar.distributed&&"bar"===e.config.chart.type&&(i=e.globals.series[0].length*e.globals.series.length);for(var r=e.config.theme.monochrome.color,a=1/(i/e.config.theme.monochrome.shadeIntensity),o=e.config.theme.monochrome.shadeTo,s=0,l=0;l<i;l++){var c=void 0;"dark"===o?(c=t.shadeColor(-1*s,r),s+=a):(c=t.shadeColor(s,r),s+=a),n.push(c)}e.globals.colors=n.slice()}var u=e.globals.colors.slice();this.pushExtraColors(e.globals.colors),void 0===e.config.stroke.colors?e.globals.stroke.colors=u:e.globals.stroke.colors=e.config.stroke.colors,this.pushExtraColors(e.globals.stroke.colors),void 0===e.config.fill.colors?e.globals.fill.colors=u:e.globals.fill.colors=e.config.fill.colors,this.pushExtraColors(e.globals.fill.colors),void 0===e.config.dataLabels.style.colors?e.globals.dataLabels.style.colors=u:e.globals.dataLabels.style.colors=e.config.dataLabels.style.colors,this.pushExtraColors(e.globals.dataLabels.style.colors,50),void 0===e.config.plotOptions.radar.polygons.fill.colors?e.globals.radarPolygons.fill.colors=["dark"===e.config.theme.mode?"#202D48":"#fff"]:e.globals.radarPolygons.fill.colors=e.config.plotOptions.radar.polygons.fill.colors,this.pushExtraColors(e.globals.radarPolygons.fill.colors,20),void 0===e.config.markers.colors?e.globals.markers.colors=u:e.globals.markers.colors=e.config.markers.colors,this.pushExtraColors(e.globals.markers.colors)}},{key:"pushExtraColors",value:function(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:null,i=this.w,r=t||i.globals.series.length;if(null===n&&(n="bar"===i.config.chart.type&&i.config.plotOptions.bar.distributed||"heatmap"===i.config.chart.type&&i.config.plotOptions.heatmap.colorScale.inverse),n&&(r=i.globals.series[0].length*i.globals.series.length),e.length<r)for(var a=r-e.length,o=0;o<a;o++)e.push(e[o])}},{key:"updateThemeOptions",value:function(e){e.chart=e.chart||{},e.tooltip=e.tooltip||{};var t=e.theme.mode||"light",n=e.theme.palette?e.theme.palette:"dark"===t?"palette4":"palette1",i=e.chart.foreColor?e.chart.foreColor:"dark"===t?"#f6f7f8":"#373d3f";return e.tooltip.theme=t,e.chart.foreColor=i,e.theme.palette=n,e}},{key:"predefined",value:function(){switch(this.w.config.theme.palette){case"palette1":this.colors=["#008FFB","#00E396","#FEB019","#FF4560","#775DD0"];break;case"palette2":this.colors=["#3f51b5","#03a9f4","#4caf50","#f9ce1d","#FF9800"];break;case"palette3":this.colors=["#33b2df","#546E7A","#d4526e","#13d8aa","#A5978B"];break;case"palette4":this.colors=["#4ecdc4","#c7f464","#81D4FA","#fd6a6a","#546E7A"];break;case"palette5":this.colors=["#2b908f","#f9a3a4","#90ee7e","#fa4443","#69d2e7"];break;case"palette6":this.colors=["#449DD1","#F86624","#EA3546","#662E9B","#C5D86D"];break;case"palette7":this.colors=["#D7263D","#1B998B","#2E294E","#F46036","#E2C044"];break;case"palette8":this.colors=["#662E9B","#F86624","#F9C80E","#EA3546","#43BCCD"];break;case"palette9":this.colors=["#5C4742","#A5978B","#8D5B4C","#5A2A27","#C4BBAF"];break;case"palette10":this.colors=["#A300D6","#7D02EB","#5653FE","#2983FF","#00B1F2"];break;default:this.colors=["#008FFB","#00E396","#FEB019","#FF4560","#775DD0"]}return this.colors}}]),e}(),pe=function(){function e(t){o(this,e),this.w=t.w,this.ttCtx=t,this.ctx=t.ctx}return l(e,[{key:"getNearestValues",value:function(e){var t=e.hoverArea,n=e.elGrid,i=e.clientX,r=e.clientY,a=e.hasBars,o=this.w,s=o.globals.gridWidth,l=s/(o.globals.dataPoints-1),c=n.getBoundingClientRect();(a&&o.globals.comboCharts||a)&&(l=s/o.globals.dataPoints);var u=i-c.left,h=r-c.top;u<0||h<0||u>o.globals.gridWidth||h>o.globals.gridHeight?(t.classList.remove("hovering-zoom"),t.classList.remove("hovering-pan")):o.globals.zoomEnabled?(t.classList.remove("hovering-pan"),t.classList.add("hovering-zoom")):o.globals.panEnabled&&(t.classList.remove("hovering-zoom"),t.classList.add("hovering-pan"));var d=Math.round(u/l);a&&(d=Math.ceil(u/l),d-=1);for(var f,p=null,g=null,m=[],v=0;v<o.globals.seriesXvalues.length;v++)m.push([o.globals.seriesXvalues[v][0]-1e-6].concat(o.globals.seriesXvalues[v]));return m=m.map(function(e){return e.filter(function(e){return e})}),f=o.globals.seriesYvalues.map(function(e){return e.filter(function(e){return e})}),o.globals.isXNumeric&&(p=(g=this.closestInMultiArray(u,h,m,f)).index,d=g.j,null!==p&&(m=o.globals.seriesXvalues[p],d=(g=this.closestInArray(u,m)).index)),(!d||d<1)&&(d=0),{capturedSeries:p,j:d,hoverX:u,hoverY:h}}},{key:"closestInMultiArray",value:function(e,t,n,i){var r=this.w,a=0,o=null,s=-1;r.globals.series.length>1?a=this.getFirstActiveXArray(n):o=0;var l=i[a][0],c=n[a][0],u=Math.abs(e-c),h=Math.abs(t-l),d=h+u;return i.map(function(r,a){r.map(function(r,l){var c=Math.abs(t-i[a][l]),f=Math.abs(e-n[a][l]),p=f+c;p<d&&(d=p,u=f,h=c,o=a,s=l)})}),{index:o,j:s}}},{key:"getFirstActiveXArray",value:function(e){for(var t=0,n=new C(this.ctx),i=e.map(function(e,t){return e.length>0?t:-1}),r=0;r<i.length;r++){var a=n.getSeriesTotalByIndex(r);if(-1!==i[r]&&0!==a&&!n.seriesHaveSameValues(r)){t=i[r];break}}return t}},{key:"closestInArray",value:function(e,t){for(var n=t[0],i=null,r=Math.abs(e-n),a=0;a<t.length;a++){var o=Math.abs(e-t[a]);o<r&&(r=o,n=t[a],i=a)}return{index:i}}},{key:"isXoverlap",value:function(e){var t=[],n=this.w.globals.seriesX.filter(function(e){return void 0!==e[0]});if(n.length>0)for(var i=0;i<n.length-1;i++)void 0!==n[i][e]&&void 0!==n[i+1][e]&&n[i][e]!==n[i+1][e]&&t.push("unEqual");return 0===t.length}},{key:"isinitialSeriesSameLen",value:function(){for(var e=!0,t=this.w.globals.initialSeries,n=0;n<t.length-1;n++)if(t[n].data.length!==t[n+1].data.length){e=!1;break}return e}},{key:"getBarsHeight",value:function(e){return g(e).reduce(function(e,t){return e+t.getBBox().height},0)}},{key:"toggleAllTooltipSeriesGroups",value:function(e){var t=this.w,n=this.ttCtx;0===n.allTooltipSeriesGroups.length&&(n.allTooltipSeriesGroups=t.globals.dom.baseEl.querySelectorAll(".apexcharts-tooltip-series-group"));for(var i=n.allTooltipSeriesGroups,r=0;r<i.length;r++)"enable"===e?(i[r].classList.add("active"),i[r].style.display=t.config.tooltip.items.display):(i[r].classList.remove("active"),i[r].style.display="none")}}]),e}(),ge=function(){function e(t){o(this,e),this.w=t.w,this.ctx=t.ctx,this.ttCtx=t,this.tooltipUtil=new pe(t)}return l(e,[{key:"drawSeriesTexts",value:function(e){var t=e.shared,n=void 0===t||t,i=e.ttItems,r=e.i,a=void 0===r?0:r,o=e.j,s=void 0===o?null:o;void 0!==this.w.config.tooltip.custom?this.handleCustomTooltip({i:a,j:s}):this.toggleActiveInactiveSeries(n);var l=this.getValuesToPrint({i:a,j:s});this.printLabels({i:a,j:s,values:l,ttItems:i,shared:n});var c=this.ttCtx.getElTooltip();this.ttCtx.tooltipRect.ttWidth=c.getBoundingClientRect().width,this.ttCtx.tooltipRect.ttHeight=c.getBoundingClientRect().height}},{key:"printLabels",value:function(e){var t,n=e.i,i=e.j,r=e.values,a=e.ttItems,o=e.shared,s=this.w,l=r.xVal,c=r.zVal,u=r.xAxisTTVal,h="",d=s.globals.colors[n];null!==i&&s.config.plotOptions.bar.distributed&&(d=s.globals.colors[i]);for(var f=0,p=s.globals.series.length-1;f<s.globals.series.length;f++,p--){var g=this.getFormatters(n);if(h=this.getSeriesName({fn:g.yLbTitleFormatter,index:n,seriesIndex:n,j:i}),o){var m=s.config.tooltip.inverseOrder?p:f;g=this.getFormatters(m),h=this.getSeriesName({fn:g.yLbTitleFormatter,index:m,seriesIndex:n,j:i}),d=s.globals.colors[m],t=g.yLbFormatter(s.globals.series[m][i],{series:s.globals.series,seriesIndex:m,dataPointIndex:i,w:s}),(this.ttCtx.hasBars()&&s.config.chart.stacked&&0===s.globals.series[m][i]||void 0===s.globals.series[m][i])&&(t=void 0)}else t=g.yLbFormatter(s.globals.series[n][i],{series:s.globals.series,seriesIndex:n,dataPointIndex:i,w:s});null===i&&(t=g.yLbFormatter(s.globals.series[n],s)),this.DOMHandling({t:f,ttItems:a,values:{val:t,xVal:l,xAxisTTVal:u,zVal:c},seriesName:h,shared:o,pColor:d})}}},{key:"getFormatters",value:function(e){var t,n=this.w,i=n.globals.yLabelFormatters[e];return void 0!==n.globals.ttVal?Array.isArray(n.globals.ttVal)?(i=n.globals.ttVal[e]&&n.globals.ttVal[e].formatter,t=n.globals.ttVal[e]&&n.globals.ttVal[e].title&&n.globals.ttVal[e].title.formatter):(i=n.globals.ttVal.formatter,"function"==typeof n.globals.ttVal.title.formatter&&(t=n.globals.ttVal.title.formatter)):t=n.config.tooltip.y.title.formatter,"function"!=typeof i&&(i=n.globals.yLabelFormatters[0]?n.globals.yLabelFormatters[0]:function(e){return e}),"function"!=typeof t&&(t=function(e){return e}),{yLbFormatter:i,yLbTitleFormatter:t}}},{key:"getSeriesName",value:function(e){var t=e.fn,n=e.index,i=e.seriesIndex,r=e.j,a=this.w;return t(String(a.globals.seriesNames[n]),{series:a.globals.series,seriesIndex:i,dataPointIndex:r,w:a})}},{key:"DOMHandling",value:function(e){var t=e.t,n=e.ttItems,i=e.values,r=e.seriesName,a=e.shared,o=e.pColor,s=this.w,l=this.ttCtx,c=i.val,u=i.xVal,h=i.xAxisTTVal,d=i.zVal,f=null;f=n[t].children,s.config.tooltip.fillSeriesColor&&(n[t].style.backgroundColor=o,f[0].style.display="none"),l.showTooltipTitle&&(null===l.tooltipTitle&&(l.tooltipTitle=s.globals.dom.baseEl.querySelector(".apexcharts-tooltip-title")),l.tooltipTitle.innerHTML=u),l.blxaxisTooltip&&(l.xaxisTooltipText.innerHTML=""!==h?h:u);var p=n[t].querySelector(".apexcharts-tooltip-text-label");p&&(p.innerHTML=r?r+": ":"");var g=n[t].querySelector(".apexcharts-tooltip-text-value");(g&&(g.innerHTML=c),f[0]&&f[0].classList.contains("apexcharts-tooltip-marker")&&(f[0].style.backgroundColor=o),s.config.tooltip.marker.show||(f[0].style.display="none"),null!==d)&&(n[t].querySelector(".apexcharts-tooltip-text-z-label").innerHTML=s.config.tooltip.z.title,n[t].querySelector(".apexcharts-tooltip-text-z-value").innerHTML=d);a&&f[0]&&(null==c||s.globals.collapsedSeriesIndices.indexOf(t)>-1?f[0].parentNode.style.display="none":f[0].parentNode.style.display=s.config.tooltip.items.display)}},{key:"toggleActiveInactiveSeries",value:function(e){var t=this.w;if(e)this.tooltipUtil.toggleAllTooltipSeriesGroups("enable");else{this.tooltipUtil.toggleAllTooltipSeriesGroups("disable");var n=t.globals.dom.baseEl.querySelector(".apexcharts-tooltip-series-group");n&&(n.classList.add("active"),n.style.display=t.config.tooltip.items.display)}}},{key:"getValuesToPrint",value:function(e){var t=e.i,n=e.j,i=this.w,r=this.ctx.series.filteredSeriesX(),a="",o=null,s=null,l={series:i.globals.series,seriesIndex:t,dataPointIndex:n,w:i},c=i.globals.ttZFormatter;null===n?s=i.globals.series[t]:i.globals.isXNumeric?(a=r[t][n],0===r[t].length&&(a=r[this.tooltipUtil.getFirstActiveXArray(r)][n])):a=void 0!==i.globals.labels[n]?i.globals.labels[n]:"";var u=a;i.globals.isXNumeric&&"datetime"===i.config.xaxis.type?a=new X(this.ctx).xLabelFormat(i.globals.ttKeyFormatter,u):a=i.globals.xLabelFormatter(u,l);return void 0!==i.config.tooltip.x.formatter&&(a=i.globals.ttKeyFormatter(u,l)),i.globals.seriesZ.length>0&&i.globals.seriesZ[0].length>0&&(o=c(i.globals.seriesZ[t][n],i)),{val:s,xVal:a,xAxisTTVal:"function"==typeof i.config.xaxis.tooltip.formatter?i.globals.xaxisTooltipFormatter(u,l):a,zVal:o}}},{key:"handleCustomTooltip",value:function(e){var t=e.i,n=e.j,i=this.w;this.ttCtx.getElTooltip().innerHTML=i.config.tooltip.custom({ctx:this.ctx,series:i.globals.series,seriesIndex:t,dataPointIndex:n,w:i})}}]),e}(),ve=function(){function e(t){o(this,e),this.ttCtx=t,this.ctx=t.ctx,this.w=t.w}return l(e,[{key:"moveXCrosshairs",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:null,n=this.ttCtx,i=this.w,r=n.getElXCrosshairs(),a=e-n.xcrosshairsWidth/2,o=i.globals.labels.slice().length;if(null!==t&&(a=i.globals.gridWidth/o*t),"tickWidth"===i.config.xaxis.crosshairs.width||"barWidth"===i.config.xaxis.crosshairs.width?a+n.xcrosshairsWidth>i.globals.gridWidth&&(a=i.globals.gridWidth-n.xcrosshairsWidth):null!==t&&(a+=i.globals.gridWidth/o/2),a<0&&(a=0),a>i.globals.gridWidth&&(a=i.globals.gridWidth),null!==r&&(r.setAttribute("x",a),r.setAttribute("x1",a),r.setAttribute("x2",a),r.setAttribute("y2",i.globals.gridHeight),r.classList.add("active")),n.blxaxisTooltip){var s=a;"tickWidth"!==i.config.xaxis.crosshairs.width&&"barWidth"!==i.config.xaxis.crosshairs.width||(s=a+n.xcrosshairsWidth/2),this.moveXAxisTooltip(s)}}},{key:"moveYCrosshairs",value:function(e){var t=this.ttCtx;null!==t.ycrosshairs&&(b.setAttrs(t.ycrosshairs,{y1:e,y2:e}),b.setAttrs(t.ycrosshairsHidden,{y1:e,y2:e}))}},{key:"moveXAxisTooltip",value:function(e){var t=this.w,n=this.ttCtx;if(null!==n.xaxisTooltip){n.xaxisTooltip.classList.add("active");var i=n.xaxisOffY+t.config.xaxis.tooltip.offsetY+t.globals.translateY+1+t.config.xaxis.offsetY;if(e-=n.xaxisTooltip.getBoundingClientRect().width/2,!isNaN(e)){e+=t.globals.translateX;var r;r=new b(this.ctx).getTextRects(n.xaxisTooltipText.innerHTML),n.xaxisTooltipText.style.minWidth=r.width+"px",n.xaxisTooltip.style.left=e+"px",n.xaxisTooltip.style.top=i+"px"}}}},{key:"moveYAxisTooltip",value:function(e){var t=this.w,n=this.ttCtx;null===n.yaxisTTEls&&(n.yaxisTTEls=t.globals.dom.baseEl.querySelectorAll(".apexcharts-yaxistooltip"));var i=parseInt(n.ycrosshairsHidden.getAttribute("y1")),r=t.globals.translateY+i,a=n.yaxisTTEls[e].getBoundingClientRect().height,o=t.globals.translateYAxisX[e]-2;t.config.yaxis[e].opposite&&(o-=26),r-=a/2,-1===t.globals.ignoreYAxisIndexes.indexOf(e)?(n.yaxisTTEls[e].classList.add("active"),n.yaxisTTEls[e].style.top=r+"px",n.yaxisTTEls[e].style.left=o+t.config.yaxis[e].tooltip.offsetX+"px"):n.yaxisTTEls[e].classList.remove("active")}},{key:"moveTooltip",value:function(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:null,i=this.w,r=this.ttCtx,a=r.getElTooltip(),o=r.tooltipRect,s=null!==n?parseInt(n):1,l=parseInt(e)+s+5,c=parseInt(t)+s/2;if(l>i.globals.gridWidth/2&&(l=l-o.ttWidth-s-15),l>i.globals.gridWidth-o.ttWidth-10&&(l=i.globals.gridWidth-o.ttWidth),l<-20&&(l=-20),i.config.tooltip.followCursor){var u=r.getElGrid().getBoundingClientRect();c=r.e.clientY+i.globals.translateY-u.top-o.ttHeight/2}var h=this.positionChecks(o,l,c);l=h.x,c=h.y,isNaN(l)||(l+=i.globals.translateX,a.style.left=l+"px",a.style.top=c+"px")}},{key:"positionChecks",value:function(e,t,n){var i=this.w;return e.ttHeight+n>i.globals.gridHeight&&(n=i.globals.gridHeight-e.ttHeight+i.globals.translateY),n<0&&(n=0),{x:t,y:n}}},{key:"moveMarkers",value:function(e,t){var n=this.w,i=this.ttCtx;if(n.globals.markers.size[e]>0)for(var r=n.globals.dom.baseEl.querySelectorAll(" .apexcharts-series[data\\:realIndex='".concat(e,"'] .apexcharts-marker")),a=0;a<r.length;a++)parseInt(r[a].getAttribute("rel"))===t&&(i.marker.resetPointsSize(),i.marker.enlargeCurrentPoint(t,r[a]));else i.marker.resetPointsSize(),this.moveDynamicPointOnHover(t,e)}},{key:"moveDynamicPointOnHover",value:function(e,t){var n,i,r=this.w,a=this.ttCtx,o=r.globals.pointsArray,s=r.config.markers.hover.size;void 0===s&&(s=r.globals.markers.size[t]+r.config.markers.hover.sizeOffset),n=o[t][e][0],i=o[t][e][1]?o[t][e][1]:0;var l=r.globals.dom.baseEl.querySelector(".apexcharts-series[data\\:realIndex='".concat(t,"'] .apexcharts-series-markers circle"));l&&(l.setAttribute("r",s),l.setAttribute("cx",n),l.setAttribute("cy",i)),this.moveXCrosshairs(n),a.fixedTooltip||this.moveTooltip(n,i,s)}},{key:"moveDynamicPointsOnHover",value:function(e){var t,n=this.ttCtx,i=n.w,r=0,a=0,o=i.globals.pointsArray;t=new Q(this.ctx).getActiveSeriesIndex();var s=i.config.markers.hover.size;void 0===s&&(s=i.globals.markers.size[t]+i.config.markers.hover.sizeOffset),o[t]&&(r=o[t][e][0],a=o[t][e][1]);var l=null,c=n.getAllMarkers();if(null!==(l=null!==c?c:i.globals.dom.baseEl.querySelectorAll(".apexcharts-series-markers circle")))for(var u=0;u<l.length;u++){var h=o[u];if(h&&h.length){var d=o[u][e][1];l[u].setAttribute("cx",r);var f=parseInt(l[u].parentNode.parentNode.parentNode.getAttribute("data:realIndex"));null!==d?(l[f]&&l[f].setAttribute("r",s),l[f]&&l[f].setAttribute("cy",d)):l[f]&&l[f].setAttribute("r",0)}}if(this.moveXCrosshairs(r),!n.fixedTooltip){var p=a||i.globals.gridHeight;this.moveTooltip(r,p,s)}}},{key:"moveStickyTooltipOverBars",value:function(e){var t,n=this.w,i=this.ttCtx,r=n.globals.dom.baseEl.querySelector(".apexcharts-bar-series .apexcharts-series[rel='1'] path[j='".concat(e,"'], .apexcharts-candlestick-series .apexcharts-series[rel='1'] path[j='").concat(e,"'], .apexcharts-rangebar-series .apexcharts-series[rel='1'] path[j='").concat(e,"']")),a=r?parseFloat(r.getAttribute("cx")):0,o=r?parseFloat(r.getAttribute("barWidth")):0;n.globals.isXNumeric?a-=o/2:(a=i.xAxisTicksPositions[e-1]+i.dataPointsDividedWidth/2,isNaN(a)&&(a=i.xAxisTicksPositions[e]-i.dataPointsDividedWidth/2));var s=i.getElGrid().getBoundingClientRect();if(t=i.e.clientY-s.top-i.tooltipRect.ttHeight/2,this.moveXCrosshairs(a),!i.fixedTooltip){var l=t||n.globals.gridHeight;this.moveTooltip(a,l)}}}]),e}(),ye=function(){function e(t){o(this,e),this.w=t.w,this.ttCtx=t,this.ctx=t.ctx,this.tooltipPosition=new ve(t)}return l(e,[{key:"drawDynamicPoints",value:function(){for(var e=this.w,t=new b(this.ctx),n=new P(this.ctx),i=e.globals.dom.baseEl.querySelectorAll(".apexcharts-series"),r=0;r<i.length;r++){var a=parseInt(i[r].getAttribute("data:realIndex")),o=e.globals.dom.baseEl.querySelector(".apexcharts-series[data\\:realIndex='".concat(a,"'] .apexcharts-series-markers-wrap"));if(null!==o){var s=void 0,l="apexcharts-marker w".concat((Math.random()+1).toString(36).substring(4));"line"!==e.config.chart.type&&"area"!==e.config.chart.type||e.globals.comboCharts||e.config.tooltip.intersect||(l+=" no-pointer-events");var c=n.getMarkerConfig(l,a);(s=t.drawMarker(0,0,c)).node.setAttribute("default-marker-size",0);var u=document.createElementNS(e.globals.SVGNS,"g");u.classList.add("apexcharts-series-markers"),u.appendChild(s.node),o.appendChild(u)}}}},{key:"enlargeCurrentPoint",value:function(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:null,i=arguments.length>3&&void 0!==arguments[3]?arguments[3]:null,r=this.w;"bubble"!==r.config.chart.type&&this.newPointSize(e,t);var a=t.getAttribute("cx"),o=t.getAttribute("cy");if(null!==n&&null!==i&&(a=n,o=i),this.tooltipPosition.moveXCrosshairs(a),!this.fixedTooltip){if("radar"===r.config.chart.type){var s=this.ttCtx.getElGrid().getBoundingClientRect();a=this.ttCtx.e.clientX-s.left}this.tooltipPosition.moveTooltip(a,o,r.config.markers.hover.size)}}},{key:"enlargePoints",value:function(e){for(var t=this.w,n=this.ttCtx,i=e,r=t.globals.dom.baseEl.querySelectorAll(".apexcharts-series:not(.apexcharts-series-collapsed) .apexcharts-marker"),a=t.config.markers.hover.size,o=0;o<r.length;o++){var s=r[o].getAttribute("rel"),l=r[o].getAttribute("index");if(void 0===a&&(a=t.globals.markers.size[l]+t.config.markers.hover.sizeOffset),i===parseInt(s)){this.newPointSize(i,r[o]);var c=r[o].getAttribute("cx"),u=r[o].getAttribute("cy");this.tooltipPosition.moveXCrosshairs(c),n.fixedTooltip||this.tooltipPosition.moveTooltip(c,u,a)}else this.oldPointSize(r[o])}}},{key:"newPointSize",value:function(e,t){var n=this.w,i=n.config.markers.hover.size,r=null;r=0===e?t.parentNode.firstChild:t.parentNode.lastChild;var a=parseInt(r.getAttribute("index"));void 0===i&&(i=n.globals.markers.size[a]+n.config.markers.hover.sizeOffset),r.setAttribute("r",i)}},{key:"oldPointSize",value:function(e){var t=parseInt(e.getAttribute("default-marker-size"));e.setAttribute("r",t)}},{key:"resetPointsSize",value:function(){for(var e=this.w.globals.dom.baseEl.querySelectorAll(".apexcharts-series:not(.apexcharts-series-collapsed) .apexcharts-marker"),t=0;t<e.length;t++){var n=parseInt(e[t].getAttribute("default-marker-size"));m.isNumber(n)?e[t].setAttribute("r",n):e[t].setAttribute("r",0)}}}]),e}(),be=function(){function e(t){o(this,e),this.w=t.w,this.ttCtx=t}return l(e,[{key:"getAttr",value:function(e,t){return parseFloat(e.target.getAttribute(t))}},{key:"handleHeatTooltip",value:function(e){var t=e.e,n=e.opt,i=e.x,r=e.y,a=this.ttCtx,o=this.w;if(t.target.classList.contains("apexcharts-heatmap-rect")){var s=this.getAttr(t,"i"),l=this.getAttr(t,"j"),c=this.getAttr(t,"cx"),u=this.getAttr(t,"cy"),h=this.getAttr(t,"width"),d=this.getAttr(t,"height");if(a.tooltipLabels.drawSeriesTexts({ttItems:n.ttItems,i:s,j:l,shared:!1}),i=c+a.tooltipRect.ttWidth/2+h,r=u+a.tooltipRect.ttHeight/2-d/2,a.tooltipPosition.moveXCrosshairs(c+h/2),i>o.globals.gridWidth/2&&(i=c-a.tooltipRect.ttWidth/2+h),a.w.config.tooltip.followCursor){var f=a.getElGrid().getBoundingClientRect();r=a.e.clientY-f.top+o.globals.translateY/2-10}}return{x:i,y:r}}},{key:"handleMarkerTooltip",value:function(e){var t,n,i=e.e,r=e.opt,a=e.x,o=e.y,s=this.w,l=this.ttCtx;if(i.target.classList.contains("apexcharts-marker")){var c=parseInt(r.paths.getAttribute("cx")),u=parseInt(r.paths.getAttribute("cy")),h=parseFloat(r.paths.getAttribute("val"));if(n=parseInt(r.paths.getAttribute("rel")),t=parseInt(r.paths.parentNode.parentNode.parentNode.getAttribute("rel"))-1,l.intersect){var d=m.findAncestor(r.paths,"apexcharts-series");d&&(t=parseInt(d.getAttribute("data:realIndex")))}if(l.tooltipLabels.drawSeriesTexts({ttItems:r.ttItems,i:t,j:n,shared:!l.showOnIntersect&&s.config.tooltip.shared}),"mouseup"===i.type&&l.markerClick(i,t,n),a=c,o=u+s.globals.translateY-1.4*l.tooltipRect.ttHeight,l.w.config.tooltip.followCursor){var f=l.getElGrid().getBoundingClientRect();o=l.e.clientY+s.globals.translateY-f.top}h<0&&(o=u),l.marker.enlargeCurrentPoint(n,r.paths,a,o)}return{x:a,y:o}}},{key:"handleBarTooltip",value:function(e){var t,n,i=e.e,r=e.opt,a=this.w,o=this.ttCtx,s=o.getElTooltip(),l=0,c=0,u=0,h=this.getBarTooltipXY({e:i,opt:r});t=h.i;var d=h.barHeight,f=h.j;if(a.globals.isBarHorizontal&&o.hasBars()||!a.config.tooltip.shared?(c=h.x,u=h.y,n=Array.isArray(a.config.stroke.width)?a.config.stroke.width[t]:a.config.stroke.width,l=c):a.globals.comboCharts||a.config.tooltip.shared||(l/=2),isNaN(u)&&(u=a.globals.svgHeight-o.tooltipRect.ttHeight),c+o.tooltipRect.ttWidth>a.globals.gridWidth?c-=o.tooltipRect.ttWidth:c<0&&(c+=o.tooltipRect.ttWidth),o.w.config.tooltip.followCursor){var p=o.getElGrid().getBoundingClientRect();u=o.e.clientY-p.top}if(null===o.tooltip&&(o.tooltip=a.globals.dom.baseEl.querySelector(".apexcharts-tooltip")),a.config.tooltip.shared||(a.globals.comboChartsHasBars?o.tooltipPosition.moveXCrosshairs(l+n/2):o.tooltipPosition.moveXCrosshairs(l)),!o.fixedTooltip&&(!a.config.tooltip.shared||a.globals.isBarHorizontal&&o.hasBars())){m&&(c=a.globals.gridWidth-c),s.style.left=c+a.globals.translateX+"px";var g=parseInt(r.paths.parentNode.getAttribute("data:realIndex")),m=a.globals.isMultipleYAxis?a.config.yaxis[g]&&a.config.yaxis[g].reversed:a.config.yaxis[0].reversed;!m||a.globals.isBarHorizontal&&o.hasBars()||(u=u+d-2*(a.globals.series[t][f]<0?d:0)),o.tooltipRect.ttHeight+u>a.globals.gridHeight?(u=a.globals.gridHeight-o.tooltipRect.ttHeight+a.globals.translateY,s.style.top=u+"px"):s.style.top=u+a.globals.translateY-o.tooltipRect.ttHeight/2+"px"}}},{key:"getBarTooltipXY",value:function(e){var t=e.e,n=e.opt,i=this.w,r=null,a=this.ttCtx,o=0,s=0,l=0,c=0,u=0,h=t.target.classList;if(h.contains("apexcharts-bar-area")||h.contains("apexcharts-candlestick-area")||h.contains("apexcharts-rangebar-area")){var d=t.target,f=d.getBoundingClientRect(),p=n.elGrid.getBoundingClientRect(),g=f.height;u=f.height;var m=f.width,v=parseInt(d.getAttribute("cx")),y=parseInt(d.getAttribute("cy"));c=parseFloat(d.getAttribute("barWidth"));var b="touchmove"===t.type?t.touches[0].clientX:t.clientX;r=parseInt(d.getAttribute("j")),o=parseInt(d.parentNode.getAttribute("rel"))-1,i.globals.comboCharts&&(o=parseInt(d.parentNode.getAttribute("data:realIndex"))),a.tooltipLabels.drawSeriesTexts({ttItems:n.ttItems,i:o,j:r,shared:!a.showOnIntersect&&i.config.tooltip.shared}),i.config.tooltip.followCursor?i.globals.isBarHorizontal?(s=b-p.left+15,l=y-a.dataPointsDividedHeight+g/2-a.tooltipRect.ttHeight/2):(s=i.globals.isXNumeric?v-m/2:v-a.dataPointsDividedWidth+m/2,l=t.clientY-p.top-a.tooltipRect.ttHeight/2-15):i.globals.isBarHorizontal?((s=v)<a.xyRatios.baseLineInvertedY&&(s=v-a.tooltipRect.ttWidth),l=y-a.dataPointsDividedHeight+g/2-a.tooltipRect.ttHeight/2):(s=i.globals.isXNumeric?v-m/2:v-a.dataPointsDividedWidth+m/2,l=y)}return{x:s,y:l,barHeight:u,barWidth:c,i:o,j:r}}}]),e}(),xe=function(){function e(t){o(this,e),this.w=t.w,this.ttCtx=t}return l(e,[{key:"drawXaxisTooltip",value:function(){var e=this.w,t=this.ttCtx,n="bottom"===e.config.xaxis.position;t.xaxisOffY=n?e.globals.gridHeight+1:1;var i=n?"apexcharts-xaxistooltip apexcharts-xaxistooltip-bottom":"apexcharts-xaxistooltip apexcharts-xaxistooltip-top",r=e.globals.dom.elWrap;t.blxaxisTooltip&&(null===e.globals.dom.baseEl.querySelector(".apexcharts-xaxistooltip")&&(t.xaxisTooltip=document.createElement("div"),t.xaxisTooltip.setAttribute("class",i+" "+e.config.tooltip.theme),r.appendChild(t.xaxisTooltip),t.xaxisTooltipText=document.createElement("div"),t.xaxisTooltipText.classList.add("apexcharts-xaxistooltip-text"),t.xaxisTooltipText.style.fontFamily=e.config.xaxis.tooltip.style.fontFamily||e.config.chart.fontFamily,t.xaxisTooltipText.style.fontSize=e.config.xaxis.tooltip.style.fontSize,t.xaxisTooltip.appendChild(t.xaxisTooltipText)))}},{key:"drawYaxisTooltip",value:function(){for(var e=this.w,t=this.ttCtx,n=0;n<e.config.yaxis.length;n++){var i=e.config.yaxis[n].opposite||e.config.yaxis[n].crosshairs.opposite;t.yaxisOffX=i?e.globals.gridWidth+1:1;var r="apexcharts-yaxistooltip apexcharts-yaxistooltip-".concat(n,i?" apexcharts-yaxistooltip-right":" apexcharts-yaxistooltip-left"),a=e.globals.dom.elWrap;if(t.blyaxisTooltip)null===e.globals.dom.baseEl.querySelector(".apexcharts-yaxistooltip apexcharts-yaxistooltip-".concat(n))&&(t.yaxisTooltip=document.createElement("div"),t.yaxisTooltip.setAttribute("class",r+" "+e.config.tooltip.theme),a.appendChild(t.yaxisTooltip),0===n&&(t.yaxisTooltipText=[]),t.yaxisTooltipText.push(document.createElement("div")),t.yaxisTooltipText[n].classList.add("apexcharts-yaxistooltip-text"),t.yaxisTooltip.appendChild(t.yaxisTooltipText[n]))}}},{key:"setXCrosshairWidth",value:function(){var e=this.w,t=this.ttCtx,n=t.getElXCrosshairs();if(t.xcrosshairsWidth=parseInt(e.config.xaxis.crosshairs.width),e.globals.comboCharts){var i=e.globals.dom.baseEl.querySelector(".apexcharts-bar-area");if(null!==i&&"barWidth"===e.config.xaxis.crosshairs.width){var r=parseFloat(i.getAttribute("barWidth"));t.xcrosshairsWidth=r}else if("tickWidth"===e.config.xaxis.crosshairs.width){var a=e.globals.labels.length;t.xcrosshairsWidth=e.globals.gridWidth/a}}else if("tickWidth"===e.config.xaxis.crosshairs.width){var o=e.globals.labels.length;t.xcrosshairsWidth=e.globals.gridWidth/o}else if("barWidth"===e.config.xaxis.crosshairs.width){var s=e.globals.dom.baseEl.querySelector(".apexcharts-bar-area");if(null!==s){var l=parseFloat(s.getAttribute("barWidth"));t.xcrosshairsWidth=l}else t.xcrosshairsWidth=1}e.globals.isBarHorizontal&&(t.xcrosshairsWidth=0),null!==n&&t.xcrosshairsWidth>0&&n.setAttribute("width",t.xcrosshairsWidth)}},{key:"handleYCrosshair",value:function(){var e=this.w,t=this.ttCtx;t.ycrosshairs=e.globals.dom.baseEl.querySelector(".apexcharts-ycrosshairs"),t.ycrosshairsHidden=e.globals.dom.baseEl.querySelector(".apexcharts-ycrosshairs-hidden")}},{key:"drawYaxisTooltipText",value:function(e,t,n){var i=this.ttCtx,r=this.w,a=r.globals.yLabelFormatters[e];if(i.blyaxisTooltip){var o=i.getElGrid().getBoundingClientRect(),s=(t-o.top)*n.yRatio[e],l=r.globals.maxYArr[e]-r.globals.minYArr[e],c=r.globals.minYArr[e]+(l-s);i.tooltipPosition.moveYCrosshairs(t-o.top),i.yaxisTooltipText[e].innerHTML=a(c),i.tooltipPosition.moveYAxisTooltip(e)}}}]),e}(),we=function(){function e(t){o(this,e),this.ctx=t,this.w=t.w;var n=this.w;this.tooltipUtil=new pe(this),this.tooltipLabels=new ge(this),this.tooltipPosition=new ve(this),this.marker=new ye(this),this.intersect=new be(this),this.axesTooltip=new xe(this),this.showOnIntersect=n.config.tooltip.intersect,this.showTooltipTitle=n.config.tooltip.x.show,this.fixedTooltip=n.config.tooltip.fixed.enabled,this.xaxisTooltip=null,this.yaxisTTEls=null,this.isBarShared=!n.globals.isBarHorizontal&&n.config.tooltip.shared}return l(e,[{key:"getElTooltip",value:function(e){return e||(e=this),e.w.globals.dom.baseEl.querySelector(".apexcharts-tooltip")}},{key:"getElXCrosshairs",value:function(){return this.w.globals.dom.baseEl.querySelector(".apexcharts-xcrosshairs")}},{key:"getElGrid",value:function(){return this.w.globals.dom.baseEl.querySelector(".apexcharts-grid")}},{key:"drawTooltip",value:function(e){var t=this.w;this.xyRatios=e,this.blxaxisTooltip=t.config.xaxis.tooltip.enabled&&t.globals.axisCharts,this.blyaxisTooltip=t.config.yaxis[0].tooltip.enabled&&t.globals.axisCharts,this.allTooltipSeriesGroups=[],t.globals.axisCharts||(this.showTooltipTitle=!1);var n=document.createElement("div");if(n.classList.add("apexcharts-tooltip"),n.classList.add(t.config.tooltip.theme),t.globals.dom.elWrap.appendChild(n),t.globals.axisCharts){this.axesTooltip.drawXaxisTooltip(),this.axesTooltip.drawYaxisTooltip(),this.axesTooltip.setXCrosshairWidth(),this.axesTooltip.handleYCrosshair();var i=new B(this.ctx);this.xAxisTicksPositions=i.getXAxisTicksPositions()}if((t.globals.comboCharts&&!t.config.tooltip.shared||t.config.tooltip.intersect&&!t.config.tooltip.shared||("bar"===t.config.chart.type||"rangeBar"===t.config.chart.type)&&!t.config.tooltip.shared)&&(this.showOnIntersect=!0),0!==t.config.markers.size&&0!==t.globals.markers.largestSize||this.marker.drawDynamicPoints(this),t.globals.collapsedSeries.length!==t.globals.series.length){this.dataPointsDividedHeight=t.globals.gridHeight/t.globals.dataPoints,this.dataPointsDividedWidth=t.globals.gridWidth/t.globals.dataPoints,this.showTooltipTitle&&(this.tooltipTitle=document.createElement("div"),this.tooltipTitle.classList.add("apexcharts-tooltip-title"),this.tooltipTitle.style.fontFamily=t.config.tooltip.style.fontFamily||t.config.chart.fontFamily,this.tooltipTitle.style.fontSize=t.config.tooltip.style.fontSize,n.appendChild(this.tooltipTitle));var r=t.globals.series.length;(t.globals.xyCharts||t.globals.comboCharts)&&t.config.tooltip.shared&&(r=this.showOnIntersect?1:t.globals.series.length),this.ttItems=this.createTTElements(r),this.addSVGEvents()}}},{key:"createTTElements",value:function(e){for(var t=this.w,n=[],i=this.getElTooltip(),r=0;r<e;r++){var a=document.createElement("div");a.classList.add("apexcharts-tooltip-series-group");var o=document.createElement("span");o.classList.add("apexcharts-tooltip-marker"),o.style.backgroundColor=t.globals.colors[r],a.appendChild(o);var s=document.createElement("div");s.classList.add("apexcharts-tooltip-text"),s.style.fontFamily=t.config.tooltip.style.fontFamily||t.config.chart.fontFamily,s.style.fontSize=t.config.tooltip.style.fontSize;var l=document.createElement("div");l.classList.add("apexcharts-tooltip-y-group");var c=document.createElement("span");c.classList.add("apexcharts-tooltip-text-label"),l.appendChild(c);var u=document.createElement("span");u.classList.add("apexcharts-tooltip-text-value"),l.appendChild(u);var h=document.createElement("div");h.classList.add("apexcharts-tooltip-z-group");var d=document.createElement("span");d.classList.add("apexcharts-tooltip-text-z-label"),h.appendChild(d);var f=document.createElement("span");f.classList.add("apexcharts-tooltip-text-z-value"),h.appendChild(f),s.appendChild(l),s.appendChild(h),a.appendChild(s),i.appendChild(a),n.push(a)}return n}},{key:"addSVGEvents",value:function(){var e=this.w,t=e.config.chart.type,n=this.getElTooltip(),i=!("bar"!==t&&"candlestick"!==t&&"rangeBar"!==t),r=e.globals.dom.Paper.node,a=this.getElGrid();a&&(this.seriesBound=a.getBoundingClientRect());var o,s=[],l=[],c={hoverArea:r,elGrid:a,tooltipEl:n,tooltipY:s,tooltipX:l,ttItems:this.ttItems};if(e.globals.axisCharts&&("area"===t||"line"===t||"scatter"===t||"bubble"===t?o=e.globals.dom.baseEl.querySelectorAll(".apexcharts-series[data\\:longestSeries='true'] .apexcharts-marker"):i?o=e.globals.dom.baseEl.querySelectorAll(".apexcharts-series .apexcharts-bar-area, .apexcharts-series .apexcharts-candlestick-area, .apexcharts-series .apexcharts-rangebar-area"):"heatmap"===t?o=e.globals.dom.baseEl.querySelectorAll(".apexcharts-series .apexcharts-heatmap"):"radar"===t&&(o=e.globals.dom.baseEl.querySelectorAll(".apexcharts-series .apexcharts-marker")),o&&o.length))for(var u=0;u<o.length;u++)s.push(o[u].getAttribute("cy")),l.push(o[u].getAttribute("cx"));if(e.globals.xyCharts&&!this.showOnIntersect||e.globals.comboCharts&&!this.showOnIntersect||i&&this.hasBars()&&e.config.tooltip.shared)this.addPathsEventListeners([r],c);else if(i&&!e.globals.comboCharts)this.addBarsEventListeners(c);else if("bubble"===t||"scatter"===t||"radar"===t||this.showOnIntersect&&("area"===t||"line"===t))this.addPointsEventsListeners(c);else if(!e.globals.axisCharts||"heatmap"===t){var h=e.globals.dom.baseEl.querySelectorAll(".apexcharts-series");this.addPathsEventListeners(h,c)}if(this.showOnIntersect){var d=e.globals.dom.baseEl.querySelectorAll(".apexcharts-line-series .apexcharts-marker");d.length>0&&this.addPathsEventListeners(d,c);var f=e.globals.dom.baseEl.querySelectorAll(".apexcharts-area-series .apexcharts-marker");f.length>0&&this.addPathsEventListeners(f,c),this.hasBars()&&!e.config.tooltip.shared&&this.addBarsEventListeners(c)}}},{key:"drawFixedTooltipRect",value:function(){var e=this.w,t=this.getElTooltip(),n=t.getBoundingClientRect(),i=n.width+10,r=n.height+10,a=e.config.tooltip.fixed.offsetX,o=e.config.tooltip.fixed.offsetY;return e.config.tooltip.fixed.position.toLowerCase().indexOf("right")>-1&&(a=a+e.globals.svgWidth-i+10),e.config.tooltip.fixed.position.toLowerCase().indexOf("bottom")>-1&&(o=o+e.globals.svgHeight-r-10),t.style.left=a+"px",t.style.top=o+"px",{x:a,y:o,ttWidth:i,ttHeight:r}}},{key:"addPointsEventsListeners",value:function(e){var t=this.w.globals.dom.baseEl.querySelectorAll(".apexcharts-series-markers .apexcharts-marker");this.addPathsEventListeners(t,e)}},{key:"addBarsEventListeners",value:function(e){var t=this.w.globals.dom.baseEl.querySelectorAll(".apexcharts-bar-area, .apexcharts-candlestick-area, .apexcharts-rangebar-area");this.addPathsEventListeners(t,e)}},{key:"addPathsEventListeners",value:function(e,t){for(var n=this,i=this,r=function(r){var a={paths:e[r],tooltipEl:t.tooltipEl,tooltipY:t.tooltipY,tooltipX:t.tooltipX,elGrid:t.elGrid,hoverArea:t.hoverArea,ttItems:t.ttItems};n.w.globals.tooltipOpts=a;["mousemove","mouseup","touchmove","mouseout","touchend"].map(function(t){return e[r].addEventListener(t,i.seriesHover.bind(i,a),{capture:!1,passive:!0})})},a=0;a<e.length;a++)r(a)}},{key:"seriesHover",value:function(e,t){var n=this,i=[],r=this.w;r.config.chart.group&&(i=this.ctx.getGroupedCharts()),r.globals.axisCharts&&(r.globals.minX===-1/0&&r.globals.maxX===1/0||0===r.globals.dataPoints)||(i.length?i.forEach(function(i){var r=n.getElTooltip(i),a={paths:e.paths,tooltipEl:r,tooltipY:e.tooltipY,tooltipX:e.tooltipX,elGrid:e.elGrid,hoverArea:e.hoverArea,ttItems:i.w.globals.tooltip.ttItems};i.w.globals.minX===n.w.globals.minX&&i.w.globals.maxX===n.w.globals.maxX&&i.w.globals.tooltip.seriesHoverByContext({chartCtx:i,ttCtx:i.w.globals.tooltip,opt:a,e:t})}):this.seriesHoverByContext({chartCtx:this.ctx,ttCtx:this.w.globals.tooltip,opt:e,e:t}))}},{key:"seriesHoverByContext",value:function(e){var t=e.chartCtx,n=e.ttCtx,i=e.opt,r=e.e,a=t.w,o=this.getElTooltip();(n.tooltipRect={x:0,y:0,ttWidth:o.getBoundingClientRect().width,ttHeight:o.getBoundingClientRect().height},n.e=r,!n.hasBars()||a.globals.comboCharts||n.isBarShared)||a.config.tooltip.onDatasetHover.highlightDataSeries&&new Q(t).toggleSeriesOnHover(r,r.target.parentNode);n.fixedTooltip&&n.drawFixedTooltipRect(),a.globals.axisCharts?n.axisChartsTooltips({e:r,opt:i,tooltipRect:n.tooltipRect}):n.nonAxisChartsTooltips({e:r,opt:i,tooltipRect:n.tooltipRect})}},{key:"axisChartsTooltips",value:function(e){var t,n,i,r=e.e,a=e.opt,o=this.w,s=null,l=a.elGrid.getBoundingClientRect(),c="touchmove"===r.type?r.touches[0].clientX:r.clientX,u="touchmove"===r.type?r.touches[0].clientY:r.clientY;if(this.clientY=u,this.clientX=c,u<l.top||u>l.top+l.height)this.handleMouseOut(a);else{var h=this.getElTooltip(),d=this.getElXCrosshairs(),f=o.globals.xyCharts||"bar"===o.config.chart.type&&!o.globals.isBarHorizontal&&this.hasBars()&&o.config.tooltip.shared||o.globals.comboCharts&&this.hasBars;if(o.globals.isBarHorizontal&&this.hasBars()&&(f=!1),"mousemove"===r.type||"touchmove"===r.type||"mouseup"===r.type){if(null!==d&&d.classList.add("active"),null!==this.ycrosshairs&&this.blyaxisTooltip&&this.ycrosshairs.classList.add("active"),f&&!this.showOnIntersect){t=(s=this.tooltipUtil.getNearestValues({context:this,hoverArea:a.hoverArea,elGrid:a.elGrid,clientX:c,clientY:u,hasBars:this.hasBars})).j;var p=s.capturedSeries;if(s.hoverX<0||s.hoverX>o.globals.gridWidth)return void this.handleMouseOut(a);if(null!==p){if(null===o.globals.series[p][t])return void a.tooltipEl.classList.remove("active");void 0!==o.globals.series[p][t]?o.config.tooltip.shared&&this.tooltipUtil.isXoverlap(t)&&this.tooltipUtil.isinitialSeriesSameLen()?this.create(r,this,p,t,a.ttItems):this.create(r,this,p,t,a.ttItems,!1):this.tooltipUtil.isXoverlap(t)&&this.create(r,this,0,t,a.ttItems)}else this.tooltipUtil.isXoverlap(t)&&this.create(r,this,0,t,a.ttItems)}else if("heatmap"===o.config.chart.type){var g=this.intersect.handleHeatTooltip({e:r,opt:a,x:n,y:i});n=g.x,i=g.y,h.style.left=n+"px",h.style.top=i+"px"}else this.hasBars&&this.intersect.handleBarTooltip({e:r,opt:a}),this.hasMarkers&&this.intersect.handleMarkerTooltip({e:r,opt:a,x:n,y:i});if(this.blyaxisTooltip)for(var m=0;m<o.config.yaxis.length;m++)this.axesTooltip.drawYaxisTooltipText(m,u,this.xyRatios);a.tooltipEl.classList.add("active")}else"mouseout"!==r.type&&"touchend"!==r.type||this.handleMouseOut(a)}}},{key:"nonAxisChartsTooltips",value:function(e){var t=e.e,n=e.opt,i=e.tooltipRect,r=this.w,a=n.paths.getAttribute("rel"),o=this.getElTooltip(),s=r.globals.dom.elWrap.getBoundingClientRect();if("mousemove"===t.type||"touchmove"===t.type){o.classList.add("active"),this.tooltipLabels.drawSeriesTexts({ttItems:n.ttItems,i:parseInt(a)-1,shared:!1});var l=r.globals.clientX-s.left-i.ttWidth/2,c=r.globals.clientY-s.top-i.ttHeight-10;o.style.left=l+"px",o.style.top=c+"px"}else"mouseout"!==t.type&&"touchend"!==t.type||o.classList.remove("active")}},{key:"deactivateHoverFilter",value:function(){for(var e=this.w,t=new b(this.ctx),n=e.globals.dom.Paper.select(".apexcharts-bar-area"),i=0;i<n.length;i++)t.pathMouseLeave(n[i])}},{key:"handleMouseOut",value:function(e){var t=this.w,n=this.getElXCrosshairs();if(e.tooltipEl.classList.remove("active"),this.deactivateHoverFilter(),"bubble"!==t.config.chart.type&&this.marker.resetPointsSize(),null!==n&&n.classList.remove("active"),null!==this.ycrosshairs&&this.ycrosshairs.classList.remove("active"),this.blxaxisTooltip&&this.xaxisTooltip.classList.remove("active"),this.blyaxisTooltip){null===this.yaxisTTEls&&(this.yaxisTTEls=t.globals.dom.baseEl.querySelectorAll(".apexcharts-yaxistooltip"));for(var i=0;i<this.yaxisTTEls.length;i++)this.yaxisTTEls[i].classList.remove("active")}}},{key:"getElMarkers",value:function(){return this.w.globals.dom.baseEl.querySelectorAll(" .apexcharts-series-markers")}},{key:"getAllMarkers",value:function(){return this.w.globals.dom.baseEl.querySelectorAll(".apexcharts-series-markers .apexcharts-marker")}},{key:"hasMarkers",value:function(){return this.getElMarkers().length>0}},{key:"getElBars",value:function(){return this.w.globals.dom.baseEl.querySelectorAll(".apexcharts-bar-series,  .apexcharts-candlestick-series, .apexcharts-rangebar-series")}},{key:"hasBars",value:function(){return this.getElBars().length>0}},{key:"markerClick",value:function(e,t,n){var i=this.w;"function"==typeof i.config.chart.events.markerClick&&i.config.chart.events.markerClick(e,this.ctx,{seriesIndex:t,dataPointIndex:n,w:i}),this.ctx.fireEvent("markerClick",[e,this.ctx,{seriesIndex:t,dataPointIndex:n,w:i}])}},{key:"create",value:function(e,t,n,i,r){var a=arguments.length>5&&void 0!==arguments[5]?arguments[5]:null,o=this.w,s=t;"mouseup"===e.type&&this.markerClick(e,n,i),null===a&&(a=o.config.tooltip.shared);var l=this.hasMarkers(),c=this.getElBars();if(a){if(s.tooltipLabels.drawSeriesTexts({ttItems:r,i:n,j:i,shared:!this.showOnIntersect&&o.config.tooltip.shared}),l&&(o.globals.markers.largestSize>0?s.marker.enlargePoints(i):s.tooltipPosition.moveDynamicPointsOnHover(i)),this.hasBars()&&(this.barSeriesHeight=this.tooltipUtil.getBarsHeight(c),this.barSeriesHeight>0)){var u=new b(this.ctx),h=o.globals.dom.Paper.select(".apexcharts-bar-area[j='".concat(i,"']"));this.deactivateHoverFilter(),this.tooltipPosition.moveStickyTooltipOverBars(i);for(var d=0;d<h.length;d++)u.pathMouseEnter(h[d])}}else s.tooltipLabels.drawSeriesTexts({shared:!1,ttItems:r,i:n,j:i}),this.hasBars()&&s.tooltipPosition.moveStickyTooltipOverBars(i),l&&s.tooltipPosition.moveMarkers(n,i)}}]),e}(),ke=function(){function e(t){o(this,e),this.ctx=t,this.w=t.w,this.ev=this.w.config.chart.events,this.localeValues=this.w.globals.locale.toolbar}return l(e,[{key:"createToolbar",value:function(){var e=this.w,t=document.createElement("div");if(t.setAttribute("class","apexcharts-toolbar"),e.globals.dom.elWrap.appendChild(t),this.elZoom=document.createElement("div"),this.elZoomIn=document.createElement("div"),this.elZoomOut=document.createElement("div"),this.elPan=document.createElement("div"),this.elSelection=document.createElement("div"),this.elZoomReset=document.createElement("div"),this.elMenuIcon=document.createElement("div"),this.elMenu=document.createElement("div"),this.elCustomIcons=[],this.t=e.config.chart.toolbar.tools,Array.isArray(this.t.customIcons))for(var n=0;n<this.t.customIcons.length;n++)this.elCustomIcons.push(document.createElement("div"));this.elMenuItems=[];var i=[];this.t.zoomin&&e.config.chart.zoom.enabled&&i.push({el:this.elZoomIn,icon:"string"==typeof this.t.zoomin?this.t.zoomin:'<svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24">\n    <path d="M0 0h24v24H0z" fill="none"/>\n    <path d="M13 7h-2v4H7v2h4v4h2v-4h4v-2h-4V7zm-1-5C6.48 2 2 6.48 2 12s4.48 10 10 10 10-4.48 10-10S17.52 2 12 2zm0 18c-4.41 0-8-3.59-8-8s3.59-8 8-8 8 3.59 8 8-3.59 8-8 8z"/>\n</svg>\n',title:this.localeValues.zoomIn,class:"apexcharts-zoom-in-icon"}),this.t.zoomout&&e.config.chart.zoom.enabled&&i.push({el:this.elZoomOut,icon:"string"==typeof this.t.zoomout?this.t.zoomout:'<svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24">\n    <path d="M0 0h24v24H0z" fill="none"/>\n    <path d="M7 11v2h10v-2H7zm5-9C6.48 2 2 6.48 2 12s4.48 10 10 10 10-4.48 10-10S17.52 2 12 2zm0 18c-4.41 0-8-3.59-8-8s3.59-8 8-8 8 3.59 8 8-3.59 8-8 8z"/>\n</svg>\n',title:this.localeValues.zoomOut,class:"apexcharts-zoom-out-icon"}),this.t.zoom&&e.config.chart.zoom.enabled&&i.push({el:this.elZoom,icon:"string"==typeof this.t.zoom?this.t.zoom:'<svg xmlns="http://www.w3.org/2000/svg" fill="#000000" height="24" viewBox="0 0 24 24" width="24">\n    <path d="M15.5 14h-.79l-.28-.27C15.41 12.59 16 11.11 16 9.5 16 5.91 13.09 3 9.5 3S3 5.91 3 9.5 5.91 16 9.5 16c1.61 0 3.09-.59 4.23-1.57l.27.28v.79l5 4.99L20.49 19l-4.99-5zm-6 0C7.01 14 5 11.99 5 9.5S7.01 5 9.5 5 14 7.01 14 9.5 11.99 14 9.5 14z"/>\n    <path d="M0 0h24v24H0V0z" fill="none"/>\n    <path d="M12 10h-2v2H9v-2H7V9h2V7h1v2h2v1z"/>\n</svg>',title:this.localeValues.selectionZoom,class:e.globals.isTouchDevice?"hidden":"apexcharts-zoom-icon"}),this.t.selection&&e.config.chart.selection.enabled&&i.push({el:this.elSelection,icon:"string"==typeof this.t.selection?this.t.selection:'<svg fill="#6E8192" height="24" viewBox="0 0 24 24" width="24" xmlns="http://www.w3.org/2000/svg">\n    <path d="M0 0h24v24H0z" fill="none"/>\n    <path d="M3 5h2V3c-1.1 0-2 .9-2 2zm0 8h2v-2H3v2zm4 8h2v-2H7v2zM3 9h2V7H3v2zm10-6h-2v2h2V3zm6 0v2h2c0-1.1-.9-2-2-2zM5 21v-2H3c0 1.1.9 2 2 2zm-2-4h2v-2H3v2zM9 3H7v2h2V3zm2 18h2v-2h-2v2zm8-8h2v-2h-2v2zm0 8c1.1 0 2-.9 2-2h-2v2zm0-12h2V7h-2v2zm0 8h2v-2h-2v2zm-4 4h2v-2h-2v2zm0-16h2V3h-2v2z"/>\n</svg>',title:this.localeValues.selection,class:e.globals.isTouchDevice?"hidden":"apexcharts-selection-icon"}),this.t.pan&&e.config.chart.zoom.enabled&&i.push({el:this.elPan,icon:"string"==typeof this.t.pan?this.t.pan:'<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" fill="#000000" height="24" viewBox="0 0 24 24" width="24">\n    <defs>\n        <path d="M0 0h24v24H0z" id="a"/>\n    </defs>\n    <clipPath id="b">\n        <use overflow="visible" xlink:href="#a"/>\n    </clipPath>\n    <path clip-path="url(#b)" d="M23 5.5V20c0 2.2-1.8 4-4 4h-7.3c-1.08 0-2.1-.43-2.85-1.19L1 14.83s1.26-1.23 1.3-1.25c.22-.19.49-.29.79-.29.22 0 .42.06.6.16.04.01 4.31 2.46 4.31 2.46V4c0-.83.67-1.5 1.5-1.5S11 3.17 11 4v7h1V1.5c0-.83.67-1.5 1.5-1.5S15 .67 15 1.5V11h1V2.5c0-.83.67-1.5 1.5-1.5s1.5.67 1.5 1.5V11h1V5.5c0-.83.67-1.5 1.5-1.5s1.5.67 1.5 1.5z"/>\n</svg>',title:this.localeValues.pan,class:e.globals.isTouchDevice?"hidden":"apexcharts-pan-icon"}),this.t.reset&&e.config.chart.zoom.enabled&&i.push({el:this.elZoomReset,icon:"string"==typeof this.t.reset?this.t.reset:'<svg fill="#000000" height="24" viewBox="0 0 24 24" width="24" xmlns="http://www.w3.org/2000/svg">\n    <path d="M10 20v-6h4v6h5v-8h3L12 3 2 12h3v8z"/>\n    <path d="M0 0h24v24H0z" fill="none"/>\n</svg>',title:this.localeValues.reset,class:"apexcharts-reset-zoom-icon"}),this.t.download&&i.push({el:this.elMenuIcon,icon:"string"==typeof this.t.download?this.t.download:'<svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path fill="none" d="M0 0h24v24H0V0z"/><path d="M3 18h18v-2H3v2zm0-5h18v-2H3v2zm0-7v2h18V6H3z"/></svg>',title:this.localeValues.menu,class:"apexcharts-menu-icon"});for(var r=0;r<this.elCustomIcons.length;r++)i.push({el:this.elCustomIcons[r],icon:this.t.customIcons[r].icon,title:this.t.customIcons[r].title,index:this.t.customIcons[r].index,class:"apexcharts-toolbar-custom-icon "+this.t.customIcons[r].class});i.forEach(function(e,t){e.index&&m.moveIndexInArray(i,t,e.index)});for(var a=0;a<i.length;a++)b.setAttrs(i[a].el,{class:i[a].class,title:i[a].title}),i[a].el.innerHTML=i[a].icon,t.appendChild(i[a].el);t.appendChild(this.elMenu),b.setAttrs(this.elMenu,{class:"apexcharts-menu"});for(var o=[{name:"exportSVG",title:this.localeValues.exportToSVG},{name:"exportPNG",title:this.localeValues.exportToPNG}],s=0;s<o.length;s++)this.elMenuItems.push(document.createElement("div")),this.elMenuItems[s].innerHTML=o[s].title,b.setAttrs(this.elMenuItems[s],{class:"apexcharts-menu-item ".concat(o[s].name),title:o[s].title}),this.elMenu.appendChild(this.elMenuItems[s]);e.globals.zoomEnabled?this.elZoom.classList.add("selected"):e.globals.panEnabled?this.elPan.classList.add("selected"):e.globals.selectionEnabled&&this.elSelection.classList.add("selected"),this.addToolbarEventListeners()}},{key:"addToolbarEventListeners",value:function(){var e=this;this.elZoomReset.addEventListener("click",this.handleZoomReset.bind(this)),this.elSelection.addEventListener("click",this.toggleSelection.bind(this)),this.elZoom.addEventListener("click",this.toggleZooming.bind(this)),this.elZoomIn.addEventListener("click",this.handleZoomIn.bind(this)),this.elZoomOut.addEventListener("click",this.handleZoomOut.bind(this)),this.elPan.addEventListener("click",this.togglePanning.bind(this)),this.elMenuIcon.addEventListener("click",this.toggleMenu.bind(this)),this.elMenuItems.forEach(function(t){t.classList.contains("exportSVG")?t.addEventListener("click",e.downloadSVG.bind(e)):t.classList.contains("exportPNG")&&t.addEventListener("click",e.downloadPNG.bind(e))});for(var t=0;t<this.t.customIcons.length;t++)this.elCustomIcons[t].addEventListener("click",this.t.customIcons[t].click)}},{key:"toggleSelection",value:function(){this.toggleOtherControls(),this.w.globals.selectionEnabled=!this.w.globals.selectionEnabled,this.elSelection.classList.contains("selected")?this.elSelection.classList.remove("selected"):this.elSelection.classList.add("selected")}},{key:"toggleZooming",value:function(){this.toggleOtherControls(),this.w.globals.zoomEnabled=!this.w.globals.zoomEnabled,this.elZoom.classList.contains("selected")?this.elZoom.classList.remove("selected"):this.elZoom.classList.add("selected")}},{key:"getToolbarIconsReference",value:function(){var e=this.w;this.elZoom||(this.elZoom=e.globals.dom.baseEl.querySelector(".apexcharts-zoom-icon")),this.elPan||(this.elPan=e.globals.dom.baseEl.querySelector(".apexcharts-pan-icon")),this.elSelection||(this.elSelection=e.globals.dom.baseEl.querySelector(".apexcharts-selection-icon"))}},{key:"enableZooming",value:function(){this.toggleOtherControls(),this.w.globals.zoomEnabled=!0,this.elZoom&&this.elZoom.classList.add("selected"),this.elPan&&this.elPan.classList.remove("selected")}},{key:"enablePanning",value:function(){this.toggleOtherControls(),this.w.globals.panEnabled=!0,this.elPan&&this.elPan.classList.add("selected"),this.elZoom&&this.elZoom.classList.remove("selected")}},{key:"togglePanning",value:function(){this.toggleOtherControls(),this.w.globals.panEnabled=!this.w.globals.panEnabled,this.elPan.classList.contains("selected")?this.elPan.classList.remove("selected"):this.elPan.classList.add("selected")}},{key:"toggleOtherControls",value:function(){var e=this.w;e.globals.panEnabled=!1,e.globals.zoomEnabled=!1,e.globals.selectionEnabled=!1,this.getToolbarIconsReference(),this.elPan&&this.elPan.classList.remove("selected"),this.elSelection&&this.elSelection.classList.remove("selected"),this.elZoom&&this.elZoom.classList.remove("selected")}},{key:"handleZoomIn",value:function(){var e=this.w,t=(e.globals.minX+e.globals.maxX)/2,n=(e.globals.minX+t)/2,i=(e.globals.maxX+t)/2;e.globals.disableZoomIn||this.zoomUpdateOptions(n,i)}},{key:"handleZoomOut",value:function(){var e=this.w;if(!("datetime"===e.config.xaxis.type&&new Date(e.globals.minX).getUTCFullYear()<1e3)){var t=(e.globals.minX+e.globals.maxX)/2,n=e.globals.minX-(t-e.globals.minX),i=e.globals.maxX-(t-e.globals.maxX);e.globals.disableZoomOut||this.zoomUpdateOptions(n,i)}}},{key:"zoomUpdateOptions",value:function(e,t){var n={min:e,max:t},i=this.getBeforeZoomRange(n);i&&(n=i.xaxis),this.w.globals.zoomed=!0,this.ctx._updateOptions({xaxis:n},!1,this.w.config.chart.animations.dynamicAnimation.enabled),this.zoomCallback(n)}},{key:"zoomCallback",value:function(e,t){"function"==typeof this.ev.zoomed&&this.ev.zoomed(this.ctx,{xaxis:e,yaxis:t})}},{key:"getBeforeZoomRange",value:function(e,t){var n=null;return"function"==typeof this.ev.beforeZoom&&(n=this.ev.beforeZoom(this,{xaxis:e,yaxis:t})),n}},{key:"toggleMenu",value:function(){this.elMenu.classList.contains("open")?this.elMenu.classList.remove("open"):this.elMenu.classList.add("open")}},{key:"downloadPNG",value:function(){var e=new ce(this.ctx);e.exportToPng(this.ctx),this.toggleMenu()}},{key:"downloadSVG",value:function(){var e=new ce(this.ctx);e.exportToSVG(),this.toggleMenu()}},{key:"handleZoomReset",value:function(e){var t=this;this.ctx.getSyncedCharts().forEach(function(e){var n=e.w;n.globals.minX!==n.globals.initialminX&&n.globals.maxX!==n.globals.initialmaxX&&(e.revertDefaultAxisMinMax(),"function"==typeof n.config.chart.events.zoomed&&t.zoomCallback({min:n.config.xaxis.min,max:n.config.xaxis.max}),n.globals.zoomed=!1,e._updateSeries(n.globals.initialSeries,n.config.chart.animations.dynamicAnimation.enabled))})}},{key:"destroy",value:function(){this.elZoomReset&&(this.elZoomReset.removeEventListener("click",this.handleZoomReset.bind(this)),this.elSelection.removeEventListener("click",this.toggleSelection.bind(this)),this.elZoom.removeEventListener("click",this.toggleZooming.bind(this)),this.elZoomIn.removeEventListener("click",this.handleZoomIn.bind(this)),this.elZoomOut.removeEventListener("click",this.handleZoomOut.bind(this)),this.elPan.removeEventListener("click",this.togglePanning.bind(this)),this.elMenuIcon.removeEventListener("click",this.toggleMenu.bind(this))),this.elZoom=null,this.elZoomIn=null,this.elZoomOut=null,this.elPan=null,this.elSelection=null,this.elZoomReset=null,this.elMenuIcon=null}}]),e}(),Se=function(e){function t(e){var n;return o(this,t),(n=p(this,d(t).call(this,e))).ctx=e,n.w=e.w,n.dragged=!1,n.graphics=new b(n.ctx),n.eventList=["mousedown","mousemove","touchstart","touchmove","mouseup","touchend"],n.clientX=0,n.clientY=0,n.startX=0,n.endX=0,n.dragX=0,n.startY=0,n.endY=0,n.dragY=0,n}return h(t,ke),l(t,[{key:"init",value:function(e){var t=this,n=e.xyRatios,i=this.w,r=this;this.xyRatios=n,this.zoomRect=this.graphics.drawRect(0,0,0,0),this.selectionRect=this.graphics.drawRect(0,0,0,0),this.gridRect=i.globals.dom.baseEl.querySelector(".apexcharts-grid"),this.zoomRect.node.classList.add("apexcharts-zoom-rect"),this.selectionRect.node.classList.add("apexcharts-selection-rect"),i.globals.dom.elGraphical.add(this.zoomRect),i.globals.dom.elGraphical.add(this.selectionRect),"x"===i.config.chart.selection.type?this.slDraggableRect=this.selectionRect.draggable({minX:0,minY:0,maxX:i.globals.gridWidth,maxY:i.globals.gridHeight}).on("dragmove",this.selectionDragging.bind(this,"dragging")):"y"===i.config.chart.selection.type?this.slDraggableRect=this.selectionRect.draggable({minX:0,maxX:i.globals.gridWidth}).on("dragmove",this.selectionDragging.bind(this,"dragging")):this.slDraggableRect=this.selectionRect.draggable().on("dragmove",this.selectionDragging.bind(this,"dragging")),this.preselectedSelection(),this.hoverArea=i.globals.dom.baseEl.querySelector(i.globals.chartClass),this.hoverArea.classList.add("zoomable"),this.eventList.forEach(function(e){t.hoverArea.addEventListener(e,r.svgMouseEvents.bind(r,n),{capture:!1,passive:!0})})}},{key:"destroy",value:function(){var e=this,t=this;this.eventList.forEach(function(n){e.hoverArea&&e.hoverArea.removeEventListener(n,t.svgMouseEvents.bind(t,t.xyRatios),{capture:!1,passive:!0})}),this.slDraggableRect&&(this.slDraggableRect.draggable(!1),this.slDraggableRect.off(),this.selectionRect.off()),this.selectionRect=null,this.zoomRect=null,this.gridRect=null}},{key:"svgMouseEvents",value:function(e,t){var n=this.w,i=this,r=this.ctx.toolbar,a=n.globals.zoomEnabled?n.config.chart.zoom.type:n.config.chart.selection.type;if(t.shiftKey?(this.shiftWasPressed=!0,r.enablePanning()):this.shiftWasPressed&&(r.enableZooming(),this.shiftWasPressed=!1),!(t.target.classList.contains("apexcharts-selection-rect")||t.target.parentNode.classList.contains("apexcharts-toolbar"))){if(i.clientX="touchmove"===t.type||"touchstart"===t.type?t.touches[0].clientX:"touchend"===t.type?t.changedTouches[0].clientX:t.clientX,i.clientY="touchmove"===t.type||"touchstart"===t.type?t.touches[0].clientY:"touchend"===t.type?t.changedTouches[0].clientY:t.clientY,"mousedown"===t.type&&1===t.which){var o=i.gridRect.getBoundingClientRect();i.startX=i.clientX-o.left,i.startY=i.clientY-o.top,i.dragged=!1,i.w.globals.mousedown=!0}if(("mousemove"===t.type&&1===t.which||"touchmove"===t.type)&&(i.dragged=!0,n.globals.panEnabled?(n.globals.selection=null,i.w.globals.mousedown&&i.panDragging({context:i,zoomtype:a,xyRatios:e})):(i.w.globals.mousedown&&n.globals.zoomEnabled||i.w.globals.mousedown&&n.globals.selectionEnabled)&&(i.selection=i.selectionDrawing({context:i,zoomtype:a}))),"mouseup"===t.type||"touchend"===t.type){var s=i.gridRect.getBoundingClientRect();i.w.globals.mousedown&&(i.endX=i.clientX-s.left,i.endY=i.clientY-s.top,i.dragX=Math.abs(i.endX-i.startX),i.dragY=Math.abs(i.endY-i.startY),(n.globals.zoomEnabled||n.globals.selectionEnabled)&&i.selectionDrawn({context:i,zoomtype:a})),n.globals.zoomEnabled&&i.hideSelectionRect(this.selectionRect),i.dragged=!1,i.w.globals.mousedown=!1}this.makeSelectionRectDraggable()}}},{key:"makeSelectionRectDraggable",value:function(){var e=this.w;if(this.selectionRect){var t=this.selectionRect.node.getBoundingClientRect();t.width>0&&t.height>0&&this.slDraggableRect.selectize().resize({constraint:{minX:0,minY:0,maxX:e.globals.gridWidth,maxY:e.globals.gridHeight}}).on("resizing",this.selectionDragging.bind(this,"resizing"))}}},{key:"preselectedSelection",value:function(){var e=this.w,t=this.xyRatios;if(!e.globals.zoomEnabled)if(void 0!==e.globals.selection&&null!==e.globals.selection)this.drawSelectionRect(e.globals.selection);else if(void 0!==e.config.chart.selection.xaxis.min&&void 0!==e.config.chart.selection.xaxis.max){var n=(e.config.chart.selection.xaxis.min-e.globals.minX)/t.xRatio,i={x:n,y:0,width:e.globals.gridWidth-(e.globals.maxX-e.config.chart.selection.xaxis.max)/t.xRatio-n,height:e.globals.gridHeight,translateX:0,translateY:0,selectionEnabled:!0};this.drawSelectionRect(i),this.makeSelectionRectDraggable(),"function"==typeof e.config.chart.events.selection&&e.config.chart.events.selection(this.ctx,{xaxis:{min:e.config.chart.selection.xaxis.min,max:e.config.chart.selection.xaxis.max},yaxis:{}})}}},{key:"drawSelectionRect",value:function(e){var t=e.x,n=e.y,i=e.width,r=e.height,a=e.translateX,o=e.translateY,s=this.w,l=this.zoomRect,c=this.selectionRect;if(this.dragged||null!==s.globals.selection){var u={transform:"translate("+a+", "+o+")"};s.globals.zoomEnabled&&this.dragged&&(l.attr({x:t,y:n,width:i,height:r,fill:s.config.chart.zoom.zoomedArea.fill.color,"fill-opacity":s.config.chart.zoom.zoomedArea.fill.opacity,stroke:s.config.chart.zoom.zoomedArea.stroke.color,"stroke-width":s.config.chart.zoom.zoomedArea.stroke.width,"stroke-opacity":s.config.chart.zoom.zoomedArea.stroke.opacity}),b.setAttrs(l.node,u)),s.globals.selectionEnabled&&(c.attr({x:t,y:n,width:i>0?i:0,height:r>0?r:0,fill:s.config.chart.selection.fill.color,"fill-opacity":s.config.chart.selection.fill.opacity,stroke:s.config.chart.selection.stroke.color,"stroke-width":s.config.chart.selection.stroke.width,"stroke-dasharray":s.config.chart.selection.stroke.dashArray,"stroke-opacity":s.config.chart.selection.stroke.opacity}),b.setAttrs(c.node,u))}}},{key:"hideSelectionRect",value:function(e){e&&e.attr({x:0,y:0,width:0,height:0})}},{key:"selectionDrawing",value:function(e){var t=e.context,n=e.zoomtype,i=this.w,r=t,a=this.gridRect.getBoundingClientRect(),o=r.startX-1,s=r.startY,l=r.clientX-a.left-o,c=r.clientY-a.top-s,u=0,h=0,d={};return(Math.abs(l+o)>i.globals.gridWidth||r.clientX-a.left<0)&&(r.hideSelectionRect(this.zoomRect),r.dragged=!1,r.w.globals.mousedown=!1),o>r.clientX-a.left&&(u=-(l=Math.abs(l))),s>r.clientY-a.top&&(h=-(c=Math.abs(c))),d="x"===n?{x:o,y:0,width:l,height:i.globals.gridHeight,translateX:u,translateY:0}:"y"===n?{x:0,y:s,width:i.globals.gridWidth,height:c,translateX:0,translateY:h}:{x:o,y:s,width:l,height:c,translateX:u,translateY:h},r.drawSelectionRect(d),r.selectionDragging("resizing"),d}},{key:"selectionDragging",value:function(e,t){var n=this,i=this.w,r=this.xyRatios,a=this.selectionRect,o=0;"resizing"===e&&(o=30),"function"==typeof i.config.chart.events.selection&&i.globals.selectionEnabled&&(clearTimeout(this.w.globals.selectionResizeTimer),this.w.globals.selectionResizeTimer=window.setTimeout(function(){var e=n.gridRect.getBoundingClientRect(),t=a.node.getBoundingClientRect(),o=i.globals.xAxisScale.niceMin+(t.left-e.left)*r.xRatio,s=i.globals.xAxisScale.niceMin+(t.right-e.left)*r.xRatio,l=i.globals.yAxisScale[0].niceMin+(e.bottom-t.bottom)*r.yRatio[0],c=i.globals.yAxisScale[0].niceMax-(t.top-e.top)*r.yRatio[0];i.config.chart.events.selection(n.ctx,{xaxis:{min:o,max:s},yaxis:{min:l,max:c}})},o))}},{key:"selectionDrawn",value:function(e){var t=e.context,n=e.zoomtype,i=this.w,r=t,a=this.xyRatios,o=this.ctx.toolbar;if(r.startX>r.endX){var s=r.startX;r.startX=r.endX,r.endX=s}if(r.startY>r.endY){var l=r.startY;r.startY=r.endY,r.endY=l}var c=i.globals.xAxisScale.niceMin+r.startX*a.xRatio,u=i.globals.xAxisScale.niceMin+r.endX*a.xRatio,h=[],d=[];if(i.config.yaxis.forEach(function(e,t){h.push(Math.floor(i.globals.yAxisScale[t].niceMax-a.yRatio[t]*r.startY)),d.push(Math.floor(i.globals.yAxisScale[t].niceMax-a.yRatio[t]*r.endY))}),r.dragged&&(r.dragX>10||r.dragY>10)&&c!==u)if(i.globals.zoomEnabled){var f=m.clone(i.config.yaxis);i.globals.zoomed||(i.globals.lastXAxis=m.clone(i.config.xaxis),i.globals.lastYAxis=m.clone(i.config.yaxis));var p={min:c,max:u};if("xy"!==n&&"y"!==n||f.forEach(function(e,t){f[t].min=d[t],f[t].max=h[t]}),i.config.chart.zoom.autoScaleYaxis){var g=new U(r.ctx);f=g.autoScaleY(r.ctx,{xaxis:p})}if(o){var v=o.getBeforeZoomRange(p,f);v&&(p=v.xaxis?v.xaxis:p,f=v.yaxis?v.yaxe:f)}r.ctx._updateOptions({xaxis:p,yaxis:f},!1,r.w.config.chart.animations.dynamicAnimation.enabled),"function"==typeof i.config.chart.events.zoomed&&o.zoomCallback(p,f),i.globals.zoomed=!0}else if(i.globals.selectionEnabled){var y,b=null;y={min:c,max:u},"xy"!==n&&"y"!==n||(b=m.clone(i.config.yaxis)).forEach(function(e,t){b[t].min=d[t],b[t].max=h[t]}),i.globals.selection=r.selection,"function"==typeof i.config.chart.events.selection&&i.config.chart.events.selection(r.ctx,{xaxis:y,yaxis:b})}}},{key:"panDragging",value:function(e){var t,n=e.context,i=(e.zoomtype,this.w),r=n;if(void 0!==i.globals.lastClientPosition.x){var a=i.globals.lastClientPosition.x-r.clientX,o=i.globals.lastClientPosition.y-r.clientY;Math.abs(a)>Math.abs(o)&&a>0?t="left":Math.abs(a)>Math.abs(o)&&a<0?t="right":Math.abs(o)>Math.abs(a)&&o>0?t="up":Math.abs(o)>Math.abs(a)&&o<0&&(t="down")}i.globals.lastClientPosition={x:r.clientX,y:r.clientY};var s=i.globals.minX,l=i.globals.maxX;this.panScrolled(t,s,l)}},{key:"panScrolled",value:function(e,t,n){var i=this.w,r=this.xyRatios,a=m.clone(i.config.yaxis);"left"===e?(t=i.globals.minX+i.globals.gridWidth/15*r.xRatio,n=i.globals.maxX+i.globals.gridWidth/15*r.xRatio):"right"===e&&(t=i.globals.minX-i.globals.gridWidth/15*r.xRatio,n=i.globals.maxX-i.globals.gridWidth/15*r.xRatio),(t<i.globals.initialminX||n>i.globals.initialmaxX)&&(t=i.globals.minX,n=i.globals.maxX);var o={min:t,max:n};i.config.chart.zoom.autoScaleYaxis&&(a=new U(me.ctx).autoScaleY(me.ctx,{xaxis:o}));this.ctx._updateOptions({xaxis:{min:t,max:n},yaxis:a},!1,!1),"function"==typeof i.config.chart.events.scrolled&&i.config.chart.events.scrolled(this.ctx,{xaxis:{min:t,max:n}})}}]),t}(),Ee=function(){function e(t){o(this,e),this.ctx=t,this.w=t.w}return l(e,[{key:"draw",value:function(){this.drawTitleSubtitle("title"),this.drawTitleSubtitle("subtitle")}},{key:"drawTitleSubtitle",value:function(e){var t=this.w,n="title"===e?t.config.title:t.config.subtitle,i=t.globals.svgWidth/2,r=n.offsetY,a="middle";if("left"===n.align?(i=10,a="start"):"right"===n.align&&(i=t.globals.svgWidth-10,a="end"),i+=n.offsetX,r=r+parseInt(n.style.fontSize)+2,void 0!==n.text){var o=new b(this.ctx).drawText({x:i,y:r,text:n.text,textAnchor:a,fontSize:n.style.fontSize,fontFamily:n.style.fontFamily,foreColor:n.style.color,opacity:1});o.node.setAttribute("class","apexcharts-".concat(e,"-text")),t.globals.dom.Paper.add(o)}}}]),e}();le="undefined"!=typeof window?window:void 0,void 0===(r=function(){return function(e,t){var n=(void 0!==this?this:e).SVG=function(e){if(n.supported)return e=new n.Doc(e),n.parser.draw||n.prepare(),e};if(n.ns="http://www.w3.org/2000/svg",n.xmlns="http://www.w3.org/2000/xmlns/",n.xlink="http://www.w3.org/1999/xlink",n.svgjs="http://svgjs.com/svgjs",n.supported=!0,!n.supported)return!1;n.did=1e3,n.eid=function(e){return"Svgjs"+h(e)+n.did++},n.create=function(e){var n=t.createElementNS(this.ns,e);return n.setAttribute("id",this.eid(e)),n},n.extend=function(){var e,t,i,r;for(e=[].slice.call(arguments),t=e.pop(),r=e.length-1;r>=0;r--)if(e[r])for(i in t)e[r].prototype[i]=t[i];n.Set&&n.Set.inherit&&n.Set.inherit()},n.invent=function(e){var t="function"==typeof e.create?e.create:function(){this.constructor.call(this,n.create(e.create))};return e.inherit&&(t.prototype=new e.inherit),e.extend&&n.extend(t,e.extend),e.construct&&n.extend(e.parent||n.Container,e.construct),t},n.adopt=function(t){return t?t.instance?t.instance:((i="svg"==t.nodeName?t.parentNode instanceof e.SVGElement?new n.Nested:new n.Doc:"linearGradient"==t.nodeName?new n.Gradient("linear"):"radialGradient"==t.nodeName?new n.Gradient("radial"):n[h(t.nodeName)]?new(n[h(t.nodeName)]):new n.Element(t)).type=t.nodeName,i.node=t,t.instance=i,i instanceof n.Doc&&i.namespace().defs(),i.setData(JSON.parse(t.getAttribute("svgjs:data"))||{}),i):null;var i},n.prepare=function(){var e=t.getElementsByTagName("body")[0],i=(e?new n.Doc(e):n.adopt(t.documentElement).nested()).size(2,0);n.parser={body:e||t.documentElement,draw:i.style("opacity:0;position:absolute;left:-100%;top:-100%;overflow:hidden").node,poly:i.polyline().node,path:i.path().node,native:n.create("svg")}},n.parser={native:n.create("svg")},t.addEventListener("DOMContentLoaded",function(){n.parser.draw||n.prepare()},!1),n.regex={numberAndUnit:/^([+-]?(\d+(\.\d*)?|\.\d+)(e[+-]?\d+)?)([a-z%]*)$/i,hex:/^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i,rgb:/rgb\((\d+),(\d+),(\d+)\)/,reference:/#([a-z0-9\-_]+)/i,transforms:/\)\s*,?\s*/,whitespace:/\s/g,isHex:/^#[a-f0-9]{3,6}$/i,isRgb:/^rgb\(/,isCss:/[^:]+:[^;]+;?/,isBlank:/^(\s+)?$/,isNumber:/^[+-]?(\d+(\.\d*)?|\.\d+)(e[+-]?\d+)?$/i,isPercent:/^-?[\d\.]+%$/,isImage:/\.(jpg|jpeg|png|gif|svg)(\?[^=]+.*)?/i,delimiter:/[\s,]+/,hyphen:/([^e])\-/gi,pathLetters:/[MLHVCSQTAZ]/gi,isPathLetter:/[MLHVCSQTAZ]/i,numbersWithDots:/((\d?\.\d+(?:e[+-]?\d+)?)((?:\.\d+(?:e[+-]?\d+)?)+))+/gi,dots:/\./g},n.utils={map:function(e,t){var n,i=e.length,r=[];for(n=0;n<i;n++)r.push(t(e[n]));return r},filter:function(e,t){var n,i=e.length,r=[];for(n=0;n<i;n++)t(e[n])&&r.push(e[n]);return r},radians:function(e){return e%360*Math.PI/180},degrees:function(e){return 180*e/Math.PI%360},filterSVGElements:function(t){return this.filter(t,function(t){return t instanceof e.SVGElement})}},n.defaults={attrs:{"fill-opacity":1,"stroke-opacity":1,"stroke-width":0,"stroke-linejoin":"miter","stroke-linecap":"butt",fill:"#000000",stroke:"#000000",opacity:1,x:0,y:0,cx:0,cy:0,width:0,height:0,r:0,rx:0,ry:0,offset:0,"stop-opacity":1,"stop-color":"#000000","font-size":16,"font-family":"Helvetica, Arial, sans-serif","text-anchor":"start"}},n.Color=function(e){var t,i;this.r=0,this.g=0,this.b=0,e&&("string"==typeof e?n.regex.isRgb.test(e)?(t=n.regex.rgb.exec(e.replace(n.regex.whitespace,"")),this.r=parseInt(t[1]),this.g=parseInt(t[2]),this.b=parseInt(t[3])):n.regex.isHex.test(e)&&(t=n.regex.hex.exec(4==(i=e).length?["#",i.substring(1,2),i.substring(1,2),i.substring(2,3),i.substring(2,3),i.substring(3,4),i.substring(3,4)].join(""):i),this.r=parseInt(t[1],16),this.g=parseInt(t[2],16),this.b=parseInt(t[3],16)):"object"===a(e)&&(this.r=e.r,this.g=e.g,this.b=e.b))},n.extend(n.Color,{toString:function(){return this.toHex()},toHex:function(){return"#"+d(this.r)+d(this.g)+d(this.b)},toRgb:function(){return"rgb("+[this.r,this.g,this.b].join()+")"},brightness:function(){return this.r/255*.3+this.g/255*.59+this.b/255*.11},morph:function(e){return this.destination=new n.Color(e),this},at:function(e){return this.destination?(e=e<0?0:e>1?1:e,new n.Color({r:~~(this.r+(this.destination.r-this.r)*e),g:~~(this.g+(this.destination.g-this.g)*e),b:~~(this.b+(this.destination.b-this.b)*e)})):this}}),n.Color.test=function(e){return e+="",n.regex.isHex.test(e)||n.regex.isRgb.test(e)},n.Color.isRgb=function(e){return e&&"number"==typeof e.r&&"number"==typeof e.g&&"number"==typeof e.b},n.Color.isColor=function(e){return n.Color.isRgb(e)||n.Color.test(e)},n.Array=function(e,t){0==(e=(e||[]).valueOf()).length&&t&&(e=t.valueOf()),this.value=this.parse(e)},n.extend(n.Array,{morph:function(e){if(this.destination=this.parse(e),this.value.length!=this.destination.length){for(var t=this.value[this.value.length-1],n=this.destination[this.destination.length-1];this.value.length>this.destination.length;)this.destination.push(n);for(;this.value.length<this.destination.length;)this.value.push(t)}return this},settle:function(){for(var e=0,t=this.value.length,n=[];e<t;e++)-1==n.indexOf(this.value[e])&&n.push(this.value[e]);return this.value=n},at:function(e){if(!this.destination)return this;for(var t=0,i=this.value.length,r=[];t<i;t++)r.push(this.value[t]+(this.destination[t]-this.value[t])*e);return new n.Array(r)},toString:function(){return this.value.join(" ")},valueOf:function(){return this.value},parse:function(e){return e=e.valueOf(),Array.isArray(e)?e:this.split(e)},split:function(e){return e.trim().split(n.regex.delimiter).map(parseFloat)},reverse:function(){return this.value.reverse(),this},clone:function(){var e=new this.constructor;return e.value=function e(t){for(var n=t.slice(0),i=n.length;i--;)Array.isArray(n[i])&&(n[i]=e(n[i]));return n}(this.value),e}}),n.PointArray=function(e,t){n.Array.call(this,e,t||[[0,0]])},n.PointArray.prototype=new n.Array,n.PointArray.prototype.constructor=n.PointArray,n.extend(n.PointArray,{toString:function(){for(var e=0,t=this.value.length,n=[];e<t;e++)n.push(this.value[e].join(","));return n.join(" ")},toLine:function(){return{x1:this.value[0][0],y1:this.value[0][1],x2:this.value[1][0],y2:this.value[1][1]}},at:function(e){if(!this.destination)return this;for(var t=0,i=this.value.length,r=[];t<i;t++)r.push([this.value[t][0]+(this.destination[t][0]-this.value[t][0])*e,this.value[t][1]+(this.destination[t][1]-this.value[t][1])*e]);return new n.PointArray(r)},parse:function(e){var t=[];if(e=e.valueOf(),Array.isArray(e)){if(Array.isArray(e[0]))return e.map(function(e){return e.slice()});if(null!=e[0].x)return e.map(function(e){return[e.x,e.y]})}else e=e.trim().split(n.regex.delimiter).map(parseFloat);e.length%2!=0&&e.pop();for(var i=0,r=e.length;i<r;i+=2)t.push([e[i],e[i+1]]);return t},move:function(e,t){var n=this.bbox();if(e-=n.x,t-=n.y,!isNaN(e)&&!isNaN(t))for(var i=this.value.length-1;i>=0;i--)this.value[i]=[this.value[i][0]+e,this.value[i][1]+t];return this},size:function(e,t){var n,i=this.bbox();for(n=this.value.length-1;n>=0;n--)i.width&&(this.value[n][0]=(this.value[n][0]-i.x)*e/i.width+i.x),i.height&&(this.value[n][1]=(this.value[n][1]-i.y)*t/i.height+i.y);return this},bbox:function(){return n.parser.draw||n.prepare(),n.parser.poly.setAttribute("points",this.toString()),n.parser.poly.getBBox()}});for(var i={M:function(e,t,n){return t.x=n.x=e[0],t.y=n.y=e[1],["M",t.x,t.y]},L:function(e,t){return t.x=e[0],t.y=e[1],["L",e[0],e[1]]},H:function(e,t){return t.x=e[0],["H",e[0]]},V:function(e,t){return t.y=e[0],["V",e[0]]},C:function(e,t){return t.x=e[4],t.y=e[5],["C",e[0],e[1],e[2],e[3],e[4],e[5]]},S:function(e,t){return t.x=e[2],t.y=e[3],["S",e[0],e[1],e[2],e[3]]},Q:function(e,t){return t.x=e[2],t.y=e[3],["Q",e[0],e[1],e[2],e[3]]},T:function(e,t){return t.x=e[0],t.y=e[1],["T",e[0],e[1]]},Z:function(e,t,n){return t.x=n.x,t.y=n.y,["Z"]},A:function(e,t){return t.x=e[5],t.y=e[6],["A",e[0],e[1],e[2],e[3],e[4],e[5],e[6]]}},r="mlhvqtcsaz".split(""),o=0,s=r.length;o<s;++o)i[r[o]]=function(e){return function(t,n,r){if("H"==e)t[0]=t[0]+n.x;else if("V"==e)t[0]=t[0]+n.y;else if("A"==e)t[5]=t[5]+n.x,t[6]=t[6]+n.y;else for(var a=0,o=t.length;a<o;++a)t[a]=t[a]+(a%2?n.y:n.x);return i[e](t,n,r)}}(r[o].toUpperCase());n.PathArray=function(e,t){n.Array.call(this,e,t||[["M",0,0]])},n.PathArray.prototype=new n.Array,n.PathArray.prototype.constructor=n.PathArray,n.extend(n.PathArray,{toString:function(){return function(e){for(var t=0,n=e.length,i="";t<n;t++)i+=e[t][0],null!=e[t][1]&&(i+=e[t][1],null!=e[t][2]&&(i+=" ",i+=e[t][2],null!=e[t][3]&&(i+=" ",i+=e[t][3],i+=" ",i+=e[t][4],null!=e[t][5]&&(i+=" ",i+=e[t][5],i+=" ",i+=e[t][6],null!=e[t][7]&&(i+=" ",i+=e[t][7])))));return i+" "}(this.value)},move:function(e,t){var n=this.bbox();if(e-=n.x,t-=n.y,!isNaN(e)&&!isNaN(t))for(var i,r=this.value.length-1;r>=0;r--)"M"==(i=this.value[r][0])||"L"==i||"T"==i?(this.value[r][1]+=e,this.value[r][2]+=t):"H"==i?this.value[r][1]+=e:"V"==i?this.value[r][1]+=t:"C"==i||"S"==i||"Q"==i?(this.value[r][1]+=e,this.value[r][2]+=t,this.value[r][3]+=e,this.value[r][4]+=t,"C"==i&&(this.value[r][5]+=e,this.value[r][6]+=t)):"A"==i&&(this.value[r][6]+=e,this.value[r][7]+=t);return this},size:function(e,t){var n,i,r=this.bbox();for(n=this.value.length-1;n>=0;n--)"M"==(i=this.value[n][0])||"L"==i||"T"==i?(this.value[n][1]=(this.value[n][1]-r.x)*e/r.width+r.x,this.value[n][2]=(this.value[n][2]-r.y)*t/r.height+r.y):"H"==i?this.value[n][1]=(this.value[n][1]-r.x)*e/r.width+r.x:"V"==i?this.value[n][1]=(this.value[n][1]-r.y)*t/r.height+r.y:"C"==i||"S"==i||"Q"==i?(this.value[n][1]=(this.value[n][1]-r.x)*e/r.width+r.x,this.value[n][2]=(this.value[n][2]-r.y)*t/r.height+r.y,this.value[n][3]=(this.value[n][3]-r.x)*e/r.width+r.x,this.value[n][4]=(this.value[n][4]-r.y)*t/r.height+r.y,"C"==i&&(this.value[n][5]=(this.value[n][5]-r.x)*e/r.width+r.x,this.value[n][6]=(this.value[n][6]-r.y)*t/r.height+r.y)):"A"==i&&(this.value[n][1]=this.value[n][1]*e/r.width,this.value[n][2]=this.value[n][2]*t/r.height,this.value[n][6]=(this.value[n][6]-r.x)*e/r.width+r.x,this.value[n][7]=(this.value[n][7]-r.y)*t/r.height+r.y);return this},equalCommands:function(e){var t,i,r;for(e=new n.PathArray(e),r=this.value.length===e.value.length,t=0,i=this.value.length;r&&t<i;t++)r=this.value[t][0]===e.value[t][0];return r},morph:function(e){return e=new n.PathArray(e),this.equalCommands(e)?this.destination=e:this.destination=null,this},at:function(e){if(!this.destination)return this;var t,i,r,a,o=this.value,s=this.destination.value,l=[],c=new n.PathArray;for(t=0,i=o.length;t<i;t++){for(l[t]=[o[t][0]],r=1,a=o[t].length;r<a;r++)l[t][r]=o[t][r]+(s[t][r]-o[t][r])*e;"A"===l[t][0]&&(l[t][4]=+(0!=l[t][4]),l[t][5]=+(0!=l[t][5]))}return c.value=l,c},parse:function(e){if(e instanceof n.PathArray)return e.valueOf();var t,r={M:2,L:2,H:1,V:1,C:6,S:4,Q:4,T:2,A:7,Z:0};e="string"==typeof e?e.replace(n.regex.numbersWithDots,c).replace(n.regex.pathLetters," $& ").replace(n.regex.hyphen,"$1 -").trim().split(n.regex.delimiter):e.reduce(function(e,t){return[].concat.call(e,t)},[]);var a=[],o=new n.Point,s=new n.Point,l=0,u=e.length;do{n.regex.isPathLetter.test(e[l])?(t=e[l],++l):"M"==t?t="L":"m"==t&&(t="l"),a.push(i[t].call(null,e.slice(l,l+=r[t.toUpperCase()]).map(parseFloat),o,s))}while(u>l);return a},bbox:function(){return n.parser.draw||n.prepare(),n.parser.path.setAttribute("d",this.toString()),n.parser.path.getBBox()}}),n.Number=n.invent({create:function(e,t){this.value=0,this.unit=t||"","number"==typeof e?this.value=isNaN(e)?0:isFinite(e)?e:e<0?-3.4e38:3.4e38:"string"==typeof e?(t=e.match(n.regex.numberAndUnit))&&(this.value=parseFloat(t[1]),"%"==t[5]?this.value/=100:"s"==t[5]&&(this.value*=1e3),this.unit=t[5]):e instanceof n.Number&&(this.value=e.valueOf(),this.unit=e.unit)},extend:{toString:function(){return("%"==this.unit?~~(1e8*this.value)/1e6:"s"==this.unit?this.value/1e3:this.value)+this.unit},toJSON:function(){return this.toString()},valueOf:function(){return this.value},plus:function(e){return e=new n.Number(e),new n.Number(this+e,this.unit||e.unit)},minus:function(e){return e=new n.Number(e),new n.Number(this-e,this.unit||e.unit)},times:function(e){return e=new n.Number(e),new n.Number(this*e,this.unit||e.unit)},divide:function(e){return e=new n.Number(e),new n.Number(this/e,this.unit||e.unit)},to:function(e){var t=new n.Number(this);return"string"==typeof e&&(t.unit=e),t},morph:function(e){return this.destination=new n.Number(e),e.relative&&(this.destination.value+=this.value),this},at:function(e){return this.destination?new n.Number(this.destination).minus(this).times(e).plus(this):this}}}),n.Element=n.invent({create:function(e){this._stroke=n.defaults.attrs.stroke,this._event=null,this.dom={},(this.node=e)&&(this.type=e.nodeName,this.node.instance=this,this._stroke=e.getAttribute("stroke")||this._stroke)},extend:{x:function(e){return this.attr("x",e)},y:function(e){return this.attr("y",e)},cx:function(e){return null==e?this.x()+this.width()/2:this.x(e-this.width()/2)},cy:function(e){return null==e?this.y()+this.height()/2:this.y(e-this.height()/2)},move:function(e,t){return this.x(e).y(t)},center:function(e,t){return this.cx(e).cy(t)},width:function(e){return this.attr("width",e)},height:function(e){return this.attr("height",e)},size:function(e,t){var i=f(this,e,t);return this.width(new n.Number(i.width)).height(new n.Number(i.height))},clone:function(e){this.writeDataToDom();var t=v(this.node.cloneNode(!0));return e?e.add(t):this.after(t),t},remove:function(){return this.parent()&&this.parent().removeElement(this),this},replace:function(e){return this.after(e).remove(),e},addTo:function(e){return e.put(this)},putIn:function(e){return e.add(this)},id:function(e){return this.attr("id",e)},inside:function(e,t){var n=this.bbox();return e>n.x&&t>n.y&&e<n.x+n.width&&t<n.y+n.height},show:function(){return this.style("display","")},hide:function(){return this.style("display","none")},visible:function(){return"none"!=this.style("display")},toString:function(){return this.attr("id")},classes:function(){var e=this.attr("class");return null==e?[]:e.trim().split(n.regex.delimiter)},hasClass:function(e){return-1!=this.classes().indexOf(e)},addClass:function(e){if(!this.hasClass(e)){var t=this.classes();t.push(e),this.attr("class",t.join(" "))}return this},removeClass:function(e){return this.hasClass(e)&&this.attr("class",this.classes().filter(function(t){return t!=e}).join(" ")),this},toggleClass:function(e){return this.hasClass(e)?this.removeClass(e):this.addClass(e)},reference:function(e){return n.get(this.attr(e))},parent:function(t){var i=this;if(!i.node.parentNode)return null;if(i=n.adopt(i.node.parentNode),!t)return i;for(;i&&i.node instanceof e.SVGElement;){if("string"==typeof t?i.matches(t):i instanceof t)return i;if(!i.node.parentNode||"#document"==i.node.parentNode.nodeName)return null;i=n.adopt(i.node.parentNode)}},doc:function(){return this instanceof n.Doc?this:this.parent(n.Doc)},parents:function(e){var t=[],n=this;do{if(!(n=n.parent(e))||!n.node)break;t.push(n)}while(n.parent);return t},matches:function(e){return function(e,t){return(e.matches||e.matchesSelector||e.msMatchesSelector||e.mozMatchesSelector||e.webkitMatchesSelector||e.oMatchesSelector).call(e,t)}(this.node,e)},native:function(){return this.node},svg:function(e){var i=t.createElement("svg");if(!(e&&this instanceof n.Parent))return i.appendChild(e=t.createElement("svg")),this.writeDataToDom(),e.appendChild(this.node.cloneNode(!0)),i.innerHTML.replace(/^<svg>/,"").replace(/<\/svg>$/,"");i.innerHTML="<svg>"+e.replace(/\n/,"").replace(/<([\w:-]+)([^<]+?)\/>/g,"<$1$2></$1>")+"</svg>";for(var r=0,a=i.firstChild.childNodes.length;r<a;r++)this.node.appendChild(i.firstChild.firstChild);return this},writeDataToDom:function(){if(this.each||this.lines){var e=this.each?this:this.lines();e.each(function(){this.writeDataToDom()})}return this.node.removeAttribute("svgjs:data"),Object.keys(this.dom).length&&this.node.setAttribute("svgjs:data",JSON.stringify(this.dom)),this},setData:function(e){return this.dom=e,this},is:function(e){return function(e,t){return e instanceof t}(this,e)}}}),n.easing={"-":function(e){return e},"<>":function(e){return-Math.cos(e*Math.PI)/2+.5},">":function(e){return Math.sin(e*Math.PI/2)},"<":function(e){return 1-Math.cos(e*Math.PI/2)}},n.morph=function(e){return function(t,i){return new n.MorphObj(t,i).at(e)}},n.Situation=n.invent({create:function(e){this.init=!1,this.reversed=!1,this.reversing=!1,this.duration=new n.Number(e.duration).valueOf(),this.delay=new n.Number(e.delay).valueOf(),this.start=+new Date+this.delay,this.finish=this.start+this.duration,this.ease=e.ease,this.loop=0,this.loops=!1,this.animations={},this.attrs={},this.styles={},this.transforms=[],this.once={}}}),n.FX=n.invent({create:function(e){this._target=e,this.situations=[],this.active=!1,this.situation=null,this.paused=!1,this.lastPos=0,this.pos=0,this.absPos=0,this._speed=1},extend:{animate:function(e,t,i){"object"===a(e)&&(t=e.ease,i=e.delay,e=e.duration);var r=new n.Situation({duration:e||1e3,delay:i||0,ease:n.easing[t||"-"]||t});return this.queue(r),this},delay:function(e){var t=new n.Situation({duration:e,delay:0,ease:n.easing["-"]});return this.queue(t)},target:function(e){return e&&e instanceof n.Element?(this._target=e,this):this._target},timeToAbsPos:function(e){return(e-this.situation.start)/(this.situation.duration/this._speed)},absPosToTime:function(e){return this.situation.duration/this._speed*e+this.situation.start},startAnimFrame:function(){this.stopAnimFrame(),this.animationFrame=e.requestAnimationFrame(function(){this.step()}.bind(this))},stopAnimFrame:function(){e.cancelAnimationFrame(this.animationFrame)},start:function(){return!this.active&&this.situation&&(this.active=!0,this.startCurrent()),this},startCurrent:function(){return this.situation.start=+new Date+this.situation.delay/this._speed,this.situation.finish=this.situation.start+this.situation.duration/this._speed,this.initAnimations().step()},queue:function(e){return("function"==typeof e||e instanceof n.Situation)&&this.situations.push(e),this.situation||(this.situation=this.situations.shift()),this},dequeue:function(){return this.stop(),this.situation=this.situations.shift(),this.situation&&(this.situation instanceof n.Situation?this.start():this.situation.call(this)),this},initAnimations:function(){var e,t,i,r=this.situation;if(r.init)return this;for(e in r.animations)for(i=this.target()[e](),Array.isArray(i)||(i=[i]),Array.isArray(r.animations[e])||(r.animations[e]=[r.animations[e]]),t=i.length;t--;)r.animations[e][t]instanceof n.Number&&(i[t]=new n.Number(i[t])),r.animations[e][t]=i[t].morph(r.animations[e][t]);for(e in r.attrs)r.attrs[e]=new n.MorphObj(this.target().attr(e),r.attrs[e]);for(e in r.styles)r.styles[e]=new n.MorphObj(this.target().style(e),r.styles[e]);return r.initialTransformation=this.target().matrixify(),r.init=!0,this},clearQueue:function(){return this.situations=[],this},clearCurrent:function(){return this.situation=null,this},stop:function(e,t){var n=this.active;return this.active=!1,t&&this.clearQueue(),e&&this.situation&&(!n&&this.startCurrent(),this.atEnd()),this.stopAnimFrame(),this.clearCurrent()},reset:function(){if(this.situation){var e=this.situation;this.stop(),this.situation=e,this.atStart()}return this},finish:function(){for(this.stop(!0,!1);this.dequeue().situation&&this.stop(!0,!1););return this.clearQueue().clearCurrent(),this},atStart:function(){return this.at(0,!0)},atEnd:function(){return!0===this.situation.loops&&(this.situation.loops=this.situation.loop+1),"number"==typeof this.situation.loops?this.at(this.situation.loops,!0):this.at(1,!0)},at:function(e,t){var n=this.situation.duration/this._speed;return this.absPos=e,t||(this.situation.reversed&&(this.absPos=1-this.absPos),this.absPos+=this.situation.loop),this.situation.start=+new Date-this.absPos*n,this.situation.finish=this.situation.start+n,this.step(!0)},speed:function(e){return 0===e?this.pause():e?(this._speed=e,this.at(this.absPos,!0)):this._speed},loop:function(e,t){var n=this.last();return n.loops=null==e||e,n.loop=0,t&&(n.reversing=!0),this},pause:function(){return this.paused=!0,this.stopAnimFrame(),this},play:function(){return this.paused?(this.paused=!1,this.at(this.absPos,!0)):this},reverse:function(e){var t=this.last();return t.reversed=void 0===e?!t.reversed:e,this},progress:function(e){return e?this.situation.ease(this.pos):this.pos},after:function(e){var t=this.last();return this.target().on("finished.fx",function n(i){i.detail.situation==t&&(e.call(this,t),this.off("finished.fx",n))}),this._callStart()},during:function(e){var t=this.last(),i=function(i){i.detail.situation==t&&e.call(this,i.detail.pos,n.morph(i.detail.pos),i.detail.eased,t)};return this.target().off("during.fx",i).on("during.fx",i),this.after(function(){this.off("during.fx",i)}),this._callStart()},afterAll:function(e){var t=function t(n){e.call(this),this.off("allfinished.fx",t)};return this.target().off("allfinished.fx",t).on("allfinished.fx",t),this._callStart()},duringAll:function(e){var t=function(t){e.call(this,t.detail.pos,n.morph(t.detail.pos),t.detail.eased,t.detail.situation)};return this.target().off("during.fx",t).on("during.fx",t),this.afterAll(function(){this.off("during.fx",t)}),this._callStart()},last:function(){return this.situations.length?this.situations[this.situations.length-1]:this.situation},add:function(e,t,n){return this.last()[n||"animations"][e]=t,this._callStart()},step:function(e){var t,n,i;e||(this.absPos=this.timeToAbsPos(+new Date)),!1!==this.situation.loops?(t=Math.max(this.absPos,0),n=Math.floor(t),!0===this.situation.loops||n<this.situation.loops?(this.pos=t-n,i=this.situation.loop,this.situation.loop=n):(this.absPos=this.situation.loops,this.pos=1,i=this.situation.loop-1,this.situation.loop=this.situation.loops),this.situation.reversing&&(this.situation.reversed=this.situation.reversed!=Boolean((this.situation.loop-i)%2))):(this.absPos=Math.min(this.absPos,1),this.pos=this.absPos),this.pos<0&&(this.pos=0),this.situation.reversed&&(this.pos=1-this.pos);var r=this.situation.ease(this.pos);for(var a in this.situation.once)a>this.lastPos&&a<=r&&(this.situation.once[a].call(this.target(),this.pos,r),delete this.situation.once[a]);return this.active&&this.target().fire("during",{pos:this.pos,eased:r,fx:this,situation:this.situation}),this.situation?(this.eachAt(),1==this.pos&&!this.situation.reversed||this.situation.reversed&&0==this.pos?(this.stopAnimFrame(),this.target().fire("finished",{fx:this,situation:this.situation}),this.situations.length||(this.target().fire("allfinished"),this.situations.length||(this.target().off(".fx"),this.active=!1)),this.active?this.dequeue():this.clearCurrent()):!this.paused&&this.active&&this.startAnimFrame(),this.lastPos=r,this):this},eachAt:function(){var e,t,i,r=this,a=this.target(),o=this.situation;for(e in o.animations)i=[].concat(o.animations[e]).map(function(e){return"string"!=typeof e&&e.at?e.at(o.ease(r.pos),r.pos):e}),a[e].apply(a,i);for(e in o.attrs)i=[e].concat(o.attrs[e]).map(function(e){return"string"!=typeof e&&e.at?e.at(o.ease(r.pos),r.pos):e}),a.attr.apply(a,i);for(e in o.styles)i=[e].concat(o.styles[e]).map(function(e){return"string"!=typeof e&&e.at?e.at(o.ease(r.pos),r.pos):e}),a.style.apply(a,i);if(o.transforms.length){for(i=o.initialTransformation,e=0,t=o.transforms.length;e<t;e++){var s=o.transforms[e];s instanceof n.Matrix?i=s.relative?i.multiply((new n.Matrix).morph(s).at(o.ease(this.pos))):i.morph(s).at(o.ease(this.pos)):(s.relative||s.undo(i.extract()),i=i.multiply(s.at(o.ease(this.pos))))}a.matrix(i)}return this},once:function(e,t,n){var i=this.last();return n||(e=i.ease(e)),i.once[e]=t,this},_callStart:function(){return setTimeout(function(){this.start()}.bind(this),0),this}},parent:n.Element,construct:{animate:function(e,t,i){return(this.fx||(this.fx=new n.FX(this))).animate(e,t,i)},delay:function(e){return(this.fx||(this.fx=new n.FX(this))).delay(e)},stop:function(e,t){return this.fx&&this.fx.stop(e,t),this},finish:function(){return this.fx&&this.fx.finish(),this},pause:function(){return this.fx&&this.fx.pause(),this},play:function(){return this.fx&&this.fx.play(),this},speed:function(e){if(this.fx){if(null==e)return this.fx.speed();this.fx.speed(e)}return this}}}),n.MorphObj=n.invent({create:function(e,t){return n.Color.isColor(t)?new n.Color(e).morph(t):n.regex.delimiter.test(e)?n.regex.pathLetters.test(e)?new n.PathArray(e).morph(t):new n.Array(e).morph(t):n.regex.numberAndUnit.test(t)?new n.Number(e).morph(t):(this.value=e,void(this.destination=t))},extend:{at:function(e,t){return t<1?this.value:this.destination},valueOf:function(){return this.value}}}),n.extend(n.FX,{attr:function(e,t,n){if("object"===a(e))for(var i in e)this.attr(i,e[i]);else this.add(e,t,"attrs");return this},style:function(e,t){if("object"===a(e))for(var n in e)this.style(n,e[n]);else this.add(e,t,"styles");return this},x:function(e,t){if(this.target()instanceof n.G)return this.transform({x:e},t),this;var i=new n.Number(e);return i.relative=t,this.add("x",i)},y:function(e,t){if(this.target()instanceof n.G)return this.transform({y:e},t),this;var i=new n.Number(e);return i.relative=t,this.add("y",i)},cx:function(e){return this.add("cx",new n.Number(e))},cy:function(e){return this.add("cy",new n.Number(e))},move:function(e,t){return this.x(e).y(t)},center:function(e,t){return this.cx(e).cy(t)},size:function(e,t){var i;return this.target()instanceof n.Text?this.attr("font-size",e):(e&&t||(i=this.target().bbox()),e||(e=i.width/i.height*t),t||(t=i.height/i.width*e),this.add("width",new n.Number(e)).add("height",new n.Number(t))),this},width:function(e){return this.add("width",new n.Number(e))},height:function(e){return this.add("height",new n.Number(e))},plot:function(e,t,n,i){return 4==arguments.length?this.plot([e,t,n,i]):this.add("plot",new(this.target().morphArray)(e))},leading:function(e){return this.target().leading?this.add("leading",new n.Number(e)):this},viewbox:function(e,t,i,r){return this.target()instanceof n.Container&&this.add("viewbox",new n.ViewBox(e,t,i,r)),this},update:function(e){if(this.target()instanceof n.Stop){if("number"==typeof e||e instanceof n.Number)return this.update({offset:arguments[0],color:arguments[1],opacity:arguments[2]});null!=e.opacity&&this.attr("stop-opacity",e.opacity),null!=e.color&&this.attr("stop-color",e.color),null!=e.offset&&this.attr("offset",e.offset)}return this}}),n.Box=n.invent({create:function(e,t,i,r){if(!("object"!==a(e)||e instanceof n.Element))return n.Box.call(this,null!=e.left?e.left:e.x,null!=e.top?e.top:e.y,e.width,e.height);4==arguments.length&&(this.x=e,this.y=t,this.width=i,this.height=r),y(this)},extend:{merge:function(e){var t=new this.constructor;return t.x=Math.min(this.x,e.x),t.y=Math.min(this.y,e.y),t.width=Math.max(this.x+this.width,e.x+e.width)-t.x,t.height=Math.max(this.y+this.height,e.y+e.height)-t.y,y(t)},transform:function(e){var t,i=1/0,r=-1/0,a=1/0,o=-1/0,s=[new n.Point(this.x,this.y),new n.Point(this.x2,this.y),new n.Point(this.x,this.y2),new n.Point(this.x2,this.y2)];return s.forEach(function(t){t=t.transform(e),i=Math.min(i,t.x),r=Math.max(r,t.x),a=Math.min(a,t.y),o=Math.max(o,t.y)}),(t=new this.constructor).x=i,t.width=r-i,t.y=a,t.height=o-a,y(t),t}}}),n.BBox=n.invent({create:function(e){if(n.Box.apply(this,[].slice.call(arguments)),e instanceof n.Element){var i;try{if(!t.documentElement.contains){for(var r=e.node;r.parentNode;)r=r.parentNode;if(r!=t)throw new Error("Element not in the dom")}i=e.node.getBBox()}catch(t){if(e instanceof n.Shape){n.parser.draw||n.prepare();var a=e.clone(n.parser.draw.instance).show();i=a.node.getBBox(),a.remove()}else i={x:e.node.clientLeft,y:e.node.clientTop,width:e.node.clientWidth,height:e.node.clientHeight}}n.Box.call(this,i)}},inherit:n.Box,parent:n.Element,construct:{bbox:function(){return new n.BBox(this)}}}),n.BBox.prototype.constructor=n.BBox,n.extend(n.Element,{tbox:function(){return console.warn("Use of TBox is deprecated and mapped to RBox. Use .rbox() instead."),this.rbox(this.doc())}}),n.RBox=n.invent({create:function(e){n.Box.apply(this,[].slice.call(arguments)),e instanceof n.Element&&n.Box.call(this,e.node.getBoundingClientRect())},inherit:n.Box,parent:n.Element,extend:{addOffset:function(){return this.x+=e.pageXOffset,this.y+=e.pageYOffset,this}},construct:{rbox:function(e){return e?new n.RBox(this).transform(e.screenCTM().inverse()):new n.RBox(this).addOffset()}}}),n.RBox.prototype.constructor=n.RBox,n.Matrix=n.invent({create:function(e){var t,i=g([1,0,0,1,0,0]);for(e=e instanceof n.Element?e.matrixify():"string"==typeof e?g(e.split(n.regex.delimiter).map(parseFloat)):6==arguments.length?g([].slice.call(arguments)):Array.isArray(e)?g(e):"object"===a(e)?e:i,t=x.length-1;t>=0;--t)this[x[t]]=null!=e[x[t]]?e[x[t]]:i[x[t]]},extend:{extract:function(){var e=p(this,0,1),t=p(this,1,0),i=180/Math.PI*Math.atan2(e.y,e.x)-90;return{x:this.e,y:this.f,transformedX:(this.e*Math.cos(i*Math.PI/180)+this.f*Math.sin(i*Math.PI/180))/Math.sqrt(this.a*this.a+this.b*this.b),transformedY:(this.f*Math.cos(i*Math.PI/180)+this.e*Math.sin(-i*Math.PI/180))/Math.sqrt(this.c*this.c+this.d*this.d),skewX:-i,skewY:180/Math.PI*Math.atan2(t.y,t.x),scaleX:Math.sqrt(this.a*this.a+this.b*this.b),scaleY:Math.sqrt(this.c*this.c+this.d*this.d),rotation:i,a:this.a,b:this.b,c:this.c,d:this.d,e:this.e,f:this.f,matrix:new n.Matrix(this)}},clone:function(){return new n.Matrix(this)},morph:function(e){return this.destination=new n.Matrix(e),this},at:function(e){if(!this.destination)return this;var t=new n.Matrix({a:this.a+(this.destination.a-this.a)*e,b:this.b+(this.destination.b-this.b)*e,c:this.c+(this.destination.c-this.c)*e,d:this.d+(this.destination.d-this.d)*e,e:this.e+(this.destination.e-this.e)*e,f:this.f+(this.destination.f-this.f)*e});return t},multiply:function(e){return new n.Matrix(this.native().multiply(function(e){return e instanceof n.Matrix||(e=new n.Matrix(e)),e}(e).native()))},inverse:function(){return new n.Matrix(this.native().inverse())},translate:function(e,t){return new n.Matrix(this.native().translate(e||0,t||0))},scale:function(e,t,i,r){return 1==arguments.length?t=e:3==arguments.length&&(r=i,i=t,t=e),this.around(i,r,new n.Matrix(e,0,0,t,0,0))},rotate:function(e,t,i){return e=n.utils.radians(e),this.around(t,i,new n.Matrix(Math.cos(e),Math.sin(e),-Math.sin(e),Math.cos(e),0,0))},flip:function(e,t){return"x"==e?this.scale(-1,1,t,0):"y"==e?this.scale(1,-1,0,t):this.scale(-1,-1,e,null!=t?t:e)},skew:function(e,t,i,r){return 1==arguments.length?t=e:3==arguments.length&&(r=i,i=t,t=e),e=n.utils.radians(e),t=n.utils.radians(t),this.around(i,r,new n.Matrix(1,Math.tan(t),Math.tan(e),1,0,0))},skewX:function(e,t,n){return this.skew(e,0,t,n)},skewY:function(e,t,n){return this.skew(0,e,t,n)},around:function(e,t,i){return this.multiply(new n.Matrix(1,0,0,1,e||0,t||0)).multiply(i).multiply(new n.Matrix(1,0,0,1,-e||0,-t||0))},native:function(){for(var e=n.parser.native.createSVGMatrix(),t=x.length-1;t>=0;t--)e[x[t]]=this[x[t]];return e},toString:function(){return"matrix("+b(this.a)+","+b(this.b)+","+b(this.c)+","+b(this.d)+","+b(this.e)+","+b(this.f)+")"}},parent:n.Element,construct:{ctm:function(){return new n.Matrix(this.node.getCTM())},screenCTM:function(){if(this instanceof n.Nested){var e=this.rect(1,1),t=e.node.getScreenCTM();return e.remove(),new n.Matrix(t)}return new n.Matrix(this.node.getScreenCTM())}}}),n.Point=n.invent({create:function(e,t){var n;n=Array.isArray(e)?{x:e[0],y:e[1]}:"object"===a(e)?{x:e.x,y:e.y}:null!=e?{x:e,y:null!=t?t:e}:{x:0,y:0},this.x=n.x,this.y=n.y},extend:{clone:function(){return new n.Point(this)},morph:function(e,t){return this.destination=new n.Point(e,t),this},at:function(e){if(!this.destination)return this;var t=new n.Point({x:this.x+(this.destination.x-this.x)*e,y:this.y+(this.destination.y-this.y)*e});return t},native:function(){var e=n.parser.native.createSVGPoint();return e.x=this.x,e.y=this.y,e},transform:function(e){return new n.Point(this.native().matrixTransform(e.native()))}}}),n.extend(n.Element,{point:function(e,t){return new n.Point(e,t).transform(this.screenCTM().inverse())}}),n.extend(n.Element,{attr:function(e,t,i){if(null==e){for(e={},t=this.node.attributes,i=t.length-1;i>=0;i--)e[t[i].nodeName]=n.regex.isNumber.test(t[i].nodeValue)?parseFloat(t[i].nodeValue):t[i].nodeValue;return e}if("object"===a(e))for(t in e)this.attr(t,e[t]);else if(null===t)this.node.removeAttribute(e);else{if(null==t)return null==(t=this.node.getAttribute(e))?n.defaults.attrs[e]:n.regex.isNumber.test(t)?parseFloat(t):t;"stroke-width"==e?this.attr("stroke",parseFloat(t)>0?this._stroke:null):"stroke"==e&&(this._stroke=t),"fill"!=e&&"stroke"!=e||(n.regex.isImage.test(t)&&(t=this.doc().defs().image(t,0,0)),t instanceof n.Image&&(t=this.doc().defs().pattern(0,0,function(){this.add(t)}))),"number"==typeof t?t=new n.Number(t):n.Color.isColor(t)?t=new n.Color(t):Array.isArray(t)&&(t=new n.Array(t)),"leading"==e?this.leading&&this.leading(t):"string"==typeof i?this.node.setAttributeNS(i,e,t.toString()):this.node.setAttribute(e,t.toString()),!this.rebuild||"font-size"!=e&&"x"!=e||this.rebuild(e,t)}return this}}),n.extend(n.Element,{transform:function(e,t){var i,r;if("object"!==a(e))return i=new n.Matrix(this).extract(),"string"==typeof e?i[e]:i;if(i=new n.Matrix(this),t=!!t||!!e.relative,null!=e.a)i=t?i.multiply(new n.Matrix(e)):new n.Matrix(e);else if(null!=e.rotation)m(e,this),i=t?i.rotate(e.rotation,e.cx,e.cy):i.rotate(e.rotation-i.extract().rotation,e.cx,e.cy);else if(null!=e.scale||null!=e.scaleX||null!=e.scaleY){if(m(e,this),e.scaleX=null!=e.scale?e.scale:null!=e.scaleX?e.scaleX:1,e.scaleY=null!=e.scale?e.scale:null!=e.scaleY?e.scaleY:1,!t){var o=i.extract();e.scaleX=1*e.scaleX/o.scaleX,e.scaleY=1*e.scaleY/o.scaleY}i=i.scale(e.scaleX,e.scaleY,e.cx,e.cy)}else if(null!=e.skew||null!=e.skewX||null!=e.skewY){if(m(e,this),e.skewX=null!=e.skew?e.skew:null!=e.skewX?e.skewX:0,e.skewY=null!=e.skew?e.skew:null!=e.skewY?e.skewY:0,!t){var o=i.extract();i=i.multiply((new n.Matrix).skew(o.skewX,o.skewY,e.cx,e.cy).inverse())}i=i.skew(e.skewX,e.skewY,e.cx,e.cy)}else e.flip?("x"==e.flip||"y"==e.flip?e.offset=null==e.offset?this.bbox()["c"+e.flip]:e.offset:null==e.offset?(r=this.bbox(),e.flip=r.cx,e.offset=r.cy):e.flip=e.offset,i=(new n.Matrix).flip(e.flip,e.offset)):null==e.x&&null==e.y||(t?i=i.translate(e.x,e.y):(null!=e.x&&(i.e=e.x),null!=e.y&&(i.f=e.y)));return this.attr("transform",i)}}),n.extend(n.FX,{transform:function(e,t){var i,r,o=this.target();return"object"!==a(e)?(i=new n.Matrix(o).extract(),"string"==typeof e?i[e]:i):(t=!!t||!!e.relative,null!=e.a?i=new n.Matrix(e):null!=e.rotation?(m(e,o),i=new n.Rotate(e.rotation,e.cx,e.cy)):null!=e.scale||null!=e.scaleX||null!=e.scaleY?(m(e,o),e.scaleX=null!=e.scale?e.scale:null!=e.scaleX?e.scaleX:1,e.scaleY=null!=e.scale?e.scale:null!=e.scaleY?e.scaleY:1,i=new n.Scale(e.scaleX,e.scaleY,e.cx,e.cy)):null!=e.skewX||null!=e.skewY?(m(e,o),e.skewX=null!=e.skewX?e.skewX:0,e.skewY=null!=e.skewY?e.skewY:0,i=new n.Skew(e.skewX,e.skewY,e.cx,e.cy)):e.flip?("x"==e.flip||"y"==e.flip?e.offset=null==e.offset?o.bbox()["c"+e.flip]:e.offset:null==e.offset?(r=o.bbox(),e.flip=r.cx,e.offset=r.cy):e.flip=e.offset,i=(new n.Matrix).flip(e.flip,e.offset)):null==e.x&&null==e.y||(i=new n.Translate(e.x,e.y)),i?(i.relative=t,this.last().transforms.push(i),this._callStart()):this)}}),n.extend(n.Element,{untransform:function(){return this.attr("transform",null)},matrixify:function(){var e=(this.attr("transform")||"").split(n.regex.transforms).slice(0,-1).map(function(e){var t=e.trim().split("(");return[t[0],t[1].split(n.regex.delimiter).map(function(e){return parseFloat(e)})]}).reduce(function(e,t){return"matrix"==t[0]?e.multiply(g(t[1])):e[t[0]].apply(e,t[1])},new n.Matrix);return e},toParent:function(e){if(this==e)return this;var t=this.screenCTM(),n=e.screenCTM().inverse();return this.addTo(e).untransform().transform(n.multiply(t)),this},toDoc:function(){return this.toParent(this.doc())}}),n.Transformation=n.invent({create:function(e,t){if(arguments.length>1&&"boolean"!=typeof t)return this.constructor.call(this,[].slice.call(arguments));if(Array.isArray(e))for(var n=0,i=this.arguments.length;n<i;++n)this[this.arguments[n]]=e[n];else if("object"===a(e))for(var n=0,i=this.arguments.length;n<i;++n)this[this.arguments[n]]=e[this.arguments[n]];this.inversed=!1,!0===t&&(this.inversed=!0)},extend:{arguments:[],method:"",at:function(e){for(var t=[],i=0,r=this.arguments.length;i<r;++i)t.push(this[this.arguments[i]]);var a=this._undo||new n.Matrix;return a=(new n.Matrix).morph(n.Matrix.prototype[this.method].apply(a,t)).at(e),this.inversed?a.inverse():a},undo:function(e){for(var t=0,i=this.arguments.length;t<i;++t)e[this.arguments[t]]=void 0===this[this.arguments[t]]?0:e[this.arguments[t]];return e.cx=this.cx,e.cy=this.cy,this._undo=new(n[h(this.method)])(e,!0).at(1),this}}}),n.Translate=n.invent({parent:n.Matrix,inherit:n.Transformation,create:function(e,t){this.constructor.apply(this,[].slice.call(arguments))},extend:{arguments:["transformedX","transformedY"],method:"translate"}}),n.Rotate=n.invent({parent:n.Matrix,inherit:n.Transformation,create:function(e,t){this.constructor.apply(this,[].slice.call(arguments))},extend:{arguments:["rotation","cx","cy"],method:"rotate",at:function(e){var t=(new n.Matrix).rotate((new n.Number).morph(this.rotation-(this._undo?this._undo.rotation:0)).at(e),this.cx,this.cy);return this.inversed?t.inverse():t},undo:function(e){return this._undo=e,this}}}),n.Scale=n.invent({parent:n.Matrix,inherit:n.Transformation,create:function(e,t){this.constructor.apply(this,[].slice.call(arguments))},extend:{arguments:["scaleX","scaleY","cx","cy"],method:"scale"}}),n.Skew=n.invent({parent:n.Matrix,inherit:n.Transformation,create:function(e,t){this.constructor.apply(this,[].slice.call(arguments))},extend:{arguments:["skewX","skewY","cx","cy"],method:"skew"}}),n.extend(n.Element,{style:function(e,t){if(0==arguments.length)return this.node.style.cssText||"";if(arguments.length<2)if("object"===a(e))for(t in e)this.style(t,e[t]);else{if(!n.regex.isCss.test(e))return this.node.style[u(e)];for(e=e.split(/\s*;\s*/).filter(function(e){return!!e}).map(function(e){return e.split(/\s*:\s*/)});t=e.pop();)this.style(t[0],t[1])}else this.node.style[u(e)]=null===t||n.regex.isBlank.test(t)?"":t;return this}}),n.Parent=n.invent({create:function(e){this.constructor.call(this,e)},inherit:n.Element,extend:{children:function(){return n.utils.map(n.utils.filterSVGElements(this.node.childNodes),function(e){return n.adopt(e)})},add:function(e,t){return null==t?this.node.appendChild(e.node):e.node!=this.node.childNodes[t]&&this.node.insertBefore(e.node,this.node.childNodes[t]),this},put:function(e,t){return this.add(e,t),e},has:function(e){return this.index(e)>=0},index:function(e){return[].slice.call(this.node.childNodes).indexOf(e.node)},get:function(e){return n.adopt(this.node.childNodes[e])},first:function(){return this.get(0)},last:function(){return this.get(this.node.childNodes.length-1)},each:function(e,t){var i,r,a=this.children();for(i=0,r=a.length;i<r;i++)a[i]instanceof n.Element&&e.apply(a[i],[i,a]),t&&a[i]instanceof n.Container&&a[i].each(e,t);return this},removeElement:function(e){return this.node.removeChild(e.node),this},clear:function(){for(;this.node.hasChildNodes();)this.node.removeChild(this.node.lastChild);return delete this._defs,this},defs:function(){return this.doc().defs()}}}),n.extend(n.Parent,{ungroup:function(e,t){return 0===t||this instanceof n.Defs||this.node==n.parser.draw?this:(e=e||(this instanceof n.Doc?this:this.parent(n.Parent)),t=t||1/0,this.each(function(){return this instanceof n.Defs?this:this instanceof n.Parent?this.ungroup(e,t-1):this.toParent(e)}),this.node.firstChild||this.remove(),this)},flatten:function(e,t){return this.ungroup(e,t)}}),n.Container=n.invent({create:function(e){this.constructor.call(this,e)},inherit:n.Parent}),n.ViewBox=n.invent({create:function(e){var t,i,r,o,s,l,c,u=1,h=1,d=/[+-]?(?:\d+(?:\.\d*)?|\.\d+)(?:e[+-]?\d+)?/gi;if(e instanceof n.Element){for(l=e,c=e,s=(e.attr("viewBox")||"").match(d),e.bbox,r=new n.Number(e.width()),o=new n.Number(e.height());"%"==r.unit;)u*=r.value,r=new n.Number(l instanceof n.Doc?l.parent().offsetWidth:l.parent().width()),l=l.parent();for(;"%"==o.unit;)h*=o.value,o=new n.Number(c instanceof n.Doc?c.parent().offsetHeight:c.parent().height()),c=c.parent();this.x=0,this.y=0,this.width=r*u,this.height=o*h,this.zoom=1,s&&(t=parseFloat(s[0]),i=parseFloat(s[1]),r=parseFloat(s[2]),o=parseFloat(s[3]),this.zoom=this.width/this.height>r/o?this.height/o:this.width/r,this.x=t,this.y=i,this.width=r,this.height=o)}else e="string"==typeof e?e.match(d).map(function(e){return parseFloat(e)}):Array.isArray(e)?e:"object"===a(e)?[e.x,e.y,e.width,e.height]:4==arguments.length?[].slice.call(arguments):[0,0,0,0],this.x=e[0],this.y=e[1],this.width=e[2],this.height=e[3]},extend:{toString:function(){return this.x+" "+this.y+" "+this.width+" "+this.height},morph:function(e,t,i,r){return this.destination=new n.ViewBox(e,t,i,r),this},at:function(e){return this.destination?new n.ViewBox([this.x+(this.destination.x-this.x)*e,this.y+(this.destination.y-this.y)*e,this.width+(this.destination.width-this.width)*e,this.height+(this.destination.height-this.height)*e]):this}},parent:n.Container,construct:{viewbox:function(e,t,i,r){return 0==arguments.length?new n.ViewBox(this):this.attr("viewBox",new n.ViewBox(e,t,i,r))}}}),["click","dblclick","mousedown","mouseup","mouseover","mouseout","mousemove","touchstart","touchmove","touchleave","touchend","touchcancel"].forEach(function(e){n.Element.prototype[e]=function(t){return n.on(this.node,e,t),this}}),n.listeners=[],n.handlerMap=[],n.listenerId=0,n.on=function(e,t,i,r,a){var o=i.bind(r||e.instance||e),s=(n.handlerMap.indexOf(e)+1||n.handlerMap.push(e))-1,l=t.split(".")[0],c=t.split(".")[1]||"*";n.listeners[s]=n.listeners[s]||{},n.listeners[s][l]=n.listeners[s][l]||{},n.listeners[s][l][c]=n.listeners[s][l][c]||{},i._svgjsListenerId||(i._svgjsListenerId=++n.listenerId),n.listeners[s][l][c][i._svgjsListenerId]=o,e.addEventListener(l,o,a||!1)},n.off=function(e,t,i){var r=n.handlerMap.indexOf(e),a=t&&t.split(".")[0],o=t&&t.split(".")[1],s="";if(-1!=r)if(i){if("function"==typeof i&&(i=i._svgjsListenerId),!i)return;n.listeners[r][a]&&n.listeners[r][a][o||"*"]&&(e.removeEventListener(a,n.listeners[r][a][o||"*"][i],!1),delete n.listeners[r][a][o||"*"][i])}else if(o&&a){if(n.listeners[r][a]&&n.listeners[r][a][o]){for(i in n.listeners[r][a][o])n.off(e,[a,o].join("."),i);delete n.listeners[r][a][o]}}else if(o)for(t in n.listeners[r])for(s in n.listeners[r][t])o===s&&n.off(e,[t,o].join("."));else if(a){if(n.listeners[r][a]){for(s in n.listeners[r][a])n.off(e,[a,s].join("."));delete n.listeners[r][a]}}else{for(t in n.listeners[r])n.off(e,t);delete n.listeners[r],delete n.handlerMap[r]}},n.extend(n.Element,{on:function(e,t,i,r){return n.on(this.node,e,t,i,r),this},off:function(e,t){return n.off(this.node,e,t),this},fire:function(t,i){return t instanceof e.Event?this.node.dispatchEvent(t):this.node.dispatchEvent(t=new n.CustomEvent(t,{detail:i,cancelable:!0})),this._event=t,this},event:function(){return this._event}}),n.Defs=n.invent({create:"defs",inherit:n.Container}),n.G=n.invent({create:"g",inherit:n.Container,extend:{x:function(e){return null==e?this.transform("x"):this.transform({x:e-this.x()},!0)},y:function(e){return null==e?this.transform("y"):this.transform({y:e-this.y()},!0)},cx:function(e){return null==e?this.gbox().cx:this.x(e-this.gbox().width/2)},cy:function(e){return null==e?this.gbox().cy:this.y(e-this.gbox().height/2)},gbox:function(){var e=this.bbox(),t=this.transform();return e.x+=t.x,e.x2+=t.x,e.cx+=t.x,e.y+=t.y,e.y2+=t.y,e.cy+=t.y,e}},construct:{group:function(){return this.put(new n.G)}}}),n.Doc=n.invent({create:function(e){e&&("svg"==(e="string"==typeof e?t.getElementById(e):e).nodeName?this.constructor.call(this,e):(this.constructor.call(this,n.create("svg")),e.appendChild(this.node),this.size("100%","100%")),this.namespace().defs())},inherit:n.Container,extend:{namespace:function(){return this.attr({xmlns:n.ns,version:"1.1"}).attr("xmlns:xlink",n.xlink,n.xmlns).attr("xmlns:svgjs",n.svgjs,n.xmlns)},defs:function(){var e;return this._defs||((e=this.node.getElementsByTagName("defs")[0])?this._defs=n.adopt(e):this._defs=new n.Defs,this.node.appendChild(this._defs.node)),this._defs},parent:function(){return this.node.parentNode&&"#document"!=this.node.parentNode.nodeName?this.node.parentNode:null},spof:function(){var e=this.node.getScreenCTM();return e&&this.style("left",-e.e%1+"px").style("top",-e.f%1+"px"),this},remove:function(){return this.parent()&&this.parent().removeChild(this.node),this},clear:function(){for(;this.node.hasChildNodes();)this.node.removeChild(this.node.lastChild);return delete this._defs,n.parser.draw&&!n.parser.draw.parentNode&&this.node.appendChild(n.parser.draw),this},clone:function(e){this.writeDataToDom();var t=this.node,n=v(t.cloneNode(!0));return e?(e.node||e).appendChild(n.node):t.parentNode.insertBefore(n.node,t.nextSibling),n}}}),n.extend(n.Element,{siblings:function(){return this.parent().children()},position:function(){return this.parent().index(this)},next:function(){return this.siblings()[this.position()+1]},previous:function(){return this.siblings()[this.position()-1]},forward:function(){var e=this.position()+1,t=this.parent();return t.removeElement(this).add(this,e),t instanceof n.Doc&&t.node.appendChild(t.defs().node),this},backward:function(){var e=this.position();return e>0&&this.parent().removeElement(this).add(this,e-1),this},front:function(){var e=this.parent();return e.node.appendChild(this.node),e instanceof n.Doc&&e.node.appendChild(e.defs().node),this},back:function(){return this.position()>0&&this.parent().removeElement(this).add(this,0),this},before:function(e){e.remove();var t=this.position();return this.parent().add(e,t),this},after:function(e){e.remove();var t=this.position();return this.parent().add(e,t+1),this}}),n.Mask=n.invent({create:function(){this.constructor.call(this,n.create("mask")),this.targets=[]},inherit:n.Container,extend:{remove:function(){for(var e=this.targets.length-1;e>=0;e--)this.targets[e]&&this.targets[e].unmask();return this.targets=[],n.Element.prototype.remove.call(this),this}},construct:{mask:function(){return this.defs().put(new n.Mask)}}}),n.extend(n.Element,{maskWith:function(e){return this.masker=e instanceof n.Mask?e:this.parent().mask().add(e),this.masker.targets.push(this),this.attr("mask",'url("#'+this.masker.attr("id")+'")')},unmask:function(){return delete this.masker,this.attr("mask",null)}}),n.ClipPath=n.invent({create:function(){this.constructor.call(this,n.create("clipPath")),this.targets=[]},inherit:n.Container,extend:{remove:function(){for(var e=this.targets.length-1;e>=0;e--)this.targets[e]&&this.targets[e].unclip();return this.targets=[],this.parent().removeElement(this),this}},construct:{clip:function(){return this.defs().put(new n.ClipPath)}}}),n.extend(n.Element,{clipWith:function(e){return this.clipper=e instanceof n.ClipPath?e:this.parent().clip().add(e),this.clipper.targets.push(this),this.attr("clip-path",'url("#'+this.clipper.attr("id")+'")')},unclip:function(){return delete this.clipper,this.attr("clip-path",null)}}),n.Gradient=n.invent({create:function(e){this.constructor.call(this,n.create(e+"Gradient")),this.type=e},inherit:n.Container,extend:{at:function(e,t,i){return this.put(new n.Stop).update(e,t,i)},update:function(e){return this.clear(),"function"==typeof e&&e.call(this,this),this},fill:function(){return"url(#"+this.id()+")"},toString:function(){return this.fill()},attr:function(e,t,i){return"transform"==e&&(e="gradientTransform"),n.Container.prototype.attr.call(this,e,t,i)}},construct:{gradient:function(e,t){return this.defs().gradient(e,t)}}}),n.extend(n.Gradient,n.FX,{from:function(e,t){return"radial"==(this._target||this).type?this.attr({fx:new n.Number(e),fy:new n.Number(t)}):this.attr({x1:new n.Number(e),y1:new n.Number(t)})},to:function(e,t){return"radial"==(this._target||this).type?this.attr({cx:new n.Number(e),cy:new n.Number(t)}):this.attr({x2:new n.Number(e),y2:new n.Number(t)})}}),n.extend(n.Defs,{gradient:function(e,t){return this.put(new n.Gradient(e)).update(t)}}),n.Stop=n.invent({create:"stop",inherit:n.Element,extend:{update:function(e){return("number"==typeof e||e instanceof n.Number)&&(e={offset:arguments[0],color:arguments[1],opacity:arguments[2]}),null!=e.opacity&&this.attr("stop-opacity",e.opacity),null!=e.color&&this.attr("stop-color",e.color),null!=e.offset&&this.attr("offset",new n.Number(e.offset)),this}}}),n.Pattern=n.invent({create:"pattern",inherit:n.Container,extend:{fill:function(){return"url(#"+this.id()+")"},update:function(e){return this.clear(),"function"==typeof e&&e.call(this,this),this},toString:function(){return this.fill()},attr:function(e,t,i){return"transform"==e&&(e="patternTransform"),n.Container.prototype.attr.call(this,e,t,i)}},construct:{pattern:function(e,t,n){return this.defs().pattern(e,t,n)}}}),n.extend(n.Defs,{pattern:function(e,t,i){return this.put(new n.Pattern).update(i).attr({x:0,y:0,width:e,height:t,patternUnits:"userSpaceOnUse"})}}),n.Shape=n.invent({create:function(e){this.constructor.call(this,e)},inherit:n.Element}),n.Bare=n.invent({create:function(e,t){if(this.constructor.call(this,n.create(e)),t)for(var i in t.prototype)"function"==typeof t.prototype[i]&&(this[i]=t.prototype[i])},inherit:n.Element,extend:{words:function(e){for(;this.node.hasChildNodes();)this.node.removeChild(this.node.lastChild);return this.node.appendChild(t.createTextNode(e)),this}}}),n.extend(n.Parent,{element:function(e,t){return this.put(new n.Bare(e,t))}}),n.Symbol=n.invent({create:"symbol",inherit:n.Container,construct:{symbol:function(){return this.put(new n.Symbol)}}}),n.Use=n.invent({create:"use",inherit:n.Shape,extend:{element:function(e,t){return this.attr("href",(t||"")+"#"+e,n.xlink)}},construct:{use:function(e,t){return this.put(new n.Use).element(e,t)}}}),n.Rect=n.invent({create:"rect",inherit:n.Shape,construct:{rect:function(e,t){return this.put(new n.Rect).size(e,t)}}}),n.Circle=n.invent({create:"circle",inherit:n.Shape,construct:{circle:function(e){return this.put(new n.Circle).rx(new n.Number(e).divide(2)).move(0,0)}}}),n.extend(n.Circle,n.FX,{rx:function(e){return this.attr("r",e)},ry:function(e){return this.rx(e)}}),n.Ellipse=n.invent({create:"ellipse",inherit:n.Shape,construct:{ellipse:function(e,t){return this.put(new n.Ellipse).size(e,t).move(0,0)}}}),n.extend(n.Ellipse,n.Rect,n.FX,{rx:function(e){return this.attr("rx",e)},ry:function(e){return this.attr("ry",e)}}),n.extend(n.Circle,n.Ellipse,{x:function(e){return null==e?this.cx()-this.rx():this.cx(e+this.rx())},y:function(e){return null==e?this.cy()-this.ry():this.cy(e+this.ry())},cx:function(e){return null==e?this.attr("cx"):this.attr("cx",e)},cy:function(e){return null==e?this.attr("cy"):this.attr("cy",e)},width:function(e){return null==e?2*this.rx():this.rx(new n.Number(e).divide(2))},height:function(e){return null==e?2*this.ry():this.ry(new n.Number(e).divide(2))},size:function(e,t){var i=f(this,e,t);return this.rx(new n.Number(i.width).divide(2)).ry(new n.Number(i.height).divide(2))}}),n.Line=n.invent({create:"line",inherit:n.Shape,extend:{array:function(){return new n.PointArray([[this.attr("x1"),this.attr("y1")],[this.attr("x2"),this.attr("y2")]])},plot:function(e,t,i,r){return null==e?this.array():(e=void 0!==t?{x1:e,y1:t,x2:i,y2:r}:new n.PointArray(e).toLine(),this.attr(e))},move:function(e,t){return this.attr(this.array().move(e,t).toLine())},size:function(e,t){var n=f(this,e,t);return this.attr(this.array().size(n.width,n.height).toLine())}},construct:{line:function(e,t,i,r){return n.Line.prototype.plot.apply(this.put(new n.Line),null!=e?[e,t,i,r]:[0,0,0,0])}}}),n.Polyline=n.invent({create:"polyline",inherit:n.Shape,construct:{polyline:function(e){return this.put(new n.Polyline).plot(e||new n.PointArray)}}}),n.Polygon=n.invent({create:"polygon",inherit:n.Shape,construct:{polygon:function(e){return this.put(new n.Polygon).plot(e||new n.PointArray)}}}),n.extend(n.Polyline,n.Polygon,{array:function(){return this._array||(this._array=new n.PointArray(this.attr("points")))},plot:function(e){return null==e?this.array():this.clear().attr("points","string"==typeof e?e:this._array=new n.PointArray(e))},clear:function(){return delete this._array,this},move:function(e,t){return this.attr("points",this.array().move(e,t))},size:function(e,t){var n=f(this,e,t);return this.attr("points",this.array().size(n.width,n.height))}}),n.extend(n.Line,n.Polyline,n.Polygon,{morphArray:n.PointArray,x:function(e){return null==e?this.bbox().x:this.move(e,this.bbox().y)},y:function(e){return null==e?this.bbox().y:this.move(this.bbox().x,e)},width:function(e){var t=this.bbox();return null==e?t.width:this.size(e,t.height)},height:function(e){var t=this.bbox();return null==e?t.height:this.size(t.width,e)}}),n.Path=n.invent({create:"path",inherit:n.Shape,extend:{morphArray:n.PathArray,array:function(){return this._array||(this._array=new n.PathArray(this.attr("d")))},plot:function(e){return null==e?this.array():this.clear().attr("d","string"==typeof e?e:this._array=new n.PathArray(e))},clear:function(){return delete this._array,this},move:function(e,t){return this.attr("d",this.array().move(e,t))},x:function(e){return null==e?this.bbox().x:this.move(e,this.bbox().y)},y:function(e){return null==e?this.bbox().y:this.move(this.bbox().x,e)},size:function(e,t){var n=f(this,e,t);return this.attr("d",this.array().size(n.width,n.height))},width:function(e){return null==e?this.bbox().width:this.size(e,this.bbox().height)},height:function(e){return null==e?this.bbox().height:this.size(this.bbox().width,e)}},construct:{path:function(e){return this.put(new n.Path).plot(e||new n.PathArray)}}}),n.Image=n.invent({create:"image",inherit:n.Shape,extend:{load:function(t){if(!t)return this;var i=this,r=new e.Image;return n.on(r,"load",function(){n.off(r);var e=i.parent(n.Pattern);null!==e&&(0==i.width()&&0==i.height()&&i.size(r.width,r.height),e&&0==e.width()&&0==e.height()&&e.size(i.width(),i.height()),"function"==typeof i._loaded&&i._loaded.call(i,{width:r.width,height:r.height,ratio:r.width/r.height,url:t}))}),n.on(r,"error",function(e){n.off(r),"function"==typeof i._error&&i._error.call(i,e)}),this.attr("href",r.src=this.src=t,n.xlink)},loaded:function(e){return this._loaded=e,this},error:function(e){return this._error=e,this}},construct:{image:function(e,t,i){return this.put(new n.Image).load(e).size(t||0,i||t||0)}}}),n.Text=n.invent({create:function(){this.constructor.call(this,n.create("text")),this.dom.leading=new n.Number(1.3),this._rebuild=!0,this._build=!1,this.attr("font-family",n.defaults.attrs["font-family"])},inherit:n.Shape,extend:{x:function(e){return null==e?this.attr("x"):this.attr("x",e)},y:function(e){var t=this.attr("y"),n="number"==typeof t?t-this.bbox().y:0;return null==e?"number"==typeof t?t-n:t:this.attr("y","number"==typeof e.valueOf()?e+n:e)},cx:function(e){return null==e?this.bbox().cx:this.x(e-this.bbox().width/2)},cy:function(e){return null==e?this.bbox().cy:this.y(e-this.bbox().height/2)},text:function(e){if(void 0===e){for(var e="",t=this.node.childNodes,i=0,r=t.length;i<r;++i)0!=i&&3!=t[i].nodeType&&1==n.adopt(t[i]).dom.newLined&&(e+="\n"),e+=t[i].textContent;return e}if(this.clear().build(!0),"function"==typeof e)e.call(this,this);else{e=e.split("\n");for(var i=0,a=e.length;i<a;i++)this.tspan(e[i]).newLine()}return this.build(!1).rebuild()},size:function(e){return this.attr("font-size",e).rebuild()},leading:function(e){return null==e?this.dom.leading:(this.dom.leading=new n.Number(e),this.rebuild())},lines:function(){var e=(this.textPath&&this.textPath()||this).node,t=n.utils.map(n.utils.filterSVGElements(e.childNodes),function(e){return n.adopt(e)});return new n.Set(t)},rebuild:function(e){if("boolean"==typeof e&&(this._rebuild=e),this._rebuild){var t=this,i=0,r=this.dom.leading*new n.Number(this.attr("font-size"));this.lines().each(function(){this.dom.newLined&&(t.textPath()||this.attr("x",t.attr("x")),"\n"==this.text()?i+=r:(this.attr("dy",r+i),i=0))}),this.fire("rebuild")}return this},build:function(e){return this._build=!!e,this},setData:function(e){return this.dom=e,this.dom.leading=new n.Number(e.leading||1.3),this}},construct:{text:function(e){return this.put(new n.Text).text(e)},plain:function(e){return this.put(new n.Text).plain(e)}}}),n.Tspan=n.invent({create:"tspan",inherit:n.Shape,extend:{text:function(e){return null==e?this.node.textContent+(this.dom.newLined?"\n":""):("function"==typeof e?e.call(this,this):this.plain(e),this)},dx:function(e){return this.attr("dx",e)},dy:function(e){return this.attr("dy",e)},newLine:function(){var e=this.parent(n.Text);return this.dom.newLined=!0,this.dy(e.dom.leading*e.attr("font-size")).attr("x",e.x())}}}),n.extend(n.Text,n.Tspan,{plain:function(e){return!1===this._build&&this.clear(),this.node.appendChild(t.createTextNode(e)),this},tspan:function(e){var t=(this.textPath&&this.textPath()||this).node,i=new n.Tspan;return!1===this._build&&this.clear(),t.appendChild(i.node),i.text(e)},clear:function(){for(var e=(this.textPath&&this.textPath()||this).node;e.hasChildNodes();)e.removeChild(e.lastChild);return this},length:function(){return this.node.getComputedTextLength()}}),n.TextPath=n.invent({create:"textPath",inherit:n.Parent,parent:n.Text,construct:{morphArray:n.PathArray,path:function(e){for(var t=new n.TextPath,i=this.doc().defs().path(e);this.node.hasChildNodes();)t.node.appendChild(this.node.firstChild);return this.node.appendChild(t.node),t.attr("href","#"+i,n.xlink),this},array:function(){var e=this.track();return e?e.array():null},plot:function(e){var t=this.track(),n=null;return t&&(n=t.plot(e)),null==e?n:this},track:function(){var e=this.textPath();if(e)return e.reference("href")},textPath:function(){if(this.node.firstChild&&"textPath"==this.node.firstChild.nodeName)return n.adopt(this.node.firstChild)}}}),n.Nested=n.invent({create:function(){this.constructor.call(this,n.create("svg")),this.style("overflow","visible")},inherit:n.Container,construct:{nested:function(){return this.put(new n.Nested)}}}),n.A=n.invent({create:"a",inherit:n.Container,extend:{to:function(e){return this.attr("href",e,n.xlink)},show:function(e){return this.attr("show",e,n.xlink)},target:function(e){return this.attr("target",e)}},construct:{link:function(e){return this.put(new n.A).to(e)}}}),n.extend(n.Element,{linkTo:function(e){var t=new n.A;return"function"==typeof e?e.call(t,t):t.to(e),this.parent().put(t).put(this)}}),n.Marker=n.invent({create:"marker",inherit:n.Container,extend:{width:function(e){return this.attr("markerWidth",e)},height:function(e){return this.attr("markerHeight",e)},ref:function(e,t){return this.attr("refX",e).attr("refY",t)},update:function(e){return this.clear(),"function"==typeof e&&e.call(this,this),this},toString:function(){return"url(#"+this.id()+")"}},construct:{marker:function(e,t,n){return this.defs().marker(e,t,n)}}}),n.extend(n.Defs,{marker:function(e,t,i){return this.put(new n.Marker).size(e,t).ref(e/2,t/2).viewbox(0,0,e,t).attr("orient","auto").update(i)}}),n.extend(n.Line,n.Polyline,n.Polygon,n.Path,{marker:function(e,t,i,r){var a=["marker"];return"all"!=e&&a.push(e),a=a.join("-"),e=arguments[1]instanceof n.Marker?arguments[1]:this.doc().marker(t,i,r),this.attr(a,e)}});var l={stroke:["color","width","opacity","linecap","linejoin","miterlimit","dasharray","dashoffset"],fill:["color","opacity","rule"],prefix:function(e,t){return"color"==t?e:e+"-"+t}};function c(e,t,i,r){return i+r.replace(n.regex.dots," .")}function u(e){return e.toLowerCase().replace(/-(.)/g,function(e,t){return t.toUpperCase()})}function h(e){return e.charAt(0).toUpperCase()+e.slice(1)}function d(e){var t=e.toString(16);return 1==t.length?"0"+t:t}function f(e,t,n){if(null==t||null==n){var i=e.bbox();null==t?t=i.width/i.height*n:null==n&&(n=i.height/i.width*t)}return{width:t,height:n}}function p(e,t,n){return{x:t*e.a+n*e.c+0,y:t*e.b+n*e.d+0}}function g(e){return{a:e[0],b:e[1],c:e[2],d:e[3],e:e[4],f:e[5]}}function m(e,t){e.cx=null==e.cx?t.bbox().cx:e.cx,e.cy=null==e.cy?t.bbox().cy:e.cy}function v(t){for(var i=t.childNodes.length-1;i>=0;i--)t.childNodes[i]instanceof e.SVGElement&&v(t.childNodes[i]);return n.adopt(t).id(n.eid(t.nodeName))}function y(e){return null==e.x&&(e.x=0,e.y=0,e.width=0,e.height=0),e.w=e.width,e.h=e.height,e.x2=e.x+e.width,e.y2=e.y+e.height,e.cx=e.x+e.width/2,e.cy=e.y+e.height/2,e}function b(e){return Math.abs(e)>1e-37?e:0}["fill","stroke"].forEach(function(e){var t,i={};i[e]=function(i){if(void 0===i)return this;if("string"==typeof i||n.Color.isRgb(i)||i&&"function"==typeof i.fill)this.attr(e,i);else for(t=l[e].length-1;t>=0;t--)null!=i[l[e][t]]&&this.attr(l.prefix(e,l[e][t]),i[l[e][t]]);return this},n.extend(n.Element,n.FX,i)}),n.extend(n.Element,n.FX,{rotate:function(e,t,n){return this.transform({rotation:e,cx:t,cy:n})},skew:function(e,t,n,i){return 1==arguments.length||3==arguments.length?this.transform({skew:e,cx:t,cy:n}):this.transform({skewX:e,skewY:t,cx:n,cy:i})},scale:function(e,t,n,i){return 1==arguments.length||3==arguments.length?this.transform({scale:e,cx:t,cy:n}):this.transform({scaleX:e,scaleY:t,cx:n,cy:i})},translate:function(e,t){return this.transform({x:e,y:t})},flip:function(e,t){return t="number"==typeof e?e:t,this.transform({flip:e||"both",offset:t})},matrix:function(e){return this.attr("transform",new n.Matrix(6==arguments.length?[].slice.call(arguments):e))},opacity:function(e){return this.attr("opacity",e)},dx:function(e){return this.x(new n.Number(e).plus(this instanceof n.FX?0:this.x()),!0)},dy:function(e){return this.y(new n.Number(e).plus(this instanceof n.FX?0:this.y()),!0)},dmove:function(e,t){return this.dx(e).dy(t)}}),n.extend(n.Rect,n.Ellipse,n.Circle,n.Gradient,n.FX,{radius:function(e,t){var i=(this._target||this).type;return"radial"==i||"circle"==i?this.attr("r",new n.Number(e)):this.rx(e).ry(null==t?e:t)}}),n.extend(n.Path,{length:function(){return this.node.getTotalLength()},pointAt:function(e){return this.node.getPointAtLength(e)}}),n.extend(n.Parent,n.Text,n.Tspan,n.FX,{font:function(e,t){if("object"===a(e))for(t in e)this.font(t,e[t]);return"leading"==e?this.leading(t):"anchor"==e?this.attr("text-anchor",t):"size"==e||"family"==e||"weight"==e||"stretch"==e||"variant"==e||"style"==e?this.attr("font-"+e,t):this.attr(e,t)}}),n.Set=n.invent({create:function(e){Array.isArray(e)?this.members=e:this.clear()},extend:{add:function(){var e,t,n=[].slice.call(arguments);for(e=0,t=n.length;e<t;e++)this.members.push(n[e]);return this},remove:function(e){var t=this.index(e);return t>-1&&this.members.splice(t,1),this},each:function(e){for(var t=0,n=this.members.length;t<n;t++)e.apply(this.members[t],[t,this.members]);return this},clear:function(){return this.members=[],this},length:function(){return this.members.length},has:function(e){return this.index(e)>=0},index:function(e){return this.members.indexOf(e)},get:function(e){return this.members[e]},first:function(){return this.get(0)},last:function(){return this.get(this.members.length-1)},valueOf:function(){return this.members},bbox:function(){if(0==this.members.length)return new n.RBox;var e=this.members[0].rbox(this.members[0].doc());return this.each(function(){e=e.merge(this.rbox(this.doc()))}),e}},construct:{set:function(e){return new n.Set(e)}}}),n.FX.Set=n.invent({create:function(e){this.set=e}}),n.Set.inherit=function(){var e=[];for(var t in n.Shape.prototype)"function"==typeof n.Shape.prototype[t]&&"function"!=typeof n.Set.prototype[t]&&e.push(t);for(var t in e.forEach(function(e){n.Set.prototype[e]=function(){for(var t=0,i=this.members.length;t<i;t++)this.members[t]&&"function"==typeof this.members[t][e]&&this.members[t][e].apply(this.members[t],arguments);return"animate"==e?this.fx||(this.fx=new n.FX.Set(this)):this}}),e=[],n.FX.prototype)"function"==typeof n.FX.prototype[t]&&"function"!=typeof n.FX.Set.prototype[t]&&e.push(t);e.forEach(function(e){n.FX.Set.prototype[e]=function(){for(var t=0,n=this.set.members.length;t<n;t++)this.set.members[t].fx[e].apply(this.set.members[t].fx,arguments);return this}})},n.extend(n.Element,{data:function(e,t,n){if("object"===a(e))for(t in e)this.data(t,e[t]);else if(arguments.length<2)try{return JSON.parse(this.attr("data-"+e))}catch(t){return this.attr("data-"+e)}else this.attr("data-"+e,null===t?null:!0===n||"string"==typeof t||"number"==typeof t?t:JSON.stringify(t));return this}}),n.extend(n.Element,{remember:function(e,t){if("object"===a(arguments[0]))for(var t in e)this.remember(t,e[t]);else{if(1==arguments.length)return this.memory()[e];this.memory()[e]=t}return this},forget:function(){if(0==arguments.length)this._memory={};else for(var e=arguments.length-1;e>=0;e--)delete this.memory()[arguments[e]];return this},memory:function(){return this._memory||(this._memory={})}}),n.get=function(e){var i=t.getElementById(function(e){var t=(e||"").toString().match(n.regex.reference);if(t)return t[1]}(e)||e);return n.adopt(i)},n.select=function(e,i){return new n.Set(n.utils.map((i||t).querySelectorAll(e),function(e){return n.adopt(e)}))},n.extend(n.Parent,{select:function(e){return n.select(e,this.node)}});var x="abcdef".split("");if("function"!=typeof e.CustomEvent){var w=function(e,n){n=n||{bubbles:!1,cancelable:!1,detail:void 0};var i=t.createEvent("CustomEvent");return i.initCustomEvent(e,n.bubbles,n.cancelable,n.detail),i};w.prototype=e.Event.prototype,n.CustomEvent=w}else n.CustomEvent=e.CustomEvent;return function(t){for(var n=0,i=["moz","webkit"],r=0;r<i.length&&!e.requestAnimationFrame;++r)t.requestAnimationFrame=t[i[r]+"RequestAnimationFrame"],t.cancelAnimationFrame=t[i[r]+"CancelAnimationFrame"]||t[i[r]+"CancelRequestAnimationFrame"];t.requestAnimationFrame=t.requestAnimationFrame||function(e){var i=(new Date).getTime(),r=Math.max(0,16-(i-n)),a=t.setTimeout(function(){e(i+r)},r);return n=i+r,a},t.cancelAnimationFrame=t.cancelAnimationFrame||t.clearTimeout}(e),n}(le,le.document)}.call(t,n,t,e))||(e.exports=r),
/*! svg.filter.js - v2.0.2 - 2016-02-24
* https://github.com/wout/svg.filter.js
* Copyright (c) 2016 Wout Fierens; Licensed MIT */
function(){SVG.Filter=SVG.invent({create:"filter",inherit:SVG.Parent,extend:{source:"SourceGraphic",sourceAlpha:"SourceAlpha",background:"BackgroundImage",backgroundAlpha:"BackgroundAlpha",fill:"FillPaint",stroke:"StrokePaint",autoSetIn:!0,put:function(e,t){return this.add(e,t),!e.attr("in")&&this.autoSetIn&&e.attr("in",this.source),e.attr("result")||e.attr("result",e),e},blend:function(e,t,n){return this.put(new SVG.BlendEffect(e,t,n))},colorMatrix:function(e,t){return this.put(new SVG.ColorMatrixEffect(e,t))},convolveMatrix:function(e){return this.put(new SVG.ConvolveMatrixEffect(e))},componentTransfer:function(e){return this.put(new SVG.ComponentTransferEffect(e))},composite:function(e,t,n){return this.put(new SVG.CompositeEffect(e,t,n))},flood:function(e,t){return this.put(new SVG.FloodEffect(e,t))},offset:function(e,t){return this.put(new SVG.OffsetEffect(e,t))},image:function(e){return this.put(new SVG.ImageEffect(e))},merge:function(){var e=[void 0];for(var t in arguments)e.push(arguments[t]);return this.put(new(SVG.MergeEffect.bind.apply(SVG.MergeEffect,e)))},gaussianBlur:function(e,t){return this.put(new SVG.GaussianBlurEffect(e,t))},morphology:function(e,t){return this.put(new SVG.MorphologyEffect(e,t))},diffuseLighting:function(e,t,n){return this.put(new SVG.DiffuseLightingEffect(e,t,n))},displacementMap:function(e,t,n,i,r){return this.put(new SVG.DisplacementMapEffect(e,t,n,i,r))},specularLighting:function(e,t,n,i){return this.put(new SVG.SpecularLightingEffect(e,t,n,i))},tile:function(){return this.put(new SVG.TileEffect)},turbulence:function(e,t,n,i,r){return this.put(new SVG.TurbulenceEffect(e,t,n,i,r))},toString:function(){return"url(#"+this.attr("id")+")"}}}),SVG.extend(SVG.Defs,{filter:function(e){var t=this.put(new SVG.Filter);return"function"==typeof e&&e.call(t,t),t}}),SVG.extend(SVG.Container,{filter:function(e){return this.defs().filter(e)}}),SVG.extend(SVG.Element,SVG.G,SVG.Nested,{filter:function(e){return this.filterer=e instanceof SVG.Element?e:this.doc().filter(e),this.doc()&&this.filterer.doc()!==this.doc()&&this.doc().defs().add(this.filterer),this.attr("filter",this.filterer),this.filterer},unfilter:function(e){return this.filterer&&!0===e&&this.filterer.remove(),delete this.filterer,this.attr("filter",null)}}),SVG.Effect=SVG.invent({create:function(){this.constructor.call(this)},inherit:SVG.Element,extend:{in:function(e){return null==e?this.parent()&&this.parent().select('[result="'+this.attr("in")+'"]').get(0)||this.attr("in"):this.attr("in",e)},result:function(e){return null==e?this.attr("result"):this.attr("result",e)},toString:function(){return this.result()}}}),SVG.ParentEffect=SVG.invent({create:function(){this.constructor.call(this)},inherit:SVG.Parent,extend:{in:function(e){return null==e?this.parent()&&this.parent().select('[result="'+this.attr("in")+'"]').get(0)||this.attr("in"):this.attr("in",e)},result:function(e){return null==e?this.attr("result"):this.attr("result",e)},toString:function(){return this.result()}}});var e={blend:function(e,t){return this.parent()&&this.parent().blend(this,e,t)},colorMatrix:function(e,t){return this.parent()&&this.parent().colorMatrix(e,t).in(this)},convolveMatrix:function(e){return this.parent()&&this.parent().convolveMatrix(e).in(this)},componentTransfer:function(e){return this.parent()&&this.parent().componentTransfer(e).in(this)},composite:function(e,t){return this.parent()&&this.parent().composite(this,e,t)},flood:function(e,t){return this.parent()&&this.parent().flood(e,t)},offset:function(e,t){return this.parent()&&this.parent().offset(e,t).in(this)},image:function(e){return this.parent()&&this.parent().image(e)},merge:function(){return this.parent()&&this.parent().merge.apply(this.parent(),[this].concat(arguments))},gaussianBlur:function(e,t){return this.parent()&&this.parent().gaussianBlur(e,t).in(this)},morphology:function(e,t){return this.parent()&&this.parent().morphology(e,t).in(this)},diffuseLighting:function(e,t,n){return this.parent()&&this.parent().diffuseLighting(e,t,n).in(this)},displacementMap:function(e,t,n,i){return this.parent()&&this.parent().displacementMap(this,e,t,n,i)},specularLighting:function(e,t,n,i){return this.parent()&&this.parent().specularLighting(e,t,n,i).in(this)},tile:function(){return this.parent()&&this.parent().tile().in(this)},turbulence:function(e,t,n,i,r){return this.parent()&&this.parent().turbulence(e,t,n,i,r).in(this)}};SVG.extend(SVG.Effect,e),SVG.extend(SVG.ParentEffect,e),SVG.ChildEffect=SVG.invent({create:function(){this.constructor.call(this)},inherit:SVG.Element,extend:{in:function(e){this.attr("in",e)}}});var t={blend:function(e,t,n){this.attr({in:e,in2:t,mode:n||"normal"})},colorMatrix:function(e,t){"matrix"==e&&(t=r(t)),this.attr({type:e,values:void 0===t?null:t})},convolveMatrix:function(e){e=r(e),this.attr({order:Math.sqrt(e.split(" ").length),kernelMatrix:e})},composite:function(e,t,n){this.attr({in:e,in2:t,operator:n})},flood:function(e,t){this.attr("flood-color",e),null!=t&&this.attr("flood-opacity",t)},offset:function(e,t){this.attr({dx:e,dy:t})},image:function(e){this.attr("href",e,SVG.xlink)},displacementMap:function(e,t,n,i,r){this.attr({in:e,in2:t,scale:n,xChannelSelector:i,yChannelSelector:r})},gaussianBlur:function(e,t){null!=e||null!=t?this.attr("stdDeviation",function(e){if(!Array.isArray(e))return e;for(var t=0,n=e.length,i=[];t<n;t++)i.push(e[t]);return i.join(" ")}(Array.prototype.slice.call(arguments))):this.attr("stdDeviation","0 0")},morphology:function(e,t){this.attr({operator:e,radius:t})},tile:function(){},turbulence:function(e,t,n,i,r){this.attr({numOctaves:t,seed:n,stitchTiles:i,baseFrequency:e,type:r})}},n={merge:function(){var e;if(arguments[0]instanceof SVG.Set){var t=this;arguments[0].each(function(e){this instanceof SVG.MergeNode?t.put(this):(this instanceof SVG.Effect||this instanceof SVG.ParentEffect)&&t.put(new SVG.MergeNode(this))})}else{e=Array.isArray(arguments[0])?arguments[0]:arguments;for(var n=0;n<e.length;n++)e[n]instanceof SVG.MergeNode?this.put(e[n]):this.put(new SVG.MergeNode(e[n]))}},componentTransfer:function(e){if(this.rgb=new SVG.Set,["r","g","b","a"].forEach(function(e){this[e]=new(SVG["Func"+e.toUpperCase()])("identity"),this.rgb.add(this[e]),this.node.appendChild(this[e].node)}.bind(this)),e)for(var t in e.rgb&&(["r","g","b"].forEach(function(t){this[t].attr(e.rgb)}.bind(this)),delete e.rgb),e)this[t].attr(e[t])},diffuseLighting:function(e,t,n){this.attr({surfaceScale:e,diffuseConstant:t,kernelUnitLength:n})},specularLighting:function(e,t,n,i){this.attr({surfaceScale:e,diffuseConstant:t,specularExponent:n,kernelUnitLength:i})}},i={distantLight:function(e,t){this.attr({azimuth:e,elevation:t})},pointLight:function(e,t,n){this.attr({x:e,y:t,z:n})},spotLight:function(e,t,n,i,r,a){this.attr({x:e,y:t,z:n,pointsAtX:i,pointsAtY:r,pointsAtZ:a})},mergeNode:function(e){this.attr("in",e)}};function r(e){return Array.isArray(e)&&(e=new SVG.Array(e)),e.toString().replace(/^\s+/,"").replace(/\s+$/,"").replace(/\s+/g," ")}function a(){var e=function(){};for(var t in"function"==typeof arguments[arguments.length-1]&&(e=arguments[arguments.length-1],Array.prototype.splice.call(arguments,arguments.length-1,1)),arguments)for(var n in arguments[t])e(arguments[t][n],n,arguments[t])}["r","g","b","a"].forEach(function(e){i["Func"+e.toUpperCase()]=function(e){switch(this.attr("type",e),e){case"table":this.attr("tableValues",arguments[1]);break;case"linear":this.attr("slope",arguments[1]),this.attr("intercept",arguments[2]);break;case"gamma":this.attr("amplitude",arguments[1]),this.attr("exponent",arguments[2]),this.attr("offset",arguments[2])}}}),a(t,function(e,t){var n=t.charAt(0).toUpperCase()+t.slice(1);SVG[n+"Effect"]=SVG.invent({create:function(){this.constructor.call(this,SVG.create("fe"+n)),e.apply(this,arguments),this.result(this.attr("id")+"Out")},inherit:SVG.Effect,extend:{}})}),a(n,function(e,t){var n=t.charAt(0).toUpperCase()+t.slice(1);SVG[n+"Effect"]=SVG.invent({create:function(){this.constructor.call(this,SVG.create("fe"+n)),e.apply(this,arguments),this.result(this.attr("id")+"Out")},inherit:SVG.ParentEffect,extend:{}})}),a(i,function(e,t){var n=t.charAt(0).toUpperCase()+t.slice(1);SVG[n]=SVG.invent({create:function(){this.constructor.call(this,SVG.create("fe"+n)),e.apply(this,arguments)},inherit:SVG.ChildEffect,extend:{}})}),SVG.extend(SVG.MergeEffect,{in:function(e){return e instanceof SVG.MergeNode?this.add(e,0):this.add(new SVG.MergeNode(e),0),this}}),SVG.extend(SVG.CompositeEffect,SVG.BlendEffect,SVG.DisplacementMapEffect,{in2:function(e){return null==e?this.parent()&&this.parent().select('[result="'+this.attr("in2")+'"]').get(0)||this.attr("in2"):this.attr("in2",e)}}),SVG.filter={sepiatone:[.343,.669,.119,0,0,.249,.626,.13,0,0,.172,.334,.111,0,0,0,0,0,1,0]}}.call(void 0),function(){function e(e,r,a,o,s,l,c){for(var u=e.slice(r,a||c),h=o.slice(s,l||c),d=0,f={pos:[0,0],start:[0,0]},p={pos:[0,0],start:[0,0]};;){if(u[d]=t.call(f,u[d]),h[d]=t.call(p,h[d]),u[d][0]!=h[d][0]||"M"==u[d][0]||"A"==u[d][0]&&(u[d][4]!=h[d][4]||u[d][5]!=h[d][5])?(Array.prototype.splice.apply(u,[d,1].concat(i.call(f,u[d]))),Array.prototype.splice.apply(h,[d,1].concat(i.call(p,h[d])))):(u[d]=n.call(f,u[d]),h[d]=n.call(p,h[d])),++d==u.length&&d==h.length)break;d==u.length&&u.push(["C",f.pos[0],f.pos[1],f.pos[0],f.pos[1],f.pos[0],f.pos[1]]),d==h.length&&h.push(["C",p.pos[0],p.pos[1],p.pos[0],p.pos[1],p.pos[0],p.pos[1]])}return{start:u,dest:h}}function t(e){switch(e[0]){case"z":case"Z":e[0]="L",e[1]=this.start[0],e[2]=this.start[1];break;case"H":e[0]="L",e[2]=this.pos[1];break;case"V":e[0]="L",e[2]=e[1],e[1]=this.pos[0];break;case"T":e[0]="Q",e[3]=e[1],e[4]=e[2],e[1]=this.reflection[1],e[2]=this.reflection[0];break;case"S":e[0]="C",e[6]=e[4],e[5]=e[3],e[4]=e[2],e[3]=e[1],e[2]=this.reflection[1],e[1]=this.reflection[0]}return e}function n(e){var t=e.length;return this.pos=[e[t-2],e[t-1]],-1!="SCQT".indexOf(e[0])&&(this.reflection=[2*this.pos[0]-e[t-4],2*this.pos[1]-e[t-3]]),e}function i(e){var t=[e];switch(e[0]){case"M":return this.pos=this.start=[e[1],e[2]],t;case"L":e[5]=e[3]=e[1],e[6]=e[4]=e[2],e[1]=this.pos[0],e[2]=this.pos[1];break;case"Q":e[6]=e[4],e[5]=e[3],e[4]=1*e[4]/3+2*e[2]/3,e[3]=1*e[3]/3+2*e[1]/3,e[2]=1*this.pos[1]/3+2*e[2]/3,e[1]=1*this.pos[0]/3+2*e[1]/3;break;case"A":e=(t=function(e,t){var n,i,r,a,o,s,l,c,u,h,d,f,p,g,m,v,y,b,x,w,k,S,E,C,T,A,_=Math.abs(t[1]),O=Math.abs(t[2]),P=t[3]%360,M=t[4],I=t[5],D=t[6],N=t[7],L=new SVG.Point(e),R=new SVG.Point(D,N),F=[];if(0===_||0===O||L.x===R.x&&L.y===R.y)return[["C",L.x,L.y,R.x,R.y,R.x,R.y]];n=new SVG.Point((L.x-R.x)/2,(L.y-R.y)/2).transform((new SVG.Matrix).rotate(P)),(i=n.x*n.x/(_*_)+n.y*n.y/(O*O))>1&&(i=Math.sqrt(i),_*=i,O*=i);r=(new SVG.Matrix).rotate(P).scale(1/_,1/O).rotate(-P),L=L.transform(r),R=R.transform(r),a=[R.x-L.x,R.y-L.y],s=a[0]*a[0]+a[1]*a[1],o=Math.sqrt(s),a[0]/=o,a[1]/=o,l=s<4?Math.sqrt(1-s/4):0,M===I&&(l*=-1);c=new SVG.Point((R.x+L.x)/2+l*-a[1],(R.y+L.y)/2+l*a[0]),u=new SVG.Point(L.x-c.x,L.y-c.y),h=new SVG.Point(R.x-c.x,R.y-c.y),d=Math.acos(u.x/Math.sqrt(u.x*u.x+u.y*u.y)),u.y<0&&(d*=-1);f=Math.acos(h.x/Math.sqrt(h.x*h.x+h.y*h.y)),h.y<0&&(f*=-1);I&&d>f&&(f+=2*Math.PI);!I&&d<f&&(f-=2*Math.PI);for(g=Math.ceil(2*Math.abs(d-f)/Math.PI),v=[],y=d,p=(f-d)/g,m=4*Math.tan(p/4)/3,k=0;k<=g;k++)x=Math.cos(y),b=Math.sin(y),w=new SVG.Point(c.x+x,c.y+b),v[k]=[new SVG.Point(w.x+m*b,w.y-m*x),w,new SVG.Point(w.x-m*b,w.y+m*x)],y+=p;for(v[0][0]=v[0][1].clone(),v[v.length-1][2]=v[v.length-1][1].clone(),r=(new SVG.Matrix).rotate(P).scale(_,O).rotate(-P),k=0,S=v.length;k<S;k++)v[k][0]=v[k][0].transform(r),v[k][1]=v[k][1].transform(r),v[k][2]=v[k][2].transform(r);for(k=1,S=v.length;k<S;k++)w=v[k-1][2],E=w.x,C=w.y,w=v[k][0],T=w.x,A=w.y,w=v[k][1],D=w.x,N=w.y,F.push(["C",E,C,T,A,D,N]);return F}(this.pos,e))[0]}return e[0]="C",this.pos=[e[5],e[6]],this.reflection=[2*e[5]-e[3],2*e[6]-e[4]],t}function r(e,t){if(!1===t)return!1;for(var n=t,i=e.length;n<i;++n)if("M"==e[n][0])return n;return!1}SVG.extend(SVG.PathArray,{morph:function(t){for(var n=this.value,i=this.parse(t),a=0,o=0,s=!1,l=!1;!1!==a||!1!==o;){var c;s=r(n,!1!==a&&a+1),l=r(i,!1!==o&&o+1),!1===a&&(a=0==(c=new SVG.PathArray(u.start).bbox()).height||0==c.width?n.push(n[0])-1:n.push(["M",c.x+c.width/2,c.y+c.height/2])-1),!1===o&&(o=0==(c=new SVG.PathArray(u.dest).bbox()).height||0==c.width?i.push(i[0])-1:i.push(["M",c.x+c.width/2,c.y+c.height/2])-1);var u=e(n,a,s,i,o,l);n=n.slice(0,a).concat(u.start,!1===s?[]:n.slice(s)),i=i.slice(0,o).concat(u.dest,!1===l?[]:i.slice(l)),a=!1!==s&&a+u.start.length,o=!1!==l&&o+u.dest.length}return this.value=n,this.destination=new SVG.PathArray,this.destination.value=i,this}})}(),
/*! svg.draggable.js - v2.2.1 - 2016-08-25
* https://github.com/wout/svg.draggable.js
* Copyright (c) 2016 Wout Fierens; Licensed MIT */
function(){function e(e){e.remember("_draggable",this),this.el=e}e.prototype.init=function(e,t){var n=this;this.constraint=e,this.value=t,this.el.on("mousedown.drag",function(e){n.start(e)}),this.el.on("touchstart.drag",function(e){n.start(e)})},e.prototype.transformPoint=function(e,t){var n=(e=e||window.event).changedTouches&&e.changedTouches[0]||e;return this.p.x=n.pageX-(t||0),this.p.y=n.pageY,this.p.matrixTransform(this.m)},e.prototype.getBBox=function(){var e=this.el.bbox();return this.el instanceof SVG.Nested&&(e=this.el.rbox()),(this.el instanceof SVG.G||this.el instanceof SVG.Use||this.el instanceof SVG.Nested)&&(e.x=this.el.x(),e.y=this.el.y()),e},e.prototype.start=function(e){if("click"!=e.type&&"mousedown"!=e.type&&"mousemove"!=e.type||1==(e.which||e.buttons)){var t=this;this.el.fire("beforedrag",{event:e,handler:this}),this.parent=this.parent||this.el.parent(SVG.Nested)||this.el.parent(SVG.Doc),this.p=this.parent.node.createSVGPoint(),this.m=this.el.node.getScreenCTM().inverse();var n,i=this.getBBox();if(this.el instanceof SVG.Text)switch(n=this.el.node.getComputedTextLength(),this.el.attr("text-anchor")){case"middle":n/=2;break;case"start":n=0}this.startPoints={point:this.transformPoint(e,n),box:i,transform:this.el.transform()},SVG.on(window,"mousemove.drag",function(e){t.drag(e)}),SVG.on(window,"touchmove.drag",function(e){t.drag(e)}),SVG.on(window,"mouseup.drag",function(e){t.end(e)}),SVG.on(window,"touchend.drag",function(e){t.end(e)}),this.el.fire("dragstart",{event:e,p:this.startPoints.point,m:this.m,handler:this}),e.preventDefault(),e.stopPropagation()}},e.prototype.drag=function(e){var t=this.getBBox(),n=this.transformPoint(e),i=this.startPoints.box.x+n.x-this.startPoints.point.x,r=this.startPoints.box.y+n.y-this.startPoints.point.y,a=this.constraint,o=n.x-this.startPoints.point.x,s=n.y-this.startPoints.point.y,l=new CustomEvent("dragmove",{detail:{event:e,p:n,m:this.m,handler:this},cancelable:!0});if(this.el.fire(l),l.defaultPrevented)return n;if("function"==typeof a){var c=a.call(this.el,i,r,this.m);"boolean"==typeof c&&(c={x:c,y:c}),!0===c.x?this.el.x(i):!1!==c.x&&this.el.x(c.x),!0===c.y?this.el.y(r):!1!==c.y&&this.el.y(c.y)}else"object"==typeof a&&(null!=a.minX&&i<a.minX?i=a.minX:null!=a.maxX&&i>a.maxX-t.width&&(i=a.maxX-t.width),null!=a.minY&&r<a.minY?r=a.minY:null!=a.maxY&&r>a.maxY-t.height&&(r=a.maxY-t.height),this.el instanceof SVG.G?this.el.matrix(this.startPoints.transform).transform({x:o,y:s},!0):this.el.move(i,r));return n},e.prototype.end=function(e){var t=this.drag(e);this.el.fire("dragend",{event:e,p:t,m:this.m,handler:this}),SVG.off(window,"mousemove.drag"),SVG.off(window,"touchmove.drag"),SVG.off(window,"mouseup.drag"),SVG.off(window,"touchend.drag")},SVG.extend(SVG.Element,{draggable:function(t,n){"function"!=typeof t&&"object"!=typeof t||(n=t,t=!0);var i=this.remember("_draggable")||new e(this);return(t=void 0===t||t)?i.init(n||{},t):(this.off("mousedown.drag"),this.off("touchstart.drag")),this}})}.call(void 0),function(){function e(e){this.el=e,e.remember("_selectHandler",this),this.pointSelection={isSelected:!1},this.rectSelection={isSelected:!1}}e.prototype.init=function(e,t){var n=this.el.bbox();for(var i in this.options={},this.el.selectize.defaults)this.options[i]=this.el.selectize.defaults[i],void 0!==t[i]&&(this.options[i]=t[i]);this.parent=this.el.parent(),this.nested=this.nested||this.parent.group(),this.nested.matrix(new SVG.Matrix(this.el).translate(n.x,n.y)),this.options.deepSelect&&-1!==["line","polyline","polygon"].indexOf(this.el.type)?this.selectPoints(e):this.selectRect(e),this.observe(),this.cleanup()},e.prototype.selectPoints=function(e){return this.pointSelection.isSelected=e,this.pointSelection.set?this:(this.pointSelection.set=this.parent.set(),this.drawCircles(),this)},e.prototype.getPointArray=function(){var e=this.el.bbox();return this.el.array().valueOf().map(function(t){return[t[0]-e.x,t[1]-e.y]})},e.prototype.drawCircles=function(){for(var e=this,t=this.getPointArray(),n=0,i=t.length;n<i;++n){var r=function(t){return function(n){(n=n||window.event).preventDefault?n.preventDefault():n.returnValue=!1,n.stopPropagation();var i=n.pageX||n.touches[0].pageX,r=n.pageY||n.touches[0].pageY;e.el.fire("point",{x:i,y:r,i:t,event:n})}}(n);this.pointSelection.set.add(this.nested.circle(this.options.radius).center(t[n][0],t[n][1]).addClass(this.options.classPoints).addClass(this.options.classPoints+"_point").on("touchstart",r).on("mousedown",r))}},e.prototype.updatePointSelection=function(){var e=this.getPointArray();this.pointSelection.set.each(function(t){this.cx()===e[t][0]&&this.cy()===e[t][1]||this.center(e[t][0],e[t][1])})},e.prototype.updateRectSelection=function(){var e=this.el.bbox();this.rectSelection.set.get(0).attr({width:e.width,height:e.height}),this.options.points&&(this.rectSelection.set.get(2).center(e.width,0),this.rectSelection.set.get(3).center(e.width,e.height),this.rectSelection.set.get(4).center(0,e.height),this.rectSelection.set.get(5).center(e.width/2,0),this.rectSelection.set.get(6).center(e.width,e.height/2),this.rectSelection.set.get(7).center(e.width/2,e.height),this.rectSelection.set.get(8).center(0,e.height/2)),this.options.rotationPoint&&(this.options.points?this.rectSelection.set.get(9).center(e.width/2,20):this.rectSelection.set.get(1).center(e.width/2,20))},e.prototype.selectRect=function(e){var t=this,n=this.el.bbox();function i(e){return function(n){(n=n||window.event).preventDefault?n.preventDefault():n.returnValue=!1,n.stopPropagation();var i=n.pageX||n.touches[0].pageX,r=n.pageY||n.touches[0].pageY;t.el.fire(e,{x:i,y:r,event:n})}}if(this.rectSelection.isSelected=e,this.rectSelection.set=this.rectSelection.set||this.parent.set(),this.rectSelection.set.get(0)||this.rectSelection.set.add(this.nested.rect(n.width,n.height).addClass(this.options.classRect)),this.options.points&&!this.rectSelection.set.get(1)){var r="touchstart",a="mousedown";this.rectSelection.set.add(this.nested.circle(this.options.radius).center(0,0).attr("class",this.options.classPoints+"_lt").on(a,i("lt")).on(r,i("lt"))),this.rectSelection.set.add(this.nested.circle(this.options.radius).center(n.width,0).attr("class",this.options.classPoints+"_rt").on(a,i("rt")).on(r,i("rt"))),this.rectSelection.set.add(this.nested.circle(this.options.radius).center(n.width,n.height).attr("class",this.options.classPoints+"_rb").on(a,i("rb")).on(r,i("rb"))),this.rectSelection.set.add(this.nested.circle(this.options.radius).center(0,n.height).attr("class",this.options.classPoints+"_lb").on(a,i("lb")).on(r,i("lb"))),this.rectSelection.set.add(this.nested.circle(this.options.radius).center(n.width/2,0).attr("class",this.options.classPoints+"_t").on(a,i("t")).on(r,i("t"))),this.rectSelection.set.add(this.nested.circle(this.options.radius).center(n.width,n.height/2).attr("class",this.options.classPoints+"_r").on(a,i("r")).on(r,i("r"))),this.rectSelection.set.add(this.nested.circle(this.options.radius).center(n.width/2,n.height).attr("class",this.options.classPoints+"_b").on(a,i("b")).on(r,i("b"))),this.rectSelection.set.add(this.nested.circle(this.options.radius).center(0,n.height/2).attr("class",this.options.classPoints+"_l").on(a,i("l")).on(r,i("l"))),this.rectSelection.set.each(function(){this.addClass(t.options.classPoints)})}if(this.options.rotationPoint&&(this.options.points&&!this.rectSelection.set.get(9)||!this.options.points&&!this.rectSelection.set.get(1))){var o=function(e){(e=e||window.event).preventDefault?e.preventDefault():e.returnValue=!1,e.stopPropagation();var n=e.pageX||e.touches[0].pageX,i=e.pageY||e.touches[0].pageY;t.el.fire("rot",{x:n,y:i,event:e})};this.rectSelection.set.add(this.nested.circle(this.options.radius).center(n.width/2,20).attr("class",this.options.classPoints+"_rot").on("touchstart",o).on("mousedown",o))}},e.prototype.handler=function(){var e=this.el.bbox();this.nested.matrix(new SVG.Matrix(this.el).translate(e.x,e.y)),this.rectSelection.isSelected&&this.updateRectSelection(),this.pointSelection.isSelected&&this.updatePointSelection()},e.prototype.observe=function(){var e=this;if(MutationObserver)if(this.rectSelection.isSelected||this.pointSelection.isSelected)this.observerInst=this.observerInst||new MutationObserver(function(){e.handler()}),this.observerInst.observe(this.el.node,{attributes:!0});else try{this.observerInst.disconnect(),delete this.observerInst}catch(e){}else this.el.off("DOMAttrModified.select"),(this.rectSelection.isSelected||this.pointSelection.isSelected)&&this.el.on("DOMAttrModified.select",function(){e.handler()})},e.prototype.cleanup=function(){!this.rectSelection.isSelected&&this.rectSelection.set&&(this.rectSelection.set.each(function(){this.remove()}),this.rectSelection.set.clear(),delete this.rectSelection.set),!this.pointSelection.isSelected&&this.pointSelection.set&&(this.pointSelection.set.each(function(){this.remove()}),this.pointSelection.set.clear(),delete this.pointSelection.set),this.pointSelection.isSelected||this.rectSelection.isSelected||(this.nested.remove(),delete this.nested)},SVG.extend(SVG.Element,{selectize:function(t,n){return"object"==typeof t&&(n=t,t=!0),(this.remember("_selectHandler")||new e(this)).init(void 0===t||t,n||{}),this}}),SVG.Element.prototype.selectize.defaults={points:!0,classRect:"svg_select_boundingRect",classPoints:"svg_select_points",radius:7,rotationPoint:!0,deepSelect:!1}}(),function(){(function(){function e(e){e.remember("_resizeHandler",this),this.el=e,this.parameters={},this.lastUpdateCall=null,this.p=e.doc().node.createSVGPoint()}e.prototype.transformPoint=function(e,t,n){return this.p.x=e-(this.offset.x-window.pageXOffset),this.p.y=t-(this.offset.y-window.pageYOffset),this.p.matrixTransform(n||this.m)},e.prototype._extractPosition=function(e){return{x:null!=e.clientX?e.clientX:e.touches[0].clientX,y:null!=e.clientY?e.clientY:e.touches[0].clientY}},e.prototype.init=function(e){var t=this;if(this.stop(),"stop"!==e){for(var n in this.options={},this.el.resize.defaults)this.options[n]=this.el.resize.defaults[n],void 0!==e[n]&&(this.options[n]=e[n]);this.el.on("lt.resize",function(e){t.resize(e||window.event)}),this.el.on("rt.resize",function(e){t.resize(e||window.event)}),this.el.on("rb.resize",function(e){t.resize(e||window.event)}),this.el.on("lb.resize",function(e){t.resize(e||window.event)}),this.el.on("t.resize",function(e){t.resize(e||window.event)}),this.el.on("r.resize",function(e){t.resize(e||window.event)}),this.el.on("b.resize",function(e){t.resize(e||window.event)}),this.el.on("l.resize",function(e){t.resize(e||window.event)}),this.el.on("rot.resize",function(e){t.resize(e||window.event)}),this.el.on("point.resize",function(e){t.resize(e||window.event)}),this.update()}},e.prototype.stop=function(){return this.el.off("lt.resize"),this.el.off("rt.resize"),this.el.off("rb.resize"),this.el.off("lb.resize"),this.el.off("t.resize"),this.el.off("r.resize"),this.el.off("b.resize"),this.el.off("l.resize"),this.el.off("rot.resize"),this.el.off("point.resize"),this},e.prototype.resize=function(e){var t=this;this.m=this.el.node.getScreenCTM().inverse(),this.offset={x:window.pageXOffset,y:window.pageYOffset};var n=this._extractPosition(e.detail.event);if(this.parameters={type:this.el.type,p:this.transformPoint(n.x,n.y),x:e.detail.x,y:e.detail.y,box:this.el.bbox(),rotation:this.el.transform().rotation},"text"===this.el.type&&(this.parameters.fontSize=this.el.attr()["font-size"]),void 0!==e.detail.i){var i=this.el.array().valueOf();this.parameters.i=e.detail.i,this.parameters.pointCoords=[i[e.detail.i][0],i[e.detail.i][1]]}switch(e.type){case"lt":this.calc=function(e,t){var n=this.snapToGrid(e,t);if(this.parameters.box.width-n[0]>0&&this.parameters.box.height-n[1]>0){if("text"===this.parameters.type)return this.el.move(this.parameters.box.x+n[0],this.parameters.box.y),void this.el.attr("font-size",this.parameters.fontSize-n[0]);n=this.checkAspectRatio(n),this.el.move(this.parameters.box.x+n[0],this.parameters.box.y+n[1]).size(this.parameters.box.width-n[0],this.parameters.box.height-n[1])}};break;case"rt":this.calc=function(e,t){var n=this.snapToGrid(e,t,2);if(this.parameters.box.width+n[0]>0&&this.parameters.box.height-n[1]>0){if("text"===this.parameters.type)return this.el.move(this.parameters.box.x-n[0],this.parameters.box.y),void this.el.attr("font-size",this.parameters.fontSize+n[0]);n=this.checkAspectRatio(n),this.el.move(this.parameters.box.x,this.parameters.box.y+n[1]).size(this.parameters.box.width+n[0],this.parameters.box.height-n[1])}};break;case"rb":this.calc=function(e,t){var n=this.snapToGrid(e,t,0);if(this.parameters.box.width+n[0]>0&&this.parameters.box.height+n[1]>0){if("text"===this.parameters.type)return this.el.move(this.parameters.box.x-n[0],this.parameters.box.y),void this.el.attr("font-size",this.parameters.fontSize+n[0]);n=this.checkAspectRatio(n),this.el.move(this.parameters.box.x,this.parameters.box.y).size(this.parameters.box.width+n[0],this.parameters.box.height+n[1])}};break;case"lb":this.calc=function(e,t){var n=this.snapToGrid(e,t,1);if(this.parameters.box.width-n[0]>0&&this.parameters.box.height+n[1]>0){if("text"===this.parameters.type)return this.el.move(this.parameters.box.x+n[0],this.parameters.box.y),void this.el.attr("font-size",this.parameters.fontSize-n[0]);n=this.checkAspectRatio(n),this.el.move(this.parameters.box.x+n[0],this.parameters.box.y).size(this.parameters.box.width-n[0],this.parameters.box.height+n[1])}};break;case"t":this.calc=function(e,t){var n=this.snapToGrid(e,t,2);if(this.parameters.box.height-n[1]>0){if("text"===this.parameters.type)return;this.el.move(this.parameters.box.x,this.parameters.box.y+n[1]).height(this.parameters.box.height-n[1])}};break;case"r":this.calc=function(e,t){var n=this.snapToGrid(e,t,0);if(this.parameters.box.width+n[0]>0){if("text"===this.parameters.type)return;this.el.move(this.parameters.box.x,this.parameters.box.y).width(this.parameters.box.width+n[0])}};break;case"b":this.calc=function(e,t){var n=this.snapToGrid(e,t,0);if(this.parameters.box.height+n[1]>0){if("text"===this.parameters.type)return;this.el.move(this.parameters.box.x,this.parameters.box.y).height(this.parameters.box.height+n[1])}};break;case"l":this.calc=function(e,t){var n=this.snapToGrid(e,t,1);if(this.parameters.box.width-n[0]>0){if("text"===this.parameters.type)return;this.el.move(this.parameters.box.x+n[0],this.parameters.box.y).width(this.parameters.box.width-n[0])}};break;case"rot":this.calc=function(e,t){var n=e+this.parameters.p.x,i=t+this.parameters.p.y,r=Math.atan2(this.parameters.p.y-this.parameters.box.y-this.parameters.box.height/2,this.parameters.p.x-this.parameters.box.x-this.parameters.box.width/2),a=180*(Math.atan2(i-this.parameters.box.y-this.parameters.box.height/2,n-this.parameters.box.x-this.parameters.box.width/2)-r)/Math.PI;this.el.center(this.parameters.box.cx,this.parameters.box.cy).rotate(this.parameters.rotation+a-a%this.options.snapToAngle,this.parameters.box.cx,this.parameters.box.cy)};break;case"point":this.calc=function(e,t){var n=this.snapToGrid(e,t,this.parameters.pointCoords[0],this.parameters.pointCoords[1]),i=this.el.array().valueOf();i[this.parameters.i][0]=this.parameters.pointCoords[0]+n[0],i[this.parameters.i][1]=this.parameters.pointCoords[1]+n[1],this.el.plot(i)}}this.el.fire("resizestart",{dx:this.parameters.x,dy:this.parameters.y,event:e}),SVG.on(window,"touchmove.resize",function(e){t.update(e||window.event)}),SVG.on(window,"touchend.resize",function(){t.done()}),SVG.on(window,"mousemove.resize",function(e){t.update(e||window.event)}),SVG.on(window,"mouseup.resize",function(){t.done()})},e.prototype.update=function(e){if(e){var t=this._extractPosition(e),n=this.transformPoint(t.x,t.y),i=n.x-this.parameters.p.x,r=n.y-this.parameters.p.y;this.lastUpdateCall=[i,r],this.calc(i,r),this.el.fire("resizing",{dx:i,dy:r,event:e})}else this.lastUpdateCall&&this.calc(this.lastUpdateCall[0],this.lastUpdateCall[1])},e.prototype.done=function(){this.lastUpdateCall=null,SVG.off(window,"mousemove.resize"),SVG.off(window,"mouseup.resize"),SVG.off(window,"touchmove.resize"),SVG.off(window,"touchend.resize"),this.el.fire("resizedone")},e.prototype.snapToGrid=function(e,t,n,i){var r;return void 0!==i?r=[(n+e)%this.options.snapToGrid,(i+t)%this.options.snapToGrid]:(n=null==n?3:n,r=[(this.parameters.box.x+e+(1&n?0:this.parameters.box.width))%this.options.snapToGrid,(this.parameters.box.y+t+(2&n?0:this.parameters.box.height))%this.options.snapToGrid]),e-=Math.abs(r[0])<this.options.snapToGrid/2?r[0]:r[0]-(e<0?-this.options.snapToGrid:this.options.snapToGrid),t-=Math.abs(r[1])<this.options.snapToGrid/2?r[1]:r[1]-(t<0?-this.options.snapToGrid:this.options.snapToGrid),this.constraintToBox(e,t,n,i)},e.prototype.constraintToBox=function(e,t,n,i){var r,a,o=this.options.constraint||{};return void 0!==i?(r=n,a=i):(r=this.parameters.box.x+(1&n?0:this.parameters.box.width),a=this.parameters.box.y+(2&n?0:this.parameters.box.height)),void 0!==o.minX&&r+e<o.minX&&(e=o.minX-r),void 0!==o.maxX&&r+e>o.maxX&&(e=o.maxX-r),void 0!==o.minY&&a+t<o.minY&&(t=o.minY-a),void 0!==o.maxY&&a+t>o.maxY&&(t=o.maxY-a),[e,t]},e.prototype.checkAspectRatio=function(e){if(!this.options.saveAspectRatio)return e;var t=e.slice(),n=this.parameters.box.width/this.parameters.box.height,i=this.parameters.box.width+e[0],r=this.parameters.box.height-e[1],a=i/r;return a<n?t[1]=i/n-this.parameters.box.height:a>n&&(t[0]=this.parameters.box.width-r*n),t},SVG.extend(SVG.Element,{resize:function(t){return(this.remember("_resizeHandler")||new e(this)).init(t||{}),this}}),SVG.Element.prototype.resize.defaults={snapToAngle:.1,snapToGrid:1,constraint:{},saveAspectRatio:!1}}).call(this)}();!function(e,t){void 0===t&&(t={});var n=t.insertAt;if(e&&"undefined"!=typeof document){var i=document.head||document.getElementsByTagName("head")[0],r=document.createElement("style");r.type="text/css","top"===n&&i.firstChild?i.insertBefore(r,i.firstChild):i.appendChild(r),r.styleSheet?r.styleSheet.cssText=e:r.appendChild(document.createTextNode(e))}}('.apexcharts-canvas {\n  position: relative;\n  user-select: none;\n  /* cannot give overflow: hidden as it will crop tooltips which overflow outside chart area */\n}\n\n/* scrollbar is not visible by default for legend, hence forcing the visibility */\n.apexcharts-canvas ::-webkit-scrollbar {\n  -webkit-appearance: none;\n  width: 6px;\n}\n.apexcharts-canvas ::-webkit-scrollbar-thumb {\n  border-radius: 4px;\n  background-color: rgba(0,0,0,.5);\n  box-shadow: 0 0 1px rgba(255,255,255,.5);\n  -webkit-box-shadow: 0 0 1px rgba(255,255,255,.5);\n}\n.apexcharts-canvas.dark {\n  background: #343F57;\n}\n\n.apexcharts-inner {\n  position: relative;\n}\n\n.legend-mouseover-inactive {\n  transition: 0.15s ease all;\n  opacity: 0.20;\n}\n\n.apexcharts-series-collapsed {\n  opacity: 0;\n}\n\n.apexcharts-gridline, .apexcharts-text {\n  pointer-events: none;\n}\n\n.apexcharts-tooltip {\n  border-radius: 5px;\n  box-shadow: 2px 2px 6px -4px #999;\n  cursor: default;\n  font-size: 14px;\n  left: 62px;\n  opacity: 0;\n  pointer-events: none;\n  position: absolute;\n  top: 20px;\n  overflow: hidden;\n  white-space: nowrap;\n  z-index: 12;\n  transition: 0.15s ease all;\n}\n.apexcharts-tooltip.light {\n  border: 1px solid #e3e3e3;\n  background: rgba(255, 255, 255, 0.96);\n}\n.apexcharts-tooltip.dark {\n  color: #fff;\n  background: rgba(30,30,30, 0.8);\n}\n.apexcharts-tooltip * {\n  font-family: inherit;\n}\n\n.apexcharts-tooltip .apexcharts-marker,\n.apexcharts-area-series .apexcharts-area,\n.apexcharts-line {\n  pointer-events: none;\n}\n\n.apexcharts-tooltip.active {\n  opacity: 1;\n  transition: 0.15s ease all;\n}\n\n.apexcharts-tooltip-title {\n  padding: 6px;\n  font-size: 15px;\n  margin-bottom: 4px;\n}\n.apexcharts-tooltip.light .apexcharts-tooltip-title {\n  background: #ECEFF1;\n  border-bottom: 1px solid #ddd;\n}\n.apexcharts-tooltip.dark .apexcharts-tooltip-title {\n  background: rgba(0, 0, 0, 0.7);\n  border-bottom: 1px solid #333;\n}\n\n.apexcharts-tooltip-text-value,\n.apexcharts-tooltip-text-z-value {\n  display: inline-block;\n  font-weight: 600;\n  margin-left: 5px;\n}\n\n.apexcharts-tooltip-text-z-label:empty,\n.apexcharts-tooltip-text-z-value:empty {\n  display: none;\n}\n\n.apexcharts-tooltip-text-value, \n.apexcharts-tooltip-text-z-value {\n  font-weight: 600;\n}\n\n.apexcharts-tooltip-marker {\n  width: 12px;\n  height: 12px;\n  position: relative;\n  top: 0px;\n  margin-right: 10px;\n  border-radius: 50%;\n}\n\n.apexcharts-tooltip-series-group {\n  padding: 0 10px;\n  display: none;\n  text-align: left;\n  justify-content: left;\n  align-items: center;\n}\n\n.apexcharts-tooltip-series-group.active .apexcharts-tooltip-marker {\n  opacity: 1;\n}\n.apexcharts-tooltip-series-group.active, .apexcharts-tooltip-series-group:last-child {\n  padding-bottom: 4px;\n}\n.apexcharts-tooltip-y-group {\n  padding: 6px 0 5px;\n}\n.apexcharts-tooltip-candlestick {\n  padding: 4px 8px;\n}\n.apexcharts-tooltip-candlestick > div {\n  margin: 4px 0;\n}\n.apexcharts-tooltip-candlestick span.value {\n  font-weight: bold;\n}\n\n.apexcharts-tooltip-rangebar {\n  padding: 5px 8px;\n}\n\n.apexcharts-tooltip-rangebar .category {\n  font-weight: 600;\n  color: #777;\n}\n\n.apexcharts-tooltip-rangebar .series-name {\n  font-weight: bold;\n  display: block;\n  margin-bottom: 5px;\n}\n\n.apexcharts-xaxistooltip {\n  opacity: 0;\n  padding: 9px 10px;\n  pointer-events: none;\n  color: #373d3f;\n  font-size: 13px;\n  text-align: center;\n  border-radius: 2px;\n  position: absolute;\n  z-index: 10;\n\tbackground: #ECEFF1;\n  border: 1px solid #90A4AE;\n  transition: 0.15s ease all;\n}\n\n.apexcharts-xaxistooltip.dark {\n  background: rgba(0, 0, 0, 0.7);\n  border: 1px solid rgba(0, 0, 0, 0.5);\n  color: #fff;\n}\n\n.apexcharts-xaxistooltip:after, .apexcharts-xaxistooltip:before {\n\tleft: 50%;\n\tborder: solid transparent;\n\tcontent: " ";\n\theight: 0;\n\twidth: 0;\n\tposition: absolute;\n\tpointer-events: none;\n}\n\n.apexcharts-xaxistooltip:after {\n\tborder-color: rgba(236, 239, 241, 0);\n\tborder-width: 6px;\n\tmargin-left: -6px;\n}\n.apexcharts-xaxistooltip:before {\n\tborder-color: rgba(144, 164, 174, 0);\n\tborder-width: 7px;\n\tmargin-left: -7px;\n}\n\n.apexcharts-xaxistooltip-bottom:after, .apexcharts-xaxistooltip-bottom:before {\n  bottom: 100%;\n}\n\n.apexcharts-xaxistooltip-top:after, .apexcharts-xaxistooltip-top:before {\n  top: 100%;\n}\n\n.apexcharts-xaxistooltip-bottom:after {\n  border-bottom-color: #ECEFF1;\n}\n.apexcharts-xaxistooltip-bottom:before {\n  border-bottom-color: #90A4AE;\n}\n\n.apexcharts-xaxistooltip-bottom.dark:after {\n  border-bottom-color: rgba(0, 0, 0, 0.5);\n}\n.apexcharts-xaxistooltip-bottom.dark:before {\n  border-bottom-color: rgba(0, 0, 0, 0.5);\n}\n\n.apexcharts-xaxistooltip-top:after {\n  border-top-color:#ECEFF1\n}\n.apexcharts-xaxistooltip-top:before {\n  border-top-color: #90A4AE;\n}\n.apexcharts-xaxistooltip-top.dark:after {\n  border-top-color:rgba(0, 0, 0, 0.5);\n}\n.apexcharts-xaxistooltip-top.dark:before {\n  border-top-color: rgba(0, 0, 0, 0.5);\n}\n\n\n.apexcharts-xaxistooltip.active {\n  opacity: 1;\n  transition: 0.15s ease all;\n}\n\n.apexcharts-yaxistooltip {\n  opacity: 0;\n  padding: 4px 10px;\n  pointer-events: none;\n  color: #373d3f;\n  font-size: 13px;\n  text-align: center;\n  border-radius: 2px;\n  position: absolute;\n  z-index: 10;\n\tbackground: #ECEFF1;\n  border: 1px solid #90A4AE;\n}\n\n.apexcharts-yaxistooltip.dark {\n  background: rgba(0, 0, 0, 0.7);\n  border: 1px solid rgba(0, 0, 0, 0.5);\n  color: #fff;\n}\n\n.apexcharts-yaxistooltip:after, .apexcharts-yaxistooltip:before {\n\ttop: 50%;\n\tborder: solid transparent;\n\tcontent: " ";\n\theight: 0;\n\twidth: 0;\n\tposition: absolute;\n\tpointer-events: none;\n}\n.apexcharts-yaxistooltip:after {\n\tborder-color: rgba(236, 239, 241, 0);\n\tborder-width: 6px;\n\tmargin-top: -6px;\n}\n.apexcharts-yaxistooltip:before {\n\tborder-color: rgba(144, 164, 174, 0);\n\tborder-width: 7px;\n\tmargin-top: -7px;\n}\n\n.apexcharts-yaxistooltip-left:after, .apexcharts-yaxistooltip-left:before {\n  left: 100%;\n}\n\n.apexcharts-yaxistooltip-right:after, .apexcharts-yaxistooltip-right:before {\n  right: 100%;\n}\n\n.apexcharts-yaxistooltip-left:after {\n  border-left-color: #ECEFF1;\n}\n.apexcharts-yaxistooltip-left:before {\n  border-left-color: #90A4AE;\n}\n.apexcharts-yaxistooltip-left.dark:after {\n  border-left-color: rgba(0, 0, 0, 0.5);\n}\n.apexcharts-yaxistooltip-left.dark:before {\n  border-left-color: rgba(0, 0, 0, 0.5);\n}\n\n.apexcharts-yaxistooltip-right:after {\n  border-right-color: #ECEFF1;\n}\n.apexcharts-yaxistooltip-right:before {\n  border-right-color: #90A4AE;\n}\n.apexcharts-yaxistooltip-right.dark:after {\n  border-right-color: rgba(0, 0, 0, 0.5);\n}\n.apexcharts-yaxistooltip-right.dark:before {\n  border-right-color: rgba(0, 0, 0, 0.5);\n}\n\n.apexcharts-yaxistooltip.active {\n  opacity: 1;\n}\n\n.apexcharts-xcrosshairs, .apexcharts-ycrosshairs {\n  pointer-events: none;\n  opacity: 0;\n  transition: 0.15s ease all;\n}\n\n.apexcharts-xcrosshairs.active, .apexcharts-ycrosshairs.active {\n  opacity: 1;\n  transition: 0.15s ease all;\n}\n\n.apexcharts-ycrosshairs-hidden {\n  opacity: 0;\n}\n\n.apexcharts-zoom-rect {\n  pointer-events: none;\n}\n.apexcharts-selection-rect {\n  cursor: move;\n}\n\n.svg_select_points, .svg_select_points_rot {\n  opacity: 0;\n  visibility: hidden;\n}\n.svg_select_points_l, .svg_select_points_r {\n  cursor: ew-resize;\n  opacity: 1;\n  visibility: visible;\n  fill: #888;\n}\n.apexcharts-canvas.zoomable .hovering-zoom {\n  cursor: crosshair\n}\n.apexcharts-canvas.zoomable .hovering-pan {\n  cursor: move\n}\n\n.apexcharts-xaxis,\n.apexcharts-yaxis {\n  pointer-events: none;\n}\n\n.apexcharts-zoom-icon, \n.apexcharts-zoom-in-icon,\n.apexcharts-zoom-out-icon,\n.apexcharts-reset-zoom-icon, \n.apexcharts-pan-icon, \n.apexcharts-selection-icon,\n.apexcharts-menu-icon, \n.apexcharts-toolbar-custom-icon {\n  cursor: pointer;\n  width: 20px;\n  height: 20px;\n  line-height: 24px;\n  color: #6E8192;\n  text-align: center;\n}\n\n\n.apexcharts-zoom-icon svg, \n.apexcharts-zoom-in-icon svg,\n.apexcharts-zoom-out-icon svg,\n.apexcharts-reset-zoom-icon svg,\n.apexcharts-menu-icon svg {\n  fill: #6E8192;\n}\n.apexcharts-selection-icon svg {\n  fill: #444;\n  transform: scale(0.76)\n}\n\n.dark .apexcharts-zoom-icon svg, \n.dark .apexcharts-zoom-in-icon svg,\n.dark .apexcharts-zoom-out-icon svg,\n.dark .apexcharts-reset-zoom-icon svg, \n.dark .apexcharts-pan-icon svg, \n.dark .apexcharts-selection-icon svg,\n.dark .apexcharts-menu-icon svg, \n.dark .apexcharts-toolbar-custom-icon svg{\n  fill: #f3f4f5;\n}\n\n.apexcharts-canvas .apexcharts-zoom-icon.selected svg, \n.apexcharts-canvas .apexcharts-selection-icon.selected svg, \n.apexcharts-canvas .apexcharts-reset-zoom-icon.selected svg {\n  fill: #008FFB;\n}\n.light .apexcharts-selection-icon:not(.selected):hover svg,\n.light .apexcharts-zoom-icon:not(.selected):hover svg, \n.light .apexcharts-zoom-in-icon:hover svg, \n.light .apexcharts-zoom-out-icon:hover svg, \n.light .apexcharts-reset-zoom-icon:hover svg, \n.light .apexcharts-menu-icon:hover svg {\n  fill: #333;\n}\n\n.apexcharts-selection-icon, .apexcharts-menu-icon {\n  position: relative;\n}\n.apexcharts-reset-zoom-icon {\n  margin-left: 5px;\n}\n.apexcharts-zoom-icon, .apexcharts-reset-zoom-icon, .apexcharts-menu-icon {\n  transform: scale(0.85);\n}\n\n.apexcharts-zoom-in-icon, .apexcharts-zoom-out-icon {\n  transform: scale(0.7)\n}\n\n.apexcharts-zoom-out-icon {\n  margin-right: 3px;\n}\n\n.apexcharts-pan-icon {\n  transform: scale(0.62);\n  position: relative;\n  left: 1px;\n  top: 0px;\n}\n.apexcharts-pan-icon svg {\n  fill: #fff;\n  stroke: #6E8192;\n  stroke-width: 2;\n}\n.apexcharts-pan-icon.selected svg {\n  stroke: #008FFB;\n}\n.apexcharts-pan-icon:not(.selected):hover svg {\n  stroke: #333;\n}\n\n.apexcharts-toolbar {\n  position: absolute;\n  z-index: 11;\n  top: 0px;\n  right: 3px;\n  max-width: 176px;\n  text-align: right;\n  border-radius: 3px;\n  padding: 0px 6px 2px 6px;\n  display: flex;\n  justify-content: space-between;\n  align-items: center; \n}\n\n.apexcharts-toolbar svg {\n  pointer-events: none;\n}\n\n.apexcharts-menu {\n  background: #fff;\n  position: absolute;\n  top: 100%;\n  border: 1px solid #ddd;\n  border-radius: 3px;\n  padding: 3px;\n  right: 10px;\n  opacity: 0;\n  min-width: 110px;\n  transition: 0.15s ease all;\n  pointer-events: none;\n}\n\n.apexcharts-menu.open {\n  opacity: 1;\n  pointer-events: all;\n  transition: 0.15s ease all;\n}\n\n.apexcharts-menu-item {\n  padding: 6px 7px;\n  font-size: 12px;\n  cursor: pointer;\n}\n.light .apexcharts-menu-item:hover {\n  background: #eee;\n}\n.dark .apexcharts-menu {\n  background: rgba(0, 0, 0, 0.7);\n  color: #fff;\n}\n\n@media screen and (min-width: 768px) {\n  .apexcharts-toolbar {\n    /*opacity: 0;*/\n  }\n\n  .apexcharts-canvas:hover .apexcharts-toolbar {\n    opacity: 1;\n  } \n}\n\n.apexcharts-datalabel.hidden {\n  opacity: 0;\n}\n\n.apexcharts-pie-label,\n.apexcharts-datalabel, .apexcharts-datalabel-label, .apexcharts-datalabel-value {\n  cursor: default;\n  pointer-events: none;\n}\n\n.apexcharts-pie-label-delay {\n  opacity: 0;\n  animation-name: opaque;\n  animation-duration: 0.3s;\n  animation-fill-mode: forwards;\n  animation-timing-function: ease;\n}\n\n.apexcharts-canvas .hidden {\n  opacity: 0;\n}\n\n.apexcharts-hide .apexcharts-series-points {\n  opacity: 0;\n}\n\n.apexcharts-area-series .apexcharts-series-markers .apexcharts-marker.no-pointer-events,\n.apexcharts-line-series .apexcharts-series-markers .apexcharts-marker.no-pointer-events, .apexcharts-radar-series path, .apexcharts-radar-series polygon {\n  pointer-events: none;\n}\n\n/* markers */\n\n.apexcharts-marker {\n  transition: 0.15s ease all;\n}\n\n@keyframes opaque {\n  0% {\n    opacity: 0;\n  }\n  100% {\n    opacity: 1;\n  }\n}'),
/*! @source http://purl.eligrey.com/github/classList.js/blob/master/classList.js */
"document"in self&&("classList"in document.createElement("_")&&(!document.createElementNS||"classList"in document.createElementNS("http://www.w3.org/2000/svg","g"))||function(e){if("Element"in e){var t=e.Element.prototype,n=Object,i=String.prototype.trim||function(){return this.replace(/^\s+|\s+$/g,"")},r=Array.prototype.indexOf||function(e){for(var t=0,n=this.length;t<n;t++)if(t in this&&this[t]===e)return t;return-1},a=function(e,t){this.name=e,this.code=DOMException[e],this.message=t},o=function(e,t){if(""===t)throw new a("SYNTAX_ERR","The token must not be empty.");if(/\s/.test(t))throw new a("INVALID_CHARACTER_ERR","The token must not contain space characters.");return r.call(e,t)},s=function(e){for(var t=i.call(e.getAttribute("class")||""),n=t?t.split(/\s+/):[],r=0,a=n.length;r<a;r++)this.push(n[r]);this._updateClassName=function(){e.setAttribute("class",this.toString())}},l=s.prototype=[],c=function(){return new s(this)};if(a.prototype=Error.prototype,l.item=function(e){return this[e]||null},l.contains=function(e){return~o(this,e+"")},l.add=function(){var e,t=arguments,n=0,i=t.length,r=!1;do{e=t[n]+"",~o(this,e)||(this.push(e),r=!0)}while(++n<i);r&&this._updateClassName()},l.remove=function(){var e,t,n=arguments,i=0,r=n.length,a=!1;do{for(e=n[i]+"",t=o(this,e);~t;)this.splice(t,1),a=!0,t=o(this,e)}while(++i<r);a&&this._updateClassName()},l.toggle=function(e,t){var n=this.contains(e),i=n?!0!==t&&"remove":!1!==t&&"add";return i&&this[i](e),!0===t||!1===t?t:!n},l.replace=function(e,t){var n=o(e+"");~n&&(this.splice(n,1,t),this._updateClassName())},l.toString=function(){return this.join(" ")},n.defineProperty){var u={get:c,enumerable:!0,configurable:!0};try{n.defineProperty(t,"classList",u)}catch(e){void 0!==e.number&&-2146823252!==e.number||(u.enumerable=!1,n.defineProperty(t,"classList",u))}}else n.prototype.__defineGetter__&&t.__defineGetter__("classList",c)}}(self),function(){var e=document.createElement("_");if(e.classList.add("c1","c2"),!e.classList.contains("c2")){var t=function(e){var t=DOMTokenList.prototype[e];DOMTokenList.prototype[e]=function(e){var n,i=arguments.length;for(n=0;n<i;n++)e=arguments[n],t.call(this,e)}};t("add"),t("remove")}if(e.classList.toggle("c3",!1),e.classList.contains("c3")){var n=DOMTokenList.prototype.toggle;DOMTokenList.prototype.toggle=function(e,t){return 1 in arguments&&!this.contains(e)==!t?t:n.call(this,e)}}"replace"in document.createElement("_").classList||(DOMTokenList.prototype.replace=function(e,t){var n=this.toString().split(" "),i=n.indexOf(e+"");~i&&(n=n.slice(i),this.remove.apply(this,n),this.add(t),this.add.apply(this,n.slice(1)))}),e=null}()),function(){var e=!1;function t(e){var t=e.__resizeTriggers__,n=t.firstElementChild,i=t.lastElementChild,r=n.firstElementChild;i.scrollLeft=i.scrollWidth,i.scrollTop=i.scrollHeight,r.style.width=n.offsetWidth+1+"px",r.style.height=n.offsetHeight+1+"px",n.scrollLeft=n.scrollWidth,n.scrollTop=n.scrollHeight}function n(e){var n=this;t(this),this.__resizeRAF__&&o(this.__resizeRAF__),this.__resizeRAF__=a(function(){(function(e){return e.offsetWidth!=e.__resizeLast__.width||e.offsetHeight!=e.__resizeLast__.height})(n)&&(n.__resizeLast__.width=n.offsetWidth,n.__resizeLast__.height=n.offsetHeight,n.__resizeListeners__.forEach(function(t){t.call(e)}))})}var i,r,a=(i=window.requestAnimationFrame||window.mozRequestAnimationFrame||window.webkitRequestAnimationFrame||function(e){return window.setTimeout(e,20)},function(e){return i(e)}),o=(r=window.cancelAnimationFrame||window.mozCancelAnimationFrame||window.webkitCancelAnimationFrame||window.clearTimeout,function(e){return r(e)}),s=!1,l="",c="animationstart",u="Webkit Moz O ms".split(" "),h="webkitAnimationStart animationstart oAnimationStart MSAnimationStart".split(" "),d=document.createElement("fakeelement");if(void 0!==d.style.animationName&&(s=!0),!1===s)for(var f=0;f<u.length;f++)if(void 0!==d.style[u[f]+"AnimationName"]){l="-"+u[f].toLowerCase()+"-",c=h[f];break}var p="@"+l+"keyframes resizeanim { from { opacity: 0; } to { opacity: 0; } } ",g=l+"animation: 1ms resizeanim; ";window.addResizeListener=function(i,r){i.__resizeTriggers__||("static"==getComputedStyle(i).position&&(i.style.position="relative"),function(){if(!e){var t=(p||"")+".resize-triggers { "+(g||"")+'visibility: hidden; opacity: 0; } .resize-triggers, .resize-triggers > div, .contract-trigger:before { content: " "; display: block; position: absolute; top: 0; left: 0; height: 100%; width: 100%; overflow: hidden; } .resize-triggers > div { background: #eee; overflow: auto; } .contract-trigger:before { width: 200%; height: 200%; }',n=document.head||document.getElementsByTagName("head")[0],i=document.createElement("style");i.type="text/css",i.styleSheet?i.styleSheet.cssText=t:i.appendChild(document.createTextNode(t)),n.appendChild(i),e=!0}}(),i.__resizeLast__={},i.__resizeListeners__=[],(i.__resizeTriggers__=document.createElement("div")).className="resize-triggers",i.__resizeTriggers__.innerHTML='<div class="expand-trigger"><div></div></div><div class="contract-trigger"></div>',i.appendChild(i.__resizeTriggers__),t(i),i.addEventListener("scroll",n,!0),c&&i.__resizeTriggers__.addEventListener(c,function(e){"resizeanim"==e.animationName&&t(i)})),i.__resizeListeners__.push(r)},window.removeResizeListener=function(e,t){e&&(e.__resizeListeners__.splice(e.__resizeListeners__.indexOf(t),1),e.__resizeListeners__.length||(e.removeEventListener("scroll",n),e.__resizeTriggers__=!e.removeChild(e.__resizeTriggers__)))}}(),window.Apex={};var Ce=function(){function e(t,n){o(this,e),this.opts=n,this.ctx=this,this.w=new _(n).init(),this.el=t,this.w.globals.cuid=(Math.random()+1).toString(36).substring(4),this.w.globals.chartID=this.w.config.chart.id?this.w.config.chart.id:this.w.globals.cuid,this.initModules(),this.create=m.bind(this.create,this),this.windowResizeHandler=this.windowResize.bind(this)}return l(e,[{key:"render",value:function(){var e=this;return new te(function(t,n){if(null!==e.el){void 0===Apex._chartInstances&&(Apex._chartInstances=[]),e.w.config.chart.id&&Apex._chartInstances.push({id:e.w.globals.chartID,group:e.w.config.chart.group,chart:e}),e.setLocale(e.w.config.chart.defaultLocale);var i=e.w.config.chart.events.beforeMount;"function"==typeof i&&i(e,e.w),e.fireEvent("beforeMount",[e,e.w]),window.addEventListener("resize",e.windowResizeHandler),window.addResizeListener(e.el.parentNode,e.parentResizeCallback.bind(e));var r=e.create(e.w.config.series,{});if(!r)return t(e);e.mount(r).then(function(){t(r),"function"==typeof e.w.config.chart.events.mounted&&e.w.config.chart.events.mounted(e,e.w),e.fireEvent("mounted",[e,e.w])}).catch(function(e){n(e)})}else n(new Error("Element not found"))})}},{key:"initModules",value:function(){this.animations=new y(this),this.annotations=new k(this),this.core=new K(this.el,this),this.grid=new ue(this),this.coreUtils=new C(this),this.config=new T({}),this.crosshairs=new R(this),this.options=new w,this.responsive=new de(this),this.series=new Q(this),this.theme=new fe(this),this.formatters=new X(this),this.titleSubtitle=new Ee(this),this.legend=new he(this),this.toolbar=new ke(this),this.dimensions=new $(this),this.zoomPanSelection=new Se(this),this.w.globals.tooltip=new we(this)}},{key:"addEventListener",value:function(e,t){var n=this.w;n.globals.events.hasOwnProperty(e)?n.globals.events[e].push(t):n.globals.events[e]=[t]}},{key:"removeEventListener",value:function(e,t){var n=this.w;if(n.globals.events.hasOwnProperty(e)){var i=n.globals.events[e].indexOf(t);-1!==i&&n.globals.events[e].splice(i,1)}}},{key:"fireEvent",value:function(e,t){var n=this.w;if(n.globals.events.hasOwnProperty(e)){t&&t.length||(t=[]);for(var i=n.globals.events[e],r=i.length,a=0;a<r;a++)i[a].apply(null,t)}}},{key:"create",value:function(e,t){var n=this.w;this.initModules();var i=this.w.globals;if(i.noData=!1,i.animationEnded=!1,this.responsive.checkResponsiveConfig(t),null===this.el)return i.animationEnded=!0,null;if(this.core.setupElements(),0===i.svgWidth)return i.animationEnded=!0,null;var r=C.checkComboSeries(e);i.comboCharts=r.comboCharts,i.comboChartsHasBars=r.comboChartsHasBars,(0===e.length||1===e.length&&e[0].data&&0===e[0].data.length)&&this.series.handleNoData(),this.setupEventHandlers(),this.core.parseData(e),this.theme.init(),new P(this).setGlobalMarkerSize(),this.formatters.setLabelFormatters(),this.titleSubtitle.draw(),this.legend.init(),this.series.hasAllSeriesEqualX(),i.axisCharts&&(this.core.coreCalculations(),"category"!==n.config.xaxis.type&&this.formatters.setLabelFormatters()),this.formatters.heatmapLabelFormatters(),this.dimensions.plotCoords();var a=this.core.xySettings();this.grid.createGridMask();var o=this.core.plotChartType(e,a);this.core.shiftGraphPosition();var s={plot:{left:n.globals.translateX,top:n.globals.translateY,width:n.globals.gridWidth,height:n.globals.gridHeight}};return{elGraph:o,xyRatios:a,elInner:n.globals.dom.elGraphical,dimensions:s}}},{key:"mount",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:null,t=this,n=t.w;return new te(function(i,r){if(null===t.el)return r(new Error("Not enough data to display or target element not found"));if((null===e||n.globals.allSeriesCollapsed)&&t.series.handleNoData(),t.core.drawAxis(n.config.chart.type,e.xyRatios),t.grid=new ue(t),"back"===n.config.grid.position&&t.grid.drawGrid(),"back"===n.config.annotations.position&&t.annotations.drawAnnotations(),e.elGraph instanceof Array)for(var a=0;a<e.elGraph.length;a++)n.globals.dom.elGraphical.add(e.elGraph[a]);else n.globals.dom.elGraphical.add(e.elGraph);if("front"===n.config.grid.position&&t.grid.drawGrid(),"front"===n.config.xaxis.crosshairs.position&&t.crosshairs.drawXCrosshairs(),"front"===n.config.yaxis[0].crosshairs.position&&t.crosshairs.drawYCrosshairs(),"front"===n.config.annotations.position&&t.annotations.drawAnnotations(),!n.globals.noData){if(n.config.tooltip.enabled&&!n.globals.noData&&t.w.globals.tooltip.drawTooltip(e.xyRatios),n.globals.axisCharts&&n.globals.isXNumeric)(n.config.chart.zoom.enabled||n.config.chart.selection&&n.config.chart.selection.enabled||n.config.chart.pan&&n.config.chart.pan.enabled)&&t.zoomPanSelection.init({xyRatios:e.xyRatios});else{var o=n.config.chart.toolbar.tools;o.zoom=!1,o.zoomin=!1,o.zoomout=!1,o.selection=!1,o.pan=!1,o.reset=!1}n.config.chart.toolbar.show&&!n.globals.allSeriesCollapsed&&t.toolbar.createToolbar()}n.globals.memory.methodsToExec.length>0&&n.globals.memory.methodsToExec.forEach(function(e){e.method(e.params,!1,e.context)}),i(t)})}},{key:"clearPreviousPaths",value:function(){var e=this.w;e.globals.previousPaths=[],e.globals.allSeriesCollapsed=!1,e.globals.collapsedSeries=[],e.globals.collapsedSeriesIndices=[]}},{key:"updateOptions",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]&&arguments[1],n=!(arguments.length>2&&void 0!==arguments[2])||arguments[2],i=!(arguments.length>3&&void 0!==arguments[3])||arguments[3],r=this.w;return e.series&&(e.series[0].data&&(e.series=e.series.map(function(e,t){return u({},r.config.series[t],{name:e.name?e.name:r.config.series[t]&&r.config.series[t].name,type:e.type?e.type:r.config.series[t]&&r.config.series[t].type,data:e.data?e.data:r.config.series[t]&&r.config.series[t].data})})),this.revertDefaultAxisMinMax()),e.xaxis&&((e.xaxis.min||e.xaxis.max)&&this.forceXAxisUpdate(e),e.xaxis.categories&&e.xaxis.categories.length&&r.config.xaxis.convertedCatToNumeric&&(e=E.convertCatToNumeric(e))),r.globals.collapsedSeriesIndices.length>0&&this.clearPreviousPaths(),e.theme&&(e=this.theme.updateThemeOptions(e)),this._updateOptions(e,t,n,i)}},{key:"_updateOptions",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]&&arguments[1],n=!(arguments.length>2&&void 0!==arguments[2])||arguments[2],i=arguments.length>3&&void 0!==arguments[3]&&arguments[3];this.getSyncedCharts().forEach(function(r){var o=r.w;return o.globals.shouldAnimate=n,t||(o.globals.resized=!0,o.globals.dataChanged=!0,n&&r.series.getPreviousPaths()),e&&"object"===a(e)&&(r.config=new T(e),e=C.extendArrayProps(r.config,e),o.config=m.extend(o.config,e),i&&(o.globals.lastXAxis=[],o.globals.lastYAxis=[],o.globals.initialConfig=m.extend({},o.config),o.globals.initialSeries=JSON.parse(JSON.stringify(o.config.series)))),r.update(e)})}},{key:"updateSeries",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:[],t=!(arguments.length>1&&void 0!==arguments[1])||arguments[1],n=!(arguments.length>2&&void 0!==arguments[2])||arguments[2];return this.revertDefaultAxisMinMax(),this._updateSeries(e,t,n)}},{key:"appendSeries",value:function(e){var t=!(arguments.length>1&&void 0!==arguments[1])||arguments[1],n=!(arguments.length>2&&void 0!==arguments[2])||arguments[2],i=this.w.config.series.slice();return i.push(e),this.revertDefaultAxisMinMax(),this._updateSeries(i,t,n)}},{key:"_updateSeries",value:function(e,t){var n,i=arguments.length>2&&void 0!==arguments[2]&&arguments[2],r=this.w;return this.w.globals.shouldAnimate=t,r.globals.dataChanged=!0,r.globals.allSeriesCollapsed&&(r.globals.allSeriesCollapsed=!1),t&&this.series.getPreviousPaths(),r.globals.axisCharts?(0===(n=e.map(function(e,t){return u({},r.config.series[t],{name:e.name?e.name:r.config.series[t]&&r.config.series[t].name,type:e.type?e.type:r.config.series[t]&&r.config.series[t].type,data:e.data?e.data:r.config.series[t]&&r.config.series[t].data})})).length&&(n=[{data:[]}]),r.config.series=n):r.config.series=e.slice(),i&&(r.globals.initialConfig.series=JSON.parse(JSON.stringify(r.config.series)),r.globals.initialSeries=JSON.parse(JSON.stringify(r.config.series))),this.update()}},{key:"getSyncedCharts",value:function(){var e=this.getGroupedCharts(),t=[this];return e.length&&(t=[],e.forEach(function(e){t.push(e)})),t}},{key:"getGroupedCharts",value:function(){var e=this;return Apex._chartInstances.filter(function(e){if(e.group)return!0}).map(function(t){return e.w.config.chart.group===t.group?t.chart:e})}},{key:"appendData",value:function(e){var t=!(arguments.length>1&&void 0!==arguments[1])||arguments[1],n=this;n.w.globals.dataChanged=!0,n.series.getPreviousPaths();for(var i=n.w.config.series.slice(),r=0;r<i.length;r++)if(void 0!==e[r])for(var a=0;a<e[r].data.length;a++)i[r].data.push(e[r].data[a]);return n.w.config.series=i,t&&(n.w.globals.initialSeries=JSON.parse(JSON.stringify(n.w.config.series))),this.update()}},{key:"update",value:function(e){var t=this;return new te(function(n,i){t.clear();var r=t.create(t.w.config.series,e);if(!r)return n(t);t.mount(r).then(function(){"function"==typeof t.w.config.chart.events.updated&&t.w.config.chart.events.updated(t,t.w),t.fireEvent("updated",[t,t.w]),t.w.globals.isDirty=!0,n(t)}).catch(function(e){i(e)})})}},{key:"forceXAxisUpdate",value:function(e){var t=this.w;void 0!==e.xaxis.min&&(t.config.xaxis.min=e.xaxis.min,t.globals.lastXAxis.min=e.xaxis.min),void 0!==e.xaxis.max&&(t.config.xaxis.max=e.xaxis.max,t.globals.lastXAxis.max=e.xaxis.max)}},{key:"revertDefaultAxisMinMax",value:function(){var e=this.w;e.config.xaxis.min=e.globals.lastXAxis.min,e.config.xaxis.max=e.globals.lastXAxis.max,e.config.yaxis.map(function(t,n){e.globals.zoomed&&void 0!==e.globals.lastYAxis[n]&&(t.min=e.globals.lastYAxis[n].min,t.max=e.globals.lastYAxis[n].max)})}},{key:"clear",value:function(){this.zoomPanSelection&&this.zoomPanSelection.destroy(),this.toolbar&&this.toolbar.destroy(),this.animations=null,this.annotations=null,this.core=null,this.grid=null,this.series=null,this.responsive=null,this.theme=null,this.formatters=null,this.titleSubtitle=null,this.legend=null,this.dimensions=null,this.options=null,this.crosshairs=null,this.zoomPanSelection=null,this.toolbar=null,this.w.globals.tooltip=null,this.clearDomElements()}},{key:"killSVG",value:function(e){return new te(function(t,n){e.each(function(e,t){this.removeClass("*"),this.off(),this.stop()},!0),e.ungroup(),e.clear(),t("done")})}},{key:"clearDomElements",value:function(){var e=this.w.globals.dom;if(null!==this.el)for(;this.el.firstChild;)this.el.removeChild(this.el.firstChild);this.killSVG(e.Paper),e.Paper.remove(),e.elWrap=null,e.elGraphical=null,e.elLegendWrap=null,e.baseEl=null,e.elGridRect=null,e.elGridRectMask=null,e.elGridRectMarkerMask=null,e.elDefs=null}},{key:"destroy",value:function(){this.clear();var e=this.w.config.chart.id;e&&Apex._chartInstances.forEach(function(t,n){t.id===e&&Apex._chartInstances.splice(n,1)}),window.removeEventListener("resize",this.windowResizeHandler),window.removeResizeListener(this.el.parentNode,this.parentResizeCallback.bind(this))}},{key:"toggleSeries",value:function(e){var t=this.series.getSeriesByName(e),n=parseInt(t.getAttribute("data:realIndex")),i=t.classList.contains("apexcharts-series-collapsed");this.legend.toggleDataSeries(n,i)}},{key:"resetToggleSeries",value:function(){this.legend.resetToggleDataSeries()}},{key:"setupEventHandlers",value:function(){var e=this.w,t=this,n=e.globals.dom.baseEl.querySelector(e.globals.chartClass),i=["mousedown","mousemove","touchstart","touchmove","mouseup","touchend"];i.forEach(function(i){n.addEventListener(i,function(n){"mousedown"===n.type&&1===n.which||("mouseup"===n.type&&1===n.which||"touchend"===n.type)&&("function"==typeof e.config.chart.events.click&&e.config.chart.events.click(n,t,e),t.fireEvent("click",[n,t,e]))},{capture:!1,passive:!0})}),i.forEach(function(t){document.addEventListener(t,function(t){e.globals.clientX="touchmove"===t.type?t.touches[0].clientX:t.clientX,e.globals.clientY="touchmove"===t.type?t.touches[0].clientY:t.clientY})}),this.core.setupBrushHandler()}},{key:"addXaxisAnnotation",value:function(e){var t=!(arguments.length>1&&void 0!==arguments[1])||arguments[1],n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:void 0,i=this;n&&(i=n),i.annotations.addXaxisAnnotationExternal(e,t,i)}},{key:"addYaxisAnnotation",value:function(e){var t=!(arguments.length>1&&void 0!==arguments[1])||arguments[1],n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:void 0,i=this;n&&(i=n),i.annotations.addYaxisAnnotationExternal(e,t,i)}},{key:"addPointAnnotation",value:function(e){var t=!(arguments.length>1&&void 0!==arguments[1])||arguments[1],n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:void 0,i=this;n&&(i=n),i.annotations.addPointAnnotationExternal(e,t,i)}},{key:"clearAnnotations",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:void 0,t=this;e&&(t=e),t.annotations.clearAnnotations(t)}},{key:"addText",value:function(e){var t=!(arguments.length>1&&void 0!==arguments[1])||arguments[1],n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:void 0,i=this;n&&(i=n),i.annotations.addText(e,t,i)}},{key:"getChartArea",value:function(){return this.w.globals.dom.baseEl.querySelector(".apexcharts-inner")}},{key:"getSeriesTotalXRange",value:function(e,t){return this.coreUtils.getSeriesTotalsXRange(e,t)}},{key:"getHighestValueInSeries",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:0;return new G(this.ctx).getMinYMaxY(e).highestY}},{key:"getLowestValueInSeries",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:0;return new G(this.ctx).getMinYMaxY(e).lowestY}},{key:"getSeriesTotal",value:function(){return this.w.globals.seriesTotals}},{key:"setLocale",value:function(e){this.setCurrentLocaleValues(e)}},{key:"setCurrentLocaleValues",value:function(e){var t=this.w.config.chart.locales;window.Apex.chart&&window.Apex.chart.locales&&window.Apex.chart.locales.length>0&&(t=this.w.config.chart.locales.concat(window.Apex.chart.locales));var n=t.filter(function(t){return t.name===e})[0];if(!n)throw new Error("Wrong locale name provided. Please make sure you set the correct locale name in options");var i=m.extend(x,n);this.w.globals.locale=i.options}},{key:"dataURI",value:function(){return new ce(this.ctx).dataURI()}},{key:"paper",value:function(){return this.w.globals.dom.Paper}},{key:"parentResizeCallback",value:function(){this.w.globals.animationEnded&&this.windowResize()}},{key:"windowResize",value:function(){var e=this;clearTimeout(this.w.globals.resizeTimer),this.w.globals.resizeTimer=window.setTimeout(function(){e.w.globals.resized=!0,e.w.globals.dataChanged=!1,e.update()},150)}}],[{key:"initOnLoad",value:function(){for(var t=document.querySelectorAll("[data-apexcharts]"),n=0;n<t.length;n++){new e(t[n],JSON.parse(t[n].getAttribute("data-options"))).render()}}},{key:"exec",value:function(e,t){var n=this.getChartByID(e);if(n){for(var i=arguments.length,r=new Array(i>2?i-2:0),a=2;a<i;a++)r[a-2]=arguments[a];switch(t){case"updateOptions":return n.updateOptions.apply(n,r);case"updateSeries":return n.updateSeries.apply(n,r);case"appendData":return n.appendData.apply(n,r);case"appendSeries":return n.appendSeries.apply(n,r);case"toggleSeries":return n.toggleSeries.apply(n,r);case"dataURI":return n.dataURI.apply(n,r);case"addXaxisAnnotation":return n.addXaxisAnnotation.apply(n,r);case"addYaxisAnnotation":return n.addYaxisAnnotation.apply(n,r);case"addPointAnnotation":return n.addPointAnnotation.apply(n,r);case"addText":return n.addText.apply(n,r);case"clearAnnotations":return n.clearAnnotations.apply(n,r);case"paper":return n.paper.apply(n,r);case"destroy":return n.destroy()}}}},{key:"merge",value:function(e,t){return m.extend(e,t)}},{key:"getChartByID",value:function(e){return Apex._chartInstances.filter(function(t){return t.id===e})[0].chart}}]),e}();e.exports=Ce}).call(this,n(85).setImmediate)},function(e,t,n){(function(e){var i=void 0!==e&&e||"undefined"!=typeof self&&self||window,r=Function.prototype.apply;function a(e,t){this._id=e,this._clearFn=t}t.setTimeout=function(){return new a(r.call(setTimeout,i,arguments),clearTimeout)},t.setInterval=function(){return new a(r.call(setInterval,i,arguments),clearInterval)},t.clearTimeout=t.clearInterval=function(e){e&&e.close()},a.prototype.unref=a.prototype.ref=function(){},a.prototype.close=function(){this._clearFn.call(i,this._id)},t.enroll=function(e,t){clearTimeout(e._idleTimeoutId),e._idleTimeout=t},t.unenroll=function(e){clearTimeout(e._idleTimeoutId),e._idleTimeout=-1},t._unrefActive=t.active=function(e){clearTimeout(e._idleTimeoutId);var t=e._idleTimeout;t>=0&&(e._idleTimeoutId=setTimeout(function(){e._onTimeout&&e._onTimeout()},t))},n(86),t.setImmediate="undefined"!=typeof self&&self.setImmediate||void 0!==e&&e.setImmediate||this&&this.setImmediate,t.clearImmediate="undefined"!=typeof self&&self.clearImmediate||void 0!==e&&e.clearImmediate||this&&this.clearImmediate}).call(this,n(14))},function(e,t,n){(function(e,t){!function(e,n){"use strict";if(!e.setImmediate){var i,r,a,o,s,l=1,c={},u=!1,h=e.document,d=Object.getPrototypeOf&&Object.getPrototypeOf(e);d=d&&d.setTimeout?d:e,"[object process]"==={}.toString.call(e.process)?i=function(e){t.nextTick(function(){p(e)})}:!function(){if(e.postMessage&&!e.importScripts){var t=!0,n=e.onmessage;return e.onmessage=function(){t=!1},e.postMessage("","*"),e.onmessage=n,t}}()?e.MessageChannel?((a=new MessageChannel).port1.onmessage=function(e){p(e.data)},i=function(e){a.port2.postMessage(e)}):h&&"onreadystatechange"in h.createElement("script")?(r=h.documentElement,i=function(e){var t=h.createElement("script");t.onreadystatechange=function(){p(e),t.onreadystatechange=null,r.removeChild(t),t=null},r.appendChild(t)}):i=function(e){setTimeout(p,0,e)}:(o="setImmediate$"+Math.random()+"$",s=function(t){t.source===e&&"string"==typeof t.data&&0===t.data.indexOf(o)&&p(+t.data.slice(o.length))},e.addEventListener?e.addEventListener("message",s,!1):e.attachEvent("onmessage",s),i=function(t){e.postMessage(o+t,"*")}),d.setImmediate=function(e){"function"!=typeof e&&(e=new Function(""+e));for(var t=new Array(arguments.length-1),n=0;n<t.length;n++)t[n]=arguments[n+1];var r={callback:e,args:t};return c[l]=r,i(l),l++},d.clearImmediate=f}function f(e){delete c[e]}function p(e){if(u)setTimeout(p,0,e);else{var t=c[e];if(t){u=!0;try{!function(e){var t=e.callback,i=e.args;switch(i.length){case 0:t();break;case 1:t(i[0]);break;case 2:t(i[0],i[1]);break;case 3:t(i[0],i[1],i[2]);break;default:t.apply(n,i)}}(t)}finally{f(e),u=!1}}}}}("undefined"==typeof self?void 0===e?this:e:self)}).call(this,n(14),n(15))},function(e,t,n){"use strict";var i=n(88);function r(){}function a(){}a.resetWarningCache=r,e.exports=function(){function e(e,t,n,r,a,o){if(o!==i){var s=new Error("Calling PropTypes validators directly is not supported by the `prop-types` package. Use PropTypes.checkPropTypes() to call them. Read more at http://fb.me/use-check-prop-types");throw s.name="Invariant Violation",s}}function t(){return e}e.isRequired=e;var n={array:e,bool:e,func:e,number:e,object:e,string:e,symbol:e,any:e,arrayOf:t,element:e,elementType:e,instanceOf:t,node:e,objectOf:t,oneOf:t,oneOfType:t,shape:t,exact:t,checkPropTypes:a,resetWarningCache:r};return n.PropTypes=n,n}},function(e,t,n){"use strict";e.exports="SECRET_DO_NOT_PASS_THIS_OR_YOU_WILL_BE_FIRED"},function(e,t,n){"use strict";function i(e){this.map=new Map,this.newest=null,this.oldest=null,this.max=e&&e.max,this.dispose=e&&e.dispose}t.Cache=i;var r=i.prototype;function a(e,t){var n=e.map.get(t);if(n&&n!==e.newest){var i=n.older,r=n.newer;r&&(r.older=i),i&&(i.newer=r),n.older=e.newest,n.older.newer=n,n.newer=null,e.newest=n,n===e.oldest&&(e.oldest=r)}return n}r.has=function(e){return this.map.has(e)},r.get=function(e){var t=a(this,e);return t&&t.value},r.set=function(e,t){var n=a(this,e);return n?n.value=t:(n={key:e,value:t,newer:null,older:this.newest},this.newest&&(this.newest.newer=n),this.newest=n,this.oldest=this.oldest||n,this.map.set(e,n),n.value)},r.clean=function(){if("number"==typeof this.max)for(;this.oldest&&this.map.size>this.max;)this.delete(this.oldest.key)},r.delete=function(e){var t=this.map.get(e);return!!t&&(t===this.newest&&(this.newest=t.older),t===this.oldest&&(this.oldest=t.newer),t.newer&&(t.newer.older=t.older),t.older&&(t.older.newer=t.newer),this.map.delete(e),"function"==typeof this.dispose&&this.dispose(e,t.value),!0)}},function(e,t,n){"use strict";n.r(t),n.d(t,"tuple",function(){return f}),n.d(t,"lookup",function(){return h}),n.d(t,"lookupArray",function(){return d});var i="function"==typeof Symbol&&"function"==typeof Symbol.for,r=i?Symbol.for("immutable-tuple"):"@@__IMMUTABLE_TUPLE__@@",a=i?Symbol.for("immutable-tuple-root"):"@@__IMMUTABLE_TUPLE_ROOT__@@";function o(e,t,n,i){return Object.defineProperty(e,t,{value:n,enumerable:!!i,writable:!1,configurable:!1}),n}var s=Object.freeze||function(e){return e};function l(e){switch(typeof e){case"object":if(null===e)return!1;case"function":return!0;default:return!1}}var c=function(){this._weakMap=null,this._strongMap=null,this.data=null};c.prototype.get=function(e){var t=this._getMap(e,!1);if(t)return t.get(e)},c.prototype.set=function(e,t){return this._getMap(e,!0).set(e,t),t},c.prototype._getMap=function(e,t){return t?l(e)?this._weakMap||(this._weakMap=new WeakMap):this._strongMap||(this._strongMap=new Map):l(e)?this._weakMap:this._strongMap};var u=Array[a]||o(Array,a,new c,!1);function h(){return d(arguments)}function d(e){for(var t=u,n=e.length,i=0;i<n;++i){var r=e[i];t=t.get(r)||t.set(r,new c)}return t.data||(t.data=Object.create(null))}function f(){var e=arguments,t=h.apply(null,arguments);if(t.tuple)return t.tuple;for(var n=Object.create(f.prototype),i=arguments.length,r=0;r<i;++r)n[r]=e[r];return o(n,"length",i,!1),s(t.tuple=n)}function p(e){return!(!e||!0!==e[r])}function g(e){for(var t=[],n=e.length;n--;)t[n]=e[n];return t}o(f.prototype,r,!0,!1),f.isTuple=p,function(e){function t(t,n){var i=Object.getOwnPropertyDescriptor(Array.prototype,t);e(t,i,!!n)}t("every"),t("filter"),t("find"),t("findIndex"),t("forEach"),t("includes"),t("indexOf"),t("join"),t("lastIndexOf"),t("map"),t("reduce"),t("reduceRight"),t("slice"),t("some"),t("toLocaleString"),t("toString"),t("reverse",!0),t("sort",!0),t(i&&Symbol.iterator||"@@iterator")}(function(e,t,n){var i=t&&t.value;"function"==typeof i&&(t.value=function(){for(var e=[],t=arguments.length;t--;)e[t]=arguments[t];var r=i.apply(n?g(this):this,e);return Array.isArray(r)?f.apply(void 0,r):r},Object.defineProperty(f.prototype,e,t))});var m=Array.prototype.concat;f.prototype.concat=function(){for(var e=[],t=arguments.length;t--;)e[t]=arguments[t];return f.apply(void 0,m.apply(g(this),e.map(function(e){return p(e)?g(e):e})))},t.default=f},function(e,t,n){"use strict";var i=n(57).get,r=Object.create(null),a=[],o=[];function s(e,t){if(!e)throw new Error(t||"assertion failure")}function l(e,t,n){this.parents=new Set,this.childValues=new Map,this.dirtyChildren=null,c(this,e,t,n),++l.count}function c(e,t,n,i){e.fn=t,e.key=n,e.args=i,e.value=r,e.dirty=!0,e.subscribe=null,e.unsubscribe=null,e.recomputing=!1,e.reportOrphan=null}t.POOL_TARGET_SIZE=100,l.count=0,l.acquire=function(e,t,n){var i=o.pop();return i?(c(i,e,t,n),i):new l(e,t,n)},t.Entry=l;var u=l.prototype;function h(e){var t=e.reportOrphan;return"function"==typeof t&&0===e.parents.size&&!0===t(e)}function d(e){e.parents.forEach(function(t){g(t,e)})}function f(e){e.parents.forEach(function(t){m(t,e)})}function p(e){return e.dirty||e.dirtyChildren&&e.dirtyChildren.size}function g(e,t){if(s(e.childValues.has(t)),s(p(t)),e.dirtyChildren){if(e.dirtyChildren.has(t))return}else e.dirtyChildren=a.pop()||new Set;e.dirtyChildren.add(t),d(e)}function m(e,t){var n=e.childValues;s(n.has(t)),s(!p(t));var i=n.get(t);i===r?n.set(t,t.value):i!==t.value&&e.setDirty(),v(e,t),p(e)||f(e)}function v(e,n){var i=e.dirtyChildren;i&&(i.delete(n),0===i.size&&(a.length<t.POOL_TARGET_SIZE&&a.push(i),e.dirtyChildren=null))}function y(e){s(!e.recomputing,"already recomputing"),e.recomputing=!0;var t=x(e),n=i(),r=n.currentParentEntry;n.currentParentEntry=e;var a=!0;try{e.value=e.fn.apply(null,e.args),a=!1}finally{e.recomputing=!1,s(n.currentParentEntry===e),n.currentParentEntry=r,a||!function(e){if("function"==typeof e.subscribe)try{k(e),e.unsubscribe=e.subscribe.apply(null,e.args)}catch(t){return e.setDirty(),!1}return!0}(e)?e.setDirty():function(e){e.dirty=!1,p(e)||f(e)}(e)}return t.forEach(h),e.value}u.recompute=function(){if(function(e){var t=i().currentParentEntry;if(t)return e.parents.add(t),t.childValues.has(e)||t.childValues.set(e,r),p(e)?g(t,e):m(t,e),t}(this)||!h(this))return function e(t){if(t.dirty)return y(t);if(p(t)&&(t.dirtyChildren.forEach(function(n){s(t.childValues.has(n));try{e(n)}catch(e){t.setDirty()}}),t.dirty))return y(t);s(t.value!==r);return t.value}(this)},u.setDirty=function(){this.dirty||(this.dirty=!0,this.value=r,d(this),k(this))},u.dispose=function(){var e=this;x(e).forEach(h),k(e),e.parents.forEach(function(t){t.setDirty(),w(t,e)}),function(e){s(0===e.parents.size),s(0===e.childValues.size),s(null===e.dirtyChildren),o.length<t.POOL_TARGET_SIZE&&o.push(e)}(e)};var b=[];function x(e){var t=b;return e.childValues.size>0&&(t=[],e.childValues.forEach(function(n,i){w(e,i),t.push(i)})),s(null===e.dirtyChildren),t}function w(e,t){t.parents.delete(e),e.childValues.delete(t),v(e,t)}function k(e){var t=e.unsubscribe;"function"==typeof t&&(e.unsubscribe=null,t())}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var i=function(){function e(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}return function(t,n,i){return n&&e(t.prototype,n),i&&e(t,i),t}}();function r(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}var a=function(){return"function"==typeof Symbol},o=function(e){return a()&&Boolean(Symbol[e])},s=function(e){return o(e)?Symbol[e]:"@@"+e};a()&&!o("observable")&&(Symbol.observable=Symbol("observable"));var l=s("iterator"),c=s("observable"),u=s("species");function h(e,t){var n=e[t];if(null!=n){if("function"!=typeof n)throw new TypeError(n+" is not a function");return n}}function d(e){var t=e.constructor;return void 0!==t&&null===(t=t[u])&&(t=void 0),void 0!==t?t:k}function f(e){return e instanceof k}function p(e){p.log?p.log(e):setTimeout(function(){throw e})}function g(e){Promise.resolve().then(function(){try{e()}catch(e){p(e)}})}function m(e){var t=e._cleanup;if(void 0!==t&&(e._cleanup=void 0,t))try{if("function"==typeof t)t();else{var n=h(t,"unsubscribe");n&&n.call(t)}}catch(e){p(e)}}function v(e){e._observer=void 0,e._queue=void 0,e._state="closed"}function y(e,t,n){e._state="running";var i=e._observer;try{var r=h(i,t);switch(t){case"next":r&&r.call(i,n);break;case"error":if(v(e),!r)throw n;r.call(i,n);break;case"complete":v(e),r&&r.call(i)}}catch(e){p(e)}"closed"===e._state?m(e):"running"===e._state&&(e._state="ready")}function b(e,t,n){if("closed"!==e._state){if("buffering"!==e._state)return"ready"!==e._state?(e._state="buffering",e._queue=[{type:t,value:n}],void g(function(){return function(e){var t=e._queue;if(t){e._queue=void 0,e._state="ready";for(var n=0;n<t.length&&(y(e,t[n].type,t[n].value),"closed"!==e._state);++n);}}(e)})):void y(e,t,n);e._queue.push({type:t,value:n})}}var x=function(){function e(t,n){r(this,e),this._cleanup=void 0,this._observer=t,this._queue=void 0,this._state="initializing";var i=new w(this);try{this._cleanup=n.call(void 0,i)}catch(e){i.error(e)}"initializing"===this._state&&(this._state="ready")}return i(e,[{key:"unsubscribe",value:function(){"closed"!==this._state&&(v(this),m(this))}},{key:"closed",get:function(){return"closed"===this._state}}]),e}(),w=function(){function e(t){r(this,e),this._subscription=t}return i(e,[{key:"next",value:function(e){b(this._subscription,"next",e)}},{key:"error",value:function(e){b(this._subscription,"error",e)}},{key:"complete",value:function(){b(this._subscription,"complete")}},{key:"closed",get:function(){return"closed"===this._subscription._state}}]),e}(),k=t.Observable=function(){function e(t){if(r(this,e),!(this instanceof e))throw new TypeError("Observable cannot be called as a function");if("function"!=typeof t)throw new TypeError("Observable initializer must be a function");this._subscriber=t}return i(e,[{key:"subscribe",value:function(e){return"object"==typeof e&&null!==e||(e={next:e,error:arguments[1],complete:arguments[2]}),new x(e,this._subscriber)}},{key:"forEach",value:function(e){var t=this;return new Promise(function(n,i){if("function"==typeof e)var r=t.subscribe({next:function(t){try{e(t,a)}catch(e){i(e),r.unsubscribe()}},error:i,complete:n});else i(new TypeError(e+" is not a function"));function a(){r.unsubscribe(),n()}})}},{key:"map",value:function(e){var t=this;if("function"!=typeof e)throw new TypeError(e+" is not a function");return new(d(this))(function(n){return t.subscribe({next:function(t){try{t=e(t)}catch(e){return n.error(e)}n.next(t)},error:function(e){n.error(e)},complete:function(){n.complete()}})})}},{key:"filter",value:function(e){var t=this;if("function"!=typeof e)throw new TypeError(e+" is not a function");return new(d(this))(function(n){return t.subscribe({next:function(t){try{if(!e(t))return}catch(e){return n.error(e)}n.next(t)},error:function(e){n.error(e)},complete:function(){n.complete()}})})}},{key:"reduce",value:function(e){var t=this;if("function"!=typeof e)throw new TypeError(e+" is not a function");var n=d(this),i=arguments.length>1,r=!1,a=arguments[1];return new n(function(n){return t.subscribe({next:function(t){var o=!r;if(r=!0,!o||i)try{a=e(a,t)}catch(e){return n.error(e)}else a=t},error:function(e){n.error(e)},complete:function(){if(!r&&!i)return n.error(new TypeError("Cannot reduce an empty sequence"));n.next(a),n.complete()}})})}},{key:"concat",value:function(){for(var e=this,t=arguments.length,n=Array(t),i=0;i<t;i++)n[i]=arguments[i];var r=d(this);return new r(function(t){var i=void 0,a=0;return function e(o){i=o.subscribe({next:function(e){t.next(e)},error:function(e){t.error(e)},complete:function(){a===n.length?(i=void 0,t.complete()):e(r.from(n[a++]))}})}(e),function(){i&&(i.unsubscribe(),i=void 0)}})}},{key:"flatMap",value:function(e){var t=this;if("function"!=typeof e)throw new TypeError(e+" is not a function");var n=d(this);return new n(function(i){var r=[],a=t.subscribe({next:function(t){if(e)try{t=e(t)}catch(e){return i.error(e)}var a=n.from(t).subscribe({next:function(e){i.next(e)},error:function(e){i.error(e)},complete:function(){var e=r.indexOf(a);e>=0&&r.splice(e,1),o()}});r.push(a)},error:function(e){i.error(e)},complete:function(){o()}});function o(){a.closed&&0===r.length&&i.complete()}return function(){r.forEach(function(e){return e.unsubscribe()}),a.unsubscribe()}})}},{key:c,value:function(){return this}}],[{key:"from",value:function(t){var n="function"==typeof this?this:e;if(null==t)throw new TypeError(t+" is not an object");var i=h(t,c);if(i){var r=i.call(t);if(Object(r)!==r)throw new TypeError(r+" is not an object");return f(r)&&r.constructor===n?r:new n(function(e){return r.subscribe(e)})}if(o("iterator")&&(i=h(t,l)))return new n(function(e){g(function(){if(!e.closed){var n=!0,r=!1,a=void 0;try{for(var o,s=i.call(t)[Symbol.iterator]();!(n=(o=s.next()).done);n=!0){var l=o.value;if(e.next(l),e.closed)return}}catch(e){r=!0,a=e}finally{try{!n&&s.return&&s.return()}finally{if(r)throw a}}e.complete()}})});if(Array.isArray(t))return new n(function(e){g(function(){if(!e.closed){for(var n=0;n<t.length;++n)if(e.next(t[n]),e.closed)return;e.complete()}})});throw new TypeError(t+" is not observable")}},{key:"of",value:function(){for(var t=arguments.length,n=Array(t),i=0;i<t;i++)n[i]=arguments[i];return new("function"==typeof this?this:e)(function(e){g(function(){if(!e.closed){for(var t=0;t<n.length;++t)if(e.next(n[t]),e.closed)return;e.complete()}})})}},{key:u,get:function(){return this}}]),e}();a()&&Object.defineProperty(k,Symbol("extensions"),{value:{symbol:c,hostReportError:p},configurable:!0})},function(e,t){e.exports=function(e){if(!e.webpackPolyfill){var t=Object.create(e);t.children||(t.children=[]),Object.defineProperty(t,"loaded",{enumerable:!0,get:function(){return t.l}}),Object.defineProperty(t,"id",{enumerable:!0,get:function(){return t.i}}),Object.defineProperty(t,"exports",{enumerable:!0}),t.webpackPolyfill=1}return t}},function(e,t,n){"use strict";e.exports=n(95)},function(e,t,n){"use strict";
/** @license React v16.8.6
 * react-is.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */Object.defineProperty(t,"__esModule",{value:!0});var i="function"==typeof Symbol&&Symbol.for,r=i?Symbol.for("react.element"):60103,a=i?Symbol.for("react.portal"):60106,o=i?Symbol.for("react.fragment"):60107,s=i?Symbol.for("react.strict_mode"):60108,l=i?Symbol.for("react.profiler"):60114,c=i?Symbol.for("react.provider"):60109,u=i?Symbol.for("react.context"):60110,h=i?Symbol.for("react.async_mode"):60111,d=i?Symbol.for("react.concurrent_mode"):60111,f=i?Symbol.for("react.forward_ref"):60112,p=i?Symbol.for("react.suspense"):60113,g=i?Symbol.for("react.memo"):60115,m=i?Symbol.for("react.lazy"):60116;function v(e){if("object"==typeof e&&null!==e){var t=e.$$typeof;switch(t){case r:switch(e=e.type){case h:case d:case o:case l:case s:case p:return e;default:switch(e=e&&e.$$typeof){case u:case f:case c:return e;default:return t}}case m:case g:case a:return t}}}function y(e){return v(e)===d}t.typeOf=v,t.AsyncMode=h,t.ConcurrentMode=d,t.ContextConsumer=u,t.ContextProvider=c,t.Element=r,t.ForwardRef=f,t.Fragment=o,t.Lazy=m,t.Memo=g,t.Portal=a,t.Profiler=l,t.StrictMode=s,t.Suspense=p,t.isValidElementType=function(e){return"string"==typeof e||"function"==typeof e||e===o||e===d||e===l||e===s||e===p||"object"==typeof e&&null!==e&&(e.$$typeof===m||e.$$typeof===g||e.$$typeof===c||e.$$typeof===u||e.$$typeof===f)},t.isAsyncMode=function(e){return y(e)||v(e)===h},t.isConcurrentMode=y,t.isContextConsumer=function(e){return v(e)===u},t.isContextProvider=function(e){return v(e)===c},t.isElement=function(e){return"object"==typeof e&&null!==e&&e.$$typeof===r},t.isForwardRef=function(e){return v(e)===f},t.isFragment=function(e){return v(e)===o},t.isLazy=function(e){return v(e)===m},t.isMemo=function(e){return v(e)===g},t.isPortal=function(e){return v(e)===a},t.isProfiler=function(e){return v(e)===l},t.isStrictMode=function(e){return v(e)===s},t.isSuspense=function(e){return v(e)===p}},function(e,t,n){"use strict";
/** @license React v16.8.6
 * react-dom-server.browser.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var i=n(36),r=n(0);function a(e){for(var t=arguments.length-1,n="https://reactjs.org/docs/error-decoder.html?invariant="+e,i=0;i<t;i++)n+="&args[]="+encodeURIComponent(arguments[i+1]);!function(e,t,n,i,r,a,o,s){if(!e){if(e=void 0,void 0===t)e=Error("Minified exception occurred; use the non-minified dev environment for the full error message and additional helpful warnings.");else{var l=[n,i,r,a,o,s],c=0;(e=Error(t.replace(/%s/g,function(){return l[c++]}))).name="Invariant Violation"}throw e.framesToPop=1,e}}(!1,"Minified React error #"+e+"; visit %s for the full message or use the non-minified dev environment for full errors and additional helpful warnings. ",n)}var o="function"==typeof Symbol&&Symbol.for,s=o?Symbol.for("react.portal"):60106,l=o?Symbol.for("react.fragment"):60107,c=o?Symbol.for("react.strict_mode"):60108,u=o?Symbol.for("react.profiler"):60114,h=o?Symbol.for("react.provider"):60109,d=o?Symbol.for("react.context"):60110,f=o?Symbol.for("react.concurrent_mode"):60111,p=o?Symbol.for("react.forward_ref"):60112,g=o?Symbol.for("react.suspense"):60113,m=o?Symbol.for("react.memo"):60115,v=o?Symbol.for("react.lazy"):60116;function y(e){if(null==e)return null;if("function"==typeof e)return e.displayName||e.name||null;if("string"==typeof e)return e;switch(e){case f:return"ConcurrentMode";case l:return"Fragment";case s:return"Portal";case u:return"Profiler";case c:return"StrictMode";case g:return"Suspense"}if("object"==typeof e)switch(e.$$typeof){case d:return"Context.Consumer";case h:return"Context.Provider";case p:var t=e.render;return t=t.displayName||t.name||"",e.displayName||(""!==t?"ForwardRef("+t+")":"ForwardRef");case m:return y(e.type);case v:if(e=1===e._status?e._result:null)return y(e)}return null}var b=r.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED;b.hasOwnProperty("ReactCurrentDispatcher")||(b.ReactCurrentDispatcher={current:null});var x={};function w(e,t){for(var n=0|e._threadCount;n<=t;n++)e[n]=e._currentValue2,e._threadCount=n+1}for(var k=new Uint16Array(16),S=0;15>S;S++)k[S]=S+1;k[15]=0;var E=/^[:A-Z_a-z\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u02FF\u0370-\u037D\u037F-\u1FFF\u200C-\u200D\u2070-\u218F\u2C00-\u2FEF\u3001-\uD7FF\uF900-\uFDCF\uFDF0-\uFFFD][:A-Z_a-z\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u02FF\u0370-\u037D\u037F-\u1FFF\u200C-\u200D\u2070-\u218F\u2C00-\u2FEF\u3001-\uD7FF\uF900-\uFDCF\uFDF0-\uFFFD\-.0-9\u00B7\u0300-\u036F\u203F-\u2040]*$/,C=Object.prototype.hasOwnProperty,T={},A={};function _(e){return!!C.call(A,e)||!C.call(T,e)&&(E.test(e)?A[e]=!0:(T[e]=!0,!1))}function O(e,t,n,i){if(null==t||function(e,t,n,i){if(null!==n&&0===n.type)return!1;switch(typeof t){case"function":case"symbol":return!0;case"boolean":return!i&&(null!==n?!n.acceptsBooleans:"data-"!==(e=e.toLowerCase().slice(0,5))&&"aria-"!==e);default:return!1}}(e,t,n,i))return!0;if(i)return!1;if(null!==n)switch(n.type){case 3:return!t;case 4:return!1===t;case 5:return isNaN(t);case 6:return isNaN(t)||1>t}return!1}function P(e,t,n,i,r){this.acceptsBooleans=2===t||3===t||4===t,this.attributeName=i,this.attributeNamespace=r,this.mustUseProperty=n,this.propertyName=e,this.type=t}var M={};"children dangerouslySetInnerHTML defaultValue defaultChecked innerHTML suppressContentEditableWarning suppressHydrationWarning style".split(" ").forEach(function(e){M[e]=new P(e,0,!1,e,null)}),[["acceptCharset","accept-charset"],["className","class"],["htmlFor","for"],["httpEquiv","http-equiv"]].forEach(function(e){var t=e[0];M[t]=new P(t,1,!1,e[1],null)}),["contentEditable","draggable","spellCheck","value"].forEach(function(e){M[e]=new P(e,2,!1,e.toLowerCase(),null)}),["autoReverse","externalResourcesRequired","focusable","preserveAlpha"].forEach(function(e){M[e]=new P(e,2,!1,e,null)}),"allowFullScreen async autoFocus autoPlay controls default defer disabled formNoValidate hidden loop noModule noValidate open playsInline readOnly required reversed scoped seamless itemScope".split(" ").forEach(function(e){M[e]=new P(e,3,!1,e.toLowerCase(),null)}),["checked","multiple","muted","selected"].forEach(function(e){M[e]=new P(e,3,!0,e,null)}),["capture","download"].forEach(function(e){M[e]=new P(e,4,!1,e,null)}),["cols","rows","size","span"].forEach(function(e){M[e]=new P(e,6,!1,e,null)}),["rowSpan","start"].forEach(function(e){M[e]=new P(e,5,!1,e.toLowerCase(),null)});var I=/[\-:]([a-z])/g;function D(e){return e[1].toUpperCase()}"accent-height alignment-baseline arabic-form baseline-shift cap-height clip-path clip-rule color-interpolation color-interpolation-filters color-profile color-rendering dominant-baseline enable-background fill-opacity fill-rule flood-color flood-opacity font-family font-size font-size-adjust font-stretch font-style font-variant font-weight glyph-name glyph-orientation-horizontal glyph-orientation-vertical horiz-adv-x horiz-origin-x image-rendering letter-spacing lighting-color marker-end marker-mid marker-start overline-position overline-thickness paint-order panose-1 pointer-events rendering-intent shape-rendering stop-color stop-opacity strikethrough-position strikethrough-thickness stroke-dasharray stroke-dashoffset stroke-linecap stroke-linejoin stroke-miterlimit stroke-opacity stroke-width text-anchor text-decoration text-rendering underline-position underline-thickness unicode-bidi unicode-range units-per-em v-alphabetic v-hanging v-ideographic v-mathematical vector-effect vert-adv-y vert-origin-x vert-origin-y word-spacing writing-mode xmlns:xlink x-height".split(" ").forEach(function(e){var t=e.replace(I,D);M[t]=new P(t,1,!1,e,null)}),"xlink:actuate xlink:arcrole xlink:href xlink:role xlink:show xlink:title xlink:type".split(" ").forEach(function(e){var t=e.replace(I,D);M[t]=new P(t,1,!1,e,"http://www.w3.org/1999/xlink")}),["xml:base","xml:lang","xml:space"].forEach(function(e){var t=e.replace(I,D);M[t]=new P(t,1,!1,e,"http://www.w3.org/XML/1998/namespace")}),["tabIndex","crossOrigin"].forEach(function(e){M[e]=new P(e,1,!1,e.toLowerCase(),null)});var N=/["'&<>]/;function L(e){if("boolean"==typeof e||"number"==typeof e)return""+e;e=""+e;var t=N.exec(e);if(t){var n,i="",r=0;for(n=t.index;n<e.length;n++){switch(e.charCodeAt(n)){case 34:t="&quot;";break;case 38:t="&amp;";break;case 39:t="&#x27;";break;case 60:t="&lt;";break;case 62:t="&gt;";break;default:continue}r!==n&&(i+=e.substring(r,n)),r=n+1,i+=t}e=r!==n?i+e.substring(r,n):i}return e}var R=null,F=null,j=null,z=!1,Y=!1,H=null,W=0;function X(){return null===R&&a("321"),R}function V(){return 0<W&&a("312"),{memoizedState:null,queue:null,next:null}}function B(){return null===j?null===F?(z=!1,F=j=V()):(z=!0,j=F):null===j.next?(z=!1,j=j.next=V()):(z=!0,j=j.next),j}function q(e,t,n,i){for(;Y;)Y=!1,W+=1,j=null,n=e(t,i);return F=R=null,W=0,j=H=null,n}function U(e,t){return"function"==typeof t?t(e):t}function G(e,t,n){if(R=X(),j=B(),z){var i=j.queue;if(t=i.dispatch,null!==H&&void 0!==(n=H.get(i))){H.delete(i),i=j.memoizedState;do{i=e(i,n.action),n=n.next}while(null!==n);return j.memoizedState=i,[i,t]}return[j.memoizedState,t]}return e=e===U?"function"==typeof t?t():t:void 0!==n?n(t):t,j.memoizedState=e,e=(e=j.queue={last:null,dispatch:null}).dispatch=function(e,t,n){if(25>W||a("301"),e===R)if(Y=!0,e={action:n,next:null},null===H&&(H=new Map),void 0===(n=H.get(t)))H.set(t,e);else{for(t=n;null!==t.next;)t=t.next;t.next=e}}.bind(null,R,e),[j.memoizedState,e]}function Q(){}var $=0,Z={readContext:function(e){var t=$;return w(e,t),e[t]},useContext:function(e){X();var t=$;return w(e,t),e[t]},useMemo:function(e,t){if(R=X(),t=void 0===t?null:t,null!==(j=B())){var n=j.memoizedState;if(null!==n&&null!==t){e:{var i=n[1];if(null===i)i=!1;else{for(var r=0;r<i.length&&r<t.length;r++){var a=t[r],o=i[r];if((a!==o||0===a&&1/a!=1/o)&&(a==a||o==o)){i=!1;break e}}i=!0}}if(i)return n[0]}}return e=e(),j.memoizedState=[e,t],e},useReducer:G,useRef:function(e){R=X();var t=(j=B()).memoizedState;return null===t?(e={current:e},j.memoizedState=e):t},useState:function(e){return G(U,e)},useLayoutEffect:function(){},useCallback:function(e){return e},useImperativeHandle:Q,useEffect:Q,useDebugValue:Q},K={html:"http://www.w3.org/1999/xhtml",mathml:"http://www.w3.org/1998/Math/MathML",svg:"http://www.w3.org/2000/svg"};function J(e){switch(e){case"svg":return"http://www.w3.org/2000/svg";case"math":return"http://www.w3.org/1998/Math/MathML";default:return"http://www.w3.org/1999/xhtml"}}var ee={area:!0,base:!0,br:!0,col:!0,embed:!0,hr:!0,img:!0,input:!0,keygen:!0,link:!0,meta:!0,param:!0,source:!0,track:!0,wbr:!0},te=i({menuitem:!0},ee),ne={animationIterationCount:!0,borderImageOutset:!0,borderImageSlice:!0,borderImageWidth:!0,boxFlex:!0,boxFlexGroup:!0,boxOrdinalGroup:!0,columnCount:!0,columns:!0,flex:!0,flexGrow:!0,flexPositive:!0,flexShrink:!0,flexNegative:!0,flexOrder:!0,gridArea:!0,gridRow:!0,gridRowEnd:!0,gridRowSpan:!0,gridRowStart:!0,gridColumn:!0,gridColumnEnd:!0,gridColumnSpan:!0,gridColumnStart:!0,fontWeight:!0,lineClamp:!0,lineHeight:!0,opacity:!0,order:!0,orphans:!0,tabSize:!0,widows:!0,zIndex:!0,zoom:!0,fillOpacity:!0,floodOpacity:!0,stopOpacity:!0,strokeDasharray:!0,strokeDashoffset:!0,strokeMiterlimit:!0,strokeOpacity:!0,strokeWidth:!0},ie=["Webkit","ms","Moz","O"];Object.keys(ne).forEach(function(e){ie.forEach(function(t){t=t+e.charAt(0).toUpperCase()+e.substring(1),ne[t]=ne[e]})});var re=/([A-Z])/g,ae=/^ms-/,oe=r.Children.toArray,se=b.ReactCurrentDispatcher,le={listing:!0,pre:!0,textarea:!0},ce=/^[a-zA-Z][a-zA-Z:_\.\-\d]*$/,ue={},he={};var de=Object.prototype.hasOwnProperty,fe={children:null,dangerouslySetInnerHTML:null,suppressContentEditableWarning:null,suppressHydrationWarning:null};function pe(e,t){void 0===e&&a("152",y(t)||"Component")}function ge(e,t,n){function o(r,o){var s=function(e,t,n){var i=e.contextType;if("object"==typeof i&&null!==i)return w(i,n),i[n];if(e=e.contextTypes){for(var r in n={},e)n[r]=t[r];t=n}else t=x;return t}(o,t,n),l=[],c=!1,u={isMounted:function(){return!1},enqueueForceUpdate:function(){if(null===l)return null},enqueueReplaceState:function(e,t){c=!0,l=[t]},enqueueSetState:function(e,t){if(null===l)return null;l.push(t)}},h=void 0;if(o.prototype&&o.prototype.isReactComponent){if(h=new o(r.props,s,u),"function"==typeof o.getDerivedStateFromProps){var d=o.getDerivedStateFromProps.call(null,r.props,h.state);null!=d&&(h.state=i({},h.state,d))}}else if(R={},h=o(r.props,s,u),null==(h=q(o,r.props,h,s))||null==h.render)return void pe(e=h,o);if(h.props=r.props,h.context=s,h.updater=u,void 0===(u=h.state)&&(h.state=u=null),"function"==typeof h.UNSAFE_componentWillMount||"function"==typeof h.componentWillMount)if("function"==typeof h.componentWillMount&&"function"!=typeof o.getDerivedStateFromProps&&h.componentWillMount(),"function"==typeof h.UNSAFE_componentWillMount&&"function"!=typeof o.getDerivedStateFromProps&&h.UNSAFE_componentWillMount(),l.length){u=l;var f=c;if(l=null,c=!1,f&&1===u.length)h.state=u[0];else{d=f?u[0]:h.state;var p=!0;for(f=f?1:0;f<u.length;f++){var g=u[f];null!=(g="function"==typeof g?g.call(h,d,r.props,s):g)&&(p?(p=!1,d=i({},d,g)):i(d,g))}h.state=d}}else l=null;if(pe(e=h.render(),o),r=void 0,"function"==typeof h.getChildContext&&"object"==typeof(s=o.childContextTypes))for(var m in r=h.getChildContext())m in s||a("108",y(o)||"Unknown",m);r&&(t=i({},t,r))}for(;r.isValidElement(e);){var s=e,l=s.type;if("function"!=typeof l)break;o(s,l)}return{child:e,context:t}}var me=function(){function e(t,n){if(!(this instanceof e))throw new TypeError("Cannot call a class as a function");r.isValidElement(t)?t.type!==l?t=[t]:(t=t.props.children,t=r.isValidElement(t)?[t]:oe(t)):t=oe(t),t={type:null,domNamespace:K.html,children:t,childIndex:0,context:x,footer:""};var i=k[0];if(0===i){var o=k,s=2*(i=o.length);65536>=s||a("304");var c=new Uint16Array(s);for(c.set(o),(k=c)[0]=i+1,o=i;o<s-1;o++)k[o]=o+1;k[s-1]=0}else k[0]=k[i];this.threadID=i,this.stack=[t],this.exhausted=!1,this.currentSelectValue=null,this.previousWasTextNode=!1,this.makeStaticMarkup=n,this.suspenseDepth=0,this.contextIndex=-1,this.contextStack=[],this.contextValueStack=[]}return e.prototype.destroy=function(){if(!this.exhausted){this.exhausted=!0,this.clearProviders();var e=this.threadID;k[e]=k[0],k[0]=e}},e.prototype.pushProvider=function(e){var t=++this.contextIndex,n=e.type._context,i=this.threadID;w(n,i);var r=n[i];this.contextStack[t]=n,this.contextValueStack[t]=r,n[i]=e.props.value},e.prototype.popProvider=function(){var e=this.contextIndex,t=this.contextStack[e],n=this.contextValueStack[e];this.contextStack[e]=null,this.contextValueStack[e]=null,this.contextIndex--,t[this.threadID]=n},e.prototype.clearProviders=function(){for(var e=this.contextIndex;0<=e;e--)this.contextStack[e][this.threadID]=this.contextValueStack[e]},e.prototype.read=function(e){if(this.exhausted)return null;var t=$;$=this.threadID;var n=se.current;se.current=Z;try{for(var i=[""],r=!1;i[0].length<e;){if(0===this.stack.length){this.exhausted=!0;var o=this.threadID;k[o]=k[0],k[0]=o;break}var s=this.stack[this.stack.length-1];if(r||s.childIndex>=s.children.length){var l=s.footer;if(""!==l&&(this.previousWasTextNode=!1),this.stack.pop(),"select"===s.type)this.currentSelectValue=null;else if(null!=s.type&&null!=s.type.type&&s.type.type.$$typeof===h)this.popProvider(s.type);else if(s.type===g){this.suspenseDepth--;var c=i.pop();if(r){r=!1;var u=s.fallbackFrame;u||a("303"),this.stack.push(u);continue}i[this.suspenseDepth]+=c}i[this.suspenseDepth]+=l}else{var d=s.children[s.childIndex++],f="";try{f+=this.render(d,s.context,s.domNamespace)}catch(e){throw e}i.length<=this.suspenseDepth&&i.push(""),i[this.suspenseDepth]+=f}}return i[0]}finally{se.current=n,$=t}},e.prototype.render=function(e,t,n){if("string"==typeof e||"number"==typeof e)return""===(n=""+e)?"":this.makeStaticMarkup?L(n):this.previousWasTextNode?"\x3c!-- --\x3e"+L(n):(this.previousWasTextNode=!0,L(n));if(e=(t=ge(e,t,this.threadID)).child,t=t.context,null===e||!1===e)return"";if(!r.isValidElement(e)){if(null!=e&&null!=e.$$typeof){var o=e.$$typeof;o===s&&a("257"),a("258",o.toString())}return e=oe(e),this.stack.push({type:null,domNamespace:n,children:e,childIndex:0,context:t,footer:""}),""}if("string"==typeof(o=e.type))return this.renderDOM(e,t,n);switch(o){case c:case f:case u:case l:return e=oe(e.props.children),this.stack.push({type:null,domNamespace:n,children:e,childIndex:0,context:t,footer:""}),"";case g:a("294")}if("object"==typeof o&&null!==o)switch(o.$$typeof){case p:R={};var y=o.render(e.props,e.ref);return y=q(o.render,e.props,y,e.ref),y=oe(y),this.stack.push({type:null,domNamespace:n,children:y,childIndex:0,context:t,footer:""}),"";case m:return e=[r.createElement(o.type,i({ref:e.ref},e.props))],this.stack.push({type:null,domNamespace:n,children:e,childIndex:0,context:t,footer:""}),"";case h:return n={type:e,domNamespace:n,children:o=oe(e.props.children),childIndex:0,context:t,footer:""},this.pushProvider(e),this.stack.push(n),"";case d:o=e.type,y=e.props;var b=this.threadID;return w(o,b),o=oe(y.children(o[b])),this.stack.push({type:e,domNamespace:n,children:o,childIndex:0,context:t,footer:""}),"";case v:a("295")}a("130",null==o?o:typeof o,"")},e.prototype.renderDOM=function(e,t,n){var o=e.type.toLowerCase();n===K.html&&J(o),ue.hasOwnProperty(o)||(ce.test(o)||a("65",o),ue[o]=!0);var s=e.props;if("input"===o)s=i({type:void 0},s,{defaultChecked:void 0,defaultValue:void 0,value:null!=s.value?s.value:s.defaultValue,checked:null!=s.checked?s.checked:s.defaultChecked});else if("textarea"===o){var l=s.value;if(null==l){l=s.defaultValue;var c=s.children;null!=c&&(null!=l&&a("92"),Array.isArray(c)&&(1>=c.length||a("93"),c=c[0]),l=""+c),null==l&&(l="")}s=i({},s,{value:void 0,children:""+l})}else if("select"===o)this.currentSelectValue=null!=s.value?s.value:s.defaultValue,s=i({},s,{value:void 0});else if("option"===o){c=this.currentSelectValue;var u=function(e){if(null==e)return e;var t="";return r.Children.forEach(e,function(e){null!=e&&(t+=e)}),t}(s.children);if(null!=c){var h=null!=s.value?s.value+"":u;if(l=!1,Array.isArray(c)){for(var d=0;d<c.length;d++)if(""+c[d]===h){l=!0;break}}else l=""+c===h;s=i({selected:void 0,children:void 0},s,{selected:l,children:u})}}for(x in(l=s)&&(te[o]&&(null!=l.children||null!=l.dangerouslySetInnerHTML)&&a("137",o,""),null!=l.dangerouslySetInnerHTML&&(null!=l.children&&a("60"),"object"==typeof l.dangerouslySetInnerHTML&&"__html"in l.dangerouslySetInnerHTML||a("61")),null!=l.style&&"object"!=typeof l.style&&a("62","")),l=s,c=this.makeStaticMarkup,u=1===this.stack.length,h="<"+e.type,l)if(de.call(l,x)){var f=l[x];if(null!=f){if("style"===x){d=void 0;var p="",g="";for(d in f)if(f.hasOwnProperty(d)){var m=0===d.indexOf("--"),v=f[d];if(null!=v){var y=d;if(he.hasOwnProperty(y))y=he[y];else{var b=y.replace(re,"-$1").toLowerCase().replace(ae,"-ms-");y=he[y]=b}p+=g+y+":",g=d,p+=m=null==v||"boolean"==typeof v||""===v?"":m||"number"!=typeof v||0===v||ne.hasOwnProperty(g)&&ne[g]?(""+v).trim():v+"px",g=";"}}f=p||null}d=null;e:if(m=o,v=l,-1===m.indexOf("-"))m="string"==typeof v.is;else switch(m){case"annotation-xml":case"color-profile":case"font-face":case"font-face-src":case"font-face-uri":case"font-face-format":case"font-face-name":case"missing-glyph":m=!1;break e;default:m=!0}m?fe.hasOwnProperty(x)||(d=_(d=x)&&null!=f?d+'="'+L(f)+'"':""):(m=x,d=f,f=M.hasOwnProperty(m)?M[m]:null,(v="style"!==m)&&(v=null!==f?0===f.type:2<m.length&&("o"===m[0]||"O"===m[0])&&("n"===m[1]||"N"===m[1])),v||O(m,d,f,!1)?d="":null!==f?(m=f.attributeName,d=3===(f=f.type)||4===f&&!0===d?m+'=""':m+'="'+L(d)+'"'):d=_(m)?m+'="'+L(d)+'"':""),d&&(h+=" "+d)}}c||u&&(h+=' data-reactroot=""');var x=h;l="",ee.hasOwnProperty(o)?x+="/>":(x+=">",l="</"+e.type+">");e:{if(null!=(c=s.dangerouslySetInnerHTML)){if(null!=c.__html){c=c.__html;break e}}else if("string"==typeof(c=s.children)||"number"==typeof c){c=L(c);break e}c=null}return null!=c?(s=[],le[o]&&"\n"===c.charAt(0)&&(x+="\n"),x+=c):s=oe(s.children),e=e.type,n=null==n||"http://www.w3.org/1999/xhtml"===n?J(e):"http://www.w3.org/2000/svg"===n&&"foreignObject"===e?"http://www.w3.org/1999/xhtml":n,this.stack.push({domNamespace:n,type:o,children:s,childIndex:0,context:t,footer:l}),this.previousWasTextNode=!1,x},e}(),ve={renderToString:function(e){e=new me(e,!1);try{return e.read(1/0)}finally{e.destroy()}},renderToStaticMarkup:function(e){e=new me(e,!0);try{return e.read(1/0)}finally{e.destroy()}},renderToNodeStream:function(){a("207")},renderToStaticNodeStream:function(){a("208")},version:"16.8.6"},ye={default:ve},be=ye&&ve||ye;e.exports=be.default||be},function(e,t,n){"use strict";var i=n(98),r=n(99),a=n(38),o=n(39);e.exports=n(101)(Array,"Array",function(e,t){this._t=o(e),this._i=0,this._k=t},function(){var e=this._t,t=this._k,n=this._i++;return!e||n>=e.length?(this._t=void 0,r(1)):r(0,"keys"==t?n:"values"==t?e[n]:[n,e[n]])},"values"),a.Arguments=a.Array,i("keys"),i("values"),i("entries")},function(e,t,n){var i=n(10)("unscopables"),r=Array.prototype;null==r[i]&&n(12)(r,i,{}),e.exports=function(e){r[i][e]=!0}},function(e,t){e.exports=function(e,t){return{value:t,done:!!e}}},function(e,t,n){var i=n(48);e.exports=Object("z").propertyIsEnumerable(0)?Object:function(e){return"String"==i(e)?e.split(""):Object(e)}},function(e,t,n){"use strict";var i=n(49),r=n(52),a=n(23),o=n(12),s=n(38),l=n(102),c=n(60),u=n(109),h=n(10)("iterator"),d=!([].keys&&"next"in[].keys()),f=function(){return this};e.exports=function(e,t,n,p,g,m,v){l(n,t,p);var y,b,x,w=function(e){if(!d&&e in C)return C[e];switch(e){case"keys":case"values":return function(){return new n(this,e)}}return function(){return new n(this,e)}},k=t+" Iterator",S="values"==g,E=!1,C=e.prototype,T=C[h]||C["@@iterator"]||g&&C[g],A=T||w(g),_=g?S?w("entries"):A:void 0,O="Array"==t&&C.entries||T;if(O&&(x=u(O.call(new e)))!==Object.prototype&&x.next&&(c(x,k,!0),i||"function"==typeof x[h]||o(x,h,f)),S&&T&&"values"!==T.name&&(E=!0,A=function(){return T.call(this)}),i&&!v||!d&&!E&&C[h]||o(C,h,A),s[t]=A,s[k]=f,g)if(y={values:S?A:w("values"),keys:m?A:w("keys"),entries:_},v)for(b in y)b in C||a(C,b,y[b]);else r(r.P+r.F*(d||E),t,y);return y}},function(e,t,n){"use strict";var i=n(103),r=n(54),a=n(60),o={};n(12)(o,n(10)("iterator"),function(){return this}),e.exports=function(e,t,n){e.prototype=i(o,{next:r(1,n)}),a(e,t+" Iterator")}},function(e,t,n){var i=n(13),r=n(104),a=n(59),o=n(40)("IE_PROTO"),s=function(){},l=function(){var e,t=n(53)("iframe"),i=a.length;for(t.style.display="none",n(108).appendChild(t),t.src="javascript:",(e=t.contentWindow.document).open(),e.write("<script>document.F=Object<\/script>"),e.close(),l=e.F;i--;)delete l.prototype[a[i]];return l()};e.exports=Object.create||function(e,t){var n;return null!==e?(s.prototype=i(e),n=new s,s.prototype=null,n[o]=e):n=l(),void 0===t?n:r(n,t)}},function(e,t,n){var i=n(33),r=n(13),a=n(58);e.exports=n(22)?Object.defineProperties:function(e,t){r(e);for(var n,o=a(t),s=o.length,l=0;s>l;)i.f(e,n=o[l++],t[n]);return e}},function(e,t,n){var i=n(24),r=n(39),a=n(106)(!1),o=n(40)("IE_PROTO");e.exports=function(e,t){var n,s=r(e),l=0,c=[];for(n in s)n!=o&&i(s,n)&&c.push(n);for(;t.length>l;)i(s,n=t[l++])&&(~a(c,n)||c.push(n));return c}},function(e,t,n){var i=n(39),r=n(29),a=n(107);e.exports=function(e){return function(t,n,o){var s,l=i(t),c=r(l.length),u=a(o,c);if(e&&n!=n){for(;c>u;)if((s=l[u++])!=s)return!0}else for(;c>u;u++)if((e||u in l)&&l[u]===n)return e||u||0;return!e&&-1}}},function(e,t,n){var i=n(21),r=Math.max,a=Math.min;e.exports=function(e,t){return(e=i(e))<0?r(e+t,0):a(e,t)}},function(e,t,n){var i=n(11).document;e.exports=i&&i.documentElement},function(e,t,n){var i=n(24),r=n(45),a=n(40)("IE_PROTO"),o=Object.prototype;e.exports=Object.getPrototypeOf||function(e){return e=r(e),i(e,a)?e[a]:"function"==typeof e.constructor&&e instanceof e.constructor?e.constructor.prototype:e instanceof Object?o:null}},function(e,t,n){"use strict";var i=n(13),r=n(29),a=n(46),o=n(47);n(50)("match",1,function(e,t,n,s){return[function(n){var i=e(this),r=null==n?void 0:n[t];return void 0!==r?r.call(n,i):new RegExp(n)[t](String(i))},function(e){var t=s(n,e,this);if(t.done)return t.value;var l=i(e),c=String(this);if(!l.global)return o(l,c);var u=l.unicode;l.lastIndex=0;for(var h,d=[],f=0;null!==(h=o(l,c));){var p=String(h[0]);d[f]=p,""===p&&(l.lastIndex=a(c,r(l.lastIndex),u)),f++}return 0===f?null:d}]})},function(e,t,n){"use strict";n.r(t);n(44);var i=n(8),r=n.n(i),a=(n(25),n(76),n(77),n(78),n(6)),o=n.n(a);n(79);function s(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function l(e,t){for(var n=0;n<t.length;n++){var i=t[n];i.enumerable=i.enumerable||!1,i.configurable=!0,"value"in i&&(i.writable=!0),Object.defineProperty(e,i.key,i)}}function c(e,t,n){return t&&l(e.prototype,t),n&&l(e,n),e}function u(e){return(u="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function h(e){return(h="function"==typeof Symbol&&"symbol"===u(Symbol.iterator)?function(e){return u(e)}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":u(e)})(e)}function d(e,t){return!t||"object"!==h(t)&&"function"!=typeof t?function(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}(e):t}function f(e){return(f=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}function p(e,t){return(p=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function g(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),t&&p(e,t)}function m(e,t){return t||(t=e.slice(0)),Object.freeze(Object.defineProperties(e,{raw:{value:Object.freeze(t)}}))}var v=n(1),y=n(2),b=n(63),x=n.n(b).a,w=n(41),k=n(5),S=n(27);function E(e){return Object(k.b)(e,{leave:C})}var C={Name:function(e){return e.value},Variable:function(e){return"$"+e.name},Document:function(e){return A(e.definitions,"\n\n")+"\n"},OperationDefinition:function(e){var t=e.operation,n=e.name,i=O("(",A(e.variableDefinitions,", "),")"),r=A(e.directives," "),a=e.selectionSet;return n||r||i||"query"!==t?A([t,A([n,i]),r,a]," "):a},VariableDefinition:function(e){var t=e.variable,n=e.type,i=e.defaultValue,r=e.directives;return t+": "+n+O(" = ",i)+O(" ",A(r," "))},SelectionSet:function(e){return _(e.selections)},Field:function(e){var t=e.alias,n=e.name,i=e.arguments,r=e.directives,a=e.selectionSet;return A([O("",t,": ")+n+O("(",A(i,", "),")"),A(r," "),a]," ")},Argument:function(e){return e.name+": "+e.value},FragmentSpread:function(e){return"..."+e.name+O(" ",A(e.directives," "))},InlineFragment:function(e){var t=e.typeCondition,n=e.directives,i=e.selectionSet;return A(["...",O("on ",t),A(n," "),i]," ")},FragmentDefinition:function(e){var t=e.name,n=e.typeCondition,i=e.variableDefinitions,r=e.directives,a=e.selectionSet;return"fragment ".concat(t).concat(O("(",A(i,", "),")")," ")+"on ".concat(n," ").concat(O("",A(r," ")," "))+a},IntValue:function(e){return e.value},FloatValue:function(e){return e.value},StringValue:function(e,t){var n=e.value;return e.block?Object(S.b)(n,"description"===t?"":"  "):JSON.stringify(n)},BooleanValue:function(e){return e.value?"true":"false"},NullValue:function(){return"null"},EnumValue:function(e){return e.value},ListValue:function(e){return"["+A(e.values,", ")+"]"},ObjectValue:function(e){return"{"+A(e.fields,", ")+"}"},ObjectField:function(e){return e.name+": "+e.value},Directive:function(e){return"@"+e.name+O("(",A(e.arguments,", "),")")},NamedType:function(e){return e.name},ListType:function(e){return"["+e.type+"]"},NonNullType:function(e){return e.type+"!"},SchemaDefinition:function(e){var t=e.directives,n=e.operationTypes;return A(["schema",A(t," "),_(n)]," ")},OperationTypeDefinition:function(e){return e.operation+": "+e.type},ScalarTypeDefinition:T(function(e){return A(["scalar",e.name,A(e.directives," ")]," ")}),ObjectTypeDefinition:T(function(e){var t=e.name,n=e.interfaces,i=e.directives,r=e.fields;return A(["type",t,O("implements ",A(n," & ")),A(i," "),_(r)]," ")}),FieldDefinition:T(function(e){var t=e.name,n=e.arguments,i=e.type,r=e.directives;return t+(I(n)?O("(\n",P(A(n,"\n")),"\n)"):O("(",A(n,", "),")"))+": "+i+O(" ",A(r," "))}),InputValueDefinition:T(function(e){var t=e.name,n=e.type,i=e.defaultValue,r=e.directives;return A([t+": "+n,O("= ",i),A(r," ")]," ")}),InterfaceTypeDefinition:T(function(e){var t=e.name,n=e.directives,i=e.fields;return A(["interface",t,A(n," "),_(i)]," ")}),UnionTypeDefinition:T(function(e){var t=e.name,n=e.directives,i=e.types;return A(["union",t,A(n," "),i&&0!==i.length?"= "+A(i," | "):""]," ")}),EnumTypeDefinition:T(function(e){var t=e.name,n=e.directives,i=e.values;return A(["enum",t,A(n," "),_(i)]," ")}),EnumValueDefinition:T(function(e){return A([e.name,A(e.directives," ")]," ")}),InputObjectTypeDefinition:T(function(e){var t=e.name,n=e.directives,i=e.fields;return A(["input",t,A(n," "),_(i)]," ")}),DirectiveDefinition:T(function(e){var t=e.name,n=e.arguments,i=e.locations;return"directive @"+t+(I(n)?O("(\n",P(A(n,"\n")),"\n)"):O("(",A(n,", "),")"))+" on "+A(i," | ")}),SchemaExtension:function(e){var t=e.directives,n=e.operationTypes;return A(["extend schema",A(t," "),_(n)]," ")},ScalarTypeExtension:function(e){return A(["extend scalar",e.name,A(e.directives," ")]," ")},ObjectTypeExtension:function(e){var t=e.name,n=e.interfaces,i=e.directives,r=e.fields;return A(["extend type",t,O("implements ",A(n," & ")),A(i," "),_(r)]," ")},InterfaceTypeExtension:function(e){var t=e.name,n=e.directives,i=e.fields;return A(["extend interface",t,A(n," "),_(i)]," ")},UnionTypeExtension:function(e){var t=e.name,n=e.directives,i=e.types;return A(["extend union",t,A(n," "),i&&0!==i.length?"= "+A(i," | "):""]," ")},EnumTypeExtension:function(e){var t=e.name,n=e.directives,i=e.values;return A(["extend enum",t,A(n," "),_(i)]," ")},InputObjectTypeExtension:function(e){var t=e.name,n=e.directives,i=e.fields;return A(["extend input",t,A(n," "),_(i)]," ")}};function T(e){return function(t){return A([t.description,e(t)],"\n")}}function A(e,t){return e?e.filter(function(e){return e}).join(t||""):""}function _(e){return e&&0!==e.length?"{\n"+P(A(e,"\n"))+"\n}":""}function O(e,t,n){return t?e+t+(n||""):""}function P(e){return e&&"  "+e.replace(/\n/g,"\n  ")}function M(e){return-1!==e.indexOf("\n")}function I(e){return e&&e.some(M)}!function(e){function t(t,n){var i=e.call(this,t)||this;return i.link=n,i}Object(v.c)(t,e)}(Error);function D(e){return e.request.length<=1}function N(e){return new x(function(t){t.error(e)})}function L(e,t){var n=Object(v.a)({},e);return Object.defineProperty(t,"setContext",{enumerable:!1,value:function(e){n="function"==typeof e?Object(v.a)({},n,e(n)):Object(v.a)({},n,e)}}),Object.defineProperty(t,"getContext",{enumerable:!1,value:function(){return Object(v.a)({},n)}}),Object.defineProperty(t,"toKey",{enumerable:!1,value:function(){return function(e){return E(e.query)+"|"+JSON.stringify(e.variables)+"|"+e.operationName}(t)}}),t}function R(e,t){return t?t(e):x.of()}function F(e){return"function"==typeof e?new H(e):e}function j(){return new H(function(){return x.of()})}function z(e){return 0===e.length?j():e.map(F).reduce(function(e,t){return e.concat(t)})}function Y(e,t,n){var i=F(t),r=F(n||new H(R));return D(i)&&D(r)?new H(function(t){return e(t)?i.request(t)||x.of():r.request(t)||x.of()}):new H(function(t,n){return e(t)?i.request(t,n)||x.of():r.request(t,n)||x.of()})}var H=function(){function e(e){e&&(this.request=e)}return e.prototype.split=function(t,n,i){return this.concat(Y(t,n,i||new e(R)))},e.prototype.concat=function(e){return function(e,t){var n=F(e);if(D(n))return n;var i=F(t);return D(i)?new H(function(e){return n.request(e,function(e){return i.request(e)||x.of()})||x.of()}):new H(function(e,t){return n.request(e,function(e){return i.request(e,t)||x.of()})||x.of()})}(this,e)},e.prototype.request=function(e,t){throw new w.a(1)},e.empty=j,e.from=z,e.split=Y,e.execute=W,e}();function W(e,t){return e.request(L(t.context,function(e){var t={variables:e.variables||{},extensions:e.extensions||{},operationName:e.operationName,query:e.query};return t.operationName||(t.operationName="string"!=typeof t.query?Object(y.n)(t.query):""),t}(function(e){for(var t=["query","operationName","variables","extensions","context"],n=0,i=Object.keys(e);n<i.length;n++){var r=i[n];if(t.indexOf(r)<0)throw new w.a(2)}return e}(t))))||x.of()}var X,V=n(64),B=n(3),q=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.inFlightRequestObservables=new Map,t.subscribers=new Map,t}return Object(v.c)(t,e),t.prototype.request=function(e,t){var n=this;if(e.getContext().forceFetch)return t(e);var i=e.toKey();if(!this.inFlightRequestObservables.get(i)){var r,a=t(e),o=new x(function(e){return n.subscribers.has(i)||n.subscribers.set(i,new Set),n.subscribers.get(i).add(e),r||(r=a.subscribe({next:function(e){var t=n.subscribers.get(i);n.subscribers.delete(i),n.inFlightRequestObservables.delete(i),t&&(t.forEach(function(t){return t.next(e)}),t.forEach(function(e){return e.complete()}))},error:function(e){var t=n.subscribers.get(i);n.subscribers.delete(i),n.inFlightRequestObservables.delete(i),t&&t.forEach(function(t){return t.error(e)})}})),function(){n.subscribers.has(i)&&(n.subscribers.get(i).delete(e),0===n.subscribers.get(i).size&&(n.inFlightRequestObservables.delete(i),r&&r.unsubscribe()))}});this.inFlightRequestObservables.set(i,o)}return this.inFlightRequestObservables.get(i)},t}(H);function U(e){return e<7}!function(e){e[e.loading=1]="loading",e[e.setVariables=2]="setVariables",e[e.fetchMore=3]="fetchMore",e[e.refetch=4]="refetch",e[e.poll=6]="poll",e[e.ready=7]="ready",e[e.error=8]="error"}(X||(X={}));var G=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Object(v.c)(t,e),t.prototype[V.a]=function(){return this},t.prototype["@@observable"]=function(){return this},t}(x);var Q,$=function(e){var t="";return Array.isArray(e.graphQLErrors)&&0!==e.graphQLErrors.length&&e.graphQLErrors.forEach(function(e){var n=e?e.message:"Error message not found.";t+="GraphQL error: "+n+"\n"}),e.networkError&&(t+="Network error: "+e.networkError.message+"\n"),t=t.replace(/\n$/,"")},Z=function(e){function t(n){var i=n.graphQLErrors,r=n.networkError,a=n.errorMessage,o=n.extraInfo,s=e.call(this,a)||this;return s.graphQLErrors=i||[],s.networkError=r||null,s.message=a||$(s),s.extraInfo=o,s.__proto__=t.prototype,s}return Object(v.c)(t,e),t}(Error);!function(e){e[e.normal=1]="normal",e[e.refetch=2]="refetch",e[e.poll=3]="poll"}(Q||(Q={}));var K=function(e){function t(t){var n=t.queryManager,i=t.options,r=t.shouldSubscribe,a=void 0===r||r,o=e.call(this,function(e){return o.onSubscribe(e)})||this;return o.isTornDown=!1,o.options=i,o.variables=i.variables||{},o.queryId=n.generateQueryId(),o.shouldSubscribe=a,o.queryManager=n,o.observers=[],o.subscriptionHandles=[],o}return Object(v.c)(t,e),t.prototype.result=function(){var e=this;return new Promise(function(t,n){var i,r={next:function(n){t(n),e.observers.some(function(e){return e!==r})||e.queryManager.removeQuery(e.queryId),setTimeout(function(){i.unsubscribe()},0)},error:function(e){n(e)}};i=e.subscribe(r)})},t.prototype.currentResult=function(){var e=this.getCurrentResult();return void 0===e.data&&(e.data={}),e},t.prototype.getCurrentResult=function(){if(this.isTornDown)return{data:this.lastError?void 0:this.lastResult?this.lastResult.data:void 0,error:this.lastError,loading:!1,networkStatus:X.error};var e,t,n=this.queryManager.queryStore.get(this.queryId);if(e=n,void 0===(t=this.options.errorPolicy)&&(t="none"),e&&(e.graphQLErrors&&e.graphQLErrors.length>0&&"none"===t||e.networkError))return{data:void 0,loading:!1,networkStatus:n.networkStatus,error:new Z({graphQLErrors:n.graphQLErrors,networkError:n.networkError})};n&&n.variables&&(this.options.variables=Object.assign({},this.options.variables,n.variables));var i,r=this.queryManager.getCurrentQueryResult(this),a=r.data,o=r.partial,s=!n||n.networkStatus===X.loading,l="network-only"===this.options.fetchPolicy&&s||o&&"cache-only"!==this.options.fetchPolicy,c={data:a,loading:U(i=n?n.networkStatus:l?X.loading:X.ready),networkStatus:i};return n&&n.graphQLErrors&&"all"===this.options.errorPolicy&&(c.errors=n.graphQLErrors),o||(this.lastResult=Object(v.a)({},c,{stale:!1}),this.lastResultSnapshot=Object(y.e)(this.lastResult)),Object(v.a)({},c,{partial:o})},t.prototype.isDifferentFromLastResult=function(e){var t=this.lastResultSnapshot;return!(t&&e&&t.networkStatus===e.networkStatus&&t.stale===e.stale&&Object(y.t)(t.data,e.data))},t.prototype.getLastResult=function(){return this.lastResult},t.prototype.getLastError=function(){return this.lastError},t.prototype.resetLastResults=function(){delete this.lastResult,delete this.lastResultSnapshot,delete this.lastError,this.isTornDown=!1},t.prototype.refetch=function(e){var t=this.options.fetchPolicy;if("cache-only"===t)return Promise.reject(new Error("cache-only fetchPolicy option should not be used together with query refetch."));Object(y.t)(this.variables,e)||(this.variables=Object.assign({},this.variables,e)),Object(y.t)(this.options.variables,this.variables)||(this.options.variables=Object.assign({},this.options.variables,this.variables));var n="network-only"===t||"no-cache"===t,i=Object(v.a)({},this.options,{fetchPolicy:n?t:"network-only"});return this.queryManager.fetchQuery(this.queryId,i,Q.refetch).then(function(e){return e})},t.prototype.fetchMore=function(e){var t,n=this;return Object(B.b)(e.updateQuery),Promise.resolve().then(function(){var i=n.queryManager.generateQueryId();return(t=e.query?e:Object(v.a)({},n.options,e,{variables:Object.assign({},n.variables,e.variables)})).fetchPolicy="network-only",n.queryManager.fetchQuery(i,t,Q.normal,n.queryId)}).then(function(i){return n.updateQuery(function(n){return e.updateQuery(n,{fetchMoreResult:i.data,variables:t.variables})}),i})},t.prototype.subscribeToMore=function(e){var t=this,n=this.queryManager.startGraphQLSubscription({query:e.document,variables:e.variables}).subscribe({next:function(n){e.updateQuery&&t.updateQuery(function(t,i){var r=i.variables;return e.updateQuery(t,{subscriptionData:n,variables:r})})},error:function(t){e.onError?e.onError(t):console.error("Unhandled GraphQL subscription error",t)}});return this.subscriptionHandles.push(n),function(){var e=t.subscriptionHandles.indexOf(n);e>=0&&(t.subscriptionHandles.splice(e,1),n.unsubscribe())}},t.prototype.setOptions=function(e){var t=this.options;this.options=Object.assign({},this.options,e),e.pollInterval?this.startPolling(e.pollInterval):0===e.pollInterval&&this.stopPolling();var n="network-only"!==t.fetchPolicy&&"network-only"===e.fetchPolicy||"cache-only"===t.fetchPolicy&&"cache-only"!==e.fetchPolicy||"standby"===t.fetchPolicy&&"standby"!==e.fetchPolicy||!1;return this.setVariables(this.options.variables,n,e.fetchResults)},t.prototype.setVariables=function(e,t,n){void 0===t&&(t=!1),void 0===n&&(n=!0),this.isTornDown=!1;var i=e||this.variables;return Object(y.t)(i,this.variables)&&!t?0!==this.observers.length&&n?this.result():new Promise(function(e){return e()}):(this.variables=i,this.options.variables=i,0===this.observers.length?new Promise(function(e){return e()}):this.queryManager.fetchQuery(this.queryId,Object(v.a)({},this.options,{variables:this.variables})).then(function(e){return e}))},t.prototype.updateQuery=function(e){var t=this.queryManager.getQueryWithPreviousResult(this.queryId),n=t.previousResult,i=t.variables,r=t.document,a=Object(y.I)(function(){return e(n,{variables:i})});a&&(this.queryManager.dataStore.markUpdateQueryResult(r,i,a),this.queryManager.broadcastQueries())},t.prototype.stopPolling=function(){this.queryManager.stopPollingQuery(this.queryId),this.options.pollInterval=void 0},t.prototype.startPolling=function(e){J(this),this.options.pollInterval=e,this.queryManager.startPollingQuery(this.options,this.queryId)},t.prototype.onSubscribe=function(e){var t=this;return e._subscription&&e._subscription._observer&&!e._subscription._observer.error&&(e._subscription._observer.error=function(e){console.error("Unhandled error",e.message,e.stack)}),this.observers.push(e),e.next&&this.lastResult&&e.next(this.lastResult),e.error&&this.lastError&&e.error(this.lastError),1===this.observers.length&&this.setUpQuery(),function(){t.observers=t.observers.filter(function(t){return t!==e}),0===t.observers.length&&t.tearDownQuery()}},t.prototype.setUpQuery=function(){var e=this;this.shouldSubscribe&&this.queryManager.addObservableQuery(this.queryId,this),this.options.pollInterval&&(J(this),this.queryManager.startPollingQuery(this.options,this.queryId));var t={next:function(t){e.lastResult=t,e.lastResultSnapshot=Object(y.e)(t),e.observers.forEach(function(e){return e.next&&e.next(t)})},error:function(t){e.lastError=t,e.observers.forEach(function(e){return e.error&&e.error(t)})}};this.queryManager.startQuery(this.queryId,this.options,this.queryManager.queryListenerForObserver(this.queryId,this.options,t))},t.prototype.tearDownQuery=function(){this.isTornDown=!0,this.queryManager.stopPollingQuery(this.queryId),this.subscriptionHandles.forEach(function(e){return e.unsubscribe()}),this.subscriptionHandles=[],this.queryManager.removeObservableQuery(this.queryId),this.queryManager.stopQuery(this.queryId),this.observers=[]},t}(G);function J(e){var t=e.options.fetchPolicy;Object(B.b)("cache-first"!==t&&"cache-only"!==t)}var ee=function(){function e(){this.store={}}return e.prototype.getStore=function(){return this.store},e.prototype.get=function(e){return this.store[e]},e.prototype.initMutation=function(e,t,n){this.store[e]={mutation:t,variables:n||{},loading:!0,error:null}},e.prototype.markMutationError=function(e,t){var n=this.store[e];n&&(n.loading=!1,n.error=t)},e.prototype.markMutationResult=function(e){var t=this.store[e];t&&(t.loading=!1,t.error=null)},e.prototype.reset=function(){this.store={}},e}(),te=function(){function e(){this.store={}}return e.prototype.getStore=function(){return this.store},e.prototype.get=function(e){return this.store[e]},e.prototype.initQuery=function(e){var t=this.store[e.queryId];if(t&&t.document!==e.document&&!Object(y.t)(t.document,e.document))throw new B.a;var n,i=!1,r=null;e.storePreviousVariables&&t&&t.networkStatus!==X.loading&&(Object(y.t)(t.variables,e.variables)||(i=!0,r=t.variables)),n=i?X.setVariables:e.isPoll?X.poll:e.isRefetch?X.refetch:X.loading;var a=[];t&&t.graphQLErrors&&(a=t.graphQLErrors),this.store[e.queryId]={document:e.document,variables:e.variables,previousVariables:r,networkError:null,graphQLErrors:a,networkStatus:n,metadata:e.metadata},"string"==typeof e.fetchMoreForQueryId&&this.store[e.fetchMoreForQueryId]&&(this.store[e.fetchMoreForQueryId].networkStatus=X.fetchMore)},e.prototype.markQueryResult=function(e,t,n){this.store&&this.store[e]&&(this.store[e].networkError=null,this.store[e].graphQLErrors=t.errors&&t.errors.length?t.errors:[],this.store[e].previousVariables=null,this.store[e].networkStatus=X.ready,"string"==typeof n&&this.store[n]&&(this.store[n].networkStatus=X.ready))},e.prototype.markQueryError=function(e,t,n){this.store&&this.store[e]&&(this.store[e].networkError=t,this.store[e].networkStatus=X.error,"string"==typeof n&&this.markQueryResultClient(n,!0))},e.prototype.markQueryResultClient=function(e,t){this.store&&this.store[e]&&(this.store[e].networkError=null,this.store[e].previousVariables=null,this.store[e].networkStatus=t?X.ready:X.loading)},e.prototype.stopQuery=function(e){delete this.store[e]},e.prototype.reset=function(e){var t=this;this.store=Object.keys(this.store).filter(function(t){return e.indexOf(t)>-1}).reduce(function(e,n){return e[n]=Object(v.a)({},t.store[n],{networkStatus:X.loading}),e},{})},e}();var ne=function(){function e(e){var t=e.cache,n=e.client,i=e.resolvers,r=e.fragmentMatcher;this.cache=t,n&&(this.client=n),i&&this.addResolvers(i),r&&this.setFragmentMatcher(r)}return e.prototype.addResolvers=function(e){var t=this;this.resolvers=this.resolvers||{},Array.isArray(e)?e.forEach(function(e){t.resolvers=Object(y.A)(t.resolvers,e)}):this.resolvers=Object(y.A)(this.resolvers,e)},e.prototype.setResolvers=function(e){this.resolvers={},this.addResolvers(e)},e.prototype.getResolvers=function(){return this.resolvers||{}},e.prototype.runResolvers=function(e){var t=e.document,n=e.remoteResult,i=e.context,r=e.variables,a=e.onlyRunForcedResolvers,o=void 0!==a&&a;return Object(v.b)(this,void 0,void 0,function(){return Object(v.d)(this,function(e){return t?[2,this.resolveDocument(t,n.data,i,r,this.fragmentMatcher,o).then(function(e){return Object(v.a)({},n,{data:e.result})})]:[2,n]})})},e.prototype.setFragmentMatcher=function(e){this.fragmentMatcher=e},e.prototype.getFragmentMatcher=function(){return this.fragmentMatcher},e.prototype.clientQuery=function(e){return Object(y.s)(["client"],e)&&this.resolvers?e:null},e.prototype.serverQuery=function(e){return this.resolvers?Object(y.C)(e):e},e.prototype.prepareContext=function(e){void 0===e&&(e={});var t=this.cache;return Object(v.a)({},e,{cache:t,getCacheKey:function(e){if(t.config)return t.config.dataIdFromObject(e);Object(B.b)(!1)}})},e.prototype.addExportedVariables=function(e,t,n){return void 0===t&&(t={}),void 0===n&&(n={}),Object(v.b)(this,void 0,void 0,function(){return Object(v.d)(this,function(i){return e?[2,this.resolveDocument(e,this.buildRootValueFromCache(e,t)||{},this.prepareContext(n),t).then(function(e){return Object(v.a)({},t,e.exportedVariables)})]:[2,Object(v.a)({},t)]})})},e.prototype.shouldForceResolvers=function(e){var t=!1;return Object(k.b)(e,{Directive:{enter:function(e){if("client"===e.name.value&&e.arguments&&(t=e.arguments.some(function(e){return"always"===e.name.value&&"BooleanValue"===e.value.kind&&!0===e.value.value})))return k.a}}}),t},e.prototype.shouldForceResolver=function(e){return this.shouldForceResolvers(e)},e.prototype.buildRootValueFromCache=function(e,t){return this.cache.diff({query:Object(y.d)(e),variables:t,optimistic:!1}).result},e.prototype.resolveDocument=function(e,t,n,i,r,a){return void 0===n&&(n={}),void 0===i&&(i={}),void 0===r&&(r=function(){return!0}),void 0===a&&(a=!1),Object(v.b)(this,void 0,void 0,function(){var o,s,l,c,u,h,d,f,p;return Object(v.d)(this,function(g){var m;return o=Object(y.k)(e),s=Object(y.i)(e),l=Object(y.f)(s),c=o.operation,u=c?(m=c).charAt(0).toUpperCase()+m.slice(1):"Query",d=(h=this).cache,f=h.client,p={fragmentMap:l,context:Object(v.a)({},n,{cache:d,client:f}),variables:i,fragmentMatcher:r,defaultOperationType:u,exportedVariables:{},onlyRunForcedResolvers:a},[2,this.resolveSelectionSet(o.selectionSet,t,p).then(function(e){return{result:e,exportedVariables:p.exportedVariables}})]})})},e.prototype.resolveSelectionSet=function(e,t,n){return Object(v.b)(this,void 0,void 0,function(){var i,r,a,o,s,l=this;return Object(v.d)(this,function(c){return i=n.fragmentMap,r=n.context,a=n.variables,o=[t],s=function(e){return Object(v.b)(l,void 0,void 0,function(){var s,l;return Object(v.d)(this,function(c){return Object(y.F)(e,a)?Object(y.u)(e)?[2,this.resolveField(e,t,n).then(function(t){var n;void 0!==t&&o.push(((n={})[Object(y.E)(e)]=t,n))})]:(Object(y.w)(e)?s=e:(s=i[e.name.value],Object(B.b)(s)),s&&s.typeCondition&&(l=s.typeCondition.name.value,n.fragmentMatcher(t,l,r))?[2,this.resolveSelectionSet(s.selectionSet,t,n).then(function(e){o.push(e)})]:[2]):[2]})})},[2,Promise.all(e.selections.map(s)).then(function(){return Object(y.B)(o)})]})})},e.prototype.resolveField=function(e,t,n){return Object(v.b)(this,void 0,void 0,function(){var i,r,a,o,s,l,c,u,h,d=this;return Object(v.d)(this,function(f){return i=n.variables,r=e.name.value,a=Object(y.E)(e),o=r!==a,s=t[a]||t[r],l=Promise.resolve(s),n.onlyRunForcedResolvers&&!this.shouldForceResolver(e)||(c=t.__typename||n.defaultOperationType,(u=this.resolvers&&this.resolvers[c])&&(h=u[o?r:a])&&(l=Promise.resolve(h(t,Object(y.b)(e,i),n.context,{field:e})))),[2,l.then(function(t){return void 0===t&&(t=s),e.directives&&e.directives.forEach(function(e){"export"===e.name.value&&e.arguments&&e.arguments.forEach(function(e){"as"===e.name.value&&"StringValue"===e.value.kind&&(n.exportedVariables[e.value.value]=t)})}),e.selectionSet?null==t?t:Array.isArray(t)?d.resolveSubSelectedArray(e,t,n):e.selectionSet?d.resolveSelectionSet(e.selectionSet,t,n):void 0:t})]})})},e.prototype.resolveSubSelectedArray=function(e,t,n){var i=this;return Promise.all(t.map(function(t){return null===t?null:Array.isArray(t)?i.resolveSubSelectedArray(e,t,n):e.selectionSet?i.resolveSelectionSet(e.selectionSet,t,n):void 0}))},e}(),ie=function(){function e(e){var t=e.link,n=e.queryDeduplication,i=void 0!==n&&n,r=e.store,a=e.onBroadcast,o=void 0===a?function(){}:a,s=e.ssrMode,l=void 0!==s&&s,c=e.clientAwareness,u=void 0===c?{}:c,h=e.localState;this.mutationStore=new ee,this.queryStore=new te,this.clientAwareness={},this.idCounter=1,this.queries=new Map,this.fetchQueryRejectFns=new Map,this.queryIdsByName={},this.pollingInfoByQueryId=new Map,this.nextPoll=null,this.link=t,this.deduplicator=H.from([new q,t]),this.queryDeduplication=i,this.dataStore=r,this.onBroadcast=o,this.clientAwareness=u,this.localState=h||new ne({cache:r.getCache()}),this.ssrMode=l}return e.prototype.stop=function(){var e=this;this.queries.forEach(function(t,n){e.stopQueryNoBroadcast(n)}),this.fetchQueryRejectFns.forEach(function(e){e(new Error("QueryManager stopped while query was in flight"))})},e.prototype.mutate=function(e){var t=e.mutation,n=e.variables,i=e.optimisticResponse,r=e.updateQueries,a=e.refetchQueries,o=void 0===a?[]:a,s=e.awaitRefetchQueries,l=void 0!==s&&s,c=e.update,u=e.errorPolicy,h=void 0===u?"none":u,d=e.fetchPolicy,f=e.context,p=void 0===f?{}:f;return Object(v.b)(this,void 0,void 0,function(){var e,a,s,u,f,g=this;return Object(v.d)(this,function(m){switch(m.label){case 0:return Object(B.b)(t),Object(B.b)(!d||"no-cache"===d),e=this.generateQueryId(),a=this.dataStore.getCache(),t=a.transformDocument(t),n=Object(y.c)({},Object(y.g)(Object(y.l)(t)),n),this.setQuery(e,function(){return{document:t}}),s=function(){var e={};return r&&Object.keys(r).forEach(function(t){return(g.queryIdsByName[t]||[]).forEach(function(n){e[n]={updater:r[t],query:g.queryStore.get(n)}})}),e},Object(y.r)(t)?[4,this.localState.addExportedVariables(t,n,p)]:[3,2];case 1:return f=m.sent(),[3,3];case 2:f=n,m.label=3;case 3:return u=f,this.mutationStore.initMutation(e,t,u),this.dataStore.markMutationInit({mutationId:e,document:t,variables:u||{},updateQueries:s(),update:c,optimisticResponse:i}),this.broadcastQueries(),[2,new Promise(function(n,r){var a,f,m=g.buildOperationForLink(t,u,Object(v.a)({},p,{optimisticResponse:i})),b=function(){if(f&&g.mutationStore.markMutationError(e,f),g.dataStore.markMutationComplete({mutationId:e,optimisticResponse:i}),g.broadcastQueries(),f)return Promise.reject(f);"function"==typeof o&&(o=o(a));for(var t=[],n=0,r=o;n<r.length;n++){var s=r[n];if("string"!=typeof s){var c={query:s.query,variables:s.variables,fetchPolicy:"network-only"};s.context&&(c.context=s.context),t.push(g.query(c))}else{var u=g.refetchQueryByName(s);u&&t.push(u)}}return Promise.all(l?t:[]).then(function(){return g.setQuery(e,function(){return{document:null}}),"ignore"===h&&a&&Object(y.q)(a)&&delete a.errors,a})},x=g.localState.clientQuery(m.query),w=g.localState.serverQuery(m.query);w&&(m.query=w);var k=w?W(g.link,m):G.of({data:{}}),S=g,E=!1,C=!1;k.subscribe({next:function(i){return Object(v.b)(g,void 0,void 0,function(){var o,l,p;return Object(v.d)(this,function(g){switch(g.label){case 0:return C=!0,Object(y.q)(i)&&"none"===h?(C=!1,f=new Z({graphQLErrors:i.errors}),[2]):(S.mutationStore.markMutationResult(e),o=i,l=m.context,p=m.variables,x&&Object(y.s)(["client"],x)?[4,S.localState.runResolvers({document:x,remoteResult:i,context:l,variables:p}).catch(function(e){return C=!1,r(e),i})]:[3,2]);case 1:o=g.sent(),g.label=2;case 2:return"no-cache"!==d&&S.dataStore.markMutationResult({mutationId:e,result:o,document:t,variables:u||{},updateQueries:s(),update:c}),a=o,C=!1,E&&b().then(n,r),[2]}})})},error:function(t){S.mutationStore.markMutationError(e,t),S.dataStore.markMutationComplete({mutationId:e,optimisticResponse:i}),S.broadcastQueries(),S.setQuery(e,function(){return{document:null}}),r(new Z({networkError:t}))},complete:function(){C||b().then(n,r),E=!0}})})]}})})},e.prototype.fetchQuery=function(e,t,n,i){return Object(v.b)(this,void 0,void 0,function(){var r,a,o,s,l,c,u,h,d,f,p,g,m,b,x,w,k,S,E,C,T,A,_=this;return Object(v.d)(this,function(O){switch(O.label){case 0:return r=t.variables,a=void 0===r?{}:r,o=t.metadata,s=void 0===o?null:o,l=t.fetchPolicy,c=void 0===l?"cache-first":l,u=t.context,h=void 0===u?{}:u,d=this.dataStore.getCache(),f=d.transformDocument(t.query),Object(y.r)(f)?[4,this.localState.addExportedVariables(f,a,h)]:[3,2];case 1:return g=O.sent(),[3,3];case 2:g=a,O.label=3;case 3:if(p=g,m=Object(v.a)({},t,{variables:p}),x="network-only"===c||"no-cache"===c,n!==Q.refetch&&"network-only"!==c&&"no-cache"!==c&&(w=this.dataStore.getCache().diff({query:f,variables:p,returnPartialData:!0,optimistic:!1}),k=w.complete,S=w.result,x=!k||"cache-and-network"===c,b=S),E=x&&"cache-only"!==c&&"standby"!==c,Object(y.s)(["live"],f)&&(E=!0),C=this.generateRequestId(),T=this.updateQueryWatch(e,f,m),this.setQuery(e,function(){return{document:f,lastRequestId:C,invalidated:!0,cancel:T}}),this.invalidate(!0,i),this.queryStore.initQuery({queryId:e,document:f,storePreviousVariables:E,variables:p,isPoll:n===Q.poll,isRefetch:n===Q.refetch,metadata:s,fetchMoreForQueryId:i}),this.broadcastQueries(),(!E||"cache-and-network"===c)&&(this.queryStore.markQueryResultClient(e,!E),this.invalidate(!0,e,i),this.broadcastQueries(this.localState.shouldForceResolvers(f))),E){if(A=this.fetchRequest({requestId:C,queryId:e,document:f,options:m,fetchMoreForQueryId:i}).catch(function(t){if(t.hasOwnProperty("graphQLErrors"))throw t;var n=_.getQuery(e).lastRequestId;throw C>=(n||1)&&(_.queryStore.markQueryError(e,t,i),_.invalidate(!0,e,i),_.broadcastQueries()),new Z({networkError:t})}),"cache-and-network"!==c)return[2,A];A.catch(function(){})}return[2,Promise.resolve({data:b})]}})})},e.prototype.queryListenerForObserver=function(e,t,n){var i=this,r=!1;return function(a,o,s){return Object(v.b)(i,void 0,void 0,function(){var i,l,c,u,h,d,f,p,g,m,y,b,x,w,k,S,E,C,T,A;return Object(v.d)(this,function(_){switch(_.label){case 0:if(this.invalidate(!1,e),!a)return[2];if(i=this.getQuery(e).observableQuery,"standby"===(l=i?i.options.fetchPolicy:t.fetchPolicy))return[2];if(c=i?i.options.errorPolicy:t.errorPolicy,u=i?i.getLastResult():null,h=i?i.getLastError():null,d=!o&&null!=a.previousVariables||"cache-only"===l||"cache-and-network"===l,f=Boolean(u&&a.networkStatus!==u.networkStatus),p=c&&(h&&h.graphQLErrors)!==a.graphQLErrors&&"none"!==c,!(!U(a.networkStatus)||f&&t.notifyOnNetworkStatusChange||d))return[3,8];if((!c||"none"===c)&&a.graphQLErrors&&a.graphQLErrors.length>0||a.networkError){if(g=new Z({graphQLErrors:a.graphQLErrors,networkError:a.networkError}),r=!0,n.error)try{n.error(g)}catch(e){setTimeout(function(){throw e},0)}else setTimeout(function(){throw g},0);return[2]}_.label=1;case 1:if(_.trys.push([1,7,,8]),m=void 0,y=void 0,o?("no-cache"!==l&&"network-only"!==l&&this.setQuery(e,function(){return{newData:null}}),m=o.result,y=!o.complete||!1):u&&u.data&&!p?(m=u.data,y=!1):(b=this.getQuery(e).document,x=this.dataStore.getCache().diff({query:b,variables:a.previousVariables||a.variables,optimistic:!0}),m=x.result,y=!x.complete),w=void 0,w=y&&"cache-only"!==l?{data:u&&u.data,loading:U(a.networkStatus),networkStatus:a.networkStatus,stale:!0}:{data:m,loading:U(a.networkStatus),networkStatus:a.networkStatus,stale:!1},"all"===c&&a.graphQLErrors&&a.graphQLErrors.length>0&&(w.errors=a.graphQLErrors),!n.next)return[3,6];if(!r&&i&&!i.isDifferentFromLastResult(w))return[3,6];_.label=2;case 2:return _.trys.push([2,5,,6]),s?(k=t.query,S=t.variables,E=t.context,[4,this.localState.runResolvers({document:k,remoteResult:w,context:E,variables:S,onlyRunForcedResolvers:s})]):[3,4];case 3:C=_.sent(),w=Object(v.a)({},w,C),_.label=4;case 4:return n.next(w),[3,6];case 5:return T=_.sent(),setTimeout(function(){throw T},0),[3,6];case 6:return r=!1,[3,8];case 7:return A=_.sent(),r=!0,n.error&&n.error(new Z({networkError:A})),[2];case 8:return[2]}})})}},e.prototype.watchQuery=function(e,t){void 0===t&&(t=!0),Object(B.b)("standby"!==e.fetchPolicy);var n=Object(y.o)(e.query);if(n.variableDefinitions&&n.variableDefinitions.length){var i=Object(y.g)(n);e.variables=Object(y.c)({},i,e.variables)}void 0===e.notifyOnNetworkStatusChange&&(e.notifyOnNetworkStatusChange=!1);var r=Object(v.a)({},e);return new K({queryManager:this,options:r,shouldSubscribe:t})},e.prototype.query=function(e){var t=this;return Object(B.b)(e.query),Object(B.b)("Document"===e.query.kind),Object(B.b)(!e.returnPartialData),Object(B.b)(!e.pollInterval),new Promise(function(n,i){var r=t.watchQuery(e,!1);t.fetchQueryRejectFns.set("query:"+r.queryId,i),r.result().then(n,i).then(function(){return t.fetchQueryRejectFns.delete("query:"+r.queryId)})})},e.prototype.generateQueryId=function(){var e=this.idCounter.toString();return this.idCounter++,e},e.prototype.stopQueryInStore=function(e){this.stopQueryInStoreNoBroadcast(e),this.broadcastQueries()},e.prototype.stopQueryInStoreNoBroadcast=function(e){this.stopPollingQuery(e),this.queryStore.stopQuery(e),this.invalidate(!0,e)},e.prototype.addQueryListener=function(e,t){this.setQuery(e,function(e){var n=e.listeners;return{listeners:(void 0===n?[]:n).concat([t]),invalidated:!1}})},e.prototype.updateQueryWatch=function(e,t,n){var i=this,r=this.getQuery(e).cancel;r&&r();return this.dataStore.getCache().watch({query:t,variables:n.variables,optimistic:!0,previousResult:function(){var t=null,n=i.getQuery(e).observableQuery;if(n){var r=n.getLastResult();r&&(t=r.data)}return t},callback:function(t){i.setQuery(e,function(){return{invalidated:!0,newData:t}})}})},e.prototype.addObservableQuery=function(e,t){this.setQuery(e,function(){return{observableQuery:t}});var n=Object(y.o)(t.options.query);if(n.name&&n.name.value){var i=n.name.value;this.queryIdsByName[i]=this.queryIdsByName[i]||[],this.queryIdsByName[i].push(t.queryId)}},e.prototype.removeObservableQuery=function(e){var t=this.getQuery(e),n=t.observableQuery,i=t.cancel;if(i&&i(),n){var r=Object(y.o)(n.options.query),a=r.name?r.name.value:null;this.setQuery(e,function(){return{observableQuery:null}}),a&&(this.queryIdsByName[a]=this.queryIdsByName[a].filter(function(e){return!(n.queryId===e)}))}},e.prototype.clearStore=function(){this.fetchQueryRejectFns.forEach(function(e){e(new Error("Store reset while query was in flight(not completed in link chain)"))});var e=[];return this.queries.forEach(function(t,n){t.observableQuery&&e.push(n)}),this.queryStore.reset(e),this.mutationStore.reset(),this.dataStore.reset()},e.prototype.resetStore=function(){var e=this;return this.clearStore().then(function(){return e.reFetchObservableQueries()})},e.prototype.reFetchObservableQueries=function(e){var t=this.getObservableQueryPromises(e);return this.broadcastQueries(),Promise.all(t)},e.prototype.startQuery=function(e,t,n){return this.addQueryListener(e,n),this.fetchQuery(e,t).catch(function(){}),e},e.prototype.startGraphQLSubscription=function(e){var t,n=this,i=e.query,r=!(e.fetchPolicy&&"no-cache"===e.fetchPolicy),a=this.dataStore.getCache().transformDocument(i),o=Object(y.c)({},Object(y.g)(Object(y.m)(i)),e.variables),s=o,l=[],c=this.localState.clientQuery(a);return new G(function(e){if(l.push(e),1===l.length){var i=0,u=!1,h={next:function(e){return Object(v.b)(n,void 0,void 0,function(){var t;return Object(v.d)(this,function(n){switch(n.label){case 0:return i+=1,t=e,c&&Object(y.s)(["client"],c)?[4,this.localState.runResolvers({document:c,remoteResult:e,context:{},variables:s})]:[3,2];case 1:t=n.sent(),n.label=2;case 2:return r&&(this.dataStore.markSubscriptionResult(t,a,s),this.broadcastQueries()),l.forEach(function(e){Object(y.q)(t)&&e.error?e.error(new Z({graphQLErrors:t.errors})):e.next&&e.next(t),i-=1}),0===i&&u&&h.complete(),[2]}})})},error:function(e){l.forEach(function(t){t.error&&t.error(e)})},complete:function(){0===i&&l.forEach(function(e){e.complete&&e.complete()}),u=!0}};Object(v.b)(n,void 0,void 0,function(){var e,n,i,r;return Object(v.d)(this,function(s){switch(s.label){case 0:return Object(y.r)(a)?[4,this.localState.addExportedVariables(a,o)]:[3,2];case 1:return n=s.sent(),[3,3];case 2:n=o,s.label=3;case 3:return e=n,(i=this.localState.serverQuery(a))?(r=this.buildOperationForLink(i,e),t=W(this.link,r).subscribe(h)):t=G.of({data:{}}).subscribe(h),[2]}})})}return function(){0===(l=l.filter(function(t){return t!==e})).length&&t&&t.unsubscribe()}})},e.prototype.stopQuery=function(e){this.stopQueryNoBroadcast(e),this.broadcastQueries()},e.prototype.stopQueryNoBroadcast=function(e){this.stopQueryInStoreNoBroadcast(e),this.removeQuery(e)},e.prototype.removeQuery=function(e){var t=this.getQuery(e).subscriptions;this.fetchQueryRejectFns.delete("query:"+e),this.fetchQueryRejectFns.delete("fetchRequest:"+e),t.forEach(function(e){return e.unsubscribe()}),this.queries.delete(e)},e.prototype.getCurrentQueryResult=function(e,t){void 0===t&&(t=!0);var n=e.options,i=n.variables,r=n.query,a=n.fetchPolicy,o=e.getLastResult(),s=this.getQuery(e.queryId).newData;if(s&&s.complete)return{data:s.result,partial:!1};if("no-cache"===a||"network-only"===a)return{data:void 0,partial:!1};try{return{data:this.dataStore.getCache().read({query:r,variables:i,previousResult:o?o.data:void 0,optimistic:t})||void 0,partial:!1}}catch(e){return{data:void 0,partial:!0}}},e.prototype.getQueryWithPreviousResult=function(e){var t;if("string"==typeof e){var n=this.getQuery(e).observableQuery;Object(B.b)(n),t=n}else t=e;var i=t.options,r=i.variables,a=i.query;return{previousResult:this.getCurrentQueryResult(t,!1).data,variables:r,document:a}},e.prototype.broadcastQueries=function(e){var t=this;void 0===e&&(e=!1),this.onBroadcast(),this.queries.forEach(function(n,i){n.invalidated&&n.listeners&&n.listeners.filter(function(e){return!!e}).forEach(function(r){r(t.queryStore.get(i),n.newData,e)})})},e.prototype.getLocalState=function(){return this.localState},e.prototype.getObservableQueryPromises=function(e){var t=this,n=[];return this.queries.forEach(function(i,r){var a=i.observableQuery;if(a){var o=a.options.fetchPolicy;a.resetLastResults(),"cache-only"===o||!e&&"standby"===o||n.push(a.refetch()),t.setQuery(r,function(){return{newData:null}}),t.invalidate(!0,r)}}),n},e.prototype.fetchRequest=function(e){var t,n,i=this,r=e.requestId,a=e.queryId,o=e.document,s=e.options,l=e.fetchMoreForQueryId,c=s.variables,u=s.context,h=s.errorPolicy,d=void 0===h?"none":h,f=s.fetchPolicy;return new Promise(function(e,s){var h,p={},g=i.localState.clientQuery(o),m=i.localState.serverQuery(o);if(m){var b=i.buildOperationForLink(m,c,Object(v.a)({},u,{forceFetch:!i.queryDeduplication}));p=b.context,h=W(i.deduplicator,b)}else p=i.prepareContext(u),h=G.of({data:{}});i.fetchQueryRejectFns.set("fetchRequest:"+a,s);var x=!1,w=!0,k={next:function(e){return Object(v.b)(i,void 0,void 0,function(){var i,u;return Object(v.d)(this,function(h){switch(h.label){case 0:return w=!0,i=e,u=this.getQuery(a).lastRequestId,r>=(u||1)?g&&Object(y.s)(["client"],g)?[4,this.localState.runResolvers({document:g,remoteResult:e,context:p,variables:c}).catch(function(t){return w=!1,s(t),e})]:[3,2]:[3,3];case 1:i=h.sent(),h.label=2;case 2:if("no-cache"!==f)try{this.dataStore.markQueryResult(i,o,c,l,"ignore"===d||"all"===d)}catch(e){return w=!1,s(e),[2]}else this.setQuery(a,function(){return{newData:{result:i.data,complete:!0}}});this.queryStore.markQueryResult(a,i,l),this.invalidate(!0,a,l),this.broadcastQueries(),h.label=3;case 3:if(i.errors&&"none"===d)return w=!1,s(new Z({graphQLErrors:i.errors})),[2];if("all"===d&&(n=i.errors),l||"no-cache"===f)t=i.data;else try{t=this.dataStore.getCache().read({variables:c,query:o,optimistic:!1})}catch(e){}return w=!1,x&&k.complete(),[2]}})})},error:function(e){i.fetchQueryRejectFns.delete("fetchRequest:"+a),i.setQuery(a,function(e){return{subscriptions:e.subscriptions.filter(function(e){return e!==S})}}),s(e)},complete:function(){w||(i.fetchQueryRejectFns.delete("fetchRequest:"+a),i.setQuery(a,function(e){return{subscriptions:e.subscriptions.filter(function(e){return e!==S})}}),e({data:t,errors:n,loading:!1,networkStatus:X.ready,stale:!1})),x=!0}},S=h.subscribe(k);i.setQuery(a,function(e){return{subscriptions:e.subscriptions.concat([S])}})}).catch(function(e){throw i.fetchQueryRejectFns.delete("fetchRequest:"+a),e})},e.prototype.refetchQueryByName=function(e){var t=this,n=this.queryIdsByName[e];if(void 0!==n)return Promise.all(n.map(function(e){return t.getQuery(e).observableQuery}).filter(function(e){return!!e}).map(function(e){return e.refetch()}))},e.prototype.generateRequestId=function(){var e=this.idCounter;return this.idCounter++,e},e.prototype.getQuery=function(e){return this.queries.get(e)||{listeners:[],invalidated:!1,document:null,newData:null,lastRequestId:null,observableQuery:null,subscriptions:[]}},e.prototype.setQuery=function(e,t){var n=this.getQuery(e),i=Object(v.a)({},n,t(n));this.queries.set(e,i)},e.prototype.invalidate=function(e,t,n){t&&this.setQuery(t,function(){return{invalidated:e}}),n&&this.setQuery(n,function(){return{invalidated:e}})},e.prototype.buildOperationForLink=function(e,t,n){var i=this.dataStore.getCache();return{query:i.transformForLink?i.transformForLink(e):e,variables:t,operationName:Object(y.n)(e)||void 0,context:this.prepareContext(n)}},e.prototype.prepareContext=function(e){void 0===e&&(e={});var t=this.localState.prepareContext(e);return Object(v.a)({},t,{clientAwareness:this.clientAwareness})},e.prototype.checkInFlight=function(e){var t=this.queryStore.get(e);return t&&t.networkStatus!==X.ready&&t.networkStatus!==X.error},e.prototype.startPollingQuery=function(e,t,n){var i=e.pollInterval;return Object(B.b)(i),this.ssrMode||(this.pollingInfoByQueryId.set(t,{interval:i,lastPollTimeMs:Date.now()-10,options:Object(v.a)({},e,{fetchPolicy:"network-only"})}),n&&this.addQueryListener(t,n),this.schedulePoll(i)),t},e.prototype.stopPollingQuery=function(e){this.pollingInfoByQueryId.delete(e)},e.prototype.schedulePoll=function(e){var t=this,n=Date.now();if(this.nextPoll){if(!(e<this.nextPoll.time-n))return;clearTimeout(this.nextPoll.timeout)}this.nextPoll={time:n+e,timeout:setTimeout(function(){t.nextPoll=null;var e=1/0;t.pollingInfoByQueryId.forEach(function(n,i){if(n.interval<e&&(e=n.interval),!t.checkInFlight(i)&&Date.now()-n.lastPollTimeMs>=n.interval){var r=function(){n.lastPollTimeMs=Date.now()};t.fetchQuery(i,n.options,Q.poll).then(r,r)}}),isFinite(e)&&t.schedulePoll(e)},e)}},e}(),re=function(){function e(e){this.cache=e}return e.prototype.getCache=function(){return this.cache},e.prototype.markQueryResult=function(e,t,n,i,r){void 0===r&&(r=!1);var a=!Object(y.q)(e);r&&Object(y.q)(e)&&e.data&&(a=!0),!i&&a&&this.cache.write({result:e.data,dataId:"ROOT_QUERY",query:t,variables:n})},e.prototype.markSubscriptionResult=function(e,t,n){Object(y.q)(e)||this.cache.write({result:e.data,dataId:"ROOT_SUBSCRIPTION",query:t,variables:n})},e.prototype.markMutationInit=function(e){var t=this;if(e.optimisticResponse){var n;n="function"==typeof e.optimisticResponse?e.optimisticResponse(e.variables):e.optimisticResponse;this.cache.recordOptimisticTransaction(function(i){var r=t.cache;t.cache=i;try{t.markMutationResult({mutationId:e.mutationId,result:{data:n},document:e.document,variables:e.variables,updateQueries:e.updateQueries,update:e.update})}finally{t.cache=r}},e.mutationId)}},e.prototype.markMutationResult=function(e){var t=this;if(!Object(y.q)(e.result)){var n=[];n.push({result:e.result.data,dataId:"ROOT_MUTATION",query:e.document,variables:e.variables}),e.updateQueries&&Object.keys(e.updateQueries).filter(function(t){return e.updateQueries[t]}).forEach(function(i){var r=e.updateQueries[i],a=r.query,o=r.updater,s=t.cache.diff({query:a.document,variables:a.variables,returnPartialData:!0,optimistic:!1}),l=s.result;if(s.complete){var c=Object(y.I)(function(){return o(l,{mutationResult:e.result,queryName:Object(y.n)(a.document)||void 0,queryVariables:a.variables})});c&&n.push({result:c,dataId:"ROOT_QUERY",query:a.document,variables:a.variables})}}),this.cache.performTransaction(function(e){n.forEach(function(t){return e.write(t)})});var i=e.update;i&&this.cache.performTransaction(function(t){Object(y.I)(function(){return i(t,e.result)})})}},e.prototype.markMutationComplete=function(e){var t=e.mutationId;e.optimisticResponse&&this.cache.removeOptimistic(t)},e.prototype.markUpdateQueryResult=function(e,t,n){this.cache.write({result:n,dataId:"ROOT_QUERY",variables:t,query:e})},e.prototype.reset=function(){return this.cache.reset()},e}(),ae="2.5.1",oe=function(){function e(e){var t=this;this.defaultOptions={},this.resetStoreCallbacks=[],this.clearStoreCallbacks=[],this.clientAwareness={};var n=e.cache,i=e.ssrMode,r=void 0!==i&&i,a=e.ssrForceFetchDelay,o=void 0===a?0:a,s=e.connectToDevTools,l=e.queryDeduplication,c=void 0===l||l,u=e.defaultOptions,h=e.resolvers,d=e.typeDefs,f=e.fragmentMatcher,p=e.name,g=e.version,m=e.link;if(!m&&h&&(m=H.empty()),!m||!n)throw new B.a;var v=new Map,b=new H(function(e,t){var n=v.get(e.query);return n||(n=Object(y.D)(e.query),v.set(e.query,n),v.set(n,n)),e.query=n,t(e)});this.link=b.concat(m),this.cache=n,this.store=new re(n),this.disableNetworkFetches=r||o>0,this.queryDeduplication=c,this.ssrMode=r,this.defaultOptions=u||{},this.typeDefs=d,o&&setTimeout(function(){return t.disableNetworkFetches=!1},o),this.watchQuery=this.watchQuery.bind(this),this.query=this.query.bind(this),this.mutate=this.mutate.bind(this),this.resetStore=this.resetStore.bind(this),this.reFetchObservableQueries=this.reFetchObservableQueries.bind(this);void 0!==s&&(s&&"undefined"!=typeof window)&&(window.__APOLLO_CLIENT__=this),this.version=ae,p&&(this.clientAwareness.name=p),g&&(this.clientAwareness.version=g),this.localState=new ne({cache:n,client:this,resolvers:h,fragmentMatcher:f})}return e.prototype.stop=function(){this.queryManager&&this.queryManager.stop()},e.prototype.watchQuery=function(e){return this.defaultOptions.watchQuery&&(e=Object(v.a)({},this.defaultOptions.watchQuery,e)),!this.disableNetworkFetches||"network-only"!==e.fetchPolicy&&"cache-and-network"!==e.fetchPolicy||(e=Object(v.a)({},e,{fetchPolicy:"cache-first"})),this.initQueryManager().watchQuery(e)},e.prototype.query=function(e){return this.defaultOptions.query&&(e=Object(v.a)({},this.defaultOptions.query,e)),Object(B.b)("cache-and-network"!==e.fetchPolicy),this.disableNetworkFetches&&"network-only"===e.fetchPolicy&&(e=Object(v.a)({},e,{fetchPolicy:"cache-first"})),this.initQueryManager().query(e)},e.prototype.mutate=function(e){return this.defaultOptions.mutate&&(e=Object(v.a)({},this.defaultOptions.mutate,e)),this.initQueryManager().mutate(e)},e.prototype.subscribe=function(e){return this.initQueryManager().startGraphQLSubscription(e)},e.prototype.readQuery=function(e,t){return void 0===t&&(t=!1),this.initProxy().readQuery(e,t)},e.prototype.readFragment=function(e,t){return void 0===t&&(t=!1),this.initProxy().readFragment(e,t)},e.prototype.writeQuery=function(e){var t=this.initProxy().writeQuery(e);return this.initQueryManager().broadcastQueries(),t},e.prototype.writeFragment=function(e){var t=this.initProxy().writeFragment(e);return this.initQueryManager().broadcastQueries(),t},e.prototype.writeData=function(e){var t=this.initProxy().writeData(e);return this.initQueryManager().broadcastQueries(),t},e.prototype.__actionHookForDevTools=function(e){this.devToolsHookCb=e},e.prototype.__requestRaw=function(e){return W(this.link,e)},e.prototype.initQueryManager=function(){var e=this;return this.queryManager||(this.queryManager=new ie({link:this.link,store:this.store,queryDeduplication:this.queryDeduplication,ssrMode:this.ssrMode,clientAwareness:this.clientAwareness,localState:this.localState,onBroadcast:function(){e.devToolsHookCb&&e.devToolsHookCb({action:{},state:{queries:e.queryManager?e.queryManager.queryStore.getStore():{},mutations:e.queryManager?e.queryManager.mutationStore.getStore():{}},dataWithOptimisticResults:e.cache.extract(!0)})}})),this.queryManager},e.prototype.resetStore=function(){var e=this;return Promise.resolve().then(function(){return e.queryManager?e.queryManager.clearStore():Promise.resolve(null)}).then(function(){return Promise.all(e.resetStoreCallbacks.map(function(e){return e()}))}).then(function(){return e.queryManager&&e.queryManager.reFetchObservableQueries?e.queryManager.reFetchObservableQueries():Promise.resolve(null)})},e.prototype.clearStore=function(){var e=this,t=this.queryManager;return Promise.resolve().then(function(){return Promise.all(e.clearStoreCallbacks.map(function(e){return e()}))}).then(function(){return t?t.clearStore():Promise.resolve(null)})},e.prototype.onResetStore=function(e){var t=this;return this.resetStoreCallbacks.push(e),function(){t.resetStoreCallbacks=t.resetStoreCallbacks.filter(function(t){return t!==e})}},e.prototype.onClearStore=function(e){var t=this;return this.clearStoreCallbacks.push(e),function(){t.clearStoreCallbacks=t.clearStoreCallbacks.filter(function(t){return t!==e})}},e.prototype.reFetchObservableQueries=function(e){return this.queryManager?this.queryManager.reFetchObservableQueries(e):Promise.resolve(null)},e.prototype.extract=function(e){return this.initProxy().extract(e)},e.prototype.restore=function(e){return this.initProxy().restore(e)},e.prototype.addResolvers=function(e){this.localState.addResolvers(e)},e.prototype.setResolvers=function(e){this.localState.setResolvers(e)},e.prototype.getResolvers=function(){return this.localState.getResolvers()},e.prototype.setLocalStateFragmentMatcher=function(e){this.localState.setFragmentMatcher(e)},e.prototype.initProxy=function(){return this.proxy||(this.initQueryManager(),this.proxy=this.cache),this.proxy},e}();function se(e){return{kind:"Document",definitions:[{kind:"OperationDefinition",operation:"query",name:{kind:"Name",value:"GeneratedClientQuery"},selectionSet:le(e)}]}}function le(e){if("number"==typeof e||"boolean"==typeof e||"string"==typeof e||null==e)return null;if(Array.isArray(e))return le(e[0]);var t=[];return Object.keys(e).forEach(function(n){var i={kind:"Field",name:{kind:"Name",value:n},selectionSet:le(e[n])||void 0};t.push(i)}),{kind:"SelectionSet",selections:t}}var ce,ue={kind:"Document",definitions:[{kind:"OperationDefinition",operation:"query",name:null,variableDefinitions:null,directives:[],selectionSet:{kind:"SelectionSet",selections:[{kind:"Field",alias:null,name:{kind:"Name",value:"__typename"},arguments:[],directives:[],selectionSet:null}]}}]},he=function(){function e(){}return e.prototype.transformDocument=function(e){return e},e.prototype.transformForLink=function(e){return e},e.prototype.readQuery=function(e,t){return void 0===t&&(t=!1),this.read({query:e.query,variables:e.variables,optimistic:t})},e.prototype.readFragment=function(e,t){return void 0===t&&(t=!1),this.read({query:Object(y.j)(e.fragment,e.fragmentName),variables:e.variables,rootId:e.id,optimistic:t})},e.prototype.writeQuery=function(e){this.write({dataId:"ROOT_QUERY",result:e.data,query:e.query,variables:e.variables})},e.prototype.writeFragment=function(e){this.write({dataId:e.id,result:e.data,variables:e.variables,query:Object(y.j)(e.fragment,e.fragmentName)})},e.prototype.writeData=function(e){var t,n,i=e.id,r=e.data;if(void 0!==i){var a=null;try{a=this.read({rootId:i,optimistic:!1,query:ue})}catch(e){}var o=a&&a.__typename||"__ClientData",s=Object.assign({__typename:o},r);this.writeFragment({id:i,fragment:(t=s,n=o,{kind:"Document",definitions:[{kind:"FragmentDefinition",typeCondition:{kind:"NamedType",name:{kind:"Name",value:n||"__FakeType"}},name:{kind:"Name",value:"GeneratedClientQuery"},selectionSet:le(t)}]}),data:s})}else this.writeQuery({query:se(r),data:r})},e}();ce||(ce={});var de=n(18),fe=new Map;if(fe.set(1,2)!==fe){var pe=fe.set;Map.prototype.set=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return pe.apply(this,e),this}}var ge=new Set;if(ge.add(3)!==ge){var me=ge.add;Set.prototype.add=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return me.apply(this,e),this}}var ve={};"function"==typeof Object.freeze&&Object.freeze(ve);try{fe.set(ve,ve).delete(ve)}catch(e){var ye=function(e){return e&&function(t){try{fe.set(t,t).delete(t)}finally{return e.call(Object,t)}}};Object.freeze=ye(Object.freeze),Object.seal=ye(Object.seal),Object.preventExtensions=ye(Object.preventExtensions)}var be=!1;function xe(){var e=!be;return Object(y.z)()||(be=!0),e}var we=function(){function e(){}return e.prototype.ensureReady=function(){return Promise.resolve()},e.prototype.canBypassInit=function(){return!0},e.prototype.match=function(e,t,n){var i=n.store.get(e.id);return!i&&"ROOT_QUERY"===e.id||!!i&&(i.__typename&&i.__typename===t||(xe(),"heuristic"))},e}(),ke=(function(){function e(e){e&&e.introspectionQueryResultData?(this.possibleTypesMap=this.parseIntrospectionResult(e.introspectionQueryResultData),this.isReady=!0):this.isReady=!1,this.match=this.match.bind(this)}e.prototype.match=function(e,t,n){Object(B.b)(this.isReady);var i=n.store.get(e.id);if(!i)return!1;if(Object(B.b)(i.__typename),i.__typename===t)return!0;var r=this.possibleTypesMap[t];return!!(r&&r.indexOf(i.__typename)>-1)},e.prototype.parseIntrospectionResult=function(e){var t={};return e.__schema.types.forEach(function(e){"UNION"!==e.kind&&"INTERFACE"!==e.kind||(t[e.name]=e.possibleTypes.map(function(e){return e.name}))}),t}}(),function(){function e(){this.children=null,this.key=null}return e.prototype.lookup=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return this.lookupArray(e)},e.prototype.lookupArray=function(e){var t=this;return e.forEach(function(e){t=t.getOrCreate(e)}),t.key||(t.key=Object.create(null))},e.prototype.getOrCreate=function(t){var n=this.children||(this.children=new Map),i=n.get(t);return i||n.set(t,i=new e),i},e}()),Se=Object.prototype.hasOwnProperty,Ee=function(){function e(e){void 0===e&&(e=Object.create(null));var t=this;this.data=e,this.depend=Object(de.wrap)(function(e){return t.data[e]},{disposable:!0,makeCacheKey:function(e){return e}})}return e.prototype.toObject=function(){return this.data},e.prototype.get=function(e){return this.depend(e),this.data[e]},e.prototype.set=function(e,t){t!==this.data[e]&&(this.data[e]=t,this.depend.dirty(e))},e.prototype.delete=function(e){Se.call(this.data,e)&&(delete this.data[e],this.depend.dirty(e))},e.prototype.clear=function(){this.replace(null)},e.prototype.replace=function(e){var t=this;e?(Object.keys(e).forEach(function(n){t.set(n,e[n])}),Object.keys(this.data).forEach(function(n){Se.call(e,n)||t.delete(n)})):Object.keys(this.data).forEach(function(e){t.delete(e)})},e}();function Ce(e){return new Ee(e)}var Te=function(){function e(e){void 0===e&&(e=new ke);var t=this;this.cacheKeyRoot=e;var n=this,i=n.executeStoreQuery,r=n.executeSelectionSet;this.executeStoreQuery=Object(de.wrap)(function(e){return i.call(t,e)},{makeCacheKey:function(e){var t=e.query,i=e.rootValue,r=e.contextValue,a=e.variableValues,o=e.fragmentMatcher;if(r.store instanceof Ee)return n.cacheKeyRoot.lookup(t,r.store,o,JSON.stringify(a),i.id)}}),this.executeSelectionSet=Object(de.wrap)(function(e){return r.call(t,e)},{makeCacheKey:function(e){var t=e.selectionSet,i=e.rootValue,r=e.execContext;if(r.contextValue.store instanceof Ee)return n.cacheKeyRoot.lookup(t,r.contextValue.store,r.fragmentMatcher,JSON.stringify(r.variableValues),i.id)}})}return e.prototype.readQueryFromStore=function(e){return this.diffQueryAgainstStore(Object(v.a)({},e,{returnPartialData:!1})).result},e.prototype.diffQueryAgainstStore=function(e){var t=e.store,n=e.query,i=e.variables,r=e.previousResult,a=e.returnPartialData,o=void 0===a||a,s=e.rootId,l=void 0===s?"ROOT_QUERY":s,c=e.fragmentMatcherFunction,u=e.config,h=Object(y.o)(n);i=Object(y.c)({},Object(y.g)(h),i);var d={store:t,dataIdFromObject:u&&u.dataIdFromObject||null,cacheRedirects:u&&u.cacheRedirects||{}},f=this.executeStoreQuery({query:n,rootValue:{type:"id",id:l,generated:!0,typename:"Query"},contextValue:d,variableValues:i,fragmentMatcher:c}),p=f.missing&&f.missing.length>0;return p&&!o&&f.missing.forEach(function(e){if(!e.tolerable)throw new B.a}),r&&Object(y.t)(r,f.result)&&(f.result=r),{result:f.result,complete:!p}},e.prototype.executeStoreQuery=function(e){var t=e.query,n=e.rootValue,i=e.contextValue,r=e.variableValues,a=e.fragmentMatcher,o=void 0===a?_e:a,s=Object(y.k)(t),l=Object(y.i)(t),c={query:t,fragmentMap:Object(y.f)(l),contextValue:i,variableValues:r,fragmentMatcher:o};return this.executeSelectionSet({selectionSet:s.selectionSet,rootValue:n,execContext:c})},e.prototype.executeSelectionSet=function(e){var t=this,n=e.selectionSet,i=e.rootValue,r=e.execContext,a=r.fragmentMap,o=r.contextValue,s=r.variableValues,l={result:null},c=[],u=o.store.get(i.id),h=u&&u.__typename||"ROOT_QUERY"===i.id&&"Query"||void 0;function d(e){var t;return e.missing&&(l.missing=l.missing||[],(t=l.missing).push.apply(t,e.missing)),e.result}return n.selections.forEach(function(e){var n;if(Object(y.F)(e,s))if(Object(y.u)(e)){var l=d(t.executeField(u,h,e,r));void 0!==l&&c.push(((n={})[Object(y.E)(e)]=l,n))}else{var f=void 0;if(Object(y.w)(e))f=e;else if(!(f=a[e.name.value]))throw new B.a;var p=f.typeCondition.name.value,g=r.fragmentMatcher(i,p,o);if(g){var m=t.executeSelectionSet({selectionSet:f.selectionSet,rootValue:i,execContext:r});"heuristic"===g&&m.missing&&(m=Object(v.a)({},m,{missing:m.missing.map(function(e){return Object(v.a)({},e,{tolerable:!0})})})),c.push(d(m))}}}),l.result=Object(y.B)(c),l},e.prototype.executeField=function(e,t,n,i){var r=i.variableValues,a=i.contextValue,o=function(e,t,n,i,r,a){a.resultKey;var o=a.directives,s=n;(i||o)&&(s=Object(y.p)(s,i,o));var l=void 0;if(e&&void 0===(l=e[s])&&r.cacheRedirects&&"string"==typeof t){var c=r.cacheRedirects[t];if(c){var u=c[n];u&&(l=u(e,i,{getCacheKey:function(e){return Object(y.H)({id:r.dataIdFromObject(e),typename:e.__typename})}}))}}if(void 0===l)return{result:l,missing:[{object:e,fieldName:s,tolerable:!1}]};Object(y.x)(l)&&(l=l.json);return{result:l}}(e,t,n.name.value,Object(y.b)(n,r),a,{resultKey:Object(y.E)(n),directives:Object(y.h)(n,r)});return Array.isArray(o.result)?this.combineExecResults(o,this.executeSubSelectedArray(n,o.result,i)):n.selectionSet?null==o.result?o:this.combineExecResults(o,this.executeSelectionSet({selectionSet:n.selectionSet,rootValue:o.result,execContext:i})):(Ae(n,o.result),o)},e.prototype.combineExecResults=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];var n=null;return e.forEach(function(e){e.missing&&(n=n||[]).push.apply(n,e.missing)}),{result:e.pop().result,missing:n}},e.prototype.executeSubSelectedArray=function(e,t,n){var i=this,r=null;function a(e){return e.missing&&(r=r||[]).push.apply(r,e.missing),e.result}return{result:t=t.map(function(t){return null===t?null:Array.isArray(t)?a(i.executeSubSelectedArray(e,t,n)):e.selectionSet?a(i.executeSelectionSet({selectionSet:e.selectionSet,rootValue:t,execContext:n})):(Ae(e,t),t)}),missing:r}},e}();function Ae(e,t){if(!e.selectionSet&&Object(y.v)(t))throw new B.a}function _e(){return!0}var Oe=function(){function e(e){void 0===e&&(e=Object.create(null)),this.data=e}return e.prototype.toObject=function(){return this.data},e.prototype.get=function(e){return this.data[e]},e.prototype.set=function(e,t){this.data[e]=t},e.prototype.delete=function(e){this.data[e]=void 0},e.prototype.clear=function(){this.data=Object.create(null)},e.prototype.replace=function(e){this.data=e||Object.create(null)},e}();var Pe=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.type="WriteError",t}return Object(v.c)(t,e),t}(Error);var Me=function(){function e(){}return e.prototype.writeQueryToStore=function(e){var t=e.query,n=e.result,i=e.store,r=void 0===i?Ce():i,a=e.variables,o=e.dataIdFromObject,s=e.fragmentMatcherFunction;return this.writeResultToStore({dataId:"ROOT_QUERY",result:n,document:t,store:r,variables:a,dataIdFromObject:o,fragmentMatcherFunction:s})},e.prototype.writeResultToStore=function(e){var t=e.dataId,n=e.result,i=e.document,r=e.store,a=void 0===r?Ce():r,o=e.variables,s=e.dataIdFromObject,l=e.fragmentMatcherFunction,c=Object(y.m)(i);try{return this.writeSelectionSetToStore({result:n,dataId:t,selectionSet:c.selectionSet,context:{store:a,processedData:{},variables:Object(y.c)({},Object(y.g)(c),o),dataIdFromObject:s,fragmentMap:Object(y.f)(Object(y.i)(i)),fragmentMatcherFunction:l}})}catch(e){throw function(e,t){var n=new Pe("Error writing result to store for query:\n "+JSON.stringify(t));return n.message+="\n"+e.message,n.stack=e.stack,n}(e,i)}},e.prototype.writeSelectionSetToStore=function(e){var t=this,n=e.result,i=e.dataId,r=e.selectionSet,a=e.context,o=a.variables,s=a.store,l=a.fragmentMap;return r.selections.forEach(function(e){if(Object(y.F)(e,o))if(Object(y.u)(e)){var r=Object(y.E)(e),s=n[r];if(void 0!==s)t.writeFieldToStore({dataId:i,value:s,field:e,context:a});else{var c=!1,u=!1;e.directives&&e.directives.length&&(c=e.directives.some(function(e){return e.name&&"defer"===e.name.value}),u=e.directives.some(function(e){return e.name&&"client"===e.name.value})),!c&&!u&&a.fragmentMatcherFunction}}else{var h=void 0;Object(y.w)(e)?h=e:(h=(l||{})[e.name.value],Object(B.b)(h));var d=!0;if(a.fragmentMatcherFunction&&h.typeCondition){var f=Object(y.H)({id:"self",typename:void 0}),p={store:new Oe({self:n}),cacheRedirects:{}},g=a.fragmentMatcherFunction(f,h.typeCondition.name.value,p);Object(y.y)(),d=!!g}d&&t.writeSelectionSetToStore({result:n,selectionSet:h.selectionSet,dataId:i,context:a})}}),s},e.prototype.writeFieldToStore=function(e){var t,n,i,r=e.field,a=e.value,o=e.dataId,s=e.context,l=s.variables,c=s.dataIdFromObject,u=s.store,h=Object(y.G)(r,l);if(r.selectionSet&&null!==a)if(Array.isArray(a)){var d=o+"."+h;n=this.processArrayValue(a,d,r.selectionSet,s)}else{var f=o+"."+h,p=!0;if(Ie(f)||(f="$"+f),c){var g=c(a);Object(B.b)(!g||!Ie(g)),(g||"number"==typeof g&&0===g)&&(f=g,p=!1)}De(f,r,s.processedData)||this.writeSelectionSetToStore({dataId:f,result:a,selectionSet:r.selectionSet,context:s});var m=a.__typename;n=Object(y.H)({id:f,typename:m},p);var b=(i=u.get(o))&&i[h];if(b!==n&&Object(y.v)(b)){var x=void 0!==b.typename,w=void 0!==m,k=x&&w&&b.typename!==m;Object(B.b)(!p||b.generated||k),Object(B.b)(!x||w),b.generated&&(k?p||u.delete(b.id):function e(t,n,i){if(t===n)return!1;var r=i.get(t);var a=i.get(n);var o=!1;Object.keys(r).forEach(function(t){var n=r[t],s=a[t];Object(y.v)(n)&&Ie(n.id)&&Object(y.v)(s)&&!Object(y.t)(n,s)&&e(n.id,s.id,i)&&(o=!0)});i.delete(t);var s=Object(v.a)({},r,a);if(Object(y.t)(s,a))return o;i.set(n,s);return!0}(b.id,n.id,u))}}else n=null!=a&&"object"==typeof a?{type:"json",json:a}:a;(i=u.get(o))&&Object(y.t)(n,i[h])||u.set(o,Object(v.a)({},i,((t={})[h]=n,t)))},e.prototype.processArrayValue=function(e,t,n,i){var r=this;return e.map(function(e,a){if(null===e)return null;var o=t+"."+a;if(Array.isArray(e))return r.processArrayValue(e,o,n,i);var s=!0;if(i.dataIdFromObject){var l=i.dataIdFromObject(e);l&&(o=l,s=!1)}return De(o,n,i.processedData)||r.writeSelectionSetToStore({dataId:o,result:e,selectionSet:n,context:i}),Object(y.H)({id:o,typename:e.__typename},s)})},e}();function Ie(e){return"$"===e[0]}function De(e,t,n){if(!n)return!1;if(n[e]){if(n[e].indexOf(t)>=0)return!0;n[e].push(t)}else n[e]=[t];return!1}var Ne={fragmentMatcher:new we,dataIdFromObject:function(e){if(e.__typename){if(void 0!==e.id)return e.__typename+":"+e.id;if(void 0!==e._id)return e.__typename+":"+e._id}return null},addTypename:!0,resultCaching:!0};var Le=Object.prototype.hasOwnProperty,Re=function(e){function t(t,n,i){var r=e.call(this,Object.create(null))||this;return r.optimisticId=t,r.parent=n,r.transaction=i,r}return Object(v.c)(t,e),t.prototype.toObject=function(){return Object(v.a)({},this.parent.toObject(),this.data)},t.prototype.get=function(e){return Le.call(this.data,e)?this.data[e]:this.parent.get(e)},t}(Oe),Fe=function(e){function t(t){void 0===t&&(t={});var n=e.call(this)||this;n.watches=new Set,n.typenameDocumentCache=new Map,n.cacheKeyRoot=new ke,n.silenceBroadcast=!1,n.config=Object(v.a)({},Ne,t),n.config.customResolvers&&(n.config.cacheRedirects=n.config.customResolvers),n.config.cacheResolvers&&(n.config.cacheRedirects=n.config.cacheResolvers),n.addTypename=n.config.addTypename,n.data=n.config.resultCaching?new Ee:new Oe,n.optimisticData=n.data,n.storeReader=new Te(n.cacheKeyRoot),n.storeWriter=new Me;var i=n,r=i.maybeBroadcastWatch;return n.maybeBroadcastWatch=Object(de.wrap)(function(e){return r.call(n,e)},{makeCacheKey:function(e){if(!e.optimistic&&!e.previousResult)return i.data instanceof Ee?i.cacheKeyRoot.lookup(e.query,JSON.stringify(e.variables)):void 0}}),n}return Object(v.c)(t,e),t.prototype.restore=function(e){return e&&this.data.replace(e),this},t.prototype.extract=function(e){return void 0===e&&(e=!1),(e?this.optimisticData:this.data).toObject()},t.prototype.read=function(e){return"string"==typeof e.rootId&&void 0===this.data.get(e.rootId)?null:this.storeReader.readQueryFromStore({store:e.optimistic?this.optimisticData:this.data,query:this.transformDocument(e.query),variables:e.variables,rootId:e.rootId,fragmentMatcherFunction:this.config.fragmentMatcher.match,previousResult:e.previousResult,config:this.config})},t.prototype.write=function(e){this.storeWriter.writeResultToStore({dataId:e.dataId,result:e.result,variables:e.variables,document:this.transformDocument(e.query),store:this.data,dataIdFromObject:this.config.dataIdFromObject,fragmentMatcherFunction:this.config.fragmentMatcher.match}),this.broadcastWatches()},t.prototype.diff=function(e){return this.storeReader.diffQueryAgainstStore({store:e.optimistic?this.optimisticData:this.data,query:this.transformDocument(e.query),variables:e.variables,returnPartialData:e.returnPartialData,previousResult:e.previousResult,fragmentMatcherFunction:this.config.fragmentMatcher.match,config:this.config})},t.prototype.watch=function(e){var t=this;return this.watches.add(e),function(){t.watches.delete(e)}},t.prototype.evict=function(e){throw new B.a},t.prototype.reset=function(){return this.data.clear(),this.broadcastWatches(),Promise.resolve()},t.prototype.removeOptimistic=function(e){for(var t=[],n=0,i=this.optimisticData;i instanceof Re;)i.optimisticId===e?++n:t.push(i),i=i.parent;if(n>0){for(this.optimisticData=i;t.length>0;){var r=t.pop();this.performTransaction(r.transaction,r.optimisticId)}this.broadcastWatches()}},t.prototype.performTransaction=function(e,t){var n=this.data,i=this.silenceBroadcast;this.silenceBroadcast=!0,"string"==typeof t&&(this.data=this.optimisticData=new Re(t,this.optimisticData,e));try{e(this)}finally{this.silenceBroadcast=i,this.data=n}this.broadcastWatches()},t.prototype.recordOptimisticTransaction=function(e,t){return this.performTransaction(e,t)},t.prototype.transformDocument=function(e){if(this.addTypename){var t=this.typenameDocumentCache.get(e);return t||(t=Object(y.a)(e),this.typenameDocumentCache.set(e,t),this.typenameDocumentCache.set(t,t)),t}return e},t.prototype.broadcastWatches=function(){var e=this;this.silenceBroadcast||this.watches.forEach(function(t){return e.maybeBroadcastWatch(t)})},t.prototype.maybeBroadcastWatch=function(e){e.callback(this.diff({query:e.query,variables:e.variables,previousResult:e.previousResult&&e.previousResult(),optimistic:e.optimistic}))},t}(he),je=n(42),ze={http:{includeQuery:!0,includeExtensions:!1},headers:{accept:"*/*","content-type":"application/json"},options:{method:"POST"}},Ye=function(e,t,n){var i=new Error(n);throw i.name="ServerError",i.response=e,i.statusCode=e.status,i.result=t,i},He=function(e,t){var n;try{n=JSON.stringify(e)}catch(e){var i=new je.a(2);throw i.parseError=e,i}return n},We=function(e){void 0===e&&(e={});var t=e.uri,n=void 0===t?"/graphql":t,i=e.fetch,r=e.includeExtensions,a=e.useGETForQueries,o=Object(v.e)(e,["uri","fetch","includeExtensions","useGETForQueries"]);!function(e){if(!e&&"undefined"==typeof fetch)throw new je.a(1)}(i),i||(i=fetch);var s={http:{includeExtensions:r},options:o.fetchOptions,credentials:o.credentials,headers:o.headers};return new H(function(e){var t=function(e,t){var n=e.getContext().uri;return n||("function"==typeof t?t(e):t||"/graphql")}(e,n),r=e.getContext(),o={};if(r.clientAwareness){var l=r.clientAwareness,c=l.name,u=l.version;c&&(o["apollographql-client-name"]=c),u&&(o["apollographql-client-version"]=u)}var h,d=Object(v.a)({},o,r.headers),f={http:r.http,options:r.fetchOptions,credentials:r.credentials,headers:d},p=function(e,t){for(var n=[],i=2;i<arguments.length;i++)n[i-2]=arguments[i];var r=Object(v.a)({},t.options,{headers:t.headers,credentials:t.credentials}),a=t.http;n.forEach(function(e){r=Object(v.a)({},r,e.options,{headers:Object(v.a)({},r.headers,e.headers)}),e.credentials&&(r.credentials=e.credentials),a=Object(v.a)({},a,e.http)});var o=e.operationName,s=e.extensions,l=e.variables,c=e.query,u={operationName:o,variables:l};return a.includeExtensions&&(u.extensions=s),a.includeQuery&&(u.query=E(c)),{options:r,body:u}}(e,ze,s,f),g=p.options,m=p.body;if(!g.signal){var y=function(){if("undefined"==typeof AbortController)return{controller:!1,signal:!1};var e=new AbortController;return{controller:e,signal:e.signal}}(),b=y.controller,w=y.signal;(h=b)&&(g.signal=w)}if(a&&!e.query.definitions.some(function(e){return"OperationDefinition"===e.kind&&"mutation"===e.operation})&&(g.method="GET"),"GET"===g.method){var k=function(e,t){var n=[],i=function(e,t){n.push(e+"="+encodeURIComponent(t))};"query"in t&&i("query",t.query);t.operationName&&i("operationName",t.operationName);if(t.variables){var r=void 0;try{r=He(t.variables,"Variables map")}catch(e){return{parseError:e}}i("variables",r)}if(t.extensions){var a=void 0;try{a=He(t.extensions,"Extensions map")}catch(e){return{parseError:e}}i("extensions",a)}var o="",s=e,l=e.indexOf("#");-1!==l&&(o=e.substr(l),s=e.substr(0,l));var c=-1===s.indexOf("?")?"?":"&";return{newURI:s+c+n.join("&")+o}}(t,m),S=k.newURI,C=k.parseError;if(C)return N(C);t=S}else try{g.body=He(m,"Payload")}catch(C){return N(C)}return new x(function(n){var r;return i(t,g).then(function(t){return e.setContext({response:t}),t}).then((r=e,function(e){return e.text().then(function(t){try{return JSON.parse(t)}catch(i){var n=i;return n.name="ServerParseError",n.response=e,n.statusCode=e.status,n.bodyText=t,Promise.reject(n)}}).then(function(t){return e.status>=300&&Ye(e,t,"Response not successful: Received status code "+e.status),Array.isArray(t)||t.hasOwnProperty("data")||t.hasOwnProperty("errors")||Ye(e,t,"Server response was missing for query '"+(Array.isArray(r)?r.map(function(e){return e.operationName}):r.operationName)+"'."),t})})).then(function(e){return n.next(e),n.complete(),e}).catch(function(e){"AbortError"!==e.name&&(e.result&&e.result.errors&&e.result.data&&n.next(e.result),n.error(e))}),function(){h&&h.abort()}})})};var Xe=function(e){function t(t){return e.call(this,We(t).request)||this}return Object(v.c)(t,e),t}(H);function Ve(e){return new H(function(t,n){return new x(function(i){var r,a,o;try{r=n(t).subscribe({next:function(r){r.errors&&(o=e({graphQLErrors:r.errors,response:r,operation:t,forward:n}))?a=o.subscribe({next:i.next.bind(i),error:i.error.bind(i),complete:i.complete.bind(i)}):i.next(r)},error:function(r){(o=e({operation:t,networkError:r,graphQLErrors:r&&r.result&&r.result.errors,forward:n}))?a=o.subscribe({next:i.next.bind(i),error:i.error.bind(i),complete:i.complete.bind(i)}):i.error(r)},complete:function(){o||i.complete.bind(i)()}})}catch(r){e({networkError:r,operation:t,forward:n}),i.error(r)}return function(){r&&r.unsubscribe(),a&&r.unsubscribe()}})})}!function(e){function t(t){var n=e.call(this)||this;return n.link=Ve(t),n}Object(v.c)(t,e),t.prototype.request=function(e,t){return this.link.request(e,t)}}(H);var Be=n(56),qe=n.n(Be),Ue=["request","uri","credentials","headers","fetch","fetchOptions","clientState","onError","cacheRedirects","cache","name","version","resolvers","typeDefs","fragmentMatcher"],Ge=function(e){function t(t){void 0===t&&(t={});t&&Object.keys(t).filter(function(e){return-1===Ue.indexOf(e)}).length;var n=t.request,i=t.uri,r=t.credentials,a=t.headers,o=t.fetch,s=t.fetchOptions,l=t.clientState,c=t.cacheRedirects,u=t.onError,h=t.name,d=t.version,f=t.resolvers,p=t.typeDefs,g=t.fragmentMatcher,m=t.cache;Object(B.b)(!m||!c),m||(m=c?new Fe({cacheRedirects:c}):new Fe);var v=Ve(u||function(e){var t=e.graphQLErrors;e.networkError;t&&t.map(function(e){e.message,e.locations,e.path;return!0})}),y=!!n&&new H(function(e,t){return new x(function(i){var r;return Promise.resolve(e).then(function(e){return n(e)}).then(function(){r=t(e).subscribe({next:i.next.bind(i),error:i.error.bind(i),complete:i.complete.bind(i)})}).catch(i.error.bind(i)),function(){r&&r.unsubscribe()}})}),b=new Xe({uri:i||"/graphql",fetch:o,fetchOptions:s||{},credentials:r||"same-origin",headers:a||{}}),w=H.from([v,y,b].filter(function(e){return!!e})),k=f,S=p,E=g;return l&&(l.defaults&&m.writeData({data:l.defaults}),k=l.resolvers,S=l.typeDefs,E=l.fragmentMatcher),e.call(this,{cache:m,link:w,name:h,version:d,resolvers:k,typeDefs:S,fragmentMatcher:E})||this}return Object(v.c)(t,e),t}(oe),Qe=n(0),$e=n.n(Qe),Ze=n(9),Ke=n.n(Ze),Je=n(61),et=n.n(Je),tt=n(4),nt=n(7),it=n(43),rt=n.n(it),at=(n(19),Qe.createContext?Object(Qe.createContext)(void 0):null),ot=function(e,t){function n(t){if(!t||!t.client)throw new nt.a;return e.children(t.client)}return at?Object(Qe.createElement)(at.Consumer,null,n):n(t)};ot.contextTypes={client:tt.object.isRequired},ot.propTypes={children:tt.func.isRequired};var st,lt=function(e){function t(t,n){var i=e.call(this,t,n)||this;return i.operations=new Map,Object(nt.b)(t.client),t.client.__operations_cache__||(t.client.__operations_cache__=i.operations),i}return Object(v.c)(t,e),t.prototype.getChildContext=function(){return{client:this.props.client,operations:this.props.client.__operations_cache__}},t.prototype.render=function(){return at?Object(Qe.createElement)(at.Provider,{value:this.getChildContext()},this.props.children):this.props.children},t.propTypes={client:tt.object.isRequired,children:tt.node.isRequired},t.childContextTypes={client:tt.object.isRequired,operations:tt.object},t}(Qe.Component);!function(e){e[e.Query=0]="Query",e[e.Mutation=1]="Mutation",e[e.Subscription=2]="Subscription"}(st||(st={}));var ct=new Map;function ut(e){var t,n,i=ct.get(e);if(i)return i;Object(nt.b)(!!e&&!!e.kind);var r=e.definitions.filter(function(e){return"FragmentDefinition"===e.kind}),a=e.definitions.filter(function(e){return"OperationDefinition"===e.kind&&"query"===e.operation}),o=e.definitions.filter(function(e){return"OperationDefinition"===e.kind&&"mutation"===e.operation}),s=e.definitions.filter(function(e){return"OperationDefinition"===e.kind&&"subscription"===e.operation});Object(nt.b)(!r.length||a.length||o.length||s.length),Object(nt.b)(a.length+o.length+s.length<=1),n=a.length?st.Query:st.Mutation,a.length||o.length||(n=st.Subscription);var l=a.length?a:o.length?o:s;Object(nt.b)(1===l.length);var c=l[0];t=c.variableDefinitions||[];var u={name:c.name&&"Name"===c.name.kind?c.name.value:"data",type:n,variables:t};return ct.set(e,u),u}function ht(e,t){var n=e.client||t.client;return Object(nt.b)(!!n),n}var dt=Object.prototype.hasOwnProperty;function ft(e,t){return e===t?0!==e||0!==t||1/e==1/t:e!=e&&t!=t}function pt(e){return null!==e&&"object"==typeof e}function gt(e,t){if(ft(e,t))return!0;if(!pt(e)||!pt(t))return!1;var n=Object.keys(e);return n.length===Object.keys(t).length&&n.every(function(n){return dt.call(t,n)&&ft(e[n],t[n])})}var mt=function(e){function t(t,n){var i=e.call(this,t,n)||this;return i.previousData={},i.hasMounted=!1,i.lastResult=null,i.startQuerySubscription=function(e){if(void 0===e&&(e=!1),e||(i.lastResult=i.queryObservable.getLastResult()),!i.querySubscription){var t=i.getQueryResult();i.querySubscription=i.queryObservable.subscribe({next:function(e){var n=e.loading,r=e.networkStatus,a=e.data;t&&7===t.networkStatus&&gt(t.data,a)?t=void 0:i.lastResult&&i.lastResult.loading===n&&i.lastResult.networkStatus===r&&gt(i.lastResult.data,a)||(t=void 0,i.lastResult&&(i.lastResult=i.queryObservable.getLastResult()),i.updateCurrentData())},error:function(e){if(i.lastResult||i.resubscribeToQuery(),!e.hasOwnProperty("graphQLErrors"))throw e;i.updateCurrentData()}})}},i.removeQuerySubscription=function(){i.querySubscription&&(i.lastResult=i.queryObservable.getLastResult(),i.querySubscription.unsubscribe(),delete i.querySubscription)},i.updateCurrentData=function(){i.handleErrorOrCompleted(),i.hasMounted&&i.forceUpdate()},i.handleErrorOrCompleted=function(){var e=i.queryObservable.currentResult(),t=e.data,n=e.loading,r=e.error,a=i.props,o=a.onCompleted,s=a.onError;!o||n||r?s&&!n&&r&&s(r):o(t)},i.getQueryResult=function(){var e,t={data:Object.create(null)};if(Object.assign(t,{variables:(e=i.queryObservable).variables,refetch:e.refetch.bind(e),fetchMore:e.fetchMore.bind(e),updateQuery:e.updateQuery.bind(e),startPolling:e.startPolling.bind(e),stopPolling:e.stopPolling.bind(e),subscribeToMore:e.subscribeToMore.bind(e)}),i.props.skip)t=Object(v.a)({},t,{data:void 0,error:void 0,loading:!1});else{var n=i.queryObservable.currentResult(),r=n.loading,a=n.partial,o=n.networkStatus,s=n.errors,l=n.error;s&&s.length>0&&(l=new Z({graphQLErrors:s}));var c=i.queryObservable.options.fetchPolicy;if(Object.assign(t,{loading:r,networkStatus:o,error:l}),r)Object.assign(t.data,i.previousData,n.data);else if(l)Object.assign(t,{data:(i.queryObservable.getLastResult()||{}).data});else if("no-cache"===c&&0===Object.keys(n.data).length)t.data=i.previousData;else{if(i.props.partialRefetch&&0===Object.keys(n.data).length&&a&&"cache-only"!==c)return Object.assign(t,{loading:!0,networkStatus:X.loading}),t.refetch(),t;Object.assign(t.data,n.data),i.previousData=n.data}}if(!i.querySubscription){var u=t.refetch;t.refetch=function(e){return i.querySubscription?u(e):new Promise(function(t,n){i.refetcherQueue={resolve:t,reject:n,args:e}})}}return t.client=i.client,t},i.client=ht(t,n),i.initializeQueryObservable(t),i}return Object(v.c)(t,e),t.prototype.fetchData=function(){if(this.props.skip)return!1;var e=this.props,t=(e.children,e.ssr),n=(e.displayName,e.skip,e.client,e.onCompleted,e.onError,e.partialRefetch,Object(v.e)(e,["children","ssr","displayName","skip","client","onCompleted","onError","partialRefetch"])),i=n.fetchPolicy;if(!1===t)return!1;"network-only"!==i&&"cache-and-network"!==i||(i="cache-first");var r=this.client.watchQuery(Object(v.a)({},n,{fetchPolicy:i}));return this.context&&this.context.renderPromises&&this.context.renderPromises.registerSSRObservable(this,r),!!this.queryObservable.currentResult().loading&&r.result()},t.prototype.componentDidMount=function(){if(this.hasMounted=!0,!this.props.skip&&(this.startQuerySubscription(!0),this.refetcherQueue)){var e=this.refetcherQueue,t=e.args,n=e.resolve,i=e.reject;this.queryObservable.refetch(t).then(n).catch(i)}},t.prototype.componentWillReceiveProps=function(e,t){if(!e.skip||this.props.skip){var n=ht(e,t);gt(this.props,e)&&this.client===n||(this.client!==n&&(this.client=n,this.removeQuerySubscription(),this.queryObservable=null,this.previousData={},this.updateQuery(e)),this.props.query!==e.query&&this.removeQuerySubscription(),this.updateQuery(e),e.skip||this.startQuerySubscription())}else this.removeQuerySubscription()},t.prototype.componentWillUnmount=function(){this.removeQuerySubscription(),this.hasMounted=!1},t.prototype.componentDidUpdate=function(e){(!rt()(e.query,this.props.query)||!rt()(e.variables,this.props.variables))&&this.handleErrorOrCompleted()},t.prototype.render=function(){var e=this,t=this.context,n=function(){return e.props.children(e.getQueryResult())};return t&&t.renderPromises?t.renderPromises.addQueryPromise(this,n):n()},t.prototype.extractOptsFromProps=function(e){this.operation=ut(e.query),Object(nt.b)(this.operation.type===st.Query);var t=e.displayName||"Query";return Object(v.a)({},e,{displayName:t,context:e.context||{},metadata:{reactComponent:{displayName:t}}})},t.prototype.initializeQueryObservable=function(e){var t=this.extractOptsFromProps(e);this.setOperations(t),this.context&&this.context.renderPromises&&(this.queryObservable=this.context.renderPromises.getSSRObservable(this)),this.queryObservable||(this.queryObservable=this.client.watchQuery(t))},t.prototype.setOperations=function(e){this.context.operations&&this.context.operations.set(this.operation.name,{query:e.query,variables:e.variables})},t.prototype.updateQuery=function(e){this.queryObservable?this.setOperations(e):this.initializeQueryObservable(e),this.queryObservable.setOptions(this.extractOptsFromProps(e)).catch(function(){return null})},t.prototype.resubscribeToQuery=function(){this.removeQuerySubscription();var e=this.queryObservable.getLastError(),t=this.queryObservable.getLastResult();this.queryObservable.resetLastResults(),this.startQuerySubscription(),Object.assign(this.queryObservable,{lastError:e,lastResult:t})},t.contextTypes={client:tt.object,operations:tt.object,renderPromises:tt.object},t.propTypes={client:tt.object,children:tt.func.isRequired,fetchPolicy:tt.string,notifyOnNetworkStatusChange:tt.bool,onCompleted:tt.func,onError:tt.func,pollInterval:tt.number,query:tt.object.isRequired,variables:tt.object,ssr:tt.bool,partialRefetch:tt.bool},t}(Qe.Component),vt={loading:!1,called:!1,error:void 0,data:void 0};(function(e){function t(t,n){var i=e.call(this,t,n)||this;return i.hasMounted=!1,i.runMutation=function(e){void 0===e&&(e={}),i.onMutationStart();var t=i.generateNewMutationId();return i.mutate(e).then(function(e){return i.onMutationCompleted(e,t),e}).catch(function(e){if(i.onMutationError(e,t),!i.props.onError)throw e})},i.mutate=function(e){var t=i.props,n=t.mutation,r=t.variables,a=t.optimisticResponse,o=t.update,s=t.context,l=void 0===s?{}:s,c=t.awaitRefetchQueries,u=void 0!==c&&c,h=t.fetchPolicy,d=Object(v.a)({},e),f=d.refetchQueries||i.props.refetchQueries;f&&f.length&&Array.isArray(f)&&(f=f.map(function(e){return"string"==typeof e&&i.context.operations&&i.context.operations.get(e)||e}),delete d.refetchQueries);var p=Object.assign({},r,d.variables);return delete d.variables,i.client.mutate(Object(v.a)({mutation:n,optimisticResponse:a,refetchQueries:f,awaitRefetchQueries:u,update:o,context:l,fetchPolicy:h,variables:p},d))},i.onMutationStart=function(){i.state.loading||i.props.ignoreResults||i.setState({loading:!0,error:void 0,data:void 0,called:!0})},i.onMutationCompleted=function(e,t){var n=i.props,r=n.onCompleted,a=n.ignoreResults,o=e.data,s=e.errors,l=s&&s.length>0?new Z({graphQLErrors:s}):void 0,c=function(){return r?r(o):null};i.hasMounted&&i.isMostRecentMutation(t)&&!a?i.setState({loading:!1,data:o,error:l},c):c()},i.onMutationError=function(e,t){var n=i.props.onError,r=function(){return n?n(e):null};i.hasMounted&&i.isMostRecentMutation(t)?i.setState({loading:!1,error:e},r):r()},i.generateNewMutationId=function(){return i.mostRecentMutationId=i.mostRecentMutationId+1,i.mostRecentMutationId},i.isMostRecentMutation=function(e){return i.mostRecentMutationId===e},i.verifyDocumentIsMutation=function(e){var t=ut(e);Object(nt.b)(t.type===st.Mutation)},i.client=ht(t,n),i.verifyDocumentIsMutation(t.mutation),i.mostRecentMutationId=0,i.state=vt,i}Object(v.c)(t,e),t.prototype.componentDidMount=function(){this.hasMounted=!0},t.prototype.componentWillUnmount=function(){this.hasMounted=!1},t.prototype.componentWillReceiveProps=function(e,t){var n=ht(e,t);gt(this.props,e)&&this.client===n||(this.props.mutation!==e.mutation&&this.verifyDocumentIsMutation(e.mutation),this.client!==n&&(this.client=n,this.setState(vt)))},t.prototype.render=function(){var e=this.props.children,t=this.state,n=t.loading,i=t.data,r=t.error,a={called:t.called,loading:n,data:i,error:r,client:this.client};return e(this.runMutation,a)},t.contextTypes={client:tt.object,operations:tt.object},t.propTypes={mutation:tt.object.isRequired,variables:tt.object,optimisticResponse:tt.object,refetchQueries:Object(tt.oneOfType)([Object(tt.arrayOf)(Object(tt.oneOfType)([tt.string,tt.object])),tt.func]),awaitRefetchQueries:tt.bool,update:tt.func,children:tt.func.isRequired,onCompleted:tt.func,onError:tt.func,fetchPolicy:tt.string}})(Qe.Component),function(e){function t(t,n){var i=e.call(this,t,n)||this;return i.initialize=function(e){i.queryObservable||(i.queryObservable=i.client.subscribe({query:e.subscription,variables:e.variables,fetchPolicy:e.fetchPolicy}))},i.startSubscription=function(){i.querySubscription||(i.querySubscription=i.queryObservable.subscribe({next:i.updateCurrentData,error:i.updateError,complete:i.completeSubscription}))},i.getInitialState=function(){return{loading:!0,error:void 0,data:void 0}},i.updateCurrentData=function(e){var t=i,n=t.client,r=t.props.onSubscriptionData;r&&r({client:n,subscriptionData:e}),i.setState({data:e.data,loading:!1,error:void 0})},i.updateError=function(e){i.setState({error:e,loading:!1})},i.completeSubscription=function(){var e=i.props.onSubscriptionComplete;e&&e(),i.endSubscription()},i.endSubscription=function(){i.querySubscription&&(i.querySubscription.unsubscribe(),delete i.querySubscription)},i.client=ht(t,n),i.initialize(t),i.state=i.getInitialState(),i}Object(v.c)(t,e),t.prototype.componentDidMount=function(){this.startSubscription()},t.prototype.componentWillReceiveProps=function(e,t){var n=ht(e,t);if(!gt(this.props.variables,e.variables)||this.client!==n||this.props.subscription!==e.subscription){var i=e.shouldResubscribe;"function"==typeof i&&(i=!!i(this.props,e));var r=!1===i;if(this.client!==n&&(this.client=n),!r)return this.endSubscription(),delete this.queryObservable,this.initialize(e),this.startSubscription(),void this.setState(this.getInitialState());this.initialize(e),this.startSubscription()}},t.prototype.componentWillUnmount=function(){this.endSubscription()},t.prototype.render=function(){var e=this.props.children;return e?e(Object.assign({},this.state,{variables:this.props.variables})):null},t.contextTypes={client:tt.object},t.propTypes={subscription:tt.object.isRequired,variables:tt.object,children:tt.func,onSubscriptionData:tt.func,onSubscriptionComplete:tt.func,shouldResubscribe:Object(tt.oneOfType)([tt.func,tt.bool])}}(Qe.Component);!function(e){function t(t){var n=e.call(this,t)||this;return n.withRef=!1,n.setWrappedInstance=n.setWrappedInstance.bind(n),n}Object(v.c)(t,e),t.prototype.getWrappedInstance=function(){return Object(nt.b)(this.withRef),this.wrappedInstance},t.prototype.setWrappedInstance=function(e){this.wrappedInstance=e}}(Qe.Component);!function(){function e(){this.queryPromises=new Map,this.queryInfoTrie=new Map}e.prototype.registerSSRObservable=function(e,t){this.lookupQueryInfo(e).observable=t},e.prototype.getSSRObservable=function(e){return this.lookupQueryInfo(e).observable},e.prototype.addQueryPromise=function(e,t){return this.lookupQueryInfo(e).seen?t():(this.queryPromises.set(e,new Promise(function(t){t(e.fetchData())})),null)},e.prototype.hasPromises=function(){return this.queryPromises.size>0},e.prototype.consumeAndAwaitPromises=function(){var e=this,t=[];return this.queryPromises.forEach(function(n,i){e.lookupQueryInfo(i).seen=!0,t.push(n)}),this.queryPromises.clear(),Promise.all(t)},e.prototype.lookupQueryInfo=function(e){var t=this.queryInfoTrie,n=e.props,i=n.query,r=n.variables,a=t.get(i)||new Map;t.has(i)||t.set(i,a);var o=JSON.stringify(r),s=a.get(o)||{seen:!1,observable:null};return a.has(o)||a.set(o,s),s}}();function yt(){var e=m(["\n  query getAnalytics($span: Int!) {\n    analytics(span: $span) {\n      users {\n        ...data\n      }\n      userDeletions {\n        ...data\n      }\n      threads {\n        ...data\n      }\n      posts {\n        ...data\n      }\n      attachments {\n        ...data\n      }\n      dataDownloads {\n        ...data\n      }\n    }\n  }\n\n  fragment data on AnalyticsData {\n    current\n    currentCumulative\n    previous\n    previousCumulative\n  }\n"]);return yt=function(){return e},e}var bt=qe()(yt()),xt=function(e){function t(){var e,n;s(this,t);for(var i=arguments.length,r=new Array(i),a=0;a<i;a++)r[a]=arguments[a];return(n=d(this,(e=f(t)).call.apply(e,[this].concat(r)))).state={span:30},n.setSpan=function(e){n.setState({span:e})},n}return g(t,$e.a.Component),c(t,[{key:"render",value:function(){var e=this.props,t=e.errorMessage,n=e.labels,i=e.title,r=this.state.span;return $e.a.createElement("div",{className:"card card-admin-info"},$e.a.createElement("div",{className:"card-body"},$e.a.createElement("div",{className:"row align-items-center"},$e.a.createElement("div",{className:"col"},$e.a.createElement("h4",{className:"card-title"},i)),$e.a.createElement("div",{className:"col-auto"},$e.a.createElement(wt,{span:r,setSpan:this.setSpan})))),$e.a.createElement(mt,{query:bt,variables:{span:r}},function(e){var i=e.loading,a=e.error,o=e.data;if(i)return $e.a.createElement(kt,null);if(a)return $e.a.createElement(St,{message:t});var s=o.analytics;return $e.a.createElement($e.a.Fragment,null,$e.a.createElement(Et,{data:s.threads,name:n.threads,span:r}),$e.a.createElement(Et,{data:s.posts,name:n.posts,span:r}),$e.a.createElement(Et,{data:s.attachments,name:n.attachments,span:r}),$e.a.createElement(Et,{data:s.users,name:n.users,span:r}),$e.a.createElement(Et,{data:s.userDeletions,name:n.userDeletions,negative:!0,span:r}),$e.a.createElement(Et,{data:s.dataDownloads,name:n.dataDownloads,span:r}))}))}}]),t}(),wt=function(e){var t=e.span,n=e.setSpan;return $e.a.createElement("div",null,[30,90,180,360].map(function(e){return $e.a.createElement("button",{key:e,className:e===t?"btn btn-primary btn-sm ml-3":"btn btn-light btn-sm ml-3",type:"button",onClick:function(){return n(e)}},e)}))},kt=function(){return $e.a.createElement("div",{className:"card-body border-top"},$e.a.createElement("div",{className:"text-center py-5"},$e.a.createElement("div",{className:"spinner-border text-light",role:"status"},$e.a.createElement("span",{className:"sr-only"},"Loading..."))))},St=function(e){var t=e.message;return $e.a.createElement("div",{className:"card-body border-top"},$e.a.createElement("div",{className:"text-center py-5"},t))},Et=function(e){var t=e.data,n=(e.legend,e.name),i=e.negative,r=e.span,a={legend:{show:!1},chart:{animations:{enabled:!1},parentHeightOffset:0,toolbar:{show:!1}},colors:["#6554c0","#b3d4ff"],grid:{padding:{top:0}},stroke:{width:2},tooltip:{x:{show:!1},y:{title:{formatter:function(e,t){var n=t.dataPointIndex,i=o()();return"P"===e&&i.subtract(r,"days"),i.subtract(r-n-1,"days"),i.format("ll")}},formatter:function(e,n){var i=n.dataPointIndex;return 0===n.seriesIndex?t.current[i]:t.previous[i]}}},xaxis:{axisBorder:{show:!1},axisTicks:{show:!1},labels:{show:!1},categories:[],tooltip:{enabled:!1}},yaxis:{tickAmount:2,max:function(e){return e||1},show:!1}},s=[{name:"C",data:t.currentCumulative},{name:"P",data:t.previousCumulative}];return $e.a.createElement("div",{className:"card-body border-top pb-1"},$e.a.createElement("h5",{className:"m-0"},n),$e.a.createElement("div",{className:"row align-items-center"},$e.a.createElement("div",{className:"col-auto"},$e.a.createElement(Ct,{data:t,negative:i})),$e.a.createElement("div",{className:"col"},$e.a.createElement(Tt,null,function(e){var t=e.width;return t>1&&$e.a.createElement(et.a,{options:a,series:s,type:"line",width:t,height:140})}))))},Ct=function(e){var t=e.data,n=e.negative,i=t.current.reduce(function(e,t){return e+t}),r=i-t.previous.reduce(function(e,t){return e+t}),a="text-light",o="fas fa-equals";return n?(r>0&&(a="text-danger"),r<0&&(a="text-success")):(r>0&&(a="text-success"),r<0&&(a="text-danger")),r>0&&(o="fas fa-chevron-up"),r<0&&(o="fas fa-chevron-down"),$e.a.createElement("div",{className:"card-admin-analytics-summary"},$e.a.createElement("div",null,i),$e.a.createElement("small",{className:a},$e.a.createElement("span",{className:o})," ",Math.abs(r)))},Tt=function(e){function t(){var e,n;s(this,t);for(var i=arguments.length,r=new Array(i),a=0;a<i;a++)r[a]=arguments[a];return(n=d(this,(e=f(t)).call.apply(e,[this].concat(r)))).state={width:1,height:1},n.element=$e.a.createRef(),n.updateSize=function(){n.setState({width:n.element.current.clientWidth,height:n.element.current.clientHeight})},n}return g(t,$e.a.Component),c(t,[{key:"componentDidMount",value:function(){this.timer=window.setInterval(this.updateSize,3e3),this.updateSize()}},{key:"componentWillUnmount",value:function(){window.clearInterval(this.timer)}},{key:"render",value:function(){return $e.a.createElement("div",{className:"card-admin-analytics-chart",ref:this.element},this.props.children(this.state))}}]),t}(),At=function(e){var t=e.elementId,n=e.errorMessage,i=e.labels,r=e.title,a=e.uri,o=document.getElementById(t);o||console.error("Element with id "+o+"doesn't exist!");var s=new Ge({credentials:"same-origin",uri:a});Ke.a.render($e.a.createElement(lt,{client:s},$e.a.createElement(xt,{errorMessage:n,labels:i,title:r})),o)},_t=function(e){function t(){var e,n;s(this,t);for(var i=arguments.length,r=new Array(i),a=0;a<i;a++)r[a]=arguments[a];return(n=d(this,(e=f(t)).call.apply(e,[this].concat(r)))).state={value:n.props.value},n.onChange=function(e){var t=e.target;n.setState({value:t.value})},n}return g(t,$e.a.Component),c(t,[{key:"render",value:function(){return $e.a.createElement("div",{className:"row"},$e.a.createElement("div",{className:"col-auto pr-0"},$e.a.createElement("input",{type:"color",className:"form-control",style:{width:"48px"},value:Pt(this.state.value),onChange:this.onChange})),$e.a.createElement("div",{className:"col"},$e.a.createElement("input",{type:"text",className:"form-control",name:this.props.name,value:this.state.value,onChange:this.onChange})))}}]),t}(),Ot=/^#[0-9a-fA-F]{6}$/,Pt=function(e){return Ot.test(e)?e:"#ffffff"},Mt=function(e){var t=e.elementId,n=document.getElementById(t);n||console.error("Element with id "+n+"doesn't exist!");var i=n.name,r=n.value,a=document.createElement("div");n.parentNode.insertBefore(a,n),n.remove(),Ke.a.render($e.a.createElement(_t,{name:i,value:r}),a)},It=(n(16),function(e,t){var n=document.querySelectorAll(e),i=function(e){if(!window.confirm(t))return e.preventDefault(),!1};n.forEach(function(e){var t="form"===e.tagName.toLowerCase()?"submit":"click";e.addEventListener(t,i)})}),Dt=(n(110),function(e){function t(){var e,n;s(this,t);for(var i=arguments.length,r=new Array(i),a=0;a<i;a++)r[a]=arguments[a];return(n=d(this,(e=f(t)).call.apply(e,[this].concat(r)))).state={defaultValue:n.props.value,value:n.props.value},n.setNever=function(){n.setState({value:null})},n.setInitialValue=function(){n.setState(function(e){var t=e.defaultValue;e.value;if(t)return{value:t};var n=o()();return n.add(1,"hour"),{value:n}})},n.setValue=function(e){n.setState({value:e})},n}return g(t,$e.a.Component),c(t,[{key:"render",value:function(){var e=this.props,t=e.name,n=e.never,i=e.setDate,r=this.state.value;return $e.a.createElement("div",{onBlur:this.handleBlur,onFocus:this.handleFocus},$e.a.createElement("input",{type:"hidden",name:t,value:r?r.format():""}),$e.a.createElement("div",null,$e.a.createElement("button",{className:Nt(null===r),type:"button",onClick:this.setNever},n),$e.a.createElement("button",{className:Nt(null!==r)+" ml-3",type:"button",onClick:this.setInitialValue},r?r.format("L LT"):i)),$e.a.createElement(Lt,{value:r,onChange:this.setValue}))}}]),t}()),Nt=function(e){return e?"btn btn-outline-primary btn-sm":"btn btn-outline-secondary btn-sm"},Lt=function(e){var t=e.value,n=e.onChange;return t?$e.a.createElement("div",{className:"row mt-3"},$e.a.createElement("div",{className:"col-auto"},$e.a.createElement(jt,{value:t,onChange:n})),$e.a.createElement("div",{className:"col-auto"},$e.a.createElement(Wt,{value:t,onChange:n}))):null},Rt=[1,2,3,4,5,6],Ft=[1,2,3,4,5,6,7],jt=function(e){function t(){var e,n;s(this,t);for(var i=arguments.length,r=new Array(i),a=0;a<i;a++)r[a]=arguments[a];return(n=d(this,(e=f(t)).call.apply(e,[this].concat(r)))).decreaseMonth=function(){n.setState(function(e,t){var n=t.value.clone();n.subtract(1,"month"),t.onChange(n)})},n.increaseMonth=function(){n.setState(function(e,t){var n=t.value.clone();n.add(1,"month"),t.onChange(n)})},n}return g(t,$e.a.Component),c(t,[{key:"render",value:function(){var e=this.props,t=e.value,n=e.onChange,i=t.clone().startOf("month").isoWeekday(),r=t.clone();return r.date(1),r.hour(t.hour()),r.minute(t.minute()),r.subtract(i+1,"day"),$e.a.createElement("div",{className:"control-month-picker"},$e.a.createElement(zt,{decreaseMonth:this.decreaseMonth,increaseMonth:this.increaseMonth,value:t}),$e.a.createElement(Yt,null),Rt.map(function(e){return $e.a.createElement("div",{className:"row align-items-center m-0",key:e},Ft.map(function(e){return $e.a.createElement(Ht,{calendar:r,key:e,value:t,onSelect:n})}))}))}}]),t}(),zt=function(e){var t=e.decreaseMonth,n=e.increaseMonth,i=e.value;return $e.a.createElement("div",{className:"row align-items-center"},$e.a.createElement("div",{className:"col-auto text-center"},$e.a.createElement("button",{className:"btn btn-block py-1 px-3",type:"button",onClick:t},$e.a.createElement("span",{className:"fas fa-chevron-left"}))),$e.a.createElement("div",{className:"col text-center font-weight-bold"},i.format("MMMM YYYY")),$e.a.createElement("div",{className:"col-auto text-center"},$e.a.createElement("button",{className:"btn btn-block py-1 px-3",type:"button",onClick:n},$e.a.createElement("span",{className:"fas fa-chevron-right"}))))},Yt=function(){return $e.a.createElement("div",{className:"row align-items-center m-0"},o.a.weekdaysMin(!1).map(function(e,t){return $e.a.createElement("div",{className:"col text-center px-1 "+(0===t?"text-danger":"text-muted"),key:e},e)}))},Ht=function(e){var t=e.calendar,n=e.value,i=e.onSelect;t.add(1,"day");var r=t.clone(),a=r.format("D M Y")===n.format("D M Y");return $e.a.createElement("div",{className:"col text-center px-1"},$e.a.createElement("button",{className:"btn btn-sm btn-block px-0"+(a?" btn-primary":""),type:"button",onClick:function(){return i(r)},disabled:r.month()!==n.month()},r.format("D")))},Wt=function(e){function t(){var e,n;s(this,t);for(var i=arguments.length,r=new Array(i),a=0;a<i;a++)r[a]=arguments[a];return(n=d(this,(e=f(t)).call.apply(e,[this].concat(r)))).handleHourChange=function(e){var t=e.target.value;t.match(/^[0-2][0-9]?[0-9]?$/)&&n.setState(function(e,n){var i=Xt(t,2),r=n.value.clone();r.hour(i),n.onChange(r)})},n.handleMinuteChange=function(e){var t=e.target.value;t.match(/^[0-5][0-9]?[0-9]?$/)&&n.setState(function(e,n){var i=Xt(t,5),r=n.value.clone();r.minute(i),n.onChange(r)})},n}return g(t,$e.a.Component),c(t,[{key:"render",value:function(){return $e.a.createElement("div",{className:"control-time-picker"},$e.a.createElement("div",{className:"row align-items-center m-0"},$e.a.createElement("div",{className:"col px-0"},$e.a.createElement(Vt,{format:"HH",value:this.props.value,onChange:this.handleHourChange})),$e.a.createElement("div",{className:"col-auto px-0"},$e.a.createElement("span",null,":")),$e.a.createElement("div",{className:"col px-0"},$e.a.createElement(Vt,{format:"mm",value:this.props.value,onChange:this.handleMinuteChange}))))}}]),t}(),Xt=function(e,t){var n=e;return 3===n.length&&(n=n.substring(1,3),parseInt(n[0])>t&&(n=t+""+n[1])),n},Vt=function(e){var t=e.format,n=e.value,i=e.onChange;return $e.a.createElement("input",{className:"form-control text-center",placeholder:"00",type:"text",value:n.format(t),onChange:i})},Bt=function(e){var t=e.elementId,n=e.never,i=e.setDate,r=document.getElementById(t);r||console.error("Element with id "+r+"doesn't exist!"),r.type="hidden";var a=r.name,s=r.value.length?o()(r.value):null;s&&s.local();var l=document.createElement("div");r.parentNode.insertBefore(l,r),r.remove(),Ke.a.render($e.a.createElement(Dt,{name:a,never:n,value:s,setDate:i}),l)},qt=function(e,t,n){document.querySelectorAll(e).forEach(function(e){e.addEventListener(t,n)})},Ut=function(e,t){var n=document.querySelector("#mass-action .dropdown-toggle"),i=n.querySelector("span:last-child"),r=function(){var r=document.querySelectorAll(".row-select input:checked");n.disabled=0===r.length,r.length?i.textContent=t.replace("0",r.length):i.textContent=e};r(),qt(".row-select input","change",function(){r()}),qt("#mass-action [data-confirmation]","click",function(e){if(!window.confirm(e.target.dataset.confirmation))return e.preventDefault(),!1})},Gt=function(e,t){var n=e.querySelector("form");if(null!==n){var i=n.querySelector("button"),r=e.querySelector("th input[type=checkbox]"),a=e.querySelectorAll("td input[type=checkbox]"),o=function(){var t=e.querySelectorAll("td input:checked");r.checked=a.length===t.length,i.disabled=0===t.length};o(),r.addEventListener("change",function(e){a.forEach(function(t){return t.checked=e.target.checked}),o()}),a.forEach(function(e){e.addEventListener("change",o)}),n.addEventListener("submit",function(n){if(0===e.querySelectorAll("td input:checked").length||!window.confirm(t))return n.preventDefault(),!1})}},Qt=function(e){document.querySelectorAll(".card-admin-table").forEach(function(t){return Gt(t,e)})},$t=function(e){var t=o()(e.dataset.timestamp);e.title=t.format("LLLL"),r()(e).tooltip()},Zt=function(e){var t=o()();e.forEach(function(e){Kt(e,t)})},Kt=function(e,t){var n=o()(e.dataset.timestamp);if(Math.abs(n.diff(t,"seconds"))<21600)e.textContent=n.from(t);else{var i=Math.abs(n.diff(t,"days"));e.textContent=i<5?n.calendar(t):n.format(e.dataset.format)}},Jt=function(){var e=document.querySelectorAll("[data-timestamp]");e.forEach($t),Zt(e),window.setInterval(function(){Zt(e)},2e4)},en=function(){r()('[data-tooltip="top"]').tooltip({placement:"top"}),r()('[data-tooltip="bottom"]').tooltip({placement:"bottom"})},tn=function(){document.querySelectorAll(".form-group.has-error").forEach(function(e){e.querySelectorAll(".form-control").forEach(function(e){e.classList.add("is-invalid")})})};function nn(){var e=m(["\n  query getVersion {\n    version {\n      status\n      message\n      description\n    }\n  }\n"]);return nn=function(){return e},e}var rn=qe()(nn()),an=function(e){var t=e.errorMessage,n=e.loadingMessage;return $e.a.createElement(mt,{query:rn},function(e){var i=e.loading,r=e.error,a=e.data;return i?$e.a.createElement(on,n):r?$e.a.createElement(sn,t):$e.a.createElement(ln,a.version)})},on=function(e){var t=e.description,n=e.message;return $e.a.createElement("div",{className:"media media-admin-check"},$e.a.createElement("div",{className:"media-check-icon"},$e.a.createElement("div",{className:"spinner-border",role:"status"},$e.a.createElement("span",{className:"sr-only"},"Loading..."))),$e.a.createElement("div",{className:"media-body"},$e.a.createElement("h5",null,n),t))},sn=function(e){var t=e.description,n=e.message;return $e.a.createElement("div",{className:"media media-admin-check"},$e.a.createElement("div",{className:"media-check-icon media-check-icon-danger"},$e.a.createElement("span",{className:"fas fa-times"})),$e.a.createElement("div",{className:"media-body"},$e.a.createElement("h5",null,n),t))},ln=function(e){var t=e.description,n=e.message,i=e.status;return $e.a.createElement("div",{className:"media media-admin-check"},$e.a.createElement(cn,{status:i}),$e.a.createElement("div",{className:"media-body"},$e.a.createElement("h5",null,n),t))},cn=function(e){var t=e.status,n="media-check-icon media-check-icon-";return"SUCCESS"===t&&(n+="success"),"WARNING"===t&&(n+="warning"),"ERROR"===t&&(n+="danger"),$e.a.createElement("div",{className:n},$e.a.createElement(un,{status:t}))},un=function(e){var t=e.status;return"SUCCESS"===t?$e.a.createElement("span",{className:"fas fa-check"}):"WARNING"===t?$e.a.createElement("span",{className:"fas fa-question"}):"ERROR"===t?$e.a.createElement("span",{className:"fas fa-times"}):null},hn=function(e){var t=e.elementId,n=e.errorMessage,i=e.loadingMessage,r=e.uri,a=document.getElementById(t);a||console.error("Element with id "+a+"doesn't exist!");var o=new Ge({credentials:"same-origin",uri:r});Ke.a.render($e.a.createElement(lt,{client:o},$e.a.createElement(an,{errorMessage:n,loadingMessage:i})),a)};window.moment=o.a,window.misago={initAnalytics:At,initColorpicker:Mt,initConfirmation:It,initDatepicker:Bt,initMassActions:Ut,initMassDelete:Qt,initVersionCheck:hn,init:function(){var e=document.querySelector("html").lang;o.a.locale(e.replace("_","-").toLowerCase()),en(),Jt(),tn()}}},function(e,t,n){"use strict";n.r(t);var i=n(26),r=n(17);function a(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:e.prototype.toString;e.prototype.toJSON=t,e.prototype.inspect=t,r.a&&(e.prototype[r.a]=t)}function o(e,t){if(!e)throw new Error(t)}var s,l=function(e,t,n){this.body=e,this.name=t||"GraphQL request",this.locationOffset=n||{line:1,column:1},this.locationOffset.line>0||o(0,"line in locationOffset is 1-indexed and must be positive"),this.locationOffset.column>0||o(0,"column in locationOffset is 1-indexed and must be positive")};function c(e,t){for(var n,i=/\r\n|[\n\r]/g,r=1,a=t+1;(n=i.exec(e.body))&&n.index<t;)r+=1,a=t+1-(n.index+n[0].length);return{line:r,column:a}}function u(e,t){var n=e.locationOffset.column-1,i=h(n)+e.body,r=t.line-1,a=e.locationOffset.line-1,o=t.line+a,s=1===t.line?n:0,l=t.column+s,c=i.split(/\r\n|[\n\r]/g);return"".concat(e.name," (").concat(o,":").concat(l,")\n")+function(e){var t=e.filter(function(e){e[0];var t=e[1];return void 0!==t}),n=0,i=!0,r=!1,a=void 0;try{for(var o,s=t[Symbol.iterator]();!(i=(o=s.next()).done);i=!0){var l=o.value,c=l[0];n=Math.max(n,c.length)}}catch(e){r=!0,a=e}finally{try{i||null==s.return||s.return()}finally{if(r)throw a}}return t.map(function(e){var t,i=e[0],r=e[1];return h(n-(t=i).length)+t+r}).join("\n")}([["".concat(o-1,": "),c[r-1]],["".concat(o,": "),c[r]],["",h(l-1)+"^"],["".concat(o+1,": "),c[r+1]]])}function h(e){return Array(e+1).join(" ")}function d(e,t,n,i,r,a,o){var s=Array.isArray(t)?0!==t.length?t:void 0:t?[t]:void 0,l=n;if(!l&&s){var u=s[0];l=u&&u.loc&&u.loc.source}var h,f=i;!f&&s&&(f=s.reduce(function(e,t){return t.loc&&e.push(t.loc.start),e},[])),f&&0===f.length&&(f=void 0),i&&n?h=i.map(function(e){return c(n,e)}):s&&(h=s.reduce(function(e,t){return t.loc&&e.push(c(t.loc.source,t.loc.start)),e},[]));var p=o||a&&a.extensions;Object.defineProperties(this,{message:{value:e,enumerable:!0,writable:!0},locations:{value:h||void 0,enumerable:Boolean(h)},path:{value:r||void 0,enumerable:Boolean(r)},nodes:{value:s||void 0},source:{value:l||void 0},positions:{value:f||void 0},originalError:{value:a},extensions:{value:p||void 0,enumerable:Boolean(p)}}),a&&a.stack?Object.defineProperty(this,"stack",{value:a.stack,writable:!0,configurable:!0}):Error.captureStackTrace?Error.captureStackTrace(this,d):Object.defineProperty(this,"stack",{value:Error().stack,writable:!0,configurable:!0})}function f(e,t,n){return new d("Syntax Error: ".concat(n),void 0,e,[t])}s=l,"function"==typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(s.prototype,Symbol.toStringTag,{get:function(){return this.constructor.name}}),d.prototype=Object.create(Error.prototype,{constructor:{value:d},name:{value:"GraphQLError"},toString:{value:function(){return function(e){var t=[];if(e.nodes){var n=!0,i=!1,r=void 0;try{for(var a,o=e.nodes[Symbol.iterator]();!(n=(a=o.next()).done);n=!0){var s=a.value;s.loc&&t.push(u(s.loc.source,c(s.loc.source,s.loc.start)))}}catch(e){i=!0,r=e}finally{try{n||null==o.return||o.return()}finally{if(i)throw r}}}else if(e.source&&e.locations){var l=e.source,h=!0,d=!1,f=void 0;try{for(var p,g=e.locations[Symbol.iterator]();!(h=(p=g.next()).done);h=!0){var m=p.value;t.push(u(l,m))}}catch(e){d=!0,f=e}finally{try{h||null==g.return||g.return()}finally{if(d)throw f}}}return 0===t.length?e.message:[e.message].concat(t).join("\n\n")+"\n"}(this)}}});var p=n(27);function g(e,t){var n=new x(y.SOF,0,0,0,0,null);return{source:e,options:t,lastToken:n,token:n,line:1,lineStart:0,advance:m,lookahead:v}}function m(){return this.lastToken=this.token,this.token=this.lookahead()}function v(){var e=this.token;if(e.kind!==y.EOF)do{e=e.next||(e.next=k(this,e))}while(e.kind===y.COMMENT);return e}var y=Object.freeze({SOF:"<SOF>",EOF:"<EOF>",BANG:"!",DOLLAR:"$",AMP:"&",PAREN_L:"(",PAREN_R:")",SPREAD:"...",COLON:":",EQUALS:"=",AT:"@",BRACKET_L:"[",BRACKET_R:"]",BRACE_L:"{",PIPE:"|",BRACE_R:"}",NAME:"Name",INT:"Int",FLOAT:"Float",STRING:"String",BLOCK_STRING:"BlockString",COMMENT:"Comment"});function b(e){var t=e.value;return t?"".concat(e.kind,' "').concat(t,'"'):e.kind}function x(e,t,n,i,r,a,o){this.kind=e,this.start=t,this.end=n,this.line=i,this.column=r,this.value=o,this.prev=a,this.next=null}function w(e){return isNaN(e)?y.EOF:e<127?JSON.stringify(String.fromCharCode(e)):'"\\u'.concat(("00"+e.toString(16).toUpperCase()).slice(-4),'"')}function k(e,t){var n=e.source,i=n.body,r=i.length,a=function(e,t,n){var i=e.length,r=t;for(;r<i;){var a=e.charCodeAt(r);if(9===a||32===a||44===a||65279===a)++r;else if(10===a)++r,++n.line,n.lineStart=r;else{if(13!==a)break;10===e.charCodeAt(r+1)?r+=2:++r,++n.line,n.lineStart=r}}return r}(i,t.end,e),o=e.line,s=1+a-e.lineStart;if(a>=r)return new x(y.EOF,r,r,o,s,t);var l=i.charCodeAt(a);switch(l){case 33:return new x(y.BANG,a,a+1,o,s,t);case 35:return function(e,t,n,i,r){var a,o=e.body,s=t;do{a=o.charCodeAt(++s)}while(!isNaN(a)&&(a>31||9===a));return new x(y.COMMENT,t,s,n,i,r,o.slice(t+1,s))}(n,a,o,s,t);case 36:return new x(y.DOLLAR,a,a+1,o,s,t);case 38:return new x(y.AMP,a,a+1,o,s,t);case 40:return new x(y.PAREN_L,a,a+1,o,s,t);case 41:return new x(y.PAREN_R,a,a+1,o,s,t);case 46:if(46===i.charCodeAt(a+1)&&46===i.charCodeAt(a+2))return new x(y.SPREAD,a,a+3,o,s,t);break;case 58:return new x(y.COLON,a,a+1,o,s,t);case 61:return new x(y.EQUALS,a,a+1,o,s,t);case 64:return new x(y.AT,a,a+1,o,s,t);case 91:return new x(y.BRACKET_L,a,a+1,o,s,t);case 93:return new x(y.BRACKET_R,a,a+1,o,s,t);case 123:return new x(y.BRACE_L,a,a+1,o,s,t);case 124:return new x(y.PIPE,a,a+1,o,s,t);case 125:return new x(y.BRACE_R,a,a+1,o,s,t);case 65:case 66:case 67:case 68:case 69:case 70:case 71:case 72:case 73:case 74:case 75:case 76:case 77:case 78:case 79:case 80:case 81:case 82:case 83:case 84:case 85:case 86:case 87:case 88:case 89:case 90:case 95:case 97:case 98:case 99:case 100:case 101:case 102:case 103:case 104:case 105:case 106:case 107:case 108:case 109:case 110:case 111:case 112:case 113:case 114:case 115:case 116:case 117:case 118:case 119:case 120:case 121:case 122:return function(e,t,n,i,r){var a=e.body,o=a.length,s=t+1,l=0;for(;s!==o&&!isNaN(l=a.charCodeAt(s))&&(95===l||l>=48&&l<=57||l>=65&&l<=90||l>=97&&l<=122);)++s;return new x(y.NAME,t,s,n,i,r,a.slice(t,s))}(n,a,o,s,t);case 45:case 48:case 49:case 50:case 51:case 52:case 53:case 54:case 55:case 56:case 57:return function(e,t,n,i,r,a){var o=e.body,s=n,l=t,c=!1;45===s&&(s=o.charCodeAt(++l));if(48===s){if((s=o.charCodeAt(++l))>=48&&s<=57)throw f(e,l,"Invalid number, unexpected digit after 0: ".concat(w(s),"."))}else l=S(e,l,s),s=o.charCodeAt(l);46===s&&(c=!0,s=o.charCodeAt(++l),l=S(e,l,s),s=o.charCodeAt(l));69!==s&&101!==s||(c=!0,43!==(s=o.charCodeAt(++l))&&45!==s||(s=o.charCodeAt(++l)),l=S(e,l,s));return new x(c?y.FLOAT:y.INT,t,l,i,r,a,o.slice(t,l))}(n,a,l,o,s,t);case 34:return 34===i.charCodeAt(a+1)&&34===i.charCodeAt(a+2)?function(e,t,n,i,r,a){var o=e.body,s=t+3,l=s,c=0,u="";for(;s<o.length&&!isNaN(c=o.charCodeAt(s));){if(34===c&&34===o.charCodeAt(s+1)&&34===o.charCodeAt(s+2))return u+=o.slice(l,s),new x(y.BLOCK_STRING,t,s+3,n,i,r,Object(p.a)(u));if(c<32&&9!==c&&10!==c&&13!==c)throw f(e,s,"Invalid character within String: ".concat(w(c),"."));10===c?(++s,++a.line,a.lineStart=s):13===c?(10===o.charCodeAt(s+1)?s+=2:++s,++a.line,a.lineStart=s):92===c&&34===o.charCodeAt(s+1)&&34===o.charCodeAt(s+2)&&34===o.charCodeAt(s+3)?(u+=o.slice(l,s)+'"""',l=s+=4):++s}throw f(e,s,"Unterminated string.")}(n,a,o,s,t,e):function(e,t,n,i,r){var a=e.body,o=t+1,s=o,l=0,c="";for(;o<a.length&&!isNaN(l=a.charCodeAt(o))&&10!==l&&13!==l;){if(34===l)return c+=a.slice(s,o),new x(y.STRING,t,o+1,n,i,r,c);if(l<32&&9!==l)throw f(e,o,"Invalid character within String: ".concat(w(l),"."));if(++o,92===l){switch(c+=a.slice(s,o-1),l=a.charCodeAt(o)){case 34:c+='"';break;case 47:c+="/";break;case 92:c+="\\";break;case 98:c+="\b";break;case 102:c+="\f";break;case 110:c+="\n";break;case 114:c+="\r";break;case 116:c+="\t";break;case 117:var u=(h=a.charCodeAt(o+1),d=a.charCodeAt(o+2),p=a.charCodeAt(o+3),g=a.charCodeAt(o+4),E(h)<<12|E(d)<<8|E(p)<<4|E(g));if(u<0)throw f(e,o,"Invalid character escape sequence: "+"\\u".concat(a.slice(o+1,o+5),"."));c+=String.fromCharCode(u),o+=4;break;default:throw f(e,o,"Invalid character escape sequence: \\".concat(String.fromCharCode(l),"."))}s=++o}}var h,d,p,g;throw f(e,o,"Unterminated string.")}(n,a,o,s,t)}throw f(n,a,function(e){if(e<32&&9!==e&&10!==e&&13!==e)return"Cannot contain the invalid character ".concat(w(e),".");if(39===e)return"Unexpected single quote character ('), did you mean to use a double quote (\")?";return"Cannot parse the unexpected character ".concat(w(e),".")}(l))}function S(e,t,n){var i=e.body,r=t,a=n;if(a>=48&&a<=57){do{a=i.charCodeAt(++r)}while(a>=48&&a<=57);return r}throw f(e,r,"Invalid number, expected digit but got: ".concat(w(a),"."))}function E(e){return e>=48&&e<=57?e-48:e>=65&&e<=70?e-55:e>=97&&e<=102?e-87:-1}a(x,function(){return{kind:this.kind,value:this.value,line:this.line,column:this.column}});var C=Object.freeze({NAME:"Name",DOCUMENT:"Document",OPERATION_DEFINITION:"OperationDefinition",VARIABLE_DEFINITION:"VariableDefinition",SELECTION_SET:"SelectionSet",FIELD:"Field",ARGUMENT:"Argument",FRAGMENT_SPREAD:"FragmentSpread",INLINE_FRAGMENT:"InlineFragment",FRAGMENT_DEFINITION:"FragmentDefinition",VARIABLE:"Variable",INT:"IntValue",FLOAT:"FloatValue",STRING:"StringValue",BOOLEAN:"BooleanValue",NULL:"NullValue",ENUM:"EnumValue",LIST:"ListValue",OBJECT:"ObjectValue",OBJECT_FIELD:"ObjectField",DIRECTIVE:"Directive",NAMED_TYPE:"NamedType",LIST_TYPE:"ListType",NON_NULL_TYPE:"NonNullType",SCHEMA_DEFINITION:"SchemaDefinition",OPERATION_TYPE_DEFINITION:"OperationTypeDefinition",SCALAR_TYPE_DEFINITION:"ScalarTypeDefinition",OBJECT_TYPE_DEFINITION:"ObjectTypeDefinition",FIELD_DEFINITION:"FieldDefinition",INPUT_VALUE_DEFINITION:"InputValueDefinition",INTERFACE_TYPE_DEFINITION:"InterfaceTypeDefinition",UNION_TYPE_DEFINITION:"UnionTypeDefinition",ENUM_TYPE_DEFINITION:"EnumTypeDefinition",ENUM_VALUE_DEFINITION:"EnumValueDefinition",INPUT_OBJECT_TYPE_DEFINITION:"InputObjectTypeDefinition",DIRECTIVE_DEFINITION:"DirectiveDefinition",SCHEMA_EXTENSION:"SchemaExtension",SCALAR_TYPE_EXTENSION:"ScalarTypeExtension",OBJECT_TYPE_EXTENSION:"ObjectTypeExtension",INTERFACE_TYPE_EXTENSION:"InterfaceTypeExtension",UNION_TYPE_EXTENSION:"UnionTypeExtension",ENUM_TYPE_EXTENSION:"EnumTypeExtension",INPUT_OBJECT_TYPE_EXTENSION:"InputObjectTypeExtension"}),T=Object.freeze({QUERY:"QUERY",MUTATION:"MUTATION",SUBSCRIPTION:"SUBSCRIPTION",FIELD:"FIELD",FRAGMENT_DEFINITION:"FRAGMENT_DEFINITION",FRAGMENT_SPREAD:"FRAGMENT_SPREAD",INLINE_FRAGMENT:"INLINE_FRAGMENT",VARIABLE_DEFINITION:"VARIABLE_DEFINITION",SCHEMA:"SCHEMA",SCALAR:"SCALAR",OBJECT:"OBJECT",FIELD_DEFINITION:"FIELD_DEFINITION",ARGUMENT_DEFINITION:"ARGUMENT_DEFINITION",INTERFACE:"INTERFACE",UNION:"UNION",ENUM:"ENUM",ENUM_VALUE:"ENUM_VALUE",INPUT_OBJECT:"INPUT_OBJECT",INPUT_FIELD_DEFINITION:"INPUT_FIELD_DEFINITION"});function A(e,t){var n="string"==typeof e?new l(e):e;if(!(n instanceof l))throw new TypeError("Must provide Source. Received: ".concat(Object(i.a)(n)));return function(e){var t=e.token;return{kind:C.DOCUMENT,definitions:we(e,y.SOF,M,y.EOF),loc:de(e,t)}}(g(n,t||{}))}function _(e,t){var n=g("string"==typeof e?new l(e):e,t||{});ge(n,y.SOF);var i=V(n,!1);return ge(n,y.EOF),i}function O(e,t){var n=g("string"==typeof e?new l(e):e,t||{});ge(n,y.SOF);var i=$(n);return ge(n,y.EOF),i}function P(e){var t=ge(e,y.NAME);return{kind:C.NAME,value:t.value,loc:de(e,t)}}function M(e){if(pe(e,y.NAME))switch(e.token.value){case"query":case"mutation":case"subscription":case"fragment":return I(e);case"schema":case"scalar":case"type":case"interface":case"union":case"enum":case"input":case"directive":return K(e);case"extend":return function(e){var t=e.lookahead();if(t.kind===y.NAME)switch(t.value){case"schema":return function(e){var t=e.token;ve(e,"extend"),ve(e,"schema");var n=G(e,!0),i=pe(e,y.BRACE_L)?we(e,y.BRACE_L,te,y.BRACE_R):[];if(0===n.length&&0===i.length)throw be(e);return{kind:C.SCHEMA_EXTENSION,directives:n,operationTypes:i,loc:de(e,t)}}(e);case"scalar":return function(e){var t=e.token;ve(e,"extend"),ve(e,"scalar");var n=P(e),i=G(e,!0);if(0===i.length)throw be(e);return{kind:C.SCALAR_TYPE_EXTENSION,name:n,directives:i,loc:de(e,t)}}(e);case"type":return function(e){var t=e.token;ve(e,"extend"),ve(e,"type");var n=P(e),i=ne(e),r=G(e,!0),a=ie(e);if(0===i.length&&0===r.length&&0===a.length)throw be(e);return{kind:C.OBJECT_TYPE_EXTENSION,name:n,interfaces:i,directives:r,fields:a,loc:de(e,t)}}(e);case"interface":return function(e){var t=e.token;ve(e,"extend"),ve(e,"interface");var n=P(e),i=G(e,!0),r=ie(e);if(0===i.length&&0===r.length)throw be(e);return{kind:C.INTERFACE_TYPE_EXTENSION,name:n,directives:i,fields:r,loc:de(e,t)}}(e);case"union":return function(e){var t=e.token;ve(e,"extend"),ve(e,"union");var n=P(e),i=G(e,!0),r=se(e);if(0===i.length&&0===r.length)throw be(e);return{kind:C.UNION_TYPE_EXTENSION,name:n,directives:i,types:r,loc:de(e,t)}}(e);case"enum":return function(e){var t=e.token;ve(e,"extend"),ve(e,"enum");var n=P(e),i=G(e,!0),r=le(e);if(0===i.length&&0===r.length)throw be(e);return{kind:C.ENUM_TYPE_EXTENSION,name:n,directives:i,values:r,loc:de(e,t)}}(e);case"input":return function(e){var t=e.token;ve(e,"extend"),ve(e,"input");var n=P(e),i=G(e,!0),r=ue(e);if(0===i.length&&0===r.length)throw be(e);return{kind:C.INPUT_OBJECT_TYPE_EXTENSION,name:n,directives:i,fields:r,loc:de(e,t)}}(e)}throw be(e,t)}(e)}else{if(pe(e,y.BRACE_L))return I(e);if(J(e))return K(e)}throw be(e)}function I(e){if(pe(e,y.NAME))switch(e.token.value){case"query":case"mutation":case"subscription":return D(e);case"fragment":return function(e){var t=e.token;if(ve(e,"fragment"),e.options.experimentalFragmentVariables)return{kind:C.FRAGMENT_DEFINITION,name:X(e),variableDefinitions:L(e),typeCondition:(ve(e,"on"),Z(e)),directives:G(e,!1),selectionSet:j(e),loc:de(e,t)};return{kind:C.FRAGMENT_DEFINITION,name:X(e),typeCondition:(ve(e,"on"),Z(e)),directives:G(e,!1),selectionSet:j(e),loc:de(e,t)}}(e)}else if(pe(e,y.BRACE_L))return D(e);throw be(e)}function D(e){var t=e.token;if(pe(e,y.BRACE_L))return{kind:C.OPERATION_DEFINITION,operation:"query",name:void 0,variableDefinitions:[],directives:[],selectionSet:j(e),loc:de(e,t)};var n,i=N(e);return pe(e,y.NAME)&&(n=P(e)),{kind:C.OPERATION_DEFINITION,operation:i,name:n,variableDefinitions:L(e),directives:G(e,!1),selectionSet:j(e),loc:de(e,t)}}function N(e){var t=ge(e,y.NAME);switch(t.value){case"query":return"query";case"mutation":return"mutation";case"subscription":return"subscription"}throw be(e,t)}function L(e){return pe(e,y.PAREN_L)?we(e,y.PAREN_L,R,y.PAREN_R):[]}function R(e){var t=e.token;return{kind:C.VARIABLE_DEFINITION,variable:F(e),type:(ge(e,y.COLON),$(e)),defaultValue:me(e,y.EQUALS)?V(e,!0):void 0,directives:G(e,!0),loc:de(e,t)}}function F(e){var t=e.token;return ge(e,y.DOLLAR),{kind:C.VARIABLE,name:P(e),loc:de(e,t)}}function j(e){var t=e.token;return{kind:C.SELECTION_SET,selections:we(e,y.BRACE_L,z,y.BRACE_R),loc:de(e,t)}}function z(e){return pe(e,y.SPREAD)?function(e){var t=e.token;ge(e,y.SPREAD);var n=ye(e,"on");if(!n&&pe(e,y.NAME))return{kind:C.FRAGMENT_SPREAD,name:X(e),directives:G(e,!1),loc:de(e,t)};return{kind:C.INLINE_FRAGMENT,typeCondition:n?Z(e):void 0,directives:G(e,!1),selectionSet:j(e),loc:de(e,t)}}(e):function(e){var t,n,i=e.token,r=P(e);me(e,y.COLON)?(t=r,n=P(e)):n=r;return{kind:C.FIELD,alias:t,name:n,arguments:Y(e,!1),directives:G(e,!1),selectionSet:pe(e,y.BRACE_L)?j(e):void 0,loc:de(e,i)}}(e)}function Y(e,t){var n=t?W:H;return pe(e,y.PAREN_L)?we(e,y.PAREN_L,n,y.PAREN_R):[]}function H(e){var t=e.token,n=P(e);return ge(e,y.COLON),{kind:C.ARGUMENT,name:n,value:V(e,!1),loc:de(e,t)}}function W(e){var t=e.token;return{kind:C.ARGUMENT,name:P(e),value:(ge(e,y.COLON),q(e)),loc:de(e,t)}}function X(e){if("on"===e.token.value)throw be(e);return P(e)}function V(e,t){var n=e.token;switch(n.kind){case y.BRACKET_L:return function(e,t){var n=e.token,i=t?q:U;return{kind:C.LIST,values:xe(e,y.BRACKET_L,i,y.BRACKET_R),loc:de(e,n)}}(e,t);case y.BRACE_L:return function(e,t){var n=e.token;return{kind:C.OBJECT,fields:xe(e,y.BRACE_L,function(){return function(e,t){var n=e.token,i=P(e);return ge(e,y.COLON),{kind:C.OBJECT_FIELD,name:i,value:V(e,t),loc:de(e,n)}}(e,t)},y.BRACE_R),loc:de(e,n)}}(e,t);case y.INT:return e.advance(),{kind:C.INT,value:n.value,loc:de(e,n)};case y.FLOAT:return e.advance(),{kind:C.FLOAT,value:n.value,loc:de(e,n)};case y.STRING:case y.BLOCK_STRING:return B(e);case y.NAME:return"true"===n.value||"false"===n.value?(e.advance(),{kind:C.BOOLEAN,value:"true"===n.value,loc:de(e,n)}):"null"===n.value?(e.advance(),{kind:C.NULL,loc:de(e,n)}):(e.advance(),{kind:C.ENUM,value:n.value,loc:de(e,n)});case y.DOLLAR:if(!t)return F(e)}throw be(e)}function B(e){var t=e.token;return e.advance(),{kind:C.STRING,value:t.value,block:t.kind===y.BLOCK_STRING,loc:de(e,t)}}function q(e){return V(e,!0)}function U(e){return V(e,!1)}function G(e,t){for(var n=[];pe(e,y.AT);)n.push(Q(e,t));return n}function Q(e,t){var n=e.token;return ge(e,y.AT),{kind:C.DIRECTIVE,name:P(e),arguments:Y(e,t),loc:de(e,n)}}function $(e){var t,n=e.token;return me(e,y.BRACKET_L)?(t=$(e),ge(e,y.BRACKET_R),t={kind:C.LIST_TYPE,type:t,loc:de(e,n)}):t=Z(e),me(e,y.BANG)?{kind:C.NON_NULL_TYPE,type:t,loc:de(e,n)}:t}function Z(e){var t=e.token;return{kind:C.NAMED_TYPE,name:P(e),loc:de(e,t)}}function K(e){var t=J(e)?e.lookahead():e.token;if(t.kind===y.NAME)switch(t.value){case"schema":return function(e){var t=e.token;ve(e,"schema");var n=G(e,!0),i=we(e,y.BRACE_L,te,y.BRACE_R);return{kind:C.SCHEMA_DEFINITION,directives:n,operationTypes:i,loc:de(e,t)}}(e);case"scalar":return function(e){var t=e.token,n=ee(e);ve(e,"scalar");var i=P(e),r=G(e,!0);return{kind:C.SCALAR_TYPE_DEFINITION,description:n,name:i,directives:r,loc:de(e,t)}}(e);case"type":return function(e){var t=e.token,n=ee(e);ve(e,"type");var i=P(e),r=ne(e),a=G(e,!0),o=ie(e);return{kind:C.OBJECT_TYPE_DEFINITION,description:n,name:i,interfaces:r,directives:a,fields:o,loc:de(e,t)}}(e);case"interface":return function(e){var t=e.token,n=ee(e);ve(e,"interface");var i=P(e),r=G(e,!0),a=ie(e);return{kind:C.INTERFACE_TYPE_DEFINITION,description:n,name:i,directives:r,fields:a,loc:de(e,t)}}(e);case"union":return function(e){var t=e.token,n=ee(e);ve(e,"union");var i=P(e),r=G(e,!0),a=se(e);return{kind:C.UNION_TYPE_DEFINITION,description:n,name:i,directives:r,types:a,loc:de(e,t)}}(e);case"enum":return function(e){var t=e.token,n=ee(e);ve(e,"enum");var i=P(e),r=G(e,!0),a=le(e);return{kind:C.ENUM_TYPE_DEFINITION,description:n,name:i,directives:r,values:a,loc:de(e,t)}}(e);case"input":return function(e){var t=e.token,n=ee(e);ve(e,"input");var i=P(e),r=G(e,!0),a=ue(e);return{kind:C.INPUT_OBJECT_TYPE_DEFINITION,description:n,name:i,directives:r,fields:a,loc:de(e,t)}}(e);case"directive":return function(e){var t=e.token,n=ee(e);ve(e,"directive"),ge(e,y.AT);var i=P(e),r=ae(e);ve(e,"on");var a=function(e){me(e,y.PIPE);var t=[];do{t.push(he(e))}while(me(e,y.PIPE));return t}(e);return{kind:C.DIRECTIVE_DEFINITION,description:n,name:i,arguments:r,locations:a,loc:de(e,t)}}(e)}throw be(e,t)}function J(e){return pe(e,y.STRING)||pe(e,y.BLOCK_STRING)}function ee(e){if(J(e))return B(e)}function te(e){var t=e.token,n=N(e);ge(e,y.COLON);var i=Z(e);return{kind:C.OPERATION_TYPE_DEFINITION,operation:n,type:i,loc:de(e,t)}}function ne(e){var t=[];if(ye(e,"implements")){me(e,y.AMP);do{t.push(Z(e))}while(me(e,y.AMP)||e.options.allowLegacySDLImplementsInterfaces&&pe(e,y.NAME))}return t}function ie(e){return e.options.allowLegacySDLEmptyFields&&pe(e,y.BRACE_L)&&e.lookahead().kind===y.BRACE_R?(e.advance(),e.advance(),[]):pe(e,y.BRACE_L)?we(e,y.BRACE_L,re,y.BRACE_R):[]}function re(e){var t=e.token,n=ee(e),i=P(e),r=ae(e);ge(e,y.COLON);var a=$(e),o=G(e,!0);return{kind:C.FIELD_DEFINITION,description:n,name:i,arguments:r,type:a,directives:o,loc:de(e,t)}}function ae(e){return pe(e,y.PAREN_L)?we(e,y.PAREN_L,oe,y.PAREN_R):[]}function oe(e){var t=e.token,n=ee(e),i=P(e);ge(e,y.COLON);var r,a=$(e);me(e,y.EQUALS)&&(r=q(e));var o=G(e,!0);return{kind:C.INPUT_VALUE_DEFINITION,description:n,name:i,type:a,defaultValue:r,directives:o,loc:de(e,t)}}function se(e){var t=[];if(me(e,y.EQUALS)){me(e,y.PIPE);do{t.push(Z(e))}while(me(e,y.PIPE))}return t}function le(e){return pe(e,y.BRACE_L)?we(e,y.BRACE_L,ce,y.BRACE_R):[]}function ce(e){var t=e.token,n=ee(e),i=P(e),r=G(e,!0);return{kind:C.ENUM_VALUE_DEFINITION,description:n,name:i,directives:r,loc:de(e,t)}}function ue(e){return pe(e,y.BRACE_L)?we(e,y.BRACE_L,oe,y.BRACE_R):[]}function he(e){var t=e.token,n=P(e);if(T.hasOwnProperty(n.value))return n;throw be(e,t)}function de(e,t){if(!e.options.noLocation)return new fe(t,e.lastToken,e.source)}function fe(e,t,n){this.start=e.start,this.end=t.end,this.startToken=e,this.endToken=t,this.source=n}function pe(e,t){return e.token.kind===t}function ge(e,t){var n=e.token;if(n.kind===t)return e.advance(),n;throw f(e.source,n.start,"Expected ".concat(t,", found ").concat(b(n)))}function me(e,t){var n=e.token;if(n.kind===t)return e.advance(),n}function ve(e,t){var n=e.token;if(n.kind===y.NAME&&n.value===t)return e.advance(),n;throw f(e.source,n.start,'Expected "'.concat(t,'", found ').concat(b(n)))}function ye(e,t){var n=e.token;if(n.kind===y.NAME&&n.value===t)return e.advance(),n}function be(e,t){var n=t||e.token;return f(e.source,n.start,"Unexpected ".concat(b(n)))}function xe(e,t,n,i){ge(e,t);for(var r=[];!me(e,i);)r.push(n(e));return r}function we(e,t,n,i){ge(e,t);for(var r=[n(e)];!me(e,i);)r.push(n(e));return r}n.d(t,"parse",function(){return A}),n.d(t,"parseValue",function(){return _}),n.d(t,"parseType",function(){return O}),n.d(t,"parseConstValue",function(){return q}),n.d(t,"parseTypeReference",function(){return $}),n.d(t,"parseNamedType",function(){return Z}),a(fe,function(){return{start:this.start,end:this.end}})}]);